/*
 * trizeps.h: Keith & Koep Trizeps board specific defines
 *
 * Copyright (C) 2002	Wolfgang Mller (wolfgang.mueller@dsa-ac.de)
 *
 * $Id: trizeps.h,v 1.1 2002/04/23 13:29:56 seletz Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: trizeps.h,v 1.1 2002/04/23 13:29:56 seletz Exp $"

#ifndef BLOB_ARCH_TRIZEPS_H
#define BLOB_ARCH_TRIZEPS_H

/* boot CPU speed */
#define CPU_SPEED		CPU_CORE_SPEED_191mhz

/* serial port, depends on motherboard where trizeps is plugged in */
#define USE_SERIAL3
#define TERMINAL_SPEED baud_38400

/* GPIO for the LED */
#define LED_GPIO			(0)

/* the base address were BLOB is loaded by the first stage loader */
#define BLOB_ABS_BASE_ADDR	(0xC0200400)


/* where do various parts live in RAM */
#define BLOB_RAM_BASE		(0xC0100000)
#define PARAM_RAM_BASE		(0xC0110000)
#define KERNEL_RAM_BASE		(0xC0008000)
#define RAMDISK_RAM_BASE	(0xC0400000)


/* their size (in bytes) and where do they live in flash*/
#define BLOB_FLASH_BASE		(0x00000000)
#define BLOB_FLASH_LEN		(1024 * 512)
#define PARAM_FLASH_BASE	(BLOB_FLASH_BASE + BLOB_FLASH_LEN)
#define PARAM_FLASH_LEN		(1024 * 512)
#define KERNEL_FLASH_BASE	(PARAM_FLASH_BASE + PARAM_FLASH_LEN)
#define KERNEL_FLASH_LEN	(1024 * 1024 * 1)
#define RAMDISK_FLASH_BASE	(KERNEL_FLASH_BASE + KERNEL_FLASH_LEN)
#define RAMDISK_FLASH_LEN	(1024 * 1024 * 16 - BLOB_FLASH_LEN \
						  - PARAM_FLASH_LEN \
						  - KERNEL_FLASH_LEN)
#define LOAD_RAMDISK		0 /* leave ramdisk in flash */
// #define LOAD_RAMDISK		1 /* load ramdisk into ram */


/* the position of the kernel boot parameters */
#define BOOT_PARAMS		(0xC0000100)


/* the size (in kbytes) to which the compressed ramdisk expands */
#define RAMDISK_SIZE		(1024 * 8)


/* Memory configuration */
#ifdef BLOB_NEED_MEMCONFIG	/* Values extracted from original K & K boot */
#warning "use defines from memsetup.h for better readability"
#	define MDCNFG_VALUE	0x7354B455		/* 0x00 MDCNFG */
#	define MDCAS00_VALUE	0xAAAAAA9F		/* 0x04 MDCAS00 */
#	define MDCAS01_VALUE	0xAAAAAAAA		/* 0x08 MDCAS01 */
#	define MDCAS02_VALUE	0xAAAAAAAA		/* 0x0C MDCAS02 */
#	define MSC0_VALUE	0x4C804C84		/* 0x10 MCS0 */
#	define MSC1_VALUE	0x497572D4		/* 0x14 MCS1 */
#	define MECR_VALUE	0x08640464		/* 0x18 MECR */
#	define MDREFR_VALUE	DO_NOT_USE_THIS_VALUE__GETS_AUTOMAGICALLY_COMPUTED
#	define MDCAS20_VALUE	0xAAAAAA7F		/* 0x20 MDCAS20 */
#	define MDCAS21_VALUE	0xAAAAAAAA		/* 0x24 MDCAS21 */
#	define MDCAS22_VALUE	0xAAAAAAAA		/* 0x28 MDCAS22 */
#	define MSC2_VALUE	0x211154E0		/* 0x2C MCS2 */
#	define SMCNFG_VALUE	0xA040A040		/* 0x30 SMCNFG */
#endif

#define TRIZEPS_DEBUG 1

#endif /* BLOB_ARCH_TRIZEPS_H */

/* eof */
