/**********************************************************************
 * lcd.h
 *
 * AUTOR:	SELETZ
 *
 * Generic lcd framework
 *
 * Copyright (C) 2001,2002 Stefan Eletzhofer <stefan.eletzhofer@www.eletztrick.de>
 * Copyright (C) 2002 Urs Kaufmann <u_kaufmann@bluewin.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef BLOB_LCD_H
#define BLOB_LCD_H

#if defined(CONFIG_LCD_SUPPORT)

#ifdef PT_SYSTEM3
#	define	LCD_LCCR0	0x000AA03C
#	define	LCD_LCCR1	0x01000270
#	define	LCD_LCCR2	0x000004EF
#	define	LCD_LCCR3	0x0000FF16
#	define	LCD_BPP			8
#	define	LCD_COLS		640
#	define	LCD_ROWS		480
#	define	LCD_VIDEORAM_SIZE	((LCD_BPP*LCD_ROWS*LCD_COLS)>>3)
#	define	LCD_PALETTE_ENTRIES	256
#	define	LCD_PALETTE_SIZE	(LCD_PALETTE_ENTRIES * 2)
#	define	LCD_RAM_BASE		0xd0000000
#	define	LCD_PALETTE_DMA_ADR	(LCD_RAM_BASE + 0x0)
#	define	LCD_VIDEORAM_START	(LCD_RAM_BASE + LCD_PALETTE_SIZE)
#	define	LCD_VIDEORAM_END	(LCD_VIDEORAM_START + LCD_VIDEORAM_SIZE)
#	define	LCD_VIDEORAM_DMA_ADR	(LCD_PALETTE_DMA_ADR + LCD_PALETTE_SIZE + (LCD_VIDEORAM_SIZE>>1))

#elif MINIPRINT
#       define  LCD_LCCR0       0x00000038
/* LEN = 0   not enabled
   CMS = 0   Color Mode
   SDS = 0   Single Panel
   LDM = 1   Done IRQ Disable
   BAM = 1   Base Addr update IRQ disable
   ERM = 1   Disable Error IRQ
   PAS = 0   Passive Mode
   BLE = 0   Little Endian
   DPD = 0   for monochrome only
   VSC = 0   Vert Slant
   PDD = 0   No DMA Delay */
#       define  LCD_LCCR1       0x01010530
/* PPL = 304  (320-16=304)
   HSW = 1    H-Sync
   ELW = 1    EndOfLine Sync
   BLW = 1    BeginOfLine Sync */
#       define  LCD_LCCR2       0x000000EF
/* LPP = 239   Lines
   VSW = 0    Xtra Clock
   EFW = 0    Xtra Clock
   BFW = 0    Xtra Clock */
#       define  LCD_LCCR3       0x0000FF1E
/* PCD = 0x1E 75Hz Refresh
   ACB = 0
   API = 0
   VSP = 0
   HSP = 0
   PCP = 0
   OEP = 0  */
#       define  LCD_BPP                 16
#       define  LCD_COLS                320
#       define  LCD_ROWS                240
#       define  LCD_VIDEORAM_SIZE       ((LCD_BPP*LCD_ROWS*LCD_COLS)>>3)
#       define  LCD_PALETTE_ENTRIES     256
#       define  LCD_PALETTE_SIZE        (LCD_PALETTE_ENTRIES * 2)
#       define  LCD_RAM_BASE            0xc0900000
#       define  LCD_PALETTE_DMA_ADR     (LCD_RAM_BASE + 0x0)
#       define  LCD_VIDEORAM_START      (LCD_RAM_BASE + LCD_PALETTE_SIZE)
#       define  LCD_VIDEORAM_END        (LCD_VIDEORAM_START + LCD_VIDEORAM_SIZE)
#       define  LCD_VIDEORAM_DMA_ADR    (LCD_PALETTE_DMA_ADR + LCD_PALETTE_SIZE + (LCD_VIDEORAM_SIZE>>1))
# define  LCD_REVERSE
#else
#	error "Cowardly refusing to break your LCD, please supply correct LCD parameters."
#endif

#endif

int lcd_power_up( void );
int lcd_power_down( void );

int lcd_gpio_setup( void );

int lcd_controller_setup( void );
int lcd_controller_enable( void );
int lcd_controller_disable( void );

int lcd_backlight_on( void );
int lcd_backlight_off( void );

int lcd_fill(int color);
int lcd_set_pixel(unsigned int x, unsigned int y, unsigned int color);

int lcd_palette_setup( void );

int lcd_contrast( int );
int lcd_brightness( int );

#endif
