/*
 * bstd.c: Intel Monahans specific stuff
 *
 * Modified from src/blob/lubbock.c
 *
 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 * Copyright (C) 2004  Intel Corporation (stanley.cai@intel.com)
 * Copyright (C) 2003  Intel Corporation (yu.tang@intel.com)
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/time.h>
#include <blob/command.h>
#include <blob/util.h>
#include <net/ether.h>
#include <net/net.h>
#include <blob/strata.h>
#include <blob/command.h>

#include <blob/pmic_lp3972.h>
#include <xllp_pm_suspend.h>

#ifndef CONFIG_USBDNET_SUPPORT
#include <net/ether.h>
#endif

#define	FLASH_BASE 	0x10000000
int BLOB_BOOT_DELAY_TIME = 0;
static struct flash_chip flash_chips[2];

extern int blob_nand_write(unsigned int ram_addr, unsigned int flash_ofs,
	   unsigned long write_size, char * type );
extern int blob_nand_erase( unsigned int flash_ofs, 
		unsigned int size, char *type );
/*
char mtdparts[] ="mtdparts=pxa3xx-nand:0x100000@0x0(Bootloader)ro,\
0x40000@0x100000(security),0xc0000@0x140000(NVM),\
0x40000@0x200000(logo)ro,0x40000@0x240000(panic),\
0x380000@0x280000(Recovery-Kernel)ro,0x300000@0x600000(Boot)ro,\
0x6000000@0x900000(system),0x3000000@0x6900000(userdata),\
0xf00000@0x9900000(local),0x800000@0xa800000(cache),\
0x3300000@0xb000000(opl),0x220000@0xe300000(ftm),\
0x1560000@0xe520000(fota),0x80000@0xfa80000(BBT)ro;";
*/

char mtdparts[] ="mtdparts=pxa3xx-nand:0x100000@0x0(bootldr)ro,\
0x40000@0x100000(security),0xc0000@0x140000(NVM),\
0x40000@0x200000(logo)ro,0x40000@0x240000(panic),\
0x380000@0x280000(recovery)ro,0x300000@0x600000(boot)ro,\
0x6000000@0x900000(system),0xf00000@0x6900000(local),\
0x800000@0x7800000(cache),0x3300000@0x8000000(opl),\
0x3000000@0xb300000(userdata),0x1180000@0xe300000(ftm),\
0x600000@0xf480000(fota),0x80000@0xfa80000(BBT)ro;";

char mtdparts_recovery[] ="mtdparts=pxa3xx-nand:0x100000@0x0(bootldr)ro,\
0x40000@0x100000(security),0xc0000@0x140000(NVM),\
0x40000@0x200000(logo)ro,0x40000@0x240000(panic),\
0x380000@0x280000(recovery)ro,0x300000@0x600000(boot)ro,\
0x6000000@0x900000(system),0xf00000@0x6900000(local),\
0x800000@0x7800000(cache),0x3300000@0x8000000(opl),\
0x4180000@0xb300000(userdata),0x600000@0xf480000(fota),\
0x80000@0xfa80000(BBT)ro;";

#ifdef MMU_ENABLE
struct mem_area
{
	unsigned long start;
	unsigned long size;
	unsigned char attr;
};

static struct mem_area io_map[]={
	{0x00000000, 	0x10000000, 	0x2}, /* ROM, 256M */
	{0x10000000, 	0x04000000, 	0x2}, /* ROM, 64M */
	{0x14000000,    0x04000000, 	0x2}, /* LAN91c111, 64M*/
	{0x18000000,    0x04000000, 	0x2}, /* SMEMC reserved ,64M */
	{0x1c000000,    0x04000000, 	0x2}, /* SMEMC reserved,64M */
	{0x20000000,    0x04000000, 	0x2}, /* PCMCIA - CF IO space,64M */
	{0x24000000,    0x04000000, 	0x2}, /* SMEMC reserved,64M */
	{0x28000000,    0x04000000, 	0x2}, /* PCMCIA - CF Attr space,64M */
	{0x2c000000,    0x04000000, 	0x2}, /* PCMCIA - CF common space,64M */
	{0x30000000,    0x10000000, 	0x2}, /* Static chip select 1, 256M */
	{0x40000000,	0x04000000,     0x2}, /* Periperial device  */
	{0x43000000,    0x00100000,     0x2}, /* CADDO */
	{0x44000000,    0x00100000,     0x2}, /* LCD */
	{0x46000000,    0x00100000,     0x2}, /* Mini LCD & */
	{0x48000000,    0x04000000,     0x2}, /* Memory Ctl*/
	{0x48100000,    0x00100000,     0x2}, /* Dynamic Mem Ctl */
	{0x4a000000,    0x00100000,     0x2}, /* Static Mem Ctl */
	{0x4c000000,    0x00100000,     0x2}, /* USB host */
	{0x50000000,    0x00100000,     0x2}, /* Camera */
	{0x54000000,    0x00200000,     0x2}, /* 2D Graphics & USB2.0 */
	{0x58000000,    0x00100000,     0x2}, /* Internal SRAM ctl */	
	{0x5C000000,    0x00100000,     0xe}, /* SRAM */
	{0x80000000, 	KERNEL_RAM_BASE - 0x80000000 - DDR_UNCACHABLE_SIZE,
	   	0xe}, /* SDRAM part 0: cachable */
	{DDR_UNCACHABLE_BASE,	DDR_UNCACHABLE_SIZE,
	       	0x2}, /* SDRAM part 1: uncachable*/
	{0x80800000,     0x07800000, 0xe}, /* SDRAM part 2: cachable */
	{0x88000000,     0x08000000, 0xe}, /* 2nd SDRAM bank: cachable */
	{0xc0000000,     0x08000000, 0xe}, /* SDRAM part 2: cachable */
	{0xc8000000,     0x08000000, 0xe}, /* 2nd SDRAM bank: cachable */

};

// Large Page, 1M per entry 
#define MAX_PE	4096

static unsigned long mmu_table[MAX_PE] __attribute__ ( (aligned(0x4000)));

static void enable_mmu()
{
	unsigned long i, addr, end;

	/* init MMU table */
	for (i=0; i < MAX_PE; i++) {
		mmu_table[i] = 0;
	}

	for (i=0; i < sizeof(io_map)/sizeof(struct mem_area); i++) {
		addr = io_map[i].start;
		end = addr + io_map[i].size;

		while( addr < end ) {
			mmu_table[addr>>20] = addr | io_map[i].attr;
			addr += 0x100000;
		}
	}

        asm(" \
	MRC p15, 0, R2, c1, c0 \n\
	ORR R2, R2, #0x1000 \n\
	ORR R2, R2, #0x800 \n\
	MCR p15, 0, R2, c1, c0, 0 \n\
 \
	mrc p15, 0, R2, c2, c0, 0 \n\
	mov R2, R2  \n\
	sub PC, PC, #4 \n\
 \
	ldr r0, =mmu_table \n\
	mcr p15, 0, r0, c2, c0 \n\
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
 \
	mvn r0, #0 \n\
	mcr p15, 0, r0, c3, c0, 0 \n\
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
 \
	MCR p15, 0, r0, c7, c10, 4 \n\
	MRC p15, 0, r0, c1, c0, 0 \n\
	ORR r0, r0, #4 \n\
	MCR p15, 0, r0, c1, c0, 0 \n\
 \
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
 \
	MRC p15,0,R0,c1,c0,0 \n\
	ORR R0, R0, #0x1 \n\
	MCR p15,0,R0,c1,c0,0 \n\
 \
	mrc p15, 0, R0, c2, c0, 0 \n\
	mov R0, R0 \n\
	sub PC, PC, #4 \n\
        "
        );
}
#endif

/**********************************************************************
 * set gpio for nand flash
 */

 static int DfcSetGPIO()
{
	*(u32*)(PIN_nCS3) = 0x801;
	*(u32*)(PIN_DF_nCS0_MFPR)        = 0x00001801;
	*(u32*)(PIN_DF_nCS1_MFPR)        = 0x00001801;
	*(u32*)(PIN_DF_IO0_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO8_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO1_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO9_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO2_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO10_MFPR)        = 0x00001401;
	*(u32*)(PIN_DF_IO3_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO11_MFPR)        = 0x00001401;
	*(u32*)(PIN_DF_IO4_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO12_MFPR)        = 0x00001401;
	*(u32*)(PIN_DF_IO5_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO13_MFPR)        = 0x00001401;
	*(u32*)(PIN_DF_IO6_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO14_MFPR)        = 0x00001401;
	*(u32*)(PIN_DF_IO7_MFPR)         = 0x00001401;
	*(u32*)(PIN_DF_IO15_MFPR)        = 0x00001401;
	
	*(u32*)(PIN_DF_nWE_MFPR)         = 0x1901;
	*(u32*)(PIN_DF_INT_RnB_MFPR)     = 0x1900;
	*(u32*)(PIN_DF_CLE_nOE_MFPR)     = 0x1900;
#if   defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)	
    *(u32*)(PIN_DF_nRE_nOE_MFPR)     = 0x1901;
	*(u32*)(PIN_DF_nADV1_ALE_MFPR)   = 0x1901;
	*(u32*)(PIN_DF_nADV2_ALE_MFPR)   = 0x1901;
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
	*(u32*)(PIN_DF_nRE_MFPR)     	 = 0x1901;
	*(u32*)(PIN_DF_ALE_nWE_MFPR)     = 0x1901;
#else
#error "please initialize ALE pin(s) for DFC!"
#endif

	return 0;
}

/**********************************************************************
 * set gpio for Uart
 */

int UartSetGPIO()
{
#ifdef USE_SERIAL1
#if   defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)
	*(unsigned int *)GPIO41_MFPR = 0x802; /* FFRXD */
	*(unsigned int *)GPIO42_MFPR = 0x802; /* FFTXD */
	*(unsigned int *)GPIO43_MFPR = 0x802; /* FFCTS */
	*(unsigned int *)GPIO44_MFPR = 0x802; /* FFDCD */
	*(unsigned int *)GPIO45_MFPR = 0x802; /* FFDSR */
	*(unsigned int *)GPIO46_MFPR = 0x802; /* FFRI */
	*(unsigned int *)GPIO47_MFPR = 0x802; /* FFDTR */
	*(unsigned int *)GPIO48_MFPR = 0x802; /* FFRTS */
#elif defined(CONFIG_CPU_MONAHANS_L)
	*(unsigned int *)GPIO30_MFPR = 0x802; /* FFRXD */
	*(unsigned int *)GPIO31_MFPR = 0x802; /* FFTXD */
	*(unsigned int *)GPIO32_MFPR = 0x802; /* FFCTS */
	*(unsigned int *)GPIO33_MFPR = 0x802; /* FFDCD */
	*(unsigned int *)GPIO34_MFPR = 0x802; /* FFDSR */
	*(unsigned int *)GPIO35_MFPR = 0x802; /* FFRI */
	*(unsigned int *)GPIO36_MFPR = 0x802; /* FFDTR */
	*(unsigned int *)GPIO37_MFPR = 0x802; /* FFRTS */
#elif defined(CONFIG_CPU_MONAHANS_LV)
#if 0
	*(unsigned int *)GPIO99_MFPR = 0x801; /* FFRXD */
	*(unsigned int *)GPIO100_MFPR = 0x801; /* FFTXD */
	*(unsigned int *)GPIO101_MFPR = 0x801; /* FFCTS */
#if 0	
	*(unsigned int *)GPIO102_MFPR = 0x801; /* FFDCD */
	*(unsigned int *)GPIO103_MFPR = 0x801; /* FFDSR */
	*(unsigned int *)GPIO104_MFPR = 0x801; /* FFRI */
	*(unsigned int *)GPIO105_MFPR = 0x801; /* FFDTR */
#endif	
	*(unsigned int *)GPIO106_MFPR = 0x801; /* FFRTS */
#else
	*(unsigned int *)GPIO77_MFPR = 0x801; /* FFRXD */
	*(unsigned int *)GPIO78_MFPR = 0x801; /* FFTXD */
#endif
#else
#error "please define MFP pins for UART!"
#endif
	/* enable clock */
	CKENA |= CKENA_22_FFUART;
#endif

#ifdef USE_SERIAL2
#if   defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)
	*(unsigned int *)GPIO109_MFPR = 0x801; /* BTRTS */
	*(unsigned int *)GPIO110_MFPR = 0x801; /* BTRXD */
	*(unsigned int *)GPIO111_MFPR = 0x801; /* BTTXD */
	*(unsigned int *)GPIO112_MFPR = 0x801; /* BTCTS */
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
	*(unsigned int *)GPIO111_MFPR = 0x801; /* BTRTS */
	*(unsigned int *)GPIO112_MFPR = 0x801; /* BTRXD */
	*(unsigned int *)GPIO113_MFPR = 0x801; /* BTTXD */
	*(unsigned int *)GPIO114_MFPR = 0x801; /* BTCTS */
#else
#error "please define MFP pins for UART!"
#endif
    /* enable BTUART clock */
	CKENA |= CKENA_21_BTUART;
#endif

#ifdef USE_SERIAL3
#if   defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_PL)
	*(unsigned int *)GPIO107_MFPR = 0x801; /* STTXD */
	*(unsigned int *)GPIO108_MFPR = 0x801; /* STRXD */	
#elif defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
	*(unsigned int *)GPIO109_MFPR = 0xC041; /* STTXD */
	*(unsigned int *)GPIO110_MFPR = 0x4041; /* STRXD */
#else
#error "please define MFP pins for UART!"
#endif
	CKENA |= CKENA_23_STUART;
#endif
	
}

void bstd_init_hardware(void)
{
#ifdef CONFIG_NETWORK_SUPPORT
	unsigned char serverip[4]={192,168,1,100};
	unsigned char clientip[4]={192,168,1,101};

	/* set default ip */
	memcpy(serveripaddress, serverip, sizeof(serverip));
	memcpy(clientipaddress, clientip, sizeof(clientip));
#endif
	*(unsigned int*)GPIO3_MFPR = 0x801;
	*(unsigned int*)GPIO4_MFPR = 0x801;

	/* By yfw
	 * Hack for bstd Ethernet debug.
	 * CS1 is used for bstd Ethernet debug.
	 */
	CSADRCFG1 = 0x0012c80b;
	MSC0 = 0x002c0000 | (MSC0 & 0xffff);

#ifndef CONFIG_NAND_SUPPORT
	/* Set Static memory controller register for Sibley */
	//*(u32*)(MFPR3_ADDR) = 0x1;
	CSADRCFG2 = 0x00320919;
	MSC1 = (MSC1 & 0xFFFF0000) | 0x00007ff8;
	DfcSetGPIO();
#else
	/* we must set nand flash gpio here, 
	 * for ethernet/NAND/Sibley will use these pins.
	 */
	DfcSetGPIO();
#endif


	/* select serial driver */
	serial_driver = &mhn_serial_driver;
	reboot_driver = &mhn_reboot_driver;
        timer_driver = &intelarm_timer_driver;

#ifdef CONFIG_NETWORK_SUPPORT
#ifdef CONFIG_USBDNET_SUPPORT
	ether_driver = &bvdusb_ether_driver;
	//ether_driver = &mhnu2d_ether_driver;
#elif CONFIG_USBDNET2_SUPPORT
	ether_driver = &mhnu2d_ether_driver;
#else
	ether_driver = &smc91x_ether_driver;
#endif
#endif


	/* initialize flash driver */
	//init_flash_driver();

	/* turn on MMU, for better performace */
#ifdef MMU_ENABLE
	enable_mmu();
#endif
	msleep(100);
	printf("MDCNFG = 0x%08x\n", MDCNFG );
}


__initlist(bstd_init_hardware, INIT_LEVEL_DRIVER_SELECTION);

void init_product(void)
{
    P_XLLP_PI2C_T pI2c = PI2C_BASE;
    P_XLLP_OST_T pOst = OST_BASE;
    P_XLLP_PM_SPMU_T pSpmu = SPMU_BASE;
    P_XLLP_PM_BPMU_T pBpmu = BPMU_BASE;

#if defined(CHERRY)
#else    
    printf("Prepare to power on LCD \n");
    PowerOnLCD(pI2c, pOst, pSpmu, XLLP_TRUE);
    PowerOnCamera(pI2c, pOst, pSpmu, XLLP_TRUE);
    PowerOnMMC(pI2c, pOst, pSpmu, XLLP_TRUE);
#endif
}

#ifndef CONFIG_NAND_SUPPORT
#if	0
static void init_zylo_nor_flash_driver(void)
{
        int i = 0;
        flash_descriptors = NULL;
        flash_driver = NULL;

        struct flash_chip *chip = &flash_chips;

        chip->base = FLASH_BASE;
        chip->buswidth = 2;
        flash_probe(chip);
}

__initlist(init_zylo_nor_flash_driver, INIT_LEVEL_DRIVER_SELECTION);

/*********************************************************************
 * cmd_flash_write
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash write access
 *
 */
static int cmd_flash_write( int argc, char *argv[] )
{
	int ret = 0;
	u32	src	= 0L;
	u32	dest	= 0L;
	u32	len	= 0L;
	struct strata_flash_descriptor *chip=NULL;

//	printf("%s(): enter into cmd_flash_write\n", __func__);
	
	if ( argc < 4 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[1], &src );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[2], &dest );
	if ( ret < 0 )	{
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[3], &len );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}

	if (dest < 0x10000000) {
		printf("You cannot burn flash below 0x10000000.\n");
		return -1;
	}

	chip = &flash_chips[0];

//	printf("%s(): chip: 0x%x, dest: 0x%x, src: 0x%x, len: 0x%x\n", __func__, chip, dest, (u8*)src, len);
	ret = flash_write( chip, dest, (u8*) src, len); 
//	printf("%s(): flash_write return %d\n", __func__, ret);

DONE:
	return ret;
}


static char flashwritehelp[] = "fwrite srcadr destadr size(bytes)\n"
"flash a memory region\n";
__commandlist( cmd_flash_write, "fwrite", flashwritehelp );

/*********************************************************************
 * cmd_flash_erase
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash erasing
 *
 */
static int cmd_flash_erase( int argc, char *argv[] )
{
	int ret = 0;
	u32	dest	= 0L;
	u32	len	= 0L;
	struct strata_flash_descriptor *chip=NULL;

	if ( argc < 3 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[1], &dest );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}	

	ret = strtou32( argv[2], &len );
	if ( ret < 0 ){
		ret = -1;
		goto DONE;
	}

	chip = &flash_chips[0];
	ret = flash_erase(chip, dest, len);

DONE:
	return ret;
}


static char flasherasehelp[] = "ferase adr size(bytes)\n"
"erase a flash region\n";
__commandlist( cmd_flash_erase, "ferase", flasherasehelp );

#endif
#endif

#if 0
/*********************************************************************
 * cmd_clock_info
 *
 * AUTOR:	yu tang, rusty geldmacher
 * REVISED:
 *
 * prints out clock information. taken from Yu's kernel patch
 *
 */
#define BASE_CLK 13000000
static int cmd_clock_info(int argc, char *argv[])
{
	unsigned long acsr_val, clkcfg_val, mcc_val, memclkcfg_val;
	unsigned int XL, M, XN, HSS, DMSFS, SFLFS, SMCFS, DF_CLKDIV, EMPI_CLKDIV;
	unsigned int r_clk, t_clk, s_clk, ddr_clk, sram_clk, smem_clk, df_clk, empi_clk;
  
	acsr_val = ACSR;
	memclkcfg_val = MEMCLKCFG;
	asm("mrc p14, 0, %0, c6, c0, 0" : "=r" (clkcfg_val) );
  
	XL  =  acsr_val & 0x1f;
	XN  = (acsr_val >> 8) & 0x7;
	HSS = (acsr_val >> 14) & 0x3;
	DMSFS = (acsr_val >> 12) & 0x3;
	SFLFS = (acsr_val >> 18) & 0x3;
	SMCFS = (acsr_val >> 23) & 0x7;
	
	EMPI_CLKDIV = memclkcfg_val & 0x7;
	DF_CLKDIV = (memclkcfg_val >> 16) & 0x7;

	r_clk = XL * BASE_CLK;
	t_clk = XL * BASE_CLK * XN;

	switch (HSS) {
	case 0:
		s_clk = 104000000;
		break;
	case 1:
		s_clk = 156000000;
		break;
	case 2:
		s_clk = 208000000;
		break;
	case 3:
		s_clk = 312000000;
	}

	switch (DMSFS) {
	case 0:
	  ddr_clk = 13000000;
	  break;
	case 1:
	  ddr_clk = 78000000;
	  break;
	case 2:
	  ddr_clk = 104000000;
	  break;
	case 3:
	  ddr_clk = 130000000;
	}

	switch (SFLFS) {
	case 0:
	  sram_clk = 104000000;
	  break;
	case 1:
	  sram_clk = 156000000;
	  break;
	case 2:
	  sram_clk = 208000000;
	  break;
	case 3:
	  sram_clk = 312000000;
	}

	switch (SMCFS) {
	case 0:
	  smem_clk = 78000000;
	  break;
	case 1:
	  smem_clk = 89100000;
	  break;
	case 2:
	  smem_clk = 104000000;
	  break;
	case 3:
	  smem_clk = 124800000;
	  break;
	case 4:
	  smem_clk = 156000000;
	  break;
	case 5:
	  smem_clk = 208000000;
	  break;
	case 6:
	  smem_clk = 78000000;
	  break;
	case 7:
	  smem_clk = 78000000;
	  break;

	}
   
	switch (EMPI_CLKDIV) {
	case 1:
	  empi_clk = smem_clk;
	  break;
	case 2:
	  empi_clk = smem_clk / 2;
	  break;
	case 3:
	  empi_clk = smem_clk / 4;
	  break;
	default:
	  empi_clk = 0;
	}

	switch (DF_CLKDIV) {
	case 2:
	  df_clk = smem_clk / 2;
	  break;
	case 3:
	  df_clk = smem_clk / 4;
	  break;
	default:
	  df_clk = 0;
	}

	printf("Run Mode clock: %d.%02dMHz (*%d)\n",
	 	r_clk / 1000000, (r_clk % 1000000) / 10000, XL );
	printf("Turbo Mode clock: %d.%02dMHz (*%d, %sactive)\n",
		t_clk / 1000000, (t_clk % 1000000) / 10000, XN,
		(clkcfg_val & 0x1) ? "" : "in" );
	printf("DDR controller clock: %d.%02dMHz\n",
		ddr_clk / 1000000, (ddr_clk % 1000000) / 10000);
	printf("Static memory controller clock: %d.%02dMHz\n",
		smem_clk / 1000000, (smem_clk % 1000000) / 10000);
	printf("Data Flash controller clock (DF_SCLK): %d.%02dMHz\n",
		df_clk / 1000000, (df_clk % 1000000) / 10000);
	printf("EMPI clock (EMPI_SCLK): %d.%02dMHz\n",
		empi_clk / 1000000, (empi_clk % 1000000) / 10000);
	printf("Internal SRAM clock: %d.%02dMHz\n",
		sram_clk / 1000000, (sram_clk % 1000000) / 10000);
	printf("High Speed I/O Bus clock: %d.%02dMHz \n",
		s_clk / 1000000, (s_clk % 1000000) / 10000 );

	return 0;
}

static char clockinfohelp[] = "clockinfo\n"
"print out processor clock information\n";
__commandlist(cmd_clock_info, "clockinfo", clockinfohelp);
#endif


#define TEST_MAX 0x2000000

#define DDR_PHYS 0x83000000

static int cmd_memtest_byte(int argc, char *argv[])
{
  unsigned char *addr;
  unsigned char c;
  int i;

  unsigned long ddr_phys_add, test_max;

  if ( argc == 3 ){
    strtou32( argv[1], &ddr_phys_add);
    strtou32( argv[2], &test_max);
    addr = ddr_phys_add;
  }
  else {
  // base address of DDR
    ddr_phys_add = DDR_PHYS;
    test_max = TEST_MAX;
    addr = (unsigned char *)DDR_PHYS;
  }

  printf("This test will take 4-5 minutes!\nTesting DDR memory from physical 0x%08x through 0x%08x (%d MB)\n",
	 addr, addr + test_max, test_max >> 20);

  printf("Phase 1: Memory byte test... ");

  // test writing bytes
  for (i = 0; i < test_max; i++) {
    addr[i] = (unsigned char) 0xff;
  }


  // test reading back with known values
  for (i = 0; i < test_max; i++) {
    c = addr[i];
    if (c != (unsigned char) 0xff) {
      printf("failed at byte %d, wrong value is 0x%08x\n", i, c);
      return -1;
    }
  }

  // test writing bytes
  for (i = 0; i < test_max; i++) {
    addr[i] = (unsigned char) 0x00;
  }

  printf("passed\n");
  printf("Phase 2: Memory byte test... ");

  // test reading back with known values
  for (i = 0; i < test_max; i++) {
    c = addr[i];
    if (c != (unsigned char) 0x00) {
      printf("failed at byte %d, wrong value is 0x%08x\n", i, c);
      return -1;
    }
  }
  
  printf("passed\n");
  
  return 0;
}

static char memtestbytehelp[] = "1: memtest_byte\n"
"2: memtest_byte [start memory addres] [test length]\n"
"DDR test with bytes\n";
__commandlist(cmd_memtest_byte, "memtest_byte", memtestbytehelp);

static int cmd_memtest_word(int argc, char *argv[])
{
  unsigned long *addr;
  unsigned long w;
  unsigned long pattern[] = {0xaaaaaaaa, 0x55555555};
  unsigned long ddr_phys_add, test_max;
  int i;

  if ( argc == 3 ){
    strtou32( argv[1], &ddr_phys_add);
    strtou32( argv[2], &test_max);
    addr = ddr_phys_add;
  }
  else {
  // base address of DDR
    ddr_phys_add = DDR_PHYS;
    test_max = TEST_MAX;
    addr = (unsigned char *)DDR_PHYS;
  }

    printf("Memory word test... 0x%x, 0x%x \n",pattern[0], pattern[1]);
  printf("This test will take 4-5 minutes!\nTesting DDR memory from physical 0x%08x through 0x%08x (%d MB)\n",
	 ddr_phys_add, ddr_phys_add + test_max, test_max >> 20);

  printf("Memory word test... ");

  // test writing words
  i = 0;
  for (addr = (unsigned long *)ddr_phys_add; addr < (unsigned long *)(ddr_phys_add + test_max); addr++) {
    *addr = pattern[i];
    i = (i + 1) % 2;
  }

  // test reading back with known values
  i = 0;
  for (addr = (unsigned long *)ddr_phys_add; addr < (unsigned long *)(ddr_phys_add + test_max); addr++) {
    w = *addr;
    if (w != pattern[i]) {
      printf("failed at address 0x%08x\n", addr);
      return -1;
    }
    i = (i + 1) % 2;
  }

  printf("passed\n");
  
  return 0;
}

static char memtestwordhelp[] = "1: memtest_word\n"
"2: memtest_word [start memory addres] [test length]\n"
"DDR test with words\n";
__commandlist(cmd_memtest_word, "memtest_word", memtestwordhelp);


#ifdef HW_VERSION_ENABLE
int calc_hardware_ver( void )
{
	int val = 0; 
	return val;
}

static int cmd_boardversion(int argc, char *argv[])
{
  printf( "Board version is BSTD P%d.\n", calc_hardware_ver() );
  return 0;
}

static char boardversionhelp[] = "boardversion\n"
"Show board version.\n";
__commandlist(cmd_boardversion, "boardversion", boardversionhelp);

#endif

void cmd_blob_tftp_flash_image(int argc, char *argv[])
{
	unsigned long size;
/*
	if (!do_tftp("NTIM.bin", KERNEL_RAM_BASE, &size )){
		blob_nand_write( KERNEL_RAM_BASE , BLBO_NTIM_MEM_BASE, size, "-z");
	} else {
		printf("tftp NTIM.bin failed\n");
	}

	if (!do_tftp("TAVOR_LINUX_NTOBM.bin", KERNEL_RAM_BASE, &size )){
		blob_nand_write( KERNEL_RAM_BASE , BLBO_OBM_MEM_BASE, size, "-z");
	} else {
		printf("tftp TAVOR_LINUX_NTOBM.bin failed\n");
	}

	if (!do_tftp("blob", KERNEL_RAM_BASE, &size )){
		blob_nand_write( KERNEL_RAM_BASE , BLBO_BLOB_MEM_BASE0, size, "-z");
	} else {
		printf("tftp blob failed\n");
	}
*/



	if (!do_tftp("prdcfg.bin", KERNEL_RAM_BASE, &size )){
		blob_nand_write(KERNEL_RAM_BASE, BLBO_NVM_MEM_BASE, size, "-z");
	} else {
		printf("tftp prdcfg.bin failed\n");
	}

	if (!do_tftp("logo_pic.gz.out", KERNEL_RAM_BASE, &size )){
		blob_nand_write(KERNEL_RAM_BASE, BLBO_LOGO_MEM_BASE, size, "-z");
	} else {
		printf("tftp logo_pic.gz.out failed\n");
	}

	if (!do_tftp("zImage-r", KERNEL_RAM_BASE, &size )){
		blob_nand_write(KERNEL_RAM_BASE, BLBO_RKERNEL_MEM_BASE, size, "-z");
	} else {
		printf("tftp zImage-r failed\n");
	}

	if (!do_tftp("zImage-r-ramdisk-len", KERNEL_RAM_BASE, &size )){
		blob_nand_write(KERNEL_RAM_BASE, BLBO_RRAMDISK_MEM_BASE, size, "-z");
	} else {
		printf("tftp zImage-r-ramdisk-len failed\n");
	}

	if (!do_tftp("zImage", KERNEL_RAM_BASE, &size )){
		blob_nand_write(KERNEL_RAM_BASE, BLBO_KERNEL_MEM_BASE, size, "-z");
	} else {
		printf("tftp zImage failed\n");
	}

	if (!do_tftp("ramdisk.img", KERNEL_RAM_BASE, &size )){
		blob_nand_write(KERNEL_RAM_BASE, BLBO_RAMDISK_MEM_BASE, size, "-z");
	} else {
		printf("tftp ramdisk_len.img failed\n");
	}

	if (!do_tftp("system.img", KERNEL_RAM_BASE, &size )){
		blob_nand_erase(BLBO_SYSTEM_MEM_BASE, 
				BLBO_USRDATA_MEM_BASE - BLBO_SYSTEM_MEM_BASE, "-z" );
		blob_nand_write(KERNEL_RAM_BASE, BLBO_SYSTEM_MEM_BASE, size, "-y");
	} else {
		printf("tftp system.img.yaffs failed\n");
	}

	if (!do_tftp("userdata.img", KERNEL_RAM_BASE, &size )){
		blob_nand_erase(BLBO_USRDATA_MEM_BASE, 
				BLBO_LOCAL_MEM_BASE - BLBO_USRDATA_MEM_BASE, "-z" );
		blob_nand_write(KERNEL_RAM_BASE, BLBO_USRDATA_MEM_BASE, size, "-y");
	} else {
		printf("tftp userdata.img.yaffs failed\n");
	}

	if (!do_tftp("opl.img", KERNEL_RAM_BASE, &size )){
		blob_nand_erase(BLBO_OPL_MEM_BASE, 
				BLBO_FTM_MEM_BASE - BLBO_OPL_MEM_BASE, "-z" );
		blob_nand_write(KERNEL_RAM_BASE, BLBO_OPL_MEM_BASE, size, "-y");
	} else {
		printf("tftp opl.img.yaffs failed\n");
	}

	return;
}

static char blob_flash_image_help[] = "flash image in blob\n";
__commandlist(cmd_blob_tftp_flash_image, "blob_flash", blob_flash_image_help);

