/*
 * cep.c: Cep specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: cep.c,v 1.4 2003/09/04 17:39:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/led.h>
#include <blob/time.h>




/* flash descriptor for Cep flash */
/* 2x Intel 28F128J3A strataflash (16MB) */
static const flash_descriptor_t cep_flash_descriptors[] =
{
	{
		size: 2 * 128 * 1024,
		num: 64,
		lockable: 1
	},
	{
		/* NULL block */
	},
};




static void init_cep_flash_driver(void)
{
	flash_descriptors = cep_flash_descriptors;
	flash_driver = &intel32_flash_driver;
}

__initlist(init_cep_flash_driver, INIT_LEVEL_DRIVER_SELECTION);




#define RS232_ENABLE		0x00001000
#define GREEN_LED_ENABLE	0x00004000

static void cep_init_hardware(void)
{
	u32 *bcr = (u32 *)0x12000000;
	
	/* enable RS232 tranceiver */
	*bcr = RS232_ENABLE;

	/* select drivers */
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;

        /* switch on the LED D3 */
        *((volatile u8*)(0x40000004))= 0x02;
	
}


__initlist(cep_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
