/*
 * hackkit.c: hackkit specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2001,2002  Stefan Eletzhofer 
 *                               (stefan.eletzhofer@eletztrick.de)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: hackkit.c,v 1.7 2003/09/04 17:39:00 russd Exp $"

/**********************************************************************
 * includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/main.h>
#include <blob/arch.h>
#include <blob/errno.h>
#include <blob/error.h>
#include <blob/util.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/command.h>
#include <blob/uucodec.h>
#include <blob/time.h>
#include <blob/partition.h>
#include <blob/led.h>

#include <blob/arch.h>

/**********************************************************************
 * defines
 */

/* this will send a cold shiver through erik's spine ... */
#define	ERR( x )		{ ret = x; goto DONE; }

/* more readable IMHO */
#define MEM( x )        (*((u32 *)(x)))
#define SET(reg,bit) ((reg) |= (1<<(bit)))
#define RST(reg,bit) ((reg) &= ~(1<<(bit)))

#define HKIT_DEBUG 1

/**********************************************************************
 * globals
 */

extern blob_status_t blob_status;

/**********************************************************************
 * module globals
 */

#if HKIT_DEBUG
static int hkit_dbg = 1;
#else
static int hkit_dbg = 0;
#endif

/* flash descriptor for System3 flash. */
/* hackkit uses 2x INTEL 28F128J3A-150 flashes*/
static const flash_descriptor_t hackkit_flash_descriptors[] =
{
	{
		size: 2 * 128 * 1024,
		num: 128,
		lockable: 1
	},
	{
		/* NULL block */
	},
};


/**********************************************************************
 * static functions
 */
static char module_version[] = "$Id: hackkit.c,v 1.7 2003/09/04 17:39:00 russd Exp $";

/*********************************************************************
 * init_hackkit_flash - inits system 3 flash driver
 */
static void init_hackkit_flash_driver(void)
{
	flash_descriptors = hackkit_flash_descriptors;
	flash_driver = &intel32_flash_driver;
}
__initlist(init_hackkit_flash_driver, INIT_LEVEL_DRIVER_SELECTION);

/*********************************************************************
 * hackkit_init_hardware - inits system 3 LL hardware stuff
 */
static void hackkit_init_hardware(void)
{
	/* tweak blob config */
	blob_status.boot_delay = -1;
	//blob_status.terminalSpeed = baud_9600;

	/* select drivers */
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;
}
__initlist(hackkit_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
