/*
 * memsetup-s3c2500.S: memory setup for the s3c2500
 *
 * $Id: memsetup-s3c2500.S,v 1.1 2003/09/04 18:03:09 russd Exp $
 *
 * Copyright (C) 2001 Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003 Russ Dill (Russ.Dill@asu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * Documentation:
 *
 */

.ident "$Id: memsetup-s3c2500.S,v 1.1 2003/09/04 18:03:09 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

/***********************************************************************
 * NOTICE:
 * PLEASE dont use kernel headers. Why? Because it fscks up everything
 * as soon as they change. And yes, they change. And we're dependant of
 * the kernel version the user has installed, and that's bad.
 *
 * We want blob to compile right out of the box.
 *   - stefan
 */
#define BLOB_NEED_MEMCONFIG
#include <blob/arch.h>

.text
.globl memsetup

MEM_START:	.long	0x40000000

/*
 * CFGREG : SDRAM Configuration 0 register 
 * note that ras, rc, rcd, and rp regs are *actual* values, not
 * the value + 1 
 */
cfgreg:		.long	\
	(RAS(6) | RC(9) | RCD(3) | RP(3) | \
	 D1_128M | D0_128M | CL(3) | NO_AP | XW32)

 
/* CMDREG : SDRAM Configuration 1 register */
cmdreg:		.long	(0)

/* refresh rate */
refcyc:		.long	\
	(GET_MHz(BUS_SPEED) * 15 + ((GET_MHz(BUS_SPEED) * 6) / 10))

/* Flash config, may need to be put into the board file */
#define FLASH_SPEED 90
b0con:		.long	\
	(DW16 | BS(2) | \
	TACC((FLASH_SPEED * GET_MHz(BUS_SPEED) + 999) / 1000) | \
	TPA(0xE) | TACS(0) | TCOS(0) | TCOH(0))


/* Architecture headers can customized the memory setup behavior with
 * the following defines:
 *	 
 *   ARCH_SPECIFIC_MEMSETUP	
 *	arch does its own memort setup. must define memsetup somewhere
 *	which should do whatever it needs to do and do a standard
 *	assembly linkage return (mov pc, lr).
 *
 *   ARCH_SPECIFIC_MEMSETUP_STD
 *      arch needs std_memsetup procedure.  ignored except when
 *	ARCH_SPECIFIC_MEMSETUP is defined.
 */
	

/* -------------------------------------------------- */	
/* standard memory configuration */

#if !defined(ARCH_SPECIFIC_MEMSETUP) || defined(ARCH_SPECIFIC_MEMSETUP_STD)
	
.globl std_memsetup	
std_memsetup:
	/* Set up the SDRAM */
	mov	r0, #ASIC_BASE
	orr	r0, r0, #SDRAMB

	/* Step 1: Complete a power-on wait period (typically 100-200 usec)
	 * to allow clocks to come up to speed */
	mov	r4, #0x2800	/* DDD: way longer than 200 usec */
1:	subs	r4, r4, #1
	bne	1b

	/* Step 2: Program INIT[1:0] of the CFGREG to 01. This issues a 
	 * PALL command */
	/* CMDREG, CMDREG, bad documentation, BAD! */
	mov	r1, #PALL
	str	r1, [r0, #CMDREG]

	/* Step 3: Write 0xF into the refresh timer register.
	mov	r1, #0x20
	str	r1, [r0, #REFREG]

	/* Step 4: Wait 120 clock cycles to get at least 8 refreshes */
	mov	r4, #0x78
1:	subs	r4, r4, #1 /* more than 120 cycles */
	bne	1b

	/* Step 5: Program the normal operational value into the refresh timer */
	ldr	r1, refcyc
	str	r1, [r0, #REFREG]
	
	/* Step 6: Program the CFGREG to the normal value */
	ldr	r1, cfgreg
	str	r1, [r0, #CFGREG]	

	/* Step 7: Program the INIT[1:0] to b10 to issue an MRS command */
	mov	r1, #MRS
	str	r1, [r0, #CMDREG]

	/* Step 8: Program the INIT[1:0] to 00 to enter normal mode */
	mov	r1, #INIT(0)
	str	r1, [r0, #CMDREG]

	/* Step 9: Program the CMDREG and WBTOREG to normal values */
	ldr	r1, cmdreg
	str	r1, [r0, #CMDREG]

	mov	r1, #0
	str	r1, [r0, #WBTOREG]

	/* Setup the flash bank */
	mov	r0, #ASIC_BASE
	orr	r0, r0, #BCONB
	ldr	r1, b0con
	str	r1, [r0, #B0CON]

	mov	pc, lr
#endif

/* -------------------------------------------------- */	
	
#if !defined(ARCH_SPECIFIC_MEMSETUP)
memsetup:
	b	std_memsetup
#endif
	
	
/* -------------------------------------------------- */	
	
/* Arch specific setup */
