/*
 * memsetup-sa1110.S: memory setup for various SA1110 architectures
 *
 * $Id: memsetup-sa1110.S,v 1.19 2003/08/20 23:06:11 seletz Exp $
 *
 * Copyright (C) 2001 Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * Documentation:
 * Intel Corporation, "Intel StrongARM SA-1110 Microprocessor
 *     Advanced Developer's manual, December 1999
 *
 * Intel has a very hard to find SDRAM configurator on their web site:
 *   http://appzone.intel.com/hcd/sa1110/memory/index.asp
 *
 * NOTE: This code assumes that an SA1110 CPU *always* uses SDRAM. This
 * appears to be true, but it might be possible that somebody designs a
 * board with mixed EDODRAM/SDRAM memory (which is a bad idea). -- Erik
 *
 * 04-10-2001:	SELETZ
 * - separated memory config for multiple platform support
 * - perform SA1110 Hardware Reset Procedure
 *
 */

.ident "$Id: memsetup-sa1110.S,v 1.19 2003/08/20 23:06:11 seletz Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

/***********************************************************************
 * NOTICE:
 * PLEASE dont use kernel headers. Why? Because it fscks up everything
 * as soon as they change. And yes, they change. And we're dependant of
 * the kernel version the user has installed, and that's bad.
 *
 * We want blob to compile right out of the box.
 *   - stefan
 */
#define BLOB_NEED_MEMCONFIG
#include <blob/arch.h>
#include <blob/memsetup.h>

.text
.globl memsetup

MEM_REG_BASE:	.long	MEM_CONF_BASE
MEM_START:	.long	MEMORY_START
PWR_BASE:	.long	PM_BASE

/* Architecture headers can customized the memory setup behavior with
 * the following defines:
 *	 
 *   ARCH_SPECIFIC_MEMSETUP	
 *	arch does its own memort setup. must define memsetup somewhere
 *	which should do whatever it needs to do and do a standard
 *	assembly linkage return (mov pc, lr).
 *
 *   ARCH_SPECIFIC_MEMSETUP_STD
 *      arch needs std_memsetup procedure.  ignored except when
 *	ARCH_SPECIFIC_MEMSETUP is defined.
 */
	

/* -------------------------------------------------- */	
/* standard memory configuration */

#if !defined(ARCH_SPECIFIC_MEMSETUP) || defined(ARCH_SPECIFIC_MEMSETUP_STD)
	
MEMORY_CONFIG:
	.long	MDCNFG_VALUE		/* 0x0  MDCNFG */
	.long	MDCAS00_VALUE		/* 0x04 MDCAS00 */
	.long	MDCAS01_VALUE		/* 0x08 MDCAS01 */
	.long	MDCAS02_VALUE		/* 0x0c MDCAS02 */
	.long	MSC0_VALUE		/* 0x10 MSC0 */
	.long	MSC1_VALUE		/* 0x14 MSC1 */
	.long	MECR_VALUE		/* 0x18 MECR */
#ifdef MDREFR_DONT_BELIEVE_IN_MAGIC
	.long	MDREFR_VALUE
#else
	.long	0xdeadbeef		/* 0x1C MDREFR */
#endif		
	.long	MDCAS20_VALUE		/* 0x20 MDCAS20 */
	.long	MDCAS21_VALUE		/* 0x24 MDCAS21 */
	.long	MDCAS22_VALUE		/* 0x28 MDCAS22 */
	.long	MSC2_VALUE		/* 0x2C MSC2 */
	.long	SMCNFG_VALUE		/* 0x30 SMCNFG */

.globl std_memsetup	
std_memsetup:
	/* Set up the SDRAM */
	ldr	r0, MEM_REG_BASE
	adr	r1, MEMORY_CONFIG

	/* OK folks. These values are _offsets_ from
	 * MEM_REG_BASE and have _nothing_ to do with
	 * the kernel include file values, which defines
	 * them as _absolute_ values.
	 *
	 * The following works _only_ because the table above
	 * uses the same offsets.
	 */
	ldr	r2, [r1, #_MDCNFG ]
	str	r2, [r0, #_MDCNFG ]

	ldr	r2, [r1, #_MDCAS00 ]
	str	r2, [r0, #_MDCAS00 ]

	ldr	r2, [r1, #_MDCAS01 ]
	str	r2, [r0, #_MDCAS01 ]

	ldr	r2, [r1, #_MDCAS02 ]
	str	r2, [r0, #_MDCAS02 ]

	ldr	r2, [r1, #_MDCAS20 ]
	str	r2, [r0, #_MDCAS20 ]

	ldr	r2, [r1, #_MDCAS21 ]
	str	r2, [r0, #_MDCAS21 ]

	ldr	r2, [r1, #_MDCAS22 ]
	str	r2, [r0, #_MDCAS22 ]

	/*
	 * Refer to sections 9.5.3.7: Reviving DRAMS from Self-Refresh Mode
	 * and 10.2.1: Hardware or Sleep Reset Procedures
	 */
	/* Step 1: Complete a power-on wait period (typically 100-200 usec)
	 * to allow clocks to come up to speed */
	mov	r4, #0x2000        /* DDD: way longer than 200 usec */
spin:	subs	r4, r4, #1
	bne	spin

	/* Step 2 is for systems with SMROM, so we'll skip it. */

	/* Step 3: clear DH
	 * NOTE: do NOT clear SSS, the kernel wants it set */
	ldr	r1, PWR_BASE
	mov	r2, #PSSR_DH
	str	r2, [r1, #_PSSR]    /* clear DH by writing 1 to it */


#ifdef MDREFR_DONT_BELIEVE_IN_MAGIC
	ldr	r2, [r1, #_MDREFR ]
	str	r2, [r0, #_MDREFR ]
#else
	/* clear KxDB2 */
	ldr r2, [ r0, #_MDREFR ]
	bic r2, r2, #MDREFR_K0DB2
	bic r2, r2, #MDREFR_K1DB2
	bic r2, r2, #MDREFR_K2DB2
	bic r2, r2, #0xff          /* clear TRASR and DRI */
	bic r2, r2, #0xff00
	str r2, [ r0, #_MDREFR ]

	/* set TRASR and DRI, KxDB2 */
	ldr r2, [ r0, #_MDREFR ]
	orr r2, r2, #MDREFR_TRASR(7)
	orr r2, r2, #MDREFR_DRI(12)
	/* only set DB2 bits when _not_ running
	 * the SDRAM at full clock speed
	 */
#ifndef MDREFR_MEMCLK_FULLSPEED_0
	orr r2, r2, #MDREFR_K0DB2
#endif
#ifndef MDREFR_MEMCLK_FULLSPEED_1
	orr r2, r2, #MDREFR_K1DB2
#endif
#ifndef MDREFR_MEMCLK_FULLSPEED_2
	orr r2, r2, #MDREFR_K2DB2
#endif
	str r2, [ r0, #_MDREFR ]

	/* set KxRUN */
	ldr r2, [ r0, #_MDREFR ]
	orr r2, r2, #MDREFR_K0RUN
	orr r2, r2, #MDREFR_K1RUN
	orr r2, r2, #MDREFR_K2RUN
	str r2, [ r0, #_MDREFR ]

	/* clear SLFRSH */
	ldr r2, [ r0, #_MDREFR ]
	bic r2, r2, #MDREFR_SLFRSH
	str r2, [ r0, #_MDREFR ]

	/* toggle E1PIN (set -> clear ) */
	ldr r2, [ r0, #_MDREFR ]
	orr r2, r2, #MDREFR_E1PIN
	str r2, [ r0, #_MDREFR ]
#endif

	/* Issue read requests to disabled bank to start refresh */
	ldr	r1, MEM_START
.rept	8	
	ldr	r0, [r1]
.endr

	ldr	r0, MEM_REG_BASE
	adr r1, MEMORY_CONFIG
	
	/* ENABLE SDRAM BANKS */
	ldr	r2, [r0, #_MDCNFG ]
	orr	r2, r2, #0x00000003
	orr	r2, r2, #0x00030000
	str	r2, [r0, #_MDCNFG]

	/* OPTIONALLY enable Autopowerup/down */
#if 0
	ldr r2, [ r0, #_MDREFR ]
	orr r2, r2, #MDREFR_EAPD
	orr r2, r2, #MDREFR_KAPD
	str r2, [ r0, #_MDREFR ]
#endif

	ldr	r2, [r1, #_MCS0 ]
	str	r2, [r0, #_MCS0 ]

	ldr	r2, [r1, #_MCS1 ]
	str	r2, [r0, #_MCS1 ]

	ldr	r2, [r1, #_MCS2 ]
	str	r2, [r0, #_MCS2 ]

	ldr	r2, [r1, #_SMCNFG ]
	str	r2, [r0, #_SMCNFG ]

	ldr	r2, [r1, #_MECR ]
	str	r2, [r0, #_MECR ]
	
	mov	pc, lr
#endif

/* -------------------------------------------------- */	
	
#if !defined(ARCH_SPECIFIC_MEMSETUP)
memsetup:
	b	std_memsetup
#endif
	
	
/* -------------------------------------------------- */	
	
/* Arch specific setup */
