#include	"onenand_regs.h"
#include	"onenand.h"

#define		ERR_NONE	(0x00)
#define		ERR_TIMEOUT	(0x11)
#define		ERR_ECC		(0x12)
#define		ERR_CONTROLLER	(0x13)
#define		ERR_LOCK	(0x14)
#define		ERR_ERASE	(0x15)
#define		ERR_GENERIC	(0x16)

#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
extern void udelay(unsigned int usec);

struct ecc_layout {
	int	offset;
	int	length;
};

/* In Onenand document and Linux kernel driver. The offset entry
 * 1 is not avaliable. But to keep compatible with native nand
 * bbt, add it in blob. -- yfw
 */
struct ecc_layout onenand_ecc_layout[] = {
	{0, 2},	/* this entry is for mark bad information */
	{2, 3},
	{14, 2},
	{18, 3},
	{30, 2},
	{34, 3},
	{46, 2},
	{50, 3},
	{62, 2},
};

static struct onenand_info mhn_onenand_info = {
	.base		=  (void *)0x10000000,
#ifdef ONENAND_BURST_MODE
	.pDmacRegsBase	=  (void *)0x40000000,
#endif
};

static unsigned short onenand_readw(void *addr)
{
	return readw(addr);
}

static void onenand_writew(unsigned short value, void *addr)
{
	writew(value, addr);
}

#define ONENAND_CHECK_BYTE_ACCESS(addr)         (addr & 0x1)
#ifndef ONENAND_BURST_MODE
static void *memcpy16(void *dest, const void *src, size_t n)
{
	const u16 *s = (const u16 *)src;
	u16 *d = (u16 *)dest;
	size_t n16 = n >> 1;

	while (n16-- > 0)
		*d++ = *s++;

	return dest;
}
#else
void onenand_mmcontrol(struct onenand_info *info, int sync_read)
{
	static unsigned int csadrcfg2, msc1, sxcnfg;
	static unsigned int syscfg;

	if (sync_read) {
		csadrcfg2 = CSADRCFG2;
		msc1 = MSC1;
		sxcnfg = SXCNFG;
		CSADRCFG2 = 0x0032091d;
		MSC1 = 0x0f180028;
		SXCNFG = (sxcnfg & 0xffff) | 0x30110004;
		syscfg = onenand_readw(info->base + ONENAND_REG_SYS_CFG1);
		onenand_writew(0xc1c0,
			info->base + ONENAND_REG_SYS_CFG1);
	} else {
		onenand_writew(syscfg,
			info->base + ONENAND_REG_SYS_CFG1);
		SXCNFG = sxcnfg;
		MSC1 = msc1;
		CSADRCFG2 = csadrcfg2;
	}
}

#define DMA_CNUM	0
extern void xsc3_dma_flush_range(void *start, void *end);
int setupDMA(struct onenand_info *info, void *dst, void *src, size_t len)
{
	XLLP_DMAC_COMMAND_T cmd;
	u32 channel = DMA_CNUM;
	u32 timeout = 2000;

	info->pDmacRegsBase->drcmr2[9] = 0x80 | (channel & 0x3f);
	cmd.aLen = (XLLP_INT16_T) len;
	cmd.aWidth = XLLP_DMAC_WIDTH_16;
	cmd.aSize = XLLP_DMAC_BURSTSIZE_32;
	cmd.aEndian = 0;
	cmd.aEndIrqEn = XLLP_TRUE; //Enable end interrupt
	cmd.aStartIrqEn = XLLP_FALSE;

	cmd.aFlowTrg = XLLP_FALSE;
#ifdef CONFIG_SECURITY_SUPPORT
	cmd.aFlowSrc = XLLP_FALSE;
#else
	cmd.aFlowSrc = XLLP_TRUE;
#endif
	cmd.aIncTrgAddr = XLLP_TRUE;
	cmd.aIncSrcAddr = XLLP_TRUE;
	XllpDmacCfgChannelNoDescTransfer(info->pDmacRegsBase,
	(XLLP_UINT32_T)src, (XLLP_UINT32_T)dst, &cmd,
	channel, XLLP_DMAC_MEM2MEM_MOVE);

	xsc3_dma_flush_range(dst, (dst + len));
	XllpDmacStartTransfer(info->pDmacRegsBase, channel);

	/* Wait for INT register low to high transition */
	while (1) {
		if ((info->pDmacRegsBase->dint & (1 << channel))
			|| (timeout <= 0)) {
			info->pDmacRegsBase->dcsr[channel] = XLLP_DMAC_DCSR_END_INTR;
			break;
		}
		udelay(1);

		timeout--;
	}

	if (!timeout) {
		return -ERR_TIMEOUT;
	}
	XllpDmacStopTransfer(info->pDmacRegsBase,  channel);
	return 0;
}
#endif

static int onenand_read_bufferram(struct onenand_info *info, int area,
		unsigned char *buffer, int offset, size_t count)
{
	void *bufferram;

	bufferram = info->base + area;
#ifdef ONENAND_BURST_MODE
	onenand_mmcontrol(info, ONENAND_SYS_CFG1_SYNC_READ);
#endif
	if (ONENAND_CHECK_BYTE_ACCESS(count)) {
		unsigned short word;

		/* Align with word(16-bit) size */
		count--;

		/* Read word and save byte */
		word = onenand_readw(bufferram + offset + count);
		buffer[count] = (word & 0xff);
	}

#ifdef ONENAND_BURST_MODE
	setupDMA(info, buffer, bufferram + offset, count);
	onenand_mmcontrol(info, 0);
#else
	memcpy16(buffer, bufferram + offset, count);
#endif

	return 0;
}

/* Convert generic oob buf to Onenand oob buf layout.
 * This function should be called before Onenand write operation.
 */
static int convert_oob_before_write(unsigned char *oobbuf,
		unsigned char *buf, int __attribute__((unused)) len)
{
	unsigned int i, count = 0;
	unsigned char *tmp = buf;

	for (i = 0; i < ARRAY_SIZE(onenand_ecc_layout); i++) {
		memcpy(oobbuf + onenand_ecc_layout[i].offset,
			tmp, onenand_ecc_layout[i].length);
		tmp += onenand_ecc_layout[i].length;
		count += onenand_ecc_layout[i].length;
	}

	return count;
}

/* Convert Onenand oob to generic oob layout.
 * This function should be called after Onenand read operation.
 */
static int convert_oob_after_read(unsigned char *oobbuf, unsigned char *buf, int len)
{
	unsigned int i;
	unsigned char *tmp = buf;

	for (i = 0; i < ARRAY_SIZE(onenand_ecc_layout); i++) {
		memcpy(tmp, oobbuf + onenand_ecc_layout[i].offset,
			onenand_ecc_layout[i].length);
		tmp += onenand_ecc_layout[i].length;
		len += onenand_ecc_layout[i].length;
	}

	return len;
}

/* We always enable HW ECC for Onenand -- yfw */
static inline void enable_hw_ecc(int __attribute__((unused)) enable,
			struct nand_info __attribute__((unused)) *info)
{
	return;
} 

static inline int onenand_block_address(struct onenand_info *info, int block)
{
	if (block & info->density_mask)
		return ONENAND_DDP_CHIP1 | (block ^ info->density_mask);

	return block;
}

static inline int onenand_page_address(int page, int sectors)
{
	int fpa, fsa;

	fpa = page & ONENAND_FPA_MASK;
	fsa = sectors & ONENAND_FSA_MASK;

	return ((fpa << ONENAND_FPA_SHIFT) | fsa);
}

static inline int onenand_buffer_address(int dataram1, int sectors, int count)
{
	int bsa, bsc;

	bsa = sectors & ONENAND_BSA_MASK;

	if (dataram1)
		bsa |= ONENAND_BSA_DATARAM1;
	else
		bsa |= ONENAND_BSA_DATARAM0;

	bsc = count & ONENAND_BSC_MASK;

	return ((bsa << ONENAND_BSA_SHIFT) | bsc);
}

static inline int onenand_bufferram_address(struct onenand_info *info, int block)
{
	if (block & info->density_mask)
		return ONENAND_DDP_CHIP1;
	
	return ONENAND_DDP_CHIP0;
}

static int onenand_read_one_page(struct onenand_info *info, int page,
		int __attribute__((unused)) *eccpage)
{
	int value, block, timeout = 2000;
	int page_to_block_shift;
	int sectors = 4, count = 4, dataram = 0;
	unsigned short interrupt, ctrl;

	page_to_block_shift = info->phys_erase_shift - info->page_shift;
	block = page >> page_to_block_shift;

	/* write 'DFS, FBA' of Flash */
	value = onenand_block_address(info, block);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS1);

	/* write 'FPA, FSA' of Flash */
	value = onenand_page_address(page, sectors);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS8);

	/* write 'BSA, BSC' of DataRAM */
	value = onenand_buffer_address(dataram, sectors, count);
	onenand_writew(value, info->base + ONENAND_REG_START_BUFFER);

	/* Select DataRAM for DDP */
	value = onenand_bufferram_address(info, block);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS2);

	/* Write 0 to Interrupt register */
	onenand_writew(0, info->base + ONENAND_REG_INTERRUPT);

	/* Write Load Command */
	onenand_writew(ONENAND_CMD_READ, info->base + ONENAND_REG_COMMAND);

	/* Wait for INT register low to high transition */
	while (1) {
		interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);

		if ((interrupt & (1 << 15)) || (timeout <= 0))
			break;

		udelay(10);
		timeout--;
	}

	if (!timeout) {
		return -ERR_TIMEOUT;
	}

	interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);
	ctrl = onenand_readw(info->base + ONENAND_REG_CTRL_STATUS);

	if (ctrl & ONENAND_CTRL_ERROR) {
		printf("Onenand controller error: 0x%04x\n", ctrl);
		return -ERR_CONTROLLER;
	}

	if (interrupt & ONENAND_INT_READ) {
		int ecc = onenand_readw(info->base + ONENAND_REG_ECC_STATUS);
		if (ecc) {
			printf("Page 0x%x ECC error = 0x%04x\n", page, ecc);

			if (ecc & ONENAND_ECC_2BIT_ALL) {
				return -ERR_ECC;
			}
		}
	}

	return 0;
}

static int onenand_read(struct onenand_info *info, unsigned char *buf,
	int start_page, int page_num, int *eccpage)
{
	int ret;
	int page = start_page;

	while (page_num > 0) {
		ret = onenand_read_one_page(info, page, eccpage);
		if (ret) {
			if (-ERR_ECC == ret) {
				*eccpage = page;
			}
			return ret;
		}
		/* We always use DATARAM0 to read from Onenand */
		onenand_read_bufferram(info, ONENAND_DATARAM, buf, 0,
			info->page_size);
		convert_oob_after_read(info->base + ONENAND_SPARERAM,
				buf + info->page_size, info->spare_size);
		page++;
		page_num--;
		buf += (info->page_size + info->spare_size);
	}

	return 0;
}

/* read one page */
/* Need make this function can work with native NAND and Onenand -- yfw */
int mhn_read_one_page(struct nand_info *info, u8 *buf, u32 page, int option)
{
	int block, reloc_block;
	int pagesize, pages_shift;
	int eccpage = 0xFFFFFFFF;	/* ecc error page number */
	int status;

	/* page means the page number
	 * If current block is a bad block, relocate the page
	 * to a correct one in a relocated block
	 */
	pages_shift = info->phys_erase_shift - info->page_shift;
	block = page >> pages_shift;
	if (option != READ_RAW && is_bad_block(block, info))
		return -EBADBLOCK;

	reloc_block = search_rel_block(block, info);
	if (option != READ_RAW && reloc_block != block) {
		page = (reloc_block << pages_shift) + (page
			& ((1 << pages_shift) - 1));
	}

	pagesize = 1 << info->page_shift;
	memset(buf, 0xFF, pagesize + info->oobsize);

	status = onenand_read(info->onenand_info, buf, page, 1, &eccpage);
	if (status != ERR_NONE) {
		printf("Onenand read. ret: %d, page: 0x%x, eccpage: 0x%x\n",
			status, page, eccpage);
		return -ECMDFAIL;
	}
	return 0;
}

static int onenand_erase_one_block(struct onenand_info *info, unsigned int block,
		int __attribute__((unused)) *badblock)
{
	int value, timeout = 2000;
	unsigned short ctrl, interrupt;

	/* Write 'DFS, FBA' of Flash */
	value = onenand_block_address(info, block);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS1);

	/* Select DataRAM for DDP */
	value = onenand_bufferram_address(info, block);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS2);

	/* Write 0 to interrupt register */
	onenand_writew(ONENAND_INT_CLEAR, info->base + ONENAND_REG_INTERRUPT);

	/* Write 'Erase' Command */
	onenand_writew(ONENAND_CMD_ERASE, info->base + ONENAND_REG_COMMAND);

	/* Wait for INT register low to high transition */
	while (1) {
		interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);

		if ((interrupt & (1 << 15)) || (timeout <= 0))
			break;

		udelay(10);
		timeout--;
	}

	if (!timeout) {
		return -ERR_TIMEOUT;
	}

	interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);
	ctrl = onenand_readw(info->base + ONENAND_REG_CTRL_STATUS);

	if (ctrl & ONENAND_CTRL_LOCK) {
		printf("Onenand: Try to erase block locked\n");
		return -ERR_LOCK;
	}

	if (ctrl & ONENAND_CTRL_ERROR) {
		printf("Onenand controller error: 0x%04x\n", ctrl);
		return -ERR_CONTROLLER;
	}

	return ERR_NONE;
}

static int onenand_erase(struct onenand_info *info, unsigned int start_block,
		int block_num, int *badblock)
{
	int ret;
	int block = start_block;

	while (block_num > 0) {
		ret = onenand_erase_one_block(info, block, badblock);
		if (ret) {
			if (-ERR_CONTROLLER == ret) {
				*badblock = block;
			}
			return ret;
		}
		block++;
		block_num--;
	}

	return 0;
}

static int mhn_erase_direct(struct nand_info *info, unsigned int start_block,
		int block_num, int *badblock)
{
	int ret;
	int block = start_block;

	while (block_num > 0) {
		ret = onenand_erase_one_block(info->onenand_info, block, badblock);
		if (ret) {
			if (-ERR_CONTROLLER == ret) {
				*badblock = block;
			}
			return ret;
		}
		block++;
		block_num--;
	}

	return 0;
}

/* erase one block, buf length must be larger than info->pagesize
 * + info->oobsize
 */
int mhn_erase_one_block(struct nand_info *info, u32 block)
{
	int reloc_block;
	int status;
	int badblock = 0xFFFFFFFF;
	int start;

	if (is_bad_block(block, info))
		return -EBADBLOCK;

	reloc_block = search_rel_block(block, info);
	if (reloc_block)
		start = reloc_block;
	else
		start = block;

	status = onenand_erase(info->onenand_info, start, 1, &badblock);
	if (status != ERR_NONE) {
		printf("onenand_erase failed. ret: %d, block: 0x%x, badblock: 0x%x\n",
				status, block, badblock);
	}
	if (badblock == start) {
		/* If there's an relocated block, return 0.
		 * Otherwise, return error.
		 */
		return mark_badblock(block, info);
	}

	if (status != ERR_NONE)
		return  -ECMDFAIL;

	return 0;
}

int onenand_write_one_page(struct onenand_info *info, unsigned char *buf,
		int page, int __attribute__((unused)) *badblock)
{
	int value, block, timeout = 2000;
	int page_to_block_shift;
	int sectors = 4, count = 4, dataram = 0;
	unsigned short interrupt, ctrl;

	page_to_block_shift = info->phys_erase_shift - info->page_shift;
	block = page >> page_to_block_shift;

	/* Select DataRAM for DDP */
	value = onenand_bufferram_address(info, block);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS2);

	/* Write Data into DataRAM */
	memcpy(info->base + ONENAND_DATARAM, buf, info->page_size);
	convert_oob_before_write(info->base + ONENAND_SPARERAM,
			buf + info->page_size, info->spare_size);

	/* FIXME: this #97 bad block WA should be removed later */
	if(*((unsigned char *)(info->base + ONENAND_SPARERAM)) != 0xff){
		printf("Onenand controller warning: the first byte is 0x%02x\n",
			*((unsigned char *)(info->base + ONENAND_SPARERAM)));
		*((unsigned char *)(info->base + ONENAND_SPARERAM)) =
			*(buf + info->page_size);
	}

	/* Write 'DFS, FBA' of Flash */
	value = onenand_block_address(info, block);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS1);

	/* Write 'FPA, FSA' of Flash */
	value = onenand_page_address(page, sectors);
	onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS8);

	/* Write 'BSA, BSC' of DataRAM */
	value = onenand_buffer_address(dataram, sectors, count);
	onenand_writew(value, info->base + ONENAND_REG_START_BUFFER);

	/* Write 0 to interrupt register */
	onenand_writew(ONENAND_INT_CLEAR, info->base + ONENAND_REG_INTERRUPT);

	/* Write 'Program' Command */
	onenand_writew(ONENAND_CMD_PROG, info->base + ONENAND_REG_COMMAND);

	/* Wait for INT register low to high transition */
	while (1) {
		interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);

		if ((interrupt & (1 << 15)) || (timeout <= 0))
			break;

		udelay(10);
		timeout--;
	}

	if (!timeout) {
		return -ERR_TIMEOUT;
	}

	interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);
	ctrl = onenand_readw(info->base + ONENAND_REG_CTRL_STATUS);

	if (ctrl & ONENAND_CTRL_LOCK) {
		printf("Onenand: Try to erase block locked\n");
		return -ERR_LOCK;
	}

	if (ctrl & ONENAND_CTRL_ERROR) {
		printf("Onenand controller error: 0x%04x\n", ctrl);
		return -ERR_CONTROLLER;
	}

	return ERR_NONE;
}

static int onenand_write(struct onenand_info *info, unsigned char *buf,
	int start_page, int page_num, int *badblock)
{
	int ret;
	int page = start_page;
	int pages_shift, block;

	pages_shift = info->phys_erase_shift - info->page_shift;

	while (page_num > 0) {
		ret = onenand_write_one_page(info, buf, page, badblock);
		if (ret) {
			if (-ERR_LOCK == ret) {
				printf("Onenand write: try to write block locked\n");
			}

			block = page >> pages_shift;
			if (-ERR_CONTROLLER == ret) {
				*badblock = block;
			}
			return ret;
		}
		page++;
		page_num--;
		buf += (info->page_size + info->spare_size);
	}

	return 0;
}

/* Need make this function can work with native NAND and Onenand -- yfw */
int mhn_write_one_page(struct nand_info *info, u8 * buf, u32 page,
			int __attribute__((unused)) hwecc)
{
	int reloc_block, block;
	int pages_shift;
	int status;
	int badblock = 0xFFFFFFFF;

	pages_shift = info->phys_erase_shift - info->page_shift;
	/* calculate the page address with relocated block, if exists */
	block = page >> pages_shift;

	if (is_bad_block(block, info))
		return -EBADBLOCK;

	reloc_block = search_rel_block(block, info);
	if (reloc_block != block) {
		//DPRINTF("relocate page %x ", page);
		page = (reloc_block << pages_shift) + (page
			& ((1 << pages_shift) - 1));
		//DPRINTF(" to %x\n", page);
	}

	status = onenand_write(info->onenand_info, buf, page, 1, &badblock);
	if (status) {
		printf("Onenand write. ret: %d, page: 0x%x, badblock: 0x%x\n",
			status, page << info->page_shift, badblock);
	}

	if (badblock == reloc_block) {
		/* If there's an relocated block, return 0.
		 * Otherwise, return error.
		 */
		 return mark_badblock(block, info);
	}

	if (status != ERR_NONE)
		return -ECMDFAIL;

	return 0;
}

/* check whether a page and spare area is empty
 * size should be sum of page size and spare area length
 */
inline int is_buf_blank(u8 * buf, int size)
{
	int i = 0;
	while (i < size) {
		if (*((unsigned long *)(buf + i)) != 0xFFFFFFFF)
			return 0;
		i += 4;
	}
	if (i > size) {
		i -= 4;
		while (i < size) {
			if (*(buf + i) != 0xFF)
				return 0;
			i++;
		}
	}
	return 1;
}

static int onenand_unlock(struct onenand_info *info, int start_block, int block_num)
{
	int value, block, timeout = 100;
	unsigned short ctrl, interrupt, status;

	for (block = start_block; block < start_block + block_num; block++) {
		/* Write 'DFS' of Flash */
		value = onenand_block_address(info, block);
		onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS1);

		/* Select DataRAM for DDP */
		value = onenand_bufferram_address(info, block);
		onenand_writew(value, info->base + ONENAND_REG_START_ADDRESS2);

		/* Write 'SBA' of Flash */
		onenand_writew(block, info->base + ONENAND_REG_START_BLOCK_ADDRESS);

		/* Write 0 to interrupt register */
		onenand_writew(ONENAND_INT_CLEAR, info->base + ONENAND_REG_INTERRUPT);

		/* Write 'unlock' command */
		onenand_writew(ONENAND_CMD_UNLOCK, info->base + ONENAND_REG_COMMAND);

		/* Wait for INT register low to high transition */
		while (1) {
			interrupt = onenand_readw(info->base + ONENAND_REG_INTERRUPT);

			if ((interrupt & (1 << 15)) || (timeout <= 0))
				break;

			udelay(10);
			timeout--;
		}

		ctrl = onenand_readw(info->base + ONENAND_REG_CTRL_STATUS);
		if (ctrl & ONENAND_CTRL_ERROR) {
			printf("Onenand controller error: 0x%04x\n", ctrl);
			return -ECMDFAIL;
		}

		status = onenand_readw(info->base + ONENAND_REG_WP_STATUS);
		if (!(status & ONENAND_WP_US)) {
			printf("%s: unlock block %d failed. current lock status: 0x%x\n",
				__func__, block, status);
		}
	}
	return 0;
}

int unlock_nand(struct nand_info *info)
{
	return onenand_unlock(info->onenand_info, 0,
		((info->chipsize >> info->phys_erase_shift) + nand_reloc_max));
}

/**
 * onenand_print_device_info - Print device & version ID
 * @param device        device ID
 * @param version       version ID
 *
 * Print device & version ID
 *
 */
static void onenand_print_device_info(int device, int version)
{
	int vcc, demuxed, ddp, density;

	vcc = device & ONENAND_DEVICE_VCC_MASK;
	demuxed = device & ONENAND_DEVICE_IS_DEMUX;
	ddp = device & ONENAND_DEVICE_IS_DDP;
	density = device >> ONENAND_DEVICE_DENSITY_SHIFT;
	printf("%sOneNAND%s %dMB %sV 16-bit (0x%02x)\n",
			demuxed ? "" : "Muxed ",
			ddp ? "(DDP)" : "",
			(16 << density),
			vcc ? "2.65/3.3" : "1.8",
			device);
	printf("OneNAND version = 0x%04x\n", version);
}

static void monahans_smem_init(void)
{
	CSADRCFG2 = 0x00320919;
	MSC1 = (MSC1 & 0xFFFF0000) | 0x00007229;

	return;
}

void nand_driver_init(void)
{
	int manufacturer, device, version;
	int timeout=2000;
	unsigned short interrupt;

	/* Initialize the static memory controller for onenand */
	monahans_smem_init();

	monahans_nand.onenand_info = &mhn_onenand_info;
	memset(buf, 0 , BUFLEN);

	/* Read manufacturer and device IDs from Register */
	manufacturer = onenand_readw(mhn_onenand_info.base + ONENAND_REG_MANUFACTURER_ID);
	device = onenand_readw(mhn_onenand_info.base + ONENAND_REG_DEVICE_ID);
	version = onenand_readw(mhn_onenand_info.base + ONENAND_REG_VERSION_ID);

	onenand_writew(0xF3, mhn_onenand_info.base + ONENAND_BOOTRAM);
	while (1) {
		interrupt = onenand_readw(mhn_onenand_info.base + ONENAND_REG_INTERRUPT);

		if (((interrupt & (1 << 15) && interrupt & (1 << 4))) || (timeout <= 0))
			break;

		udelay(10);
		timeout--;
	}

	onenand_writew(0xF0, mhn_onenand_info.base + ONENAND_BOOTRAM);
	while (1) {
		interrupt = onenand_readw(mhn_onenand_info.base + ONENAND_REG_INTERRUPT);

		if (((interrupt & (1 << 15) && interrupt & (1 << 4))) || (timeout <= 0))
			break;

		udelay(10);
		timeout--;
	}

	onenand_print_device_info(device, version);
	monahans_nand.maf_id = manufacturer;
	monahans_nand.dev_id = device;

	mhn_onenand_info.density = device >> ONENAND_DEVICE_DENSITY_SHIFT;
	mhn_onenand_info.chipsize = (16 << mhn_onenand_info.density) << 20;

	if (device & ONENAND_DEVICE_IS_DDP)
		mhn_onenand_info.density_mask = (1 << (mhn_onenand_info.density + 6));
	else
		mhn_onenand_info.density_mask = 0;

	mhn_onenand_info.page_size = onenand_readw(mhn_onenand_info.base + ONENAND_REG_DATA_BUFFER_SIZE);
	mhn_onenand_info.spare_size = mhn_onenand_info.page_size >> 5;
	mhn_onenand_info.erasesize = mhn_onenand_info.page_size << 6;

	mhn_onenand_info.phys_erase_shift = fls(mhn_onenand_info.erasesize);
	mhn_onenand_info.page_shift = fls(mhn_onenand_info.page_size);
	mhn_onenand_info.page_mask = (1 << (mhn_onenand_info.phys_erase_shift - mhn_onenand_info.page_shift)) - 1;

	monahans_nand.page_shift = mhn_onenand_info.page_shift;
	monahans_nand.phys_erase_shift = mhn_onenand_info.phys_erase_shift;

	nand_reloc_max = ((mhn_onenand_info.chipsize / mhn_onenand_info.erasesize) / 100) * 2;
	monahans_nand.chipsize = mhn_onenand_info.chipsize - ((nand_reloc_max) << monahans_nand.phys_erase_shift);


	monahans_nand.oobsize = mhn_onenand_info.spare_size;
	monahans_nand.eccbytes = 20;
	monahans_nand.hwecc = 20;
	monahans_nand.badblockpos = 0;
	monahans_nand.badblockbytes = 2;

	monahans_bbt_main.veroffs = 6;
	monahans_bbt_main.offs = 2;
	monahans_bbt_mirror.veroffs = 6;
	monahans_bbt_mirror.offs = 2;

	monahans_nand.td = &monahans_bbt_main;
	/* mirror */
	monahans_nand.md = &monahans_bbt_mirror;

	unlock_nand(&monahans_nand);
	scan_nand(&monahans_nand);

	return;
}
