/*
 * shannon.c: SHANNON specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: shannon.c,v 1.14 2003/09/04 17:39:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/main.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/led.h>
#include <blob/time.h>

/* flash descriptor for SHANNON flash */
/* 2x AMD *whatever* flash (4MB) */
static const flash_descriptor_t shannon_flash_descriptors[] =
{
	
	{ 
		size: 2 * 16 * 1024,
		num: 1,
	},
	{ 
		size: 2 * 8 * 1024,
		num: 2,
	},
	{ 
		size: 2 * 32 * 1024,
		num: 1,
	},
	{
		size: 2 * 64 * 1024,
		num: 31,
	},
	{
		/* NULL block */
	},
};

static void init_shannon_flash_driver(void)
{
	flash_descriptors = shannon_flash_descriptors;
	flash_driver = &amd32_flash_driver;
}

__initlist(init_shannon_flash_driver, INIT_LEVEL_DRIVER_SELECTION);

static void shannon_init_hardware(void)
{
	/* select drivers */
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;

	/* tweak blob config */
	blob_status.boot_delay = 1;
}

__initlist(shannon_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
