/*
 * start.s: Mainstone/Bulverde start code using XLLI
 *
 * Copyright (C) 2004 Intel Corporation (yu.tang@intel.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

        .include  "xlli_tavor_defs.inc"                 @ Processor specific include file
        .include  "xlli_tavor_pdk_defs.inc"                @ Tavor PDK specific include file

#define MFP_I2C_SCL 0x40e106ac
#define MFP_I2C_SDA 0x40e10680
#define SCL_GPIO    89
#define SDA_GPIO    90

.text

/*
 * We start by implementing *all* exception vectors
 *
 * Reset vector: this initialises the machine
 * note here that this not yet taken sleep wakeup into account -- lets just
 * get something to work first.
 */
.macro blink, count
	mov		r3, \count
	b		endless_blink
.endm

.macro cpwait reg
	mrc  p15,0,\reg,c2,c0,0
	mov  \reg,\reg
	sub  pc,pc,#4
.endm

.globl reset
reset:
2:
	/* setup SP pointer */
	ldr	sp, =0xb0260000

        /* set CPU to SVC32 mode */
        mrs	r0, cpsr
        bic	r0, r0, #0x1f
        orr	r0, r0, #0x13
        msr	cpsr, r0
	
       /* turn everything off (caches off, MMU off, etc.) */
        mov	r0, #0x78
        mcr	p15, 0, r0, c1, c0, 0

chk_gpio_reset:
	/* check GPIO reset */
	ldr	r1, =0x40f40004
	ldr	r0, [r1]
	tst	r0, #8
	beq	control_i2c

	/* clear GPIO flag */
	/*mov	r2, #8*/
	/*str	r2, [r1]*/

control_i2c:
        /* control I2C */
	ldr		r0, =0xa840
	ldr		r1, =MFP_I2C_SCL
        str		r0, [r1]
	ldr		r1, =MFP_I2C_SDA
	str		r0, [r1]        /* configure SCL/SDA as gpio pin */
	ldr		r4, =0x40e00000
	mov		r5, #(1<<(SCL_GPIO%32)) /* corresponding gpio bit */
	ldr		r0, [r4, #(0x0c + 4*(SCL_GPIO/32))]	/* GPDR for SCL */
	orr		r0, r0, r5
	str		r0, [r4, #(0x0c + 4*(SCL_GPIO/32))] /* set SCL as output */
	mov		r5, #(1<<(SDA_GPIO%32)) /* corresponding gpio bit */
	ldr		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))]	/* GPDR for SDA */
	bic		r0, r0, r5
	str		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))] /* set SDA as input*/


loop_scl:
	mov		r2, #0x1000
loop1:
	subs		r2, r2, #1
	bne		loop1
	mov		r5, #(1<<(SCL_GPIO%32)) /* corresponding gpio bit */
	str		r5, [r4, #(0x18 + 4*(SCL_GPIO/32))]	/* GPSR: set SCL high */
	mov		r2, #0x1000
loop2:
	subs		r2, r2, #1
	bne		loop2
	str		r5, [r4, #(0x24 + 4*(SCL_GPIO/32))]	/* GPCR: set SCL low */
        mov		r2, #0x1000
loop3:
	subs		r2, r2, #1
	bne		loop3
	ldr		r0, [r4, #(0 + 4*(SDA_GPIO/32))]  /* GPLR for SDA */
        tst 		r0, #(1<<(SDA_GPIO%32)) /* Check SDA low or high */
	beq		loop_scl /* do until SDA is high */
	
	mov 		r5, #(1<<(SDA_GPIO%32))
	ldr		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))] /* GPDR for SDA */
	orr		r0, r0, r5
        str		r0, [r4, #(0x0c + 4*(SDA_GPIO/32))] /* GPDR: set SDA as output */
	mov		r2, #0x1000
loop4: 
	subs		r2, r2, #1
	bne		loop4
	str		r5, [r4, #(0x24 + 4*(SDA_GPIO/32))] /* GPCR: set SDA as low */
	mov		r2, #0x1000
loop5:
	subs		r2, r2, #1
	bne		loop5
	str		r5, [r4, #(0x18 + 4*(SDA_GPIO/32))] /* GPSR: set SDA as high */
	mov 		r2, #0x1000
loop6:
	subs		r2, r2, #1
	bne		loop6
	mov		r5, #(1<<(SDA_GPIO%32)) /* select SDA */
	str		r5, [r4, #(0x18 + 4*(SDA_GPIO/32))] /* GPSR: set SDA as high */
	mov		r2, #0x1000
loop7:
	subs		r2, r2, #1
	bne		loop7


	/* PSR, is the processor in S2/S3 state*/
	ldr	r1, =0x40f50004
	ldr	r0, [r1]
	tst     r0, #1
	bne     init_volt

	/* Initialize GPIO */
@	bl 	xlli_GPIO_init
@	bl	xlli_MFPR_init

init_volt:
	/* init I2C */
	bl	init_i2c

	/* change voltage later in blob */
	
#if 0
	/* Enable LDO5 for usb client */
	mov	r4, #0x34
	mov	r5, #0x10
	mov	r6, #0xbf
	bl	writei2c
#endif


	/* Initialize LEDs */
	bl	ledsetup
	bl	led1set
	bl 	led2set

        /* Initialize Interrupt Controller */
        /* Step 1 - Enable CP6 permission */
        mrc     p15, 0, r1, c15, c1, 0  @ read CPAR
        orr     r1, r1, #0x40
        mcr     p15, 0, r1, c15, c1, 0
        cpwait  r1
	
/*        bl      xlli_intr_init*//*check*/

	
        ldr     r0,  =0x4a000000
        ldr     r1,     =xlli_CSADRCFG3_value           @ Fetch from platform file
        str     r1,     [r0, #xlli_CSADRCFG3_offset]    @ Update CFG3 Register

	/* change frequency routine */
	@ldr	r4, =xlli_CLKREGS_PHYSICAL_BASE
	@  .... ..10 1... 11.. 1011 .010 ...0 0100
	@ldr	r0, =0x00001110
	@str	r0, [r4, #xlli_ACCR_offset]
        @orr     r2,  r2,  #3
        @mcr     p14, 0, r2, c6, c0, 0

	@bl	xlli_freq_change
	
	bl	led1clear
	bl	led2clear
	bl	normal_boot

	/* should never be here */
	blink	#1

.globl undefined_instruction
undefined_instruction:
	blink	#2

.globl software_interrupt
software_interrupt:
	blink	#3

.globl prefetch_abort
prefetch_abort:
	blink	#4

.globl data_abort
data_abort:
	blink	#5

.globl not_used
not_used:
	blink	#6

.globl irq
irq:
	blink	#7

.globl fiq
fiq:
	blink	#8

led1set:
	mov		pc, lr
	ldr		r2, =0x02000000
	ldr		r1, =0x40E00018 @ GPSR0
	str		r2, [r1]
	
	mov		pc, lr

led1clear:
	mov		pc, lr
	ldr		r2, =0x02000000
	ldr		r1, =0x40E00024 @ GPCR0
	str		r2, [r1]
	
	mov		pc, lr

led2set:
/*	ldr		r2, =0x10
	ldr		r1, =0x40E00018 @ GPSR 0
	str		r2, [r1]
*/	
	mov		pc, lr

led2clear:
/*	ldr		r2, =0x10
	ldr		r1, =0x40E00024 @ GPCR 0
	str		r2, [r1]
*/	
	mov		pc, lr

ledsetup:
	mov		pc, lr
	ldr		r2, =0x02000000
	ldr		r1, =0x40E0000C @ GPDR 0
	str		r2, [r1]
	mov		r3, lr

	bl		led1clear
	bl		led2clear
	mov		pc, r3

.macro wait, count
	mov		r0, \count
0:
	subs	r0, r0, #1
	bne		0b
.endm

ledflash:
	mov		r2, lr
	bl		led1set
	wait	#0x400000
	bl		led1clear
	wait	#0x400000
	mov		pc, r2

endless_blink:
	mov		r4, r3
0:
	bl		ledflash
	subs	r4, r4, #1
	bne		0b

	wait	#0x2000000
	b		endless_blink

init_i2c:
	/* setup MFPRs for I2C */
	ldr		r0, =0xa841
	ldr		r1, =MFP_I2C_SCL
	str		r0, [r1]
	ldr		r1, =MFP_I2C_SDA
	str		r0, [r1]

	/* enable i2c clock */
	ldr             r1, =0x41340010  /* D0CKEN_B */
	ldr             r0, [r1]
	orr             r0, r0, #0x10      /* enable i2c clock */
	str             r0, [r1]
 
	/* Initialize I2C Unit */	
	ldr		r1, =0x40301680 /* I2C register start address */
	mov		r0, #0x0
	str		r0, [r1, #0x20] /* ISAR */
	mov		r0, #0x4000     /* reset I2C */
	str		r0, [r1, #0x10] /* ICR */
	mov		r0, #0x60 	/* set IUE and SCLE */
	str		r0, [r1, #0x10]
	mov		pc, lr

writei2c:
	/* Initialize I2C Unit */	
	ldr		r3, =0x40301680 /* I2C register start address */

	/* Set slave I2C address */
	str		r4, [r3, #0x8] 	/* IDBR */

	/* send a start condition */
	ldr             r0, [r3, #0x10]
	@mov		r0, #0x9	/* START & TB */
	orr		r0, r0, #0x9    /* START & TB */
	ldr             r2, =0xffffeffd 
	and             r0, r0, r2      /* ~(STOP | ALDIE) */
	str		r0, [r3, #0x10] /* ICR */

	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
1:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		1b
	str		r2, [r3, #0x18] /* clear status */

	/* send first byte(register address) */
	str		r5, [r3, #0x8] 	/* IDBR */
	ldr             r0, [r3, #0x10] /* read ICR */
@	mov		r0, r0, #0x69	/* IUE, SCLE, START & TB */
	ldr             r2, =0x1008
	orr		r0, r0, r2	/* ALDIE, TB */
	ldr             r2, =0xfffffffe
	and		r0, r0, r2	/* ~START */
	ldr             r2, =0xfffffffd 
	and             r0, r0, r2          /* ~STOP */
	str		r0, [r3, #0x10]
	
	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
2:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		2b
	str		r2, [r3, #0x18] /* clear status */

	/* send second byte (register value) */
	str		r6, [r3, #0x8] 	/* IDBR */
	ldr             r0, [r3, #0x10] /* read ICR */
@	mov		r0, r0, #0x69	/* IUE, SCLE, START & TB */
	ldr             r2, =0x1008
	orr		r0, r0, r2	/* ALDIE, TB */
	ldr             r2, =0xfffffffe
	and		r0, r0, r2	/* ~START */
	orr             r0, r0, #0x2        /* STOP */
	str		r0, [r3, #0x10]

	/* wait until tx buffer empty */
	mov		r8, #0x80000 	/* time out */
3:
	ldr		r2, [r3, #0x18] /* ISR */
	subs		r8, r8, #1
	moveq		pc, lr
	ands		r2, r2, #0x40	/* ITE in ISR */
	beq		3b
	str		r2, [r3, #0x18] /* clear status */

	mov		pc, lr


