
/*++
"This software program is available to you under a choice of one of two 
licenses.  You may choose to be licensed under either the GNU General Public 
License (GPL) Version 2, June 1991, available at 
http://www.fsf.org/copyleft/gpl.html, or the BSD License, the text of
which follows:

Copyright (c) 1996-2005, Intel Corporation. All rights reserved.
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

Neither the name of the Intel Corporation ("Intel") nor the names of its 
contributors may be used to endorse or promote products derived from this
software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/
/**
**  FILENAME:       xllp_mfp_plat.h
**
**  PURPOSE: Contains all hardware platform-specific MFP (MultiFunction Pin)
**              macros, typedefs, and prototypes.  Processor-level MFP 
**              definitions are found in xllp_mfp_proc.h
**           Declares no storage.
**                  
**  Validity Notes: Only valid for the processor code named Monahans and the
**                  Zylonite platform.
**
**  Naming convention note: the symbols are, so far as possible, based on the 
**                   processor documentation because that will be constant
**                   across multiple platforms.
**               '<' and '>' in processor documentation symbols are represented
**               as '_' in these symbolic constants.
**
**  Ordering note: These symbols are ordered by this logic:
**                 MFP symbol for GPIOn, then GPIOn_2, in numeric order
**                 Then, alphabetic order of MFP symbols.
**                  
******************************************************************************/

#ifndef __XLLP_MFP_PLAT_P_PL_H__
#define __XLLP_MFP_PLAT_P_PL_H__
#include "xllp_defs.h"

//
// Note: For the first instance of the Zylonite, exact usage of many pins is
//       not fully defined, or the headers to which they lead are not expected
//       to be populated.  For those pins' symbols, the string "RSVD" is 
//       placed after "_MFP_".  They should not be used by any module but only 
//       set to an appropriate permanently initialized value by a single 
//       system initialization function.
//
// Note: For each "RSVD" pin on the target platform, an unassigned Alternate
//       Function code (based on internal silicon documents) has been selected
//       so that there are no side effects and minimal power effects from 
//       configuring the pin with that AF code.  It is, effectively, a null
//       AF.  This permits simpler power-sensitive test board SW than would be required if
//       GPIO were to be selected as the AF device for unattached pins.

 // Pick an unused AF on this pin for its NULL AF

//
//////////////////////////////////////////////////////////////////////
//
// Beginning of Platform MFP Values and Alternate Function for Each Pin 
//

//
// Concept note: 
//      If the XLLP modules and OS drivers are considered to be "Software ICs"
//      then the following set of constants can be considered as the
//      "Software PCB layer" in that it establishes the connection between the
//      processor pins and the target peripherals.
//

// No specified use for GPIO 0.  It goes to a generic header
#define XLLP_MFP_DREQ_GPIO_OFFSET              (XLLP_MFP_PIN_GPIO0_OFFSET)
#define XLLP_MFP_DREQ_GPIO_AF                  (XLLP_MFP_PIN_GPIO_AF_DRQ)

#define XLLP_MFP_MMC_CD_0_GPIO_OFFSET       (XLLP_MFP_PIN_GPIO1_OFFSET)
#define XLLP_MFP_MMC_CD_0_GPIO_AF	(XLLP_MFP_PIN_GPIO1_AF_GPIO_1)

#define XLLP_MFP_RDY_OFFSET                 (XLLP_MFP_PIN_GPIO2_OFFSET)
#define XLLP_MFP_RDY_AF                     (XLLP_MFP_PIN_GPIO2_AF_RDY)

#define XLLP_MFP_FLASH_NOR_CS_N_GPIO_OFFSET (XLLP_MFP_PIN_GPIO3_OFFSET)
#define XLLP_MFP_FLASH_NOR_CS_N_GPIO_AF     (XLLP_MFP_PIN_GPIO3_AF_nCS_2)

#define XLLP_MFP_DEBUG_ETH_CS_N_GPIO_OFFSET (XLLP_MFP_PIN_GPIO4_OFFSET)
#define XLLP_MFP_DEBUG_ETH_CS_N_GPIO_AF     (XLLP_MFP_PIN_GPIO4_AF_nCS_3)

#define XLLP_MFP_MMC_WP_0_N_GPIO_OFFSET     (XLLP_MFP_PIN_GPIO5_OFFSET)
#define XLLP_MFP_MMC_WP_0_N_GPIO_AF         (XLLP_MFP_PIN_GPIO5_AF_GPIO_5)
#define XLLP_MFP_NPIOR_GPIO_OFFSET		(XLLP_MFP_PIN_GPIO5_OFFSET) 
#define XLLP_MFP_NPIOR_GPIO_AF			(XLLP_MFP_PIN_GPIO5_AF_NPIOR)

#define XLLP_MFP_MMC_WP_1_N_GPIO_OFFSET     (XLLP_MFP_PIN_GPIO6_OFFSET)
#define XLLP_MFP_MMC_WP_1_N_GPIO_AF         (XLLP_MFP_PIN_GPIO6_AF_GPIO_6)
#define XLLP_MFP_NPIOW_GPIO_OFFSET		(XLLP_MFP_PIN_GPIO6_OFFSET)
#define XLLP_MFP_NPIOW_GPIO_AF			(XLLP_MFP_PIN_GPIO6_AF_nPIOW)

//#define XLLP_MFP_PHONE_CLOSED_GPIO_OFFSET   (XLLP_MFP_PIN_GPIO7_OFFSET)
//#define XLLP_MFP_PHONE_CLOSED_GPIO_AF       (XLLP_MFP_PIN_GPIO7_AF_GPIO_7)
#define XLLP_MFP_NIOIS16_GPIO_OFFSET		(XLLP_MFP_PIN_GPIO7_OFFSET)
#define XLLP_MFP_NIOIS16_GPIO_AF			(XLLP_MFP_PIN_GPIO7_AF_nIOIS16)

//#define XLLP_MFP_PHONE_FLIPPED_GPIO_OFFSET  (XLLP_MFP_PIN_GPIO8_OFFSET)
//#define XLLP_MFP_PHONE_FLIPPED_GPIO_AF      (XLLP_MFP_PIN_GPIO8_AF_GPIO_8) 
#define XLLP_MFP_NPWAIT_GPIO_OFFSET		(XLLP_MFP_PIN_GPIO8_OFFSET)
#define XLLP_MFP_NPWAIT_GPIO_AF			(XLLP_MFP_PIN_GPIO8_AF_nPWAIT)

#define XLLP_MFP_DEBUG_ETH_INT_GPIO_OFFSET  (XLLP_MFP_PIN_GPIO9_OFFSET)
#define XLLP_MFP_DEBUG_ETH_INT_GPIO_AF      (XLLP_MFP_PIN_GPIO9_AF_GPIO_9)

#define XLLP_MFP_UTM_CLK_OFFSET             (XLLP_MFP_PIN_GPIO10_OFFSET)
#define XLLP_MFP_UTM_CLK_AF                 (XLLP_MFP_PIN_GPIO10_AF_UTM_CLK)

// This PWM is an input to the Hermon daughter card, as is PWM1.  
// Its intended usage is not specified.
#define XLLP_MFP_RSVD_PWM_0_OFFSET          (XLLP_MFP_PIN_GPIO11_OFFSET)
#define XLLP_MFP_RSVD_PWM_0_AF              (XLLP_MFP_ALT_FN_4)

// This PWM is an input to the Hermon daughter card, as is PWM0.
// Its intended usage is not specified.
#define XLLP_MFP_RSVD_PWM_1_OFFSET          (XLLP_MFP_PIN_GPIO12_OFFSET)
#define XLLP_MFP_RSVD_PWM_1_AF              (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_CIR_ON_PWM_OFFSET          (XLLP_MFP_PIN_GPIO13_OFFSET)
#define XLLP_MFP_CIR_ON_PWM_AF              (XLLP_MFP_PIN_GPIO13_AF_PWM_2)

#if defined(CONFIG_CPU_MONAHANS_PL)
#define XLLP_MFP_GPIO_EXP_0_N_OFFSET		(XLLP_MFP_PIN_GPIO12_OFFSET)
#define XLLP_MFP_GPIO_EXP_1_N_OFFSET		(XLLP_MFP_PIN_GPIO13_OFFSET)
#endif

#define XLLP_MFP_BACKLIGHT_PWM_OFFSET       (XLLP_MFP_PIN_GPIO14_OFFSET)
#define XLLP_MFP_BACKLIGHT_PWM_AF           (XLLP_MFP_PIN_GPIO14_AF_PWM_3)

#define XLLP_MFP_AC97_SDATA_IN_2_OFFSET 	(XLLP_MFP_PIN_GPIO11_OFFSET)
#define XLLP_MFP_AC97_SDATA_IN_2_AF     	(XLLP_MFP_PIN_GPIO11_AF_AC97_SDATA_IN_2)
#define XLLP_MFP_AC97_INT_N_GPIO_OFFSET     (XLLP_MFP_PIN_GPIO15_OFFSET)
#define XLLP_MFP_AC97_INT_N_GPIO_AF         (XLLP_MFP_PIN_GPIO15_AF_GPIO_15)

#define XLLP_MFP_CAMERA_STROBE_EN_GPIO_OFFSET (XLLP_MFP_PIN_GPIO16_OFFSET)
#define XLLP_MFP_CAMERA_STROBE_EN_GPIO_AF     (XLLP_MFP_PIN_GPIO16_AF_GPIO_16)

#define XLLP_MFP_CAMERA_LIGHT_EN_GPIO_OFFSET (XLLP_MFP_PIN_GPIO17_OFFSET)
#define XLLP_MFP_CAMERA_LIGHT_EN_GPIO_AF     (XLLP_MFP_PIN_GPIO17_AF_GPIO_17)

#define XLLP_MFP_MMC_DAT0_OFFSET        (XLLP_MFP_PIN_GPIO18_OFFSET)
#define XLLP_MFP_MMC_DAT0_AF            (XLLP_MFP_PIN_GPIO18_AF_MM1_DAT0)

#define XLLP_MFP_MMC_DAT1_OFFSET        (XLLP_MFP_PIN_GPIO19_OFFSET)
#define XLLP_MFP_MMC_DAT1_AF            (XLLP_MFP_PIN_GPIO19_AF_MM1_DAT1)

#define XLLP_MFP_MMC_DAT2_OFFSET        (XLLP_MFP_PIN_GPIO20_OFFSET)
#define XLLP_MFP_MMC_DAT2_AF            (XLLP_MFP_PIN_GPIO20_AF_MM1_DAT2_CS0)

#define XLLP_MFP_MMC_DAT3_OFFSET        (XLLP_MFP_PIN_GPIO21_OFFSET)
#define XLLP_MFP_MMC_DAT3_AF            (XLLP_MFP_PIN_GPIO21_AF_MM1_DAT3_CS1)

#define XLLP_MFP_MMC_CLK_OFFSET             (XLLP_MFP_PIN_GPIO22_OFFSET)
#define XLLP_MFP_MMC_CLK_AF                 (XLLP_MFP_PIN_GPIO22_AF_MM1_CLK)

#define XLLP_MFP_MMC_CMD_0_OFFSET           (XLLP_MFP_PIN_GPIO23_OFFSET)
#define XLLP_MFP_MMC_CMD_0_AF               (XLLP_MFP_PIN_GPIO23_AF_MM1_CMD)

#define XLLP_MFP_MMC2_DAT0_OFFSET       (XLLP_MFP_PIN_GPIO24_OFFSET)
//#define XLLP_MFP_MMC2_DAT0_AF           (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_MMC2_DAT0_AF     (XLLP_MFP_PIN_GPIO24_AF_MM2_DAT0)

#define XLLP_MFP_MMC2_DAT1_OFFSET       (XLLP_MFP_PIN_GPIO25_OFFSET)
//#define XLLP_MFP_MMC2_DAT1_AF           (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_MMC2_DAT1_AF     (XLLP_MFP_PIN_GPIO25_AF_MM2_DAT1)

#define XLLP_MFP_MMC2_DAT2_CS0_OFFSET   (XLLP_MFP_PIN_GPIO26_OFFSET)
//#define XLLP_MFP_MMC2_DAT2_CS0_AF       (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_MMC2_DAT2_CS0_AF   (XLLP_MFP_PIN_GPIO26_AF_MM2_DAT2_CS0)

#define XLLP_MFP_MMC2_DAT3_CS1_OFFSET   (XLLP_MFP_PIN_GPIO27_OFFSET)
//#define XLLP_MFP_MMC2_DAT3_CS1_AF       (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_MMC2_DAT3_CS1_AF   (XLLP_MFP_PIN_GPIO27_AF_MM2_DAT3_CS1)

#define XLLP_MFP_MMC2_CLK_OFFSET        (XLLP_MFP_PIN_GPIO28_OFFSET)
//#define XLLP_MFP_MMC2_CLK_AF            (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_MMC2_CLK_AF     (XLLP_MFP_PIN_GPIO28_AF_MM2_CLK)

#define XLLP_MFP_MMC2_CMD_OFFSET        (XLLP_MFP_PIN_GPIO29_OFFSET)
//#define XLLP_MFP_MMC2_CMD_AF            (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_MMC2_CMD_AF          (XLLP_MFP_PIN_GPIO29_AF_MM2_CMD)

//#define XLLP_MFP_RSVD_GPIO_30_OFFSET        (XLLP_MFP_PIN_GPIO30_OFFSET)
//#define XLLP_MFP_RSVD_GPIO_30_AF            (XLLP_MFP_ALT_FN_5)
#define XLLP_MFP_CFCD_GPIO_OFFSET    (XLLP_MFP_PIN_GPIO30_OFFSET)
#define XLLP_MFP_CFCD_GPIO_AF            (XLLP_MFP_ALT_FN_0)

#define XLLP_MFP_MMC_CMD_1_OFFSET           (XLLP_MFP_PIN_GPIO31_OFFSET)
#define XLLP_MFP_MMC_CMD_1_AF               (XLLP_MFP_PIN_GPIO31_AF_MM1_CMD)

#define XLLP_MFP_SCL_OFFSET             (XLLP_MFP_PIN_GPIO32_OFFSET)
#define XLLP_MFP_SCL_AF                 (XLLP_MFP_PIN_GPIO32_AF_SCL)

#define XLLP_MFP_SDA_OFFSET             (XLLP_MFP_PIN_GPIO33_OFFSET)
#define XLLP_MFP_SDA_AF                 (XLLP_MFP_PIN_GPIO33_AF_SDA)

#define XLLP_MFP_AC97_SYSCLK_OFFSET     (XLLP_MFP_PIN_GPIO34_OFFSET)
#define XLLP_MFP_AC97_SYSCLK_AF         (XLLP_MFP_PIN_GPIO34_AF_AC97_SYSCLK)

#define XLLP_MFP_AC97_SDATA_IN_0_OFFSET (XLLP_MFP_PIN_GPIO35_OFFSET)
#define XLLP_MFP_AC97_SDATA_IN_0_AF     (XLLP_MFP_PIN_GPIO35_AF_AC97_SDATA_IN_0)

#define XLLP_MFP_AC97_SDATA_IN_1_OFFSET (XLLP_MFP_PIN_GPIO36_OFFSET)
#define XLLP_MFP_AC97_SDATA_IN_1_AF     (XLLP_MFP_PIN_GPIO36_AF_AC97_SDATA_IN_1)

#define XLLP_MFP_AC97_SDATA_OUT_OFFSET  (XLLP_MFP_PIN_GPIO37_OFFSET)
#define XLLP_MFP_AC97_SDATA_OUT_AF      (XLLP_MFP_PIN_GPIO37_AF_AC97_SDATA_OUT)

#define XLLP_MFP_AC97_SYNC_OFFSET       (XLLP_MFP_PIN_GPIO38_OFFSET)
#define XLLP_MFP_AC97_SYNC_AF           (XLLP_MFP_PIN_GPIO38_AF_AC97_SYNC)

#define XLLP_MFP_AC97_AC97_BITCLK_OFFSET (XLLP_MFP_PIN_GPIO39_OFFSET)
#define XLLP_MFP_AC97_AC97_BITCLK_AF     (XLLP_MFP_PIN_GPIO39_AF_AC97_BITCLK)

#define XLLP_MFP_AC97_nACRESET_OFFSET   (XLLP_MFP_PIN_GPIO40_OFFSET)
#define XLLP_MFP_AC97_nACRESET_AF       (XLLP_MFP_PIN_GPIO40_AF_AC97_nACRESET)

#define XLLP_MFP_FFRXD_OFFSET           (XLLP_MFP_PIN_GPIO41_OFFSET)
#define XLLP_MFP_FFRXD_AF               (XLLP_MFP_PIN_GPIO41_AF_UART1_RXD)

#define XLLP_MFP_FFTXD_OFFSET           (XLLP_MFP_PIN_GPIO42_OFFSET)
#define XLLP_MFP_FFTXD_AF               (XLLP_MFP_PIN_GPIO42_AF_UART1_TXD)

#define XLLP_MFP_FFCTS_OFFSET           (XLLP_MFP_PIN_GPIO43_OFFSET)
#define XLLP_MFP_FFCTS_AF               (XLLP_MFP_PIN_GPIO43_AF_UART1_CTS)

#define XLLP_MFP_FFDCD_OFFSET           (XLLP_MFP_PIN_GPIO44_OFFSET)
#define XLLP_MFP_FFDCD_AF               (XLLP_MFP_PIN_GPIO44_AF_UART1_DCD)

#define XLLP_MFP_FFDSR_OFFSET           (XLLP_MFP_PIN_GPIO45_OFFSET)
#define XLLP_MFP_FFDSR_AF               (XLLP_MFP_PIN_GPIO45_AF_UART1_DSR)

#define XLLP_MFP_FFRI_OFFSET            (XLLP_MFP_PIN_GPIO46_OFFSET)
#define XLLP_MFP_FFRI_AF                (XLLP_MFP_PIN_GPIO46_AF_UART1_RI)

#define XLLP_MFP_FFDTR_OFFSET           (XLLP_MFP_PIN_GPIO47_OFFSET)
#define XLLP_MFP_FFDTR_AF               (XLLP_MFP_PIN_GPIO47_AF_UART1_DTR)

#define XLLP_MFP_FFRTS_OFFSET           (XLLP_MFP_PIN_GPIO48_OFFSET)
#define XLLP_MFP_FFRTS_AF               (XLLP_MFP_PIN_GPIO48_AF_UART1_RTS)

#define XLLP_MFP_CIF_DD_0_OFFSET        (XLLP_MFP_PIN_GPIO49_OFFSET)
#define XLLP_MFP_CIF_DD_0_AF            (XLLP_MFP_PIN_GPIO49_AF_CIF_DD_0)

#define XLLP_MFP_CIF_DD_1_OFFSET        (XLLP_MFP_PIN_GPIO50_OFFSET)
#define XLLP_MFP_CIF_DD_1_AF            (XLLP_MFP_PIN_GPIO50_AF_CIF_DD_1)

#define XLLP_MFP_CIF_DD_2_OFFSET        (XLLP_MFP_PIN_GPIO51_OFFSET)
#define XLLP_MFP_CIF_DD_2_AF            (XLLP_MFP_PIN_GPIO51_AF_CIF_DD_2)

#define XLLP_MFP_CIF_DD_3_OFFSET        (XLLP_MFP_PIN_GPIO52_OFFSET)
#define XLLP_MFP_CIF_DD_3_AF            (XLLP_MFP_PIN_GPIO52_AF_CIF_DD_3)

#define XLLP_MFP_CIF_DD_4_OFFSET        (XLLP_MFP_PIN_GPIO53_OFFSET)
#define XLLP_MFP_CIF_DD_4_AF            (XLLP_MFP_PIN_GPIO53_AF_CIF_DD_4)

#define XLLP_MFP_CIF_DD_5_OFFSET        (XLLP_MFP_PIN_GPIO54_OFFSET)
#define XLLP_MFP_CIF_DD_5_AF            (XLLP_MFP_PIN_GPIO54_AF_CIF_DD_5)

#define XLLP_MFP_CIF_DD_6_OFFSET        (XLLP_MFP_PIN_GPIO55_OFFSET)
#define XLLP_MFP_CIF_DD_6_AF            (XLLP_MFP_PIN_GPIO55_AF_CIF_DD_6)

#define XLLP_MFP_CIF_DD_7_OFFSET        (XLLP_MFP_PIN_GPIO56_OFFSET)
#define XLLP_MFP_CIF_DD_7_AF            (XLLP_MFP_PIN_GPIO56_AF_CIF_DD_7)

#define XLLP_MFP_CIF_DD_8_OFFSET        (XLLP_MFP_PIN_GPIO57_OFFSET)
#define XLLP_MFP_CIF_DD_8_AF            (XLLP_MFP_PIN_GPIO57_AF_CIF_DD_8)

#define XLLP_MFP_CIF_DD_9_OFFSET        (XLLP_MFP_PIN_GPIO58_OFFSET)
#define XLLP_MFP_CIF_DD_9_AF            (XLLP_MFP_PIN_GPIO58_AF_CIF_DD_9)

#define XLLP_MFP_CIF_MCLK_OFFSET        (XLLP_MFP_PIN_GPIO59_OFFSET)
#define XLLP_MFP_CIF_MCLK_AF            (XLLP_MFP_PIN_GPIO59_AF_CIF_MCLK)

#define XLLP_MFP_CIF_PCLK_OFFSET        (XLLP_MFP_PIN_GPIO60_OFFSET)
#define XLLP_MFP_CIF_PCLK_AF            (XLLP_MFP_PIN_GPIO60_AF_CIF_PCLK)

#define XLLP_MFP_CIF_HSYNC_OFFSET       (XLLP_MFP_PIN_GPIO61_OFFSET)
#define XLLP_MFP_CIF_HSYNC_AF           (XLLP_MFP_PIN_GPIO61_AF_CIF_HSYNC)

#define XLLP_MFP_CIF_VSYNC_OFFSET       (XLLP_MFP_PIN_GPIO62_OFFSET)
#define XLLP_MFP_CIF_VSYNC_AF           (XLLP_MFP_PIN_GPIO62_AF_CIF_VSYNC)

#define XLLP_MFP_L_DD_8_OFFSET          (XLLP_MFP_PIN_GPIO63_OFFSET)
#define XLLP_MFP_L_DD_8_AF              (XLLP_MFP_PIN_GPIO63_AF_L_DD_8)

#define XLLP_MFP_L_DD_9_OFFSET          (XLLP_MFP_PIN_GPIO64_OFFSET)
#define XLLP_MFP_L_DD_9_AF              (XLLP_MFP_PIN_GPIO64_AF_L_DD_9)

#define XLLP_MFP_L_DD_10_OFFSET         (XLLP_MFP_PIN_GPIO65_OFFSET)
#define XLLP_MFP_L_DD_10_AF             (XLLP_MFP_PIN_GPIO65_AF_L_DD_10)

#define XLLP_MFP_L_DD_11_OFFSET         (XLLP_MFP_PIN_GPIO66_OFFSET)
#define XLLP_MFP_L_DD_11_AF             (XLLP_MFP_PIN_GPIO66_AF_L_DD_11)

#define XLLP_MFP_L_DD_12_OFFSET         (XLLP_MFP_PIN_GPIO67_OFFSET)
#define XLLP_MFP_L_DD_12_AF             (XLLP_MFP_PIN_GPIO67_AF_L_DD_12)

#define XLLP_MFP_L_DD_13_OFFSET         (XLLP_MFP_PIN_GPIO68_OFFSET)
#define XLLP_MFP_L_DD_13_AF             (XLLP_MFP_PIN_GPIO68_AF_L_DD_13)

#define XLLP_MFP_L_DD_14_OFFSET         (XLLP_MFP_PIN_GPIO69_OFFSET)
#define XLLP_MFP_L_DD_14_AF             (XLLP_MFP_PIN_GPIO69_AF_L_DD_14)

#define XLLP_MFP_L_DD_15_OFFSET         (XLLP_MFP_PIN_GPIO70_OFFSET)
#define XLLP_MFP_L_DD_15_AF             (XLLP_MFP_PIN_GPIO70_AF_L_DD_15)

#define XLLP_MFP_L_DD_16_OFFSET         (XLLP_MFP_PIN_GPIO71_OFFSET)
#define XLLP_MFP_L_DD_16_AF             (XLLP_MFP_PIN_GPIO71_AF_L_DD_16)

#define XLLP_MFP_L_DD_17_OFFSET         (XLLP_MFP_PIN_GPIO72_OFFSET)
#define XLLP_MFP_L_DD_17_AF             (XLLP_MFP_PIN_GPIO72_AF_L_DD_17)

#define XLLP_MFP_L_CS_OFFSET            (XLLP_MFP_PIN_GPIO73_OFFSET)
#define XLLP_MFP_L_CS_AF                (XLLP_MFP_PIN_GPIO73_AF_L_CS)

#define XLLP_MFP_L_VSYNC_OFFSET      (XLLP_MFP_PIN_GPIO74_OFFSET)
#define XLLP_MFP_L_VSYNC_AF          (XLLP_MFP_PIN_GPIO74_AF_L_VSYNC)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_0_OFFSET (XLLP_MFP_PIN_GPIO75_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_0_AF     (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_OB_DAT_0_AF (XLLP_MFP_PIN_GPIO75_AF_MSL1_OB_DAT_0)

#define XLLP_MFP_RSVD_MSL1_OB_CLK_OFFSET    (XLLP_MFP_PIN_GPIO76_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_CLK_AF        XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_OB_CLK_AF   (XLLP_MFP_PIN_GPIO76_AF_MSL1_OB_CLK)

#define XLLP_MFP_RSVD_MSL1_OB_STB_OFFSET    (XLLP_MFP_PIN_GPIO77_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_STB_AF        (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_OB_STB_AF   (XLLP_MFP_PIN_GPIO77_AF_MSL1_OB_STB)

#define XLLP_MFP_RSVD_MSL1_OB_WAIT_OFFSET   (XLLP_MFP_PIN_GPIO78_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_WAIT_AF       (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_OB_WAIT_AF  (XLLP_MFP_PIN_GPIO78_AF_MSL1_OB_WAIT)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_0_OFFSET  (XLLP_MFP_PIN_GPIO79_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_0_AF      (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_IB_DAT_0_AF (XLLP_MFP_PIN_GPIO79_AF_MSL1_IB_DAT_0)

#define XLLP_MFP_RSVD_MSL1_IB_CLK_OFFSET    (XLLP_MFP_PIN_GPIO80_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_CLK_AF        (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_IB_CLK_AF   (XLLP_MFP_PIN_GPIO80_AF_MSL1_IB_CLK)

#define XLLP_MFP_RSVD_MSL1_IB_STB_OFFSET    (XLLP_MFP_PIN_GPIO81_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_STB_AF        (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_IB_STB_AF   (XLLP_MFP_PIN_GPIO81_AF_MSL1_IB_STB)

#define XLLP_MFP_RSVD_MSL1_IB_WAIT_OFFSET   (XLLP_MFP_PIN_GPIO82_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_WAIT_AF       (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_IB_WAIT_AF  (XLLP_MFP_PIN_GPIO82_AF_MSL1_IB_WAIT)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_1_OFFSET  (XLLP_MFP_PIN_GPIO83_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_1_AF      (XLLP_MFP_ALT_FN_6)
//#define XLLP_MFP_RSVD_MSL1_IB_DAT_1_AF  (XLLP_MFP_PIN_GPIO83_AF_MSL1_IB_DAT_1)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_2_OFFSET  (XLLP_MFP_PIN_GPIO84_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_2_AF      (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_IB_DAT_2_AF (XLLP_MFP_PIN_GPIO84_AF_MSL1_IB_DAT_2)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_3_OFFSET  (XLLP_MFP_PIN_GPIO85_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_3_AF      (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_IB_DAT_3_AF (XLLP_MFP_PIN_GPIO85_AF_MSL1_IB_DAT_3)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_1_OFFSET  (XLLP_MFP_PIN_GPIO86_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_1_AF      (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_OB_DAT_1_AF (XLLP_MFP_PIN_GPIO86_AF_MSL1_OB_DAT_1)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_2_OFFSET  (XLLP_MFP_PIN_GPIO87_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_2_AF      (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_MSL1_OB_DAT_2_AF (XLLP_MFP_PIN_GPIO87_AF_MSL1_OB_DAT_2)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_3_OFFSET  (XLLP_MFP_PIN_GPIO88_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_3_AF      (XLLP_MFP_ALT_FN_6)
//#define XLLP_MFP_RSVD_MSL1_OB_DAT_3_AF (XLLP_MFP_PIN_GPIO88_AF_MSL1_OB_DAT_3)

#define XLLP_MFP_SSP_AUDIO_SCLK_OFFSET      (XLLP_MFP_PIN_GPIO89_OFFSET)
#define XLLP_MFP_SSP_AUDIO_SCLK_AF          (XLLP_MFP_PIN_GPIO89_AF_SSPSCLK3)

#define XLLP_MFP_SSP_AUDIO_FRM_OFFSET       (XLLP_MFP_PIN_GPIO90_OFFSET)
#define XLLP_MFP_SSP_AUDIO_FRM_AF           (XLLP_MFP_PIN_GPIO90_AF_SSPSFRM3)

#define XLLP_MFP_SSP_AUDIO_TXD_OFFSET       (XLLP_MFP_PIN_GPIO91_OFFSET)
#define XLLP_MFP_SSP_AUDIO_TXD_AF           (XLLP_MFP_PIN_GPIO91_AF_SSPTXD3)

#define XLLP_MFP_SSP_AUDIO_RXD_OFFSET       (XLLP_MFP_PIN_GPIO92_OFFSET)
#define XLLP_MFP_SSP_AUDIO_RXD_AF           (XLLP_MFP_PIN_GPIO92_AF_SSPRXD3)

#define XLLP_MFP_RSVD_SSP_4_CLK_OFFSET      (XLLP_MFP_PIN_GPIO93_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_CLK_AF          (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_SSP_4_CLK_AF        (XLLP_MFP_PIN_GPIO93_AF_SSPSCLK4)

#define XLLP_MFP_RSVD_SSP_4_FRM_OFFSET      (XLLP_MFP_PIN_GPIO94_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_FRM_AF          (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_SSP_4_FRM_AF        (XLLP_MFP_PIN_GPIO94_AF_SSPSFRM4)

#define XLLP_MFP_RSVD_SSP_4_TXD_OFFSET      (XLLP_MFP_PIN_GPIO95_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_TXD_AF          (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_SSP_4_TXD_AF        (XLLP_MFP_PIN_GPIO95_AF_SSPTXD4)

#define XLLP_MFP_RSVD_SSP_4_RXD_OFFSET      (XLLP_MFP_PIN_GPIO96_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_RXD_AF          (XLLP_MFP_ALT_FN_7)
//#define XLLP_MFP_RSVD_SSP_4_RXD_AF        (XLLP_MFP_PIN_GPIO96_AF_SSPRXD4)

#define XLLP_MFP_IR_SHDN_N_GPIO_OFFSET      (XLLP_MFP_PIN_GPIO97_OFFSET)
#define XLLP_MFP_IR_SHDN_N_GPIO_AF          (XLLP_MFP_PIN_GPIO97_AF_GPIO_97)

//#define XLLP_MFP_RSVD_GPIO_98_OFFSET        (XLLP_MFP_PIN_GPIO98_OFFSET)
//#define XLLP_MFP_RSVD_GPIO_98_AF            (XLLP_MFP_ALT_FN_7)
#define XLLP_MFP_CFNIRQ_GPIO_OFFSET        (XLLP_MFP_PIN_GPIO98_OFFSET)
#define XLLP_MFP_CFNIRQ_GPIO_AF            	(XLLP_MFP_ALT_FN_0)

#define XLLP_MFP_RSVD_GPIO_99_OFFSET        (XLLP_MFP_PIN_GPIO99_OFFSET)
#define XLLP_MFP_RSVD_GPIO_99_AF            (XLLP_MFP_ALT_FN_7)

#define XLLP_MFP_USB_OTG_SR_OFFSET          (XLLP_MFP_PIN_GPIO100_OFFSET)
#define XLLP_MFP_USB_OTG_SR_AF              (XLLP_MFP_PIN_GPIO100_AF_USB_P2_4)

#define XLLP_MFP_OTG_EN_OFFSET              (XLLP_MFP_PIN_GPIO101_OFFSET)
#define XLLP_MFP_OTG_EN_AF                  (XLLP_MFP_PIN_GPIO101_AF_USB_P2_8)

#define XLLP_MFP_CIF_HI_PWDN_GPI0_OFFSET    (XLLP_MFP_PIN_GPIO102_OFFSET)
#define XLLP_MFP_CIF_HI_PWDN_GPI0_AF        (XLLP_MFP_PIN_GPIO102_AF_GPIO_102)

#define XLLP_MFP_CIF_LO_PWDN_GPI0_OFFSET    (XLLP_MFP_PIN_GPIO103_OFFSET)
#define XLLP_MFP_CIF_LO_PWDN_GPI0_AF        (XLLP_MFP_PIN_GPIO103_AF_GPIO_103)

#define XLLP_MFP_OTG_ID_OFFSET              (XLLP_MFP_PIN_GPIO104_OFFSET)
#define XLLP_MFP_OTG_ID_AF                  (XLLP_MFP_PIN_GPIO104_AF_USB_P2_7)

#define XLLP_MFP_KP_DKIN_0_OFFSET           (XLLP_MFP_PIN_GPIO105_OFFSET)
#define XLLP_MFP_KP_DKIN_0_AF               (XLLP_MFP_PIN_GPIO105_AF_KP_DKIN_0)

#define XLLP_MFP_KP_DKIN_1_OFFSET           (XLLP_MFP_PIN_GPIO106_OFFSET)
#define XLLP_MFP_KP_DKIN_1_AF               (XLLP_MFP_PIN_GPIO106_AF_KP_DKIN_1)

#define XLLP_MFP_STD_TXD_OFFSET         (XLLP_MFP_PIN_GPIO107_OFFSET)
#define XLLP_MFP_STD_TXD_AF             (XLLP_MFP_PIN_GPIO107_AF_UART3_TXD)

#define XLLP_MFP_STD_RXD_OFFSET         (XLLP_MFP_PIN_GPIO108_OFFSET)
#define XLLP_MFP_STD_RXD_AF             (XLLP_MFP_PIN_GPIO108_AF_UART3_RXD)

#define XLLP_MFP_RSVD_BT_RTS_OFFSET     (XLLP_MFP_PIN_GPIO109_OFFSET)
//#define XLLP_MFP_RSVD_BT_RTS_AF         (XLLP_MFP_ALT_FN_7)
#define XLLP_MFP_RSVD_BT_RTS_AF       (XLLP_MFP_PIN_GPIO109_AF_UART2_RTS)

#define XLLP_MFP_RSVD_BT_RXD_OFFSET     (XLLP_MFP_PIN_GPIO110_OFFSET)
//#define XLLP_MFP_RSVD_BT_RXD_AF         (XLLP_MFP_ALT_FN_7)
#define XLLP_MFP_RSVD_BT_RXD_AF       (XLLP_MFP_PIN_GPIO110_AF_UART2_RXD)

#define XLLP_MFP_RSVD_BT_TXD_OFFSET     (XLLP_MFP_PIN_GPIO111_OFFSET)
//#define XLLP_MFP_RSVD_BT_TXD_AF         (XLLP_MFP_ALT_FN_7)
#define XLLP_MFP_RSVD_BT_TXD_AF       (XLLP_MFP_PIN_GPIO111_AF_UART2_TXD)

#define XLLP_MFP_RSVD_BT_CTS_OFFSET     (XLLP_MFP_PIN_GPIO112_OFFSET)
//#define XLLP_MFP_RSVD_BT_CTS_AF         (XLLP_MFP_ALT_FN_6)
#define XLLP_MFP_RSVD_BT_CTS_AF       (XLLP_MFP_PIN_GPIO112_AF_UART2_CTS)

#define XLLP_MFP_KP_MKIN_0_OFFSET       (XLLP_MFP_PIN_GPIO113_OFFSET)
#define XLLP_MFP_KP_MKIN_0_AF           (XLLP_MFP_PIN_GPIO113_AF_KP_MKIN_0)

#define XLLP_MFP_KP_MKIN_1_OFFSET       (XLLP_MFP_PIN_GPIO114_OFFSET)
#define XLLP_MFP_KP_MKIN_1_AF           (XLLP_MFP_PIN_GPIO114_AF_KP_MKIN_1)

#define XLLP_MFP_KP_MKIN_2_OFFSET       (XLLP_MFP_PIN_GPIO115_OFFSET)
#define XLLP_MFP_KP_MKIN_2_AF           (XLLP_MFP_PIN_GPIO115_AF_KP_MKIN_2)

#define XLLP_MFP_KP_MKIN_3_OFFSET       (XLLP_MFP_PIN_GPIO116_OFFSET)
#define XLLP_MFP_KP_MKIN_3_AF           (XLLP_MFP_PIN_GPIO116_AF_KP_MKIN_3)

#define XLLP_MFP_KP_MKIN_4_OFFSET       (XLLP_MFP_PIN_GPIO117_OFFSET)
#define XLLP_MFP_KP_MKIN_4_AF           (XLLP_MFP_PIN_GPIO117_AF_KP_MKIN_4)

#define XLLP_MFP_KP_MKIN_5_OFFSET       (XLLP_MFP_PIN_GPIO118_OFFSET)
#define XLLP_MFP_KP_MKIN_5_AF           (XLLP_MFP_PIN_GPIO118_AF_KP_MKIN_5)

#define XLLP_MFP_KP_MKIN_6_OFFSET       (XLLP_MFP_PIN_GPIO119_OFFSET)
#define XLLP_MFP_KP_MKIN_6_AF           (XLLP_MFP_PIN_GPIO119_AF_KP_MKIN_6)

#define XLLP_MFP_KP_MKIN_7_OFFSET       (XLLP_MFP_PIN_GPIO120_OFFSET)
#define XLLP_MFP_KP_MKIN_7_AF           (XLLP_MFP_PIN_GPIO120_AF_KP_MKIN_7)

#define XLLP_MFP_KP_MKOUT_0_OFFSET      (XLLP_MFP_PIN_GPIO121_OFFSET)
#define XLLP_MFP_KP_MKOUT_0_AF          (XLLP_MFP_PIN_GPIO121_AF_KP_MKOUT_0)

#define XLLP_MFP_KP_MKOUT_1_OFFSET      (XLLP_MFP_PIN_GPIO122_OFFSET)
#define XLLP_MFP_KP_MKOUT_1_AF          (XLLP_MFP_PIN_GPIO122_AF_KP_MKOUT_1)

#define XLLP_MFP_KP_MKOUT_2_OFFSET      (XLLP_MFP_PIN_GPIO123_OFFSET)
#define XLLP_MFP_KP_MKOUT_2_AF          (XLLP_MFP_PIN_GPIO123_AF_KP_MKOUT_2)

#define XLLP_MFP_KP_MKOUT_3_OFFSET      (XLLP_MFP_PIN_GPIO124_OFFSET)
#define XLLP_MFP_KP_MKOUT_3_AF          (XLLP_MFP_PIN_GPIO124_AF_KP_MKOUT_3)

#define XLLP_MFP_KP_MKOUT_4_OFFSET      (XLLP_MFP_PIN_GPIO125_OFFSET)
#define XLLP_MFP_KP_MKOUT_4_AF          (XLLP_MFP_PIN_GPIO125_AF_KP_MKOUT_4)

#define XLLP_MFP_KP_MKOUT_5_OFFSET      (XLLP_MFP_PIN_GPIO126_OFFSET)
#define XLLP_MFP_KP_MKOUT_5_AF          (XLLP_MFP_PIN_GPIO126_AF_KP_MKOUT_5)

#define XLLP_MFP_KP_MKOUT_6_OFFSET      (XLLP_MFP_PIN_GPIO127_OFFSET)
#define XLLP_MFP_KP_MKOUT_6_AF          (XLLP_MFP_PIN_GPIO127_AF_KP_MKOUT_6)

#define XLLP_MFP_ONE_WIRE_OFFSET        (XLLP_MFP_PIN_GPIO0_2_OFFSET)
#define XLLP_MFP_ONE_WIRE_AF            (XLLP_MFP_PIN_GPIO0_2_AF_ONE_WIRE)

#define XLLP_MFP_RSVD_GPIO_1_OFFSET     (XLLP_MFP_PIN_GPIO1_2_OFFSET)
#define XLLP_MFP_RSVD_GPIO_1_AF         (XLLP_MFP_ALT_FN_6)

#define XLLP_MFP_USBHPEN_OFFSET         (XLLP_MFP_PIN_GPIO2_2_OFFSET)
#define XLLP_MFP_USBHPEN_AF             (XLLP_MFP_PIN_GPIO2_2_AF_USBHPEN)

#define XLLP_MFP_USBHPWR_OFFSET         (XLLP_MFP_PIN_GPIO3_2_OFFSET)
#define XLLP_MFP_USBHPWR_AF             (XLLP_MFP_PIN_GPIO3_2_AF_USBHPWR)

#define XLLP_MFP_RSVD_GPIO_4_OFFSET     (XLLP_MFP_PIN_GPIO4_2_OFFSET)
#define XLLP_MFP_RSVD_GPIO_4_AF         (XLLP_MFP_ALT_FN_7)

#define XLLP_MFP_KP_MKOUT_7_OFFSET      (XLLP_MFP_PIN_GPIO5_2_OFFSET)
#define XLLP_MFP_KP_MKOUT_7_AF          (XLLP_MFP_PIN_GPIO5_2_AF_KP_MKOUT_7)

#define XLLP_MFP_L_DD_0_OFFSET          (XLLP_MFP_PIN_GPIO6_2_OFFSET)
#define XLLP_MFP_L_DD_0_AF              (XLLP_MFP_PIN_GPIO6_2_AF_L_DD_0)

#define XLLP_MFP_L_DD_1_OFFSET          (XLLP_MFP_PIN_GPIO7_2_OFFSET)
#define XLLP_MFP_L_DD_1_AF              (XLLP_MFP_PIN_GPIO7_2_AF_L_DD_1)

#define XLLP_MFP_L_DD_2_OFFSET          (XLLP_MFP_PIN_GPIO8_2_OFFSET)
#define XLLP_MFP_L_DD_2_AF              (XLLP_MFP_PIN_GPIO8_2_AF_L_DD_2)

#define XLLP_MFP_L_DD_3_OFFSET          (XLLP_MFP_PIN_GPIO9_2_OFFSET)
#define XLLP_MFP_L_DD_3_AF              (XLLP_MFP_PIN_GPIO9_2_AF_L_DD_3)

#define XLLP_MFP_L_DD_4_OFFSET          (XLLP_MFP_PIN_GPIO10_2_OFFSET)
#define XLLP_MFP_L_DD_4_AF              (XLLP_MFP_PIN_GPIO10_2_AF_L_DD_4)

#define XLLP_MFP_L_DD_5_OFFSET          (XLLP_MFP_PIN_GPIO11_2_OFFSET)
#define XLLP_MFP_L_DD_5_AF              (XLLP_MFP_PIN_GPIO11_2_AF_L_DD_5)

#define XLLP_MFP_L_DD_6_OFFSET          (XLLP_MFP_PIN_GPIO12_2_OFFSET)
#define XLLP_MFP_L_DD_6_AF              (XLLP_MFP_PIN_GPIO12_2_AF_L_DD_6)

#define XLLP_MFP_L_DD_7_OFFSET          (XLLP_MFP_PIN_GPIO13_2_OFFSET)
#define XLLP_MFP_L_DD_7_AF              (XLLP_MFP_PIN_GPIO13_2_AF_L_DD_7)

#define XLLP_MFP_L_FCLK_OFFSET          (XLLP_MFP_PIN_GPIO14_2_OFFSET)
#define XLLP_MFP_L_FCLK_AF              (XLLP_MFP_PIN_GPIO14_2_AF_L_FCLK_RD)

#define XLLP_MFP_L_LCLK_OFFSET            (XLLP_MFP_PIN_GPIO15_2_OFFSET)
#define XLLP_MFP_L_LCLK_AF                (XLLP_MFP_PIN_GPIO15_2_AF_L_LCLK_A0)

#define XLLP_MFP_L_PCLK_OFFSET          (XLLP_MFP_PIN_GPIO16_2_OFFSET)
#define XLLP_MFP_L_PCLK_AF              (XLLP_MFP_PIN_GPIO16_2_AF_L_PCLK_WR)

#define XLLP_MFP_L_BIAS_OFFSET          (XLLP_MFP_PIN_GPIO17_2_OFFSET)
#define XLLP_MFP_L_BIAS_AF              (XLLP_MFP_PIN_GPIO17_2_AF_L_BIAS)

#define XLLP_MFP_RSVD_DF_ADDR0_OFFSET   (XLLP_MFP_PIN_DF_ADDR0_OFFSET)
#define XLLP_MFP_RSVD_DF_ADDR0_AF       (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_DF_ADDR1_OFFSET   (XLLP_MFP_PIN_DF_ADDR1_OFFSET)
#define XLLP_MFP_RSVD_DF_ADDR1_AF       (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_DF_ADDR2_OFFSET        (XLLP_MFP_PIN_DF_ADDR2_OFFSET)
#define XLLP_MFP_RSVD_DF_ADDR2_AF            (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_DF_ADDR3_OFFSET        (XLLP_MFP_PIN_DF_ADDR3_OFFSET)
#define XLLP_MFP_RSVD_DF_ADDR3_AF            (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_DF_INT_RnB_OFFSET      (XLLP_MFP_PIN_DF_INT_RnB_OFFSET)
#define XLLP_MFP_DF_INT_RnB_AF          (XLLP_MFP_PIN_DF_INT_RnB_AF_DF_INT_RnB)

#define XLLP_MFP_SIR_TXD_OFFSET         (XLLP_MFP_STD_TXD_OFFSET)
#define XLLP_MFP_SIR_TXD_AF             (XLLP_MFP_STD_TXD_AF)

#define XLLP_MFP_SIR_RXD_OFFSET         (XLLP_MFP_STD_RXD_OFFSET)
#define XLLP_MFP_SIR_RXD_AF             (XLLP_MFP_STD_RXD_AF)

 


// Note: the following 16 sets of constants are listed in numeric order
//      of [n] within XLLP_MFP_PIN_ND_IO[n]_OFFSET

#define XLLP_MFP_DF_IO_0_OFFSET         (XLLP_MFP_PIN_DF_IO0_OFFSET)
#define XLLP_MFP_DF_IO_0_AF             (XLLP_MFP_PIN_DF_IO_0_AF_ND)

#define XLLP_MFP_DF_IO_1_OFFSET         (XLLP_MFP_PIN_DF_IO1_OFFSET)
#define XLLP_MFP_DF_IO_1_AF             (XLLP_MFP_PIN_DF_IO_1_AF_ND)

#define XLLP_MFP_DF_IO_2_OFFSET         (XLLP_MFP_PIN_DF_IO2_OFFSET)
#define XLLP_MFP_DF_IO_2_AF             (XLLP_MFP_PIN_DF_IO_2_AF_ND)

#define XLLP_MFP_DF_IO_3_OFFSET         (XLLP_MFP_PIN_DF_IO3_OFFSET)
#define XLLP_MFP_DF_IO_3_AF             (XLLP_MFP_PIN_DF_IO_3_AF_ND)

#define XLLP_MFP_DF_IO_4_OFFSET         (XLLP_MFP_PIN_DF_IO4_OFFSET)
#define XLLP_MFP_DF_IO_4_AF             (XLLP_MFP_PIN_DF_IO_4_AF_ND)

#define XLLP_MFP_DF_IO_5_OFFSET         (XLLP_MFP_PIN_DF_IO5_OFFSET)
#define XLLP_MFP_DF_IO_5_AF             (XLLP_MFP_PIN_DF_IO_5_AF_ND)

#define XLLP_MFP_DF_IO_6_OFFSET         (XLLP_MFP_PIN_DF_IO6_OFFSET)
#define XLLP_MFP_DF_IO_6_AF             (XLLP_MFP_PIN_DF_IO_6_AF_ND)

#define XLLP_MFP_DF_IO_7_OFFSET         (XLLP_MFP_PIN_DF_IO7_OFFSET)
#define XLLP_MFP_DF_IO_7_AF             (XLLP_MFP_PIN_DF_IO_7_AF_ND)

#define XLLP_MFP_DF_IO_8_OFFSET         (XLLP_MFP_PIN_DF_IO8_OFFSET)
#define XLLP_MFP_DF_IO_8_AF             (XLLP_MFP_PIN_DF_IO_8_AF_ND)

#define XLLP_MFP_DF_IO_9_OFFSET         (XLLP_MFP_PIN_DF_IO9_OFFSET)
#define XLLP_MFP_DF_IO_9_AF             (XLLP_MFP_PIN_DF_IO_9_AF_ND)

#define XLLP_MFP_DF_IO_10_OFFSET        (XLLP_MFP_PIN_DF_IO10_OFFSET)
#define XLLP_MFP_DF_IO_10_AF            (XLLP_MFP_PIN_DF_IO_10_AF_ND)

#define XLLP_MFP_DF_IO_11_OFFSET        (XLLP_MFP_PIN_DF_IO11_OFFSET)
#define XLLP_MFP_DF_IO_11_AF            (XLLP_MFP_PIN_DF_IO_11_AF_ND)

#define XLLP_MFP_DF_IO_12_OFFSET        (XLLP_MFP_PIN_DF_IO12_OFFSET)
#define XLLP_MFP_DF_IO_12_AF            (XLLP_MFP_PIN_DF_IO_12_AF_ND)

#define XLLP_MFP_DF_IO_13_OFFSET        (XLLP_MFP_PIN_DF_IO13_OFFSET)
#define XLLP_MFP_DF_IO_13_AF            (XLLP_MFP_PIN_DF_IO_13_AF_ND)

#define XLLP_MFP_DF_IO_14_OFFSET        (XLLP_MFP_PIN_DF_IO14_OFFSET)
#define XLLP_MFP_DF_IO_14_AF            (XLLP_MFP_PIN_DF_IO_14_AF_ND)

#define XLLP_MFP_DF_IO_15_OFFSET        (XLLP_MFP_PIN_DF_IO15_OFFSET)
#define XLLP_MFP_DF_IO_15_AF            (XLLP_MFP_PIN_DF_IO_15_AF_ND)

#define XLLP_MFP_DF_nADV1_OFFSET        (XLLP_MFP_PIN_DF_nADV1_ALE_OFFSET)
#define XLLP_MFP_DF_nADV1_AF            (XLLP_MFP_PIN_DF_nADV1_ALE_AF_ND_ALE)

#define XLLP_MFP_DF_nADV2_OFFSET        (XLLP_MFP_PIN_DF_nADV2_ALE_OFFSET)
#define XLLP_MFP_DF_nADV2_AF            (XLLP_MFP_ALT_FN_7)

#define XLLP_MFP_DF_NCS0_OFFSET         (XLLP_MFP_PIN_DF_nCS0_OFFSET)
#define XLLP_MFP_DF_NCS0_AF             (XLLP_MFP_PIN_DF_nCS0_AF_DF_NCS0)

#define XLLP_MFP_DF_NCS1_OFFSET         (XLLP_MFP_PIN_DF_nCS1_OFFSET)
#define XLLP_MFP_DF_NCS1_AF             (XLLP_MFP_ALT_FN_7)

#define XLLP_MFP_DF_nRE_OFFSET          (XLLP_MFP_PIN_DF_nRE_nOE_OFFSET)
#define XLLP_MFP_DF_nRE_AF              (XLLP_MFP_PIN_DF_nRE_nOE_AF_DF_nOE)

#define XLLP_MFP_DF_nWE_OFFSET          (XLLP_MFP_PIN_DF_nWE_OFFSET)
#define XLLP_MFP_DF_nWE_AF              (XLLP_MFP_PIN_DF_nWE_AF_ND_nWE)

#define XLLP_MFP_DF_SCLK_E_OFFSET       (XLLP_MFP_PIN_DF_SCLK_E_OFFSET)
#define XLLP_MFP_DF_SCLK_E_AF           (XLLP_MFP_PIN_DF_SCLK_E_AF_DF_SCLK_E)

#define XLLP_MFP_DF_SCLK_S_OFFSET       (XLLP_MFP_PIN_DF_SCLK_S_OFFSET)
#define XLLP_MFP_DF_SCLK_S_AF           (XLLP_MFP_PIN_DF_SCLK_S_AF_DF_SCLK_S)

#define XLLP_MFP_DEBUG_ETH_nBE0_OFFSET  (XLLP_MFP_PIN_nBE0_OFFSET)
#define XLLP_MFP_DEBUG_ETH_nBE0_AF      (XLLP_MFP_PIN_nBE0_AF_nBE0)

#define XLLP_MFP_DEBUG_ETH_nBE1_OFFSET  (XLLP_MFP_PIN_nBE1_OFFSET)
#define XLLP_MFP_DEBUG_ETH_nBE1_AF      (XLLP_MFP_PIN_nBE1_AF_nBE1)

#define XLLP_MFP_ND_CLE_OFFSET          (XLLP_MFP_PIN_ND_CLE_OFFSET)
#define XLLP_MFP_ND_CLE_AF              (XLLP_MFP_PIN_ND_CLE_AF_ND_CLE)

#define XLLP_MFP_nLLA_OFFSET            (XLLP_MFP_PIN_nLLA_OFFSET)
#define XLLP_MFP_nLLA_AF                (XLLP_MFP_PIN_nLLA_AF_nLLA)

#define XLLP_MFP_nLUA_OFFSET            (XLLP_MFP_PIN_nLUA_OFFSET)
#define XLLP_MFP_nLUA_AF                (XLLP_MFP_PIN_nLUA_AF_nLUA)

#define XLLP_MFP_RSVD_nXCVREN_OFFSET    (XLLP_MFP_PIN_nXCVREN_OFFSET)
#define XLLP_MFP_RSVD_nXCVREN_AF        (XLLP_MFP_ALT_FN_5)
//#define XLLP_MFP_nXCVREN_AF     (XLLP_MFP_PIN_nXCVREN_AF_nXCVREN)



//USB 2.0
#define XLLP_MFP_U2D_UTM_CLK_OFFSET		(XLLP_MFP_PIN_GPIO10_OFFSET)
#define XLLP_MFP_U2D_UTM_CLK_AF			(XLLP_MFP_PIN_GPIO10_AF_UTM_CLK)

#define XLLP_MFP_U2D_DATA0_OFFSET		(XLLP_MFP_PIN_GPIO49_OFFSET)
#define XLLP_MFP_U2D_DATA0_AF			(XLLP_MFP_PIN_GPIO49_AF_U2D_DATA0)

#define XLLP_MFP_U2D_DATA1_OFFSET		(XLLP_MFP_PIN_GPIO50_OFFSET)
#define XLLP_MFP_U2D_DATA1_AF			(XLLP_MFP_PIN_GPIO50_AF_U2D_DATA1)

#define XLLP_MFP_U2D_DATA2_OFFSET		(XLLP_MFP_PIN_GPIO51_OFFSET)
#define XLLP_MFP_U2D_DATA2_AF			(XLLP_MFP_PIN_GPIO51_AF_U2D_DATA2)

#define XLLP_MFP_U2D_DATA3_OFFSET		(XLLP_MFP_PIN_GPIO52_OFFSET)
#define XLLP_MFP_U2D_DATA3_AF			(XLLP_MFP_PIN_GPIO52_AF_U2D_DATA3)

#define XLLP_MFP_U2D_DATA4_OFFSET		(XLLP_MFP_PIN_GPIO53_OFFSET)
#define XLLP_MFP_U2D_DATA4_AF			(XLLP_MFP_PIN_GPIO53_AF_U2D_DATA4)

#define XLLP_MFP_U2D_DATA5_OFFSET		(XLLP_MFP_PIN_GPIO54_OFFSET)
#define XLLP_MFP_U2D_DATA5_AF			(XLLP_MFP_PIN_GPIO54_AF_U2D_DATA5)

#define XLLP_MFP_U2D_DATA6_OFFSET		(XLLP_MFP_PIN_GPIO55_OFFSET)
#define XLLP_MFP_U2D_DATA6_AF			(XLLP_MFP_PIN_GPIO55_AF_U2D_DATA6)

#define XLLP_MFP_U2D_DATA7_OFFSET		(XLLP_MFP_PIN_GPIO56_OFFSET)
#define XLLP_MFP_U2D_DATA7_AF			(XLLP_MFP_PIN_GPIO56_AF_U2D_DATA7)

#define XLLP_MFP_U2D_RESET_OFFSET		(XLLP_MFP_PIN_GPIO98_OFFSET)
#define XLLP_MFP_U2D_RESET_AF			(XLLP_MFP_PIN_GPIO98_AF_U2D_RESET)

#define XLLP_MFP_U2D_XCVR_SELECT_OFFSET (XLLP_MFP_PIN_GPIO99_OFFSET)
#define XLLP_MFP_U2D_XCVR_SELECT_AF		(XLLP_MFP_PIN_GPIO99_AF_U2D_XCVR_SELECT)

#define XLLP_MFP_U2D_TERM_SELECT_OFFSET (XLLP_MFP_PIN_GPIO100_OFFSET)
#define XLLP_MFP_U2D_TERM_SELECT_AF		(XLLP_MFP_PIN_GPIO100_AF_U2D_TERM_SELECT)

#define XLLP_MFP_U2D_SUSPENDM_X_OFFSET  (XLLP_MFP_PIN_GPIO101_OFFSET)
#define XLLP_MFP_U2D_SUSPENDM_X_AF		(XLLP_MFP_PIN_GPIO101_AF_U2D_SUSPENDM_X)

#define XLLP_MFP_U2D_LINESTATE0_OFFSET  (XLLP_MFP_PIN_GPIO102_OFFSET)
#define XLLP_MFP_U2D_LINESTATE0_AF		(XLLP_MFP_PIN_GPIO102_AF_UTM_LINESTATE_0)

#define XLLP_MFP_U2D_LINESTATE1_OFFSET  (XLLP_MFP_PIN_GPIO103_OFFSET)
#define XLLP_MFP_U2D_LINESTATE1_AF		(XLLP_MFP_PIN_GPIO103_AF_UTM_LINESTATE_1)


#define XLLP_MFP_U2D_TXVALID_OFFSET 	(XLLP_MFP_PIN_GPIO83_OFFSET)
#define XLLP_MFP_U2D_TXVALID_AF			(XLLP_MFP_PIN_GPIO83_AF_U2D_TXVALID)

#define XLLP_MFP_U2D_TXREADY_OFFSET		(XLLP_MFP_PIN_GPIO73_OFFSET)
#define XLLP_MFP_U2D_TXREADY_AF			(XLLP_MFP_PIN_GPIO73_AF_UTM_TXREADY)

#define XLLP_MFP_U2D_RXVALID_OFFSET 	(XLLP_MFP_PIN_GPIO58_OFFSET)
#define XLLP_MFP_U2D_RXVALID_AF			(XLLP_MFP_PIN_GPIO58_AF_UTM_RXVALID)

#define XLLP_MFP_U2D_RXACTIVE_OFFSET	(XLLP_MFP_PIN_GPIO59_OFFSET)
#define XLLP_MFP_U2D_RXACTIVE_AF		(XLLP_MFP_PIN_GPIO59_AF_UTM_RXACTIVE)

#define XLLP_MFP_U2D_RXERROR_OFFSET		(XLLP_MFP_PIN_GPIO60_OFFSET)
#define XLLP_MFP_U2D_RXERROR_AF			(XLLP_MFP_PIN_GPIO60_AF_U2D_RXERROR)

#define XLLP_MFP_U2D_OPMODE0_OFFSET 	(XLLP_MFP_PIN_GPIO61_OFFSET)
#define XLLP_MFP_U2D_OPMODE0_AF			(XLLP_MFP_PIN_GPIO61_AF_U2D_OPMODE0)

#define XLLP_MFP_U2D_OPMODE1_OFFSET 	(XLLP_MFP_PIN_GPIO62_OFFSET)
#define XLLP_MFP_U2D_OPMODE1_AF			(XLLP_MFP_PIN_GPIO62_AF_U2D_OPMODE1)
//
// End of Platform MFP Values and Alternate Function for Each Pin 
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
// Beginning of Platform Drive Strength Values  
//

//
// Note: The actual best values for the Zylonite platform are not known.
//       These values are based on silicon team estimates for certain
//       voltages in ideal circumstances and the known voltage domains
//       on the Zylonite.
//    

#define XLLP_MFP_DEFAULT_DS         (XLLP_MFP_DS_03X)
#define XLLP_MFP_USB2_DS            (XLLP_MFP_DS_08X)
#define XLLP_MFP_CIF_DS             (XLLP_MFP_DS_04X)
#define XLLP_MFP_LCD_DS             (XLLP_MFP_DS_04X)
#define XLLP_MFP_SSP_DS             (XLLP_MFP_DS_04X)
#define XLLP_MFP_MSL_DEFAULT_DS     (XLLP_MFP_DS_04X) //3.0V: 3 pins
#define XLLP_MFP_MSL_SPECIAL_DS     (XLLP_MFP_DS_08X) //1.8V: 1 pin 
#define XLLP_MFP_ND_DATA_DS         (XLLP_MFP_DS_08X)
#define XLLP_MFP_ND_NON_DATA_DS     (XLLP_MFP_DS_10X)
#define XLLP_MFP_AC97_DS            (XLLP_MFP_DS_04X)

//
// Beginning of Platform Drive Strength Values 
//
//////////////////////////////////////////////////////////////////////


#ifdef  COMPILING_CODE_MUSEUM
// This section is reserved for symbols that are used in various XLLP modules but
// are not supported on this platform.

    #define XLLP_MFP_MMC_CD_1_GPIO_OFFSET       (XLLP_MFP_PIN_GPIO4_OFFSET)
    #define XLLP_MFP_MMC_CD_1_GPIO_AF           (XLLP_MFP_PIN_GPIO4_AF_GPIO_4)

#endif  // def  COMPILING_CODE_MUSEUM


#endif // __XLLP_MFP_PLAT_P_PL_H__


