/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __XLLP_PCCARDSOCKET_H__
#define __XLLP_PCCARDSOCKET_H__

#include "xllp_defs.h"
#include "xllp_gpio.h"
#include "xllp_memc.h"



//****************************************************
//
//                  SYMBOLIC CONSTANTS
//
//****************************************************

//
// PC Card interface socket definitions
//
#define XLLP_PCCARD_SOCKET0 0

//
//Supported voltage settings for PC Cards on the 
//Zylonite reference platform
//
#define XLLP_PCCARD_3_30VOLTS 1

//
//Miscellaneous definitions
//
#define XLLP_GPIO_CFCD 30
#define XLLP_GPIO_NIRQ 98

//
//PC Card Expansion Memory Timing Configuration Registers structure
//
typedef struct __Xllp_CardTiming_S
{
	XLLP_VUINT32_T mcmem0;
	XLLP_VUINT32_T mcatt0 ;
	XLLP_VUINT32_T mcio0 ;
}XLLP_CARDTIMING_T, *P_XLLP_CARDTIMING_T;

//
//PC Card Socket State structure. This structure is used to reflect the current state
//of PC Card interface status bits (in the PC Card Status registers)
//
typedef struct __Xllp_PCCardSocketState_S
{
	XLLP_VUINT8_T blSocket0CDState;
	XLLP_VUINT8_T blSocket0BVD1State;
	XLLP_VUINT8_T blSocket0BVD2State;
	XLLP_VUINT8_T blSocket0IREQState;
 }XLLP_PCCARD_SOCKET_STATE_T, *P_XLLP_PCCARD_SOCKET_STATE_T;

//
//PC Card Socket Device Handle. This handle is common for all sockets, i.e.
//it is not socket-specific.
//
typedef struct __Xllp_PCCardSocket_S
{
	P_XLLP_GPIO_T					GpioRegsHandle;    		//Pointer to the Monahans GPIO registers base
	P_XLLP_VUINT32_T				PBase;     				// MFPR base address
  	P_XLLP_INT8_T				FPRmDb;     				// MFP Resource Manager database
 	XLLP_PCCARD_SOCKET_STATE_T	*pstrPCCardSocketState; //Pointer to the Socket State structure
	P_XLLP_CARDTIMING_T				trCardTimingHandle;  	//Pointer to the EXPANSION MEMORY TIMING 
						                                                       		//CONFIGURATION structure
	P_XLLP_SMEM_REGISTER_T 			pStaticMemCtl;			//Pointer static memory control
	P_XLLP_VUINT32_T	hd_switch_addr; // Address of the switch of TrueIDE on CF module
}XLLP_PCCARDSOCKET_T, *P_XLLP_PCCARDSOCKET_T;

#ifdef __cplusplus
extern "C"{
#endif 

//****************************************************
//
//                  FUNCTION PROTOTYPES
//
//****************************************************

XLLP_STATUS_T XllpPCCardHWSetup(XLLP_PCCARDSOCKET_T *);

XLLP_STATUS_T XllpPCCardGetSocketState(XLLP_PCCARDSOCKET_T *);

XLLP_STATUS_T XllpPCCardGetVoltageSetting(XLLP_PCCARDSOCKET_T *,
                                          XLLP_UINT32_T *);

XLLP_STATUS_T XllpPCCardSetExpMemTiming(XLLP_PCCARDSOCKET_T *);

XLLP_STATUS_T XllpPCCardGetExpMemTiming(XLLP_PCCARDSOCKET_T *, 
                                        XLLP_CARDTIMING_T *);

#ifdef __cplusplus
}
#endif

#endif //__XLLP_PCCARDSOCKET_H__
