/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:       xllp_ethernet.c
**
**  PURPOSE:        Xllp ethernet file
**
**
******************************************************************************/

/*
*******************************************************************************
*   HEADER FILES
*******************************************************************************
*/
#include "xllp_ethernet.h"
#include "xllp_ost.h"

/*
*******************************************************************************
*    LOCAL DEFINITIONS
*******************************************************************************
*/
static XLLP_BOOL_T dumpFlag = 0;
static XLLP_BOOL_T RxFlag = 0;
static XLLP_UINT32_T loggedError = 0;   // Error log
static P_XLLP_VUINT16_T LAN91C111IoP;   // I/O space

XLLP_UINT8_T wIntMask = 0;

#define FRAME_FILTER  0xAC00   // Allow broadcast

typedef struct EthernetFrameHeaderlab 
{
  XLLP_UINT16_T  wDestMAC[3];
  XLLP_UINT16_T  wSrcMAC[3];
  XLLP_UINT16_T  wFrameType;
} EthernetFrameHeader;

static volatile P_XLLP_OST_T TimerRegBaseP;

static
int getTimebase (void)
{
    return 13000000;
}

static
XLLP_UINT32_T getDelta (P_XLLP_OST_T regsP, unsigned start)
{
    volatile P_XLLP_OST_T OstControlRegsP = regsP;

    unsigned stop = OstControlRegsP->oscr0;
    if (stop < start)
        return stop + (start^0xffffffff) + 1;
    return stop - start;
}

/******************************************************************************

  Function Name: 
    XllpEthernetGetPointerToIORegistersBase

  Description: 
    This function returns the pointer to the Ethernet Controller register base.

  Global Register Modified: 
    None. 

  Input Arguments:
    None

  Output Arguments:
    None
          
  Return Value: 
    P_XLLP_UINT32_T - the pointer to the Ethernet Controller register base

*******************************************************************************/
static
P_XLLP_VUINT16_T XllpEthernetGetPointerToIORegistersBase (void)
{
    return LAN91C111IoP;
}
 
/******************************************************************************

  Function Name: 
    XllpEthernetSelectRegisterBank

  Description: 
    This function loads the bank number into the Ethernet Controller 
    bank select register.

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT32_T bank - a bank number 0-3, and 7

  Output Arguments:
    None
              
  Return Value: 
    None

*******************************************************************************/

static
void XllpEthernetSelectRegisterBank (XLLP_UINT32_T bank)
{
    P_XLLP_VUINT16_T ioRegsP = XllpEthernetGetPointerToIORegistersBase();     // Get pointer to I/O space
	*(ioRegsP + XLLP_LAN91C111_BANK_SELECT_16) = (XLLP_UINT16_T)(bank & XLLP_LAN91C111_BANKSEL_MASK) ;
}

/******************************************************************************
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
In the version for mainstone, this function reads the register through 32 bits and return a 32 value .
On Zylonite all the register can only be accessed through 16 bits. so we use two 16 bits access to simulat 
this function so as to change the codes as little as posibble.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  Function Name: 
    XllpEthernetReadDoubleWord

  Description: 
    This function reads a doubleword from the Ethernet Controller's registers 
    using two 16 -bit access

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32_T - the doubleword of data read from the register

*******************************************************************************/

static
XLLP_UINT32_T XllpEthernetReadDoubleWord(XLLP_UINT32_T offset)
{
    P_XLLP_VUINT16_T ioRegsP = XllpEthernetGetPointerToIORegistersBase();     // Get pointer to I/O space
    XLLP_UINT32_T tem_low = 0, tem_hi = 0, re=0;

   if (offset > 0)
      offset = offset >> 1;

    tem_low = (XLLP_UINT32_T)(*(ioRegsP + offset)); 
    tem_hi = (XLLP_UINT32_T)(*(ioRegsP + offset +1));
    re = (tem_low & 0xffff) |((tem_hi & 0xffff)<<16);	

    return re;
}

/******************************************************************************

  Function Name: 
    XllpEthernetReadWord16

  Description: 
    This function reads a word value from the Ethernet Controller's registers using 16-bit
    access.

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT16_T - the word of data read from the register

*******************************************************************************/

static
XLLP_UINT16_T XllpEthernetReadWord16(XLLP_UINT32_T offset)
{
    P_XLLP_VUINT16_T ioRegsP = XllpEthernetGetPointerToIORegistersBase (); // Get pointer to I/O space

    if (offset > 0)
      offset = offset >> 1;

	return (*(ioRegsP + offset));
}

/******************************************************************************

  Function Name: 
    XllpEthernetReadByte

  Description: 
    This function reads a byte value from the Ethernet Controller's registers using 16-bit
    access.

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT8_T - the byte of data read from the register

*******************************************************************************/

static
XLLP_UINT8_T XllpEthernetReadByte(XLLP_UINT32_T offset)
{
    P_XLLP_VUINT8_T ioRegsP = (P_XLLP_VUINT8_T)XllpEthernetGetPointerToIORegistersBase();     // Get pointer to I/O space

    return ( *(ioRegsP + offset));
}
/******************************************************************************
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
In the version for mainstone, this function writes the register through 32 bits access.
On Zylonite all the register can only be accessed through 16 bits. so we use two 16 bits access to simulat 
this function so as to change the codes as little as posibble.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  Function Name: 
    XllpEthernetWriteDoubleWord

  Description: 
    This function writes a doubleword to the Ethernet Controller's registers 
    using two 16-bit access
    Note: On mainstone board this function CAN NOT be used to write to the Ethernet Controller registers
          at the offset 0x0C, because it will write only to the Bank select register
          at the offset 0x0E.

  Global Register Modified: 
    None 

  Input Arguments:
    XLLP_UINT32_T value  - the value that will be written to the registers
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

static
void XllpEthernetWriteDoubleWord(XLLP_UINT32_T value, XLLP_UINT32_T offset)
{
    P_XLLP_VUINT16_T ioRegsP = XllpEthernetGetPointerToIORegistersBase();     // Get pointer to I/O space

    if (offset > 0)
      offset = offset >> 1;
    *(ioRegsP + offset) = (XLLP_UINT16_T)(value & 0xffff);
    *(ioRegsP + offset + 1) = (XLLP_UINT16_T)((value >> 16) & 0xffff);
}

/******************************************************************************

  Function Name: 
    XllpEthernetWriteWord16

  Description: 
    This function writes a word value to the Ethernet Controller's registers using 16-bit
    access.

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT16_T value  - the value that will be written to the register
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

static
void XllpEthernetWriteWord16(XLLP_UINT16_T value, XLLP_UINT32_T offset)
{
    P_XLLP_VUINT16_T ioRegsP = XllpEthernetGetPointerToIORegistersBase (); // Get pointer to I/O space

    if (offset > 0)
      offset = offset >> 1;
	*(ioRegsP + offset) = value;
}

/******************************************************************************

  Function Name: 
    XllpEthernetWriteByte

  Description: 
    This function writes a byte value to the Ethernet Controller's registers using 8-bit
    access.

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT8_T value  - the value that will be written to the register
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

static
void XllpEthernetWriteByte(XLLP_UINT8_T value, XLLP_UINT32_T offset)
{
    P_XLLP_VUINT8_T ioRegsP = (P_XLLP_VUINT8_T)XllpEthernetGetPointerToIORegistersBase();  // Get pointer to I/O space

    *(ioRegsP + offset) = value;
}
/******************************************************************************

  Function Name: 
    XllpEthernetDisplayTxStatus

  Description: 
    This function displays the bit settings for the Transmit Status Word

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT8_T value  - the value that will be written to the register
    XLLP_UINT32_T offset - offset to the register

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

void XllpEthernetDisplayTxStatus(void)
{
    XLLP_UINT16_T ephSR;

    // Get the transmit status.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteWord16(XLLP_LAN91C111_PTR_READ | XLLP_LAN91C111_PTR_AUTO_INCR, XLLP_LAN91C111_POINTER);
    ephSR = XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH);

    // Display the error bits.
    if (ephSR & XLLP_LAN91C111_EPHSR_ERRORS)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Transmit Status: %x", ephSR);
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_SNGL_COL)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Single Collision Detected");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_MUL_COL)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Multiple Collisions Detected");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_16COL)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: 16 Collisions Reached");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_SQET)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Signal Quality Error");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_TX_DEFR)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Transmit Deferred");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_LATCOL)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Late Collision");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_LOST_CARR)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Lost Carrier");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_EXC_DEF)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Excessive Deferral");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_LINK_OK)
    {
        ////DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Link Test Failed");
    }

    if (ephSR & XLLP_LAN91C111_EPHSR_TX_UNRN)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111: Transmission Underrun");
    }
}

/******************************************************************************

  Function Name: 
    XllpEthernetGetRxIntFlag

  Description: 
    This routine sets or cleares a receive interrupt status flag. 

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_BOOL_T rxIntFlag - a value that is used to set or clear the 
    receive interrupt status flag

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

static
void XllpEthernetSetRxIntFlag (XLLP_BOOL_T rxIntFlag)
{
    RxFlag = rxIntFlag;
}

/******************************************************************************

  Function Name: 
    XllpEthernetGetRxIntFlag

  Description: 
    This routine reads a status of the receive interrupt flag, and can be 
    used by the test to monitor if the receive interrupt took place

  Global Register Modified: 
    None. 

  Input Arguments:
    None.

  Output Arguments:
    P_XLLP_UINT16_T rxIntStatus - the receive interrupt status flag 
          
  Return Value: 
    None

*******************************************************************************/

void XllpEthernetGetRxIntFlag (P_XLLP_UINT16_T rxIntStatus)
{
    *rxIntStatus = RxFlag;
}

/******************************************************************************

  Function Name: 
    XllpEthernetReadEEPROM

  Description: 
    This routine reads a value from the EEPROM

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT16_T offset - Offset within the EEPROM to read.

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

static
XLLP_UINT16_T XllpEthernetReadEEPROM(XLLP_UINT16_T offset)
{
    XLLP_UINT16_T   lanReg;                          // Temporary holding register
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = XLLP_LAN91C111_TO_EEPROM * timebase;

    // Assume success.
    loggedError = 0;

    // Select Bank 2
    XllpEthernetSelectRegisterBank (BANK2);

    // Set the LAN91C111 pointer register to the specified EEPROM address.
	XllpEthernetWriteWord16(offset, XLLP_LAN91C111_POINTER);

    // Select Bank 1
    XllpEthernetSelectRegisterBank (BANK1);

    // Read the Control Register.
    lanReg = XllpEthernetReadWord16(XLLP_LAN91C111_CONTROL);

    // The LAN91C111 reads the Configuration, Base and Individual Address, and
    // STORE writes the Configuration and Base registers. Also when set it will 
    // read the EEPROM and update relevant registers with its contents. This bit
    // then Clears upon completing the operation.
    lanReg |= (XLLP_LAN91C111_CTR_RELOAD | XLLP_LAN91C111_CTR_EEPROM);

    // Store the data to EEPROM.
    XllpEthernetWriteWord16(lanReg, XLLP_LAN91C111_CONTROL);

    // Prepare for timeout by getting the initial time interval.
    start = TimerRegBaseP->oscr0;
    // Verify that the data was read successfully. This assumes Bank 1 has
    // been selected from the previous operation.
    while (1)
    {
        // Read the Control Register.
        lanReg = XllpEthernetReadWord16(XLLP_LAN91C111_CONTROL);

        // Only interested in the STORE and RELOAD bits.
        lanReg = lanReg & (XLLP_UINT16_T)(XLLP_LAN91C111_CTR_STORE | XLLP_LAN91C111_CTR_RELOAD);

        // Complete when the STORE and RELOAD bits are cleared.
        if (0 == lanReg)
        {
            break;
        }

        // Get the current time interval.
        if ((getDelta(TimerRegBaseP, start)) > timeout)
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout reading EEPROM");
            LOGERROR(&loggedError, 
                    ERR_S_EEPROM, ERR_T_TIMEOUT, 
                    0);
            return (0);
        }
    }

    // Read the EEPROM data and return.
    return (XllpEthernetReadWord16(XLLP_LAN91C111_GEN_PURPOSE));
}

/******************************************************************************

  Function Name: 
    XllpEthernetWriteEEPROM

  Description: 
    This routine provides access to the 64 x 16-bit EEPROM attached to the
    LAN91C111.

  Global Register Modified: 
    None. 

  Input Arguments:
    XLLP_UINT16_T offset - Offset within the EEPROM to write.
    XLLP_UINT16_T value - Value to write into EEPROM.
          
  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

static
void XllpEthernetWriteEEPROM(XLLP_UINT16_T offset, XLLP_UINT16_T value)
{
    XLLP_UINT16_T   lanReg;           // Temporary holding register
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = XLLP_LAN91C111_TO_EEPROM * timebase;

    // Assume success.
    loggedError = 0;

    // Select Bank 2
    XllpEthernetSelectRegisterBank (BANK2);

    // Set the LAN91C111 pointer register to the specified EEPROM address.
    XllpEthernetWriteWord16(offset, XLLP_LAN91C111_POINTER);

    // Select Bank 1
    XllpEthernetSelectRegisterBank (BANK1);

    // Write the specified data to the EEPROM.
    XllpEthernetWriteWord16(value, XLLP_LAN91C111_GEN_PURPOSE);

    // Read the Control Register. This assumes Bank1 is selected.
    lanReg = XllpEthernetReadWord16(XLLP_LAN91C111_CONTROL);
  
    // The STORE LAN91C111 bit when set, stores the contents of all relevant
    // registers in the serial EEPROM. This bit is cleared upon completing
    // the operation.
    //
    // Note: When an EEPROM access is in progress the STORE and RELOAD bits
    // will be read back as high. The remaining 14 bits of this registers will
    // be invalid. During this time, attempted read/write operations, other
    // than polling the EEPROM status, will NOT have any effect the internal
    // registers. The CPU can resume accesses to the LAN91C111 after both bits
    // are low. A worst case RELOAD operation initated by RESET or by software
    // takes less than 750 usec in either mode.
    lanReg |= (XLLP_LAN91C111_CTR_STORE | XLLP_LAN91C111_CTR_EEPROM);

    // Store the data to EEPROM. This assumes Bank 1 is selected.
    XllpEthernetWriteWord16(lanReg, XLLP_LAN91C111_CONTROL);

    // Prepare for timeout by getting the initial time interval.
//    start = ostCtxP->getTimer_fnp(ostCtxP);
    start = TimerRegBaseP->oscr0;

    // Verify that the data was read successfully. This assumes Bank 1 has
    // been selected from the previous operation.
    while (1)
    {
        // Read the Control Register.
        lanReg = XllpEthernetReadWord16(XLLP_LAN91C111_CONTROL);
        // Only interested in the STORE and RELOAD bits.
        lanReg &= (XLLP_LAN91C111_CTR_STORE | XLLP_LAN91C111_CTR_RELOAD);

        // Complete when the STORE and RELOAD bits are cleared.
        if (0 == lanReg)
        {
            break;
        }

        // Get the current time interval.
        if ((getDelta(TimerRegBaseP, start)) > timeout)
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout writing EEPROM");
            LOGERROR(&loggedError, 
                     ERR_S_EEPROM, ERR_T_TIMEOUT, 
                     0);
            return;
        }
    }
}

/******************************************************************************

  Function Name: 
    XllpEthernetGetEEPROMMACAddress

  Description: 
    This routine return the MAC address stored in the EEPROM.

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT16_T macAddressP - pointer to an array of tree 16-bit words

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32_T - error code in case of failure reading the EEPROM, zero if success.

*******************************************************************************/

XLLP_UINT32_T XllpEthernetGetEEPROMMACAddress(P_XLLP_UINT16_T macAddressP)
{
    // Assume success.
    loggedError = 0;

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Reading MAC address from EEPROM");

    // WARNING: This routine changes the selected Bank register.
    macAddressP[0] = XllpEthernetReadEEPROM(EEPROM_MAC_OFFSET_1);
    if (loggedError)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout reading EEPROM");
        // Error reading the EEPROM.
        LOGERROR(&loggedError, 
                 ERR_S_BAD_MAC, ERR_T_UNEXPECTED, 
                 macAddressP[0]);
        return (loggedError);
    }

    macAddressP[1] = XllpEthernetReadEEPROM(EEPROM_MAC_OFFSET_2);
    if (loggedError)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout reading EEPROM");
        // Error reading the EEPROM.
        LOGERROR(&loggedError, 
                 ERR_S_BAD_MAC, ERR_T_UNEXPECTED, 
                 macAddressP[1]);
        return (loggedError);
    }

    macAddressP[2] = XllpEthernetReadEEPROM(EEPROM_MAC_OFFSET_3);
    if (loggedError)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout reading EEPROM");
        // Error reading the EEPROM.
        LOGERROR(&loggedError, 
                 ERR_S_BAD_MAC, ERR_T_UNEXPECTED, 
                 macAddressP[2]);
        return (loggedError);
    }

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0,
    //               " EEPROM LAN91C111 MAC Address: %02X-%02X-%02X-%02X-%02X-%02X",
    //               (macAddressP[0] & 0x00FF), (macAddressP[0] >> 8),
    //               (macAddressP[1] & 0x00FF), (macAddressP[1] >> 8),
    //               (macAddressP[2] & 0x00FF), (macAddressP[2] >> 8));

    return (loggedError);
}

/******************************************************************************

  Function Name: 
    XllpEthernetStoreEEPROMMACAddress

  Description: 
    This routine saves the specified MAC address into EEPROM.

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT16_T macAddressP - pointer to an array of tree 16-bit words

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32_T - error code in case of failure reading the EEPROM, zero if success.

*******************************************************************************/

XLLP_UINT32_T XllpEthernetStoreEEPROMMACAddress(P_XLLP_UINT16_T macAddressP)
{
    // Assume success.
    loggedError = 0;

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Storing MAC address in EEPROM");

    // WARNING: This routine changes the selected Bank register.
    XllpEthernetWriteEEPROM(EEPROM_MAC_OFFSET_1_W, macAddressP[0]);
    if (loggedError)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout writing EEPROM");
        // Error writing the MAC address.
        LOGERROR(&loggedError, 
                ERR_S_BAD_MAC, ERR_T_UNEXPECTED, 
                 macAddressP[0]);
        return (loggedError);
    }

    // WARNING: This routine changes the selected Bank register.
    XllpEthernetWriteEEPROM(EEPROM_MAC_OFFSET_2_W, macAddressP[1]);
    if (loggedError)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout writing EEPROM");
        // Error writing the MAC address.
        LOGERROR(&loggedError, 
                 ERR_S_BAD_MAC, ERR_T_UNEXPECTED, 
                 macAddressP[1]);
        return (loggedError);
    }

    // WARNING: This routine changes the selected Bank register.
    XllpEthernetWriteEEPROM(EEPROM_MAC_OFFSET_3_W, macAddressP[2]);
    if (loggedError)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout writing EEPROM");
        // Error writing the MAC address.
        LOGERROR(&loggedError, 
                 ERR_S_BAD_MAC, ERR_T_UNEXPECTED, 
                 macAddressP[2]);
        return (loggedError);
    }

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0,
    //               " EEPROM LAN91C111 MAC Address: %02X-%02X-%02X-%02X-%02X-%02X",
    //               (macAddressP[0] & 0x00FF), (macAddressP[0] >> 8),
    //               (macAddressP[1] & 0x00FF), (macAddressP[1] >> 8),
    //               (macAddressP[2] & 0x00FF), (macAddressP[2] >> 8));

    return (loggedError);
}

/******************************************************************************

  Function Name: 
    XllpEthernetGetMACAddress

  Description: 
    This routine return the current MAC address, read from the IA0-IA5 registers.

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT16_T macAddressP - pointer to an array of tree 16-bit words

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32_T - error code in case of failure reading the EEPROM, zero if success.

*******************************************************************************/

void XllpEthernetGetMACAddress(P_XLLP_UINT16_T macAddressP)
{
    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Reading MAC address");

    // Read the MAC address from the LAN91C111. This gets read from the EEPROM
    // by the LAN91C111 during reset. This assumes Bank 1 is selected.
    XllpEthernetSelectRegisterBank (BANK1);
    macAddressP[0] = XllpEthernetReadWord16(XLLP_LAN91C111_IA0);
    macAddressP[1] = XllpEthernetReadWord16(XLLP_LAN91C111_IA2);
    macAddressP[2] = XllpEthernetReadWord16(XLLP_LAN91C111_IA4);

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0,
    //               " LAN91C111 MAC Address: %02X-%02X-%02X-%02X-%02X-%02X",
    //               (macAddressP[0] & 0x00FF), (macAddressP[0] >> 8),
    //               (macAddressP[1] & 0x00FF), (macAddressP[1] >> 8),
    //               (macAddressP[2] & 0x00FF), (macAddressP[2] >> 8));
}

/******************************************************************************

  Function Name: 
    XllpEthernetStoreMACAddress

  Description: 
    This routine saves the MAC address in the IA0-IA5 registers.

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT16_T macAddressP - pointer to an array of tree 16-bit words

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

void XllpEthernetStoreMACAddress(P_XLLP_UINT16_T macAddressP)
{
    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Storing MAC address");

    // Access the I/O space and select Bank 1.
    XllpEthernetSelectRegisterBank (BANK1);

    // Write the generated MAC address to the LAN91C111.
    XllpEthernetWriteWord16(macAddressP[0], XLLP_LAN91C111_IA0);
    XllpEthernetWriteWord16(macAddressP[1], XLLP_LAN91C111_IA2);
    XllpEthernetWriteWord16(macAddressP[2], XLLP_LAN91C111_IA4);

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0,
    //               " LAN91C111 MAC Address: %02X-%02X-%02X-%02X-%02X-%02X",
    //               (macAddressP[0] & 0x00FF), (macAddressP[0] >> 8),
    //               (macAddressP[1] & 0x00FF), (macAddressP[1] >> 8),
   //                (macAddressP[2] & 0x00FF), (macAddressP[2] >> 8));
}

/******************************************************************************

  Function Name: 
    XllpEthernetDeallocateTxPacket

  Description: 
    This function deallocates memory after the transmit frame was completed
    by the Ethernet Controller. This function should be used only if the auto
    release feature is not used.

  Global Register Modified: 
    None. 

  Input Arguments:
    None

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32_T - TRUE if success, FALSE if failed

*******************************************************************************/

static
XLLP_UINT32_T XllpEthernetDeallocateTxPacket(void)
{
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = XLLP_LAN91C111_TO_ALLOC * timebase;
    XLLP_UINT16_T resultCode;
    XLLP_UINT8_T intStatus;

	loggedError = 0;

    // Get the transmit status.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteWord16(XLLP_LAN91C111_PTR_READ, XLLP_LAN91C111_POINTER);
    resultCode = XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH);

    // Clear the statistics register.
    XllpEthernetSelectRegisterBank (BANK1);
    XllpEthernetReadWord16(XLLP_LAN91C111_COUNTER);
    // Re-enable TXENA in the Transmit Control Register.
    XllpEthernetWriteWord16((XLLP_UINT16_T)XLLP_LAN91C111_TCR_TXENA | XllpEthernetReadWord16(XLLP_LAN91C111_TCR), XLLP_LAN91C111_TCR);

    // Release the memory in the buffer for the frame that failed.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteWord16(XLLP_LAN91C111_MMUCR_RELEASE_TX, XLLP_LAN91C111_MMU);

    // Prepare for timeout by getting the initial time interval.
    start = TimerRegBaseP->oscr0;

    // Wait for the request to complete by monitoring the NO BUSY bit in
    // the MMU Command Register.
    while (!XllpEthernetReadWord16(XLLP_LAN91C111_MMU) & XLLP_LAN91C111_MMUCR_NO_BUSY)
    {
        // Get the current time interval.
        if (getDelta(TimerRegBaseP, start) > timeout)
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Error deallocating Tx frame");
            // Report timeout error.
            LOGERROR(&loggedError, 
                     ERR_S_DEALLOC, ERR_T_TIMEOUT, 
                     0);
            return (loggedError);
        }
    }

    // Get the Interrupt Status Register.
    intStatus = XllpEthernetReadByte(XLLP_LAN91C111_INT_STATS);
    // Enable transmit interrupt if set.
    if (intStatus & XLLP_LAN91C111_IST_TX_INT)
    {
        // Acknowledge the Transmit interrupt.
        XllpEthernetWriteByte(intStatus | (XLLP_UINT8_T)XLLP_LAN91C111_IST_TX_INT, XLLP_LAN91C111_INT_ACK);
    }

    return (0);
}

/******************************************************************************

  Function Name: 
    XllpEthernetAllocateTxPacket

  Description: 
    This function allocates memory for the transmit from the Ethernet Controller

  Global Register Modified: 
    None. 

  Input Arguments:
    None

  Output Arguments:
    P_XLLP_UINT16_T packetHandle - return a packet number that was allocated
          
  Return Value: 
    XLLP_UINT32_T - TRUE if success, FALSE if failed

*******************************************************************************/

static
XLLP_UINT32_T XllpEthernetAllocateTxPacket (P_XLLP_UINT16_T packetHandle)
{
    XLLP_UINT32_T errorCode = 0;
    XLLP_UINT16_T tempValue;
    XLLP_UINT16_T lastPacket;
    XLLP_UINT8_T status;
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = XLLP_LAN91C111_TO_TRANSMIT * timebase;
    XLLP_UINT16_T lanReg;

	loggedError = 0;

    // Read the Control Register.
    lanReg = XllpEthernetReadWord16(XLLP_LAN91C111_CONTROL);
    // Check the auto release bit to determine if we should release the buffer.
    if (!(lanReg & XLLP_LAN91C111_CTR_AUTO_RELEASE))
    {
    	XllpEthernetDeallocateTxPacket();
    }

    // Allocate memory in the buffer for the frame.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteWord16(XLLP_LAN91C111_MMUCR_ALLOC_TX, XLLP_LAN91C111_MMU);


    start = TimerRegBaseP->oscr0;
    // Wait for the request to complete by monitoring the ALLOC INT bit in
    // the Interrupt Status Register.
    while (!((status = XllpEthernetReadByte(XLLP_LAN91C111_INT_STATS)) & XLLP_LAN91C111_IST_ALLOC_INT))
    {
        if (getDelta(TimerRegBaseP, start) > timeout)
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Error allocating Tx frame");
            LOGERROR(&loggedError, 
                     ERR_S_ALLOC, ERR_T_NOTRANSMIT, 
                     0);
            errorCode = loggedError;
			break;
		}
	}

    if (status & XLLP_LAN91C111_IST_ALLOC_INT)
    {
        // Get the packet number just allocated.
        tempValue = XllpEthernetReadWord16(XLLP_LAN91C111_PNR);
        lastPacket = (tempValue >> 8) & 0x7;
        *packetHandle = lastPacket; 
    }

    return (errorCode);
}

/******************************************************************************

  Function Name: 
    XllpEthernetSetLoopBack

  Description: 
    This function puts the Ethernet Controller into one of the three loopback modes:
    EPH loopback - where the packet is looped back immediately after the EPH block,
                   The packet never reaches the MII bus or the internal PHY.
    PHY half loopback - where the packet is sent out of the MAC, through the internal PHY
                   and loops back after crossing MII bus. PHY half duplex mode should
                   be used to allow MAC to receive a packet with its own source address.
    PHY full loopback - is used for external loopback. The packet is sent out of the MAC, 
                   through the internal PHY, out of RJ45 connector and loops back through
                   external wiring.                    

  Global Register Modified: 
    None 

  Input Arguments:
    LAN91C111LoopbackTypeT   type     - a type of the loopback 
    LAN91C111LoopbackEnableT enable   - the loopback is enabled if one, disabled if zero

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

void XllpEthernetSetLoopBack(XllpEthernetLoopbackTypeT type,
						     XllpEthernetLoopbackEnableT enable)
{
    XLLP_UINT16_T TCR;
    XLLP_UINT16_T RPCR;

    // Select Bank 0
    XllpEthernetSelectRegisterBank (BANK0);
    // Get the Transmit Control Register.
    TCR = XllpEthernetReadWord16(XLLP_LAN91C111_TCR);
    // Get the RPCR Register.
    RPCR = XllpEthernetReadWord16(XLLP_LAN91C111_RPCR);

    if (type == XLLP_ETHERNET_EPH_LOOPBACK)
    {
        // EPH Loopback test
        if (enable)
        {
            TCR |= XLLP_LAN91C111_TCR_EPH_LOOP;
        }
        else
        {
            // Clear internal loopback.
            TCR &= ~XLLP_LAN91C111_TCR_EPH_LOOP;
        }
    }
    else if (type == XLLP_ETHERNET_PHY_FULL_LOOPBACK)
    {
        // PHY Loopback test with full duplex
        if (enable)
        {
            TCR |= XLLP_LAN91C111_TCR_SWFDUP;
            RPCR |= XLLP_LAN91C111_RPCR_DPLX;
        }
        else
        {
            // Clear internal loopback.
            TCR &= (~XLLP_LAN91C111_TCR_SWFDUP & ~XLLP_LAN91C111_TCR_FDUPLX);
            RPCR &= ~XLLP_LAN91C111_RPCR_DPLX;
        }
    }
    else if (type == XLLP_ETHERNET_PHY_HALF_LOOPBACK)
    {
        // PHY Loopback test with half duplex
        if (enable)
        {
            TCR |= XLLP_LAN91C111_TCR_FDUPLX;
        }
        else
        {
            // Clear internal loopback.
            TCR &= ~XLLP_LAN91C111_TCR_FDUPLX;
        }

	}
    // Update the Transmit Control Register.
    XllpEthernetWriteWord16(TCR, XLLP_LAN91C111_TCR);
    // Update the RPCR Register.
    XllpEthernetWriteWord16(RPCR, XLLP_LAN91C111_RPCR);
}

/******************************************************************************

  Function Name: 
    XllpEthernetDumpFrame

  Description: 
    This routine used for debugging to display the ethernet frame.

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UCHAR_T frame - a pointer to the frame.
    XLLP_UINT16_T length - length of the frame.

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/
void XllpEthernetDumpFrame(P_XLLP_UINT8_T frame, XLLP_UINT16_T length)
{
    XLLP_UINT32_T x;

    //printf(" To: %02x:%02x:%02x:%02x:%02x:%02x\r\n",
    //          frame[0], frame[1], frame[2],
    //          frame[3], frame[4], frame[5]);
    //printf(" From: %02x:%02x:%02x:%02x:%02x:%02x\r\n",
    //          frame[6], frame[7], frame[8],
    //          frame[9], frame[10], frame[11]);
    //printf(" Type: 0x%x, Length: %u\r\n",
    //          ntohs(*((P_XLLP_UINT16_T)(frame + 12))), length);

    for (x = 14; x < length; x += 8)
    {
        //printf(" %02x %02x %02x %02x %02x %02x %02x %02x\r\n",
        //          frame[x],
        //          frame[x + 1],
        //          frame[x + 2],
        //          frame[x + 3],
        //          frame[x + 4],
        //          frame[x + 5],
        //          frame[x + 6],
        //          frame[x + 7]);
    }
}

/******************************************************************************

  Function Name: 
    XllpEthernetVerifyRAMBuffer

  Description: 
    This routine used for debugging to verify the data placed into the RAM.

  Global Register Modified: 
    None 

  Input Arguments:
    XLLP_UINT32_T bufferSize - a length of the frame stored in the RAM

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/
void XllpEthernetVerifyRAMBuffer (XLLP_UINT32_T bufferSize)
{
    XLLP_UINT16_T j;
	XLLP_UINT32_T testValue;

    XllpEthernetSelectRegisterBank (BANK2);
    for (j = 0; j < (XLLP_UINT16_T)bufferSize; j+=4)
    {
        // Read data 
        XllpEthernetWriteWord16((XLLP_UINT16_T)(XLLP_LAN91C111_PTR_AUTO_INCR | XLLP_LAN91C111_PTR_READ | j), XLLP_LAN91C111_POINTER);
        testValue = XllpEthernetReadDoubleWord(XLLP_LAN91C111_DATA_HIGH);
        //printf("%d: %08X\r\n", (int)j, (int)testValue);
    }
}

/******************************************************************************

  Function Name: 
    XllpEthernetTransmitPacket

  Description: 
    This routine transmits a frame. It polls the Interrupt Status Register for the
    completion of the TX
    - It allocates a packet for transmit frame.
    - Set the Auto Increment bit in the Pointer Register to increment the internal address
      on access to the data register.
    - Writes the frame into Tx FIFO
    - Enqueue the frame number into the TX FIFO.
    - Checks the Interrupt Status Register for Transmit Empty and Tx Error.
    - Acknowledges the Transmit interrupt or errors

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT16_T buffer  - a data buffer where the frame data will be taken from
    XLLP_UINT32_T length      - a length of the frame
  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT_T - error code if failed, zero if success

*******************************************************************************/
XLLP_UINT32_T XllpEthernetTransmitPacket(P_XLLP_UINT16_T buffer,
                                         XLLP_UINT32_T length)
{
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = 1300;
    XLLP_UINT8_T intStatus; 
    XLLP_UINT16_T bufferSize, x;
    XLLP_UINT16_T resultCode;
    XLLP_UINT16_T frameHandle = 0; 
    P_XLLP_UINT16_T dataP = buffer;
    XLLP_BOOL_T oddFlag = XLLP_FALSE;
    XLLP_UINT16_T revision;
    XLLP_UINT16_T uStat = 0;
    P_XLLP_VUINT8_T ioRegsP = (P_XLLP_VUINT8_T)XllpEthernetGetPointerToIORegistersBase (); // Get pointer to I/O space

doItAgain:

	timeout = 13000;
	oddFlag = XLLP_FALSE;
	timebase = getTimebase();

//////////////////////////////////
	// Evil hack to work around the memory leak we are currently experiencing with autorelease mode.
	// Expect that future rework to the transmit routine that checks the EPH status register will fix this problem.
	// Until then, check the available memory, and if it has decreased, reset the MMU to release these packets.
	XllpEthernetSelectRegisterBank (BANK0);
	// Upper half word

	bufferSize=XllpEthernetReadWord16(XLLP_LAN91C111_MIR) >> 8;
	
	

	// read available memory
	// if too low, reset device.
	if (bufferSize < 3)
	{
		XllpEthernetSelectRegisterBank (BANK2);
		XllpEthernetWriteWord16(XLLP_LAN91C111_MMUCR_RESET_MMU, XLLP_LAN91C111_MMU);
	}
//////////////////////////////////

    // Read the LAN91C111 revision. This assumes Bank 3 is selected.
    XllpEthernetSelectRegisterBank (BANK3);
    revision = XllpEthernetReadWord16(XLLP_LAN91C111_REVISION);

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " LAN91C111 Revision: %d, Chip: %d",
    //               (revision & 0x0F),
    //               (revision & 0xF0) >> 4);

    // Clear the error log.
    loggedError = 0;

    // Allocate a transmit frame from the LAN91C111.
    if (XllpEthernetAllocateTxPacket(&frameHandle))
    {
        return loggedError;    
    }

    // Write the allocated packet number into the Packet Number Register.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteByte((XLLP_UINT8_T)frameHandle, XLLP_LAN91C111_PNR);

    // Set the Auto Increment bit in the Pointer Register to increment the
    // internal address on access to the data register.
    XllpEthernetWriteWord16(XLLP_LAN91C111_PTR_AUTO_INCR, XLLP_LAN91C111_POINTER);

    // Calculate the memory needed. This includes the status word, byte count
    // data and control rounded up.
    bufferSize = (XLLP_UINT16_T)(sizeof(XLLP_UINT16_T) + sizeof(XLLP_UINT16_T) + length + 1);
    if (bufferSize & 1)
    {
        // Round the size up.
        bufferSize++;
    }
    else
    {
        // Check for Revision A for odd byte workaround.
        if ((revision & 0x0F) <= 1)
        {
            // Add an extra 2 bytes to the internal buffer length to workaround
            // the odd byte problem.
            bufferSize += 3;
            oddFlag = XLLP_TRUE;
            //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Odd byte workaround");
        }
    }

    // Write the status word.
    XllpEthernetWriteWord16(0, XLLP_LAN91C111_DATA_HIGH);
    // Write the byte count.
    XllpEthernetWriteWord16(bufferSize, XLLP_LAN91C111_DATA_HIGH);

    // Write to the packet buffer:
    for (x = 0; x < length; x+=2)
    {
        // Write the data.
        XllpEthernetWriteWord16(*dataP++, XLLP_LAN91C111_DATA_HIGH);
    }

    // Check for Revision A for odd byte workaround.
    if ((revision & 0x0F) <= 1)
    {
        // Check for odd byte packets.
        if (oddFlag)
        {
            // Write the last byte of the packet with padding.
            XllpEthernetWriteWord16((*dataP & 0xFF), XLLP_LAN91C111_DATA_HIGH);
        }

        // Write just the control byte. Packet will alway be even.
        XllpEthernetWriteWord16((XLLP_LAN91C111_CONTROL_CRC << 8), XLLP_LAN91C111_DATA_HIGH);
    }
    else
    {
        if (length & 1)
        {
            // Write the control byte and last data byte.
             XllpEthernetWriteWord16((((XLLP_LAN91C111_CONTROL_CRC | XLLP_LAN91C111_CONTROL_ODD) << 8) | 
                                       (*(dataP) & 0xFF)), XLLP_LAN91C111_DATA_HIGH);
        }
        else
        {
            // Write just the control byte.
            XllpEthernetWriteWord16((XLLP_LAN91C111_CONTROL_CRC << 8), XLLP_LAN91C111_DATA_HIGH);
        }
    }

    // XllpEthernetVerifyRAMBuffer (bufferSize);
 
    // Enqueue the frame number into the TX FIFO.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteWord16(XLLP_LAN91C111_MMUCR_ENQUEUE, XLLP_LAN91C111_MMU);

    // Prepare for timeout by getting the initial time interval.
    start = TimerRegBaseP->oscr0;
    // Check the Interrupt Status Register for Transmit Empty.
    while (!((intStatus = XllpEthernetReadByte(XLLP_LAN91C111_INT_STATS)) &
            XLLP_LAN91C111_IST_TX_EMPTY_INT)) 
    {    
        if (getDelta(TimerRegBaseP, start) > timeout)
        {
            // Parse the transmit error bits.
            XllpEthernetDisplayTxStatus();

            // Cleanup after the transmit error.
            if (XllpEthernetDeallocateTxPacket())
            {
			 	goto doItAgain;
            }

            // Disable the transmit queue.
			XllpEthernetSelectRegisterBank (BANK0);
            uStat = XllpEthernetReadWord16(XLLP_LAN91C111_TCR);
			uStat &= ~XLLP_LAN91C111_TCR_TXENA;
			XllpEthernetWriteWord16(uStat,XLLP_LAN91C111_TCR);

			// Reset the Tx FIFO's.
		    XllpEthernetSelectRegisterBank (BANK2);
            XllpEthernetWriteWord16(XLLP_LAN91C111_MMUCR_RESET_TX, XLLP_LAN91C111_MMU);

			// Re-enable the Tx queue.
			XllpEthernetSelectRegisterBank (BANK0);
			uStat = XllpEthernetReadWord16(XLLP_LAN91C111_TCR);
			uStat |= XLLP_LAN91C111_TCR_TXENA;
			XllpEthernetWriteWord16(uStat, XLLP_LAN91C111_TCR);

            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout Tx frame");
            LOGERROR(&loggedError, 
                     ERR_S_ALLOC, ERR_T_TIMEOUT, 
                     intStatus);

			goto doItAgain;
        }
    }

    // Now check for transmit complete.
    if (intStatus & XLLP_LAN91C111_IST_TX_EMPTY_INT)
    {
        // Acknowledge Transmit Empty Interrupt.
        XllpEthernetWriteByte((XLLP_UINT8_T)(intStatus | XLLP_LAN91C111_IST_TX_EMPTY_INT), XLLP_LAN91C111_INT_STATS);
    }

    // Now check for transmit complete with no errors.
    if (intStatus & XLLP_LAN91C111_IST_TX_INT)
    {
        // Acknowledge the Transmit interrupt.
        XllpEthernetWriteByte((XLLP_UINT8_T)(intStatus | XLLP_LAN91C111_IST_TX_INT), XLLP_LAN91C111_INT_STATS);
        resultCode = XllpEthernetReadWord16(XLLP_LAN91C111_FIFO);
        if (!(resultCode & XLLP_LAN91C111_FIFO_TEMPTY))
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Tx FIFO not empty");
            XllpEthernetWriteByte((XLLP_UINT8_T)resultCode, XLLP_LAN91C111_PNR);
            LOGERROR(&loggedError, 
                     ERR_S_TRANSMIT, ERR_T_TIMEOUT, 
                     intStatus);
            return (loggedError);
        }
    }

    // Clear the collision count.
    XllpEthernetSelectRegisterBank (BANK0);
    XllpEthernetReadWord16(XLLP_LAN91C111_COUNTER);

    return (loggedError);
}

/******************************************************************************

  Function Name: 
    XllpEthernetDeallocateRxPacket

  Description: 
    This routine will deallocate a receive buffer and clear any errors.

  Global Register Modified: 
    None 

  Input Arguments:
    None

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32_T - error code if failed, zero if success

*******************************************************************************/
static
XLLP_UINT32_T XllpEthernetDeallocateRxPacket(void)
{
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = XLLP_LAN91C111_TO_ALLOC * timebase;

	loggedError = 0;

    // Release the memory from the received Frame.
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteWord16(XLLP_LAN91C111_MMUCR_REM_REL_RX, XLLP_LAN91C111_MMU);

    // Prepare for timeout by getting the initial time interval.
    start = TimerRegBaseP->oscr0;

    while (XllpEthernetReadWord16(XLLP_LAN91C111_MMU) & XLLP_LAN91C111_MMUCR_NO_BUSY)
    {
        // Get the current time interval.
        if ((getDelta(TimerRegBaseP, start)) > timeout)
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Error deallocating Rx frame");
            // Report timeout error.
            LOGERROR(&loggedError, 
                     ERR_S_RECEIVE, ERR_T_TIMEOUT, 
                     0);
            return (loggedError);
        }
    }
    return (loggedError);
}

/******************************************************************************

  Function Name: 
    XllpEthernetReceiveStatus

  Description: 
    This routine returns the status of the Interrupt Status Register for 
    Receive Interrupt of the Ethernet Controller

  Global Register Modified: 
    None 

  Input Arguments:
    None

  Output Arguments:
    P_XLLP_UINT16_T rxIntStatusP - returns TRUE if the Receive Interrupt took place, 
                                   FALSE otherwise
          
  Return Value: 
    XLLP_UINT32_T - error code if failed, zero if success

*******************************************************************************/
XLLP_UINT32_T XllpEthernetReceiveStatus(P_XLLP_UINT16_T rxIntStatusP)
{
    XLLP_UINT8_T intStatus;
    XLLP_UINT32_T start, timebase = getTimebase();
    XLLP_UINT32_T timeout = XLLP_LAN91C111_TO_TRANSMIT * timebase;

	loggedError = 0;

    // Assume the failure
    *rxIntStatusP = XLLP_FALSE;

    // Prepare for timeout by getting the initial time interval.
//    start = ostCtxP->getTimer_fnp(ostCtxP);
    start = TimerRegBaseP->oscr0;

    // Check the Interrupt Status Register for Receive Interrupt.
    XllpEthernetSelectRegisterBank (BANK2);
    while (!((intStatus = XllpEthernetReadByte(XLLP_LAN91C111_INT_STATS)) &
            XLLP_LAN91C111_IST_RCV_INT))
    {    
        if (getDelta(TimerRegBaseP, start) > timeout)
        {
            //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Timeout Rx Status");
            LOGERROR(&loggedError, 
                     ERR_S_RECEIVE, ERR_T_TIMEOUT, 
                     intStatus);
			break;
        }
    }

    // Check for receive overrun errors
    if (intStatus & XLLP_LAN91C111_IST_RX_OVRN_INT)
    {
        // Acknowledge the Receive Overrun interrupt.
        XllpEthernetWriteByte(XLLP_LAN91C111_ACK_RX_OVRN_INT, XLLP_LAN91C111_INT_ACK);
    }
    // Check the Receive Interrupt.
    else if (intStatus & XLLP_LAN91C111_IST_RCV_INT)
    {
        // Indicate receive packet pending.
        *rxIntStatusP = XLLP_TRUE;
    }

    return (loggedError);
}

/******************************************************************************

  Function Name: 
    XllpEthernetInterruptHandler

  Description: 
    This routine handles the operations normally performed by an Interrupt
    Service Routine. Since we aren't using interrupts to drive transmit and
    receive, this routine is used in place of the ISR routine.

  Global Register Modified: 
    None 

  Input Arguments:
    None

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/
static
void XllpEthernetInterruptHandler(void)
{
    XLLP_UINT8_T intStatus, maskRegister, wBankSave;

    wBankSave = XllpEthernetReadByte( XLLP_LAN91C111_BANK_SELECT );

    // Get the Interrupt Status Register.
    XllpEthernetSelectRegisterBank (BANK2);
    intStatus = XllpEthernetReadByte(XLLP_LAN91C111_INT_STATS);

    // Get the Interrupt Mask Register.
    maskRegister = XllpEthernetReadByte(XLLP_LAN91C111_INT_MASK);

    // Disable interrupts.
    XllpEthernetWriteByte(0, XLLP_LAN91C111_INT_MASK);

    // Check for receive overrun errors
    if (intStatus & XLLP_LAN91C111_IST_RX_OVRN_INT)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Error Rx overrun");
        // Acknowledge the Receive Overrun interrupt.
        XllpEthernetWriteByte(XLLP_LAN91C111_ACK_RX_OVRN_INT, XLLP_LAN91C111_INT_ACK);
        LOGERROR(&loggedError, 
                 ERR_S_RECEIVE, ERR_T_NORECEIVE, 
                 intStatus);
    }

    // Check for receive interrupt.
    if (intStatus & XLLP_LAN91C111_IST_RCV_INT)
    {
        // Set the flag indicating a frame has been received.
        XllpEthernetSetRxIntFlag(XLLP_TRUE);

        // Clear the Receive Interrupt bit mask.
        maskRegister &= ~XLLP_LAN91C111_MSK_RCV_INT;
    }

    // Update the Interrupt Mask Register.
    XllpEthernetWriteByte(maskRegister, XLLP_LAN91C111_INT_MASK);
	
    XllpEthernetWriteByte(wBankSave, XLLP_LAN91C111_BANK_SELECT);
}

/******************************************************************************

  Function Name: 
    XllpEthernetReceivePacket

  Description: 
    This function receives a packet of data, gets the frame data and places
    the data into the user's buffer 

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT32_T buffer - a data buffer where the frame data will be stored

  Output Arguments:
    P_XLLP_INT rxCount - a number of bytes received in the frame
          
  Return Value: 
    XLLP_UINT32_T - error code if frame has not been received correctly, zero if success.

*******************************************************************************/
XLLP_UINT32_T XllpEthernetReceivePacket(P_XLLP_UINT32_T buffer, P_XLLP_INT32_T rxCount)
{

    XLLP_UINT16_T statusWord, wControlWord;
	XLLP_UINT16_T frameType;
    XLLP_UINT16_T x = 0;
	P_XLLP_UINT16_T dataP;
    XLLP_UINT16_T count = 0;
	XLLP_BOOL_T fDropFrame = XLLP_FALSE;
	XLLP_UINT16_T wPointerRegVal;
	
     // Clear the error log.
    loggedError = 0;

    XllpEthernetSetRxIntFlag(XLLP_FALSE);

    // Check for received frames.
    XllpEthernetInterruptHandler();

    // Process all received frames.
    if (RxFlag)
    {

     // Setup for Receive, Auto Increment and Read access.
       XllpEthernetSelectRegisterBank (BANK2);
		wPointerRegVal = XLLP_LAN91C111_PTR_RX_FRAME;
        XllpEthernetWriteWord16(wPointerRegVal, XLLP_LAN91C111_POINTER);

        // Get status word.
	statusWord = XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH);

        // Check for broadcast frames.
        if (statusWord & XLLP_LAN91C111_FRAME_FILTER)
        {
			fDropFrame = XLLP_TRUE;
        }
        else
        {
			fDropFrame = XLLP_FALSE;
            // Process the frame, get the pointer to the user buffer.
			dataP = (P_XLLP_UINT16_T)buffer;

            // Get the packet byte count, which is the byte count minus the
            // status word, byte count and control byte.
            *rxCount = (XLLP_UINT16_T)((XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH) & 0x7FF) - 6);

            // Check we have enough room to store the receive packet.
            if (*rxCount > XLLP_ETHERNET_MAX_LENGTH)
            {
                fDropFrame = XLLP_TRUE;
                goto ReleaseFrame;
            }
			
			x = 0;

			// Filter out all broadcast packets except ARPs.
			if(statusWord & XLLP_LAN91C111_BROD_CAST)
			{
				// Get the ethernet frame header
				for (; x < sizeof(EthernetFrameHeader); x++)
				{
					*dataP++ = XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH);
				}

				// Check the protocol type.
				frameType = xllpntohs(((XllpEthernetHeaderT *)buffer)->frameType);
                if ((frameType != XLLP_ETHERNET_TYPE_ARP) && (frameType != XLLP_ETHERNET_TYPE_IP))
				{
			        fDropFrame = XLLP_TRUE;
					goto ReleaseFrame;
				}
			}

			for (; x < (*rxCount >> 1); x++)
			{
				*dataP++ = XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH);
			}

			wControlWord = XllpEthernetReadWord16(XLLP_LAN91C111_DATA_HIGH);

			if (wControlWord & 0x2000)
			{
				*dataP = (XLLP_UINT8_T)wControlWord;
				(*rxCount)++;
			} else if (statusWord & XLLP_LAN91C111_ODD_FRM)
			{
				*dataP = (XLLP_UINT8_T)wControlWord;
				(*rxCount)++;
			}
		}
		
	ReleaseFrame:

        // Now deallocate the page.
        XllpEthernetDeallocateRxPacket();

		// Re-enable RX interrupts if using them.
		if (wIntMask & XLLP_LAN91C111_MSK_RCV_INT)
		{
			XllpEthernetEnableRxInterrupt();
		}
		XllpEthernetSetRxIntFlag(XLLP_FALSE);

        // Check for errors.
        if (statusWord & XLLP_LAN91C111_TOO_SHORT)
        {
			//DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Rx Frame too short"); 
			LOGERROR(&loggedError, 
					 ERR_S_RECEIVE, ERR_T_BADRANGE, 
					 statusWord);
        }

        if (statusWord & XLLP_LAN91C111_TOO_LONG)
        {
			//DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Rx Frame too long"); 
			LOGERROR(&loggedError, 
					 ERR_S_RECEIVE, ERR_T_BADRANGE, 
					 statusWord);
        }

        if (statusWord & XLLP_LAN91C111_BAD_CRC)
        {
			//DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Rx Frame Bad CRC"); 
			LOGERROR(&loggedError, 
					 ERR_S_RECEIVE, ERR_T_CRC, 
					 statusWord);
        }

        if (statusWord & XLLP_LAN91C111_ALGN_ERR)
        {
			//DM_CwDbgPrintf(DM_CW_LAN91C111_1, " Rx Frame Align Error"); 
			LOGERROR(&loggedError, 
					 ERR_S_RECEIVE, ERR_T_ILLALIGN, 
					 statusWord);
        }

		// Check dump flag.
		if (dumpFlag)
		{
			// Display received frame.
			//printf(" Receive Packet:\r\n");
			XllpEthernetDumpFrame((P_XLLP_UINT8_T)buffer, count - XLLP_ETHERNET_CRC_LEN);
		}

        if(!fDropFrame)
        {

	        // Take away CRC value.
	        // Return the receive count.
		*rxCount -= XLLP_ETHERNET_CRC_LEN;
            return (0);
        }
		
    } 
	*rxCount = 0;
    return (0);

}
/******************************************************************************

  Function Name: 
    XllpEthernetDumpRegisters

  Description: 
    This function prints the information from the ethernet controller registers

  Global Register Modified: 
    Ethernet Bank Select Register 

  Input Arguments:
    None

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/
void XllpEthernetDumpRegisters(void)
{
    P_XLLP_VUINT16_T ioRegsP = XllpEthernetGetPointerToIORegistersBase(); // Get pointer to I/O space
    XLLP_UINT16_T bank0, bank1, bank2, bank3;
    XLLP_INT32_T x;

    //printf(" LAN91C111 Bank Registers at %08x:\r\n", (XLLP_UINT32_T)  ioRegsP);
    //printf(" Offset Bank0 Bank1 Bank2 Bank3\r\n");

    for (x = 0; x < 16; x+=2)
    {
        // Select bank 0 register.
        XllpEthernetSelectRegisterBank (BANK0);
        // Read register contains.
        bank0 = XllpEthernetReadWord16(x);
        // Select bank 1 register.
        XllpEthernetSelectRegisterBank (BANK1);
        // Read register contains.
        bank1 = XllpEthernetReadWord16(x);
        // Select bank 2 register.
        XllpEthernetSelectRegisterBank (BANK2);
        // Read register contains.
        bank2 = XllpEthernetReadWord16(x);
        // Select bank 3 register.
        XllpEthernetSelectRegisterBank (BANK3);
        // Read register contains.
        bank3 = XllpEthernetReadWord16(x);
        //printf(" %02X     %04X  %04X  %04X  %04X\r\n",
        //        (int)x, bank0, bank1, bank2, bank3);
    }
	XllpEthernetSelectRegisterBank (BANK0);

    //printf(" LAN91C111 Registers Complete\r\n");
}

/******************************************************************************

  Function Name: 
    XllpEthernetReadPhyRegister

  Description: 
    This function reads the data from the specified PHY register

  Global Register Modified: 
    Ethernet Bank Select Register 

  Input Arguments:
    XLLP_UCHAR_T phyAddr - PHY Address
    XLLP_UCHAR_T phyReg  - PHY Register offset  

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT16_T - a value from the specified PHY register

*******************************************************************************/
XLLP_UINT16_T XllpEthernetReadPhyRegister(XLLP_UINT8_T phyAddr,
                                          XLLP_UINT8_T phyReg)
{
	XLLP_INT32_T n;
	XLLP_UINT8_T st_mask;
	XLLP_UINT16_T mii_reg;
	XLLP_UINT8_T stream[64];
	XLLP_INT32_T j = 0;
	XLLP_INT32_T k;
	XLLP_UINT16_T phyData;

	// Establish sync (32 consecutive ones on MDO) 
	for (n = 0; n < 32; ++n)
	{
		stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
	}

	// Start code (01)
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;

	// Read command (10)
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE;

	// PHY address (msb first)
	st_mask = (XLLP_UINT8_T)0x10;
	for (n = 0; n < 5; ++n)
	{
		if (phyAddr & st_mask)
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
		else
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
		st_mask >>= 1;
   	}

	// Output the phy register number (msb first)
	st_mask = (XLLP_UINT8_T)0x10;
	for (n = 0; n < 5; ++n)
	{
		if (phyReg & st_mask)
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
		else
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
		st_mask >>= 1;
    }

	stream[j++] = 0;

	// Input starts
	k = j;

	// Input 16 bits
	for (n = 0; n < 16; ++n)
	{
		stream[j++] = 0;
	}

	// Final clock bit
	stream[j++] = 0;

	// Select bank 3
    XllpEthernetSelectRegisterBank (BANK3);

	// Get the current MII register value
	mii_reg = XllpEthernetReadWord16(XLLP_LAN91C111_MGMT);

	// Turn off all MII Interface bits
	mii_reg &= ~(XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MCLK | 
	            XLLP_LAN91C111_MGMT_MDI | XLLP_LAN91C111_MGMT_MDO);

	// Clock 64 cycles
	for (n = 0; n < sizeof stream; ++n)
	{
		// Clock Low - output data
		XllpEthernetWriteWord16(mii_reg | stream[n], XLLP_LAN91C111_MGMT);
        XllpOstDelayMicroSeconds(TimerRegBaseP, 50);

		// Clock Hi - input data
		XllpEthernetWriteWord16((XLLP_UINT16_T)(mii_reg | stream[n] | XLLP_LAN91C111_MGMT_MCLK), XLLP_LAN91C111_MGMT);
        XllpOstDelayMicroSeconds(TimerRegBaseP, 50);

		stream[n] |= XllpEthernetReadWord16(XLLP_LAN91C111_MGMT) & XLLP_LAN91C111_MGMT_MDI;
	}

	// Return to idle
	XllpEthernetWriteWord16(mii_reg, XLLP_LAN91C111_MGMT);
    XllpOstDelayMicroSeconds(TimerRegBaseP, 50);

	// Recover input data
	phyData = 0;
	for (n = 0; n < 16; ++n)
	{
		phyData <<= 1;
		if (stream[k++] & XLLP_LAN91C111_MGMT_MDI)
			phyData |= 0x0001;
	}

	return(phyData);	
}

/******************************************************************************

  Function Name: 
    XllpEthernetWritePhyRegister

  Description: 
    This function writes to the PHY registers

  Global Register Modified: 
    None 

  Input Arguments:
    XLLP_UCHAR_T phyAddr - PHY Address
    XLLP_UCHAR_T phyReg  - PHY Register offset  
    XLLP_UINT16_T phyData - the data need to be written to the register

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/
void XllpEthernetWritePhyRegister(XLLP_UINT8_T phyAddr, 
                                  XLLP_UINT8_T phyReg,
	                              XLLP_UINT16_T phyData)
{
	XLLP_INT32_T n;
	XLLP_UINT16_T st_mask;
	XLLP_UINT16_T mii_reg;
	XLLP_UINT8_T stream[65];
	XLLP_INT32_T j = 0;

	// 32 consecutive ones on MDO to establish sync
	for (n = 0; n < 32; ++n)
	{
		stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
	}

	// Start code <01>
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;

	// Write command <01>
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
	stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;

	// Output the PHY address (msb first)
	st_mask = (XLLP_UINT8_T)0x10;
	for (n = 0; n < 5; ++n)
	{
		if (phyAddr & st_mask)
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
		else
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
		st_mask >>= 1;
	}

	// Output the phy register number, msb first
	st_mask = (XLLP_UINT8_T)0x10;
	for (n = 0; n < 5; ++n)
	{
		if (phyReg & st_mask)
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
		else
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
		st_mask >>= 1;
	}

	// Tristate and turnaround (2 bit times)
	stream[j++] = 0;
	stream[j++] = 0;

	// Write out 16 stream of data (msb first)
	st_mask = 0x8000;
	for (n = 0; n < 16; ++n)
	{
		if (phyData & st_mask)
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MDO;
		else
			stream[j++] = XLLP_LAN91C111_MGMT_MDOE;
		st_mask >>= 1;
	}

	// Final clock bit
	stream[j++] = 0;

	// Select bank 3
    XllpEthernetSelectRegisterBank (BANK3);

	// Get the current MII register value
	mii_reg = XllpEthernetReadWord16(XLLP_LAN91C111_MGMT);

	// Turn off all MII Interface bits
	mii_reg &= ~(XLLP_LAN91C111_MGMT_MDOE | XLLP_LAN91C111_MGMT_MCLK | 
	            XLLP_LAN91C111_MGMT_MDI | XLLP_LAN91C111_MGMT_MDO);

	// Clock all cycles
	for (n = 0; n < sizeof(stream); ++n)
	{
		// Clock Low - output data
		XllpEthernetWriteWord16(mii_reg | stream[n], XLLP_LAN91C111_MGMT);
        XllpOstDelayMicroSeconds(TimerRegBaseP, 50);

		// Clock Hi - input data
		XllpEthernetWriteWord16((XLLP_UINT16_T)(mii_reg | stream[n] | XLLP_LAN91C111_MGMT_MCLK), XLLP_LAN91C111_MGMT);
        XllpOstDelayMicroSeconds(TimerRegBaseP, 50);

		stream[n] |= XllpEthernetReadWord16(XLLP_LAN91C111_MGMT) & XLLP_LAN91C111_MGMT_MDI;
	}

	// Return to idle state
	// Set clock to low, data to low, and output tristated
	XllpEthernetWriteWord16(mii_reg, XLLP_LAN91C111_MGMT);
    XllpOstDelayMicroSeconds(TimerRegBaseP, 50);
}

/******************************************************************************

  Function Name: 
    XllpEthernetComputeCrc

  Description:
    This function computes crc on the buffer of the lenght, defined by the parameter
    passed to the function 

  Global Register Modified: 
    None 

  Input Arguments:
  P_XLLP_UINT8_T buffer - a bufferP 
  XLLP_UINT32_T length -

  Output Arguments:
    None
          
  Return Value: 
    XLLP_UINT32

*******************************************************************************/

static
XLLP_UINT32_T XllpEthernetComputeCrc (P_XLLP_UINT8_T bufferP, XLLP_UINT32_T length)
{
    XLLP_UINT32_T i, j, carry;
    XLLP_UINT8_T curByte;
    XLLP_UINT32_T crcValue = 0xffffffff;
    const XLLP_INT32_T poly = 0x04c11db6;

    for(i = 0; i < length; i++)
    {
        curByte = *bufferP++;
        for (j = 0; j < 8; j++)
        {
            carry = ((crcValue & 0x80000000) ? 1 : 0) ^ (curByte & 0x01);
            crcValue <<= 1;
            curByte >>= 1;

            if (carry)
            {
                crcValue = (crcValue ^ poly) | carry;
            }
        }
    }

    return crcValue;
}

/******************************************************************************

  Function Name: 
    XllpEthernetSetMulticastList

  Description: 
    This function sets the multicast table to filter out unwanted multicast packets.
    - The routine takes the pointer to the list of the destination address.
    - Computes the CRC on that list.
    - Sets the 64-bits multicast table:
        The high 6 bits of the CRC of the multicast address are the offset into the table.
        If that bit is one, then the multicast packet is accepted. Otherwise, it's dropped.
    - The high 6 bits of the CRC is an offset into the table:
        The high 3 bits are the number of the 8 bit register.
        The low 3 bits are the bit within that register.

  Global Register Modified: 
    None 

  Input Arguments:
    P_XLLP_UINT8_T multicastAddrP - a pointer to the list of the destination address.
    XLLP_UINT32_T addrCntr      - a number of the addresses in the list.  

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/

void XllpEthernetSetMulticastList (P_XLLP_UINT8_T multicastAddrP, XLLP_UINT32_T addrCntr)
{
    XLLP_UINT8_T    multicastRegTable[8];
    P_XLLP_UINT32_T   multicastRegTableP = (P_XLLP_UINT32_T)&multicastRegTable[0];
    XLLP_UINT32_T   i;
    XLLP_UINT32_T   computedCRC;

    // Dump the multicast list
    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Multicast List of %d Addresses:", addrCntr);
    for(i = 0 ; i < addrCntr; i++)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_0,
        //               " Multicast Address %d: %02x-%02x-%02x-%02x-%02x-%02x",
        //               i,
        //               multicastAddrP[6*i],
        //               multicastAddrP[6*i + 1],
        //               multicastAddrP[6*i + 2],
         //              multicastAddrP[6*i + 3],
        //               multicastAddrP[6*i + 4],
        //               multicastAddrP[6*i + 5]);
    }

    // Clear the 64-bit multicast table
    for(i =0; i< 8; i++)
    {
	    multicastRegTable[i] = 0x0;
    }
    // Turn on the bit for each address in the multicast list.
    // The hash value is defined as the six MSB of the CRC computed 
    // on the destination addr.
    // The three MSB determine the register to be used (MT0..MT7)
    // The other three determine the bit within the register.
    for(i = 0 ; i < addrCntr ; i++)
    {
        computedCRC = XllpEthernetComputeCrc (&multicastAddrP[6*i], 6);
        //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " i = %d CRC = %08X", i, computedCRC); 
        //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Bits 5:4:3  = %X",  (computedCRC >> 29) & 0x07); 
        //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " Bits 2:1:0  = %X",  (computedCRC >> 26) & 0x07); 

        multicastRegTable[computedCRC >> 29] |= (1 << ((computedCRC >> 26) & 0x07));
    }

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0,
    //               " Multicast Reg. Table: %02x-%02x-%02x-%02x-%02x-%02x-%02x-%02x",        
    //                 multicastRegTable[0],
    //                 multicastRegTable[1],
    //                 multicastRegTable[2],
    //                 multicastRegTable[3],
    //                 multicastRegTable[4],
    //                 multicastRegTable[5],
    //                 multicastRegTable[6],
    //                 multicastRegTable[7]);

    // Store the 64-bit image in the multicast table at the offsets MT0-MT7
    XllpEthernetSelectRegisterBank (BANK3);
    XllpEthernetWriteDoubleWord(*multicastRegTableP, XLLP_LAN91C111_MULTICAST0);
    XllpEthernetWriteDoubleWord(*(multicastRegTableP + 1), XLLP_LAN91C111_MULTICAST4);
}


/******************************************************************************

  Function Name: 
    XllpEthernetHWShutdown

  Description:
    This function is used to shutdown the Ethernet Controller..
    It disables all interrupts, turns receiver and transmitter off, and clears 
    all flags used by the tests.

  Global Register Modified: 
    None. 

  Input Arguments:
    None

  Output Arguments:
    None
          
  Return Value:
    None

*******************************************************************************/
void XllpEthernetHWShutdown(void)
{
    // Disable all interrupts
    XllpEthernetSelectRegisterBank (BANK2);
    XllpEthernetWriteByte(0, XLLP_LAN91C111_INT_MASK);
        
    // Disable the transmit 
    XllpEthernetSelectRegisterBank (BANK0);
    XllpEthernetWriteWord16(0, XLLP_LAN91C111_TCR);
    
    // Disable the receive
    XllpEthernetWriteWord16(0, XLLP_LAN91C111_RCR);

    // Clear all flags used by the tests and the driver
    XllpEthernetSetRxIntFlag(XLLP_FALSE);
    loggedError = 0;
    dumpFlag = 0; 
}

/******************************************************************************

  Function Name: 
    XllpEthernetHWSetup

  Description: 
    This function initilizes Ethernet Controller's hardware.
    It enables transmitter, receiver, interrupts, and autorelease of packets

  Global Register Modified: 
    None. 

  Input Arguments:
    None

  Output Arguments:
    P_XLLP_Uint16_T macAddressP - pointer to the user's MAC Address 
          
  Return Value: 
    XLLP_UINT32_T - error code if failed, zero if success

*******************************************************************************/
XLLP_UINT32_T XllpEthernetHWSetup(P_XLLP_UINT16_T macAddressP,
                                   XllpEthernetLoopbackEnableT loopback)
{
    XLLP_UINT16_T lanReg, revision, maskRegister;
    XLLP_UINT16_T chipMacAddr[3];

    // Verify the I/O address space by reading the bank select register. 
    // The bank select register at offset E should return 0x3301.
    XllpEthernetSelectRegisterBank (BANK1);
    lanReg = XllpEthernetReadWord16(XLLP_LAN91C111_BANK_SELECT);
    //It is used to determine the location of the LAN91C111/*DEBUG*/
    if ((lanReg & 0xFF00) != XLLP_LAN91C111_HIGH_SIGNATURE)
    {
        //DM_CwDbgPrintf(DM_CW_LAN91C111_1, " LAN91C111 not detected");
        LOGERROR(&loggedError, 
                 ERR_S_INV_DEV, ERR_T_NODEVICE, 
                 0);
        return (loggedError);
    }

    // Read the Configuration Register. This assumes Bank 1 is selected./*DEBUG*/
    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " LAN91C111 Configuration Register: %02X",
    //               (XLLP_UINT32_T)XllpEthernetReadWord(XLLP_LAN91C111_CONFIG));

    // Read the Base Address Register.
    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " LAN91C111 Base Address Register: %02X",
    //               (XLLP_UINT32_T)XllpEthernetReadWord(XLLP_LAN91C111_BASE));

    // Read the LAN91C111 revision. This assumes Bank 3 is selected./*DEBUG*/
    XllpEthernetSelectRegisterBank (BANK3);
    revision = XllpEthernetReadWord16(XLLP_LAN91C111_REVISION);

    //DM_CwDbgPrintf(DM_CW_LAN91C111_0, " LAN91C111 Revision: %d, Chip: %d",
    //               (revision & 0x0F),
    //               (revision & 0xF0) >> 4);

    // Give the LAN91C111 time to settle down.
    XllpOstDelayMicroSeconds(TimerRegBaseP, 750);

    // Get MAC address
    XllpEthernetGetMACAddress(&chipMacAddr[0]);
    // Check to see if we have a mac address...
    if ((chipMacAddr[0] == 0xFFFF) &&
        (chipMacAddr[1] == 0xFFFF) &&
        (chipMacAddr[2] == 0xFFFF))
    {
        // Apparently we don't. Were we passed in one?
        // Here we test to see if the mac address is FF:FF:FF:FF:FF:FF
        // or 00:00:00:00:00:00. If it is we use a hardcoded special
        // value. If not, use the passed in value.
        if (((macAddressP[0] == 0xFFFF) &&
             (macAddressP[1] == 0xFFFF) &&
             (macAddressP[2] == 0xFFFF)) ||
            ((macAddressP[0] == 0x0000) &&
             (macAddressP[1] == 0x0000) &&
             (macAddressP[2] == 0x0000)))
        {
            // No. Use a hardcoded valid sentinel value
            macAddressP[0] = 0x1200; // 00:12
            macAddressP[1] = 0x5634; // 34:56
            macAddressP[2] = 0x9A78; // 78:9A
            XllpEthernetStoreMACAddress(macAddressP);
        }
        else
        {
            // Yes. Use it.
            XllpEthernetStoreMACAddress(macAddressP);
        }
    }
    else
    {
        // Chip has a programmed MAC address. Report it.
        macAddressP[0] = chipMacAddr[0];
        macAddressP[1] = chipMacAddr[1];
        macAddressP[2] = chipMacAddr[2];
    }

    // Initialize the Control Register
    XllpEthernetSelectRegisterBank (BANK1);
  
    lanReg = XLLP_LAN91C111_CTR_RCV_BAD      |
                 XLLP_LAN91C111_CTR_AUTO_RELEASE |
                 XLLP_LAN91C111_CTR_TE_ENABLE;
   
    XllpEthernetWriteWord16(lanReg, XLLP_LAN91C111_CONTROL);

    // Enable internal PHY
    // Note: 
    //  SMSC came up with a datasheet errata:
    //  - Internal PHY address: 00000.
    //  - The internal PHY is placed in isolation mode at power up and reset.
    //    It can be removed from isolation mode by clearing the MII_DIS bit in 
    //    the PHY Control Register. 
    if (loopback == XLLP_ETHERNET_LOOPBACK_ENABLE)
    {
        XllpEthernetWritePhyRegister (0,0, XLLP_LAN91C111_CONTROL_LPBK);
    }
    else
    {
        XllpEthernetWritePhyRegister (0,0, XLLP_LAN91C111_CONTROL_MII_DIS);
    }

    // Initialize the Transmit Control Register - Set the Transmit enable and
    // Pad bit, which will pad transmit packets less than 64 bytes with zeroes.
    // This assumes Bank 0 is selected.
    XllpEthernetSelectRegisterBank (BANK0);

    
 
	XllpEthernetWriteWord16((XLLP_UINT16_T)(XLLP_LAN91C111_TCR_TXENA | XLLP_LAN91C111_TCR_PAD_EN | XLLP_LAN91C111_TCR_SWFDUP), 
							XLLP_LAN91C111_TCR);
     
    
    // Enable auto-negotiation of link speed
    XllpEthernetSelectRegisterBank (BANK0);
    maskRegister = 0x0800;
    // Set nLEDA to be 10/100 Link = 000b
    // nLEDB to be TX/RX = 100b
    maskRegister |= 0x0010; // LS2..0A = 000b, LS2..0B = 100
    XllpEthernetWriteWord16(maskRegister, XLLP_LAN91C111_RPCR);
  

    // Enable interrupts.
    XllpEthernetSelectRegisterBank (BANK2);
    maskRegister = XllpEthernetReadByte(XLLP_LAN91C111_INT_MASK);
    maskRegister |= XLLP_LAN91C111_MSK_TX_INT | XLLP_LAN91C111_MSK_TX_EMPTY_INT;
    // Enable interrupts.
    XllpEthernetWriteByte((XLLP_UINT8_T)maskRegister, XLLP_LAN91C111_INT_MASK);

    // The receive register should be the last thing initialized to avoid
    // receiving packets before we're ready. Now set the Receive Enable bit.
    XllpEthernetSelectRegisterBank (BANK0);
    XllpEthernetWriteWord16((XLLP_UINT16_T)XLLP_LAN91C111_RCR_RXEN, XLLP_LAN91C111_RCR);
    // Read the Interrupt Mask Register.
    XllpEthernetSelectRegisterBank (BANK2);
    maskRegister = XllpEthernetReadByte(XLLP_LAN91C111_INT_MASK);
    maskRegister |= XLLP_LAN91C111_MSK_RCV_INT | XLLP_LAN91C111_MSK_RX_OVRN_INT;
    // Enable Receive interrupts.
    XllpEthernetWriteByte((XLLP_UINT8_T)maskRegister, XLLP_LAN91C111_INT_MASK);

    return (0);
}

/******************************************************************************

  Function Name: 
    XllpEthernetSWInit

  Description: 
    This function initilizes Ethernet Controller's pointer to the registers
    base address.

  Global Register Modified: 
    None. 

  Input Arguments:
    void* ioRegsP - pointer to the Ethernet Controller memory space.

  Output Arguments:
    None
          
  Return Value: 
    None

*******************************************************************************/
void XllpEthernetSWInit(void* ioRegsP,  P_XLLP_OST_T OstBase)
{
    // Save the virtual address of the I/O Space.
    // Default for registers I/O space is 300h.
    LAN91C111IoP = (P_XLLP_VUINT16_T)((XLLP_UINT32_T)ioRegsP + XLLP_LAN91C111_REG_BASE_DEFAULT);
    TimerRegBaseP = (P_XLLP_OST_T)(OstBase) ;
}

void XllpEthernetEnableRxInterrupt(void)
{
	wIntMask = XLLP_LAN91C111_MSK_RCV_INT;
	XllpEthernetSelectRegisterBank (BANK2);
	XllpEthernetWriteByte(wIntMask, XLLP_LAN91C111_INT_MASK);
}

void XllpEthernetDisableRxInterrupt(void)
{
	XllpEthernetSelectRegisterBank (BANK2);
	XllpEthernetWriteByte(0, XLLP_LAN91C111_INT_MASK);
}

