
/*++
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
--*/
/**  FILENAME:  xllp_lcd_zl_q_776.c
**
**  PURPOSE:   platform-specific source file for XLLP LCD controller.
**				This file is specific for LTM035A776C (QVGA) on Zylonite. 
**
*******************************************************************************/

#include "xllp_lcd.h"
#include "xllp_lcd_cpu.h"
#include "xllp_mfp.h"
#include "xllp_gpio_proc.h"

// PCD values for LTM035A776C
// FIX ME: Update these values according real visual effect!
// Pixel clock of LTM035A776C should be 9.10 MHz
// In theory, PCD_HSS_104M = 104/9.10 - 1 = 10.43 
#define PCD_HSS_104M	10
#define PCD_HSS_156M	16
#define PCD_HSS_208M	20
#define PCD_MLCD_39M	4
  
static const XLLP_UINT32_T lcd_mfp_list []={
	XLLP_MFP_L_DD_8_OFFSET,    
	XLLP_MFP_L_DD_9_OFFSET,   
	XLLP_MFP_L_DD_10_OFFSET,   
	XLLP_MFP_L_DD_11_OFFSET,   
	XLLP_MFP_L_DD_12_OFFSET,       
	XLLP_MFP_L_DD_13_OFFSET,       
	XLLP_MFP_L_DD_14_OFFSET,       
	XLLP_MFP_L_DD_15_OFFSET,       
	XLLP_MFP_L_DD_16_OFFSET,       
	XLLP_MFP_L_DD_17_OFFSET,       
	XLLP_MFP_L_DD_0_OFFSET,   
	XLLP_MFP_L_DD_1_OFFSET,   
	XLLP_MFP_L_DD_2_OFFSET,     
	XLLP_MFP_L_DD_3_OFFSET,      
	XLLP_MFP_L_DD_4_OFFSET,      
	XLLP_MFP_L_DD_5_OFFSET,       
	XLLP_MFP_L_DD_6_OFFSET,       
	XLLP_MFP_L_DD_7_OFFSET,      
	XLLP_MFP_L_FCLK_OFFSET,       
	XLLP_MFP_L_LCLK_OFFSET,         
	XLLP_MFP_L_PCLK_OFFSET,       
	XLLP_MFP_L_BIAS_OFFSET,
	
	XLLP_MFP_BACKLIGHT_PWM_OFFSET,	
	       
	XLLP_MFP_PIN_EOLIST_MARKER 
};

static const XLLP_MFP_ALT_FN_T lcd_af_list []={
	XLLP_MFP_L_DD_8_AF, 
	XLLP_MFP_L_DD_9_AF,
	XLLP_MFP_L_DD_10_AF,
	XLLP_MFP_L_DD_11_AF,   
	XLLP_MFP_L_DD_12_AF,   
	XLLP_MFP_L_DD_13_AF,   
	XLLP_MFP_L_DD_14_AF,   
	XLLP_MFP_L_DD_15_AF,   
	XLLP_MFP_L_DD_16_AF,   
	XLLP_MFP_L_DD_17_AF,      
	XLLP_MFP_L_DD_0_AF,   
	XLLP_MFP_L_DD_1_AF,   
	XLLP_MFP_L_DD_2_AF,   
	XLLP_MFP_L_DD_3_AF,   
	XLLP_MFP_L_DD_4_AF,   
	XLLP_MFP_L_DD_5_AF,   
	XLLP_MFP_L_DD_6_AF,   
	XLLP_MFP_L_DD_7_AF,   
	XLLP_MFP_L_FCLK_AF,     
	XLLP_MFP_L_LCLK_AF,   
	XLLP_MFP_L_PCLK_AF,   
	XLLP_MFP_L_BIAS_AF,
	
	XLLP_MFP_PIN_GPIO14_AF_GPIO_14	
};

static const XLLP_MFP_ALT_FN_T mini_lcd_af_list []={
	XLLP_MFP_ALT_FN_7, 
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,      
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,     
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,   
	XLLP_MFP_ALT_FN_7,
	
	XLLP_MFP_PIN_GPIO14_AF_GPIO_14			
};

static const XLLP_MFP_DRIVE_STRENGTH_T lcd_ds_list []={
	XLLP_MFP_DS_08X,	//0
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,	
	XLLP_MFP_DS_08X,	//5
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,	
	XLLP_MFP_DS_08X,	//10
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,	
	XLLP_MFP_DS_08X,	//15
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,	
	XLLP_MFP_DS_08X,	//20
	XLLP_MFP_DS_08X,	//21
	XLLP_MFP_DS_08X,	//22	
};

static const XLLP_MFP_LPM_OUTPUT_T lcd_lpm_output_list []={
								XLLP_MFP_LPMO_PULL_LOW,       //  0
								XLLP_MFP_LPMO_PULL_LOW,       //  
								XLLP_MFP_LPMO_PULL_LOW,       //  
								XLLP_MFP_LPMO_PULL_LOW,       //  
								XLLP_MFP_LPMO_PULL_LOW,       //  
								XLLP_MFP_LPMO_PULL_LOW,       //  5
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //  10
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //  15
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //   
								XLLP_MFP_LPMO_PULL_LOW,       //  20
								XLLP_MFP_LPMO_PULL_LOW,       //  21 
								XLLP_MFP_LPMO_PULL_LOW        //  22 
};


void XllpLCDAdjustPCD(P_XLLP_LCD_T pXllpLCD, XLLP_UINT32_T hss)
{
	XLLP_UINT32_T pcd;
	XLLP_LCD_REGISTERS_T* pReg;
	
	if (hss == HSS_AUTO_DETECT) {
		hss = ((((XLLP_CLKMGR_T*)(pXllpLCD->CLKMan))->acsr)&0xc000)>>14;
	}
	
	switch (hss) {
		case HSS_104M:
			pcd = PCD_HSS_104M;
			break;
		case HSS_156M:
			pcd = PCD_HSS_156M;
			break;
		case HSS_208M:
			pcd = PCD_HSS_208M;
			break;						
		default:
			pcd = PCD_HSS_208M;
			break;
	}
	pReg = (XLLP_LCD_REGISTERS_T*)(pXllpLCD->LCDC);
	pReg->lccr4 |= XLLP_LCCR4_PIX_DIV_EQ;
	pReg->lccr3 = (pReg->lccr3&(~XLLP_LCCR3_PCD_MASK))|pcd;
	return;	  
}

XLLP_STATUS_T XllpLCDInit(P_XLLP_LCD_T pXllpLCD)
{
	XLLP_STATUS_T status = 0;
	
	pXllpLCD->FrameBufferWidth = 240;
	pXllpLCD->FrameBufferHeight = 320;

	status =  XllpMfpResourceManager_List(pXllpLCD->pMfpRmDb, 
										  (P_XLLP_UINT32_T)lcd_mfp_list, 
										  XLLP_MFP_RM_ID_XLLP_LCD, 
										  XLLP_SET);
	if ( XLLP_STATUS_SUCCESS != status) 
		goto err1;
    // Lock MFPR block until finished with all MFPRs.
    // Don't forget to unlock for early returns, too.
    //XllpLock(XLLP_RESOURCE_MFPR);
		
	// Initialize the MFP registers for proper LCD Controller operation
	status =  XllpMfpSetAfDs_List((P_XLLP_VUINT32_T)(pXllpLCD->MFP), 
								  (P_XLLP_UINT32_T)lcd_mfp_list, 
								  (P_XLLP_MFP_ALT_FN_T)lcd_af_list, 
								  (P_XLLP_MFP_DRIVE_STRENGTH_T)lcd_ds_list);
	if (status != XLLP_STATUS_SUCCESS)
		goto err1;
		
    status = XllpMfpConfigureLpmOutputLevel_List((P_XLLP_VUINT32_T)(pXllpLCD->MFP),
                								 (P_XLLP_UINT32_T)lcd_mfp_list,
                								 (P_XLLP_MFP_LPM_OUTPUT_T)lcd_lpm_output_list);
	if (status != XLLP_STATUS_SUCCESS)
		goto err1;

    status = XllpMfpActivatePullUpDown_List((P_XLLP_VUINT32_T)(pXllpLCD->MFP),
                							(P_XLLP_UINT32_T)lcd_mfp_list,
                							 XLLP_OFF);
	if (status != XLLP_STATUS_SUCCESS)
		goto err1;

	XllpGpioSetDirection((P_XLLP_GPIO_T)(pXllpLCD->GPIO), 14, XLLP_GPIO_DIRECTION_OUT);
    //XllpUnlock(XLLP_RESOURCE_MFPR);
	
	XllpLCDResume(pXllpLCD);
	return status;
	
err1:
	//XllpUnlock(XLLP_RESOURCE_MFPR);
	return status;	    
}

void XllpLCDSuspend(P_XLLP_LCD_T pXllpLCD, int SuspendType)
{
	// Turn off the backlight.
	// XllpGpioSetOutputLevel((P_XLLP_GPIO_T)(pXllpLCD->GPIO), 14, XLLP_LO);
	
	// Suspend the controller.
	LCDSuspendController(pXllpLCD, SuspendType);
}

void XllpLCDResume(P_XLLP_LCD_T pXllpLCD)
{
	XLLP_LCD_REGISTERS_T* p_LCDRegs;
	p_LCDRegs = (XLLP_LCD_REGISTERS_T*)(pXllpLCD->LCDC);
	
	// Initialize the LCD Controller and frame descriptors
	LCDInitController(pXllpLCD);
	
	// Panel-specific LCD controller initialization
	// LTM035A776C: 240x320 16bpp active matrix
	p_LCDRegs->lccr0 |= (XLLP_LCCR0_LDM | XLLP_LCCR0_SOFM0 | XLLP_LCCR0_IUM | XLLP_LCCR0_EOFM0 | 
						XLLP_LCCR0_PAS | XLLP_LCCR0_QDM | XLLP_LCCR0_BSM0  | XLLP_LCCR0_OUM |
						XLLP_LCCR0_RDSTM | XLLP_LCCR0_CMDIM | XLLP_LCCR0_OUC | XLLP_LCCR0_LDDALT);

	/*
	p_LCDRegs->lccr1 |= (XLLP_LCCR1_PPL(0xEF) | XLLP_LCCR1_HSW(0x04) | 
						XLLP_LCCR1_ELW(0x04) | XLLP_LCCR1_BLW(0x05) );
			
	p_LCDRegs->lccr2 |= (XLLP_LCCR2_LPP(0x13f) | XLLP_LCCR2_VSW(0x02) |
						XLLP_LCCR2_EFW(0x03)  | XLLP_LCCR2_BFW(0x02) );
	*/
	p_LCDRegs->lccr1 |= (XLLP_LCCR1_PPL(0xEF) | XLLP_LCCR1_HSW(0x04) | 

                                          XLLP_LCCR1_ELW(0x04) | XLLP_LCCR1_BLW(0x06) );

                     

       p_LCDRegs->lccr2 |= (XLLP_LCCR2_LPP(0x13f) | XLLP_LCCR2_VSW(0x02) |

                                          XLLP_LCCR2_EFW(0x03)  | XLLP_LCCR2_BFW(0x05) );



	p_LCDRegs->lccr3 |= ( XLLP_LCCR3_PCP | XLLP_LCCR3_HSP |
						XLLP_LCCR3_PDFOR(pXllpLCD->PixelDataFormat));
			
	p_LCDRegs->lccr4 |= XLLP_LCCR4_PAL_FOR(0);
	if ((p_LCDRegs->ovl1c1 & XLLP_OVL1C1_O1EN)||(p_LCDRegs->ovl2c1 & XLLP_OVL2C1_O2EN))
	{
		// Use 25-bit format RGB565 with T bit. 
		p_LCDRegs->lccr4 = XLLP_LCCR4_PAL_FOR(1);
	}	

	XllpLCDAdjustPCD(pXllpLCD, HSS_AUTO_DETECT);	
	
	// Clear LCD Controller status register
	LCDClearStatusReg(pXllpLCD);

	// Enable the LCD controller
	LCDEnableController(pXllpLCD);

	// If required, load the default palette into palette ram
	// and feed this to the LCD controller.
	if(pXllpLCD->BPP < BPP_16)
	{
		XllpLCDLoadPalette(pXllpLCD);
	}

	// Turn on the back light.
	XllpGpioSetDirection((P_XLLP_GPIO_T)(pXllpLCD->GPIO), 14, XLLP_GPIO_DIRECTION_OUT);
	XllpGpioSetOutputLevel((P_XLLP_GPIO_T)(pXllpLCD->GPIO), 14, XLLP_HI);
	
	return;
}

void XllpMiniLCDEnable(P_XLLP_MINI_LCD_T pXllpMiniLCD)
{
	XLLP_MINILCD_REGISTERS_T* p_MLCDRegs;
	XLLP_VUINT32_T *pMfpBacklight;

	p_MLCDRegs = (XLLP_MINILCD_REGISTERS_T*)(pXllpMiniLCD->MLCD);

	printk("p_MLCDRegs = %p\n", p_MLCDRegs);

	XllpClockEnable((XLLP_CLKMGR_T*)(pXllpMiniLCD->CLKMan),
			XLLP_CLK_MINI_LCD,
			XLLP_TRUE);
	
	XllpClockEnable((XLLP_CLKMGR_T*)(pXllpMiniLCD->CLKMan),
			XLLP_CLK_MINI_IM,
			XLLP_TRUE);

	p_MLCDRegs->mlccr0 =	XLLP_MLCCR0_PCP | XLLP_MLCCR0_HSP |
				XLLP_MLCCR0_PCD(PCD_MLCD_39M); 	
	
	p_MLCDRegs->mlccr1 =	XLLP_MLCCR1_PPL(0xEF) | XLLP_MLCCR1_HSW(0x04) | 
				XLLP_MLCCR1_ELW(0x04) | XLLP_MLCCR1_BLW(0x07);
	
	p_MLCDRegs->mlccr2 =	XLLP_MLCCR2_LPP(0x13f) | XLLP_MLCCR2_VSW(0x02) |
				XLLP_MLCCR2_EFW(0x03)  | XLLP_MLCCR2_BFW(0x05);

	p_MLCDRegs->mlsadd   = pXllpMiniLCD->SRAM_phys;
	p_MLCDRegs->mlfrmcnt = pXllpMiniLCD->FrameCount;
						
	// Initialize the MFP registers for proper LCD Controller operation
	XllpMfpSetAfDs_List(	(P_XLLP_VUINT32_T)(pXllpMiniLCD->MFP), 
				(P_XLLP_UINT32_T)lcd_mfp_list, 
				(P_XLLP_MFP_ALT_FN_T)mini_lcd_af_list, 
				(P_XLLP_MFP_DRIVE_STRENGTH_T)lcd_ds_list);

	pMfpBacklight = (XLLP_VUINT32_T *)(pXllpMiniLCD->MFP) +
			(XLLP_MFP_BACKLIGHT_PWM_OFFSET >> 2);

	if (pXllpMiniLCD->BackLight) {
		*pMfpBacklight &= ~0xffff0280;
		*pMfpBacklight |=  0x00000100;
	} else {
		*pMfpBacklight |=  0x00000080;
	}

	return;
}

void XllpMiniLCDDisable(P_XLLP_MINI_LCD_T pXllpMiniLCD)
{
	XllpClockEnable((XLLP_CLKMGR_T*)(pXllpMiniLCD->CLKMan),
			XLLP_CLK_MINI_LCD,
			XLLP_FALSE);

	XllpClockEnable((XLLP_CLKMGR_T*)(pXllpMiniLCD->CLKMan),
			XLLP_CLK_MINI_IM,
			XLLP_FALSE);

	// Initialize the MFP registers for proper LCD Controller operation
	XllpMfpSetAfDs_List(	(P_XLLP_VUINT32_T)(pXllpMiniLCD->MFP), 
				(P_XLLP_UINT32_T)lcd_mfp_list, 
				(P_XLLP_MFP_ALT_FN_T)lcd_af_list, 
				(P_XLLP_MFP_DRIVE_STRENGTH_T)lcd_ds_list);
	return;						
}

XLLP_STATUS_T XllpMiniLCDEncodeRGB565(
		P_XLLP_MINI_LCD_T pXllpMiniLCD, 
		XLLP_UINT32_T FrameSource, 
		P_XLLP_UINT32_T pByteCount)
{
	return MiniLCDEncodeRGB565(pXllpMiniLCD, FrameSource, pByteCount, 240, 320);
}
