/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_ssp_peri.h"

// In a 16-entry FIFO, 8 would probably be a good threshold.
// Actual value is one less than real threshold number.
#define kFifoThreshold	7	

static const XLLP_UINT32_T sspPortAddrPhys[] = { 
	0x41000010,
	0x41700010,
	0x41900010,
	0x41a00010
}; 

XLLP_STATUS_T XllpSSPPortConfigure (XLLP_SSP_CONFIG_T sspConfig,
			XLLP_SSP_PORT_SETUP_T *setup,
			XLLP_SSP_OP_STATE_T opState,
			XLLP_SSP_REGISTER_T *pReg)
{
	pReg->sscr1 = (kFifoThreshold << 6) | (kFifoThreshold << 10);

	switch(sspConfig) {
		case XLLP_SSP_CEDARCREEK_I2S:
			//FIXME: add specific register settings here.
			break;
		case XLLP_SSP_BT:
			//FIXME: add specific register settings here.
			break;
		case XLLP_SSP_USER_DEF:
			XllpSSPPortSetup(pReg, setup);
			break;
		
		default:
			return XLLP_STATUS_WRONG_PARAMETER;
	}
	
	switch(opState) {
		case XLLP_SSP_MASTER:
			pReg->sscr1 &= ~(XLLP_SSCR1_SCLKDIR|XLLP_SSCR1_SFRMDIR);
			break;
		case XLLP_SSP_SLAVE:
			pReg->sscr1 |= XLLP_SSCR1_SCLKDIR|XLLP_SSCR1_SFRMDIR;
			break;
		default:
			return XLLP_STATUS_WRONG_PARAMETER;
	}
				
	return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpSSPDMADescriptor (XLLP_UINT32_T portNo, 
			XLLP_SSP_CONFIG_T sspConfigure,
			XLLP_BOOL_T dataDirectionTx, XLLP_UINT32_T dataLength,
			XLLP_UINT32_T bufferPhy, XLLP_UINT32_T nextDescPhy, 
			XLLP_UINT32_T dmaInterruptMask,
			P_XLLP_DMAC_DESCRIPTOR_T pDescriptor)
{
	XLLP_UINT32_T dmaWidth = 0, dmaSize = 0;
	pDescriptor->ddadr = nextDescPhy;
	
	switch(sspConfigure) {
			// when sampleDataSize > 16 or fifoPacking
			//dma_width = (0x11)<<14; // 32-bit width
			//dma_burst = (0x11)<<16; // 32bytes can fill 8 entries.(kFifoThreshold+1)
			
			// when 16 >= sampleDataSize > 8
			//dma_width = (0x10)<<14; // 16-bit width
			//dma_burst = (0x10)<<16; // 16bytes can fill 8 entries.(kFifoThreshold+1)

			// when 8 >= sampleDataSize > 0 
			//dma_width = (0x01)<<14; // 8-bit width
			//dma_burst = (0x01)<<16; // 8bytes can fill 8 entries.(kFifoThreshold+1)
	
		case XLLP_SSP_CEDARCREEK_I2S:
			//FIXME: add specific settings here.
			// dmaWidth =
			// dmaSize =
			break;
		case XLLP_SSP_BT:
			//FIXME: add specific settings here.
			// dmaWidth =
			// dmaSize =
			break;
		default:
			return XLLP_STATUS_WRONG_PARAMETER;
	}
	
	if (dataDirectionTx == XLLP_TRUE) {
		pDescriptor->dsadr = bufferPhy;
		pDescriptor->dtadr = sspPortAddrPhys[portNo-1];
		pDescriptor->dcmd = XLLP_DMAC_DCMD_INC_SRC_ADDR
					| XLLP_DMAC_DCMD_FLOW_TRG
					| dmaWidth|dmaSize
					| dataLength;
	} else {
		pDescriptor->dsadr = sspPortAddrPhys[portNo-1];
		pDescriptor->dtadr = bufferPhy;
		pDescriptor->dcmd = XLLP_DMAC_DCMD_INC_TRG_ADDR
					| XLLP_DMAC_DCMD_FLOW_SRC
					| dmaWidth|dmaSize
					| dataLength;
	}
	pDescriptor->ddadr &= ~(XLLP_DMAC_DDADR_STOP);
	if (nextDescPhy == NULL)
		pDescriptor->ddadr |= XLLP_DMAC_DDADR_STOP;
	
	return XLLP_STATUS_SUCCESS;
}
