/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:       xllp_uart_plat.c
**
**  PURPOSE:        Definitions the low-level primitives that access the UART
**
******************************************************************************/
#if (BSTD)
#include "xllp_defs.h"
#include "xllp_serialization.h"
#include "xllp_mfp.h"
#include "xllp_gpio.h"
#include "xllp_uart_plat.h"

// MFP defination for FFUART 
#ifdef	XLLP_MFP_FFRXD_OFFSET
static XLLP_UINT32_T xllp_ffuart_mfp_list[] = {
	XLLP_MFP_FFRXD_OFFSET,
	XLLP_MFP_FFTXD_OFFSET,
	XLLP_MFP_FFCTS_OFFSET,
#ifndef CONFIG_CPU_MONAHANS_LV
	XLLP_MFP_FFDCD_OFFSET,
	XLLP_MFP_FFDSR_OFFSET,
	XLLP_MFP_FFRI_OFFSET,
	XLLP_MFP_FFDTR_OFFSET,
#endif	
	XLLP_MFP_FFRTS_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};

static XLLP_MFP_ALT_FN_T xllp_ffuart_af_list[] = {
	XLLP_MFP_FFRXD_AF,
	XLLP_MFP_FFTXD_AF,
	XLLP_MFP_FFCTS_AF,
#ifndef CONFIG_CPU_MONAHANS_LV	
	XLLP_MFP_FFDCD_AF,
	XLLP_MFP_FFDSR_AF,
	XLLP_MFP_FFRI_AF,
	XLLP_MFP_FFDTR_AF,
#endif	
	XLLP_MFP_FFRTS_AF
};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_ffuart_ds_list[] = {
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
#ifndef CONFIG_CPU_MONAHANS_LV	
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
#endif	
	XLLP_MFP_DEFAULT_DS
};

static XLLP_MFP_LPM_OUTPUT_T xllp_ffuart_lpm_output_list[] = {
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
#ifndef CONFIG_CPU_MONAHANS_LV	
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
#endif	
	XLLP_MFP_LPMO_FLOAT
};
#endif	//#ifdef	XLLP_MFP_FFRXD_OFFSET

// MFP definition for BTUART
// The BTUART definition maybe be changed in the future
#ifdef	XLLP_MFP_RSVD_BT_RXD_OFFSET
static XLLP_UINT32_T xllp_btuart_mfp_list[] = {
	XLLP_MFP_RSVD_BT_RTS_OFFSET,
	XLLP_MFP_RSVD_BT_RXD_OFFSET,
	XLLP_MFP_RSVD_BT_TXD_OFFSET,
	XLLP_MFP_RSVD_BT_CTS_OFFSET,	
	XLLP_MFP_PIN_EOLIST_MARKER
};

static XLLP_MFP_ALT_FN_T xllp_btuart_af_list[] = {
	XLLP_MFP_RSVD_BT_RTS_AF,
	XLLP_MFP_RSVD_BT_RXD_AF,
	XLLP_MFP_RSVD_BT_TXD_AF,
	XLLP_MFP_RSVD_BT_CTS_AF	
};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_btuart_ds_list[] = {
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS
};

static XLLP_MFP_LPM_OUTPUT_T xllp_btuart_lpm_output_list[] = {
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT
};
#endif	//#ifdef	XLLP_MFP_BT_RTS_OFFSET

// MFP definition for STUART
#ifdef	XLLP_MFP_STD_TXD_OFFSET
static XLLP_UINT32_T xllp_stuart_mfp_list[] = {
	XLLP_MFP_STD_TXD_OFFSET,
	XLLP_MFP_STD_RXD_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};

static XLLP_MFP_ALT_FN_T xllp_stuart_af_list[] = {
	XLLP_MFP_STD_TXD_AF,
	XLLP_MFP_STD_RXD_AF
};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_stuart_ds_list[] = {
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS
};

static XLLP_MFP_LPM_OUTPUT_T xllp_stuart_lpm_output_list[] = {
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT
};
#endif	//#ifdef	XLLP_MFP_STD_TXD_OFFSET

static XLLP_STATUS_T Config_FFUART(P_XLLP_VUINT32_T pMfpRegBase,
				P_XLLP_MFP_RM_DB_ID_T pMfpRmDb,
			    XLLP_MFP_RM_DB_ID_T MfpRmDBID)
{
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;

	/* If hasn't FFUART defined, return XLLP_STATUS_WRONG_PARAMETER */
#ifndef	XLLP_MFP_FFRXD_OFFSET
	return XLLP_STATUS_WRONG_PARAMETER;
#endif	

	if (NULL == pMfpRegBase || NULL == pMfpRmDb)
		return XLLP_STATUS_WRONG_PARAMETER;

	//Get Resource Manager
	status = XllpMfpResourceManager_List(pMfpRmDb,
					     &xllp_ffuart_mfp_list[0],
					     MfpRmDBID, XLLP_SET);
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	status = XllpMfpSetAfDs_List(pMfpRegBase,
				     &xllp_ffuart_mfp_list[0],
				     &xllp_ffuart_af_list[0],
				     &xllp_ffuart_ds_list[0]);
	if (XLLP_STATUS_SUCCESS != status) 
		return status;
	
	status = XllpMfpConfigureLpmOutputLevel_List(pMfpRegBase,
				&xllp_ffuart_mfp_list[0],
				&xllp_ffuart_lpm_output_list[0]);
	if (XLLP_STATUS_SUCCESS != status) 
		return status;

	return status;
}				// end of Config_FFUART()

static XLLP_STATUS_T Config_BTUART(P_XLLP_VUINT32_T pMfpRegBase,
				P_XLLP_MFP_RM_DB_ID_T pMfpRmDb,
			    XLLP_MFP_RM_DB_ID_T MfpRmDBID)
{
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;

	/* If hasn't BTUART defined, return XLLP_STATUS_WRONG_PARAMETER */
#ifndef	XLLP_MFP_RSVD_BT_RXD_OFFSET //XLLP_MFP_BT_RXD_OFFSET
	return XLLP_STATUS_WRONG_PARAMETER;
#endif	

	if (NULL == pMfpRegBase || NULL == pMfpRmDb)
		return XLLP_STATUS_WRONG_PARAMETER;

	status = XllpMfpResourceManager_List(pMfpRmDb,
					     &xllp_btuart_mfp_list[0],
					     MfpRmDBID, XLLP_SET);

	if (XLLP_STATUS_SUCCESS != status)
		return status;

	status = XllpMfpSetAfDs_List(pMfpRegBase,
				     &xllp_btuart_mfp_list[0],
				     &xllp_btuart_af_list[0],
				     &xllp_btuart_ds_list[0]);
	if (XLLP_STATUS_SUCCESS != status) 
		return status;

	status = XllpMfpConfigureLpmOutputLevel_List(pMfpRegBase,
				&xllp_btuart_mfp_list[0],
				&xllp_btuart_lpm_output_list[0]);
	if (XLLP_STATUS_SUCCESS != status) 
		return status;

	return status;
}				// end of Config_BTUART

static XLLP_STATUS_T Config_STUART(P_XLLP_VUINT32_T pMfpRegBase,
				P_XLLP_MFP_RM_DB_ID_T pMfpRmDb,
				    XLLP_MFP_RM_DB_ID_T MfpRmDBID)
{
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;

	/* If hasn't STUART defined, return XLLP_STATUS_WRONG_PARAMETER */
#ifndef	XLLP_MFP_STD_TXD_OFFSET
	return XLLP_STATUS_WRONG_PARAMETER;
#endif	

	if (NULL == pMfpRegBase || NULL == pMfpRmDb)
		return XLLP_STATUS_WRONG_PARAMETER;

	status = XllpMfpResourceManager_List(pMfpRmDb,
					     &xllp_stuart_mfp_list[0],
					     MfpRmDBID,XLLP_SET);
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	status = XllpMfpSetAfDs_List(pMfpRegBase,
				     &xllp_stuart_mfp_list[0],
				     &xllp_stuart_af_list[0],
				     &xllp_stuart_ds_list[0]);

	if (XLLP_STATUS_SUCCESS != status) 
		return status;

	status = XllpMfpConfigureLpmOutputLevel_List(pMfpRegBase,
				&xllp_stuart_mfp_list[0],
				&xllp_stuart_lpm_output_list[0]);
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	return status;
}				// end of Config_STUART

/******************************************************************************
 * 	XllpUartInit
 * 	This function initialze UART.
 * 	
 * 	Note: this function doesn't enable clock for UART.
 * 	
 * 	Param:
 * 		Handle: pointer to XLLP_UART_HANDLE_T
 * 	Return Value:
 * 		XLLP_STATUS_T	
 ******************************************************************************
 */
XLLP_STATUS_T XllpUartInit(P_XLLP_UART_HANDLE_T Handle)
{
	XLLP_STATUS_T status;	// get functions return value.

	// UART, MFP register, MFP resource DB ID.
	P_XLLP_VUINT32_T pMfpRegBase = Handle->pMfpRegBase;
	XLLP_MFP_RM_DB_ID_T MfpRmDBID = Handle->MfpRmDBID;
	P_XLLP_MFP_RM_DB_ID_T pMfpRmDb = Handle->pMfpRmDb;

	// Configure the GPIOs for FF or BT or STUART
	switch (Handle->UARTSelection) {
	case XLLP_FFUART:
		status = Config_FFUART(pMfpRegBase, pMfpRmDb, MfpRmDBID);
		if (XLLP_STATUS_SUCCESS != status)
			return status;
		break;
	case XLLP_BTUART:
		status = Config_BTUART(pMfpRegBase, pMfpRmDb, MfpRmDBID);
		if (XLLP_STATUS_SUCCESS != status)
			return status;
		break;
	case XLLP_STUART:
		status = Config_STUART(pMfpRegBase, pMfpRmDb, MfpRmDBID);
		if (XLLP_STATUS_SUCCESS != status)
			return status;
		break;
	}

	status = UartProcInit(Handle);

	return status;
}	// end of XllpUartInit()
#endif

