/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
 **  FILENAME:       xllp_usbohci.c
 **
 **  PURPOSE:        XLLP USB Host APIs - Platform specific
 **
 **  Valid for    :  zylonite Platforms
 **
 **
 ******************************************************************************/

#include "xllp_usbohci.h"
#include "xllp_usbohci_platform.h"

/*******************************************************************************
 *
 * FUNCTION:         XllpUsbhPortEnableOverCurrentNotification
 *
 * DESCRIPTION:      Configure platform specific gpios to be able to get over-current notification 
 *                   to the USB Host port. This feature is not supported on Zylonite.
 *                   
 *
 * INPUT PARAMETERS: P_XLLP_USBH_HANDLE_T pUsbhHandle
 *                   XLLP_USBH_PORT_T Port -     Port 1  (XLLP_USBH_PORT_1)
 *                                             or Port 2 (XLLP_USBH_PORT_2)
 *					   or Port 3 (XLLP_USBH_PORT_3)
 *
 * RETURNS:          
 *       Success:    0 (XLLP_USBH_SUCCESS)
 *       Failure:    XLLP_USBH_FEATURE_NOT_SUPPORTED for XLLP_USBH_PORT_2 and XLLP_USBH_PORT_3
 *
 * CALLS:            XllpGpioSetDirectionIn, XllpGpioSetAlternateFn
 *
 * CALLED BY:        XllpUsbhPowerUp
 *
 * PROTOTYPE:        XLLP_USBH_ERROR_T XllpUsbhPortEnableOverCurrentNotification(
 *                                                               P_XLLP_USBH_HANDLE_T pUsbhHandle,
 *																 XLLP_USBH_PORT_T Port) ;
 *
 *
 *******************************************************************************/


XLLP_USBH_ERROR_T XllpUsbhPortEnableOverCurrentNotification(P_XLLP_USBH_HANDLE_T pUsbhHandle,XLLP_USBH_PORT_T Port)
{
	if (Port == XLLP_USBH_PORT_1) {
		/* Configure GPIO3_2 as USBH_PWR */
		XllpMfpSetAfDs(pUsbhHandle->pMfpRegs, XLLP_MFP_USBHPWR_OFFSET, XLLP_MFP_USBHPWR_AF, XLLP_MFP_DEFAULT_DS);
		return XLLP_USBH_SUCCESS;
	} else
		return (XLLP_USBH_FEATURE_NOT_SUPPORTED);
}



/*******************************************************************************
 *
 * FUNCTION:         XllpUsbhPortEnablePower
 *
 * DESCRIPTION:      Configure platform specific gpios to enable/disable power to the devices connected 
 *                   to USB Host port. This is not available on Zylonite.
 *                   
 *
 * INPUT PARAMETERS: P_XLLP_USBH_HANDLE_T pUsbhHandle
 *                   XLLP_USBH_PORT_T Port -     Port 1  (XLLP_USBH_PORT_1)
 *                                             or Port 2 (XLLP_USBH_PORT_2)
 *					   or Port 3 (XLLP_USBH_PORT_3)
 *         
 * RETURNS:          
 *       Success:    0 (XLLP_USBH_SUCCESS)
 *       Failure:    XLLP_USBH_FEATURE_NOT_SUPPORTED for XLLP_USBH_PORT_2 and XLLP_USBH_PORT_3
 *
 * CALLS:            XllpGpioSetOutput0, XllpGpioSetDirectionOut, XllpGpioSetAlternateFn   
 *
 * CALLED BY:        XllpUsbhPowerUp
 *
 * PROTOTYPE:        XLLP_USBH_ERROR_T XllpUsbhPortEnablePower(P_XLLP_USBH_HANDLE_T pUsbhHandle, 
 *                                                           XLLP_USBH_PORT_T  Port);  
 *
 *
 *******************************************************************************/


XLLP_USBH_ERROR_T XllpUsbhPortEnablePower(P_XLLP_USBH_HANDLE_T pUsbhHandle, XLLP_USBH_PORT_T  Port)
{
	if (Port == XLLP_USBH_PORT_1) {
		/* Configure GPIO2_2 as USBH_PEN*/
		XllpMfpSetAfDs(pUsbhHandle->pMfpRegs, XLLP_MFP_USBHPEN_OFFSET, XLLP_MFP_USBHPEN_AF, XLLP_MFP_DEFAULT_DS);
		return XLLP_USBH_SUCCESS;
	}
		return (XLLP_USBH_FEATURE_NOT_SUPPORTED);
}



/*******************************************************************************
 *
 * FUNCTION:         XllpUsbhInitPlatformSpecificConfig
 *
 * DESCRIPTION:      Initializes XLLP_PLATFORM_USBH_T struct according to
 *                   platform specific details to enable platform specific 
 *                   USB Host reset.
 *                   
 *
 * INPUT PARAMETERS: P_XLLP_USBH_HANDLE_T pUsbhHandle
 *          
 *
 * RETURNS:          
 *       Success:    0 (XLLP_USBH_SUCCESS)
 *       Failure:    
 *
 * CALLS:            None.
 *
 * CALLED BY:        HCD
 *
 * PROTOTYPE:        XLLP_USBH_ERROR_T XllpUsbhInitPlatformSpecificConfig(P_XLLP_USBH_HANDLE_T pUsbhHandle); 
 *
 *
 *******************************************************************************/


XLLP_USBH_ERROR_T XllpUsbhInitPlatformSpecificConfig(P_XLLP_USBH_HANDLE_T pUsbhHandle)
{
	//These settings are specific for Zylonite platform
	//And, will change from platform to platform
	// Setting power sense polarity to active low
	pUsbhHandle->pPlatformUSBHConfig->usbhpwr_level = XLLP_LO;
	// Setting power control polarity to active low
	pUsbhHandle->pPlatformUSBHConfig->usbhpen_level = XLLP_LO;
	// Enable power to the USB single-ended receivers and the USB port 1 power supply 
	pUsbhHandle->pPlatformUSBHConfig->port1_ssep_enable = XLLP_TRUE;
	// Enable power to the USB single-ended receivers and the USB port 2 power supply 
	pUsbhHandle->pPlatformUSBHConfig->port2_ssep_enable = XLLP_TRUE;
	// Disable power to the USB single-ended receivers and the USB port 3 power supply 
	pUsbhHandle->pPlatformUSBHConfig->port3_ssep_enable = XLLP_FALSE;
	// Allow above settings to take effect
	pUsbhHandle->pPlatformUSBHConfig->global_sse_enable = XLLP_TRUE;
   
	return (XLLP_USBH_SUCCESS);
}


/*******************************************************************************
 *
 * FUNCTION:         XllpUsbhConfigPlatformSpecificUSBHReset
 *
 * DESCRIPTION:      Peroforms platform specific USBH Reset according to
 *                   XLLP_PLATFORM_USBH_T struct initialized by 
 *                   XllpUsbhInitPlatformSpecificConfig
 *                   
 *
 * INPUT PARAMETERS: P_XLLP_USBH_HANDLE_T pUsbhHandle
 *          
 *
 * RETURNS:          
 *       Success:    0 (XLLP_USBH_SUCCESS)
 *       Failure:    
 *
 * CALLS:                
 *
 * CALLED BY:        HCD
 *
 * PROTOTYPE:        XLLP_USBH_ERROR_T XllpUsbhConfigPlatformSpecificUSBHReset(P_XLLP_USBH_HANDLE_T pUsbhHandle);
 *
 *
 *******************************************************************************/


XLLP_USBH_ERROR_T XllpUsbhConfigPlatformSpecificUSBHReset(P_XLLP_USBH_HANDLE_T pUsbhHandle)
{
	XLLP_LEVEL_T  usbhpwr_level = pUsbhHandle->pPlatformUSBHConfig->usbhpwr_level;
	XLLP_LEVEL_T  usbhpen_level = pUsbhHandle->pPlatformUSBHConfig->usbhpen_level;
	XLLP_BOOL_T   port1_ssep_enable = pUsbhHandle->pPlatformUSBHConfig->port1_ssep_enable;
	XLLP_BOOL_T   port2_ssep_enable = pUsbhHandle->pPlatformUSBHConfig->port2_ssep_enable;
	XLLP_BOOL_T   port3_ssep_enable = pUsbhHandle->pPlatformUSBHConfig->port3_ssep_enable;
	XLLP_BOOL_T   global_sse_enable = pUsbhHandle->pPlatformUSBHConfig->global_sse_enable;

	//Configure platform specific power sense polarity
	XllpUsbhConfigurePowerSensePolarity(pUsbhHandle, usbhpwr_level) ;

	//Configure platform specific power control polarity
	XllpUsbhConfigurePowerControlPolarity(pUsbhHandle, usbhpen_level) ;

	//Enable/disable power to the USB single-ended receivers and the USB port 1 power supply
	XllpUsbhPortEnableSleepStandbyEnable(pUsbhHandle, XLLP_USBH_PORT_1, port1_ssep_enable);

	//Enable/disable power to the USB single-ended receivers and the USB port 2 power supply
	XllpUsbhPortEnableSleepStandbyEnable(pUsbhHandle, XLLP_USBH_PORT_2, port2_ssep_enable);

	//Enable/disable power to the USB single-ended receivers and the USB port 3 power supply
	XllpUsbhPortEnableSleepStandbyEnable(pUsbhHandle, XLLP_USBH_PORT_3, port3_ssep_enable);

	//For all downstream ports, enable/disable power to the USB single-ended receivers and 
	//the USB ports power supply
	XllpUsbhConfigureGlobalSleepStandbyEnable(pUsbhHandle, global_sse_enable);

	return (XLLP_USBH_SUCCESS);
}

