/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the BSD License, the text of
which follows:

Copyright (c) 1996-2005, Intel Corporation. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

Neither the name of the Intel Corporation ("Intel") nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/
/******************************************************************************
**
**  FILENAME:       xllp_mfp_plat.h
**
**  PURPOSE: Contains all hardware platform-specific MFP (MultiFunction Pin)
**              macros, typedefs, and prototypes.  Processor-level MFP
**              definitions are found in xllp_mfp_proc.h
**           Declares no storage.
**
**  Validity Notes: Only valid for the processor code named tavorP and the
**                  Zylonite platform.
**
**  Naming convention note: the symbols are, so far as possible, based on the
**                   processor documentation because that will be constant
**                   across multiple platforms.
**               '<' and '>' in processor documentation symbols are represented
**               as '_' in these symbolic constants.
**
**  Ordering note: These symbols are ordered by this logic:
**                 MFP symbol for GPIOn, then GPIOn_2, in numeric order
**                 Then, alphabetic order of MFP symbols.
**
******************************************************************************/

#ifndef __XLLP_MFP_PLAT_B0_H__
#define __XLLP_MFP_PLAT_B0_H__
#include "xllp_defs.h"

//
// Note: For the first instance of the Zylonite, exact usage of many pins is
//       not fully defined, or the headers to which they lead are not expected
//       to be populated.  For those pins' symbols, the string "RSVD" is
//       placed after "_MFP_".  They should not be used by any module but only
//       set to an appropriate permanently initialized value by a single
//       system initialization function.
//
// Note: For each "RSVD" pin on the target platform, an unassigned Alternate
//       Function code (based on internal silicon documents) has been selected
//       so that there are no side effects and minimal power effects from
//       configuring the pin with that AF code.  It is, effectively, a null
//       AF.  This permits simpler power-sensitive test board SW than would be required if
//       GPIO were to be selected as the AF device for unattached pins.

 // Pick an unused AF on this pin for its NULL AF

//
//////////////////////////////////////////////////////////////////////
//
// Beginning of Platform MFP Values and Alternate Function for Each Pin
//

//
// Concept note:
//      If the XLLP modules and OS drivers are considered to be "Software ICs"
//      then the following set of constants can be considered as the
//      "Software PCB layer" in that it establishes the connection between the
//      processor pins and the target peripherals.
//

#define XLLP_MFP_KP_MKIN_0_OFFSET       (XLLP_MFP_PIN_GPIO0_OFFSET)
#define XLLP_MFP_KP_MKIN_0_AF           (XLLP_MFP_PIN_GPIO0_AF_KP_MKIN0)

#define XLLP_MFP_KP_MKOUT_0_OFFSET      (XLLP_MFP_PIN_GPIO1_OFFSET)
#define XLLP_MFP_KP_MKOUT_0_AF          (XLLP_MFP_PIN_GPIO1_AF_KP_MKOUT0)

#define XLLP_MFP_KP_MKIN_1_OFFSET       (XLLP_MFP_PIN_GPIO2_OFFSET)
#define XLLP_MFP_KP_MKIN_1_AF           (XLLP_MFP_PIN_GPIO2_AF_KP_MKIN1)

#define XLLP_MFP_KP_MKOUT_1_OFFSET      (XLLP_MFP_PIN_GPIO3_OFFSET)
#define XLLP_MFP_KP_MKOUT_1_AF          (XLLP_MFP_PIN_GPIO3_AF_KP_MKOUT1)

#define XLLP_MFP_KP_MKIN_2_OFFSET       (XLLP_MFP_PIN_GPIO4_OFFSET)
#define XLLP_MFP_KP_MKIN_2_AF           (XLLP_MFP_PIN_GPIO4_AF_KP_MKIN2)

#define XLLP_MFP_KP_MKOUT_2_OFFSET      (XLLP_MFP_PIN_GPIO5_OFFSET)
#define XLLP_MFP_KP_MKOUT_2_AF          (XLLP_MFP_PIN_GPIO5_AF_KP_MKOUT2)

#define XLLP_MFP_KP_MKIN_3_OFFSET       (XLLP_MFP_PIN_GPIO6_OFFSET)
#define XLLP_MFP_KP_MKIN_3_AF           (XLLP_MFP_PIN_GPIO6_AF_KP_MKIN3)

#define XLLP_MFP_KP_MKOUT_3_OFFSET      (XLLP_MFP_PIN_GPIO7_OFFSET)
#define XLLP_MFP_KP_MKOUT_3_AF          (XLLP_MFP_PIN_GPIO7_AF_KP_MKOUT3)

#define XLLP_MFP_KP_MKIN_4_OFFSET       (XLLP_MFP_PIN_GPIO8_OFFSET)
#define XLLP_MFP_KP_MKIN_4_AF           (XLLP_MFP_PIN_GPIO8_AF_KP_MKIN4)

#define XLLP_MFP_KP_MKOUT_4_OFFSET      (XLLP_MFP_PIN_GPIO9_OFFSET)
#define XLLP_MFP_KP_MKOUT_4_AF          (XLLP_MFP_PIN_GPIO9_AF_KP_MKOUT4)

#if 1
#define XLLP_MFP_KP_MKIN_5_OFFSET       (XLLP_MFP_PIN_GPIO10_OFFSET)
#define XLLP_MFP_KP_MKIN_5_AF           (XLLP_MFP_PIN_GPIO10_AF_KP_MKIN5)

#define XLLP_MFP_KP_MKOUT_5_OFFSET      (XLLP_MFP_PIN_GPIO11_OFFSET)
#define XLLP_MFP_KP_MKOUT_5_AF          (XLLP_MFP_PIN_GPIO11_AF_KP_MKOUT5)

#define XLLP_MFP_KP_MKIN_6_OFFSET       (XLLP_MFP_PIN_GPIO12_OFFSET)
#define XLLP_MFP_KP_MKIN_6_AF           (XLLP_MFP_PIN_GPIO12_AF_KP_MKIN6)

#define XLLP_MFP_KP_MKOUT_6_OFFSET      (XLLP_MFP_PIN_GPIO13_OFFSET)
#define XLLP_MFP_KP_MKOUT_6_AF          (XLLP_MFP_PIN_GPIO13_AF_KP_MKOUT6)

#define XLLP_MFP_KP_DKIN_2_OFFSET		(XLLP_MFP_PIN_GPIO14_OFFSET)
#define XLLP_MFP_KP_DKIN_2_AF			(XLLP_MFP_PIN_GPIO14_AF_KP_DKIN2)

#define XLLP_MFP_KP_DKIN_3_OFFSET		(XLLP_MFP_PIN_GPIO15_OFFSET)
#define XLLP_MFP_KP_DKIN_3_AF			(XLLP_MFP_PIN_GPIO15_AF_KP_DKIN3)
#endif //0


// FIXME - camera
#define XLLP_MFP_CIF_HI_PWDN_GPI0_OFFSET    (XLLP_MFP_PIN_GPIO63_OFFSET) //Fixed Ran
#define XLLP_MFP_CIF_HI_PWDN_GPI0_AF        (XLLP_MFP_PIN_GPIO63_AF_GPIO_63)  //Fixed Ran

#define XLLP_MFP_CIF_LO_PWDN_GPI0_OFFSET    (XLLP_MFP_PIN_GPIO64_OFFSET) //Fixed Ran
#define XLLP_MFP_CIF_LO_PWDN_GPI0_AF        (XLLP_MFP_PIN_GPIO64_AF_GPIO_64) //Fixed Ran

#define XLLP_MFP_CIF_DD_7_OFFSET        (XLLP_MFP_PIN_GPIO65_OFFSET) //Fixed Ran
#define XLLP_MFP_CIF_DD_7_AF            (XLLP_MFP_PIN_GPIO65_AF_CAM_IN7) //Fixed Ran

#define XLLP_MFP_CIF_DD_6_OFFSET        (XLLP_MFP_PIN_GPIO66_OFFSET) //Fixed Ran
#define XLLP_MFP_CIF_DD_6_AF            (XLLP_MFP_PIN_GPIO66_AF_CAM_IN6) //Fixed Ran

#define XLLP_MFP_CIF_DD_5_OFFSET        (XLLP_MFP_PIN_GPIO67_OFFSET)//Fixed Ran
#define XLLP_MFP_CIF_DD_5_AF            (XLLP_MFP_PIN_GPIO67_AF_CAM_IN5)//Fixed Ran

#define XLLP_MFP_CIF_DD_4_OFFSET        (XLLP_MFP_PIN_GPIO68_OFFSET)//Fixed Ran
#define XLLP_MFP_CIF_DD_4_AF            (XLLP_MFP_PIN_GPIO68_AF_CAM_IN4)//Fixed Ran

#define XLLP_MFP_CIF_DD_3_OFFSET        (XLLP_MFP_PIN_GPIO69_OFFSET)//Fixed Ran
#define XLLP_MFP_CIF_DD_3_AF            (XLLP_MFP_PIN_GPIO69_AF_CAM_IN3)//Fixed Ran

#define XLLP_MFP_CIF_DD_2_OFFSET        (XLLP_MFP_PIN_GPIO70_OFFSET)//Fixed Ran
#define XLLP_MFP_CIF_DD_2_AF            (XLLP_MFP_PIN_GPIO70_AF_CAM_IN2)//Fixed Ran

#define XLLP_MFP_CIF_DD_1_OFFSET        (XLLP_MFP_PIN_GPIO71_OFFSET)//Fixed Ran
#define XLLP_MFP_CIF_DD_1_AF            (XLLP_MFP_PIN_GPIO71_AF_CAM_IN1)//Fixed Ran

#define XLLP_MFP_CIF_DD_0_OFFSET        (XLLP_MFP_PIN_GPIO72_OFFSET)//Fixed Ran
#define XLLP_MFP_CIF_DD_0_AF            (XLLP_MFP_PIN_GPIO72_AF_CAM_IN0)//Fixed Ran

#define XLLP_MFP_CIF_HSYNC_OFFSET       (XLLP_MFP_PIN_GPIO73_OFFSET) // Fixed Ran
#define XLLP_MFP_CIF_HSYNC_AF           (XLLP_MFP_PIN_GPIO73_AF_CAM_HSYNC) // Fixed Ran

#define XLLP_MFP_CIF_VSYNC_OFFSET       (XLLP_MFP_PIN_GPIO74_OFFSET) // Fixed Ran
#define XLLP_MFP_CIF_VSYNC_AF           (XLLP_MFP_PIN_GPIO74_AF_CAM_VSYNC) // Fixed Ran

#define XLLP_MFP_CIF_MCLK_OFFSET        (XLLP_MFP_PIN_GPIO75_OFFSET)// Fixed Ran
#define XLLP_MFP_CIF_MCLK_AF            (XLLP_MFP_PIN_GPIO75_AF_CAM_MCLK) // Fixed Ran


#define XLLP_MFP_CIF_PCLK_OFFSET        (XLLP_MFP_PIN_GPIO76_OFFSET) // Fixed Ran
#define XLLP_MFP_CIF_PCLK_AF            (XLLP_MFP_PIN_GPIO76_AF_CAM_PCLK) //Fixed Ran

// FIXME - MN_CLK is not routed to same pin.
// Intel - Not used.
//#define XLLP_MFP_MN_CLK_OFFSET             (XLLP_MFP_PIN_GPIO50_OFFSET)
//#define XLLP_MFP_MN_CLK_AF                 (XLLP_MFP_PIN_GPIO50_AF_MN_CLK)



#define XLLP_MFP_GPIO_28_OFFSET             (XLLP_MFP_PIN_GPIO28_OFFSET)
#define XLLP_MFP_GPIO_28_AF                 (XLLP_MFP_PIN_GPIO28_AF_GPIO_28)

#define XLLP_MFP_GPIO_29_OFFSET             (XLLP_MFP_PIN_GPIO29_OFFSET)
#define XLLP_MFP_GPIO_29_AF                 (XLLP_MFP_PIN_GPIO29_AF_GPIO_29)

#define XLLP_MFP_L_FCLK_OFFSET          (XLLP_MFP_PIN_GPIO17_OFFSET)
#define XLLP_MFP_L_FCLK_AF              (XLLP_MFP_PIN_GPIO17_AF_LCD_FCLK_RD)

#define XLLP_MFP_L_LCLK_OFFSET          (XLLP_MFP_PIN_GPIO18_OFFSET)
#define XLLP_MFP_L_LCLK_AF              (XLLP_MFP_PIN_GPIO18_AF_LCD_LCLK_A0)

#define XLLP_MFP_L_PCLK_OFFSET        (XLLP_MFP_PIN_GPIO19_OFFSET)
#define XLLP_MFP_L_PCLK_AF            (XLLP_MFP_PIN_GPIO19_AF_LCD_PCLK_WR)

#define XLLP_MFP_L_CS_1_OFFSET          (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_L_CS_1_AF              (XLLP_MFP_PIN_GPIOXX_AF_LCD_CS2)

#define XLLP_MFP_L_CS_0_OFFSET          (XLLP_MFP_PIN_GPIO21_OFFSET)
#define XLLP_MFP_L_CS_0_AF              (XLLP_MFP_PIN_GPIO21_AF_LCD_CS)

#define XLLP_MFP_L_CS_OFFSET            (XLLP_MFP_PIN_GPIO21_OFFSET)
#define XLLP_MFP_L_CS_AF                (XLLP_MFP_PIN_GPIO21_AF_LCD_CS)

#define XLLP_MFP_L_VSYNC_OFFSET      (XLLP_MFP_PIN_GPIO22_OFFSET)
#define XLLP_MFP_L_VSYNC_AF          (XLLP_MFP_PIN_GPIO22_AF_LCD_VSYNC)
#define XLLP_MFP_L_VSYNC_LPM       (XLLP_MFP_LPMO_FLOAT)


// FIXME - not sure where reset are. Assuming same.
#define XLLP_MFP_L_RESET_0_OFFSET          (XLLP_MFP_PIN_GPIO16_OFFSET) //Fixed Ran
#define XLLP_MFP_L_RESET_0_AF              (XLLP_MFP_PIN_GPIO16_AF_GPIO_16) //Fixed Ran

#define XLLP_MFP_L_DD_0_OFFSET        (XLLP_MFP_PIN_GPIO23_OFFSET)
#define XLLP_MFP_L_DD_0_AF            (XLLP_MFP_PIN_GPIO23_AF_LCD_DD0)

#define XLLP_MFP_L_DD_1_OFFSET        (XLLP_MFP_PIN_GPIO24_OFFSET)
#define XLLP_MFP_L_DD_1_AF            (XLLP_MFP_PIN_GPIO24_AF_LCD_DD1)

#define XLLP_MFP_L_DD_2_OFFSET        (XLLP_MFP_PIN_GPIO25_OFFSET)
#define XLLP_MFP_L_DD_2_AF            (XLLP_MFP_PIN_GPIO25_AF_LCD_DD2)

#define XLLP_MFP_L_DD_3_OFFSET        (XLLP_MFP_PIN_GPIO26_OFFSET)
#define XLLP_MFP_L_DD_3_AF            (XLLP_MFP_PIN_GPIO26_AF_LCD_DD3)

#define XLLP_MFP_L_DD_4_OFFSET        (XLLP_MFP_PIN_GPIO27_OFFSET)
#define XLLP_MFP_L_DD_4_AF            (XLLP_MFP_PIN_GPIO27_AF_LCD_DD4)

#define XLLP_MFP_L_DD_5_OFFSET        (XLLP_MFP_PIN_GPIO28_OFFSET)
#define XLLP_MFP_L_DD_5_AF            (XLLP_MFP_PIN_GPIO28_AF_LCD_DD5)

#define XLLP_MFP_L_DD_6_OFFSET        (XLLP_MFP_PIN_GPIO29_OFFSET)
#define XLLP_MFP_L_DD_6_AF            (XLLP_MFP_PIN_GPIO29_AF_LCD_DD6)

#define XLLP_MFP_L_DD_7_OFFSET        (XLLP_MFP_PIN_GPIO44_OFFSET)
#define XLLP_MFP_L_DD_7_AF            (XLLP_MFP_PIN_GPIO44_AF_LCD_DD7)

#if 0
#define XLLP_MFP_L_DD_8_OFFSET          (XLLP_MFP_PIN_GPIO31_OFFSET)
#define XLLP_MFP_L_DD_8_AF              (XLLP_MFP_PIN_GPIO31_AF_LCD_DD8)
#define XLLP_MFP_L_DD_8_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_9_OFFSET          (XLLP_MFP_PIN_GPIO32_OFFSET)
#define XLLP_MFP_L_DD_9_AF              (XLLP_MFP_PIN_GPIO32_AF_LCD_DD9)
#define XLLP_MFP_L_DD_9_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_10_OFFSET         (XLLP_MFP_PIN_GPIO33_OFFSET)
#define XLLP_MFP_L_DD_10_AF             (XLLP_MFP_PIN_GPIO33_AF_LCD_DD10)
#define XLLP_MFP_L_DD_10_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_11_OFFSET         (XLLP_MFP_PIN_GPIO34_OFFSET)
#define XLLP_MFP_L_DD_11_AF             (XLLP_MFP_PIN_GPIO34_AF_LCD_DD11)
#define XLLP_MFP_L_DD_11_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_12_OFFSET         (XLLP_MFP_PIN_GPIO35_OFFSET)
#define XLLP_MFP_L_DD_12_AF             (XLLP_MFP_PIN_GPIO35_AF_LCD_DD12)
#define XLLP_MFP_L_DD_12_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_13_OFFSET         (XLLP_MFP_PIN_GPIO36_OFFSET)
#define XLLP_MFP_L_DD_13_AF             (XLLP_MFP_PIN_GPIO36_AF_LCD_DD13)
#define XLLP_MFP_L_DD_13_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_14_OFFSET         (XLLP_MFP_PIN_GPIO37_OFFSET)
#define XLLP_MFP_L_DD_14_AF             (XLLP_MFP_PIN_GPIO37_AF_LCD_DD14)
#define XLLP_MFP_L_DD_14_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_15_OFFSET         (XLLP_MFP_PIN_GPIO38_OFFSET)
#define XLLP_MFP_L_DD_15_AF             (XLLP_MFP_PIN_GPIO38_AF_LCD_DD15)
#define XLLP_MFP_L_DD_15_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_16_OFFSET         0xFF
#define XLLP_MFP_L_DD_16_AF             (XLLP_MFP_PIN_GPIO39_AF_LCD_DD16)
#define XLLP_MFP_L_DD_16_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_17_OFFSET         0xFF
#define XLLP_MFP_L_DD_17_AF             (XLLP_MFP_PIN_GPIO40_AF_LCD_DD17)
#define XLLP_MFP_L_DD_17_LPM         (XLLP_MFP_LPMO_PULL_LOW)
#endif //0

#define XLLP_MFP_GPIO_44_OFFSET        (XLLP_MFP_PIN_GPIO44_OFFSET)
#define XLLP_MFP_GPIO_44_AF            (XLLP_MFP_PIN_GPIO44_AF_GPIO_44)

//Track Ball
#define XLLP_MFP_GPIO_47_TB_XP_OFFSET       (XLLP_MFP_PIN_GPIO47_OFFSET)
#define XLLP_MFP_GPIO_47_TB_XP_AF           (XLLP_MFP_PIN_GPIO47_AF_GPIO_47)

#define XLLP_MFP_GPIO_48_TB_XM_OFFSET       (XLLP_MFP_PIN_GPIO48_OFFSET)
#define XLLP_MFP_GPIO_48_TB_XM_AF           (XLLP_MFP_PIN_GPIO48_AF_GPIO_48)

#define XLLP_MFP_GPIO_49_TB_YP_OFFSET       (XLLP_MFP_PIN_GPIO49_OFFSET)
#define XLLP_MFP_GPIO_49_TB_YP_AF           (XLLP_MFP_PIN_GPIO49_AF_GPIO_49)

#define XLLP_MFP_GPIO_50_TB_YM_OFFSET       (XLLP_MFP_PIN_GPIO50_OFFSET)
#define XLLP_MFP_GPIO_50_TB_YM_AF           (XLLP_MFP_PIN_GPIO50_AF_GPIO_50)


#define XLLP_MFP_GPIO_54_OFFSET       (XLLP_MFP_PIN_GPIO54_OFFSET)
#define XLLP_MFP_GPIO_54_AF           (XLLP_MFP_PIN_GPIO54_AF_GPIO_54)

#define XLLP_MFP_GPIO_55_OFFSET       (XLLP_MFP_PIN_GPIO55_OFFSET)
#define XLLP_MFP_GPIO_55_AF           (XLLP_MFP_PIN_GPIO55_AF_GPIO_55)

#define XLLP_MFP_GPIO_56_OFFSET       (XLLP_MFP_PIN_GPIO56_OFFSET)
#define XLLP_MFP_GPIO_56_AF           (XLLP_MFP_PIN_GPIO56_AF_GPIO_56)

#define XLLP_MFP_GPIO_57_OFFSET       (XLLP_MFP_PIN_GPIO57_OFFSET)
#define XLLP_MFP_GPIO_57_AF           (XLLP_MFP_PIN_GPIO57_AF_GPIO_57)



#define XLLP_MFP_GPIO_62_OFFSET           (XLLP_MFP_PIN_GPIO62_OFFSET)
#define XLLP_MFP_GPIO_62_AF               (XLLP_MFP_PIN_GPIO62_AF_GPIO_62)

#define XLLP_MFP_GPIO_63_OFFSET           (XLLP_MFP_PIN_GPIO63_OFFSET)
#define XLLP_MFP_GPIO_63_AF               (XLLP_MFP_PIN_GPIO63_AF_GPIO_63)

#define XLLP_MFP_GPIO_64_OFFSET           (XLLP_MFP_PIN_GPIO64_OFFSET)
#define XLLP_MFP_GPIO_64_AF               (XLLP_MFP_PIN_GPIO64_AF_GPIO_64)

#define XLLP_MFP_GPIO_65_OFFSET           (XLLP_MFP_PIN_GPIO65_OFFSET)
#define XLLP_MFP_GPIO_65_AF               (XLLP_MFP_PIN_GPIO65_AF_GPIO_65)

/* FIXME - not sure*/
#define XLLP_MFP_STRXD_OFFSET           (XLLP_MFP_PIN_GPIO45_OFFSET) // Fixed Ran
#define XLLP_MFP_STRXD_AF               (XLLP_MFP_PIN_GPIO45_AF_STRXD) // Fixed Ran

#define XLLP_MFP_STTXD_OFFSET           (XLLP_MFP_PIN_GPIO46_OFFSET) // Fixed Ran
#define XLLP_MFP_STTXD_AF               (XLLP_MFP_PIN_GPIO46_AF_STTXD) // Fixed Ran


#define XLLP_MFP_GPIO_68_OFFSET           (XLLP_MFP_PIN_GPIO68_OFFSET)
#define XLLP_MFP_GPIO_68_AF               (XLLP_MFP_PIN_GPIO68_AF_GPIO_68)

#define XLLP_MFP_GPIO_69_OFFSET           (XLLP_MFP_PIN_GPIO69_OFFSET)
#define XLLP_MFP_GPIO_69_AF               (XLLP_MFP_PIN_GPIO69_AF_GPIO_69)



#define XLLP_MFP_GPIO_71_OFFSET         (XLLP_MFP_PIN_GPIO71_OFFSET)
#define XLLP_MFP_GPIO_71_AF             (XLLP_MFP_PIN_GPIO71_AF_GPIO_71)

#define XLLP_MFP_GPIO_72_OFFSET         (XLLP_MFP_PIN_GPIO72_OFFSET)
#define XLLP_MFP_GPIO_72_AF             (XLLP_MFP_PIN_GPIO72_AF_GPIO_72)

// From Intel:
//FFUART lines:
//FFRXD ?GPIO53
//FFTXD ?GPIO54
//FFCTS ?GPIO51
//FFRTS ?GPIO52

// FIXME - not sure. could be 51 or 69
//#define XLLP_MFP_FFCTS_OFFSET        (XLLP_MFP_PIN_GPIOXX_OFFSET)  //Fixed Ran
//#define XLLP_MFP_FFCTS_AF            (XLLP_MFP_PIN_GPIOXX_AF_FFCTS_N)  //Fixed Ran

// FIXME - not sure. could be 52 or 70
//#define XLLP_MFP_FFRTS_OFFSET        (XLLP_MFP_PIN_GPIOXX_OFFSET)  //Fixed Ran
//#define XLLP_MFP_FFRTS_AF            (XLLP_MFP_PIN_GPIOXX_AF_FFRTS_N)  //Fixed Ran

// FIXME - not sure. could be 53, 54, or 64
#define XLLP_MFP_FFRXD_OFFSET         (XLLP_MFP_PIN_GPIO53_OFFSET)  //Fixed Ran
#define XLLP_MFP_FFRXD_AF             (XLLP_MFP_PIN_GPIO53_AF_FFRXD)  //Fixed Ran

// FIXME - not sure. could be 53, 54, or 63
#define XLLP_MFP_FFTXD_OFFSET         (XLLP_MFP_PIN_GPIO54_OFFSET)  //Fixed Ran
#define XLLP_MFP_FFTXD_AF             (XLLP_MFP_PIN_GPIO54_AF_FFTXD)  //Fixed Ran


#define XLLP_MFP_GPIO_80_OFFSET          (XLLP_MFP_PIN_GPIO80_OFFSET)
#define XLLP_MFP_GPIO_80_AF              (XLLP_MFP_PIN_GPIO80_AF_GPIO_80)

#define XLLP_MFP_GPIO_81_OFFSET         (XLLP_MFP_PIN_GPIO81_OFFSET)
#define XLLP_MFP_GPIO_81_AF             (XLLP_MFP_PIN_GPIO81_AF_GPIO_81)


#define XLLP_MFP_GSSP1_CLK_OFFSET       (XLLP_MFP_PIN_GPIO79_OFFSET)
#define XLLP_MFP_GSSP1_CLK_AF           (XLLP_MFP_PIN_GPIO79_AF_GSSP1_CLK)
#define XLLP_MFP_GSSP1_CLK_LPM			(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_GSSP1_FRM_OFFSET       (XLLP_MFP_PIN_GPIO80_OFFSET)
#define XLLP_MFP_GSSP1_FRM_AF           (XLLP_MFP_PIN_GPIO80_AF_GSSP1_FRM)
#define XLLP_MFP_GSSP1_FRM_LPM			(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_GSSP1_TXD_OFFSET       (XLLP_MFP_PIN_GPIO81_OFFSET)
#define XLLP_MFP_GSSP1_TXD_AF           (XLLP_MFP_PIN_GPIO81_AF_GSSP1_TXD)
#define XLLP_MFP_GSSP1_TXD_LPM			(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_GSSP1_RXD_OFFSET       (XLLP_MFP_PIN_GPIO82_OFFSET)
#define XLLP_MFP_GSSP1_RXD_AF           (XLLP_MFP_PIN_GPIO82_AF_GSSP1_RXD)
#define XLLP_MFP_GSSP1_RXD_LPM			(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_GSSP1_SYSCLK_OFFSET    (XLLP_MFP_PIN_GPIO83_OFFSET)
#define XLLP_MFP_GSSP1_SYSCLK_AF        (XLLP_MFP_PIN_GPIO83_AF_GSSP1_SYSCLK)
#define XLLP_MFP_GSSP1_SYSCLK_LPM		(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_BSSP3_CLK_AF           (XLLP_MFP_PIN_GPIO79_AF_SSP3_CLK)
#define XLLP_MFP_BSSP3_FRM_AF           (XLLP_MFP_PIN_GPIO80_AF_SSP3_FRM)
#define XLLP_MFP_BSSP3_TXD_AF           (XLLP_MFP_PIN_GPIO81_AF_SSP3_TXD)
#define XLLP_MFP_BSSP3_RXD_AF           (XLLP_MFP_PIN_GPIO82_AF_SSP3_RXD)
#define XLLP_MFP_BSSP3_SYSCLK_AF        (XLLP_MFP_PIN_GPIO83_AF_SSP3_SYSCLK)


//I2C Interface

#define XLLP_MFP_SCL_OFFSET       (XLLP_MFP_PIN_GPIO89_OFFSET)
#define XLLP_MFP_SCL_AF           (XLLP_MFP_PIN_GPIO89_AF_CI2C_SCL)

#define XLLP_MFP_SDA_OFFSET       (XLLP_MFP_PIN_GPIO90_OFFSET)
#define XLLP_MFP_SDA_AF           (XLLP_MFP_PIN_GPIO90_AF_CI2C_SDA)

#define XLLP_MFP_PMIC_INT_OFFSET       (XLLP_MFP_PIN_GPIO83_OFFSET)
#define XLLP_MFP_PMIC_INT_AF           (XLLP_MFP_PIN_GPIO83_AF_GPIO_83)





// FIXME - HIFI audio
#define XLLP_MFP_HIFI_BITCLK_OFFSET         (XLLP_MFP_PIN_GPIO85_OFFSET)
#define XLLP_MFP_HIFI_BITCLK_AF             (XLLP_MFP_PIN_GPIO85_AF_SSP2_BITCLK)
#define XLLP_MFP_HIFI_BITCLK_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_HIFI_SDATA_IN_OFFSET       (XLLP_MFP_PIN_GPIO84_OFFSET)
#define XLLP_MFP_HIFI_SDATA_IN_AF           (XLLP_MFP_PIN_GPIO84_AF_SSP2_SDATA_IN)
#define XLLP_MFP_HIFI_SDATA_IN_LPM          (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_HIFI_SYSCLK_OFFSET         (XLLP_MFP_PIN_GPIO86_OFFSET)
#define XLLP_MFP_HIFI_SYSCLK_AF             (XLLP_MFP_PIN_GPIO86_AF_SSP2_SYSCLK)
#define XLLP_MFP_HIFI_SYSCLK_LPM           (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_HIFI_SYNC_OFFSET           (XLLP_MFP_PIN_GPIO87_OFFSET)
#define XLLP_MFP_HIFI_SYNC_AF               (XLLP_MFP_PIN_GPIO87_AF_SSP2_SYNC)
#define XLLP_MFP_HIFI_SYNC_LPM           (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_HIFI_SDATA_OUT_OFFSET      (XLLP_MFP_PIN_GPIO88_OFFSET)
#define XLLP_MFP_HIFI_SDATA_OUT_AF          (XLLP_MFP_PIN_GPIO88_AF_SSP2_DATA_OUT)
#define XLLP_MFP_HIFI_SDATA_OUT_LPM           (XLLP_MFP_LPMO_PULL_LOW)

// Voice
#define XLLP_MFP_PIN_VOICE_CLK_OFFSET		(XLLP_MFP_PIN_GPIO79_OFFSET)
#define XLLP_MFP_PIN_VOICE_CLK_AF			(XLLP_MFP_PIN_GPIO79_AF_SSP3_CLK)

#define XLLP_MFP_PIN_VOICE_FRM_OFFSET		(XLLP_MFP_PIN_GPIO80_OFFSET)
#define XLLP_MFP_PIN_VOICE_FRM_AF			(XLLP_MFP_PIN_GPIO80_AF_SSP3_FRM)

#define XLLP_MFP_PIN_VOICE_TXD_OFFSET		(XLLP_MFP_PIN_GPIO81_OFFSET)
#define XLLP_MFP_PIN_VOICE_TXD_AF			(XLLP_MFP_PIN_GPIO81_AF_SSP3_TXD)

#define XLLP_MFP_PIN_VOICE_RXD_OFFSET		(XLLP_MFP_PIN_GPIO82_OFFSET)
#define XLLP_MFP_PIN_VOICE_RXD_AF			(XLLP_MFP_PIN_GPIO82_AF_SSP3_RXD)



#define XLLP_MFP_GPIO_18_OFFSET    (XLLP_MFP_PIN_VOICE_SYSCLK_OFFSET)
#define XLLP_MFP_GPIO_18_AF        (XLLP_MFP_PIN_VOICE_SYSCLK_AF_GPIO_18)



#define	SSP_DRIVER_MFP_GPIO_BSSP3_CLK               	MFPR_GPIO79
#define	SSP_DRIVER_MFP_GPIO_BSSP3_FRM              		MFPR_GPIO80
#define SSP_DRIVER_MFP_GPIO_BSSP3_TXD                  	MFPR_GPIO81
#define SSP_DRIVER_MFP_GPIO_BSSP3_RXD               	MFPR_GPIO82


#define XLLP_MFP_DF_IO_15_OFFSET        (XLLP_MFP_PIN_ND_IO15_OFFSET)
#define XLLP_MFP_DF_IO_15_AF            (XLLP_MFP_PIN_ND_IO15_AF_DF_IO15)

#define XLLP_MFP_DF_IO_14_OFFSET        (XLLP_MFP_PIN_ND_IO14_OFFSET)
#define XLLP_MFP_DF_IO_14_AF            (XLLP_MFP_PIN_ND_IO14_AF_DF_IO14)

#define XLLP_MFP_DF_IO_13_OFFSET        (XLLP_MFP_PIN_ND_IO13_OFFSET)
#define XLLP_MFP_DF_IO_13_AF            (XLLP_MFP_PIN_ND_IO13_AF_DF_IO13)

#define XLLP_MFP_DF_IO_12_OFFSET        (XLLP_MFP_PIN_ND_IO12_OFFSET)
#define XLLP_MFP_DF_IO_12_AF            (XLLP_MFP_PIN_ND_IO12_AF_DF_IO12)

#define XLLP_MFP_DF_IO_11_OFFSET        (XLLP_MFP_PIN_ND_IO11_OFFSET)
#define XLLP_MFP_DF_IO_11_AF            (XLLP_MFP_PIN_ND_IO11_AF_DF_IO11)

#define XLLP_MFP_DF_IO_10_OFFSET        (XLLP_MFP_PIN_ND_IO10_OFFSET)
#define XLLP_MFP_DF_IO_10_AF            (XLLP_MFP_PIN_ND_IO10_AF_DF_IO10)

#define XLLP_MFP_DF_IO_9_OFFSET         (XLLP_MFP_PIN_ND_IO9_OFFSET)
#define XLLP_MFP_DF_IO_9_AF             (XLLP_MFP_PIN_ND_IO9_AF_DF_IO9)

#define XLLP_MFP_DF_IO_8_OFFSET         (XLLP_MFP_PIN_ND_IO8_OFFSET)
#define XLLP_MFP_DF_IO_8_AF             (XLLP_MFP_PIN_ND_IO8_AF_DF_IO8)

#define XLLP_MFP_DF_IO_7_OFFSET         (XLLP_MFP_PIN_ND_IO7_OFFSET)
#define XLLP_MFP_DF_IO_7_AF             (XLLP_MFP_PIN_ND_IO7_AF_DF_IO7)

#define XLLP_MFP_DF_IO_6_OFFSET         (XLLP_MFP_PIN_ND_IO6_OFFSET)
#define XLLP_MFP_DF_IO_6_AF             (XLLP_MFP_PIN_ND_IO6_AF_DF_IO6)

#define XLLP_MFP_DF_IO_5_OFFSET         (XLLP_MFP_PIN_ND_IO5_OFFSET)
#define XLLP_MFP_DF_IO_5_AF             (XLLP_MFP_PIN_ND_IO5_AF_DF_IO5)

#define XLLP_MFP_DF_IO_4_OFFSET         (XLLP_MFP_PIN_ND_IO4_OFFSET)
#define XLLP_MFP_DF_IO_4_AF             (XLLP_MFP_PIN_ND_IO4_AF_DF_IO4)

#define XLLP_MFP_DF_IO_3_OFFSET         (XLLP_MFP_PIN_ND_IO3_OFFSET)
#define XLLP_MFP_DF_IO_3_AF             (XLLP_MFP_PIN_ND_IO3_AF_DF_IO3)

#define XLLP_MFP_DF_IO_2_OFFSET         (XLLP_MFP_PIN_ND_IO2_OFFSET)
#define XLLP_MFP_DF_IO_2_AF             (XLLP_MFP_PIN_ND_IO2_AF_DF_IO2)

#define XLLP_MFP_DF_IO_1_OFFSET         (XLLP_MFP_PIN_ND_IO1_OFFSET)
#define XLLP_MFP_DF_IO_1_AF             (XLLP_MFP_PIN_ND_IO1_AF_DF_IO1)

#define XLLP_MFP_DF_IO_0_OFFSET         (XLLP_MFP_PIN_ND_IO0_OFFSET)
#define XLLP_MFP_DF_IO_0_AF             (XLLP_MFP_PIN_ND_IO0_AF_DF_IO0)

#define XLLP_MFP_DF_NCS0_OFFSET         (XLLP_MFP_PIN_ND_nCS0_OFFSET)
#define XLLP_MFP_DF_NCS0_AF             (XLLP_MFP_PIN_ND_nCS0_AF_ND_nCS0)

#define XLLP_MFP_DF_nWE_OFFSET          (XLLP_MFP_PIN_ND_nWE_OFFSET)
#define XLLP_MFP_DF_nWE_AF              (XLLP_MFP_PIN_ND_nWE_AF_ND_nWE)

#define XLLP_MFP_DF_nOE_OFFSET          (XLLP_MFP_PIN_ND_nRE_nOE_OFFSET)
#define XLLP_MFP_DF_nOE_AF              (XLLP_MFP_PIN_ND_nRE_nOE_AF_DF_nOE)

#define XLLP_MFP_ND_CLE_OFFSET          (XLLP_MFP_PIN_ND_CLE_OFFSET)
#define XLLP_MFP_ND_CLE_AF              (XLLP_MFP_PIN_ND_CLE_AF_ND_CLE)

#define XLLP_MFP_DF_nADV1_OFFSET        (XLLP_MFP_PIN_ND_nADV1_ALE_OFFSET)
#define XLLP_MFP_DF_nADV1_AF            (XLLP_MFP_PIN_ND_nADV1_ALE_AF_DF_nADV1)

#define XLLP_MFP_DF_INT_RnB_OFFSET      (XLLP_MFP_PIN_ND_INT_RnB_OFFSET)
#define XLLP_MFP_DF_INT_RnB_AF          (XLLP_MFP_PIN_ND_INT_RnB_AF_ND_INT_RnB)

#define XLLP_MFP_DF_SCLK_E_OFFSET          (XLLP_MFP_PIN_DF_SCLK_E_OFFSET)
#define XLLP_MFP_DF_SCLK_E_AF              (XLLP_MFP_PIN_DF_SCLK_E_AF_DF_SCLK_E)

#define XLLP_MFP_GPIO_106_OFFSET      (XLLP_MFP_PIN_DF_ADDR0_OFFSET)
#define XLLP_MFP_GPIO_106_AF          (XLLP_MFP_PIN_DF_ADDR0_AF_GPIO_106)

#define XLLP_MFP_GPIO_107_OFFSET      (XLLP_MFP_PIN_DF_ADDR1_OFFSET)
#define XLLP_MFP_GPIO_107_AF          (XLLP_MFP_PIN_DF_ADDR1_AF_GPIO_107)

#define XLLP_MFP_nLUA_OFFSET      (XLLP_MFP_PIN_nLUA_OFFSET)
#define XLLP_MFP_nLUA_AF          (XLLP_MFP_PIN_nLUA_AF_nLUA)

#define XLLP_MFP_nLLA_OFFSET      (XLLP_MFP_PIN_nLLA_OFFSET)
#define XLLP_MFP_nLLA_AF          (XLLP_MFP_PIN_nLLA_AF_nLLA)

#define XLLP_MFP_nBE_0_OFFSET      (XLLP_MFP_PIN_nBE0_OFFSET)
#define XLLP_MFP_nBE_0_AF          (XLLP_MFP_PIN_nBE0_AF_nBE_0)

#define XLLP_MFP_nBE_1_OFFSET      (XLLP_MFP_PIN_nBE1_OFFSET)
#define XLLP_MFP_nBE_1_AF          (XLLP_MFP_PIN_nBE1_AF_nBE_1)

#define XLLP_MFP_RDY_OFFSET      (XLLP_MFP_PIN_RDY_OFFSET)
#define XLLP_MFP_RDY_AF          (XLLP_MFP_PIN_RDY_AF_RDY)

#define XLLP_MFP_UNLOCK_OFFSET      (XLLP_MFP_PIN_UN_LOCK_OFFSET)
#define XLLP_MFP_UNLOCK_AF          (XLLP_MFP_PIN_UN_LOCK_AF_DF_UNLOCK)

/* FIXME - not sure */
#define XLLP_MFP_MMC_CMD_0_OFFSET       (XLLP_MFP_PIN_GPIO14_OFFSET) 
#define XLLP_MFP_MMC_CMD_0_AF           (XLLP_MFP_PIN_GPIO14_AF_MMC1_CMD) 

#define XLLP_MFP_MMC_CLK_OFFSET         (XLLP_MFP_PIN_GPIO15_OFFSET) 
#define XLLP_MFP_MMC_CLK_AF             (XLLP_MFP_PIN_GPIO15_AF_MMC1_CLK) 

#define XLLP_MFP_MMC_DAT0_OFFSET        (XLLP_MFP_PIN_GPIO13_OFFSET) 
#define XLLP_MFP_MMC_DAT0_AF            (XLLP_MFP_PIN_GPIO13_AF_MMC1_DAT0) 

#define XLLP_MFP_MMC_DAT1_OFFSET        (XLLP_MFP_PIN_GPIO12_OFFSET) 
#define XLLP_MFP_MMC_DAT1_AF            (XLLP_MFP_PIN_GPIO12_AF_MMC1_DAT1) 


#define XLLP_MFP_MMC_DAT2_OFFSET		(XLLP_MFP_PIN_GPIO11_OFFSET)
#define XLLP_MFP_MMC_DAT2_AF			(XLLP_MFP_PIN_GPIO11_AF_MMC1_DAT2)

#define XLLP_MFP_MMC_DAT3_OFFSET		(XLLP_MFP_PIN_GPIO10_OFFSET)
#define XLLP_MFP_MMC_DAT3_AF			(XLLP_MFP_PIN_GPIO10_AF_MMC1_DAT3)

#define XLLP_MFP_MMC_CD_0_GPIO_OFFSET       (XLLP_MFP_PIN_GPIO20_OFFSET) 
#define XLLP_MFP_MMC_CD_0_GPIO_AF           (XLLP_MFP_PIN_GPIO20_AF_GPIO_20) 

#define XLLP_MFP_MMC_WP_0_N_GPIO_OFFSET       (XLLP_MFP_PIN_DF_ADDR3_OFFSET) //Fixed Ran
#define XLLP_MFP_MMC_WP_0_N_GPIO_AF           (XLLP_MFP_PIN_DF_ADDR3_AF_GPIO_56)  // Fixed Ran


#define XLLP_MFP_GPIO_120_OFFSET        (XLLP_MFP_PIN_nPIOW_OFFSET)
#define XLLP_MFP_GPIO_120_AF            (XLLP_MFP_PIN_nPIOW_AF_GPIO_120)

#define XLLP_MFP_GPIO_121_OFFSET          (XLLP_MFP_PIN_nPWAIT_OFFSET)
#define XLLP_MFP_GPIO_121_AF              (XLLP_MFP_PIN_nPWAIT_AF_GPIO_121)

#define XLLP_MFP_nCS2_OFFSET        (XLLP_MFP_PIN_ND_nADV2_ALE_OFFSET)
#define XLLP_MFP_nCS2_AF            (XLLP_MFP_PIN_ND_nADV2_ALE_AF_nCS2)

// FIXME - schematic says nADV1_ALE, but nADV1_ALE doesn't list nCS3 as a valid CS offset...
// nCS3 is on ND_nCS1, GPIO56
// Intel - ND_nCS1
#define XLLP_MFP_nCS3_OFFSET         (XLLP_MFP_PIN_ND_nCS1_OFFSET)
#define XLLP_MFP_nCS3_AF             (XLLP_MFP_PIN_ND_nCS1_AF_nCS3)


#if 0
/* Flash/Torch support */

#define XLLP_MFP_TORCH_FLASHEN_OFFSET       (XLLP_MFP_PIN_GPIO61_OFFSET) // Flash Enable 1 = on
#define XLLP_MFP_TORCH_FLASHEN_AF           (XLLP_MFP_PIN_GPIO61_AF_GPIO_61)

//Need to resolve - M.
/*
#define XLLP_MFP_TORCH_FLASH_OFFSET         (XLLP_MFP_PIN_GPIO64_OFFSET) // Flash itself, 1 = on
#define XLLP_MFP_TORCH_FLASH_AF             (XLLP_MFP_PIN_GPIO64_AF_CAM_IN8)   */

/*
#define XLLP_MFP_TORCH_TORCH_OFFSET         (XLLP_MFP_TORCH_FLASH_OFFSET) // Torch 1 = on
#define XLLP_MFP_TORCH_TORCH_AF             (XLLP_MFP_TORCH_FLASH_AF)     */


// FIXME - LCD Backlight support
#define XLLP_MFP_BACKLIGHT_OFFSET           (XLLP_MFP_PIN_GPIO43_OFFSET)
#define XLLP_MFP_BACKLIGHT_AF               (XLLP_MFP_PIN_GPIO43_AF_PWM3)
// FIXME - 2ndLCD Backlight support
#endif //0

  //Currently NA - To be resolved with M.
//#define XLLP_MFP_2NDBACKLIGHT_OFFSET
//#define XLLP_MFP_2NDBACKLIGHT_AF

//==================================


#define XLLP_MFP_MMC_CD_0_GPIO_LPM  (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC_WP_0_N_GPIO_LPM     (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_MMC_WP_1_N_GPIO_OFFSET     (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_MMC_WP_1_N_GPIO_AF         (XLLP_MFP_PIN_GPIOXX_AF_GPIO_58)
#define XLLP_MFP_MMC_WP_1_N_GPIO_LPM	(XLLP_MFP_LPMO_PULL_HIGH)


// This PWM is an input to the Hermon daughter card, as is PWM0.
// Its intended usage is not specified.

//Firstly, disable the irda, to enable keypad
//#define XLLP_MFP_CIR_ON_PWM_OFFSET          (XLLP_MFP_PIN_GPIO13_OFFSET)	//TBD
#define XLLP_MFP_CIR_ON_PWM_OFFSET          0xff

#define XLLP_MFP_BACKLIGHT_PWM_OFFSET       (XLLP_MFP_PIN_GPIO43_OFFSET) //Need further check
#define XLLP_MFP_BACKLIGHT_PWM_AF           (XLLP_MFP_PIN_GPIO43_AF_PWM3)
#define XLLP_MFP_BACKLIGHT_PWM_LPM         (XLLP_MFP_LPMO_PULL_HIGH)

//Firstly, disable AC97 pin, to enable keypad
//#define XLLP_MFP_AC97_INT_N_GPIO_OFFSET     (XLLP_MFP_PIN_GPIO15_OFFSET)   //TBD
#define XLLP_MFP_AC97_INT_N_GPIO_OFFSET     0xff
#define XLLP_MFP_AC97_INT_N_GPIO_AF         (XLLP_MFP_PIN_GPIO15_AF_GPIO_15)  //TBD



#define XLLP_MFP_MMC_DAT0_LPM		 (XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_MMC_DAT1_LPM		 (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC_DAT2_LPM		 (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC_DAT3_LPM		 (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC_CLK_LPM		(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC_CMD_0_LPM			(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_MMC2_DAT0_OFFSET     (XLLP_MFP_PIN_GPIO104_OFFSET)
#define XLLP_MFP_MMC2_DAT0_AF         (XLLP_MFP_PIN_GPIO104_AF_MMC2_DAT0)
#define XLLP_MFP_MMC2_DAT0_LPM	      (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC2_DAT1_OFFSET       (XLLP_MFP_PIN_GPIO103_OFFSET)
#define XLLP_MFP_MMC2_DAT1_AF     (XLLP_MFP_PIN_GPIO103_AF_MMC2_DAT1)
#define XLLP_MFP_MMC2_DAT1_LPM		(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC2_DAT2_CS0_OFFSET   (XLLP_MFP_PIN_GPIO102_OFFSET)
#define XLLP_MFP_MMC2_DAT2_CS0_AF   (XLLP_MFP_PIN_GPIO102_AF_MMC2_DAT2)
#define XLLP_MFP_MMC2_DAT2_CS0_LPM	(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_MMC2_DAT3_CS1_OFFSET   (XLLP_MFP_PIN_GPIO101_OFFSET)
#define XLLP_MFP_MMC2_DAT3_CS1_AF   (XLLP_MFP_PIN_GPIO101_AF_MMC2_DAT3)
#define XLLP_MFP_MMC2_DAT3_CS1_LPM	(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC2_CLK_OFFSET        (XLLP_MFP_PIN_GPIO106_OFFSET)
#define XLLP_MFP_MMC2_CLK_AF     (XLLP_MFP_PIN_GPIO106_AF_MMC2_CLK)
#define XLLP_MFP_MMC2_CLK_LPM	   (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_MMC2_CMD_OFFSET        (XLLP_MFP_PIN_GPIO105_OFFSET)
#define XLLP_MFP_MMC2_CMD_AF          (XLLP_MFP_PIN_GPIO105_AF_MMC2_CMD)
#define XLLP_MFP_MMC2_CMD_LPM	(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_WIFI_RESET_GPIO_ID				86
#define XLLP_WIFI_RESET_OFFSET				XLLP_MFP_PIN_GPIO86_OFFSET
#define XLLP_WIFI_RESET_AF					XLLP_MFP_PIN_GPIO86_AF_GPIO_86
#define XLLP_WIFI_RESET_LPM					XLLP_MFP_LPMO_FLOAT

#define XLLP_WIFI_HOST_WKUP_GPIO_ID			95
#define XLLP_WIFI_HOST_WKUP_OFFSET			XLLP_MFP_PIN_GPIO95_OFFSET
#define XLLP_WIFI_HOST_WKUP_AF				XLLP_MFP_PIN_GPIO95_AF_GPIO_95
#define XLLP_WIFI_HOST_WKUP_LPM				XLLP_MFP_LPMO_FLOAT

#define XLLP_WIFI_MCU_WAKEUP_GPIO_ID		96
#define XLLP_WIFI_MCU_WAKEUP_OFFSET			XLLP_MFP_PIN_GPIO96_OFFSET
#define XLLP_WIFI_MCU_WAKEUP_AF				XLLP_MFP_PIN_GPIO96_AF_GPIO_96
#define XLLP_WIFI_MCU_WAKEUP_LPM			XLLP_MFP_LPMO_FLOAT

#define XLLP_BT_TRI_STATE_MICCO_GPIO_ID		0xFF
#define XLLP_BT_TRI_STATE_MICCO_OFFSET		XLLP_MFP_PIN_GPIOXX_OFFSET
#define XLLP_BT_TRI_STATE_MICCO_AF			XLLP_MFP_PIN_GPIOXX_AF_GPIO_49
#define XLLP_BT_TRI_STATE_MICCO_LPM			XLLP_MFP_LPMO_FLOAT

#define XLLP_MFP_SCL_LPM               (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_SDA_LPM               (XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_FFRXD_OFFSET           (XLLP_MFP_PIN_GPIO53_OFFSET)
#define XLLP_MFP_FFRXD_AF               (XLLP_MFP_PIN_GPIO53_AF_FFRXD)
#define XLLP_MFP_FFRXD_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_FFTXD_OFFSET           (XLLP_MFP_PIN_GPIO54_OFFSET)
#define XLLP_MFP_FFTXD_AF               (XLLP_MFP_PIN_GPIO54_AF_FFTXD)
#define XLLP_MFP_FFTXD_LPM           (XLLP_MFP_LPMO_PULL_HIGH)

#if 0
#define XLLP_MFP_FFCTS_OFFSET           (XLLP_MFP_PIN_GPIO51_OFFSET)
#define XLLP_MFP_FFCTS_AF               (XLLP_MFP_PIN_GPIO51_AF_FFCTS_N)
#define XLLP_MFP_FFCTS_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_FFRTS_OFFSET           (XLLP_MFP_PIN_GPIO52_OFFSET)
#define XLLP_MFP_FFRTS_AF               (XLLP_MFP_PIN_GPIO52_AF_FFRTS_N)
#define XLLP_MFP_FFRTS_LPM           (XLLP_MFP_LPMO_PULL_HIGH)
#endif //0

#define XLLP_MFP_CIF_DD_0_OFFSET        (XLLP_MFP_PIN_GPIO72_OFFSET)
#define XLLP_MFP_CIF_DD_0_AF            (XLLP_MFP_PIN_GPIO72_AF_CAM_IN0)
#define XLLP_MFP_CIF_DD_0_LPM           (XLLP_MFP_LPMO_FLOAT)


#define XLLP_MFP_CIF_DD_1_OFFSET        (XLLP_MFP_PIN_GPIO71_OFFSET)
#define XLLP_MFP_CIF_DD_1_AF            (XLLP_MFP_PIN_GPIO71_AF_CAM_IN1)
#define XLLP_MFP_CIF_DD_1_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_2_OFFSET        (XLLP_MFP_PIN_GPIO70_OFFSET)
#define XLLP_MFP_CIF_DD_2_AF            (XLLP_MFP_PIN_GPIO70_AF_CAM_IN2)
#define XLLP_MFP_CIF_DD_2_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_3_OFFSET        (XLLP_MFP_PIN_GPIO69_OFFSET)
#define XLLP_MFP_CIF_DD_3_AF            (XLLP_MFP_PIN_GPIO69_AF_CAM_IN3)
#define XLLP_MFP_CIF_DD_3_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_4_OFFSET        (XLLP_MFP_PIN_GPIO68_OFFSET)
#define XLLP_MFP_CIF_DD_4_AF            (XLLP_MFP_PIN_GPIO68_AF_CAM_IN4)
#define XLLP_MFP_CIF_DD_4_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_5_OFFSET        (XLLP_MFP_PIN_GPIO67_OFFSET)
#define XLLP_MFP_CIF_DD_5_AF            (XLLP_MFP_PIN_GPIO67_AF_CAM_IN5)
#define XLLP_MFP_CIF_DD_5_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_6_OFFSET        (XLLP_MFP_PIN_GPIO66_OFFSET)
#define XLLP_MFP_CIF_DD_6_AF            (XLLP_MFP_PIN_GPIO66_AF_CAM_IN6)
#define XLLP_MFP_CIF_DD_6_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_7_OFFSET        (XLLP_MFP_PIN_GPIO65_OFFSET)
#define XLLP_MFP_CIF_DD_7_AF            (XLLP_MFP_PIN_GPIO65_AF_CAM_IN7)
#define XLLP_MFP_CIF_DD_7_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_DD_8_OFFSET        (XLLP_MFP_PIN_GPIO64_OFFSET)
#define XLLP_MFP_CIF_DD_8_AF            (XLLP_MFP_PIN_GPIO64_AF_CAM_IN8)
#define XLLP_MFP_CIF_DD_8_LPM           (XLLP_MFP_LPMO_FLOAT)

#if 0 // Ido: Tavor Uses only 8 bits - GPIO 63 is used to TriState Micco on BT Call
#define XLLP_MFP_CIF_DD_9_OFFSET        (XLLP_MFP_PIN_GPIO63_OFFSET)
#define XLLP_MFP_CIF_DD_9_AF            (XLLP_MFP_PIN_GPIO63_AF_CAM_IN9)
#define XLLP_MFP_CIF_DD_9_LPM           (XLLP_MFP_LPMO_FLOAT)
#endif


#define XLLP_MFP_CIF_MCLK_OFFSET        (XLLP_MFP_PIN_GPIO75_OFFSET)
#define XLLP_MFP_CIF_MCLK_AF            (XLLP_MFP_PIN_GPIO75_AF_CAM_MCLK)
#define XLLP_MFP_CIF_MCLK_LPM           (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_CIF_PCLK_OFFSET        (XLLP_MFP_PIN_GPIO76_OFFSET)
#define XLLP_MFP_CIF_PCLK_AF            (XLLP_MFP_PIN_GPIO76_AF_CAM_PCLK)
#define XLLP_MFP_CIF_PCLK_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_HSYNC_OFFSET       (XLLP_MFP_PIN_GPIO73_OFFSET)
#define XLLP_MFP_CIF_HSYNC_AF           (XLLP_MFP_PIN_GPIO73_AF_CAM_HSYNC)
#define XLLP_MFP_CIF_HSYNC_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_CIF_VSYNC_OFFSET       (XLLP_MFP_PIN_GPIO74_OFFSET)
#define XLLP_MFP_CIF_VSYNC_AF           (XLLP_MFP_PIN_GPIO74_AF_CAM_VSYNC)
#define XLLP_MFP_CIF_VSYNC_LPM           (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_0_OFFSET (XLLP_MFP_PIN_GPIO24_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_0_AF     (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_OB_CLK_OFFSET    (XLLP_MFP_PIN_GPIO28_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_CLK_AF        XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_OB_STB_OFFSET    (XLLP_MFP_PIN_GPIO29_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_STB_AF        (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_OB_WAIT_OFFSET   (XLLP_MFP_PIN_GPIO30_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_WAIT_AF       (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_0_OFFSET  (XLLP_MFP_PIN_GPIO17_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_0_AF      (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_IB_CLK_OFFSET    (XLLP_MFP_PIN_GPIO21_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_CLK_AF        (XLLP_MFP_ALT_FN_4)

//To enable backlight, disable this
//#define XLLP_MFP_RSVD_MSL1_IB_STB_OFFSET    (XLLP_MFP_PIN_GPIO22_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_STB_OFFSET    0xFFFF
#define XLLP_MFP_RSVD_MSL1_IB_STB_AF        (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_IB_WAIT_OFFSET   (XLLP_MFP_PIN_GPIO23_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_WAIT_AF       (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_1_OFFSET  (XLLP_MFP_PIN_GPIO18_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_1_AF      (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_2_OFFSET  (XLLP_MFP_PIN_GPIO19_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_2_AF      (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_IB_DAT_3_OFFSET  (XLLP_MFP_PIN_GPIO20_OFFSET)
#define XLLP_MFP_RSVD_MSL1_IB_DAT_3_AF      (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_1_OFFSET  (XLLP_MFP_PIN_GPIO25_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_1_AF      (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_2_OFFSET  (XLLP_MFP_PIN_GPIO26_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_2_AF      (XLLP_MFP_ALT_FN_4)

#define XLLP_MFP_RSVD_MSL1_OB_DAT_3_OFFSET  (XLLP_MFP_PIN_GPIO27_OFFSET)
#define XLLP_MFP_RSVD_MSL1_OB_DAT_3_AF      (XLLP_MFP_ALT_FN_4)

//FIXME: need further check
#define XLLP_MFP_SSP_AUDIO_SCLK_OFFSET      (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_SSP_AUDIO_SCLK_AF          (XLLP_MFP_PIN_GPIOXX_AF_DSSP3_SCLK)
#define XLLP_MFP_SSP_AUDIO_SCLK_LPM	(XLLP_MFP_LPMO_PULL_LOW)
//#define XLLP_MFP_PIN_GPIO89_LPM	 (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_SSP_AUDIO_FRM_OFFSET       (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_SSP_AUDIO_FRM_AF           (XLLP_MFP_PIN_GPIOXX_AF_DSSP3_FRM)
#define XLLP_MFP_SSP_AUDIO_FRM_LPM		(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_SSP_AUDIO_TXD_OFFSET       (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_SSP_AUDIO_TXD_AF           (XLLP_MFP_PIN_GPIOXX_AF_DSSP3_TX)
#define XLLP_MFP_SSP_AUDIO_TXD_LPM		(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_SSP_AUDIO_RXD_OFFSET       (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_SSP_AUDIO_RXD_AF           (XLLP_MFP_PIN_GPIOXX_AF_DSSP3_RX)
#define XLLP_MFP_SSP_AUDIO_RXD_LPM		(XLLP_MFP_LPMO_PULL_LOW)

//FIXME
#define XLLP_MFP_RSVD_SSP_4_CLK_OFFSET      (XLLP_MFP_PIN_GPIO93_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_CLK_AF          (XLLP_MFP_ALT_FN_2)
#define XLLP_MFP_RSVD_SSP_4_CLK_LPM	(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_RSVD_SSP_4_FRM_OFFSET      (XLLP_MFP_PIN_GPIO94_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_FRM_AF          (XLLP_MFP_ALT_FN_2)
#define XLLP_MFP_RSVD_SSP_4_FRM_LPM 	(XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_RSVD_SSP_4_TXD_OFFSET      (XLLP_MFP_PIN_GPIO90_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_TXD_AF          (XLLP_MFP_ALT_FN_2)
#define XLLP_MFP_RSVD_SSP_4_TXD_LPM			(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_RSVD_SSP_4_RXD_OFFSET      (XLLP_MFP_PIN_GPIO89_OFFSET)
#define XLLP_MFP_RSVD_SSP_4_RXD_AF          (XLLP_MFP_ALT_FN_2)
#define XLLP_MFP_RSVD_SSP_4_RXD_LPM		(XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_IR_SHDN_N_GPIO_OFFSET      (XLLP_MFP_PIN_GPIO78_OFFSET)
#define XLLP_MFP_IR_SHDN_N_GPIO_AF          (XLLP_MFP_PIN_GPIO78_AF_GPIO_78)


//Fixme need further check
#define XLLP_MFP_USB_OTG_SR_OFFSET          (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_USB_OTG_SR_AF              (XLLP_MFP_PIN_GPIOXX_AF_USB_P2_4)
#define XLLP_MFP_USB_OTG_SR_LPM            (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_OTG_EN_OFFSET              (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_OTG_EN_AF                  (XLLP_MFP_PIN_GPIOXX_AF_USB_P2_8)
#define XLLP_MFP_OTG_EN_LPM                (XLLP_MFP_LPMO_PULL_HIGH)

//#define XLLP_MFP_CIF_HI_PWDN_GPI0_OFFSET    (XLLP_MFP_PIN_GPIO77_OFFSET)
//#define XLLP_MFP_CIF_HI_PWDN_GPI0_AF        (XLLP_MFP_PIN_GPIO77_AF_GPIO_77)
//#define XLLP_MFP_CIF_HI_PWDN_GPI0_LPM             (XLLP_MFP_LPMO_PULL_HIGH)

//#define XLLP_MFP_CIF_LO_PWDN_GPI0_OFFSET    (XLLP_MFP_PIN_GPIO78_OFFSET)
//#define XLLP_MFP_CIF_LO_PWDN_GPI0_AF        (XLLP_MFP_PIN_GPIO78_AF_GPIO_78)
//#define XLLP_MFP_CIF_LO_PWDN_GPI0_LPM             (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_OTG_ID_OFFSET              (XLLP_MFP_PIN_GPIOXX_OFFSET)
#define XLLP_MFP_OTG_ID_AF                  (XLLP_MFP_PIN_GPIOXX_AF_USB_P2_7)
#define XLLP_MFP_OTG_ID_LPM	           (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_STD_TXD_OFFSET         (XLLP_MFP_PIN_GPIO46_OFFSET)
#define XLLP_MFP_STD_TXD_AF             (XLLP_MFP_PIN_GPIO46_AF_STTXD)
#define XLLP_MFP_STD_TXD_LPM             (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_STD_RXD_OFFSET         (XLLP_MFP_PIN_GPIO45_OFFSET)
#define XLLP_MFP_STD_RXD_AF             (XLLP_MFP_PIN_GPIO45_AF_STRXD)
#define XLLP_MFP_STD_RXD_LPM             (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_BT_RTS_OFFSET     (XLLP_MFP_PIN_GPIO94_OFFSET)
#define XLLP_MFP_BT_RTS_AF       (XLLP_MFP_PIN_GPIO94_AF_BTRTS_N)
#define XLLP_MFP_BT_RTS_LPM             (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_BT_RXD_OFFSET     (XLLP_MFP_PIN_GPIO91_OFFSET)
#define XLLP_MFP_BT_RXD_AF       (XLLP_MFP_PIN_GPIO91_AF_BTRXD)
#define XLLP_MFP_BT_RXD_LPM             (XLLP_MFP_LPMO_FLOAT)

#define XLLP_MFP_BT_TXD_OFFSET     (XLLP_MFP_PIN_GPIO92_OFFSET)
#define XLLP_MFP_BT_TXD_AF       (XLLP_MFP_PIN_GPIO92_AF_BTTXD)
#define XLLP_MFP_BT_TXD_LPM             (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_BT_CTS_OFFSET     (XLLP_MFP_PIN_GPIO93_OFFSET)
#define XLLP_MFP_BT_CTS_AF       (XLLP_MFP_PIN_GPIO93_AF_BTCTS_N)
#define XLLP_MFP_BT_CTS_LPM             (XLLP_MFP_LPMO_FLOAT)


//fixme need further check
#define XLLP_MFP_ONE_WIRE_OFFSET        (XLLP_MFP_PIN_GPIO16_OFFSET)
#define XLLP_MFP_ONE_WIRE_AF            (XLLP_MFP_PIN_GPIO16_AF_SMART_BAT)

//TBD need further check
#define XLLP_MFP_RSVD_GPIO_1_OFFSET     0xff
#define XLLP_MFP_RSVD_GPIO_1_AF             0xff
#define XLLP_MFP_RSVD_GPIO_1_LPM	    0xff

#define XLLP_MFP_USBHPEN_OFFSET             0xff
#define XLLP_MFP_USBHPEN_AF                     0xff
#define XLLP_MFP_USBHPEN_LPM                   0xff

#define XLLP_MFP_USBHPWR_OFFSET            0xff
#define XLLP_MFP_USBHPWR_AF                    0xff
#define XLLP_MFP_USBHPWR_LPM                  0xff


#define XLLP_MFP_KP_MKOUT_7_LPM         (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_L_DD_0_LPM         (XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_L_DD_1_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_2_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_3_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_4_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_5_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_6_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_DD_7_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_FCLK_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_LCLK_LPM         (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_L_PCLK_LPM         (XLLP_MFP_LPMO_PULL_LOW)

//#define XLLP_MFP_L_BIAS_OFFSET          0xFF
//#define XLLP_MFP_L_BIAS_AF              (XLLP_MFP_PIN_GPIO20_AF_LCD_BIAS)
//#define XLLP_MFP_L_BIAS_LPM         (XLLP_MFP_LPMO_PULL_LOW)






#define XLLP_MFP_SIR_TXD_OFFSET         (XLLP_MFP_STD_TXD_OFFSET)
#define XLLP_MFP_SIR_TXD_AF             (XLLP_MFP_STD_TXD_AF)
#define XLLP_MFP_SIR_TXD_LPM             (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_SIR_RXD_OFFSET         (XLLP_MFP_STD_RXD_OFFSET)
#define XLLP_MFP_SIR_RXD_AF             (XLLP_MFP_STD_RXD_AF)
#define XLLP_MFP_SIR_RXD_LPM             (XLLP_MFP_LPMO_FLOAT)

//fixme need further check
#define XLLP_MFP_DF_INT_RnB_LPM             (XLLP_MFP_LPMO_FLOAT)

// Note: the following 16 sets of constants are listed in numeric order
//      of [n] within XLLP_MFP_PIN_ND_IO[n]_OFFSET
#define XLLP_MFP_DF_IO_0_LPM              (XLLP_MFP_LPMO_PULL_HIGH)


#define XLLP_MFP_DF_IO_1_OFFSET         (XLLP_MFP_PIN_ND_IO1_OFFSET)
#define XLLP_MFP_DF_IO_1_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_2_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_3_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_4_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_5_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_6_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_7_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_8_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_9_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_10_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_11_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_12_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_13_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_14_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_IO_15_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_nADV1_LPM              (XLLP_MFP_LPMO_PULL_LOW)

#define XLLP_MFP_DF_nADV2_OFFSET        (XLLP_MFP_PIN_ND_nADV2_ALE_OFFSET)
#define XLLP_MFP_DF_nADV2_AF            (XLLP_MFP_ALT_FN_1)
#define XLLP_MFP_DF_nADV2_LPM              (XLLP_MFP_LPMO_PULL_LOW)


#define XLLP_MFP_DF_NCS0_OFFSET         (XLLP_MFP_PIN_ND_nCS0_OFFSET)
#define XLLP_MFP_DF_NCS0_AF             (XLLP_MFP_PIN_ND_nCS0_AF_ND_nCS0)
#define XLLP_MFP_DF_NCS0_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_NCS1_OFFSET         (XLLP_MFP_PIN_ND_nCS1_OFFSET)
#define XLLP_MFP_DF_NCS1_AF             (XLLP_MFP_ALT_FN_1)
#define XLLP_MFP_DF_NCS1_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_nRE_OFFSET          (XLLP_MFP_PIN_ND_nRE_nOE_OFFSET)
#define XLLP_MFP_DF_nRE_AF              (XLLP_MFP_PIN_ND_nRE_nOE_AF_ND_nRE)
#define XLLP_MFP_DF_nRE_LPM              (XLLP_MFP_LPMO_PULL_HIGH)


#define XLLP_MFP_DF_nWE_OFFSET          (XLLP_MFP_PIN_ND_nWE_OFFSET)
#define XLLP_MFP_DF_nWE_AF              (XLLP_MFP_PIN_ND_nWE_AF_ND_nWE)
#define XLLP_MFP_DF_nWE_LPM              (XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_DF_SCLK_E_OFFSET       (XLLP_MFP_PIN_DF_SCLK_E_OFFSET)
#define XLLP_MFP_DF_SCLK_E_AF           (XLLP_MFP_PIN_DF_SCLK_E_AF_DF_SCLK_E)

#define XLLP_MFP_DF_SCLK_S_OFFSET       (XLLP_MFP_PIN_DF_SCLK_S_OFFSET)
#define XLLP_MFP_DF_SCLK_S_AF           (XLLP_MFP_PIN_DF_SCLK_S_AF_DF_SCLK_S)

#define XLLP_MFP_DEBUG_ETH_nBE0_AF      (XLLP_MFP_PIN_nBE0_AF_nBE0)

#define XLLP_MFP_DEBUG_ETH_nBE1_OFFSET  (XLLP_MFP_PIN_nBE1_OFFSET)
#define XLLP_MFP_DEBUG_ETH_nBE1_AF      (XLLP_MFP_PIN_nBE1_AF_nBE1)

#define XLLP_MFP_ND_CLE_LPM              (XLLP_MFP_LPMO_PULL_LOW)



#define XLLP_MFP_RSVD_nXCVREN_OFFSET    (XLLP_MFP_PIN_nXCVREN_OFFSET)
#define XLLP_MFP_RSVD_nXCVREN_AF        (XLLP_MFP_ALT_FN_5)
//#define XLLP_MFP_nXCVREN_AF     (XLLP_MFP_PIN_nXCVREN_AF_nXCVREN)

#define XLLP_MFP_ULPI_DATA0_OFFSET			(XLLP_MFP_PIN_GPIO31_OFFSET)
#define XLLP_MFP_ULPI_DATA0_AF				(XLLP_MFP_PIN_GPIO31_AF_UTM_PHYDATA_OUT_0)

#define XLLP_MFP_ULPI_DATA1_OFFSET			(XLLP_MFP_PIN_GPIO30_OFFSET)	
#define XLLP_MFP_ULPI_DATA1_AF				(XLLP_MFP_PIN_GPIO30_AF_UTM_PHYDATA_OUT_1)

#define XLLP_MFP_ULPI_DATA2_OFFSET			(XLLP_MFP_PIN_GPIO33_OFFSET)
#define XLLP_MFP_ULPI_DATA2_AF				(XLLP_MFP_PIN_GPIO33_AF_UTM_PHYDATA_OUT_2)

#define XLLP_MFP_ULPI_DATA3_OFFSET			(XLLP_MFP_PIN_GPIO34_OFFSET)
#define XLLP_MFP_ULPI_DATA3_AF				(XLLP_MFP_PIN_GPIO34_AF_UTM_PHYDATA_OUT_3)				

#define XLLP_MFP_ULPI_DATA4_OFFSET			(XLLP_MFP_PIN_GPIO35_OFFSET)
#define XLLP_MFP_ULPI_DATA4_AF				(XLLP_MFP_PIN_GPIO35_AF_UTM_PHYDATA_OUT_4)

#define XLLP_MFP_ULPI_DATA5_OFFSET			(XLLP_MFP_PIN_GPIO36_OFFSET)
#define XLLP_MFP_ULPI_DATA5_AF				(XLLP_MFP_PIN_GPIO36_AF_UTM_PHYDATA_OUT_5)

#define XLLP_MFP_ULPI_DATA6_OFFSET			(XLLP_MFP_PIN_GPIO41_OFFSET)
#define XLLP_MFP_ULPI_DATA6_AF				(XLLP_MFP_PIN_GPIO41_AF_UTM_PHYDATA_OUT_6)

#define XLLP_MFP_ULPI_DATA7_OFFSET			(XLLP_MFP_PIN_GPIO42_OFFSET)
#define XLLP_MFP_ULPI_DATA7_AF				(XLLP_MFP_PIN_GPIO42_AF_UTM_PHYDATA_OUT_7)

#define XLLP_MFP_ULPI_CLK_OFFSET			(XLLP_MFP_PIN_GPIO38_OFFSET)        
#define XLLP_MFP_ULPI_CLK_AF				(XLLP_MFP_PIN_GPIO38_AF_UTM_CLK)

#define XLLP_MFP_PIN_ULPI_STP_OFFSET 			(XLLP_MFP_PIN_GPIO39_OFFSET)
#define XLLP_MFP_ULPI_STP_AF				(XLLP_MFP_PIN_GPIO39_AF_STP)

#define XLLP_MFP_PIN_ULPI_DIR_OFFSET 			(XLLP_MFP_PIN_GPIO37_OFFSET)
#define XLLP_MFP_ULPI_DIR_AF				(XLLP_MFP_PIN_GPIO37_AF_DIR)

#define XLLP_MFP_PIN_ULPI_NXT_OFFSET 			(XLLP_MFP_PIN_GPIO40_OFFSET)
#define XLLP_MFP_ULPI_NXT_AF				(XLLP_MFP_PIN_GPIO40_AF_NXT)

#define XLLP_MFP_ULPI_RESET_OFFSET		(XLLP_MFP_PIN_GPIO43_OFFSET)
#define XLLP_MFP_ULPI_RESET_AF                    (XLLP_MFP_ALT_FN_0)

#define XLLP_MFP_ULPI_REFCLK_OFFSET 		(XLLP_MFP_PIN_DF_ADDR3_OFFSET)
#define XLLP_MFP_ULPI_REFCLK_AF                    (XLLP_MFP_ALT_FN_3)

//To enable Keypad
//#define XLLP_MFP_XCVR_SELECT_OFFSET (XLLP_MFP_PIN_GPIO43_OFFSET)
#define XLLP_MFP_XCVR_SELECT_OFFSET   0xff
#define XLLP_MFP_XCVR_SELECT_AF		(XLLP_MFP_PIN_GPIO43_AF_USB_HS_XCVR_SELECT)
#define XLLP_MFP_XCVR_SELECT_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

//To enable keypad
//#define XLLP_MFP_TERM_SELECT_OFFSET (XLLP_MFP_PIN_GPIO44_OFFSET)
#define XLLP_MFP_TERM_SELECT_OFFSET     0xff
#define XLLP_MFP_TERM_SELECT_AF		(XLLP_MFP_PIN_GPIO44_AF_USB_HS_TERM_SELECT)
#define XLLP_MFP_TERM_SELECT_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_SUSPENDM_X_OFFSET  (XLLP_MFP_PIN_GPIO45_OFFSET)
#define XLLP_MFP_SUSPENDM_X_AF		(XLLP_MFP_PIN_GPIO45_AF_USB_HS_SUSPENDM_X)
#define XLLP_MFP_SUSPENDM_X_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_LINESTATE0_OFFSET  (XLLP_MFP_PIN_GPIO99_OFFSET)
#define XLLP_MFP_LINESTATE0_AF		(XLLP_MFP_PIN_GPIO99_AF_USB_HS_LINESTATE_0)
#define XLLP_MFP_LINESTATE0_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_LINESTATE1_OFFSET  (XLLP_MFP_PIN_GPIO100_OFFSET)
#define XLLP_MFP_LINESTATE1_AF		(XLLP_MFP_PIN_GPIO100_AF_USB_HS_LINESTATE_1)
#define XLLP_MFP_LINESTATE1_LPM			(XLLP_MFP_LPMO_PULL_HIGH)


#define XLLP_MFP_TXVALID_OFFSET 	(XLLP_MFP_PIN_GPIO104_OFFSET)
#define XLLP_MFP_TXVALID_AF			(XLLP_MFP_PIN_GPIO104_AF_USB_HS_TXVALID)
#define XLLP_MFP_TXVALID_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_TXREADY_OFFSET		(XLLP_MFP_PIN_GPIO105_OFFSET)
#define XLLP_MFP_TXREADY_AF			(XLLP_MFP_PIN_GPIO105_AF_USB_HS_TXREADY)
#define XLLP_MFP_TXREADY_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

//Firstly, disable USB2.0, to enable keypad
//#define XLLP_MFP_RXVALID_OFFSET 	(XLLP_MFP_PIN_GPIO10_OFFSET)
#define XLLP_MFP_RXVALID_OFFSET 	      0xff
#define XLLP_MFP_RXVALID_AF			(XLLP_MFP_PIN_GPIO10_AF_USB_HS_RXVALID)
#define XLLP_MFP_RXVALID_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

//#define XLLP_MFP_RXACTIVE_OFFSET	(XLLP_MFP_PIN_GPIO11_OFFSET)
#define XLLP_MFP_RXACTIVE_OFFSET	      0xff
#define XLLP_MFP_RXACTIVE_AF		      (XLLP_MFP_PIN_GPIO11_AF_USB_HS_RXACTIVE)
#define XLLP_MFP_RXACTIVE_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

#define XLLP_MFP_RXERROR_OFFSET		(XLLP_MFP_PIN_GPIO41_OFFSET)
#define XLLP_MFP_RXERROR_AF			(XLLP_MFP_PIN_GPIO41_AF_USB_HS_RXERROR)
#define XLLP_MFP_RXERROR_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

//Firstly, disable USB2.0, to enable keypad
//#define XLLP_MFP_OPMODE0_OFFSET 	(XLLP_MFP_PIN_GPIO8_OFFSET)
#define XLLP_MFP_OPMODE0_OFFSET 	      0xff
#define XLLP_MFP_OPMODE0_AF			(XLLP_MFP_PIN_GPIO8_AF_USB_HS_OPMODE_0)
#define XLLP_MFP_OPMODE0_LPM			(XLLP_MFP_LPMO_PULL_HIGH)

//#define XLLP_MFP_OPMODE1_OFFSET 	(XLLP_MFP_PIN_GPIO9_OFFSET)
#define XLLP_MFP_OPMODE1_OFFSET 	      0xff
#define XLLP_MFP_OPMODE1_AF			(XLLP_MFP_PIN_GPIO9_AF_USB_HS_OPMODE_1)
#define XLLP_MFP_OPMODE1_LPM			(XLLP_MFP_LPMO_PULL_HIGH)
//
//===================================
//
// End of Platform MFP Values and Alternate Function for Each Pin
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
// Beginning of Platform Drive Strength Values
//

//
// Note: The actual best values for the Zylonite platform are not known.
//       These values are based on silicon team estimates for certain
//       voltages in ideal circumstances and the known voltage domains
//       on the Zylonite.
//

#define XLLP_MFP_DEFAULT_DS         (XLLP_MFP_DS_03X)
#define XLLP_MFP_USB2_DS            (XLLP_MFP_DS_08X)
#define XLLP_MFP_CIF_DS             (XLLP_MFP_DS_04X)
#define XLLP_MFP_LCD_DS             (XLLP_MFP_DS_01X)
#define XLLP_MFP_SSP_DS             (XLLP_MFP_DS_04X)
#define XLLP_MFP_MSL_DEFAULT_DS     (XLLP_MFP_DS_04X) //3.0V: 3 pins
#define XLLP_MFP_MSL_SPECIAL_DS     (XLLP_MFP_DS_08X) //1.8V: 1 pin
#define XLLP_MFP_ND_DATA_DS         (XLLP_MFP_DS_08X)
#define XLLP_MFP_ND_NON_DATA_DS     (XLLP_MFP_DS_10X)
#define XLLP_MFP_AC97_DS            (XLLP_MFP_DS_04X)
#define XLLP_MFP_MMC2_DS            (XLLP_MFP_DS_08X)

//
// Beginning of Platform Drive Strength Values
//
//////////////////////////////////////////////////////////////////////




#endif // __XLLP_MFP_PLAT_H__


