/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */
/******************************************************************************
**
**  FILENAME:       xllp_gpio_plat.h
**
**  PURPOSE: Contains all hardware platform-specific GPIO macros, typedefs, 
**              and prototypes.  Processor-level GPIO definitions are found
**              in xllp_gpio_proc.h
**           Declares no storage.
**                  
**  Validity Notes: Only valid for the processor code named Monahans and the
**                  Zylonite platform.
**
******************************************************************************/

#ifndef __XLLP_GPIO_PLAT_H__
#define __XLLP_GPIO_PLAT_H__

#include "xllp_defs.h"


// Begin of Platform GPIO symbolic IDs for each supported ID 

// Any unassigned GPIO IDs are not attached to any pin as an
//  alternate function on this platform.

#if defined(CONFIG_CPU_MONAHANS_P)
#define XLLP_GPIO_MMC_CD_0           1 
#define XLLP_GPIO_DEBUG_ETH_CS_N     3 
#define XLLP_GPIO_MMC_CD_1           4 
#define XLLP_GPIO_MMC_WP_0_N         5 
#define XLLP_GPIO_MMC_WP_1_N         6 
#define XLLP_GPIO_PHONE_CLOSED       7 
#define XLLP_GPIO_PHONE_FLIPPED      8 
#define XLLP_GPIO_DEBUG_ETH_INT      9 
#define XLLP_GPIO_AC97_INT_N        15
#define XLLP_GPIO_CAMERA_STROBE_EN  16
#define XLLP_GPIO_CAMERA_LIGHT_EN   17
#define XLLP_GPIO_MMC_CMD_0			23
#define XLLP_GPIO_MMC_CMD_1			31
#define XLLP_GPIO_MMC2_CMD			29
#define XLLP_GPIO_IR_SHDN_N         97
#define XLLP_GPIO_CFCD 				30
#define XLLP_GPIO_NIRQ 				98
#define XLLP_GPIO_CAMERA_HI_PWDN    102
#define XLLP_GPIO_CAMERA_LO_PWDN    103
#define XLLP_GPIO_CIR_OUT			13
#else

#if defined(CONFIG_CPU_MONAHANS_PL)
#define XLLP_GPIO_DEBUG_ETH_CS_N     3 
#define XLLP_GPIO_DEBUG_ETH_INT      97 
#define XLLP_GPIO_AC97_INT_N        36
#define XLLP_GPIO_CAMERA_STROBE_EN  16
#define XLLP_GPIO_MMC_CMD_0			23
#define XLLP_GPIO_MMC2_CMD			29
#define XLLP_GPIO_MMC_CMD_1			31
#define XLLP_GPIO_CIR_OUT			15
#define XLLP_GPIO_EXP_0_N			12
#define XLLP_GPIO_EXP_1_N			13
#else // L and LV
#define XLLP_GPIO_DEBUG_ETH_CS_N    1
#define XLLP_GPIO_DEBUG_ETH_INT     99 
#define XLLP_GPIO_AC97_INT_N        26
#define XLLP_GPIO_CAMERA_STROBE_EN 	126
#define XLLP_GPIO_MMC_CMD_0			8
#define XLLP_GPIO_MMC2_CMD			14
#define XLLP_GPIO_MMC_CMD_1			15
#define XLLP_GPIO_CIR_OUT			16
#define XLLP_GPIO_EXP_0_N			18
#define XLLP_GPIO_EXP_1_N			19
#endif // #ifdef CONFIG_CPU_MONAHANS_PL

#define XLLP_GPIO_MMC_CD_0          128
#define XLLP_GPIO_MMC_CD_1          129 
#define XLLP_GPIO_MMC_WP_0_N        130
#define XLLP_GPIO_MMC_WP_1_N        131
#define XLLP_GPIO_PHONE_FLIPPED     132
#define XLLP_GPIO_PHONE_CLOSED      133
#define XLLP_GPIO_USB2_DETECT		134
#define XLLP_GPIO_CFCD				135

#define XLLP_GPIO_CAMERA_LIGHT_EN	136
#define XLLP_GPIO_IR_SHDN_N			137
#define XLLP_GPIO_CAMERA_HI_PWDN	138
#define XLLP_GPIO_CAMERA_LO_PWDN	139
#define XLLP_GPIO_UTMI_TEST_EN		140
#define XLLP_GPIO_UTMI_SWITCH		141
#define	XLLP_GPIO_USB_OTG_EN	    142
#define	XLLP_GPIO_USB_OTG_SR	    143

#define	XLLP_GPIO_GPIO_A			144
#define	XLLP_GPIO_GPIO_B			145
#define	XLLP_GPIO_GPIO_C			146
#define	XLLP_GPIO_GPIO_D			147
#define	XLLP_GPIO_GPIO_E			148
#define	XLLP_GPIO_GPIO_F			149
#define	XLLP_GPIO_GPIO_G			150
#define	XLLP_GPIO_GPIO_H			151

#define	XLLP_GPIO_GPIO_I			152
#define	XLLP_GPIO_MB_SWITCH_A		153
#define	XLLP_GPIO_MB_SWITCH_B		154
#define	XLLP_GPIO_ULPI_RESET		155
#define	XLLP_GPIO_DMARQ				156
#define	XLLP_GPIO_NIRQ				157
#define	XLLP_GPIO_RESERVED1			158
#define	XLLP_GPIO_RESERVED2			159



//#define XLLP_GPIO_CFCD			XLLP_GPIO_CF_CD_N
//#define XLLP_GPIO_NIRQ 			98

#endif // #ifdef CONFIG_CPU_MONAHANS_P

#if defined (CONFIG_CPU_MONAHANS_P) || defined (CONFIG_CPU_MONAHANS_PL)

// usb 2.0 detection
#define XLLP_GPIO_USB_VBUS			46

// for LCD panel id detection
#define XLLP_GPIO_L_VSYNC	74
#define XLLP_GPIO_L_CS		73
#define XLLP_GPIO_L_FCLK	14
#define XLLP_GPIO_L_LCLK	15
#define XLLP_GPIO_L_BIAS	17
#define XLLP_GPIO_L_DD_16	71
#define XLLP_GPIO_L_DD_17	72

// use PWM pin as gpio
#define XLLP_GPIO_BACKLIGHT	14

#else

#define XLLP_GPIO_USB_VBUS			35

#define XLLP_GPIO_L_VSYNC	76
#define XLLP_GPIO_L_CS		127
#define XLLP_GPIO_L_FCLK	72
#define XLLP_GPIO_L_LCLK	73
#define XLLP_GPIO_L_BIAS	75
#define XLLP_GPIO_L_DD_16	70
#define XLLP_GPIO_L_DD_17	71
#define XLLP_GPIO_BACKLIGHT	20

#endif

// End of platform GPIO symbolic IDs

#endif // __XLLP_GPIO_PLAT_H__



