/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**
**  FILENAME:       xllp_2d_gcu.h
**
**  PURPOSE: contains all 2D Graphics Controller specific macros, typedefs, 
**           and prototypes. Declares no storage.
**
******************************************************************************/

#ifndef __XLLP_2D_GCU_H__
#define __XLLP_2D_GCU_H__

#include "xllp_defs.h"

/*****************************************************************************
 * Register defines
 */

/* 
 * Graphics Controller Configuration Register (GCCR)
 */

#define GCCR_BP_RST		(1U<<8) /* Breatpoint Reset */
#define GCCR_ABORT		(1U<<6)	/* Abort all current activity in te 2D Graphics Engine */
#define GCCR_STOP		(1U<<4)	/* Halt the 2D Graphics Engine */

#define GCCR_MASK		(0x0000075F)

/* 
 * Graphics Controller Interrupt Status Controller Register(GCISR) 
 */

#define GCISCR_FLAG_INTST	(1U<<8) /* Graphics Controller GCFLAGS Register Interrupt status */
#define GCISCR_STOP_INTST	(1U<<7) /* Graphics Controller GCCR:Stop Response interrupt status */
#define GCISCR_PF_INST		(1U<<6) /* Graphics Controller Illegal/Incompatible Pixel Format, Step Size interrupt Status */
#define GCISCR_EEOB_INTST	(1U<<5)	/* Graphics Controller Execution End of Buffer interrupt stauts */
#define GCISCR_IIN_INTST	(1U<<4)	/* Graphics Controller Illegal Instruction Interrupt Status */
#define GCISCR_IOP_INTST	(1U<<3)	/* Graphics Controller Illegal Operation Interrupt Status */
#define GCISCR_BF_INTST		(1U<<2)	/* Graphics Controller Display Buffer Flip Interrupt Status */
#define GCISCR_IN_INTST		(1U<<1)	/* Graphics Controller Instruction Interrup Status */
#define GCISCR_EOB_INTST	(1U<<0)	/* Graphics COntroller End of Buffer (Ring buffer HEAD = Ring Buffer TAIL) Interrupt Status */

/*
 * Graphics Controller Interrupt Enable Control Register (GCIECR) 
 */
#define GCIECR_FLAG_INTEN	(1U<<8) /* Graphics Controller GCFLAGS Register Interrupt Enable */
#define GCIECR_STOP_INTEN	(1U<<7) /* Graphics Controller GCCR:Stop Response interrupt Enable */
#define GCIECR_PF_INEN		(1U<<6) /* Graphics Controller Illegal/Incompatible Pixel Format, Step Size interrupt Enable */
#define GCIECR_EEOB_INTEN	(1U<<5)	/* Graphics Controller Execution End of Buffer Interrupt Enable */
#define GCIECR_IIN_INTEN	(1U<<4)	/* Graphics Controller Illegal Instruction Interrupt Enable */
#define GCIECR_IOP_INTEN	(1U<<3)	/* Graphics Controller Illegal Operation Interrupt Eanble */
#define GCIECR_BF_INTEN		(1U<<2)	/* Graphics Controller Display Buffer Flip Interrupt Enable */
#define GCIECR_IN_INTEN		(1U<<1)	/* Graphics Controller Instruction Interrup Enable */
#define GCISCR_EOB_INTEN	(1U<<0)	/* Graphics COntroller End of Buffer (Ring buffer HEAD = Ring Buffer TAIL) Interrupt Status */

/*
 * Graphics Controller ALU Flags Register (GCFLAGS)
 */

#define GCFLAGS_ZERO_R		(1U<<12) /* A calculated Red value was under flowed and has been clipped to 0x0000 */
#define GCFLAGS_SAT_G		(1U<<9)	 /* A calculated Green value was saturated and has been clipped to 0xFFFF */
#define GCFLAGS_ZOAR_G		(1U<<8)	 /* A calculated Green value was under flowed and has been clipped to 0x0000 */
#define GCFLAGS_SAT_B		(1U<<5)	 /* A calculated Blue value was saturated and has been clipped to 0xFFFF */
#define GCFLAGS_ZOAR_B		(1U<<4)	 /* A calculated Blue value was under flowed and has been clipped to 0x0000 */
#define GCFLAGS_SAT_A		(1U<<1)	 /* A calculated Alpha value was saturated and has been clipped to 0xFFFF */
#define GCFLAGS_ZOAR_A		(1U<<0)	 /* A calculated Alpha value was under flowed and has been clipped to 0x0000 */

/*****************************************************************************
 * Data structure and macro defines
 */

#define GCU_SYNCHRONOUS_RETURN 		0
#define GCU_ASYNCHRONOUS_RETURN 	1

#define XLLP_BUFFER_NOT_PHYSICALLY_CONTIGUOUS	0
#define XLLP_BUFFER_IS_PHYSICALLY_CONTIGUOUS	1

typedef struct 
{
	/* Miscellaneous Control and Interrupt Information */
	XLLP_VUINT32_T gccr;	/* Configuration Register */
	XLLP_VUINT32_T gciscr; 	/* Interrupt Status Control Register */
	XLLP_VUINT32_T gciecr; 	/* Interrupt Enable Control Register */
	XLLP_VUINT32_T gcnopid;	/* NOP ID From Instruction Stream Register */
	XLLP_VUINT32_T gcalphaset; /* Default Alpha value Control Register */
	XLLP_VUINT32_T gctset; 	/* Default Transparecy Value Control Register */
	XLLP_VUINT32_T gcflags;	/* ALU Ooperations Flags Status Control Register */

	/* Reserved 0x5400001C */
	XLLP_VUINT32_T res0[1];

	/* Ring Buffer Information */
	XLLP_VUINT32_T gcrbbr;	/* Ring Buffer Base Address Register */
	XLLP_VUINT32_T gcrblr;	/* Ring Buffer Length Register */
	XLLP_VUINT32_T gcrbhr;	/* Ring Buffer Head Register */
	XLLP_VUINT32_T gcrbtr;	/* Ring Buffer Tail Register */
	XLLP_VUINT32_T gcrbexhr; /* Ring Buffer Execution Head Register */

	/* Reserved 0x54000034-0x5400003C */
	XLLP_VUINT32_T res1[3];
	
	/* Batch Buffer Information */
	XLLP_VUINT32_T gcbbbr;	/* Batch Buffer Base Address Register */
	XLLP_VUINT32_T gcbbhr;	/* Batch Buffer Head Register */
	XLLP_VUINT32_T gcbbexhr; /* Batch Buffer Execution Head Register */

	/* Reserved 0x5400004C - 0x5400005C */
	XLLP_VUINT32_T res2[5];

	/* Destination 0 Information */
	XLLP_VUINT32_T gcd0br;	/* Destination 0 Base Address Register */
	XLLP_VUINT32_T gcd0stp;	/* Destination 0 Step Size Register */
	XLLP_VUINT32_T gcd0str;	/* Destination 0 Stride Size Register */
	XLLP_VUINT32_T gcd0pf;	/* Destination 0 Pixel Type Register */

	/* Destination 1 Information */
	XLLP_VUINT32_T gcd1br;	/* Destination 1 Base Address Register */
	XLLP_VUINT32_T gcd1stp;	/* Destination 1 Step Size Register */
	XLLP_VUINT32_T gcd1str;	/* Destination 1 Stride Size Register */
	XLLP_VUINT32_T gcd1pf;	/* Destination 1 Pixel Type Register */
	
	/* Destination 2 Information */
	XLLP_VUINT32_T gcd2br;  /* Destination 2 Base Address Register */
	XLLP_VUINT32_T gcd2stp; /* Destination 2 Step Size Register */
	XLLP_VUINT32_T gcd2str; /* Destination 2 Stride Size Register */
	XLLP_VUINT32_T gcd2pf;	/* Destination 2 Pixel Type Register */
	
	/* Reserved 0x54000090-0x540000DC */
	XLLP_VUINT32_T res3[20];

	/* Source 0 Information */
	XLLP_VUINT32_T gcs0br;	/* Source 0 Base Address Register */
	XLLP_VUINT32_T gcs0stp;	/* Source 0 Step Size Register */
	XLLP_VUINT32_T gcs0str;	/* Source 0 Stride Size Register */
	XLLP_VUINT32_T gcs0pf;	/* Source 0 Pixel Type Register */

	/* Source 1 Information */
	XLLP_VUINT32_T gcs1br;	/* Source 1 Base Address Register */
	XLLP_VUINT32_T gcs1stp;	/* Source 1 Step Size Register */
	XLLP_VUINT32_T gcs1str;	/* Source 1 Stride Size Register */
	XLLP_VUINT32_T gcs1pf;	/* Source 1 Pixel Type Register */

	/* Resvered 0x54000100-0x5400015C */
	XLLP_VUINT32_T res4[24];

	/* Pixel ALU Scratch Registers */
	XLLP_VUINT32_T gcsc0wd0; /* Pixel ALU Scratch Register 0 Word 0 */
	XLLP_VUINT32_T gcsc0wd1; /* Pixel ALU Scratch Register 0 Word 1 */
	XLLP_VUINT32_T gcsc1wd0; /* Pixel ALU Scratch Register 1 Word 0 */
	XLLP_VUINT32_T gcsc1wd1; /* Pixel ALU Scratch Register 1 Word 1 */
	XLLP_VUINT32_T gcsc2wd0; /* Pixel ALU Scratch Register 2 Word 0 */
	XLLP_VUINT32_T gcsc2wd1; /* Pixel ALU Scratch Register 2 Word 1 */
	XLLP_VUINT32_T gcsc3wd0; /* Pixel ALU Scratch Register 3 Word 0 */
	XLLP_VUINT32_T gcsc3wd1; /* Pixel ALU Scratch Register 3 Word 1 */
	XLLP_VUINT32_T gcsc4wd0; /* Pixel ALU Scratch Register 4 Word 0 */
	XLLP_VUINT32_T gcsc4wd1; /* Pixel ALU Scratch Register 5 Word 1 */
	XLLP_VUINT32_T gcsc5wd0; /* Pixel ALU Scratch Register 5 Word 0 */
	XLLP_VUINT32_T gcsc5wd1; /* Pixel ALU Scratch Register 5 Word 1 */
	XLLP_VUINT32_T gcsc6wd0; /* Pixel ALU Scratch Register 6 Word 0 */
	XLLP_VUINT32_T gcsc6wd1; /* Pixel ALU Scratch Register 6 Word 1 */
	XLLP_VUINT32_T gcsc7wd0; /* Pixel ALU Scratch Register 7 Word 0 */
	XLLP_VUINT32_T gcsc7wd1; /* Pixel ALU Scratch Register 7 Word 1 */

} XLLP_GCU_REG_T, *P_XLLP_GCU_REG_T;

typedef struct
{
	unsigned long opcode;

	// Used to determine if the given source buffer is physically contiguous or not.
	// Set to XLLP_BUFFER_NOT_PHYSICALLY_CONTIGUOUS or
	// XLLP_BUFFER_IS_PHYSICALLY_CONTIGUOUS as appropriate.
	unsigned long source0_buffer_type;
	unsigned long source1_buffer_type;
	unsigned long dest0_buffer_type;
	unsigned long dest1_buffer_type;
	unsigned long dest2_buffer_type;

	union{
		struct {
			// Physical address of the batch buffer
		 	unsigned long start_addr;

		}bbst; // Batch Buffer Start (GC_BBST)

		struct {
			// Identifies which buffer this information pertains to.  Refer to // specification for details.
			unsigned long buff_addr;

			// The base address for the buffer ID specified in buff_addr.
			unsigned long base_addr;

			// The 4-bit pixel format for the buffer being programmed.
			unsigned long pixel_format;
			// The 14-bit stride size in bytes for the buffer being programmed.
			unsigned long stride;
			// The 4-bit step size in bytes for the buffer being programmed.
			unsigned long step;

		}buffi; // Buffer Info (GC_BUFFI)

		struct {
			// A flag to indicate that immediate data is included in the
			// instruction list.  Otherwise, the address for the data is 
			// included.
			unsigned long imm;
			// Indicates which scratch register to load
			unsigned long reg;
			// Indicates pixel format of the source data
			unsigned long pixel_format;
			// 32-bit data (or address) for word 0 and data for word 1
			unsigned long data[2];
		}lreg; // Load Register (GC_LREG)

		struct {
			// Indicates which scratch register source for the store
			unsigned long reg;

			// Indicates pixel format of the destination data
			unsigned long pixel_format;
			// Indicates 32-bit destination address for the store
			unsigned long addr;
		}streg; // Store Register (GC_STREG)

		struct {
			// 20-bit NOP ID
			unsigned long nop_id;
		}nop; // NOP (GC_NOP)

		struct {
			// Determines which destination buffer to flip to.  See 
			// specifications for details.
			unsigned long mode;
		}dbflip; // Destination Buffer Flip Immediate (GC_DBFLIP)

		struct {
			// 20-bit interrupt ID field
			unsigned long int_id;
		}intr; // Interrupt (GC_INT)

		struct {
			// Indicates which event to wait for.  See specification for 
			// details.
			unsigned long event_id;
		}wait; // Wait For Event (GC_WAIT)

		/* GC_CBLT <imm> DX0, DY0, width, height, reg, color */
		struct {
			// Color value is specified directly or through a register
			unsigned long imm;
			// Immediate color pixel format
			unsigned long pixel_format;
			// Starting pixel number inside destination buffer
			unsigned long dx0;
			// Starting line number of destination buffer
			unsigned long dy0; 
			// Pixel width of the block
			unsigned long width;
			// Pixel height of the block
			unsigned long height;
			// Selects which register to use for immediate value
			unsigned long reg;
			// Contains the first and second WORD of the color data.
			unsigned long data[2];
		}cfill; // Color Fill (GC_FILL) 

		/* GC_CKBLT <invert>, DX0, DY0, SX0, SY0, width, height, threshold, key */
		struct {
			unsigned long invert;
			unsigned long dx0;
			unsigned long dy0;
			unsigned long sx0;
			unsigned long sy0;
			unsigned short width;
			unsigned short height;
			unsigned long threshold;
			unsigned long key;
		}ckblt; // Chroma-Key (GC_CKBLT)

		/* GC_LINE <imm> DX0, DY0, X_is_1, Y_is_1, X_dir, Y_dir, DelX, DelY, Length, Width, Rn or Color */
		struct{
			unsigned long imm;
			unsigned long reg;
			unsigned long dx0;
			unsigned long dy0;
			unsigned char x_is_1;
			unsigned char y_is_1;
			unsigned char x_dir;
			unsigned char y_dir;
			unsigned long delx;
			unsigned long dely;
			unsigned short width;
			unsigned short length;
			unsigned long pixel_format;
			unsigned long data[2];
		}line; // Line Draw (GC_LINE) 

		/* GC_AALINE < imm> DX0, DY0, X_is_1, Y_is_1, X_dir, Y_dir, DelX, DelY, Length, Width, adresss, Rn or Color */
		struct {
			unsigned long imm;
			unsigned long reg;
			unsigned long dx0;
			unsigned long dy0;
			unsigned char x_is_1;
			unsigned char y_is_1;
			unsigned char x_dir;
			unsigned char y_dir;
			unsigned long delx;
			unsigned long dely;
			unsigned short width;
			unsigned short length;
			unsigned long address;
			unsigned long pixel_format;
			unsigned long data[2];
		} aaline; //  Anti-aliased Line Draw (GC_AALINE)

		/* GC_SCBLT <rop> DX0, DY0,SX0, SY0, Width, Height */
		struct {
			// Defines the Raster Operation to use
			unsigned long rop;
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long sx0;
			// Starting line number of source 0 block
			unsigned long sy0;
			//Pixel height of the block
			unsigned short width;
			// Pixel width of the block
			unsigned short height;
		}scblt; // Source Copy BLT (GC_SCBLT)

		/* GC_STRBLT DX0, DY0, SX0, SY0, s_width, s_height, d_width, d_height, X_str, Y_str */
		struct {
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long sx0;
			// Starting line number of source 0 block
			unsigned long sy0;
			// Pixel width of source block
			unsigned short s_width;
			// Pixel height of source block
			unsigned short s_height;
			// Pixel width of destination block
			unsigned short d_width;
			// Pixel height of destination block
			unsigned short d_height;
			// Stretch factor in X Direction
			unsigned long x_str;
			// Stretch factor in Y Direction
			unsigned long y_str;
		}strblt; // Stretch BLT (GC_STRBLT)

		/* GC_CKSTRBLT <invert>, DX0, DY0, S0X0, S0Y0, S1X0, S1Y0, Width, Height, threshold, key, X_Str, Y_Str */
		struct {
			unsigned long imm;
			unsigned long invert;
			unsigned long dx0;
			unsigned long dy0;
			unsigned long s0x0;
			unsigned long s0y0;
			unsigned long s1x0;
			unsigned long s1y0;
			unsigned short s_width;
			unsigned short s_height;
			unsigned short d_width;
			unsigned short d_height;
			unsigned long x_str;
			unsigned long y_str;
			unsigned long dest_val;
			unsigned short key;
			unsigned short threshold;
		}ckstrblt; // Chroma Key Stretch BLT (GC_CKSTRBLT)

		/* GC_ABLND <imm>, <single_alpha>,<blend_op>,DX0, DY0, S0X0, S0Y0 ,S1X0, S1Y0, width, height */
		struct {
			// Immediate value that describes where alpha values come from.
			unsigned long imm;
			// Asserted for single alpha, deasserted for dual alpha.
			unsigned long single_alpha;
			// Encoded for add, subtract, or reverse subtract
			unsigned long blend_op;
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long s0x0;
			// Starting line number of source 0 block
			unsigned long s0y0;
			// Starting pixel number of source 1 block
			unsigned long s1x0;
			// Starting line number of source 1 block
			unsigned long s1y0;
			// Pixel width of the block
			unsigned long width;
			// Pixel height of the block
			unsigned long height;
			// 16-bit Alpha for Source 0 (if immediate)
			unsigned short alpha0;
			// 16-bit Alpha for Source 1 (if immediate)
			unsigned short alpha1;
		}ablnd; // Alpha Blend (GC_ABLND)
	
		/* GC_SCALE <imm>,DX0, DY0, SX0, SY0, SCF */
		struct {
			// Determines location of scale factor
			unsigned long imm;
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long sx0;
			// Starting line number of source 0 block
			unsigned long sy0;
			// Pixel width of the block
			unsigned short width;
			// Pixel height of the block
			unsigned short height;
			// Scale factor if using immediate value
			unsigned long scf;
		}scale; // Scale BLT (GC_SCALE)

		/* GC_BIAS <imm>,DX0, DY0, SX0, SY0, Width, Height, BIAS */
		struct {
			// Determines location of the bias value
			unsigned long imm;
			// Starting pixel of the destination block
			unsigned long dx0;
			// Starting line of the destination block
			unsigned long dy0;
			// Starting pixel of the source 0 block
			unsigned long sx0;
			// Starting line of the source 0 block
			unsigned long sy0;
			// Pixel width of the block
			unsigned short width;
			// Pixel height of the block
			unsigned short height;
			// Bias factor if using immediate value
			unsigned long bias;
		}bias; // Bias BLT (GC_BIAS)

		/* GC_ROT <rot> DX0, DY0, SX0, SY0 */
		struct {
			// Rotation angle, 1 of 4
			unsigned long rot;
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long sx0;
			// Starting line number of source 0 block
			unsigned long sy0;
			// Pixel width of the block
			unsigned short width;
			// Pixel height of the block
			unsigned short height;
		}rot; // Rotate BLT (GC_ROT)

		/* GC_RAST <rop> DX0, DY0, S0X0, S0Y0, S1X0, S1Y0 */
		struct {
			// Raster Operation
			unsigned long rop;
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long s0x0;
			// Starting line number of source 0 block
			unsigned long s0y0;
			// Starting pixel number of source 1 block
			unsigned long s1x0;
			// Starting line number of source 1 block
			unsigned long s1y0;
			// Pixel width of the block
			unsigned short width;
			// Pixel height of the block
			unsigned short height;
		}rast; // Raster Operation (GC_RAST)

		/* GC_PATT <opt> DX0, DY0, SX0, SX0, S_width, S_height, D_width, D_height */
		struct {
			unsigned long opt;
			// Starting pixel number of destination block
			unsigned long dx0;
			// Starting line number of destination block
			unsigned long dy0;
			// Starting pixel number of source 0 block
			unsigned long sx0;
			// Starting line numbe of source 0 block
			unsigned long sy0;
			// Pixel width of the source block
			unsigned short s_width;
			// Pixel height of the source block
			unsigned short s_height;
			// Pixel width of the destination block
			unsigned short d_width;
			// Pixel height of the destination block
			unsigned short d_height;
		}patt;  // Pattern Copy BLT (GC_PATT)

		/* GC_DECBLT DX0, DY0, SX0, SY0, width, height, X_dec, Y_dec */
		struct {
			// Starting pixel of the destination block
			unsigned long dx0;
			// Starting line of the destination block
			unsigned long dy0;
			// Starting pixel of the source 0 block
			unsigned long sx0;
			// Starting line of the source 0 block
			unsigned long sy0;
			// Pixel width of the source block
			unsigned short s_width;
			// Pixel height of the source block
			unsigned short s_height;
			// Pixel width of the destination block
			unsigned short d_width;
			// Pixel height of the destination block
			unsigned short d_height;
			// Integer Decimation factor in the X direction
			unsigned short x_dec_int;
			// Fractional Decimation factor in the X direction
			unsigned short x_dec_frac;
			// Integer Decimation factor in the Y direction
			unsigned short y_dec_int;
			// Fractional Decimation factor in the Y direction
			unsigned short y_dec_frac;
		}decblt; // Decimate BLT (GC_DECBLT)

	} instruction;	

	// Instruction word count 
	int word_count;

} XLLP_GCU_BLITPARMS_T, *P_XLLP_GCU_BLITPARMS_T;

// int opcode  2D Graphics Instruction instruction
enum {
	/* Control and Memory Interface Instruction */
	GC_BBST = 0,
	GC_BBEND,
	GC_BUFFI,
	GC_LREG, //GC_SREG,  CATHY
	GC_NOP,
	GC_DBFLIP,
	GC_STREG,
	GC_INT,
	GC_WAIT,

	/* 2D Graphics Instructions */
	GC_CFILL,
	GC_CKBLT,
	GC_LINE,
	GC_AALINE,
	GC_SCBLT,
	GC_STRBLT,
	GC_CKSTRBLT,
	GC_ABLND,
	GC_SCALE,
	GC_BIAS,
	GC_ROT,
	GC_RAST,
	GC_PATT,
	GC_DECBLT
};

typedef struct
{
	// Number of the instructions 
	int num_instr;
	// Instructions Array 
	P_XLLP_GCU_BLITPARMS_T instructions;

} XLLP_GCU_INSTRUCTION_LIST_T, *P_XLLP_GCU_INSTRUCTION_LIST_T;


typedef struct
{
	XLLP_VUINT32_T *GCU;

	/* Ring Buffer */
	struct {
		/* Ring Buffer Base Virtual address */
		unsigned long addr;
		/* Ring Buffer Base Phsyical address */
		unsigned long base;
		/* Ring Buffer Head */
		unsigned long head;
		/* Ring Buffer Tail */
		unsigned long tail;
		/* Ring buffer Execute head */
		unsigned long exhead;
		/* Ring buffer Length */
		unsigned long length;
	}ring;

	/* Interrupt Mask */
	unsigned long interrupt_mask;

} XLLP_GCU_CONTEXT_T, *P_XLLP_GCU_CONTEXT_T;

/*****************************************************************************
 * XLLP Routines defines
 */

extern XLLP_STATUS_T XllpGCU_Init(XLLP_GCU_CONTEXT_T *pGCU);
extern XLLP_STATUS_T XllpGCU_Enable(XLLP_GCU_CONTEXT_T *pGCU);
extern XLLP_STATUS_T XllpGCU_Disable(XLLP_GCU_CONTEXT_T *pGCU);
extern XLLP_STATUS_T XllpGCU_SubmitInstruction(XLLP_GCU_CONTEXT_T *pGCU, XLLP_GCU_BLITPARMS_T *pBlitParms, XLLP_UINT32_T mode);
extern XLLP_STATUS_T XllpGCU_SubmitInstructionList(XLLP_GCU_CONTEXT_T *pGCU, XLLP_GCU_INSTRUCTION_LIST_T *pInstructionList, XLLP_UINT32_T mode);

#endif
