/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**  FILENAME:       xllp_irda_proc.h
**
**  PURPOSE:        Definitions for Monahans IrDA 
**
**
******************************************************************************/

#ifndef	__XLLP_IRDA_PROC_H__
#define	__XLLP_IRDA_PROC_H__

#include "xllp_defs.h"
#include "xllp_uart_proc.h"

/*
 ******************************************************************************
 *				CONSTANTS
 *
 ******************************************************************************
 */
typedef	enum
{
	XLLP_FIR = 0,
	XLLP_SIR
} XLLP_IRDATYPE_T;


/*
 ******************************************************************************
 *                            DATA TYPES 
 ******************************************************************************
 */

/*
 ******************************************************************************
 * 		FICP Registers
 * 	this type is used by FICP
 ******************************************************************************
 */
typedef	struct {
	XLLP_VUINT32_T iccr0;	// FICP Control Register 0
	XLLP_VUINT32_T iccr1;	// FICP Control Register 1
	XLLP_VUINT32_T iccr2;	// FICP Control Register 2
	XLLP_VUINT32_T icdr;	// FICP Data Register
	XLLP_VUINT32_T icsr0;	// FICP Status Register 0
	XLLP_VUINT32_T icsr1;	// FICP Status Register 1
	XLLP_VUINT32_T icfor;	// FICP FIFO Occupancy Status Register
} XLLP_FICP_T, *P_XLLP_FICP_T;

typedef struct {
	P_XLLP_FICP_T pFICPRegs;	// Pointer to the FICP registers
	P_XLLP_UINT_T pMfpRegBase;	// Pointer to the MFP base registers
	P_XLLP_GPIO_T	pGPIORegs;	// Pointer to the GPIO base registers
	P_XLLP_MFP_RM_DB_ID_T pMfpRmDb; // Pointer to the resource base
	XLLP_MFP_RM_DB_ID_T MfpRmDBID;	// The resource ID of FICP

	// User specified values to write to the FICP registers.
	XLLP_VUINT32_T iccr0_Value;	// Value to write to ICCR0
	XLLP_VUINT32_T iccr1_Value;	// Value to write to ICCR1
	XLLP_VUINT32_T iccr2_Value;	// Value to write to ICCR2
	XLLP_VUINT32_T icdr_Value;	// Value to write to ICDR
	XLLP_VUINT32_T icsr0_Value;	// Value to write to ICSR0
	XLLP_VUINT32_T icsr1_Value;	// Value to write to ICCR1
	XLLP_VUINT32_T icfor_Value;	// Value to write to ICFOR
} XLLP_FICP_HANDLE_T, *P_XLLP_FICP_HANDLE_T;

typedef	struct {
	XLLP_IRDATYPE_T	IrDAType;	// IrDA type is SIR or FIR ?
	P_XLLP_UART_HANDLE_T pUARTHandle; // Pointer to SIR handle
	P_XLLP_FICP_HANDLE_T pFICPHandle; // Pointer to FIR handle
} XLLP_IRDA_HANDLE_T, *P_XLLP_IRDA_HANDLE_T;


// Masks for ICCR0 Register.
#define	XLLP_ICCR0_ITR	XLLP_BIT_0
#define	XLLP_ICCR0_LBM	XLLP_BIT_1
#define	XLLP_ICCR0_TUS	XLLP_BIT_2
#define	XLLP_ICCR0_TXE	XLLP_BIT_3
#define	XLLP_ICCR0_RXE	XLLP_BIT_4
#define	XLLP_ICCR0_RIE	XLLP_BIT_5
#define	XLLP_ICCR0_TIE	XLLP_BIT_6
#define	XLLP_ICCR0_AME	XLLP_BIT_7

// Masks for ICCR2 Register.
#define	XLLP_ICCR2_TRIG	(XLLP_BIT_0 | XLLP_BIT_1)
#define	XLLP_ICCR2_TXP	XLLP_BIT_2
#define	XLLP_ICCR2_RXP	XLLP_BIT_3
#define	XLLP_ICCR2_TRAIL	XLLP_BIT_4
#define	XLLP_ICCR2_BUS	XLLP_BIT_5

// Masks for ICSR0 Register.
#define	XLLP_ICSR0_EIF	XLLP_BIT_0
#define	XLLP_ICSR0_TUR	XLLP_BIT_1
#define	XLLP_ICSR0_RAB	XLLP_BIT_2
#define	XLLP_ICSR0_TFS	XLLP_BIT_3
#define	XLLP_ICSR0_RFS	XLLP_BIT_4
#define	XLLP_ICSR0_FRE	XLLP_BIT_5
#define	XLLP_ICSR0_EOC	XLLP_BIT_6

// Masks for ICSR1 Register.
#define	XLLP_ICSR1_RSY	XLLP_BIT_0
#define	XLLP_ICSR1_TBY	XLLP_BIT_1
#define	XLLP_ICSR1_RNE	XLLP_BIT_2
#define	XLLP_ICSR1_TNF	XLLP_BIT_3
#define	XLLP_ICSR1_EOF	XLLP_BIT_4
#define	XLLP_ICSR1_CRE	XLLP_BIT_5
#define	XLLP_ICSR1_ROR	XLLP_BIT_6

#ifdef	__cplusplus
extern "C" {
#endif

XLLP_STATUS_T XllpSIRTxRxEnable(P_XLLP_IRDA_HANDLE_T Handle,
		XLLP_BOOL_T TxEnable, XLLP_BOOL_T RxEnable);
#ifdef	__cplusplus
}
#endif

#endif	//ifdef	__XLLP_IRDA_PROC_H__
