/*++
"This software program is available to you under a choice of one of two 
licenses.  You may choose to be licensed under either the GNU General Public 
License (GPL) Version 2, June 1991, available at 
http://www.fsf.org/copyleft/gpl.html, or the BSD License, the text of
which follows:

Copyright (c) 1996-2005, Intel Corporation. All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

Neither the name of the Intel Corporation ("Intel") nor the names of its 
contributors may be used to endorse or promote products derived from this
software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/
/******************************************************************************
**  FILENAME:       xllp_mfp_proc.h
**
**  PURPOSE: Contains all processor-level MFP (MultiFunction Pin) definitions.
**              Hardware platform-specific MFP macros, typedefs, 
**              and prototypes are found in xllp_mfp_plat.h
**           Declares no storage.
**                  
**  Validity Notes: Only valid for processor code named Tavor.
**
**
******************************************************************************/

#ifndef __XLLP_MFP_PROC_B0_H__
#define __XLLP_MFP_PROC_B0_H__
#include "xllp_defs.h"



///////////////////////////////////////////////
//
//  Processor-specific MFP constants start here.
//

/************************************************************************/

/**
  MFPR array Definition and other general definitions
**/


#define XLLP_MFP_PIN_MIN_OFFSET         0x0120      // Offset of lowest usable MFPR
#define XLLP_MFP_PIN_MAX_OFFSET         0x06B8      // Offset of highest usable MFPR
#define XLLP_MFP_PIN_EOLIST_MARKER      0xFFFFFFFF  // Marker val for end of pin lists
#define XLLP_MFP_PIN_MAX_SIZE           ((XLLP_MFP_PIN_MAX_OFFSET + 1) * sizeof(XLLP_UINT32_T)) // +1 comes from the fact that the last offset is still valid

                                        // Includes reserved addresses
#define XLLP_MFP_MAX_INDEX (XLLP_MFP_PIN_MAX_OFFSET/ sizeof (XLLP_VUINT32_T))

typedef  XLLP_VUINT32_T XLLP_MFP_REG_ARRAY_T [XLLP_MFP_MAX_INDEX+1];

typedef XLLP_INT8_T XLLP_MFP_RM_DB_ID_T, *P_XLLP_MFP_RM_DB_ID_T;

#define XLLP_MFP_RM_DB_SIZE  ((XLLP_MFP_MAX_INDEX+1)*sizeof(XLLP_MFP_RM_DB_ID_T))     

typedef enum
{
    XLLP_MFP_ALT_FN_0        = 0x0,
    XLLP_MFP_ALT_FN_1        = 0x1,
    XLLP_MFP_ALT_FN_2        = 0x2,
    XLLP_MFP_ALT_FN_3        = 0x3,
    XLLP_MFP_ALT_FN_4        = 0x4,
    XLLP_MFP_ALT_FN_5        = 0x5,
    XLLP_MFP_ALT_FN_6        = 0x6,
    XLLP_MFP_ALT_FN_7        = 0x7,
    XLLP_MFP_ALT_FN_MAX      = 0x7,
    XLLP_MFP_ALT_FN_MASK     = 0x7,
    XLLP_MFP_ALT_FN_NUM_BITS = 0x3

} XLLP_MFP_ALT_FN_T, *P_XLLP_MFP_ALT_FN_T;
 
typedef enum 
{
    XLLP_MFP_PULL_NEITHER = 0,
    XLLP_MFP_PULL_DOWN    = 1,
    XLLP_MFP_PULL_UP      = 2,
    XLLP_MFP_PULL_MAX     = 2

} XLLP_MFP_CONFIGURE_PULL_T, *P_XLLP_MFP_CONFIGURE_PULL_T;

typedef enum 
{
    XLLP_MFP_NEITHER_EDGE         = 0,
    XLLP_MFP_RISING_EDGE          = 1,
    XLLP_MFP_FALLING_EDGE         = 2,
    XLLP_MFP_RISING_FALLING_EDGES = 3,
    XLLP_MFP_EDGE_MAX             = 3

} XLLP_MFP_EDGE_T, *P_XLLP_MFP_EDGE_T;

typedef enum 
{
   XLLP_MFP_DS_01X = 0,
   XLLP_MFP_DS_02X = 1,
   XLLP_MFP_DS_03X = 2,
   XLLP_MFP_DS_04X = 3,
   XLLP_MFP_DS_06X = 4,
   XLLP_MFP_DS_08X = 5,
   XLLP_MFP_DS_10X = 6,
   XLLP_MFP_DS_12X = 7,
   XLLP_MFP_DS_MAX = 7

} XLLP_MFP_DRIVE_STRENGTH_T, *P_XLLP_MFP_DRIVE_STRENGTH_T;

//
// Table that determines the low power modes outputs, with actual settings 
//  used in parentheses for don't-care values.  Except for the float output,
//  the configured driven and pulled levels match.  So if there is a need
//  for non-LPM pulled output, the same configuration could probably be
//  used.
// 
// Output value    sleep_oe_n  sleep_data  pullup_en   pulldown_en
//                  (bit 7)     (bit 8)     (bit 14d)   (bit 13d)
// 
// Drive 0             0          0           0           X (1)
// Drive 1             0          1           X (1)       0
// Pull hi (1)         1          X(1)        1           0
// Pull lo (0)         1          X(0)        0           1
// Z (float)           1          X(0)        0           0
// 


typedef enum 
{
   XLLP_MFP_LPMO_DRIVE_LOW  =  (1u << 13),
   XLLP_MFP_LPMO_DRIVE_HIGH = ((1u << 8) | (1u << 14)),
   XLLP_MFP_LPMO_PULL_HIGH  = ((1u << 7) | (1u << 8) | (1u << 14)),
   XLLP_MFP_LPMO_PULL_LOW   = ((1u << 7) | (1u << 13)),
   XLLP_MFP_LPMO_FLOAT      =  (1u << 7),
   XLLP_MFP_LPMO_MASK       = ((1u << 7) | (1u << 8) | (1u << 13) | (1u << 14))

} XLLP_MFP_LPM_OUTPUT_T, *P_XLLP_MFP_LPM_OUTPUT_T;


// for early development.  Pull these if they are defined, otherwise define 
// them with the base types.
#ifndef P_XLLP_CONTROL_T
    typedef XLLP_CONTROL_T* P_XLLP_CONTROL_T;
#endif  // ndef P_XLLP_CONTROL_T

#ifndef P_XLLP_LEVEL_T
    typedef XLLP_LEVEL_T* P_XLLP_LEVEL_T;
#endif  // ndef P_XLLP_LEVEL_T
    

/************************************************************************/
/////////////////////////////////////////////////
//
// Begin definitions of MFPR offset constants
//
// Notes:
//  - MFPR offsets are listed in numerical order of offset
//  - The address space is sparsely populated both intrinsically
//      and because some MFPRs are used only in special purpose
//      modes that do not apply to the general SW context
//  - The offset symbols generally have this syntax:
//     XLLP_MFP_PIN_[pin name as found in DM]_OFFSET
//

// Definitions from A0 documentation
// Left chip side
#define XLLP_MFP_PIN_MD2_OFFSET                 0x0000
#define XLLP_MFP_PIN_MD3_OFFSET                 0x0004
#define XLLP_MFP_PIN_DQM0_OFFSET                0x0008
#define XLLP_MFP_PIN_DQS0_OFFSET                0x000C
#define XLLP_MFP_PIN_MD4_OFFSET                 0x0010
#define XLLP_MFP_PIN_MD5_OFFSET                 0x0014
#define XLLP_MFP_PIN_MD6_OFFSET                 0x0018
#define XLLP_MFP_PIN_MD7_OFFSET                 0x001c
#define XLLP_MFP_PIN_MD8_OFFSET                 0x0020
#define XLLP_MFP_PIN_MD9_OFFSET                 0x0024
#define XLLP_MFP_PIN_MD10_OFFSET                0x0028
#define XLLP_MFP_PIN_MD11_OFFSET                0x002c
#define XLLP_MFP_PIN_DQS1_OFFSET                0x0030
#define XLLP_MFP_PIN_DQM1_OFFSET                0x0034
#define XLLP_MFP_PIN_MD12_OFFSET                0x0038
#define XLLP_MFP_PIN_MD13_OFFSET                0x003c
#define XLLP_MFP_PIN_MD14_OFFSET                0x0040
#define XLLP_MFP_PIN_MD15_OFFSET                0x0044
#define XLLP_MFP_PIN_MA3_OFFSET                 0x0048
#define XLLP_MFP_PIN_MA2_OFFSET                 0x004c
#define XLLP_MFP_PIN_MA1_OFFSET                 0x0050
#define XLLP_MFP_PIN_MA0_OFFSET                 0x0054
#define XLLP_MFP_PIN_SDMA10_OFFSET              0x0058
#define XLLP_MFP_PIN_MA10_OFFSET                0x005c
#define XLLP_MFP_PIN_MA15_OFFSET                0x0060
#define XLLP_MFP_PIN_MA14_OFFSET                0x0064
#define XLLP_MFP_PIN_MA13_OFFSET                0x0068
#define XLLP_MFP_PIN_MA12_OFFSET                0x006c
#define XLLP_MFP_PIN_nSDCS0_OFFSET              0x0070
#define XLLP_MFP_PIN_nSDCS1_OFFSET              0x0074
#define XLLP_MFP_PIN_nSDWE_OFFSET               0x0078
#define XLLP_MFP_PIN_SDCKE_OFFSET               0x007c
#define XLLP_MFP_PIN_SDCLK0_OFFSET              0x0080
#define XLLP_MFP_PIN_SDCLK1_OFFSET              0x0084
#define XLLP_MFP_PIN_nSDRAS_OFFSET              0x0088
#define XLLP_MFP_PIN_nSDCAS_OFFSET              0x008c
#define XLLP_MFP_PIN_MA11_OFFSET                0x0090
#define XLLP_MFP_PIN_MA9_OFFSET                 0x0094
#define XLLP_MFP_PIN_MA8_OFFSET                 0x0098
#define XLLP_MFP_PIN_MA7_OFFSET                 0x009c
#define XLLP_MFP_PIN_MA6_OFFSET                 0x00a0
#define XLLP_MFP_PIN_MA5_OFFSET                 0x00a4
#define XLLP_MFP_PIN_MA4_OFFSET                 0x00a8
#define XLLP_MFP_PIN_DDR_EVAL_OFFSET            0x00ac
#define XLLP_MFP_PIN_nOE_OFFSET                 0x00b0
#define XLLP_MFP_PIN_nWE_OFFSET                 0x00b4
#define XLLP_MFP_PIN_nCS0_OFFSET                0x00b8
#define XLLP_MFP_PIN_nCS1_OFFSET                0x00bc
#define XLLP_MFP_PIN_nADV_OFFSET                0x00c0
#define XLLP_MFP_PIN_EMPI_SCLK_OFFS             0x00c4
#define XLLP_MFP_PIN_nADV2_OFFSET               0x00c8
#define XLLP_MFP_PIN_MD16_OFFSET                0x00cc
#define XLLP_MFP_PIN_MD17_OFFSET                0x00d0
#define XLLP_MFP_PIN_MD18_OFFSET                0x00d4
#define XLLP_MFP_PIN_MD19_OFFSET                0x00d8
#define XLLP_MFP_PIN_DQM2_OFFSET                0x00dc
#define XLLP_MFP_PIN_DQS2_OFFSET                0x00e0
#define XLLP_MFP_PIN_MD20_OFFSET                0x00e4
#define XLLP_MFP_PIN_MD21_OFFSET                0x00e8
#define XLLP_MFP_PIN_MD22_OFFSET                0x00ec
#define XLLP_MFP_PIN_MD23_OFFSET                0x00f0
#define XLLP_MFP_PIN_MD24_OFFSET                0x00f4
#define XLLP_MFP_PIN_MD25_OFFSET                0x00f8
#define XLLP_MFP_PIN_MD26_OFFSET                0x00fc
#define XLLP_MFP_PIN_ND27_OFFSET                0x0100
#define XLLP_MFP_PIN_DQS3_OFFSET                0x0104
#define XLLP_MFP_PIN_DQM3_OFFSET                0x0108
#define XLLP_MFP_PIN_MD28_OFFSET                0x010c
#define XLLP_MFP_PIN_MD29_OFFSET                0x0110
#define XLLP_MFP_PIN_MD30_OFFSET                0x0114

    // bottom chip side
#define XLLP_MFP_PIN_MD31_OFFSET                0x0118
#define XLLP_MFP_PIN_RDY_OFFSET                 0x0200
#define XLLP_MFP_PIN_nXCVREN_OFFSET             0x0204
#define XLLP_MFP_PIN_UN_LOCK_OFFSET             0x0208
#define XLLP_MFP_PIN_ND_CLE_OFFSET              0x020c
#define XLLP_MFP_PIN_DF_SCLK_S_OFFSET           0x0210
#define XLLP_MFP_PIN_DF_SCLK_E_OFFSET           0x0214
#define XLLP_MFP_PIN_ND_nADV1_ALE_OFFSET        0x0218
#define XLLP_MFP_PIN_nBE0_OFFSET                0x021c
#define XLLP_MFP_PIN_nBE1_OFFSET                0x0220
#define XLLP_MFP_PIN_ND_nADV2_ALE_OFFSET        0x0224
#define XLLP_MFP_PIN_ND_INT_RnB_OFFSET          0x0228
#define XLLP_MFP_PIN_ND_nCS0_OFFSET             0x022c
#define XLLP_MFP_PIN_ND_nCS1_OFFSET             0x0230
#define XLLP_MFP_PIN_ND_nWE_OFFSET              0x0234
#define XLLP_MFP_PIN_ND_nRE_nOE_OFFSET          0x0238
#define XLLP_MFP_PIN_ND_IO0_OFFSET              0x023c
#define XLLP_MFP_PIN_ND_IO1_OFFSET              0x0240
#define XLLP_MFP_PIN_ND_IO2_OFFSET              0x0244
#define XLLP_MFP_PIN_ND_IO3_OFFSET              0x0248
#define XLLP_MFP_PIN_DF_ADDR0_OFFSET            0x024c
#define XLLP_MFP_PIN_DF_ADDR1_OFFSET            0x0250
#define XLLP_MFP_PIN_nLUA_OFFSET                0x0254
#define XLLP_MFP_PIN_nLLA_OFFSET                0x0258
#define XLLP_MFP_PIN_DF_ADDR2_OFFSET            0x025c
#define XLLP_MFP_PIN_DF_ADDR3_OFFSET            0x0260
#define XLLP_MFP_PIN_ND_IO4_OFFSET              0x0264
#define XLLP_MFP_PIN_ND_IO5_OFFSET              0x0268
#define XLLP_MFP_PIN_ND_IO6_OFFSET              0x026c
#define XLLP_MFP_PIN_ND_IO7_OFFSET              0x0270
#define XLLP_MFP_PIN_ND_IO8_OFFSET              0x0274
#define XLLP_MFP_PIN_ND_IO9_OFFSET              0x0278
#define XLLP_MFP_PIN_ND_IO10_OFFSET             0x027c
#define XLLP_MFP_PIN_ND_IO11_OFFSET             0x0280
#define XLLP_MFP_PIN_ND_IO12_OFFSET             0x0284
#define XLLP_MFP_PIN_ND_IO13_OFFSET             0x0288
#define XLLP_MFP_PIN_ND_IO14_OFFSET             0x028c
#define XLLP_MFP_PIN_ND_IO15_OFFSET             0x0290
#define XLLP_MFP_PIN_GPIO57_OFFSET              0x0294
#define XLLP_MFP_PIN_GPIO58_OFFSET              0x0298
#define XLLP_MFP_PIN_GPIO55_OFFSET              0x029c
#define XLLP_MFP_PIN_GPIO56_OFFSET              0x02a0
#define XLLP_MFP_PIN_GPIO59_OFFSET              0x02a4
#define XLLP_MFP_PIN_GPIO60_OFFSET              0x02a8
#define XLLP_MFP_PIN_GPIO62_OFFSET              0x02ac
#define XLLP_MFP_PIN_GPIO61_OFFSET              0x02b0
#define XLLP_MFP_PIN_GPIO48_OFFSET              0x02b4
#define XLLP_MFP_PIN_GPIO49_OFFSET              0x02b8
#define XLLP_MFP_PIN_GPIO47_OFFSET              0x02bc
#define XLLP_MFP_PIN_GPIO51_OFFSET              0x02c0
#define XLLP_MFP_PIN_GPIO52_OFFSET              0x02c4
#define XLLP_MFP_PIN_GPIO50_OFFSET              0x02c8
#define XLLP_MFP_PIN_GPIO54_OFFSET              0x02cc
#define XLLP_MFP_PIN_GPIO53_OFFSET              0x02d0
#define XLLP_MFP_PIN_GPIO9_OFFSET               0x02d4
#define XLLP_MFP_PIN_GPIO3_OFFSET               0x02d8
#define XLLP_MFP_PIN_GPIO1_OFFSET               0x02dc
#define XLLP_MFP_PIN_GPIO0_OFFSET               0x02e0
#define XLLP_MFP_PIN_GPIO4_OFFSET               0x02e4
#define XLLP_MFP_PIN_GPIO2_OFFSET               0x02e8
#define XLLP_MFP_PIN_GPIO5_OFFSET               0x02ec
#define XLLP_MFP_PIN_GPIO11_OFFSET              0x02f0
#define XLLP_MFP_PIN_GPIO10_OFFSET              0x02f4
#define XLLP_MFP_PIN_GPIO6_OFFSET               0x02f8
#define XLLP_MFP_PIN_GPIO7_OFFSET               0x02fc
#define XLLP_MFP_PIN_GPIO8_OFFSET               0x0300
#define XLLP_MFP_PIN_GPIO12_OFFSET              0x0304
#define XLLP_MFP_PIN_GPIO14_OFFSET              0x0308
#define XLLP_MFP_PIN_GPIO15_OFFSET              0x030c
#define XLLP_MFP_PIN_GPIO13_OFFSET              0x0310
#define XLLP_MFP_PIN_GSIM_UIO_OFFSET            0x0314
#define XLLP_MFP_PIN_GSIM_UCLK_OFFSET           0x0318
#define XLLP_MFP_PIN_GSIM_UDET_OFFSET           0x031c
#define XLLP_MFP_PIN_GSIM_nURST_OFFSET          0x0320
#define XLLP_MFP_PIN_GPIO102_OFFSET             0x0324
#define XLLP_MFP_PIN_GPIO105_OFFSET             0x0328
    // Right Chip side
#define XLLP_MFP_PIN_GPIO104_OFFSET             0x0400
#define XLLP_MFP_PIN_GPIO106_OFFSET             0x0404
#define XLLP_MFP_PIN_GPIO101_OFFSET             0x0408
#define XLLP_MFP_PIN_GPIO103_OFFSET             0x040c
#define XLLP_MFP_PIN_GPIO97_OFFSET              0x0418
#define XLLP_MFP_PIN_GPIO100_OFFSET             0x0414
#define XLLP_MFP_PIN_GPIO98_OFFSET              0x0410
#define XLLP_MFP_PIN_GPIO99_OFFSET              0x041c
#define XLLP_MFP_PIN_RF_IF2_OFFSET              0x0420
#define XLLP_MFP_PIN_RF_IF1_OFFSET              0x0424
#define XLLP_MFP_PIN_RF_IF3_OFFSET              0x0428
#define XLLP_MFP_PIN_RF_IF5_OFFSET              0x042c
#define XLLP_MFP_PIN_RF_IF4_OFFSET              0x0434
#define XLLP_MFP_PIN_RF_IF14_OFFSET             0x0438
#define XLLP_MFP_PIN_RF_IF9_OFFSET              0x043c
#define XLLP_MFP_PIN_RF_IF7_OFFSET              0x0440
#define XLLP_MFP_PIN_RF_IF13_OFFSET             0x0444
#define XLLP_MFP_PIN_RF_IF8_OFFSET              0x0448
#define XLLP_MFP_PIN_RF_IF12_OFFSET             0x044c
#define XLLP_MFP_PIN_RF_IF15_OFFSET             0x0450
#define XLLP_MFP_PIN_RF_IF6_OFFSET              0x0454
#define XLLP_MFP_PIN_RF_IF10_OFFSET             0x0458
#define XLLP_MFP_PIN_RF_IF11_OFFSET             0x045c
#define XLLP_MFP_PIN_RF_IF16_OFFSET             0x0460
#define XLLP_MFP_PIN_RF_IF18_OFFSET             0x0464
#define XLLP_MFP_PIN_RF_IF17_OFFSET             0x0468
#define XLLP_MFP_PIN_RF_IF20_OFFSET             0x046c
#define XLLP_MFP_PIN_RF_IF19_OFFSET             0x0470
#define XLLP_MFP_PIN_RF_IF21_OFFSET             0x0474
#define XLLP_MFP_PIN_RF_IF22_OFFSET             0x0478
#define XLLP_MFP_PIN_AFE_RXIQ0_OFFSET           0x047c
#define XLLP_MFP_PIN_AFE_RXIQ1_OFFSET           0x0480
#define XLLP_MFP_PIN_AFE_RXIQ2_OFFSET           0x0484
#define XLLP_MFP_PIN_AFE_RXIQ3_OFFSET           0x0488
#define XLLP_MFP_PIN_AFE_RXIQ4_OFFSET           0x048c
#define XLLP_MFP_PIN_AFE_RXIQ5_OFFSET           0x0490
#define XLLP_MFP_PIN_AFE_RXIQ6_OFFSET           0x0494
#define XLLP_MFP_PIN_AFE_RXIQ7_OFFSET           0x0498
#define XLLP_MFP_PIN_CONT_READ_INT_OFFSET       0x049c
#define XLLP_MFP_PIN_AFE_TXIQ0_OFFSET           0x04a0
#define XLLP_MFP_PIN_AFE_TXIQ1_OFFSET           0x04a4
#define XLLP_MFP_PIN_AFE_TXIQ2_OFFSET           0x04a8
#define XLLP_MFP_PIN_AFE_RX_CLK_OFFSET          0x04ac
#define XLLP_MFP_PIN_AFE_TXIQ3_OFFSET           0x04b0
#define XLLP_MFP_PIN_AFE_TXIQ4_OFFSET           0x04b4
#define XLLP_MFP_PIN_AFE_TXIQ5_OFFSET           0x04b8
#define XLLP_MFP_PIN_CONT_RD_WR_OFFSET          0x04bc
#define XLLP_MFP_PIN_CONT_DATA0_OFFSET          0x04c0
#define XLLP_MFP_PIN_CONT_DATA1_OFFSET          0x04c4
#define XLLP_MFP_PIN_AFE_TXIQ6_OFFSET           0x04c8
#define XLLP_MFP_PIN_AFE_TXIQ7_OFFSET           0x04cc
#define XLLP_MFP_PIN_CONT_CLK_OFFSET            0x04d0
#define XLLP_MFP_PIN_AFE_TX_CLK_OFFSET          0x04d4
#define XLLP_MFP_PIN_AFE_TXIQ8_OFFSET           0x04d8
#define XLLP_MFP_PIN_AFE_TXIQ9_OFFSET           0x04dc
#define XLLP_MFP_PIN_DAC_ST456_OFFSET           0x04e0
#define XLLP_MFP_PIN_ADC_ST_OFFSET              0x04e4
#define XLLP_MFP_PIN_GPIO16_OFFSET              0x04e8
#define XLLP_MFP_PIN_GPIO22_OFFSET              0x04ec
#define XLLP_MFP_PIN_GPIO24_OFFSET              0x04f0
#define XLLP_MFP_PIN_GPIO17_OFFSET              0x04f4
#define XLLP_MFP_PIN_GPIO18_OFFSET              0x04f8
#define XLLP_MFP_PIN_GPIO19_OFFSET              0x04fc
#define XLLP_MFP_PIN_GPIO23_OFFSET              0x0500
#define XLLP_MFP_PIN_GPIO25_OFFSET              0x0504
#define XLLP_MFP_PIN_GPIO46_OFFSET              0x0508
#define XLLP_MFP_PIN_GPIO45_OFFSET              0x050c
#define XLLP_MFP_PIN_GPIO26_OFFSET              0x0510
#define XLLP_MFP_PIN_GPIO27_OFFSET              0x0514
#define XLLP_MFP_PIN_GPIO20_OFFSET              0x0518
#define XLLP_MFP_PIN_GPIO21_OFFSET              0x051c
#define XLLP_MFP_PIN_GPIO28_OFFSET              0x0520
    // Top chip side
#define XLLP_MFP_PIN_GPIO29_OFFSET              0x0600
#define XLLP_MFP_PIN_GPIO44_OFFSET              0x0604
#define XLLP_MFP_PIN_GPIO43_OFFSET              0x0608
#define XLLP_MFP_PIN_GPIO32_OFFSET              0x060c
#define XLLP_MFP_PIN_GPIO31_OFFSET              0x0610
#define XLLP_MFP_PIN_GPIO41_OFFSET              0x0614
#define XLLP_MFP_PIN_GPIO30_OFFSET              0x0618
#define XLLP_MFP_PIN_GPIO33_OFFSET              0x061c
#define XLLP_MFP_PIN_GPIO34_OFFSET              0x0620
#define XLLP_MFP_PIN_GPIO42_OFFSET              0x0624
#define XLLP_MFP_PIN_GPIO35_OFFSET              0x0628
#define XLLP_MFP_PIN_GPIO36_OFFSET              0x062c
#define XLLP_MFP_PIN_GPIO37_OFFSET              0x0630
#define XLLP_MFP_PIN_GPIO38_OFFSET              0x0634
#define XLLP_MFP_PIN_GPIO39_OFFSET              0x0638
#define XLLP_MFP_PIN_GPIO40_OFFSET              0x063c
#define XLLP_MFP_PIN_GPIO63_OFFSET              0x0640
#define XLLP_MFP_PIN_GPIO66_OFFSET              0x0644
#define XLLP_MFP_PIN_GPIO65_OFFSET              0x0648
#define XLLP_MFP_PIN_GPIO75_OFFSET              0x064c
#define XLLP_MFP_PIN_GPIO73_OFFSET              0x0650
#define XLLP_MFP_PIN_GPIO69_OFFSET              0x0654
#define XLLP_MFP_PIN_GPIO68_OFFSET              0x0658
#define XLLP_MFP_PIN_GPIO64_OFFSET              0x065c
#define XLLP_MFP_PIN_GPIO70_OFFSET              0x0660
#define XLLP_MFP_PIN_GPIO72_OFFSET              0x0664
#define XLLP_MFP_PIN_GPIO71_OFFSET              0x0668
#define XLLP_MFP_PIN_GPIO74_OFFSET              0x066c
#define XLLP_MFP_PIN_GPIO76_OFFSET              0x0670
#define XLLP_MFP_PIN_GPIO67_OFFSET              0x0674
#define XLLP_MFP_PIN_GPIO77_OFFSET              0x0678
#define XLLP_MFP_PIN_GPIO78_OFFSET              0x067c
#define XLLP_MFP_PIN_GPIO90_OFFSET              0x0680
#define XLLP_MFP_PIN_GPIO91_OFFSET              0x0684
#define XLLP_MFP_PIN_GPIO92_OFFSET              0x0688
#define XLLP_MFP_PIN_GPIO94_OFFSET              0x068c
#define XLLP_MFP_PIN_GPIO93_OFFSET              0x0690
#define XLLP_MFP_PIN_GPIO79_OFFSET              0x0694
#define XLLP_MFP_PIN_GPIO83_OFFSET              0x0698
#define XLLP_MFP_PIN_GPIO80_OFFSET              0x069c
#define XLLP_MFP_PIN_GPIO81_OFFSET              0x06a0
#define XLLP_MFP_PIN_GPIO82_OFFSET              0x06a4
#define XLLP_MFP_PIN_GPIO95_OFFSET              0x06a8
#define XLLP_MFP_PIN_GPIO89_OFFSET              0x06ac
#define XLLP_MFP_PIN_GPIO86_OFFSET              0x06b0
#define XLLP_MFP_PIN_GPIO85_OFFSET              0x06b4
#define XLLP_MFP_PIN_GPIO96_OFFSET              0x06b8
#define XLLP_MFP_PIN_GPIO84_OFFSET              0x06bc
#define XLLP_MFP_PIN_GPIO87_OFFSET              0x06c0
#define XLLP_MFP_PIN_GPIO88_OFFSET              0x06c4
#define XLLP_MFP_PIN_PMIC_INT_OFFSET            0x06c8
#define XLLP_MFP_PIN_MD0_OFFSET                 0x06cc
#define XLLP_MFP_PIN_MD1_OFFSET                 0x06d0

#define XLLP_MFP_PIN_GPIO4_2_OFFSET		0xff //tbd used for nled
#define XLLP_MFP_PIN_GPIO4_2_AF_GPIO_4  XLLP_MFP_ALT_FN_0 //tbd used for nled
//
// End definitions of MFPR offset constants
//
/////////////////////////////////////////////////


/////////////////////////////////////////////////
//
// Begin definition of alternate functions per pin 
//
// Notes: 
//  - These definitions are in the same order as in the DM 
//      Alternate Functions table, which is by logical-numeric or 
//      alphabetical within pin categories
//      
//  - Certain pins that may be named in the DM Alternate Functions
//      table are not under SW control for normal operations.
//      They do not appear in these Alternate Functions definitions.
//      They include the various MA, MD and DQ pins
//
//
//  - The Alternate Function symbols follow the DM table that associates
//    AFs with pins.  In some cases, the same AF is represented in more
//    than one way in the same table.  For those AFs, the more informative
//    representation is chosen.
//

// Tavor A0 MFP alternate functions
// RDY
// RDY alternate function codes
#define XLLP_MFP_PIN_RDY_AF_GPIO_62                 XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RDY_AF_RDY                     XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RDY_AF_GB_BRK_IN               XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RDY_AF_GMSL_OB_WAIT            XLLP_MFP_ALT_FN_6

// nXCVREN
// nXCVREN alternate function codes
#define XLLP_MFP_PIN_nXCVREN_AF_GPIO_57             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_nXCVREN_AF_nXCVREN             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_nXCVREN_AF_MMC2_DAT2           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_nXCVREN_AF_CGPIO_REG30         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_nXCVREN_AF_GMSL_OB_DAT1        XLLP_MFP_ALT_FN_6

// UN_LOCK
// UN_LOCK alternate function codes
#define XLLP_MFP_PIN_UN_LOCK_AF_TRI_STATE           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_UN_LOCK_AF_DF_UNLOCK           XLLP_MFP_ALT_FN_1

// ND_CLE
// ND_CLE alternate function codes
#define XLLP_MFP_PIN_ND_CLE_AF_GPIO_48              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_CLE_AF_ND_CLE               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_CLE_AF_BMSL_OB_DAT0         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_CLE_AF_GMSL_IB_DAT0         XLLP_MFP_ALT_FN_6

// DF_SCLK_S
// DF_SCLK_S alternate function codes
#define XLLP_MFP_PIN_DF_SCLK_S_AF_DF_SCLK_S         XLLP_MFP_ALT_FN_1

// DF_SCLK_E
// DF_SCLK_E alternate function codes
#define XLLP_MFP_PIN_DF_SCLK_E_AF_GPIO_52           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_DF_SCLK_E_AF_DF_SCLK_E         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_DF_SCLK_E_AF_TCU_GPOB0         XLLP_MFP_ALT_FN_7

// ND_nADV1_ALE
// ND_nADV1_ALE alternate function codes
#define XLLP_MFP_PIN_ND_nADV1_ALE_AF_GPIO_49        XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_nADV1_ALE_AF_ND_ALE         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_nADV1_ALE_AF_DF_nADV1       XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_nADV1_ALE_AF_BMSL_OB_DAT1   XLLP_MFP_ALT_FN_5

// nBE0
// nBE0 alternate function codes
#define XLLP_MFP_PIN_nBE0_AF_GPIO_60                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_nBE0_AF_nBE0                   XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_nBE0_AF_ARB_BRK_IN             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_nBE0_AF_GMSL_OB_CLK            XLLP_MFP_ALT_FN_6

// nBE1
// nBE1 alternate function codes
#define XLLP_MFP_PIN_nBE1_AF_GPIO_61                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_nBE1_AF_nBE1                   XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_nBE1_AF_ARB_BRK_OUT            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_nBE1_AF_GMSL_OB_STB            XLLP_MFP_ALT_FN_6

// ND_nADV2_ALE
// ND_nADV2_ALE alternate function codes
#define XLLP_MFP_PIN_ND_nADV2_ALE_AF_GPIO_50        XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_nADV2_ALE_AF_ND_ALE         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_nADV2_ALE_AF_DF_nADV2       XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_nADV2_ALE_AF_nCS2           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_nADV2_ALE_AF_BMSL_OB_DAT2   XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_nADV2_ALE_AF_GMSL_IB_DAT2   XLLP_MFP_ALT_FN_6

// ND_INT_RnB
// ND_INT_RnB alternate function codes
#define XLLP_MFP_PIN_ND_INT_RnB_AF_GPIO_51          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_INT_RnB_AF_ND_INT_RnB       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_INT_RnB_AF_BMSL_OB_DAT3     XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_INT_RnB_AF_GMSL_IB_DAT3     XLLP_MFP_ALT_FN_6

// ND_nCS0
// ND_nCS0 alternate function codes
#define XLLP_MFP_PIN_ND_nCS0_AF_GPIO_44             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_nCS0_AF_ND_nCS0             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_nCS0_AF_DF_nCS0             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_nCS0_AF_nCS2                XLLP_MFP_ALT_FN_3

// ND_nCS1
// ND_nCS1 alternate function codes
#define XLLP_MFP_PIN_ND_nCS1_AF_GPIO_45             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_nCS1_AF_ND_nCS1             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_nCS1_AF_DF_nCS1             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_nCS1_AF_nCS3                XLLP_MFP_ALT_FN_3

// ND_nWE
// ND_nWE alternate function codes
#define XLLP_MFP_PIN_ND_nWE_AF_GPIO_46              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_nWE_AF_ND_nWE               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_nWE_AF_DF_nWE               XLLP_MFP_ALT_FN_2

// ND_nRE_nOE
// ND_nRE_nOE alternate function codes
#define XLLP_MFP_PIN_ND_nRE_nOE_AF_GPIO_47          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_nRE_nOE_AF_ND_nRE           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_nRE_nOE_AF_DF_nOE           XLLP_MFP_ALT_FN_2

// ND_IO0
// ND_IO0 alternate function codes
#define XLLP_MFP_PIN_ND_IO0_AF_GPIO_43              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO0_AF_ND_IO0               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO0_AF_DF_IO0               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO0_AF_WB_CLKO              XLLP_MFP_ALT_FN_7

// ND_IO1
// ND_IO1 alternate function codes
#define XLLP_MFP_PIN_ND_IO1_AF_GPIO_42              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO1_AF_ND_IO1               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO1_AF_DF_IO1               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO1_AF_WB_CLKE              XLLP_MFP_ALT_FN_7

// ND_IO2
// ND_IO2 alternate function codes
#define XLLP_MFP_PIN_ND_IO2_AF_GPIO_41              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO2_AF_ND_IO2               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO2_AF_DF_IO2               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO2_AF_GB_EIC2              XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_ND_IO2_AF_EXT_MSIRQ2           XLLP_MFP_ALT_FN_7

// ND_IO3
// ND_IO3 alternate function codes
#define XLLP_MFP_PIN_ND_IO3_AF_GPIO_40              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO3_AF_ND_IO3               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO3_AF_DF_IO3               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO3_AF_GB_EIC1              XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_ND_IO3_AF_EXT_MSIRQ1           XLLP_MFP_ALT_FN_7

// DF_ADDR0
// DF_ADDR0 alternate function codes
#define XLLP_MFP_PIN_DF_ADDR0_AF_GPIO_53            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_DF_ADDR0_AF_DF_ADDR0           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_DF_ADDR0_AF_MMC2_CLK           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_DF_ADDR0_AF_SYSCLK_REQ3        XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_DF_ADDR0_AF_BMSL_OB_CLK        XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_DF_ADDR0_AF_GMSL_IB_CLK        XLLP_MFP_ALT_FN_6
    
// DF_ADDR1
// DF_ADDR1 alternate function codes
#define XLLP_MFP_PIN_DF_ADDR1_AF_GPIO_54            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_DF_ADDR1_AF_DF_ADDR1           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_DF_ADDR1_AF_MMC2_CMD           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_DF_ADDR1_AF_CLK20MOUT          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_DF_ADDR1_AF_BMSL_OB_STB        XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_DF_ADDR1_AF_BSML_IB_STB        XLLP_MFP_ALT_FN_6

// nLUA
// nLUE alternate function codes
#define XLLP_MFP_PIN_nLUA_AF_GPIO_58                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_nLUA_AF_nLUA                   XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_nLUA_AF_GMSL_OB_DAT2           XLLP_MFP_ALT_FN_6

// nLLA
// nLLA alternate function codes
#define XLLP_MFP_PIN_nLLA_AF_GPIO_59                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_nLLA_AF_nLLA                   XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_nLLA_AF_GB_BRK_OUT             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_nLLA_AF_GMSL_OB_DAT3           XLLP_MFP_ALT_FN_6

// DF_ADDR2
// DF_ADDR2 alternate function codes
#define XLLP_MFP_PIN_DF_ADDR2_AF_GPIO_55            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_DF_ADDR2_AF_DF_ADDR2           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_DF_ADDR2_AF_MMC2_DAT0          XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_DF_ADDR2_AF_CLK13MOUTDMD       XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_DF_GPB_ENABLE		    XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_DF_ADDR2_AF_BMSL_OB_WAIT       XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_DF_ADDR2_AF_GMSL_IB_WAIT       XLLP_MFP_ALT_FN_6

// DF_ADDR3
// DF_ADDR3 alternate function codes
#define XLLP_MFP_PIN_DF_ADDR3_AF_GPIO_56            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_DF_ADDR3_AF_DF_ADDR3           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_DF_ADDR3_AF_MMC2_DAT1          XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_DF_ADDR3_AF_CLK26MOUTDMD       XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_DF_ADDR3_AF_CLK26MHZ           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_DF_ADDR3_AF_GMSL_OB_DAT0       XLLP_MFP_ALT_FN_6

// ND_IO4
// ND_IO4 alternate function codes
#define XLLP_MFP_PIN_ND_IO4_AF_GPIO_39              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO4_AF_ND_IO4               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO4_AF_DF_IO4               XLLP_MFP_ALT_FN_2

// ND_IO5
// ND_IO5 alternate function codes
#define XLLP_MFP_PIN_ND_IO5_AF_GPIO_38              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO5_AF_ND_IO5               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO5_AF_DF_IO5               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO5_AF_EXT_AIRQ2            XLLP_MFP_ALT_FN_6

// ND_IO6
// ND_IO6 alternate function codes
#define XLLP_MFP_PIN_ND_IO6_AF_GPIO_37              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO6_AF_ND_IO6               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO6_AF_DF_IO6               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO6_AF_TCU_GPOB1            XLLP_MFP_ALT_FN_7

// ND_IO7
// ND_IO7 alternate function codes
#define XLLP_MFP_PIN_ND_IO7_AF_GPIO_36              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO7_AF_ND_IO7               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO7_AF_DF_IO7               XLLP_MFP_ALT_FN_2

// ND_IO8
// ND_IO8 alternate function codes
#define XLLP_MFP_PIN_ND_IO8_AF_GPIO_35              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO8_AF_ND_IO8               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO8_AF_DF_IO8               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO8_AF_MN_CLK               XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO8_AF_SSP4_RXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO8_AF_USB_P3_8             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO8_AF_EXT_AIRQ1            XLLP_MFP_ALT_FN_6

// Sheet 5
// ND_IO9
// ND_IO9 alternate function codes
#define XLLP_MFP_PIN_ND_IO9_AF_GPIO_34              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO9_AF_ND_IO9               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO9_AF_DF_IO9               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO9_AF_USB_P2_7             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO9_AF_SSP4_TXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO9_AF_USB_P3_7             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO9_AF_GB_EIC0              XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_ND_IO9_AF_EXT_MSIRQ0           XLLP_MFP_ALT_FN_7

// ND_IO10
// ND_IO10 alternate function codes
#define XLLP_MFP_PIN_ND_IO10_AF_GPIO_33             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO10_AF_ND_IO10             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO10_AF_DF_IO10             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO10_AF_MMC2_DAT3           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO10_AF_SSP4_FRM            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO10_AF_USB_P3_6            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO10_AF_HSL_DATA4           XLLP_MFP_ALT_FN_7

// ND_IO11
// ND_IO11 alternate function codes
#define XLLP_MFP_PIN_ND_IO11_AF_GPIO_32             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO11_AF_ND_IO11             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO11_AF_DF_IO11             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO11_AF_MMC2_DAT2           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO11_AF_SSP4_CLK            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO11_AF_USB_P3_5            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO11_AF_HSL_DATA3           XLLP_MFP_ALT_FN_7

// ND_IO12
// ND_IO12 alternate function codes
#define XLLP_MFP_PIN_ND_IO12_AF_GPIO_31             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO12_AF_ND_Io12             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO12_AF_DF_IO12             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO12_AF_MMC2_DAT1           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO12_AF_GSSP1_RXD           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO12_AF_USB_P3_4            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO12_AF_HSL_DATA2           XLLP_MFP_ALT_FN_7

// ND_IO13
// ND_IO13 alternate function codes
#define XLLP_MFP_PIN_ND_IO13_AF_GPIO_30             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO13_AF_ND_IO13             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO13_AF_DF_IO13             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO13_AF_MMC2_DAT0           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO13_AF_GSSP1_TXD           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO13_AF_USB_P3_3            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO13_AF_HSL_DATA1           XLLP_MFP_ALT_FN_7

// ND_IO14
// ND_IO14 alternate function codes
#define XLLP_MFP_PIN_ND_IO14_AF_GPIO_29             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO14_AF_ND_IO14             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO14_AF_DF_IO14             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO14_AF_MMC2_CLK            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO14_AF_GSSP1_FRM           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO14_AF_USB_P3_2            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO14_AF_HSL_DATA0           XLLP_MFP_ALT_FN_7

// ND_IO15
// ND_IO15 alternate function codes
#define XLLP_MFP_PIN_ND_IO15_AF_GPIO_28             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ND_IO15_AF_ND_IO15             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ND_IO15_AF_DF_IO15             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_ND_IO15_AF_MMC2_CMD            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_ND_IO15_AF_GSSP1_CLK           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ND_IO15_AF_USB_P3_1            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_ND_IO15_AF_HSL_CLK             XLLP_MFP_ALT_FN_7

// GPIO57
// GPIO57 alternate function codes
#define XLLP_MFP_PIN_GPIO57_AF_GPIO_57              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO57_AF_nIOIS16              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO57_AF_USB_P3_3             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO57_AF_MMC1_DAT0            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO57_AF_CCTS_N               XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO57_AF_TCU_GPOB2            XLLP_MFP_ALT_FN_5

// GPIO58
// GPIO58 alternate function codes
#define XLLP_MFP_PIN_GPIO58_AF_GPIO_58              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO58_AF_nPIOR                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO58_AF_USB_P3_4             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO58_AF_MMC1_DAT1            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO58_AF_CRTS_N               XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO58_AF_TCU_GPOB3            XLLP_MFP_ALT_FN_5

// GPIO55
// GPIO55 alternate function codes
#define XLLP_MFP_PIN_GPIO55_AF_GPIO_55              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO55_AF_nCS2                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO55_AF_USB_P3_1             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO55_AF_MMC1_CMD             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO55_AF_CRXD                 XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO55_AF_TCU_GPOB0            XLLP_MFP_ALT_FN_5

// GPIO56
// GPIO56 alternate function codes
#define XLLP_MFP_PIN_GPIO56_AF_GPIO_56              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO56_AF_nCS3                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO56_AF_USB_P3_2             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO56_AF_MMC1_CLK             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO56_AF_CTXD                 XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO56_AF_TCU_GPOB1            XLLP_MFP_ALT_FN_5

// GPIO59
// GPIO59 alternate function codes
#define XLLP_MFP_PIN_GPIO59_AF_GPIO_59              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO59_AF_nPIOW                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO59_AF_USB_P3_5             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO59_AF_MMC1_DAT2            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO59_AF_CGPIO_REG27          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO59_AF_DSSP3_TX             XLLP_MFP_ALT_FN_5

// GPIO60
// GPIO60 alternate function codes
#define XLLP_MFP_PIN_GPIO60_AF_GPIO_60              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO60_AF_nPWAIT               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO60_AF_USB_P3_6             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO60_AF_MMC1_DAT3            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO60_AF_CGPIO_REG28          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO60_AF_DSSP3_RX             XLLP_MFP_ALT_FN_5

// GPIO62
// GPIO62 alternate function codes
#define XLLP_MFP_PIN_GPIO62_AF_GPIO_62              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO62_AF_CGPIO_REG30          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO62_AF_USB_P3_7             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO62_AF_DVAL                 XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO62_AF_MBGNT                XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO62_AF_DSSP3_FRM            XLLP_MFP_ALT_FN_5

// GPIO61
// GPIO61 alternate function codes
#define XLLP_MFP_PIN_GPIO61_AF_GPIO_61              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO61_AF_CGPIO_REG29          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO61_AF_USB_P3_8             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO61_AF_DREQ                 XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO61_AF_MBREQ                XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO61_AF_DSSP3_SCLK           XLLP_MFP_ALT_FN_5

// GPIO48
// GPIO48 alternate function codes
#define XLLP_MFP_PIN_GPIO48_AF_GPIO_48              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO48_AF_FFDTR_N              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO48_AF_USB_P2_8             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO48_AF_SEC_TMS              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO48_AF_GSSP2_FRM            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO48_AF_HSL_DATA1            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO48_AF_USB_P2_7             XLLP_MFP_ALT_FN_7

// GPIO49
// GPIO49 alternate function codes
#define XLLP_MFP_PIN_GPIO49_AF_GPIO_49              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO49_AF_FFRI                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO49_AF_USB_P2_2             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO49_AF_SEC_TCK              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO49_AF_GSSP2_RXD            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO49_AF_HSL_DATA2            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO49_AF_USB_P2_7             XLLP_MFP_ALT_FN_6

// GPIO47
// GPIO47 alternate function codes
#define XLLP_MFP_PIN_GPIO47_AF_GPIO_47              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO47_AF_FFDSR_N              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO47_AF_USB_P2_4             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO47_AF_SEC_TRST_N           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO47_AF_GSSP2_CLK            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO47_AF_HSL_DATA0            XLLP_MFP_ALT_FN_5

// GPIO51
// GPIO51 alternate function codes
#define XLLP_MFP_PIN_GPIO51_AF_GPIO_51              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO51_AF_FFCTS_N              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO51_AF_USB_P2_5             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO51_AF_CI2C_SCL             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO51_AF_SSP4_CLK             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO51_AF_HSL_DATA4            XLLP_MFP_ALT_FN_5

// GPIO52
// GPIO52 alternate function codes
#define XLLP_MFP_PIN_GPIO52_AF_GPIO_52              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO52_AF_FFRTS_N              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO52_AF_USB_P2_1             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO52_AF_CI2C_SDA             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO52_AF_SSP4_FRM             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO52_AF_HSL_CLK              XLLP_MFP_ALT_FN_5

// GPIO50
// GPIO50 alternate function codes
#define XLLP_MFP_PIN_GPIO50_AF_GPIO_50              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO50_AF_FFDCD                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO50_AF_USB_P2_7             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO50_AF_MN_CLK               XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO50_AF_GSSP2_TXD            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO50_AF_HSL_DATA3            XLLP_MFP_ALT_FN_5

// GPIO54
// GPIO54 alternate function codes
#define XLLP_MFP_PIN_GPIO54_AF_GPIO_54              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO54_AF_FFTXD                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO54_AF_USB_P2_6             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO54_AF_SEC_TDO              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO54_AF_SSP4_RXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO54_AF_FFRXD                XLLP_MFP_ALT_FN_5

// GPIO53
// GPIO53 alternate function codes
#define XLLP_MFP_PIN_GPIO53_AF_GPIO_53              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO53_AF_FFRXD                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO53_AF_USB_P2_3             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO53_AF_SEC_TDI              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO53_AF_SSP4_TXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO53_AF_FFTXD                XLLP_MFP_ALT_FN_5

// GPIO9
// GPIO9 alternate function codes
#define XLLP_MFP_PIN_GPIO9_AF_GPIO_9                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO9_AF_KP_MKOUT4             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO9_AF_SSP1_FRM              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO9_AF_KP_DKIN5              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO9_AF_SMART_BAT             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO9_AF_USB_HS_OPMODE_1       XLLP_MFP_ALT_FN_5

// GPIO3
// GPIO3 alternate function codes
#define XLLP_MFP_PIN_GPIO3_AF_GPIO_3                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO3_AF_KP_MKOUT1             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO3_AF_SSP4_RXD              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO3_AF_CLK26MOUTDMD          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO3_AF_TCU_GPOB7             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO3_AF_USB_HS_PHYDATA_3      XLLP_MFP_ALT_FN_5

// GPIO1
// GPIO1 alternate function codes
#define XLLP_MFP_PIN_GPIO1_AF_GPIO_1                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO1_AF_KP_MKOUT0             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO1_AF_SSP4_FRM              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO1_AF_SYSCLK_REQ2           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO1_AF_TCU_GPOB5             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO1_AF_USB_HS_PHYDATA_1      XLLP_MFP_ALT_FN_5

// GPIO0
// GPIO0 alternate function codes
#define XLLP_MFP_PIN_GPIO0_AF_GPIO_0                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO0_AF_KP_MKIN0              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO0_AF_SSP4_CLK              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO0_AF_USB_P2_7              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO0_AF_TCU_GPOB4             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO0_AF_USB_HS_PHYDATA_0      XLLP_MFP_ALT_FN_5

// Sheet 6
// GPIO4
// GPIO4 alternate function codes
#define XLLP_MFP_PIN_GPIO4_AF_GPIO_4                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO4_AF_KP_MKIN2              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO4_AF_SSP3_CLK              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO4_AF_KP_DKIN4              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO4_AF_GSSP1_CLK             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO4_AF_USB_HS_PHYDATA_4      XLLP_MFP_ALT_FN_5

// GPIO2
// GPIO2 alternate function codes
#define XLLP_MFP_PIN_GPIO2_AF_GPIO_2                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO2_AF_KP_MKIN1              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO2_AF_SSP4_TXD              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO2_AF_CLK13MOUTDMD          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO2_AF_TCU_GPOB6             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO2_AF_USB_HS_PHYDATA_2      XLLP_MFP_ALT_FN_5

// GPIO5
// GPIO5 alternate function codes
#define XLLP_MFP_PIN_GPIO5_AF_GPIO_5                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO5_AF_KP_MKOUT2             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO5_AF_SSP3_FRM              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO5_AF_KP_DKIN5              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO5_AF_GSSP1_FRM             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO5_AF_USB_HS_PHYDATA_5      XLLP_MFP_ALT_FN_5

// GPIO11
// GPIO11 alternate function codes
#define XLLP_MFP_PIN_GPIO11_AF_GPIO_11              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO11_AF_KP_MKOUT5            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO11_AF_SSP1_TXD             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO11_AF_KP_DKIN7             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO11_AF_MMC1_DAT2            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO11_AF_USB_HS_RXACTIVE      XLLP_MFP_ALT_FN_5

// GPIO10
// GPIO10 alternate function codes
#define XLLP_MFP_PIN_GPIO10_AF_GPIO_10              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO10_AF_KP_MKIN5             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO10_AF_SSP1_RXD             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO10_AF_KP_DKIN6             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO10_AF_MMC1_DAT3            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO10_AF_USB_HS_RXVALID       XLLP_MFP_ALT_FN_5

// GPIO6
// GPIO6 alternate function codes
#define XLLP_MFP_PIN_GPIO6_AF_GPIO_6                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO6_AF_KP_MKIN3              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO6_AF_SSP3_TXD              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO6_AF_KP_DKIN6              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO6_AF_GSSP1_TXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO6_AF_USB_HS_PHYDATA_6      XLLP_MFP_ALT_FN_5

// GPIO7
// GPIO7 alternate function codes
#define XLLP_MFP_PIN_GPIO7_AF_GPIO_7                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO7_AF_KP_MKOUT3             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO7_AF_SSP3_RXD              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO7_AF_KP_DKIN7              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO7_AF_GSSP1_RXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO7_AF_USB_HS_PHYDATA_7      XLLP_MFP_ALT_FN_5

// GPIO8
// GPIO8 alternate function codes
#define XLLP_MFP_PIN_GPIO8_AF_GPIO_8                XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO8_AF_KP_MKIN4              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO8_AF_SSP1_CLK              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO8_AF_KP_DKIN4              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO8_AF_DAC_ST23              XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO8_AF_USB_HS_OPMODE_0       XLLP_MFP_ALT_FN_5

// GPIO12
// GPIO12 alternate function codes
#define XLLP_MFP_PIN_GPIO12_AF_GPIO_12              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO12_AF_KP_MKIN6             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO12_AF_KP_DKIN0             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO12_AF_SSP1_CLK             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO12_AF_MMC1_DAT1            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO12_AF_CLK26MHZ             XLLP_MFP_ALT_FN_5

// GPIO14
// GPIO14 alternate function codes
#define XLLP_MFP_PIN_GPIO14_AF_GPIO_14              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO14_AF_KP_MKIN7             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO14_AF_KP_DKIN2             XLLP_MFP_ALT_FN_2 // KP_DKIN2 and ROT0
#define XLLP_MFP_PIN_GPIO14_AF_SSP1_RXD             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO14_AF_MMC1_CMD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO14_AF_CLK26MOUT            XLLP_MFP_ALT_FN_5

// GPIO15
// GPIO15 alternate function codes
#define XLLP_MFP_PIN_GPIO15_AF_GPIO_15              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO15_AF_KP_MKOUT7            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO15_AF_KP_DKIN3             XLLP_MFP_ALT_FN_2 // KP_DKIN3 and ROT1
#define XLLP_MFP_PIN_GPIO15_AF_SSP1_TXD             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO15_AF_MMC1_CLK             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO15_AF_USB_P2_7             XLLP_MFP_ALT_FN_5

// GPIO13
// GPIO13 alternate function codes
#define XLLP_MFP_PIN_GPIO13_AF_GPIO_13              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO13_AF_KP_MKOUT6            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO13_AF_KP_DKIN1             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO13_AF_SSP1_FRM             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO13_AF_MMC1_DAT0            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO13_AF_MN_CLK               XLLP_MFP_ALT_FN_5

// GSIM_UIO
// GSIM_UIO alternate function codes
#define XLLP_MFP_PIN_GSIM_UIO_AF_GPIO_80            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GSIM_UIO_AF_GSIM_UIO           XLLP_MFP_ALT_FN_1

// GSIM_UCLK
// GSIM_UCLK alternate function codes
#define XLLP_MFP_PIN_GSIM_UCLK_AF_GPIO_79           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GSIM_UCLK_AF_GSIM_UCLK         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GSIM_UCLK_AF_BSIM_UCLK         XLLP_MFP_ALT_FN_2

// GSIM_UDET
// GSIM_UDET alternate function codes
#define XLLP_MFP_PIN_GSIM_UDET_AF_GPIO_82           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GSIM_UDET_AF_GSIM_UDET         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GSIM_UDET_AF_BSIM_UDET         XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GSIM_UDET_AF_CIR_OUT           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GSIM_UDET_AF_MN_CLK            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GSIM_UDET_AF_USB_P2_7          XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GSIM_UDET_AF_CLK26MHZ          XLLP_MFP_ALT_FN_6

// GSIM_nURST
// GSIM_nURST alternate function codes
#define XLLP_MFP_PIN_GSIM_nURST_AF_GPIO_81          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GSIM_nURST_AF_GSIM_nURST       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GSIM_nURST_AF_BSIM_nURST       XLLP_MFP_ALT_FN_2

// GPIO102
// GPIO102 alternate function codes
#define XLLP_MFP_PIN_GPIO102_AF_GPIO_102            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO102_AF_MMC2_DAT2           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO102_AF_DSSP3_FRM           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO102_AF_TCU_GPOA11          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO102_AF_SYSCLK_REQ1         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO102_AF_SSP1_TXD            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO102_AF_CGPIO_REG24         XLLP_MFP_ALT_FN_6

// GPIO105
// GPIO105 alternate function codes
#define XLLP_MFP_PIN_GPIO105_AF_GPIO_105            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO105_AF_MMC2_CMD            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO105_AF_GSSP2_RXD           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO105_AF_RF_CONT8            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO105_AF_CLK26MOUT           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO105_AF_USB_HS_TXREADY      XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO105_AF_STX_WORK            XLLP_MFP_ALT_FN_6

// GPIO104
// GPIO104 alternate function codes
#define XLLP_MFP_PIN_GPIO104_AF_GPIO_104            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO104_AF_MMC2_DAT0           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO104_AF_GSSP2_FRM           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO104_AF_TCU_GPOA13          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO104_AF_CLK26MOUTDMD        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO104_AF_USB_HS_TXVALID      XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO104_AF_CGPIO_REG22         XLLP_MFP_ALT_FN_6

// GPIO106
// GPIO106 alternate function codes
#define XLLP_MFP_PIN_GPIO106_AF_GPIO_106            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO106_AF_MMC2_CLK            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO106_AF_GSSP2_TXD           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO106_AF_DAC_ST23            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO106_AF_CGPIO_REG31         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO106_AF_SMART_BAT           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO106_AF_STX_LATCH_REQ       XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO106_AF_WB_CLKO             XLLP_MFP_ALT_FN_7

// GPIO101
// GPIO101 alternate function codes
#define XLLP_MFP_PIN_GPIO101_AF_GPIO_101            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO101_AF_MMC2_DAT3           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO101_AF_DSSP3_SCLK          XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO101_AF_TCU_GPOA10          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO101_AF_SSP1_RXD            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO101_AF_CPGIO_REG25         XLLP_MFP_ALT_FN_6

// GPIO103
// GPIO103 alternate function codes
#define XLLP_MFP_PIN_GPIO103_AF_GPIO_103            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO103_AF_MMC2_DAT1           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO103_AF_GSSP2_CLK           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO103_AF_TCU_GPOA12          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO103_AF_CLK13MOUTDMD        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO103_AF_USB_HS_CLK          XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO103_AF_CGPIO_REG23         XLLP_MFP_ALT_FN_6

// GPIO97
// GPIO97 alternate function codes
#define XLLP_MFP_PIN_GPIO97_AF_GPIO_97              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO97_AF_CCTS_N               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO97_AF_DSSP3_TXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO97_AF_CI2C_SCL             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO97_AF_SSP1_CLK             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO97_AF_WB_CLKE              XLLP_MFP_ALT_FN_6

// GPIO100
// GPIO100 alternate function codes
#define XLLP_MFP_PIN_GPIO100_AF_GPIO_100            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO100_AF_CTXD                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO100_AF_CRXD                XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO100_AF_TCU_GPOA9           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO100_AF_CGPIO_REG26         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO100_AF_USB_HS_LINESTATE_1  XLLP_MFP_ALT_FN_5

// GPIO98
// GPIO98 alternate function codes
#define XLLP_MFP_PIN_GPIO98_AF_GPIO_98              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO98_AF_CRTS_N               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO98_AF_DSSP3_RXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO98_AF_CI2C_SDA             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO98_AF_CGPIO_REG32          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO98_AF_SSP1_FRM             XLLP_MFP_ALT_FN_5

// GPIO99
// GPIO99 alternate function codes
#define XLLP_MFP_PIN_GPIO99_AF_GPIO_99              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO99_AF_CRXD                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO99_AF_CTXD                 XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO99_AF_USB_HS_LINESTATE_0   XLLP_MFP_ALT_FN_5

// RF_IF2
// RF_IF2 alternate function codes
#define XLLP_MFP_PIN_RF_IF2_AF_GPIO_108             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF2_AF_DSSP2_FRM            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF2_AF_RF_CONT10            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF2_AF_RF_CONT3             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF2_AF_RF_SPI0_STRB0        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF2_AF_CGPIO_REG1           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF2_AF_TCU_GPOB1            XLLP_MFP_ALT_FN_6

// RF_IF1
// RF_IF1 alternate function codes
#define XLLP_MFP_PIN_RF_IF1_AF_GPIO_107             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF1_AF_DSSP2_SCLK           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF1_AF_DSSP2_TXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF1_AF_RF_CONT2             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF1_AF_RF_SPI0_CLK          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF1_AF_CGPIO_REG0           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF1_AF_RF_CONT8             XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_RF_IF1_AF_RF_CONT0             XLLP_MFP_ALT_FN_7

// Sheet 7
// RF_IF3
// RF_IF3 alternate function codes
#define XLLP_MFP_PIN_RF_IF3_AF_GPIO_109             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF3_AF_DSSP2_RX_TX          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF3_AF_RF_CONT5             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF3_AF_RF_CONT9             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF3_AF_RF_SPI0_D            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF3_AF_CGPIO_REG2           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF3_AF_TCU_GPOB2            XLLP_MFP_ALT_FN_6

// RF_IF5
// RF_IF5 alternate function codes
#define XLLP_MFP_PIN_RF_IF5_AF_GPIO_111             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF5_AF_RF_CONT8             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF5_AF_RF_CONT4             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF5_AF_RF_CONT6             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF5_AF_DAC_ST23             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF5_AF_CGPIO_REG4           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF5_AF_TCP_GPOB4            XLLP_MFP_ALT_FN_6

// RF_IF4
// RF_IF4 alternate function codes
#define XLLP_MFP_PIN_RF_IF4_AF_GPIO_110             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF4_AF_RF_CONT7             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF4_AF_CGPIO_REG3           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF4_AF_TCU_GPOB3            XLLP_MFP_ALT_FN_6

// RF_IF14
// RF_IF14 alternate function codes
#define XLLP_MFP_PIN_RF_IF14_AF_GPIO_120            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF14_AF_RF_CONT4            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF14_AF_RF_CONT6            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF14_AF_RF_CONT2            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF14_AF_CPGIO_REG13         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF14_AF_TCU_GPOB13          XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_RF_IF14_AF_STX_WORK            XLLP_MFP_ALT_FN_7

// RF_IF9
// RF_IF9 alternate function codes
#define XLLP_MFP_PIN_RF_IF9_AF_GPIO_115             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF9_AF_RF_CONT3             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF9_AF_DSSP2_FRM            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF9_AF_RF_SPI0_STRB0        XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF9_AF_TCU_GPSIG            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF9_AF_CGPIO_REG8           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF9_AF_TCU_GPOB8            XLLP_MFP_ALT_FN_6

// RF_IF7
// RF_IF7 alternate function codes
#define XLLP_MFP_PIN_RF_IF7_AF_GPIO_113             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF7_AF_RF_CONT1             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF7_AF_RF_SPI0_STRB1        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF7_AF_CGPIO_REG6           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF7_AF_TCU_GPOB6            XLLP_MFP_ALT_FN_6

// RF_IF13
// RF_IF13 alternate function codes
#define XLLP_MFP_PIN_RF_IF13_AF_GPIO_119            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF13_AF_RF_SPI1_STRB1       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF13_AF_RF_CONT9            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF13_AF_RF_SPI0_D           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF13_AF_DSSP2_RX_TX         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF13_AF_CGPIO_REG12         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF13_AF_TCU_GPOB12          XLLP_MFP_ALT_FN_6

// RF_IF8
// RF_IF8 alternate function codes
#define XLLP_MFP_PIN_RF_IF8_AF_GPIO_114             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF8_AF_RF_CONT2             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF8_AF_DSSP2_SCLK           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF8_AF_RF_SPI0_CLK          XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF8_AF_RF_CONT4             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF8_AF_CGPIO_REG7           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF8_AF_TCU_GPOB7            XLLP_MFP_ALT_FN_6

// RF_IF12
// RF_IF12 alternate function codes
#define XLLP_MFP_PIN_RF_IF12_AF_GPIO_118            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF12_AF_RF_SPI1_CLK         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF12_AF_RF_CONT10           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF12_AF_CGPIO_REG11         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF12_AF_TCU_GPOB11          XLLP_MFP_ALT_FN_6

// RF_IF15
// RF_IF15 alternate function codes
#define XLLP_MFP_PIN_RF_IF15_AF_GPIO_121            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF15_AF_RF_CONT5            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF15_AF_RF_CONT6            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF15_AF_RF_CONT2            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF15_AF_CGPIO_REG14         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF15_AF_TCU_GPOB14          XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_RF_IF15_AF_STX_LATCH_REG       XLLP_MFP_ALT_FN_7

// RF_IF6
// RF_IF6 alternate function codes
#define XLLP_MFP_PIN_RF_IF6_AF_GPIO_112             XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF6_AF_RF_CONT0             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF6_AF_RF_INT               XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF6_AF_CGPIO_REG5           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF6_AF_TCU_GPOB5            XLLP_MFP_ALT_FN_6

// RF_IF10
// RF_IF10 alternate function codes
#define XLLP_MFP_PIN_RF_IF10_AF_GPIO_116            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF10_AF_RF_SPH_STRB0        XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF10_AF_CGPIO_REG9          XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF10_AF_TCU_GPOB9           XLLP_MFP_ALT_FN_6

// RF_IF11
// RF_IF11 alternate function codes
#define XLLP_MFP_PIN_RF_IF11_AF_GPIO_117            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF11_AF_RF_SPH_D            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF11_AF_CGPIO_REG10         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF11_AF_TCU_GPOB10          XLLP_MFP_ALT_FN_6

// RF_IF16
// RF_IF16 alternate function codes
#define XLLP_MFP_PIN_RF_IF16_AF_GPIO_122            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF16_AF_RF_CONT6            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF16_AF_DAC_ST23            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF16_AF_RF_CONT5            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF16_AF_RF_INT              XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF16_AF_CGPIO_REG15         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF16_AF_TCU_GPOB15          XLLP_MFP_ALT_FN_6

// RF_IF18
// RF_IF18 alternate function codes
#define XLLP_MFP_PIN_RF_IF18_AF_GPIO_124            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF18_AF_DSSP1_FRM           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF18_AF_RF_CONT11           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF18_AF_RF_SPI0_STRB0       XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF18_AF_CGPIO_REG17         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF18_AF_TCU_GPOA9           XLLP_MFP_ALT_FN_6

// RF_IF17
// RF_IF17 alternate function codes
#define XLLP_MFP_PIN_RF_IF17_AF_GPIO_123            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF17_AF_DSSP1_SCLK          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF17_AF_RF_CONT10           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF17_AF_RF_SPI0_CLK         XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF17_AF_CGPIO_REG16         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF17_AF_TCU_GPOA8           XLLP_MFP_ALT_FN_6

// RF_IF20
// RF_IF20 alternate function codes
#define XLLP_MFP_PIN_RF_IF20_AF_GPIO_126            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF20_AF_DSSP0_SCLK          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF20_AF_RF_CONT7            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF20_AF_CGPIO_REG19         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF20_AF_TCU_GPOA11          XLLP_MFP_ALT_FN_6

// RF_IF19
// RF_IF19 alternate function codes
#define XLLP_MFP_PIN_RF_IF19_AF_GPIO_125            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF19_AF_DSSP1_RX_TX         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF19_AF_RF_CONT12           XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF19_AF_RF_SPI0_D           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF19_AF_TCU_GPOB14          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF19_AF_CGPIO_REG18         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF19_AF_TCU_GPOA10          XLLP_MFP_ALT_FN_6

// RF_IF21
// RF_IF21 alternate function codes
#define XLLP_MFP_PIN_RF_IF21_AF_GPIO_127            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF21_AF_DSSP0_FRM           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF21_AF_TCU_GPOB14          XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF21_AF_RF_CONT8            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF21_AF_DSSP2_TX            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF21_AF_CGPIO_REG20         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF21_AF_TCU_GPOA12          XLLP_MFP_ALT_FN_6

// RF_IF22
// RF_IF22 alternate function codes
#define XLLP_MFP_PIN_RF_IF22_AF_GPIO_0              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_RF_IF22_AF_DSSP0_RX_TX         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_RF_IF22_AF_TCU_GPOB15          XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_RF_IF22_AF_DSSP1_TX            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_RF_IF22_AF_RF_INT              XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_RF_IF22_AF_CGPIO_REG21         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_RF_IF22_AF_TCU_GPOA13          XLLP_MFP_ALT_FN_6

// AFE_RXIQ0
// AFE_RXIQ0 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ0_AF_GPIO_1            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ0_AF_AFE_RXIQ0         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ0_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ0_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RXIQ1
// AFE_RXIQ1 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ1_AF_GPIO_2            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ1_AF_AFE_RXIQ1         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ1_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ1_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RXIQ2
// AFE_RXIQ2 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ2_AF_GPIO_3            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ2_AF_AFE_RXIQ2         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ2_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ2_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RXIQ3
// AFE_RXIQ3 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ3_AF_GPIO_4            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ3_AF_AFE_RXIQ3         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ3_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ3_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RXIQ4
// AFE_RXIQ4 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ4_AF_GPIO_5            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ4_AF_AFE_RXIQ4         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ4_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ4_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RXIQ5
// AFE_RXIQ5 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ5_AF_GPIO_6            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ5_AF_AFE_RXIQ5         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ5_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ5_AF_GND               XLLP_MFP_ALT_FN_5

// Sheet 8
// AFE_RXIQ6
// AFE_RXIQ6 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ6_AF_GPIO_7            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ6_AF_AFE_RXIQ6         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ6_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ6_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RXIQ7
// AFE_RXIQ7 alternate function codes
#define XLLP_MFP_PIN_AFE_RXIQ7_AF_GPIO_8            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RXIQ7_AF_AFE_RXIQ7         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RXIQ7_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RXIQ7_AF_GND               XLLP_MFP_ALT_FN_5

// CONT_READ_INT
// CONT_READ_INT alternate function codes
#define XLLP_MFP_PIN_CONT_READ_INT_AF_GPIO_25       XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_CONT_READ_INT_AF_CONT_READ_INT XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_CONT_READ_INT_AF_TRI_STATE     XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_CONT_READ_INT_AF_GND           XLLP_MFP_ALT_FN_5

// AFE_TXIQ0
// AFE_TXIQ0 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ0_AF_GPIO_10           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ0_AF_AFE_TXIQ0         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ0_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ0_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_TXIQ1
// AFE_TXIQ1 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ1_AF_GPIO_11           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ1_AF_AFE_TXIQ1         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ1_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ1_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_TXIQ2
// AFE_TXIQ2 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ2_AF_GPIO_12           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ2_AF_AFE_TXIQ2         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ2_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ2_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_RX_CLK
// AFE_RX_CLK alternate function codes
#define XLLP_MFP_PIN_AFE_RX_CLK_AF_GPIO_9           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_RX_CLK_AF_AFE_RX_CLK       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_RX_CLK_AF_TRI_STATE        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_RX_CLK_AF_GND              XLLP_MFP_ALT_FN_5

// AFE_TXIQ3
// AFE_TXIQ3 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ3_AF_GPIO_13           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ3_AF_AFE_TXIQ3         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ3_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ3_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_TXIQ4
// AFE_TXIQ4 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ4_AF_GPIO_14           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ4_AF_AFE_TXIQ4         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ4_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ4_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_TXIQ5
// AFE_TXIQ5 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ5_AF_GPIO_15           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ5_AF_AFE_TXIQ5         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ5_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ5_AF_GND               XLLP_MFP_ALT_FN_5

// CONT_RD_WR
// CONT_RD_WR alternate function codes
#define XLLP_MFP_PIN_CONT_RD_WR_AF_GPIO_24          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_CONT_RD_WR_AF_CONT_RD_WR       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_CONT_RD_WR_AF_TRI_STATE        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_CONT_RD_WR_AF_GND              XLLP_MFP_ALT_FN_5

// CONT_DATA0
// CONT_DATA0 alternate function codes
#define XLLP_MFP_PIN_CONT_DATA0_AF_GPIO_22          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_CONT_DATA0_AF_CONT_DATA0       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_CONT_DATA0_AF_TRI_STATE        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_CONT_DATA0_AF_GND              XLLP_MFP_ALT_FN_5

// CONT_DATA1
// CONT_DATA1 alternate function codes
#define XLLP_MFP_PIN_CONT_DATA1_AF_GPIO_23          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_CONT_DATA1_AF_CONT_DATA1       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_CONT_DATA1_AF_TRI_STATE        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_CONT_DATA1_AF_GND              XLLP_MFP_ALT_FN_5

// AFE_TXIQ6
// AFE_TXIQ6 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ6_AF_GPIO_16           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ6_AF_AFE_TXIQ6         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ6_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ6_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_TXIQ7
// AFE_TXIQ7 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ7_AF_GPIO_17           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ7_AF_AFE_TXIQ7         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ7_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ7_AF_GND               XLLP_MFP_ALT_FN_5

// CONT_CLK
// CONT_CLK alternate function codes
#define XLLP_MFP_PIN_CONT_CLK_AF_GPIO_21            XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_CONT_CLK_AF_CONT_CLK           XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_CONT_CLK_AF_TRI_STATE          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_CONT_CLK_AF_GND                XLLP_MFP_ALT_FN_5

// AFE_TX_CLK
// AFE_TX_CLK alternate function codes
#define XLLP_MFP_PIN_AFE_TX_CLK_AF_GPIO_20          XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TX_CLK_AF_AFE_TX_CLK       XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TX_CLK_AF_TRI_STATE        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TX_CLK_AF_GND              XLLP_MFP_ALT_FN_5

// AFE_TXIQ8
// AFE_TXIQ8 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ8_AF_GPIO_18           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ8_AF_AFE_TXIQ8         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ8_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ8_AF_GND               XLLP_MFP_ALT_FN_5

// AFE_TXIQ9
// AFE_TXIQ9 alternate function codes
#define XLLP_MFP_PIN_AFE_TXIQ9_AF_GPIO_19           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_AFE_TXIQ9_AF_AFE_TXIQ9         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_AFE_TXIQ9_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_AFE_TXIQ9_AF_GND               XLLP_MFP_ALT_FN_5

// DAC_ST456
// DAC_ST456 alternate function codes
#define XLLP_MFP_PIN_DAC_ST456_AF_GPIO_26           XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_DAC_ST456_AF_DAC_ST456         XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_DAC_ST456_AF_TRI_STATE         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_DAC_ST456_AF_GND               XLLP_MFP_ALT_FN_5

// ADC_ST
// ADC_ST alternate function codes
#define XLLP_MFP_PIN_ADC_ST_AF_GPIO_27              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_ADC_ST_AF_ADC_ST               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_ADC_ST_AF_TRI_STATE            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_ADC_ST_AF_GND                  XLLP_MFP_ALT_FN_5

// GPIO16
// GPIO16 alternate function codes
#define XLLP_MFP_PIN_GPIO16_AF_GPIO_16              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO16_AF_SMART_BAT            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO16_AF_USB_P2_7             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO16_AF_MN_CLK               XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO16_AF_HZ_CLK               XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO16_AF_ELK_EXT              XLLP_MFP_ALT_FN_5

// GPIO22
// GPIO22 alternate function codes
#define XLLP_MFP_PIN_GPIO22_AF_GPIO_22              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO22_AF_LCD_VSYNC            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO22_AF_LCD_CS2              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO22_AF_PWM3                 XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO22_AF_GMSL_IB_STB          XLLP_MFP_ALT_FN_4

// GPIO24
// GPIO24 alternate function codes
#define XLLP_MFP_PIN_GPIO24_AF_GPIO_24              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO24_AF_LCD_DD1              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO24_AF_MLCD_DD1             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO24_AF_GMSL_OB_DAT0         XLLP_MFP_ALT_FN_4

// GPIO17
// GPIO17 alternate function codes
#define XLLP_MFP_PIN_GPIO17_AF_GPIO_17              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO17_AF_LCD_FCLK_RD          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO17_AF_MLCD_FCLK            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO17_AF_GMSL_IB_DAT0         XLLP_MFP_ALT_FN_4

// GPIO18
// GPIO18 alternate function codes
#define XLLP_MFP_PIN_GPIO18_AF_GPIO_18              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO18_AF_LCD_LCLK_A0          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO18_AF_MLCD_LCLK            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO18_AF_GMSL_IB_DAT1         XLLP_MFP_ALT_FN_4

// GPIO19
// GPIO19 alternate function codes
#define XLLP_MFP_PIN_GPIO19_AF_GPIO_19              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO19_AF_LCD_PCLK_WR          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO19_AF_MLCD_PCLK            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO19_AF_GMSL_IB_DAT2         XLLP_MFP_ALT_FN_4

// GPIO23
// GPIO23 alternate function codes
#define XLLP_MFP_PIN_GPIO23_AF_GPIO_23              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO23_AF_LCD_DD0              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO23_AF_MLCD_DD0             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO23_AF_GMSL_IB_WAIT         XLLP_MFP_ALT_FN_4

// Sheet 9
// GPIO25
// GPIO25 alternate function codes
#define XLLP_MFP_PIN_GPIO25_AF_GPIO_25              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO25_AF_LCD_DD2              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO25_AF_MLCD_DD2             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO25_AF_GMSL_OB_DAT1         XLLP_MFP_ALT_FN_4

// GPIO46
// GPIO46 alternate function codes
#define XLLP_MFP_PIN_GPIO46_AF_GPIO_46              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO46_AF_CIR_OUT              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO46_AF_MN_CLK               XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO46_AF_STTXD                XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO46_AF_SSP4_RXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO46_AF_USB_HS_PHYDATA_EN    XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO46_AF_HSL_CLK              XLLP_MFP_ALT_FN_7

// GPIO45
// GPIO45 alternate function codes
#define XLLP_MFP_PIN_GPIO45_AF_GPIO_45              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO45_AF_CLK26MHZ             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO45_AF_STRXD                XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO45_AF_SSP4_TXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO45_AF_USB_HS_SUSPENDM_X    XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO45_AF_HSL_DATA4            XLLP_MFP_ALT_FN_7

// GPIO26
// GPIO26 alternate function codes
#define XLLP_MFP_PIN_GPIO26_AF_GPIO_26              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO26_AF_LCD_DD3              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO26_AF_MLCD_DD3             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO26_AF_GMSL_OB_DAT2         XLLP_MFP_ALT_FN_4

// GPIO27
// GPIO27 alternate function codes
#define XLLP_MFP_PIN_GPIO27_AF_GPIO_27              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO27_AF_LCD_DD4              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO27_AF_MLCD_DD4             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO27_AF_GMSL_OB_DAT3         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO27_AF_EXT_AIRQ1            XLLP_MFP_ALT_FN_5

// GPIO20
// GPIO20 alternate function codes
#define XLLP_MFP_PIN_GPIO20_AF_GPIO_20              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO20_AF_LCD_BIAS             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO20_AF_MLCD_BIAS            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO20_AF_GMSL_IB_DAT3         XLLP_MFP_ALT_FN_4

// GPIO21
// GPIO21 alternate function codes
#define XLLP_MFP_PIN_GPIO21_AF_GPIO_21              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO21_AF_LCD_CS               XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO21_AF_PWM2                 XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO21_AF_GMSL_IB_CLK          XLLP_MFP_ALT_FN_4

// GPIO28
// GPIO28 alternate function codes
#define XLLP_MFP_PIN_GPIO28_AF_GPIO_28              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO28_AF_LCD_DD5              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO28_AF_MLCD_DD5             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO28_AF_GMSL_OB_CLK          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO28_AF_EXT_AIRQ2            XLLP_MFP_ALT_FN_5

// GPIO29
// GPIO29 alternate function codes
#define XLLP_MFP_PIN_GPIO29_AF_GPIO_29              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO29_AF_LCD_DD6              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO29_AF_MLCD_DD6             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO29_AF_GMSL_OB_STB          XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO29_AF_EXT_MSIRQ2           XLLP_MFP_ALT_FN_5

// GPIO44
// GPIO44 alternate function codes
#define XLLP_MFP_PIN_GPIO44_AF_GPIO_44              XLLP_MFP_ALT_FN_0
//#define XLLP_MFP_PIN_GPIO44_AF_PWM2                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO44_AF_LCD_DD7              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO44_AF_KP_DKIN3             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO44_AF_STCTS_N              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO44_AF_SSP4_FRM             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO44_AF_USB_HS_TERM_SELECT   XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO44_AF_HSL_DATA3            XLLP_MFP_ALT_FN_7

// GPIO43
// GPIO43 alternate function codes
#define XLLP_MFP_PIN_GPIO43_AF_GPIO_33              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO43_AF_PWM3                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO43_AF_KP_DKIN2             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO43_AF_STRTS_N              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO43_AF_SSP4_CLK             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO43_AF_USB_HS_XCVR_SELECT   XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO43_AF_HSL_DATA2            XLLP_MFP_ALT_FN_7

// GPIO32
// GPIO32 alternate function codes
#define XLLP_MFP_PIN_GPIO32_AF_GPIO_32              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO32_AF_LCD_DD9              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO32_AF_MLCD_DD9             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO32_AF_PWM0		            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO32_AF_GB_EIC2              XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO32_AF_MMC2_CLK             XLLP_MFP_ALT_FN_7

// GPIO31
// GPIO31 alternate function codes
#define XLLP_MFP_PIN_GPIO31_AF_GPIO_31              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO31_AF_LCD_DD8              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO31_AF_MLCD_DD8             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO31_AF_TCU_GPOB12           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO31_AF_GB_EIC1              XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO31_AF_MMC2_CMD             XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO31_AF_UTM_PHYDATA_OUT_0 XLLP_MFP_ALT_FN_4

// GPIO41
// GPIO41 alternate function codes
#define XLLP_MFP_PIN_GPIO41_AF_GPIO_41              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO41_AF_PWM0                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO41_AF_KP_DKIN0             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO41_AF_LCD_CS2              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO41_AF_KP_DKIN4             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO41_AF_USB_HS_RXERROR       XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO41_AF_HSL_DATA0            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO41_AF_UTM_PHYDATA_OUT_6  XLLP_MFP_ALT_FN_5

// GPIO30
// GPIO30 alternate function codes
#define XLLP_MFP_PIN_GPIO30_AF_GPIO_30              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO30_AF_LCD_DD7              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO30_AF_MLCD_DD7             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO30_AF_GMSL_OB_WAIT         XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO30_AF_EXT_MSIRQ2           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO30_AF_UTM_PHYDATA_OUT_1  XLLP_MFP_ALT_FN_7

// GPIO33
// GPIO33 alternate function codes
#define XLLP_MFP_PIN_GPIO33_AF_GPIO_33              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO33_AF_LCD_DD10             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO33_AF_MLCD_DD10            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO33_AF_TCU_GPOB14           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO33_AF_MMC2_DAT0            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO33_AF_UTM_PHYDATA_OUT_2 XLLP_MFP_ALT_FN_5

// GPIO34
// GPIO34 alternate function codes
#define XLLP_MFP_PIN_GPIO34_AF_GPIO_34              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO34_AF_LCD_DD11             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO34_AF_MLCD_DD11            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO34_AF_TCU_GPOB15           XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO34_AF_MMC2_DAT1            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO34_AF_UTM_PHYDATA_OUT_3 XLLP_MFP_ALT_FN_5

// GPIO42
// GPIO42 alternate function codes
#define XLLP_MFP_PIN_GPIO42_AF_GPIO_42              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO42_AF_PWM1                 XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO42_AF_KP_DKIN1             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO42_AF_LCD_VSYNC2           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO42_AF_KP_DKIN5             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO42_AF_USB_HS_RESET         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO42_AF_HSL_DATA1            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO42_AF_UTM_PHYDATA_OUT_7 XLLP_MFP_ALT_FN_5

// GPIO35
// GPIO35 alternate function codes
#define XLLP_MFP_PIN_GPIO35_AF_GPIO_35              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO35_AF_LCD_DD12             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO35_AF_MLCD_DD12            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO35_AF_KP_MKIN5             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO35_AF_MMC2_DAT2            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO35_AF_UTM_PHYDATA_OUT_4 XLLP_MFP_ALT_FN_5

// GPIO36
// GPIO36 alternate function codes
#define XLLP_MFP_PIN_GPIO36_AF_GPIO_36              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO36_AF_LCD_DD13             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO36_AF_MLCD_DD13            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO36_AF_KP_MKOUT5            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO36_AF_MMC2_DAT3            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO36_AF_UTM_PHYDATA_OUT_5 XLLP_MFP_ALT_FN_5 

// GPIO37
// GPIO37 alternate function codes
#define XLLP_MFP_PIN_GPIO37_AF_GPIO_37              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO37_AF_LCD_DD14             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO37_AF_MLCD_DD14            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO37_AF_KP_MKIN6             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO37_AF_DSSP3_RX             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO37_AF_SYSCLK_REQ3          XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO37_AF_DIR			XLLP_MFP_ALT_FN_4	

// GPIO38
// GPIO38 alternate function codes
#define XLLP_MFP_PIN_GPIO38_AF_GPIO_38              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO38_AF_LCD_DD15             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO38_AF_MLCD_DD15            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO38_AF_KP_MKOUT6            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO38_AF_DSSP3_TX             XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO38_AF_CLK26MOUT            XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO38_AF_UTM_CLK			XLLP_MFP_ALT_FN_4
// GPIO39
// GPIo39 alternate function codes
#define XLLP_MFP_PIN_GPIO39_AF_GPIO_39              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO39_AF_LCD_DD16             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO39_AF_CI2C_SCL             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO39_AF_KP_MKIN7             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO39_AF_DSSP3_SCLK           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO39_AF_CLK13MOUTDMD         XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO39_AF_STP			  XLLP_MFP_ALT_FN_4

// GPIO40
// GPIO40 alternate function codes
#define XLLP_MFP_PIN_GPIO40_AF_GPIO_40              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO40_AF_LCD_DD17             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO40_AF_CLK26MHZ             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO40_AF_CI2C_SDA             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO40_AF_KP_MKOUTy            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO40_AF_DSSP3_FRM            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO40_AF_CLK26MOUTDMD         XLLP_MFP_ALT_FN_7
#define XLLP_MFP_PIN_GPIO40_AF_NXT                      XLLP_MFP_ALT_FN_4

// GPIO63
// GPIO63 alternate function codes
#define XLLP_MFP_PIN_GPIO63_AF_GPIO_63              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO63_AF_CAM_IN9              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO63_AF_FFTXD                XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO63_AF_USB_P2_8             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO63_AF_CI2C_SCL             XLLP_MFP_ALT_FN_4

// GPIO66
// GPIO66 alternate function codes
#define XLLP_MFP_PIN_GPIO66_AF_GPIO_66              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO66_AF_CAM_IN6              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO66_AF_FFDTR_N              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO66_AF_USG_P2_5             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO66_AF_CHOUT0               XLLP_MFP_ALT_FN_5

// GPIO65
// GPIO65 alternate function codes
#define XLLP_MFP_PIN_GPIO65_AF_GPIO_65              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO65_AF_CAM_IN7              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO65_AF_FFDSR_N              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO65_AF_USB_P2_6             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO65_AF_EXT_SYNC0            XLLP_MFP_ALT_FN_5

// GPIO75
// GPIO75 alternate function codes
#define XLLP_MFP_PIN_GPIO75_AF_GPIO_75              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO75_AF_CAM_MCLK            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO75_AF_GSSP1_SYSCLK         XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO75_AF_HSL_DATA4            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO75_AF_STRTS_N              XLLP_MFP_ALT_FN_5

// Sheet 10
// GPIO73
// GPIO73 alternate function codes
#define XLLP_MFP_PIN_GPIO73_AF_GPIO_73              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO73_AF_CAM_HSYNC            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO73_AF_GSSP1_TXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO73_AF_HSL_DATA2            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO73_AF_TCU_GPOB12           XLLP_MFP_ALT_FN_5

// GPIO69
// GPIO69 alternate function codes
#define XLLP_MFP_PIN_GPIO69_AF_GPIO_69              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO69_AF_CAM_IN3              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO69_AF_FFCTS_N              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO69_AF_USB_P2_2             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO69_AF_GSSP2_CLK            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO69_AF_TCU_GPOB8            XLLP_MFP_ALT_FN_5

// GPIO68
// GPIO68 alternate function codes
#define XLLP_MFP_PIN_GPIO68_AF_GPIO_68              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO68_AF_CAM_IN4              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO68_AF_FFDCD                XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO68_AF_USB_P2_3             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO68_AF_CHOUT1               XLLP_MFP_ALT_FN_5

// GPIO64
// GPIO64 alternate function codes
#define XLLP_MFP_PIN_GPIO64_AF_GPIO_64              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO64_AF_CAM_IN8              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO64_AF_FFRXD                XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO64_AF_USB_P2_8             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO64_AF_CI2C_SDA             XLLP_MFP_ALT_FN_4

// GPIO70
// GPIO70 alternate function codes
#define XLLP_MFP_PIN_GPIO70_AF_GPIO_70              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO70_AF_CAM_IN2              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO70_AF_FFRTS_N              XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO70_AF_USB_P2_1             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO70_AF_GSSP2_FRM            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO70_AF_TCU_GPOB9            XLLP_MFP_ALT_FN_5

// GPIO72
// GPIO72 alternate function codes
#define XLLP_MFP_PIN_GPIO72_AF_GPIO_72              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO72_AF_CAM_IN0              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO72_AF_GSSP1_FRM            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO72_AF_HSL_DATA1            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO72_AF_GSSP2_TXD            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO72_AF_TCU_GPOB11           XLLP_MFP_ALT_FN_5

// GPIO71
// GPIO71 alternate function codes
#define XLLP_MFP_PIN_GPIO71_AF_GPIO_71              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO71_AF_CAM_IN1              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO71_AF_GSSP1_CLK            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO71_AF_HSL_DATA0            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO71_AF_GSSP2_RXD            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO71_AF_TCU_GPOB10           XLLP_MFP_ALT_FN_5

// GPIO74
// GPIO74 alternate function codes
#define XLLP_MFP_PIN_GPIO74_AF_GPIO_74              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO74_AF_CAM_VSYNC            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO74_AF_GSSP1_RXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO74_AF_HSL_DATA3            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO74_AF_TCU_GPOB13           XLLP_MFP_ALT_FN_5

// GPIO76
// GPIO76 alternate function codes
#define XLLP_MFP_PIN_GPIO76_AF_GPIO_76              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO76_AF_CAM_PCLK             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO76_AF_SMART_BAT            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO76_AF_HSL_CLK              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO76_AF_STCTS_N              XLLP_MFP_ALT_FN_5

// GPIO67
// GPIO67 alternate function codes
#define XLLP_MFP_PIN_GPIO67_AF_GPIO_67              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO67_AF_CAM_IN5              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO67_AF_FFRI                 XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO67_AF_USB_P2_4             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO67_AF_EXT_SYNC1            XLLP_MFP_ALT_FN_5

// GPIO77
// GPIO77 alternate function codes
#define XLLP_MFP_PIN_GPIO77_AF_GPIO_77              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO77_AF_EXT_SYNC0            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO77_AF_CI2C_SCL             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO77_AF_CIR_OUT              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO77_AF_STRXD                XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO77_AF_STTXD                XLLP_MFP_ALT_FN_5

// GPIO78
// GPIO78 alternate function codes
#define XLLP_MFP_PIN_GPIO78_AF_GPIO_78              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO78_AF_CLK_EXT              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO78_AF_CI2C_SDA             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO78_AF_MN_CLK               XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO78_AF_STTXD                XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO78_AF_STRXD                XLLP_MFP_ALT_FN_5

// GPIO90
// GPIO90 alternate function codes
#define XLLP_MFP_PIN_GPIO90_AF_GPIO_90              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO90_AF_CI2C_SDA             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO90_AF_SSP4_TXD             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO90_AF_nPIOR                XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO90_AF_CHOUT1               XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO90_AF_HSL_DATA0            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO90_AF_TCU_GPOB11           XLLP_MFP_ALT_FN_7

// GPIO91
// GPIO91 alternate function codes
#define XLLP_MFP_PIN_GPIO91_AF_GPIO_91              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO91_AF_BTRXD                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO91_AF_nPIOW                XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO91_AF_SYSCLK_REQ0          XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO91_AF_HSL_DATA1            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO91_AF_TCU_GPOB12           XLLP_MFP_ALT_FN_7

// GPIO92
// GPIO92 alternate function codes
#define XLLP_MFP_PIN_GPIO92_AF_GPIO_92              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO92_AF_BTTXD                XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO92_AF_CLK26MHZ             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO92_AF_nPWAIT               XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO92_AF_CLK26MOUT            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO92_AF_HSL_DATA2            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO92_AF_TCU_GPOB13           XLLP_MFP_ALT_FN_7

// GPIO94
// GPIO94 alternate function codes
#define XLLP_MFP_PIN_GPIO94_AF_GPIO_94              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO94_AF_BTRTS_N              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO94_AF_SSP4_FRM             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO94_AF_SSP3_FRM             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO94_AF_GSSP1_FRM            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO94_AF_CLK26MOUTDMD         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO94_AF_HSL_DATA4            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO94_AF_TCU_GPOB15           XLLP_MFP_ALT_FN_7

// GPIO93
// GPIO93 alternate function codes
#define XLLP_MFP_PIN_GPIO93_AF_GPIO_93              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO93_AF_BTCTS_N              XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO93_AF_SSP4_CLK             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO93_AF_SSP3_CLK             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO93_AF_GSSP1_CLK            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO93_AF_CLK13MOUTDMD         XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO93_AF_HSL_DATA3            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO93_AF_TCU_GPOB14           XLLP_MFP_ALT_FN_7

// GPIO79
// GPIO79 alternate function codes
#define XLLP_MFP_PIN_GPIO79_AF_GPIO_79              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO79_AF_SSP3_CLK             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO79_AF_GSSP1_CLK            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO79_AF_DSSP3_SCLK           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO79_AF_TCU_GPOB14           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO79_AF_TCU_GPOA8            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO79_AF_TCU_GPOB0            XLLP_MFP_ALT_FN_7

// GPIO83
// GPIO83 alternate function codes
#define XLLP_MFP_PIN_GPIO83_AF_GPIO_83              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO83_AF_SSP3_SYSCLK          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO83_AF_GSSP1_SYSCLK         XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO83_AF_BAC97_SYSCLK         XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO83_AF_SSP3_RXD             XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO83_AF_GSSP1_RXD            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO83_AF_TCU_GPOB4            XLLP_MFP_ALT_FN_7

// GPIO80
// GPIO80 alternate function codes
#define XLLP_MFP_PIN_GPIO80_AF_GPIO_80              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO80_AF_SSP3_FRM             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO80_AF_GSSP1_FRM            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO80_AF_DSSP3_FRM            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO80_AF_TCU_GPOB15           XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO80_AF_TCU_GPOA9            XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO80_AF_TCU_GPOB1            XLLP_MFP_ALT_FN_7

// GPIO81
// GPIO81 alternate function codes
#define XLLP_MFP_PIN_GPIO81_AF_GPIO_81              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO81_AF_SSP3_TXD             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO81_AF_GSSP1_TXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO81_AF_DSSP3_TXD            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO81_AF_TCU_GPOA10           XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO81_AF_TCU_GPOB2            XLLP_MFP_ALT_FN_7

// GPIO82
// GPIO82 alternate function codes
#define XLLP_MFP_PIN_GPIO82_AF_GPIO_82              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO82_AF_SSP3_RXD             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO82_AF_GSSP1_RXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO82_AF_DSSP3_RXD            XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO82_AF_TCU_GPOA11           XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO82_AF_TCU_GPOB3            XLLP_MFP_ALT_FN_7

// GPIO95
// GPIO95 alternate function codes
#define XLLP_MFP_PIN_GPIO95_AF_GPIO_95              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO95_AF_CI2C_SCL             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO95_AF_SSP3_TXD             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO95_AF_GSSP1_TXD            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO95_AF_SMART_BAT            XLLP_MFP_ALT_FN_5

// GPIO89
// GPIO89 alternate function codes
#define XLLP_MFP_PIN_GPIO89_AF_GPIO_89              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO89_AF_CI2C_SCL             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO89_AF_SSP4_RXD             XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO89_AF_nIOIS16              XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO89_AF_EXT_SYNC1            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO89_AF_HSL_CLK              XLLP_MFP_ALT_FN_6
#define XLLP_MFP_PIN_GPIO89_AF_TCU_GPOB10           XLLP_MFP_ALT_FN_7

// GPIO86
// GPIO86 alternate function codes
#define XLLP_MFP_PIN_GPIO86_AF_GPIO_86              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO86_AF_SSP2_SYSCLK          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO86_AF_GSSP2_SYSCLK         XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO86_AF_BAC97_nRESET         XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO86_AF_SSP2_SDATA_IN        XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO86_AF_GSSP2_RXD            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO86_AF_TCU_GPOB7            XLLP_MFP_ALT_FN_7

// GPIO85
// GPIO85 alternate function codes
#define XLLP_MFP_PIN_GPIO85_AF_GPIO_85              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO85_AF_SSP2_BITCLK          XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO85_AF_GSSP2_CLK            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO85_AF_BAC97_BITCLK         XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO85_AF_TCU_GPOB6            XLLP_MFP_ALT_FN_7

// GPIO96
// GPIO96 alternate function codes
#define XLLP_MFP_PIN_GPIO96_AF_GPIO_96              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO96_AF_CI2C_SDA             XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO96_AF_SSP3_RXD             XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO96_AF_GSSP1_RXD            XLLP_MFP_ALT_FN_4
#define XLLP_MFP_PIN_GPIO96_AF_MN_CLK               XLLP_MFP_ALT_FN_5

// GPIO84
// GPIO84 alternate function codes
#define XLLP_MFP_PIN_GPIO84_AF_GPIO_84              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO84_AF_SSP2_SDATA_IN        XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO84_AF_GSSP2_RXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO84_AF_BAC97_SDATA_IN0      XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO84_AF_TCU_GPOB5            XLLP_MFP_ALT_FN_7

// GPIO87
// GPIO87 alternate function codes
#define XLLP_MFP_PIN_GPIO87_AF_GPIO_87              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO87_AF_SSP2_SYNC            XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO87_AF_GSSP2_FRM            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO87_AF_BAC97_SYNC           XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO87_AF_EXT_SYNC0            XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO87_AF_TCU_GPOB8            XLLP_MFP_ALT_FN_7

// Sheet 10
// GPIO88
// GPIO88 alternate function codes
#define XLLP_MFP_PIN_GPIO88_AF_GPIO_88              XLLP_MFP_ALT_FN_0
#define XLLP_MFP_PIN_GPIO88_AF_SSP2_DATA_OUT        XLLP_MFP_ALT_FN_1
#define XLLP_MFP_PIN_GPIO88_AF_GSSP2_TXD            XLLP_MFP_ALT_FN_2
#define XLLP_MFP_PIN_GPIO88_AF_BAC97_SDATA_OUT      XLLP_MFP_ALT_FN_3
#define XLLP_MFP_PIN_GPIO88_AF_CHOUT0               XLLP_MFP_ALT_FN_5
#define XLLP_MFP_PIN_GPIO88_AF_TCU_GPOB9            XLLP_MFP_ALT_FN_7

// PMIC_INT
// PMIC_INT alternate function codes
#define XLLP_MFP_PIN_PMIC_INT_AF_GPIO_83            XLLP_MFP_ALT_FN_0

#define XLLP_MFP_GSSP1_CLK_OFFSET       (XLLP_MFP_PIN_GPIO79_OFFSET)
#define XLLP_MFP_GSSP1_CLK_AF           (XLLP_MFP_PIN_GPIO79_AF_GSSP1_CLK)

#define XLLP_MFP_GSSP1_FRM_OFFSET       (XLLP_MFP_PIN_GPIO80_OFFSET)
#define XLLP_MFP_GSSP1_FRM_AF           (XLLP_MFP_PIN_GPIO80_AF_GSSP1_FRM)

#define XLLP_MFP_GSSP1_TXD_OFFSET       (XLLP_MFP_PIN_GPIO81_OFFSET)
#define XLLP_MFP_GSSP1_TXD_AF           (XLLP_MFP_PIN_GPIO81_AF_GSSP1_TXD)

#define XLLP_MFP_GSSP1_RXD_OFFSET       (XLLP_MFP_PIN_GPIO82_OFFSET)
#define XLLP_MFP_GSSP1_RXD_AF           (XLLP_MFP_PIN_GPIO82_AF_GSSP1_RXD)

#define XLLP_MFP_GSSP1_SYSCLK_OFFSET    (XLLP_MFP_PIN_GPIO83_OFFSET)
#define XLLP_MFP_GSSP1_SYSCLK_AF        (XLLP_MFP_PIN_GPIO83_AF_GSSP1_SYSCLK)

#define XLLP_MFP_SSP3_CLK_AF           (XLLP_MFP_PIN_GPIO79_AF_SSP3_CLK)
#define XLLP_MFP_SSP3_FRM_AF           (XLLP_MFP_PIN_GPIO80_AF_SSP3_FRM)
#define XLLP_MFP_SSP3_TXD_AF           (XLLP_MFP_PIN_GPIO81_AF_SSP3_TXD)
#define XLLP_MFP_SSP3_RXD_AF           (XLLP_MFP_PIN_GPIO82_AF_SSP3_RXD)
#define XLLP_MFP_SSP3_SYSCLK_AF        (XLLP_MFP_PIN_GPIO83_AF_SSP3_SYSCLK)

//
// End definition of alternate functions per pin 
//
/////////////////////////////////////////////////


/////////////////////////////////////////////////
//
// Begin any processor scope function prototypes
//
#ifdef __cplusplus
extern "C"
{
#endif
XLLP_STATUS_T XllpMfpResourceManager (
                            P_XLLP_MFP_RM_DB_ID_T  pMfpRmDb,
                            XLLP_UINT32_T          mfpOffset, 
                            XLLP_MFP_RM_DB_ID_T    claimantId, 
                            XLLP_UINT32_T            claimAction);

XLLP_STATUS_T XllpMfpResourceManager_List (
                P_XLLP_MFP_RM_DB_ID_T   pMfpRmDb,
                P_XLLP_UINT32_T     pMfpOffsetList,
                XLLP_MFP_RM_DB_ID_T claimantId,
                XLLP_UINT32_T       claimAction);

XLLP_STATUS_T XllpMfpResourceManagerInit (P_XLLP_MFP_RM_DB_ID_T pMfpRmDb);

XLLP_STATUS_T XllpMfpGetPinConfig (
                            P_XLLP_VUINT32_T pMfpRegBase,
                            XLLP_UINT32_T      mfpOffset,
                            P_XLLP_VUINT32_T pMfpConfig);

// Individual pin functions

XLLP_STATUS_T XllpMfpInitUnusedPin (
                            P_XLLP_VUINT32_T            pMfpRegBase,
                            XLLP_UINT32_T               mfpOffset,
                            XLLP_MFP_CONFIGURE_PULL_T   pullConfigure);

XLLP_STATUS_T XllpMfpInitUnusedPin_List (
                P_XLLP_VUINT32_T        pMfpRegBase,
                P_XLLP_UINT32_T         pMfpOffsetList,
                P_XLLP_MFP_CONFIGURE_PULL_T pPullConfigureList);

XLLP_STATUS_T XllpMfpSetAfDs (
                P_XLLP_VUINT32_T        pMfpRegBase,
                XLLP_UINT32_T           mfpOffset,
                XLLP_MFP_ALT_FN_T       afCode,
                XLLP_MFP_DRIVE_STRENGTH_T   driveStrength);

XLLP_STATUS_T XllpMfpSetAfDs_List (
                P_XLLP_VUINT32_T        pMfpRegBase,
                P_XLLP_UINT32_T         pMfpOffsetList,
                P_XLLP_MFP_ALT_FN_T     pAfCodeList,
                P_XLLP_MFP_DRIVE_STRENGTH_T pDriveStrengthList);

XLLP_STATUS_T XllpMfpConfigureWakeup (
                P_XLLP_VUINT32_T pMfpRegBase,
                XLLP_UINT32_T   mfpOffset,
                XLLP_MFP_EDGE_T edgeTrigger);

XLLP_STATUS_T XllpMfpConfigureWakeup_List (
                P_XLLP_VUINT32_T    pMfpRegBase,
                P_XLLP_UINT32_T     pMfpOffsetList,
                P_XLLP_MFP_EDGE_T   pEdgeTriggerList);

XLLP_STATUS_T XllpMfpEnableWakeup (
                P_XLLP_VUINT32_T pMfpRegBase,
                XLLP_UINT32_T   mfpOffset,
                XLLP_CONTROL_T  enable);

XLLP_STATUS_T XllpMfpEnableWakeup_List (
                P_XLLP_VUINT32_T    pMfpRegBase,
                P_XLLP_UINT32_T     pMfpOffsetList,
                XLLP_CONTROL_T      enable);

XLLP_STATUS_T XllpMfpConfigurePullUpDown (
                P_XLLP_VUINT32_T        pMfpRegBase,
                XLLP_UINT32_T           mfpOffset,
                XLLP_MFP_CONFIGURE_PULL_T   pullConfigure);

XLLP_STATUS_T XllpMfpConfigurePullUpDown_List (
                P_XLLP_VUINT32_T    pMfpRegBase,
                P_XLLP_UINT32_T     pMfpOffsetList,
                P_XLLP_MFP_CONFIGURE_PULL_T pPullConfigureList);

XLLP_STATUS_T XllpMfpActivatePullUpDown (
                P_XLLP_VUINT32_T        pMfpRegBase,
                XLLP_UINT32_T           mfpOffset,
                XLLP_CONTROL_T          enable);

XLLP_STATUS_T XllpMfpActivatePullUpDown_List (
                P_XLLP_VUINT32_T    pMfpRegBase,
                P_XLLP_UINT32_T     pMfpOffsetList,
                XLLP_CONTROL_T      enable);

XLLP_STATUS_T XllpMfpConfigureDrivenLpmOutputLevel (
                P_XLLP_VUINT32_T pMfpRegBase,
                XLLP_UINT32_T   mfpOffset,
                XLLP_LEVEL_T    level);

XLLP_STATUS_T XllpMfpEnableDrivenLpmOutputLevel (
                P_XLLP_VUINT32_T pMfpRegBase,
                XLLP_UINT32_T   mfpOffset,
                XLLP_CONTROL_T  enable);

XLLP_STATUS_T XllpMfpConfigureLpmOutputLevel_List (
            P_XLLP_VUINT32_T        pMfpRegBase,
            P_XLLP_UINT32_T         pMfpOffsetList,
            P_XLLP_MFP_LPM_OUTPUT_T pLpmOutputList);

XLLP_STATUS_T XllpMfpEnableRdhDrm(
                P_XLLP_VUINT32_T    pMfpRegBase,
                XLLP_UINT32_T       mfpOffset,
                XLLP_CONTROL_T      enable);

XLLP_STATUS_T XllpMfpEnableRdhDrm_List(
                P_XLLP_VUINT32_T    pMfpRegBase,
                P_XLLP_UINT32_T     pMfpOffsetList,
                XLLP_CONTROL_T      enable);


#ifdef __cplusplus
}

#endif
//
// End any processor scope function prototypes
//
/////////////////////////////////////////////////

#endif // __XLLP_MFP_PROC_H__


