/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __XLLP_MMC_H
#define __XLLP_MMC_H

#include "xllp_defs.h"
#include "xllp_gpio.h"
#include "xllp_mfp.h"
#include "xllp_ost.h"
//#include "xllp_clkmgr.h"

// MMC controller registers definition
typedef struct 
{
  XLLP_VUINT32_T mmc_strpcl;    // Control to start/stop mmc/sd clock 
  XLLP_VUINT32_T mmc_stat;      // status register
  XLLP_VUINT32_T mmc_clkrt;     // clock rate 
  XLLP_VUINT32_T mmc_spi;       // SPI mode control bits 
  XLLP_VUINT32_T mmc_cmdat;     // cmd/response/data sequence control 
  XLLP_VUINT32_T mmc_resto;     // expected response timout 
  XLLP_VUINT32_T mmc_rdto;      // expected data read timeout 
  XLLP_VUINT32_T mmc_blklen;    // block length of data transaction 
  XLLP_VUINT32_T mmc_numblk;    // number of block for block mode 
  XLLP_VUINT32_T mmc_prtbuf;    // partial mmc_txfifo written  
  XLLP_VUINT32_T mmc_i_mask;    // interrupt mask 
  XLLP_VUINT32_T mmc_i_reg;     // interrupt register 
  XLLP_VUINT32_T mmc_cmd;       // index of current command 
  XLLP_VUINT32_T mmc_argh;      // MSW part of the current command arg 
  XLLP_VUINT32_T mmc_argl;      // LSW p art of the current command arg 
  XLLP_VUINT32_T mmc_res;       // response fifo 
  XLLP_VUINT32_T mmc_rxfifo;    // receive fifo for single byte 
  XLLP_VUINT32_T mmc_txfifo;    // transmit fifo for single byte 
  XLLP_VUINT32_T mmc_rdwait;    // reserved for different fifo 
  XLLP_VUINT32_T mmc_blks_rem;  // SD CMDAT register 
} XLLP_MMC_T, *P_XLLP_MMC_T;

// device context for MMC API, containing anything needed to operate on 
// this API. It is always first parameter for all APIs.
typedef struct {
  P_XLLP_MMC_T            pMMC1Reg;     // MMC1 register base
  P_XLLP_MMC_T            pMMC2Reg;     // MMC2 register base
  P_XLLP_VUINT32_T        pMFPBase;     // MFPR base address
  P_XLLP_MFP_RM_DB_ID_T   pMFPRmDb;     // MFP Resource Manager database
  P_XLLP_GPIO_T           pGPIOReg;     // GPIO controller register base
  //P_XLLP_CLOCK_T          pClockReg;  // Clock Manager register base
  P_XLLP_OST_T            pOSTReg;      // OST register base
  //P_XLLP_PI2C_T           pPI2CReg;     // Power I2C register base
} XLLP_MMC_CONTEXT_T, *P_XLLP_MMC_CONTEXT_T;


// register masks

// MMC bus clock control register
#define XLLP_MMC_STRPCL_STOP_CLOCK    XLLP_BIT_0;   
#define XLLP_MMC_STRPCL_START_CLOCK   XLLP_BIT_1;   

// MMC CMD_DAT register
#define  XLLP_MMC_CMDAT_DATA          XLLP_BIT_2   
#define  XLLP_MMC_CMDAT_WRITE         XLLP_BIT_3  
#define  XLLP_MMC_CMDAT_STREAM        XLLP_BIT_4
#define  XLLP_MMC_CMDAT_BUSY          XLLP_BIT_5  
#define  XLLP_MMC_CMDAT_INIT          XLLP_BIT_6
#define  XLLP_MMC_CMDAT_DMA           XLLP_BIT_7
#define  XLLP_MMC_CMDAT_4DAT          XLLP_BIT_8
#define  XLLP_MMC_CMDAT_STOP          XLLP_BIT_10
#define  XLLP_MMC_CMDAT_SDIO_INT      XLLP_BIT_11
#define  XLLP_MMC_CMDAT_SDIO_SUSPEND  XLLP_BIT_12
#define  XLLP_MMC_CMDAT_SDIO_RESUME   XLLP_BIT_13

// MMC status register
#define  XLLP_MMC_STAT_READ_TIMEOUT       XLLP_BIT_0    
#define  XLLP_MMC_STAT_RESP_TIMEOUT       XLLP_BIT_1    
#define  XLLP_MMC_STAT_WRITE_CRC          XLLP_BIT_2    
#define  XLLP_MMC_STAT_READ_CRC           XLLP_BIT_3    
#define  XLLP_MMC_STAT_SPI_ERR_TOKEN      XLLP_BIT_4    
#define  XLLP_MMC_STAT_RESP_CRC           XLLP_BIT_5  
#define  XLLP_MMC_STAT_CLOCK_ON           XLLP_BIT_8  
#define  XLLP_MMC_STAT_FLASH_ERR          XLLP_BIT_9  
#define  XLLP_MMC_STAT_SPI_WRITE_ERR      XLLP_BIT_10 
#define  XLLP_MMC_STAT_TRAN_DONE          XLLP_BIT_11 
#define  XLLP_MMC_STAT_PRG_DONE           XLLP_BIT_12 
#define  XLLP_MMC_STAT_END_CMD            XLLP_BIT_13 
#define  XLLP_MMC_STAT_SDIO_RDSTALLED     XLLP_BIT_14 
#define  XLLP_MMC_STAT_SDIO_INT           XLLP_BIT_15 
#define  XLLP_MMC_STAT_SDIO_SUSPENDACK    XLLP_BIT_16 

// MMC interrupt mask register
#define  XLLP_MMC_I_MASK_DATA_TRAN_DONE     XLLP_BIT_0  
#define  XLLP_MMC_I_MASK_PRG_DONE           XLLP_BIT_1  
#define  XLLP_MMC_I_MASK_END_CMD_RES        XLLP_BIT_2  
#define  XLLP_MMC_I_MASK_STOP_CMD           XLLP_BIT_3  
#define  XLLP_MMC_I_MASK_CLK_OFF            XLLP_BIT_4  
#define  XLLP_MMC_I_MASK_RXFIFO_RD_REQ      XLLP_BIT_5  
#define  XLLP_MMC_I_MASK_TXFIFO_WR_REQ      XLLP_BIT_6  
#define  XLLP_MMC_I_MASK_TINT               XLLP_BIT_7  
#define  XLLP_MMC_I_MASK_DAT_ERR            XLLP_BIT_8  
#define  XLLP_MMC_I_MASK_RES_ERR            XLLP_BIT_9  
#define  XLLP_MMC_I_MASK_SDIO_RD_STALLED    XLLP_BIT_10 
#define  XLLP_MMC_I_MASK_SDIO_INT           XLLP_BIT_11 
#define  XLLP_MMC_I_MASK_SDIO_SUSPEND_ACK   XLLP_BIT_12 

// MMC interrupt register
#define  XLLP_MMC_I_REG_DATA_TRAN_DONE      XLLP_BIT_0  
#define  XLLP_MMC_I_REG_PRG_DONE            XLLP_BIT_1  
#define  XLLP_MMC_I_REG_END_CMD_RES         XLLP_BIT_2  
#define  XLLP_MMC_I_REG_STOP_CMD            XLLP_BIT_3  
#define  XLLP_MMC_I_REG_CLK_OFF             XLLP_BIT_4  
#define  XLLP_MMC_I_REG_RXFIFO_RD_REQ       XLLP_BIT_5  
#define  XLLP_MMC_I_REG_TXFIFO_WR_REQ       XLLP_BIT_6  
#define  XLLP_MMC_I_REG_TINT                XLLP_BIT_7  
#define  XLLP_MMC_I_REG_DAT_ERR             XLLP_BIT_8  
#define  XLLP_MMC_I_REG_RES_ERR             XLLP_BIT_9  
#define  XLLP_MMC_I_REG_SDIO_RD_STALLED     XLLP_BIT_10 
#define  XLLP_MMC_I_REG_SDIO_INT            XLLP_BIT_11 
#define  XLLP_MMC_I_REG_SDIO_SUSPEND_ACK    XLLP_BIT_12 



// clock rate constant
typedef enum {
  XLLP_MMC_19_5MHz  = 0x0,
  XLLP_MMC_9_75MHz  = 0x1,
  XLLP_MMC_4_88MHz  = 0x2,
  XLLP_MMC_2_44MHz  = 0x3,
  XLLP_MMC_1_22MHz  = 0x4,
  XLLP_MMC_609KHz   = 0x5,
  XLLP_MMC_304KHz   = 0x6
}XLLP_MMC_CLOCK_SPEED_T;

// response size
typedef enum {
  XLLP_MMC_RESPONSE_SIZE_32 = 0,
  XLLP_MMC_RESPONSE_SIZE_120
} XLLP_MMC_RESP_SIZE_T;

// response format
typedef enum {
  XLLP_MMC_RESPONSE_FMT_NONE = 0,
  XLLP_MMC_RESPONSE_FMT_R1 = 1,
  XLLP_MMC_RESPONSE_FMT_R2 = 2,
  XLLP_MMC_RESPONSE_FMT_R3 = 3
} XLLP_MMC_RESP_FORMAT_T;


//Flags used to send command

// command types
enum {
  XLLP_MMC_NORMAL_COMMAND = 1L,
  XLLP_MMC_INIT_COMMAND = 2L,     // first command require 80 clock preceded
  XLLP_MMC_STOP_COMMAND = 3L,     // stop command, including CMD12,CMD52
  XLLP_MMC_SUSPEND_COMMAND = 4L,  // sdio CMD52, supend current transfer
  XLLP_MMC_RESUME_COMMAND = 5L,   // sdio CMD52, resume a suspended transfer
  XLLP_MMC_CMDTYPE_MASK = 0x000000FF
};

// response types
enum {
  XLLP_MMC_RESPONSE_NONE = 1L<<8,
  XLLP_MMC_RESPONSE_R1 = 2L<<8,
  XLLP_MMC_RESPONSE_R1B = 3L<<8,
  XLLP_MMC_RESPONSE_R2 = 4L<<8,
  XLLP_MMC_RESPONSE_R3 = 5L<<8,
  XLLP_MMC_RESPONSE_R4 = 6L<<8,
  XLLP_MMC_RESPONSE_R5 = 7L<<8,
  XLLP_MMC_RESPONSE_R5B = 8L<<8,
  XLLP_MMC_RESPONSE_R6 = 9L<<8,
  XLLP_MMC_RESPONSE_MASK = 0x0000FF00
};
// transfer types
enum {
  XLLP_MMC_4BIT_ENABLE = 0x01L<<16,
  XLLP_MMC_DMA_ENABLE = 0x02L<<16,
  XLLP_MMC_DATA_ENABLE = 0x04L<<16,
  XLLP_MMC_DATA_WRITE =  0x08L<<16,
  XLLP_MMC_STREAM_MODE = 0x10L<<16,
  XLLP_MMC_SDIO_INT_ENABLE = 0x20<<16,
  XLLP_MMC_SPI_MODE = 0x40<<16, 
  XLLP_MMC_TRANSFER_MODE_MASK = 0x00FF0000
};

// controller enumeration for Monahans processor
typedef enum {
  XLLP_MMC_CONTROLLER_1 = 0,
  XLLP_MMC_CONTROLLER_2,
}XLLP_MMC_CONTROLLER_ID_T;
// slot enumeration for Zylonite board
typedef enum {
  XLLP_MMC_SLOT_0 = 0,
  XLLP_MMC_SLOT_1,
  XLLP_MMC_SLOT_SDIO // currently not used
}XLLP_MMC_SLOT_ID_T;

// GPIO id used
enum {
  XLLP_MMC_CD_0_GPIO_ID = 1,
  XLLP_MMC_CD_1_GPIO_ID = 4,
  XLLP_MMC_WP_0_GPIO_ID = 5,
  XLLP_MMC_WP_1_GPIO_ID = 6,
  XLLP_MMC_CMD_0_GPIO_ID = 23,
  XLLP_MMC_CMD_1_GPIO_ID = 31
};


// start of processor scope function declaration
/*
void XllpMMCStartCtrlClock(P_XLLP_MMC_CONTEXT_T pContext,
                           XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCStopCtrlClock(P_XLLP_MMC_CONTEXT_T pContext, 
                          XLLP_MMC_CONTROLLER_ID_T controller);
*/

void XllpMMCStartBusClock(P_XLLP_MMC_CONTEXT_T pContext, 
                          XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCStopBusClock(P_XLLP_MMC_CONTEXT_T pContext, 
                         XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCSetBusRate(P_XLLP_MMC_CONTEXT_T pContext, 
                       XLLP_MMC_CLOCK_SPEED_T rate, 
                       XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCEnableInt(P_XLLP_MMC_CONTEXT_T pContext,
                      XLLP_UINT32_T imask, 
                      XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCDisableInt(P_XLLP_MMC_CONTEXT_T pContext,
                       XLLP_UINT32_T imask, 
                       XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCGetIntStatus(P_XLLP_MMC_CONTEXT_T pContext,
                         P_XLLP_UINT32_T istat, 
                         XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCReadResponse(P_XLLP_MMC_CONTEXT_T pContext,
                         P_XLLP_UINT8_T pBuffer, 
                         XLLP_MMC_RESP_SIZE_T size, 
                         XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCReadStatus(P_XLLP_MMC_CONTEXT_T pContext,
                       P_XLLP_UINT32_T mmcstat, 
                       XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCSetPartialFull(P_XLLP_MMC_CONTEXT_T pContext, 
                           XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCReadFIFO(P_XLLP_MMC_CONTEXT_T pContext, 
                     P_XLLP_UINT8_T data, 
                     XLLP_UINT8_T size, 
                     XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCWriteFIFO(P_XLLP_MMC_CONTEXT_T pContext, 
                      P_XLLP_UINT8_T data, 
                      XLLP_UINT8_T size, 
                      XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCReadBlksRem(P_XLLP_MMC_CONTEXT_T pContext,
                        P_XLLP_UINT32_T count, 
                        XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCSetRespTimeout(P_XLLP_MMC_CONTEXT_T pContext,
                          XLLP_UINT32_T to, 
                          XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCSetReadTimeout(P_XLLP_MMC_CONTEXT_T pContext,
                           XLLP_UINT32_T to, 
                           XLLP_MMC_CONTROLLER_ID_T controller);

void XllpMMCSendCommand(P_XLLP_MMC_CONTEXT_T pContext, 
                        XLLP_UINT32_T cmd,
                        XLLP_INT32_T argument, 
                        XLLP_UINT32_T flags,
                        XLLP_UINT32_T blockCount, 
                        XLLP_UINT32_T blockSize, 
                        XLLP_MMC_CONTROLLER_ID_T controller);


// start of board scope function declaration
XLLP_STATUS_T XllpMMCConfigure(P_XLLP_MMC_CONTEXT_T pContext);

XLLP_STATUS_T XllpMMC2Configure(P_XLLP_MMC_CONTEXT_T pContext);


void XllpMMCSelectSlot(P_XLLP_MMC_CONTEXT_T pContext, 
                      XLLP_MMC_SLOT_ID_T slot);

void XllpMMCEnableCardDetect(P_XLLP_MMC_CONTEXT_T pContext, 
                            XLLP_MMC_SLOT_ID_T slot); 

void XllpMMCPowerOnSlot(P_XLLP_MMC_CONTEXT_T pContext, 
                        XLLP_MMC_SLOT_ID_T slot);

void XllpMMCPowerOffSlot(P_XLLP_MMC_CONTEXT_T pContext,
                         XLLP_MMC_SLOT_ID_T slot);

XLLP_BOOL_T XllpMMCIsCardWP(P_XLLP_MMC_CONTEXT_T pContext, 
                            XLLP_MMC_SLOT_ID_T slot);

XLLP_BOOL_T XllpMMCIsCardInserted(P_XLLP_MMC_CONTEXT_T pContext, 
                                  XLLP_MMC_SLOT_ID_T slot);
#endif //__XLLP_MMC_H
