/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/** FILENAME:	xllp_pm_dvfm.h
**
** PURPOSE:	contain struct definition for frequency/voltage change.
**
******************************************************************************/

#ifndef __XLLP_PM_DVFM_H
#define __XLLP_PM_DVFM_H

#include "xllp_defs.h"
#include "xllp_pm.h"
#include "xllp_clkmgr.h"
#include "xllp_memc.h"
#include "xllp_ost.h"

/* operating point definition*/
typedef struct {
	XLLP_UINT32_T xl;
	XLLP_UINT32_T xn;
	XLLP_UINT32_T vcc_core;
	XLLP_UINT32_T vcc_sram;
	XLLP_UINT32_T smcfs;
	XLLP_UINT32_T sflfs;
	XLLP_UINT32_T hss;
	XLLP_UINT32_T dmcfs;
	XLLP_UINT32_T df_clk;
	XLLP_UINT32_T empi_clk;
	XLLP_UINT32_T d0cs;
} XLLP_PM_FV_INFO, *P_XLLP_PM_FV_INFO;

#define XLLP_PM_ACCR_SMCFS_78M	0x0
#define XLLP_PM_ACCR_SMCFS_104M	0x2
#define XLLP_PM_ACCR_SMCFS_208M	0x5

/* Set SFLFS clock in ACCR */
#define XLLP_PM_ACCR_SFLFS_104M	0x0
#define XLLP_PM_ACCR_SFLFS_156M	0x1
#define XLLP_PM_ACCR_SFLFS_208M	0x2
#define XLLP_PM_ACCR_SFLFS_312M	0x3

/* Set HSS clock in ACCR */
#define XLLP_PM_ACCR_HSS_104M	0x0
#define XLLP_PM_ACCR_HSS_156M	0x1
#define XLLP_PM_ACCR_HSS_208M	0x2

/* Set DMCFS in ACCR */
#define XLLP_PM_ACCR_DMCFS_26M	0x0
#define XLLP_PM_ACCR_DMCFS_260M	0x3

/* Set df_clkdiv in MEMCLKCFG */
#define XLLP_PM_DF_CLKDIV_1 0x1
#define XLLP_PM_DF_CLKDIV_2 0x2
#define XLLP_PM_DF_CLKDIV_4 0x3

/* set empi_clkdiv in MEMCLKCFG  */
#define XLLP_PM_EMPI_CLKDIV_1 0x1
#define XLLP_PM_EMPI_CLKDIV_2 0x2
#define XLLP_PM_EMPI_CLKDIV_4 0x3


/* CFS mode */
typedef   XLLP_UINT32_T XLLP_PM_FV_MODE;
#define XLLP_PM_FV_MODE_SET_CFS_XSPCLK_OFFSET  0x0
#define XLLP_PM_FV_MODE_SET_CFS_XSPCLK_MASK    0x3
#define	XLLP_PM_FV_MODE_SET_CFS_XSPCLK_156M	   0x0
#define	XLLP_PM_FV_MODE_SET_CFS_XSPCLK_208M    0x1
#define	XLLP_PM_FV_MODE_SET_CFS_XSPCLK_312M    0x2
#define	XLLP_PM_FV_MODE_SET_CFS_XSPCLK_DIS	   0x3

/* MEMCLKCFG macros */
#define XLLP_MEMCLKCFG_DFCLKDIV_MASK    0x00070000
#define XLLP_MEMCLKCFG_EMPICLKDIV_MASK  0x00000007

#define XLLP_MEMCLKCFG_G_DFCLKDIV(memclkcfg)   (((memclkcfg) >> 16)  & 0x7)
#define XLLP_MEMCLKCFG_G_EMPICLKDIV(memclkcfg) (((memclkcfg) >> 0)  & 0x7)
#define XLLP_MEMCLKCFG_S_DFCLKDIV(memclkcfg, df_clk)  \
							(((memclkcfg) & ~0x00070000) | ((df_clk) << 16 ))
#define XLLP_MEMCLKCFG_S_EMPICLKDIV(memclkcfg, empi_clk)  \
							(((memclkcfg) & ~0x00000007) | ((empi_clk) << 0 ))

/* ACCR macros*/
#define XLLP_ACCR_XL_MASK    		0x0000001f
#define XLLP_ACCR_XN_MASK    		0x00000700
#define XLLP_ACCR_DMCFS_MASK      	0x00003000
#define XLLP_ACCR_HSS_MASK 			0x0000c000
#define XLLP_ACCR_XSPCLK_MASK   	0x00030000
#define XLLP_ACCR_SFLFS_MASK        0x000c0000
#define XLLP_ACCR_13MEND2_MASK      0x00200000
#define XLLP_ACCR_SMCFS_MASK        0x03800000
#define XLLP_ACCR_D0CS_MASK         0x04000000
#define XLLP_ACCR_13MEND1_MASK      0x08000000
#define XLLP_ACCR_SPDIS_MASK        0x40000000
#define XLLP_ACCR_XPDIS_MASK        0x80000000

#define XLLP_ACCR_G_XL(accr)			((accr) & 0x1F)
#define XLLP_ACCR_G_XN(accr)			(((accr) >> 8) & 0x7)
#define XLLP_ACCR_G_SMCFS(accr)		(((accr) >> 23)  & 0x7)
#define XLLP_ACCR_G_SFLFS(accr)		(((accr) >> 18)  & 0x3)
#define XLLP_ACCR_G_HSS(accr)		(((accr) >> 14)  & 0x3)
#define XLLP_ACCR_G_DMCFS(accr)		(((accr) >> 12)  & 0x3)
#define XLLP_ACCR_G_XSPCLK(accr)		(((accr) >> 16)  & 0x3)
#define XLLP_ACCR_G_D0CS(accr)          (((accr) >> 26)  & 0x1)
#define XLLP_ACCR_G_SPDIS(accr)          (((accr) >> 30)  & 0x1)
#define XLLP_ACCR_G_XPDIS(accr)          (((accr) >> 31)  & 0x1)
#define XLLP_ACCR_G_D0CS(accr)          (((accr) >> 26)  & 0x1)
#define XLLP_ACCR_S_XL(accr, xl)		(((accr) & ~0x1F) | xl)
#define XLLP_ACCR_S_XN(accr, xn)		((((accr) & ~(0x7 << 8)) | ((xn) << 8)))
#define XLLP_ACCR_S_SMCFS(accr, smcfs)	((((accr) & ~(0x7 << 23)) | ((smcfs) << 23)))
#define XLLP_ACCR_S_SFLFS(accr, sflfs)	((((accr) & ~(0x3 << 18)) | ((sflfs) << 18)))
#define XLLP_ACCR_S_HSS(accr, hss)		((((accr) & ~(0x3 << 14)) | ((hss) << 14)))
#define XLLP_ACCR_S_DMCFS(accr, dmcfs)	((((accr) & ~(0x3 << 12)) | ((dmcfs) << 12)))
#define XLLP_ACCR_S_XSPCLK(accr, xspclk)	((((accr) & ~(0x3 << 16)) | ((xspclk) << 16)))
#define XLLP_ACCR_S_D0CS(accr, d0cs)	((((accr) & ~(0x1 << 26)) | ((d0cs) << 26)))
#define XLLP_ACCR_S_SPDIS(accr, spdis)	((((accr) & ~(0x1 << 30)) | ((spdis) << 30)))
#define XLLP_ACCR_S_XPDIS(accr, xpdis)	((((accr) & ~(0x1 << 31)) | ((xpdis) << 31)))
#define XLLP_XCLKCFG_G_T(clkcfg)	((clkcfg) & 0x1)
#define XLLP_XCLKCFG_SET(f, t)	(((f) << 0x1) | (t))
#define XLLP_ACSR_ACCR_MASK				0xc78cf71f

/* flags to mark those frequencies and voltages to be changed  */
#define XLLP_FV_SET_CFS		0x0001	/* set core frequency */
#define XLLP_FV_SET_SMCFS	0x0002
#define XLLP_FV_SET_SFLFS	0x0004
#define XLLP_FV_SET_HSS	    0x0008
#define XLLP_FV_SET_DMCFS	0x0010
#define XLLP_FV_SET_DFCLK   0x0020
#define XLLP_FV_SET_EMPICLK 0x0040
#define XLLP_FV_SET_D0CS    0x0080
#define XLLP_FV_SET_CV		0x8000	/* set VccSram and VccApp */

/* PMIC handler */
typedef struct {
	XLLP_STATUS_T (*PmicInit)(void);
	XLLP_STATUS_T (*PmicWrite)(XLLP_UINT8_T reg, XLLP_UINT8_T val);
	XLLP_STATUS_T (*PmicRead)(XLLP_UINT8_T reg, XLLP_UINT8_T *pval);
	XLLP_STATUS_T (*PmicSetVccSram)(XLLP_UINT32_T mv);
	XLLP_STATUS_T (*PmicSetVccCore)(XLLP_UINT32_T mv);
	XLLP_STATUS_T (*PmicGetVccSram)(XLLP_UINT32_T *pmv);
	XLLP_STATUS_T (*PmicGetVccCore)(XLLP_UINT32_T *pmv);
}XLLP_PM_PMIC_FUNC_T, *P_XLLP_PM_PMIC_FUNC_T;

/* DVFM context */
typedef struct {
	P_XLLP_PM_SPMU_T	spmu;
	P_XLLP_PM_BPMU_T	bpmu;
	P_XLLP_CLKMGR_T		clkmgr; 
	P_XLLP_DMEM_REGISTER_T	dmemc;
	P_XLLP_SMEM_REGISTER_T  smemc;
	P_XLLP_OST_T            ost;
	XLLP_PM_PMIC_FUNC_T 	pmic_funcs;
}XLLP_PM_DVFM_CONTEXT_T, *P_XLLP_PM_DVFM_CONTEXT_T;

#ifdef __cplusplus
extern "C"
{ 
#endif

/*
 * get which frequencies and voltages will be changed
 */
extern XLLP_UINT32_T XllpPmFreqVolPreSet(
			P_XLLP_PM_FV_INFO cur, 
			P_XLLP_PM_FV_INFO next, 
			XLLP_PM_FV_MODE *pmode);

/* 
 * This function is used to change the frequencies. 
 * Both frequencies and voltages changes will be performed here.
 */
extern XLLP_STATUS_T XllpPmFreqVolChange(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			P_XLLP_PM_FV_INFO cur,
			P_XLLP_PM_FV_INFO next,
			XLLP_PM_FV_MODE mode);

/* 
 * This function is used to change the frequencies except HSS. 
 * Both frequencies and voltages changes will be performed here.
 */
extern XLLP_STATUS_T XllpPmFreqVolChangeNoHss(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			P_XLLP_PM_FV_INFO cur,
			P_XLLP_PM_FV_INFO next,
			XLLP_PM_FV_MODE mode);


/*
 * This function is used current frequencies and 
 * voltages information. Both frequencies and voltages 
 * info will be obtained here.
 */
extern XLLP_STATUS_T XllpPmFreqVolGet (
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			P_XLLP_PM_FV_INFO info,
			XLLP_PM_FV_MODE *mode);

/*
 * This function is used to change HSS. It is used by 
 * OS LCD driver's interrupt handler to call it when 
 * handling LCD EOF interrupts.
 */
extern XLLP_STATUS_T XllpPmHssChange(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			XLLP_UINT32_T hss);

/*
 * This function is used to change the frequencies,
 * including HSS. Voltage changes won't be performed here. 
 */
extern XLLP_STATUS_T XllpPmFreqChange(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			P_XLLP_PM_FV_INFO cur,
			P_XLLP_PM_FV_INFO next,
			XLLP_PM_FV_MODE mode);

/*
 * This function is used to change the frequencies except HSS.
 */
extern XLLP_STATUS_T XllpPmFreqChangeNoHss(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			P_XLLP_PM_FV_INFO cur,
			P_XLLP_PM_FV_INFO next,
			XLLP_PM_FV_MODE mode);

/*
 * This function is used to get the frequencies including 
 * HSS frequency. 
 */ 
extern XLLP_STATUS_T XllpPmFreqGet(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			P_XLLP_PM_FV_INFO info,
			XLLP_PM_FV_MODE *mode);

/*
 * This function is used to change the VCC_CORE and VCC_SRAM. 
 */
extern XLLP_STATUS_T XllpPmVolChange(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			XLLP_UINT32_T vcc_core, 
			XLLP_UINT32_T vcc_sram);

/*
 * This function is used to get the VCC_CORE and VCC_SRAM.
 */
extern XLLP_STATUS_T XllpPmVolGet(
			P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context, 
			XLLP_UINT32_T *vcc_core, 
			XLLP_UINT32_T *vcc_sram);

/*
 * Put system into D0CS mode. 
 */
extern XLLP_STATUS_T XllpPmEnterD0CS(P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context);

/*
 * Let system exit D0CS mode. 
 */
extern XLLP_STATUS_T XllpPmExitD0CS(P_XLLP_PM_DVFM_CONTEXT_T p_pm_dvfm_context);

#ifdef __cplusplus
}
#endif

#endif
