/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef _XLLP_TRUE_IDE_H
#define _XLLP_TRUE_IDE_H

#ifdef __cplusplus
extern "C"{
#endif 
#include "xllp_defs.h"
#include "xllp_gpio.h"
#include "xllp_memc.h"
#include "xllp_mfp.h"

/*
 PC Card Socket Device Handle. This handle is common for all sockets, i.e.
 it is not socket-specific.
*/
typedef struct __xllp_true_ide_S
{	
	P_XLLP_GPIO_T	gpio_regs; /*base address of Monahans GPIO registers*/
	P_XLLP_VUINT32_T mfp_base; /*MFPR base address*/
  	P_XLLP_INT8_T	mfp_rs_db; /*MFP Resource Manager database*/
 	P_XLLP_SMEM_REGISTER_T 	pStaticMemCtl;/*Pointer static memory control*/
 	P_XLLP_VUINT32_T hd_switch_addr; /* Address of the switch of TrueIDE on CF module */
	/*
	pay attention: it is the physical address dma access 
	CS1 and CS2 should be 1  
	*/
	XLLP_UINT32_T dma_phy_addr;
	/*
	IDE device use io_port1, io_port2 and irq to
	access the data on disk
	io_port1 + 0 << addr_shift : PIO RD/WR data
	io_port1 + 1 << addr_shift : Error Register/Features
	io_port1 + 2 << addr_shift : Sector Count/Secor Count
	io_port1 + 3 << addr_shift : Sector No/Sector No
	io_port1 + 4 << addr_shift : Cylinder Low/Cylinder Low
	io_port1 + 5 << addr_shift : Cylinder High/Cylinder High
	io_port1 + 6 << addr_shift : Select Card/Select Card
	io_port1 + 7 << addr_shift : Status/Status
	io_port2 + 6 << addr_shift : Alt Status/Device Control
	*/
	P_XLLP_VUINT32_T io_port1;
	P_XLLP_VUINT32_T io_port2;
	/*
	the address shift for example on zylonite, addr[16:18]
	is connect to A[0:2], so the add_shift should be 16
	*/
	XLLP_UINT8_T add_shift;
	XLLP_INT32_T irq;
 }XLLP_TRUE_IDE_T, *P_XLLP_TRUE_IDE_T;

/*------------------------------------------------------------
 Function:  xllp_true_ide_init
 Purpose:  configure the MFP and set the SMC controller
 Returns:  XLLP_STATUS_T
--------------------------------------------------------------*/
XLLP_STATUS_T xllp_true_ide_init(P_XLLP_TRUE_IDE_T true_ide_handle);

/*------------------------------------------------------------
 Function:  xllp_power_on_true_ide
 Purpose:  Turn on the power supply of the mario disk
 Returns:  
 I/F type: I/F PLAT 2
 note:	this function only open the power of the true ide, the OS driver need wait for
 	a while to wait for the stable of the micro disk!
--------------------------------------------------------------*/
void xllp_power_on_true_ide(P_XLLP_TRUE_IDE_T true_ide_handle);

/*------------------------------------------------------------
 Function:  xllp_power_off_true_ide
 Purpose:  Turn off the power supply of the mario disk
 Returns:  
 I/F type: I/F PLAT 3
 note:	this function only close the power of micro disk. Before power off the 
 	disk, OS driver MUST flush the buffer of micro disk!!! 
--------------------------------------------------------------*/
void xllp_power_off_true_ide(P_XLLP_TRUE_IDE_T true_ide_handle);

/*------------------------------------------------------------
 Function:  xllp_true_ide_dump_regs
 Purpose:  dump the task registers and control registers
 out put parameters: 
 	reg[0]: Error Register
 	reg[1]: Sector Count
 	reg[2]: Sector No
 	reg[3]: Cylinder Low
 	reg[4]: Cylinder High
 	reg[5]: Head
 	reg[6]: Status
 	reg[7]: Alt Status
 Returns:  void
 I/F type: I/F PROC 1
--------------------------------------------------------------*/
void xllp_true_ide_dump_regs(
	P_XLLP_TRUE_IDE_T true_ide_handle, 
	P_XLLP_UINT8_T reg);

void xllp_true_ide_init_proc( P_XLLP_TRUE_IDE_T true_ide_handle );

#ifdef __cplusplus
}
#endif
#endif
