/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/* 
 * xllp_ci.c
 *
 * xllp_ci driver for Monahans 
 *
 */

/*++

Module Name:  $Workfile: xllp_ci.c $

Abstract:  
     contains all primitive functions for Monahans Quick Capture Interface

Notes:Only valid for processor code named Monahans.

--*/
#include "xllp_defs.h"
#include "xllp_serialization.h"
#include "xllp_ost.h"
#include "xllp_ci.h"
#include "xllp_clkmgr.h"

//-------------------------------------------------------------------------------------------------------
//      Declarations
//-------------------------------------------------------------------------------------------------------
#define READ_REG(offset)         (*(volatile unsigned int*)((unsigned)ci_reg_base + (offset)))
#define WRITE_REG(offset, value) do {volatile unsigned int regVal;\
                                    *(volatile unsigned int*)((unsigned)ci_reg_base + (offset)) = (value);\
                                    regVal = *(volatile unsigned int*)((unsigned)ci_reg_base + (offset));}while(0)

//-------------------------------------------------------------------------------------------------------
//      Configuration APIs
//-------------------------------------------------------------------------------------------------------
void XllpCISetFrameRate(unsigned int ci_reg_base, XLLP_CI_FRAME_CAPTURE_RATE frate)
{
    volatile unsigned int value;
    
    // write cicr4
    value = READ_REG(XLLP_CICR4);
    value &= ~(XLLP_CI_CICR4_FR_RATE_SMASK << XLLP_CI_CICR4_FR_RATE_SHIFT);
    value |= (unsigned)frate << XLLP_CI_CICR4_FR_RATE_SHIFT;
    WRITE_REG(XLLP_CICR4, value);   
}

XLLP_CI_FRAME_CAPTURE_RATE XllpCIGetFrameRate(unsigned int ci_reg_base)
{
    volatile unsigned int value;
    value = READ_REG(XLLP_CICR4);
    return (XLLP_CI_FRAME_CAPTURE_RATE)((value >> XLLP_CI_CICR4_FR_RATE_SHIFT) & XLLP_CI_CICR4_FR_RATE_SMASK);
}

void XllpCISetImageFormat(unsigned int ci_reg_base, XLLP_CI_IMAGE_FORMAT input_format, XLLP_CI_IMAGE_FORMAT output_format)
{
    volatile unsigned int value, tbit, rgbt_conv, rgb_conv, rgb_f, ycbcr_f, rgb_bpp, raw_bpp, cspace;

    // write cicr1: preserve ppl value and data width value
    value = READ_REG(XLLP_CICR1);
    value &= ( (XLLP_CI_CICR1_PPL_SMASK << XLLP_CI_CICR1_PPL_SHIFT) | ((XLLP_CI_CICR1_DW_SMASK) << XLLP_CI_CICR1_DW_SHIFT));

    tbit = rgbt_conv = rgb_conv = rgb_f = ycbcr_f = rgb_bpp = raw_bpp = cspace = 0;
    switch(input_format) {
        case XLLP_CI_RAW8:
            cspace = 0;
            raw_bpp = 0;
            break;
        case XLLP_CI_RAW9:
            cspace = 0;
            raw_bpp = 1;
            break;
        case XLLP_CI_RAW10:
            cspace = 0;
            raw_bpp = 2;
            break;
        case XLLP_CI_YCBCR422:
        case XLLP_CI_YCBCR422_PLANAR:
            cspace = 2;
            if (output_format == XLLP_CI_YCBCR422_PLANAR) {
                ycbcr_f = 1;
            }
            break;
        case XLLP_CI_RGB444:
            cspace = 1;
            rgb_bpp = 0;
            break;  
        case XLLP_CI_RGB555:
            cspace = 1;
            rgb_bpp = 1;
            if (output_format == XLLP_CI_RGBT555_0) {
                rgbt_conv = 2;
                tbit = 0;
            } 
            else if (output_format == XLLP_CI_RGBT555_1) {
                rgbt_conv = 2;
                tbit = 1;
            }
            break;  
        case XLLP_CI_RGB565:
            cspace = 3;
            rgb_bpp = 2;
            rgb_f = 1;
            break;  
        case XLLP_CI_RGB666:
            cspace = 1;
            rgb_bpp = 3;
            if (output_format == XLLP_CI_RGB666_PACKED) {
                rgb_f = 1;
            }
            break;  
        case XLLP_CI_RGB888:
        case XLLP_CI_RGB888_PACKED:
            cspace = 1;
            rgb_bpp = 4;
            break;  
        default:
            break;
    }

    switch(input_format) {
        case XLLP_CI_RAW8:
        case XLLP_CI_RAW9:
        case XLLP_CI_RAW10:
            if (output_format == XLLP_CI_YCBCR422_PLANAR){
                ycbcr_f = 1;
            }
        case XLLP_CI_RGB888:
        case XLLP_CI_RGB888_PACKED:
            switch(output_format) {
                case XLLP_CI_RGB888_PACKED:
                    rgb_f = 1;
                    break;
                case XLLP_CI_RGBT888_0:
                    rgbt_conv = 1;
                    tbit = 0;
                    break;
                case XLLP_CI_RGBT888_1:
                    rgbt_conv = 1;
                    tbit = 1;
                    break;
                case XLLP_CI_RGB666:
                    rgb_conv = 1;
                    break;
                case XLLP_CI_RGB565:
                    rgb_conv = 2;
                    break;
                case XLLP_CI_RGB555:
                    rgb_conv = 3;
                    break;
                case XLLP_CI_RGB444:
                    rgb_conv = 4;
                    break;
                default:
                    break;
            }
        default:
            break;
    }
            
    value |= (tbit==1) ? XLLP_CI_CICR1_TBIT : 0;
    value |= rgbt_conv << XLLP_CI_CICR1_RGBT_CONV_SHIFT;
    value |= rgb_conv << XLLP_CI_CICR1_RGB_CONV_SHIFT;
    value |= (rgb_f==1) ? XLLP_CI_CICR1_RBG_F : 0;
    value |= (ycbcr_f==1) ? XLLP_CI_CICR1_YCBCR_F : 0;
    value |= rgb_bpp << XLLP_CI_CICR1_RGB_BPP_SHIFT;
    value |= raw_bpp << XLLP_CI_CICR1_RAW_BPP_SHIFT;
    value |= cspace << XLLP_CI_CICR1_COLOR_SP_SHIFT;
    WRITE_REG(XLLP_CICR1, value);   

    return; 
}

void XllpCISetMode(unsigned int ci_reg_base, XLLP_CI_MODE mode, XLLP_CI_DATA_WIDTH data_width)
{
    volatile unsigned int value;

    // write mode field in cicr0    
    value = READ_REG(XLLP_CICR0);
    value &= ~(XLLP_CI_CICR0_SIM_SMASK << XLLP_CI_CICR0_SIM_SHIFT);
    value |= (unsigned int)mode << XLLP_CI_CICR0_SIM_SHIFT;
    WRITE_REG(XLLP_CICR0, value);   
    
    // write data width cicr1
    value = READ_REG(XLLP_CICR1);
    value &= ~(XLLP_CI_CICR1_DW_SMASK << XLLP_CI_CICR1_DW_SHIFT);
    value |= ((unsigned)data_width) << XLLP_CI_CICR1_DW_SHIFT;
    WRITE_REG(XLLP_CICR1, value);   
    return; 
}

void XllpCIConfigureMP(unsigned int ci_reg_base, unsigned int PPL, unsigned int LPF, XLLP_CI_MP_TIMING* timing)
{
    volatile unsigned int value;
    
    // write ppl field in cicr1
    value = READ_REG(XLLP_CICR1);
    value &= ~(XLLP_CI_CICR1_PPL_SMASK << XLLP_CI_CICR1_PPL_SHIFT);
    value |= (PPL & XLLP_CI_CICR1_PPL_SMASK) << XLLP_CI_CICR1_PPL_SHIFT;
    WRITE_REG(XLLP_CICR1, value);   

    // write BLW, ELW in cicr2  
    value = READ_REG(XLLP_CICR2);
    value &= ~((unsigned int)XLLP_CI_CICR2_BLW_SMASK << XLLP_CI_CICR2_BLW_SHIFT | (unsigned int)XLLP_CI_CICR2_ELW_SMASK << XLLP_CI_CICR2_ELW_SHIFT );
    value |= (timing->BLW & XLLP_CI_CICR2_BLW_SMASK) << XLLP_CI_CICR2_BLW_SHIFT;
    WRITE_REG(XLLP_CICR2, value);   
    
    // write BFW, LPF in cicr3
    value = READ_REG(XLLP_CICR3);
    value &= ~((unsigned int)XLLP_CI_CICR3_BFW_SMASK << XLLP_CI_CICR3_BFW_SHIFT | (unsigned int)XLLP_CI_CICR3_LPF_SMASK << XLLP_CI_CICR3_LPF_SHIFT );
    value |= (timing->BFW & XLLP_CI_CICR3_BFW_SMASK) << XLLP_CI_CICR3_BFW_SHIFT;
    value |= (LPF & XLLP_CI_CICR3_LPF_SMASK) << XLLP_CI_CICR3_LPF_SHIFT;
    WRITE_REG(XLLP_CICR3, value);   
    return;
}

void XllpCIConfigureSP(unsigned int ci_reg_base, unsigned int PPL, unsigned int LPF, XLLP_CI_SP_TIMING* timing)
{
    volatile unsigned int value;
    
    // write ppl field in cicr1
    value = READ_REG(XLLP_CICR1);
    value &= ~(XLLP_CI_CICR1_PPL_SMASK << XLLP_CI_CICR1_PPL_SHIFT);
    value |= (PPL & XLLP_CI_CICR1_PPL_SMASK) << XLLP_CI_CICR1_PPL_SHIFT;
    WRITE_REG(XLLP_CICR1, value);   

    // write cicr2
    value |= (timing->BLW & XLLP_CI_CICR2_BLW_SMASK) << XLLP_CI_CICR2_BLW_SHIFT;
    value |= (timing->ELW & XLLP_CI_CICR2_ELW_SMASK) << XLLP_CI_CICR2_ELW_SHIFT;
    value |= (timing->HSW & XLLP_CI_CICR2_HSW_SMASK) << XLLP_CI_CICR2_HSW_SHIFT;
    value |= (timing->BFPW & XLLP_CI_CICR2_BFPW_SMASK) << XLLP_CI_CICR2_BFPW_SHIFT;
    value |= (timing->FSW & XLLP_CI_CICR2_FSW_SMASK) << XLLP_CI_CICR2_FSW_SHIFT;
    WRITE_REG(XLLP_CICR2, value);   
    
    // write cicr3
    value |= (timing->BFW & XLLP_CI_CICR3_BFW_SMASK) << XLLP_CI_CICR3_BFW_SHIFT;
    value |= (timing->EFW & XLLP_CI_CICR3_EFW_SMASK) << XLLP_CI_CICR3_EFW_SHIFT;
    value |= (timing->VSW & XLLP_CI_CICR3_VSW_SMASK) << XLLP_CI_CICR3_VSW_SHIFT;
    value |= (LPF & XLLP_CI_CICR3_LPF_SMASK) << XLLP_CI_CICR3_LPF_SHIFT;
    WRITE_REG(XLLP_CICR3, value);   
    return;
}

void XllpCIConfigureMS(unsigned int ci_reg_base, unsigned int PPL, unsigned int LPF, XLLP_CI_MS_TIMING* timing)
{
    // the operation is same as Master-Parallel
    XllpCIConfigureMP(ci_reg_base, PPL, LPF, (XLLP_CI_MP_TIMING*)timing);
}
    
void XllpCIConfigureEP(unsigned int ci_reg_base, int parity_check)
{
    volatile unsigned int value;

    // write parity_enable field in cicr0   
    value = READ_REG(XLLP_CICR0);
    if (parity_check) {
        value |= XLLP_CI_CICR0_PAR_EN;
    }
    else {
        value &= ~XLLP_CI_CICR0_PAR_EN;
    }
    WRITE_REG(XLLP_CICR0, value);   
    return; 
}

void XllpCIConfigureES(unsigned int ci_reg_base, int parity_check)
{
    // the operationi is same as Embedded-Parallel
    XllpCIConfigureEP(ci_reg_base, parity_check);
}

void XllpCISetClock(unsigned int ci_reg_base, unsigned int clk_regs_base, int pclk_enable, int mclk_enable, unsigned int mclk_mhz)
{
    volatile unsigned int ciclk,  value, div, accr_hss;
    P_XLLP_CLKMGR_T pclk;
    int p;
    unsigned int x;

    // determine the LCLK frequency programmed into the ACCR.
    pclk = (P_XLLP_CLKMGR_T)clk_regs_base;
    accr_hss = (pclk->accr >> 14) & 0x3;

    ciclk = (accr_hss == 0x0)?104*100/2:
            (accr_hss == 0x1)?156*100/2:
                              208*100/2;/*unit: 10KHZ*/
    
    x = ((ciclk / mclk_mhz) - 2);
    p =  x/2;

    if (x%2)
        div = p + 1;
    else
        div = p;

    // write cicr4
    value = READ_REG(XLLP_CICR4);
    value &= ~(XLLP_CI_CICR4_PCLK_EN | XLLP_CI_CICR4_MCLK_EN | XLLP_CI_CICR4_DIV_SMASK<<XLLP_CI_CICR4_DIV_SHIFT);
    value |= (pclk_enable) ? XLLP_CI_CICR4_PCLK_EN : 0;
    value |= (mclk_enable) ? XLLP_CI_CICR4_MCLK_EN : 0;
    value |= div << XLLP_CI_CICR4_DIV_SHIFT;
    WRITE_REG(XLLP_CICR4, value);   
    return; 
}

void XllpCISetPolarity(unsigned int ci_reg_base, int pclk_sample_falling, int hsync_active_low, int vsync_active_low)
{
    volatile unsigned int value;

    // write cicr4
    value = READ_REG(XLLP_CICR4);
    value &= ~(XLLP_CI_CICR4_PCP | XLLP_CI_CICR4_HSP | XLLP_CI_CICR4_VSP);
    value |= (pclk_sample_falling)? XLLP_CI_CICR4_PCP : 0;
    value |= (hsync_active_low) ? XLLP_CI_CICR4_HSP : 0;
    value |= (vsync_active_low) ? XLLP_CI_CICR4_VSP : 0;
    WRITE_REG(XLLP_CICR4, value);   
    return; 
}

void XllpCISetFIFO(unsigned int ci_reg_base, unsigned int timeout, XLLP_CI_FIFO_THRESHOLD threshold, int fifo1_enable,
                   int fifo2_enable)
{
    volatile unsigned int value;

    // write citor
    WRITE_REG(XLLP_CITOR, timeout); 
    
    // write cifr0: always enable fifo 0! also reset input fifo 
    value = READ_REG(XLLP_CIFR0);
    value &= ~(XLLP_CI_CIFR_FEN0 | XLLP_CI_CIFR_FEN1 | XLLP_CI_CIFR_FEN2 | XLLP_CI_CIFR_RESETF | 
                XLLP_CI_CIFR_THL_0_SMASK<<XLLP_CI_CIFR_THL_0_SHIFT);
    value |= (unsigned int)threshold << XLLP_CI_CIFR_THL_0_SHIFT;
    value |= (fifo1_enable) ? XLLP_CI_CIFR_FEN1 : 0;
    value |= (fifo2_enable) ? XLLP_CI_CIFR_FEN2 : 0;
    value |= XLLP_CI_CIFR_RESETF | XLLP_CI_CIFR_FEN0;
    WRITE_REG(XLLP_CIFR0, value);    

    return; 
}

void XllpCIResetFIFO(unsigned int ci_reg_base)
{
    volatile unsigned int value;
    value = READ_REG(XLLP_CIFR0);
    value |= XLLP_CI_CIFR_RESETF;
    WRITE_REG(XLLP_CIFR0, value);    
}

void XllpCISetInterruptMask(unsigned int ci_reg_base, unsigned int mask)
{
    volatile unsigned int value;

    // write mask in cicr0  
    value = READ_REG(XLLP_CICR0);
    value &= ~XLLP_CI_CICR0_INTERRUPT_MASK;
    value |= (XLLP_CICR0_VAL(mask) & XLLP_CI_CICR0_INTERRUPT_MASK);
    WRITE_REG(XLLP_CICR0, value);   

    // write mask in cidcsr0
    value = READ_REG(XLLP_CIDCSR0);
    value |= XLLP_CIDCSR0_VAL(mask);
    WRITE_REG(XLLP_CIDCSR0, value);   

    // write mask in cidcsr1
    value = READ_REG(XLLP_CIDCSR1);
    value |= XLLP_CIDCSR1_VAL(mask);
    WRITE_REG(XLLP_CIDCSR1, value);   

    // write mask in cidcsr2
    value = READ_REG(XLLP_CIDCSR2);
    value |= XLLP_CIDCSR2_VAL(mask);
    WRITE_REG(XLLP_CIDCSR2, value);   

    // write mask in cidcsr3
    value = READ_REG(XLLP_CIDCSR3);
    value |= XLLP_CIDCSR3_VAL(mask);
    WRITE_REG(XLLP_CIDCSR3, value);   
    
    return; 
}

unsigned int XllpCIGetInterruptMask(unsigned int ci_reg_base)
{
    volatile unsigned int cicr0_val, cidcsr0_val, cidcsr1_val, cidcsr2_val, cidcsr3_val;

    // get mask in cicr0  
    cicr0_val = READ_REG(XLLP_CICR0);

    // get mask in cidcsr0
    cidcsr0_val = READ_REG(XLLP_CIDCSR0);

    // get mask in cidcsr1
    cidcsr1_val = READ_REG(XLLP_CIDCSR1);

    // get mask in cidcsr2
    cidcsr2_val = READ_REG(XLLP_CIDCSR2);

    // get mask in cidcsr3
    cidcsr3_val = READ_REG(XLLP_CIDCSR3);

    return XLLP_CI_INT_MASK(cicr0_val, cidcsr0_val, cidcsr1_val, cidcsr2_val, cidcsr3_val);
}

void XllpCIClearInterruptStatus(unsigned int ci_reg_base, unsigned int status)
{
    volatile unsigned int cidcsr0_val, cidcsr1_val, cidcsr2_val, cidcsr3_val;
    
    // write 1 to clear cisr interrupt status
    WRITE_REG(XLLP_CISR, XLLP_CISR_VAL(status));

    // write 1 to clear cifsr interrupt status
    WRITE_REG(XLLP_CIFSR, XLLP_CIFSR_VAL(status));

    // write 1 to clear cidcsr0 interrupt status
    if (XLLP_CIDCSR0_STATUS_VAL(status)) // if DMA channel stopped
    {
        WRITE_REG(XLLP_CIDCSR0, 0); // reset DMA channel
    }
    
    // write 1 to clear cidcsr1 interrupt status
    if (XLLP_CIDCSR1_STATUS_VAL(status)) // if DMA channel stopped
    {
        WRITE_REG(XLLP_CIDCSR1, 0); // reset DMA channel
    }

    // write 1 to clear cidcsr2 interrupt status
    if (XLLP_CIDCSR2_STATUS_VAL(status)) // if DMA channe2 stopped
    {
        WRITE_REG(XLLP_CIDCSR2, 0); // reset DMA channel
    }

    // write 1 to clear cidcsr3 interrupt status
    if (XLLP_CIDCSR3_STATUS_VAL(status)) // if DMA channe3 stopped
    {
        WRITE_REG(XLLP_CIDCSR3, 0); // reset DMA channel
    }
    
	// write 1 to clear Bus error status
	if (status & (1UL << 28)){
		cidcsr0_val = READ_REG(XLLP_CIDCSR0);
		cidcsr0_val |= 0x1;
		WRITE_REG(XLLP_CIDCSR0, cidcsr0_val);

		cidcsr1_val = READ_REG(XLLP_CIDCSR1);
		cidcsr1_val |= 0x1;
		WRITE_REG(XLLP_CIDCSR1, cidcsr1_val);

		cidcsr2_val = READ_REG(XLLP_CIDCSR2);
		cidcsr3_val |= 0x1;
		WRITE_REG(XLLP_CIDCSR2, cidcsr2_val);

		cidcsr3_val = READ_REG(XLLP_CIDCSR3);
		cidcsr3_val |= 0x1;
		WRITE_REG(XLLP_CIDCSR3, cidcsr3_val);
	}
}

unsigned int XllpCIGetInterruptStatus(unsigned int ci_reg_base)
{
    volatile unsigned int cisr_val, cifsr_val,cidcsr0_val, cidcsr1_val, cidcsr2_val, cidcsr3_val;

    // get interrupt status in cisr  
    cisr_val = READ_REG(XLLP_CISR);

    // get interrupt status in cifsr  
    cifsr_val = READ_REG(XLLP_CIFSR);
    
    // get interrupt status in cidcsr0
    cidcsr0_val = READ_REG(XLLP_CIDCSR0);

    // get interrupt status in cidcsr1
    cidcsr1_val = READ_REG(XLLP_CIDCSR1);

    // get interrupt status in cidcsr2
    cidcsr2_val = READ_REG(XLLP_CIDCSR2);

    // get interrupt status in cidcsr3
    cidcsr3_val = READ_REG(XLLP_CIDCSR3);

    return XLLP_CI_INT_STATUS(cisr_val, cifsr_val, cidcsr0_val, cidcsr1_val, cidcsr2_val, cidcsr3_val);
}

void XllpCISetRegisterValue(unsigned int ci_reg_base, unsigned int reg_offset, unsigned int value)
{
    WRITE_REG(reg_offset, value);
}

//-------------------------------------------------------------------------------------------------------
//      Control APIs
//-------------------------------------------------------------------------------------------------------
void XllpCIInit(unsigned int ci_reg_base, unsigned int clk_regs_base)
{
//    P_XLLP_CLKMGR_T pclk;

    (void) clk_regs_base;

    // clear all CI registers
    WRITE_REG(XLLP_CICR0, XLLP_CI_CICR0_INTERRUPT_MASK);   // disable all interrupts
    WRITE_REG(XLLP_CICR1, 0);
    WRITE_REG(XLLP_CICR2, 0);
    WRITE_REG(XLLP_CICR3, 0);
    WRITE_REG(XLLP_CICR4, 0);
    WRITE_REG(XLLP_CISR, ~0);
    WRITE_REG(XLLP_CIFR0,  0);
    WRITE_REG(XLLP_CITOR, 0);

/* driver is responsible for enabling CI clock
    // enable CI clock
    XllpLock(XLLP_RESOURCE_CKEN);
    pclk = (P_XLLP_CLKMGR_T)clk_regs_base;
    pclk->cken |= XLLP_CLKEN_CAMERA;
    XllpUnlock(XLLP_RESOURCE_CKEN);
*/
}

void XllpCIDeInit(unsigned int ci_reg_base, unsigned int clk_regs_base)
{
//    P_XLLP_CLKMGR_T pclk;

    (void) ci_reg_base;
    (void) clk_regs_base;

/* drive is responsible for disabling CI clock
    // disable CI clock
    XllpLock(XLLP_RESOURCE_CKEN);
    pclk = (P_XLLP_CLKMGR_T)clk_regs_base;
    pclk->cken &= ~XLLP_CLKEN_CAMERA;
    XllpUnlock(XLLP_RESOURCE_CKEN);
*/
}

void XllpCIEnable(unsigned int ci_reg_base/*, int dma_en*/)
{
    volatile unsigned int value;

    // write mask in cicr0  
    value = READ_REG(XLLP_CICR0);
    value |= XLLP_CI_CICR0_ENB;
/*
    if (dma_en) {
        value |= XLLP_CI_CICR0_DMA_EN;
    }
*/
    WRITE_REG(XLLP_CICR0, value);   
    return; 
}

void XllpCIDisableComplete(unsigned int ci_reg_base)
{
    volatile unsigned int value;
    
    // Clear the disable control bit.
    value = READ_REG(XLLP_CICR0);
    value &= ~XLLP_CI_CICR0_DIS;
    WRITE_REG( XLLP_CICR0, value );
}

int XllpCIDisable(unsigned int ci_reg_base, unsigned int ost_reg_base, int quick, int wait_for_disable_complete )
{
    volatile unsigned int value, mask;
    int retry;

 
    value = READ_REG(XLLP_CICR0);
	if(!(value&XLLP_CI_CICR0_ENB)){
		return 0;
	}

    value = READ_REG(XLLP_CISR); 
    WRITE_REG(XLLP_CISR, value);
		
    // write control bit in cicr0   
    value = READ_REG(XLLP_CICR0);
    if (quick) {
        value &= ~XLLP_CI_CICR0_ENB;
        mask = XLLP_CI_CISR_CQD;
    }
    else {
        value |= XLLP_CI_CICR0_DIS;
        mask = XLLP_CI_CISR_CDD;
    }
    WRITE_REG(XLLP_CICR0, value);   

    if( wait_for_disable_complete )
    {
        // wait shutdown complete
        retry = 50;
        while ( retry-- > 0 ) {
            value = READ_REG(XLLP_CISR);        
            if ( value & mask ) {
                WRITE_REG(XLLP_CISR, mask);
                return 0;
            }
            XllpOstDelayMilliSeconds((P_XLLP_OST_T)ost_reg_base, 10);
        }
    } else {
        return 0;
    }
    
    return -1; 
}

void XllpCISlaveCaptureEnable(unsigned int ci_reg_base)
{
    volatile unsigned int value;

    // write mask in cicr0  
    value = READ_REG(XLLP_CICR0);
    value |= XLLP_CI_CICR0_SL_CAP_EN;
    WRITE_REG(XLLP_CICR0, value);   
    return; 
}

void XllpCISlaveCaptureDisable(unsigned int ci_reg_base)
{
    volatile unsigned int value;

    // write mask in cicr0  
    value = READ_REG(XLLP_CICR0);
    value &= ~XLLP_CI_CICR0_SL_CAP_EN;
    WRITE_REG(XLLP_CICR0, value);   
    return; 
}

//-------------------------------------------------------------------------------------------------------
//      CI RAW data processing chain APIs
//-------------------------------------------------------------------------------------------------------

XLLP_STATUS_T XllpCIHSUGetHistgram (
                            unsigned int  ci_reg_base,
                            unsigned int  ost_reg_base, 
                  XLLP_CI_HSU_COLOR_TYPE  color_type,
                 XLLP_CI_HSU_MUX_SEL_TYPE mux_select,
                            unsigned int *histogram_lut_buffer_virtual, 
                            unsigned int  histogram_lut_buffer_physical , 
                            unsigned int *histogram_lut_dma_descriptors_virtual, 
                            unsigned int  histogram_lut_dma_descriptors_physical, 
                            unsigned int  histogram_size,
                            unsigned int *histogram_sum)
{
    int tries;
    volatile XLLP_CI_DMAC_DESCRIPTOR_T *dma_desc;
    volatile unsigned int reg_val;

    if (histogram_lut_buffer_virtual == NULL)
        return XLLP_STATUS_WRONG_PARAMETER;
    
    // enable FIFO3
    reg_val = READ_REG(XLLP_CIFR1);
    reg_val |= XLLP_CI_CIFR_FEN3;
    WRITE_REG(XLLP_CIFR1, reg_val);    

    // clear the CI Histgram RAM and EOF3
    reg_val = READ_REG(XLLP_CIHST);
    reg_val |= XLLP_CI_CIHST_CLR_RAM;
    WRITE_REG(XLLP_CIHST, reg_val);
    
    WRITE_REG(XLLP_CIFSR, XLLP_CI_CIFSR_EOF3);

    // wait for ram clear interrupt
    for (tries = 5; tries >= 0;tries--){
        if (READ_REG(XLLP_CISR) & XLLP_CI_CISR_HST_INT)
            break;
        XllpOstDelayMilliSeconds((P_XLLP_OST_T)ost_reg_base, 10);
        
        if (tries == 0)
            return XLLP_STATUS_TIME_OUT;
    }

    // configure the DMA for fifo3
    dma_desc = (XLLP_CI_DMAC_DESCRIPTOR_T*)histogram_lut_dma_descriptors_virtual;
    dma_desc->ddadr = histogram_lut_dma_descriptors_physical;
    dma_desc->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR3;
    dma_desc->dtadr = histogram_lut_buffer_physical;
    dma_desc->dcmd  = XLLP_CI_DMAC_DCMD_INC_TRG_ADDR | histogram_size;
    XllpCIDMALoadDescriptor(ci_reg_base, histogram_lut_dma_descriptors_physical, XLLP_CI_DMA_CHANNEL_3);

    // enable histgram
    reg_val = READ_REG(XLLP_CIHST);
    reg_val &= ~(XLLP_CI_CIHST_COLOR_SEL_SMASK << XLLP_CI_CIHST_COLOR_SEL_SHIFT);
    reg_val |= (color_type << XLLP_CI_CIHST_COLOR_SEL_SHIFT);
    reg_val &= ~(XLLP_CI_CIHST_SCALE_SMASK << XLLP_CI_CIHST_SCALE_SHIFT);
    reg_val |= ( mux_select << XLLP_CI_CIHST_SCALE_SHIFT);
    WRITE_REG(XLLP_CIHST, reg_val);

    // wait for fifo3 end of frame
    // WARNING: to judge if the getting histgram process is completed, don't clear XLLP_CI_INTSTATUS_EOF3 
    //          in  the interrupt handler!!!!!
    for (tries = 100; tries >= 0;tries--){
        reg_val = READ_REG(XLLP_CIFSR);
        if ( reg_val & XLLP_CI_CIFSR_EOF3){
            // clear EOF3
            WRITE_REG(XLLP_CIFSR, XLLP_CI_CIFSR_EOF3);
            break;
        }
        XllpOstDelayMilliSeconds((P_XLLP_OST_T)ost_reg_base, 10);
        
        if (tries == 0)
            return XLLP_STATUS_TIME_OUT;
    }

    // stop Histgram
    reg_val = READ_REG(XLLP_CIHST);
    reg_val &= ~(XLLP_CI_CIHST_COLOR_SEL_SMASK << XLLP_CI_CIHST_COLOR_SEL_SHIFT);
    reg_val |= (0 << XLLP_CI_CIHST_COLOR_SEL_SHIFT);
    WRITE_REG(XLLP_CIHST, reg_val);

    // disable FIFO3
    reg_val = READ_REG(XLLP_CIFR1);
    reg_val &= ~XLLP_CI_CIFR_FEN3;
    WRITE_REG(XLLP_CIFR1, reg_val); 
    
    // return results
    *histogram_sum = READ_REG(XLLP_CISUM);
   
    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCIPSUTagBadPixelAt(unsigned int ci_reg_base, int column, int row)
{
    volatile unsigned int reg_val;
    
    reg_val = READ_REG(XLLP_CIPBUF);

    reg_val &= ~((XLLP_CI_CIPBUF_DEADCOL_SMASK << XLLP_CI_CIPBUF_DEADCOL_SHIFT) |
                 (XLLP_CI_CIPBUF_DEADROW_SMASK << XLLP_CI_CIPBUF_DEADROW_SHIFT));

    reg_val |= ((column << XLLP_CI_CIPBUF_DEADCOL_SHIFT) |
                (row << XLLP_CI_CIPBUF_DEADROW_SHIFT));

    WRITE_REG(XLLP_CIPBUF, reg_val);

    return XLLP_STATUS_SUCCESS;    
}

XLLP_STATUS_T XllpCIPSUEnable(unsigned int ci_reg_base,  XLLP_BOOL_T enable)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CIPSS);

    if (enable == XLLP_TRUE){
        reg_val |= XLLP_CI_CIPSS_PSU_EN;
    }else{
        reg_val &= ~XLLP_CI_CIPSS_PSU_EN;
    }

    WRITE_REG(XLLP_CIPSS, reg_val);

    return XLLP_STATUS_SUCCESS;    
}

XLLP_STATUS_T XllpCICGUSetBlackLevel(unsigned int ci_reg_base, unsigned char black_level)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CICCR);

    reg_val &= ~(XLLP_CI_CICCR_BLC_SMASK << XLLP_CI_CICCR_BLC_SHIFT);
    reg_val |= (black_level << XLLP_CI_CICCR_BLC_SHIFT);

    WRITE_REG(XLLP_CICCR, reg_val);

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCICGUSetAddrMuxSelect(unsigned int ci_reg_base, XLLP_CI_CGU_MUX_SEL_TYPE mux_select)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CICCR);

    reg_val &= ~(XLLP_CI_CICCR_SCALE_SMASK << XLLP_CI_CICCR_SCALE_SHIFT);
    reg_val |= (mux_select << XLLP_CI_CICCR_SCALE_SHIFT);

    WRITE_REG(XLLP_CICCR, reg_val);

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCICGULoadLutRam(
                    unsigned int   ci_reg_base,
                    unsigned int   ost_reg_base, 
                    unsigned int  *histogram_lut_buffer_virtual, 
                    unsigned int   histogram_lut_buffer_physical,
                    unsigned int  *histogram_lut_dma_descriptors_virtual, 
                    unsigned int   histogram_lut_dma_descriptors_physical, 
                    unsigned char *lut_ram)
{
    int tries;
    volatile XLLP_CI_DMAC_DESCRIPTOR_T *dma_desc;
    volatile unsigned int reg_val;
    int i;
    unsigned char *src,*dst;

    // enable FIFO3
    reg_val = READ_REG(XLLP_CIFR1);
    reg_val |= XLLP_CI_CIFR_FEN3;
    WRITE_REG(XLLP_CIFR1, reg_val);    

    // clear CIFSR.EOF3 at first
    WRITE_REG(XLLP_CIFSR, XLLP_CI_CIFSR_EOF3);

    // memcopy the LUT data to the memory which can be used by DMA
    src = lut_ram;
    dst = (unsigned char*)histogram_lut_buffer_virtual;
    for (i = 0; i < 192; i++){
        *dst = *src;
        dst++;
        src++;
    }
    
    // configure the DMA for fifo3
    dma_desc = (XLLP_CI_DMAC_DESCRIPTOR_T*)histogram_lut_dma_descriptors_virtual;
    dma_desc->ddadr = histogram_lut_dma_descriptors_physical;
    dma_desc->dsadr = histogram_lut_buffer_physical;
    dma_desc->dtadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR3;
    dma_desc->dcmd  = 192;
    XllpCIDMALoadDescriptor(ci_reg_base, histogram_lut_dma_descriptors_physical, XLLP_CI_DMA_CHANNEL_3);
    
    // start loading the LUT ram
    reg_val = READ_REG(XLLP_CICCR);
    reg_val &= ~(XLLP_CI_CICCR_CLUT_SMASK << XLLP_CI_CICCR_CLUT_SHIFT);
    reg_val |= (XLLP_CI_CICCR_CLUT_RED << XLLP_CI_CICCR_CLUT_SHIFT);
    reg_val |= XLLP_CI_CICCR_LUT_LD;
    WRITE_REG(XLLP_CICCR, reg_val);

    // wait for LUT ram loading process completed
    for (tries = 100; tries >= 0;tries--){
        if ((READ_REG(XLLP_CICCR) & XLLP_CI_CICCR_LUT_LD) == 0){
            if ( READ_REG(XLLP_CIFSR) & XLLP_CI_CIFSR_EOF3){
                // clear EOF3
                WRITE_REG(XLLP_CIFSR, XLLP_CI_CIFSR_EOF3);
                break;
            }
        }
        XllpOstDelayMilliSeconds((P_XLLP_OST_T)ost_reg_base, 10);
        
        if (tries == 0)
            return XLLP_STATUS_TIME_OUT;
    }

    // disable FIFO3
    reg_val = READ_REG(XLLP_CIFR1);
    reg_val &= ~XLLP_CI_CIFR_FEN3;
    WRITE_REG(XLLP_CIFR1, reg_val);    

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCICGUEnable(unsigned int ci_reg_base, XLLP_BOOL_T enable)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CICCR);

    if (enable == XLLP_TRUE){
        reg_val |= XLLP_CI_CICCR_EN;
    }else{
        reg_val &= ~XLLP_CI_CICCR_EN;
    }

    WRITE_REG(XLLP_CICCR, reg_val);

    return XLLP_STATUS_SUCCESS;    
}

XLLP_STATUS_T XllpCISSUSetScale(unsigned int ci_reg_base, XLLP_CI_SSU_SCALE scale)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CISSC);

    reg_val &= ~(XLLP_CI_CISSC_SCALE_SMASK << XLLP_CI_CISSC_SCALE_SHIFT);
    reg_val |= (scale << XLLP_CI_CISSC_SCALE_SHIFT);

    WRITE_REG(XLLP_CISSC, reg_val);

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCICMUSetColorCorrectionCoe(unsigned int ci_reg_base, XLLP_CI_CMU_COE_MATRIX *coe_matrix)
{
    volatile unsigned int reg_val;
    
    // set k00, k01, k02 to CICMC0
    reg_val = READ_REG(XLLP_CICMC0);
    reg_val &= ~((XLLP_CI_CICMC0_COF02_SMASK << XLLP_CI_CICMC0_COF02_SHIFT) |
                 (XLLP_CI_CICMC0_COF01_SMASK << XLLP_CI_CICMC0_COF01_SHIFT) |
                 (XLLP_CI_CICMC0_COF00_SMASK << XLLP_CI_CICMC0_COF00_SHIFT));
    reg_val |=  ((coe_matrix->k00 << XLLP_CI_CICMC0_COF02_SHIFT) |
                 (coe_matrix->k01 << XLLP_CI_CICMC0_COF01_SHIFT) |
                 (coe_matrix->k02 << XLLP_CI_CICMC0_COF00_SHIFT));
    WRITE_REG(XLLP_CICMC0, reg_val);

    // set k10, k11, k12 to CICMC1
    reg_val = READ_REG(XLLP_CICMC1);
    reg_val &= ~((XLLP_CI_CICMC1_COF12_SMASK << XLLP_CI_CICMC1_COF12_SHIFT) |
                 (XLLP_CI_CICMC1_COF11_SMASK << XLLP_CI_CICMC1_COF11_SHIFT) |
                 (XLLP_CI_CICMC1_COF10_SMASK << XLLP_CI_CICMC1_COF10_SHIFT));
    reg_val |=  ((coe_matrix->k10 << XLLP_CI_CICMC1_COF12_SHIFT) |
                 (coe_matrix->k11 << XLLP_CI_CICMC1_COF11_SHIFT) |
                 (coe_matrix->k12 << XLLP_CI_CICMC1_COF10_SHIFT));
    WRITE_REG(XLLP_CICMC1, reg_val);

    // set k20, k21, k22 to CICMC2
    reg_val = READ_REG(XLLP_CICMC2);
    reg_val &= ~((XLLP_CI_CICMC2_COF22_SMASK << XLLP_CI_CICMC2_COF22_SHIFT) |
                 (XLLP_CI_CICMC2_COF21_SMASK << XLLP_CI_CICMC2_COF21_SHIFT) |
                 (XLLP_CI_CICMC2_COF20_SMASK << XLLP_CI_CICMC2_COF20_SHIFT));
    reg_val |=  ((coe_matrix->k20 << XLLP_CI_CICMC2_COF22_SHIFT) |
                 (coe_matrix->k21 << XLLP_CI_CICMC2_COF21_SHIFT) |
                 (coe_matrix->k22 << XLLP_CI_CICMC2_COF20_SHIFT));
    WRITE_REG(XLLP_CICMC2, reg_val);

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCICMUEnable(signed int ci_reg_base, XLLP_CI_CMU_USAGE cmu_usage)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CICMR);

    reg_val &= ~(XLLP_CI_CICMR_DMODE_SMASK << XLLP_CI_CICMR_DMODE_SHIFT);
    reg_val |= (cmu_usage << XLLP_CI_CICMR_DMODE_SHIFT);

    WRITE_REG(XLLP_CICMR, reg_val);

    return XLLP_STATUS_SUCCESS;
}

//-------------------------------------------------------------------------------------------------------
//      CI dedicated DMAC APIs
//-------------------------------------------------------------------------------------------------------
XLLP_STATUS_T XllpCIDMALoadDescriptor(unsigned int ci_reg_base, unsigned int dma_desc_phy, XLLP_CI_DMA_CHANNEL channel)
{
    WRITE_REG(XLLP_CIDADR0 + channel*16, dma_desc_phy);

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T XllpCIDMASetBranch (
           unsigned int ci_reg_base, 
           unsigned int branch_to_dma_desc_phy, 
            XLLP_BOOL_T branch_int_enable, 
            XLLP_BOOL_T branch_after_cur_frame,  
    XLLP_CI_DMA_CHANNEL channel)
{
    volatile unsigned int reg_val;

    // note: to enable Branch Interrupt, XLLP_CI_INT_BS should be enabled 
    // as well as branch_int_enable is set as XLLP_TRUE
    reg_val = (branch_to_dma_desc_phy & XLLP_CI_CIDBR_SRCADDR_SMASK);
    reg_val |= (branch_int_enable == XLLP_TRUE)?XLLP_CI_CIDBR_BINT:0;
    reg_val |= (branch_after_cur_frame == XLLP_TRUE)?XLLP_CI_CIDBR_BRA:0;

    WRITE_REG(XLLP_CIDBR0 + 4*channel, reg_val);

    return XLLP_STATUS_SUCCESS;
}

//-------------------------------------------------------------------------------------------------------
//      CI/GCU/LCD synchronization APIs
//-------------------------------------------------------------------------------------------------------
XLLP_STATUS_T XllpCISyncToLCD(unsigned int ci_reg_base, XLLP_CI_SYNC_TO_LCD_TYPE lcd_layer_type)
{
    volatile unsigned int reg_val;

    reg_val = READ_REG(XLLP_CICR0);
    reg_val &= ~(XLLP_CI_CICR0_LCD_MODE_SMASK << XLLP_CI_CICR0_LCD_MODE_SHIFT);
    reg_val |= (lcd_layer_type << XLLP_CI_CICR0_LCD_MODE_SHIFT);
    WRITE_REG(XLLP_CICR0, reg_val);

    return XLLP_STATUS_SUCCESS;
}

