/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**  FILENAME:       xllp_irda_cpu.c
**
**  PURPOSE:        Definitions the low-level primitives that access the
**		            IrDA(SIR and FIR) registers
**
**
******************************************************************************/


#include <xllp_defs.h>
#include <xllp_serialization.h>

#include "xllp_irda_proc.h"
#include "xllp_uart_proc.h"

/******************************************************************************
 * 	XllpSIRTxRxEnable
 * 	Enable/Disable Tx/Rx of FIR device
 *
 * 	Note: This functions just support SIR device.
 *	      The param Handle just need pUARTRegs/IrDAType filled with valid 
 *	      value.
 *	      TxEnable and RxEnable shouldn't be set XLLP_TURE simultaneously.
 * 	
 * 	Param:
 * 		Handle: Pointer to the XLLP_IRDA_HANDLE_T
 * 		TxEnable: 
 * 			XLLP_TRUE:  Enable Tx
 * 			XLLP_FALSE: Disable Tx
 * 		RxEnable: 
 * 			XLLP_TRUE:  Enable Rx
 * 			XLLP_FALSE: Disable Rx
 * 	Return:
 * 		the status of operation
 ******************************************************************************
 */
XLLP_STATUS_T XllpSIRTxRxEnable(P_XLLP_IRDA_HANDLE_T Handle, 
		XLLP_BOOL_T TxEnable, 
		XLLP_BOOL_T RxEnable)
{
	XLLP_VUINT32_T	tmp;	// Access STUART ISR register;
	P_XLLP_UART_T	pUARTRegs = Handle->pUARTHandle->pUARTRegs;
	
	// This functions just for FIR	
	if (XLLP_FIR == Handle->IrDAType)
		return XLLP_STATUS_WRONG_PARAMETER;

	// Make sure valid param	
	if (NULL == pUARTRegs)
		return XLLP_STATUS_WRONG_PARAMETER;

	// Tx and Rx can't be enabled simultaneously
	if ((XLLP_TRUE == TxEnable) && (XLLP_TRUE == RxEnable)) 
		return XLLP_STATUS_WRONG_PARAMETER;

	tmp = pUARTRegs->isr;
	if (XLLP_TRUE == TxEnable) {		//Enable Tx, disable Rx
		tmp |= XLLP_ISR_XMITIR;
		tmp &= ~XLLP_ISR_RCVEIR;
	} else if (XLLP_TRUE == RxEnable){	//Disable Tx, enable Rx
		tmp &= ~XLLP_ISR_XMITIR;
		tmp |= XLLP_ISR_RCVEIR;
	} else {				//Disable Tx, disable Rx
		tmp &= ~XLLP_ISR_XMITIR;
		tmp &= ~XLLP_ISR_RCVEIR;
	}
	pUARTRegs->isr = tmp;
	
	return 	XLLP_STATUS_SUCCESS;
}	// end of XllpSIRTxRxEnable


