/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:  xllp_memc.c
**
**  PURPOSE:   Source file for XLLP Memory Controller. 
**
*******************************************************************************/

#include "xllp_memc.h"

void XllpMemcRcompInit(P_XLLP_MEMC_T pXllpMemc)
{
	XLLP_UINT32_T regIsr;
	P_XLLP_DMEM_REGISTER_T pReg = (P_XLLP_DMEM_REGISTER_T)pXllpMemc->dmem;
	
	// Initialize Rcomp Registers.
	pReg->rcomp = ((pXllpMemc->rcRange)<<25)
				|XLLP_RCOMP_RCNOP
				|(pXllpMemc->rcEvalInterval);
	// Enable Rcomp interrupt.			
	regIsr = pReg->dmcisr;
		// These bits are write-1-to-clear. Don't touch irrelevant bits.
	regIsr &= ~(XLLP_DMCISR_ORF|XLLP_DMCISR_DLP);  
	pReg->dmcisr = regIsr;
	pReg->dmcier |= XLLP_DMCIER_ERCI; 	
	
	// Force an initial Rcomp evaluation.
	XllpMemcRcompEvalUpdate(pXllpMemc);
	return;
}

XLLP_STATUS_T XllpMemcRcompUpdate(P_XLLP_MEMC_T pXllpMemc)
{
	P_XLLP_DMEM_REGISTER_T pReg = (P_XLLP_DMEM_REGISTER_T)pXllpMemc->dmem;
	if (pReg->rcomp&(XLLP_RCOMP_SWEVAL|XLLP_RCOMP_UPDATE))
		return XLLP_STATUS_INVALID_STATE;
	xlli_Rcomp((XLLP_UINT32_T)pReg);
	return XLLP_STATUS_SUCCESS;
}

void XllpMemcRcompEvalUpdate(P_XLLP_MEMC_T pXllpMemc)
{
	XLLP_UINT32_T regIsr, regIer, regRcomp;	
	P_XLLP_DMEM_REGISTER_T pReg = (P_XLLP_DMEM_REGISTER_T)pXllpMemc->dmem;
	
	// Disable Rcomp interrupt.
	regIer = pReg->dmcier;
	pReg->dmcier = regIer&(~XLLP_DMCIER_ERCI);	

	while(pReg->rcomp&(XLLP_RCOMP_SWEVAL|XLLP_RCOMP_UPDATE));
		
	// Launch the software evaluation.
	regRcomp = pReg->rcomp; 
	// Set the range to be zero so that the DMCISR always get updated.
	pReg->rcomp = (regRcomp|XLLP_RCOMP_SWEVAL)&~XLLP_RCOMP_RCRNG_MASK;
	while(pReg->rcomp&XLLP_RCOMP_SWEVAL);

	// Force an update whatever the evaluation result is.
	xlli_Rcomp((XLLP_UINT32_T)pReg);
	
	// Clear Rcomp interrupt status and restore Rcomp interrupt enabling.
	regIsr = pReg->dmcisr;
		// These bits are write-1-to-clear. Don't touch irrelevant bits.
	regIsr &= ~(XLLP_DMCISR_ORF|XLLP_DMCISR_DLP);
	pReg->rcomp = regRcomp&~XLLP_RCOMP_SWEVAL;  
	pReg->dmcisr = regIsr;	
	pReg->dmcier = regIer;		
	return;
}
