/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/******************************************************************************
**
**  FILENAME:       xllp_mmc_controller.c
**
**  PURPOSE: MMC Xllp API that is controller scope. These APIs contains  all
**           needed to operate MMC controller.
**           
**            
**                  
**
******************************************************************************/
#include "xllp_mmc.h"

/*
void 
XllpMMCStartCtrlClock(P_XLLP_MMC_CONTEXT_T pContext,
                      XLLP_MMC_CONTROLLER_ID_T controller)
{
  XLLP_CLK_MODULE module = (controller==XLLP_MMC_CONTROLLER_1)? \
                            XLLP_CLKEN_MMC : XLLP_CLKEN_MMC2;
  XllpClockEnable(pContext->pClockReg, module, XLLP_TRUE);
  return;
}

void 
XllpMMCStopCtrlClock(P_XLLP_MMC_CONTEXT_T pContext, 
                     XLLP_MMC_CONTROLLER_ID_T controller)
{
  XLLP_CLK_MODULE module = (controller==XLLP_MMC_CONTROLLER_1)? \
                            XLLP_CLKEN_MMC : XLLP_CLKEN_MMC2;
  XllpClockEnable(pContext->pClockReg, module, XLLP_FALSE);
  return;
}
*/

/******************************************************************************
  Description:
    Start MMC bus clock. Only after starting bus clock, communication between 
    controller and card is possible
  Input Parameters:
    pContext--Pointer to MMC context structure
    controller--Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCStartBusClock(P_XLLP_MMC_CONTEXT_T pContext, 
                    XLLP_MMC_CONTROLLER_ID_T controller) 
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
  pMMCReg->mmc_strpcl = XLLP_MMC_STRPCL_START_CLOCK;
  return;
}

/******************************************************************************
  Description:
    Stops MMC bus clock.
  Input Parameters:
    pContext--Pointer to MMC context structure
    controller--Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCStopBusClock(P_XLLP_MMC_CONTEXT_T pContext, 
                    XLLP_MMC_CONTROLLER_ID_T controller)
{
  XLLP_UINT32_T retry_count = 0xff;
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
  
  // Request bus clock stop 
  pMMCReg->mmc_strpcl = XLLP_MMC_STRPCL_STOP_CLOCK;
  
  // Wait until clock actually stops
  while(retry_count--) {
    if (pMMCReg->mmc_i_reg & XLLP_MMC_I_REG_CLK_OFF)
      break;  
    XllpOstDelayMicroSeconds(pContext->pOSTReg, 1); 
  }
  return;
}

/******************************************************************************
  Description:
    Set a new MMC bus clock rate. This function stops and resumes bus clock.
  Input Parameters:
    pContext
      Pointer to MMC context structure
    rate
      bus clock speed
    controller
      Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCSetBusRate(P_XLLP_MMC_CONTEXT_T pContext, 
                  XLLP_MMC_CLOCK_SPEED_T rate, 
                  XLLP_MMC_CONTROLLER_ID_T controller)
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  XllpMMCStopBusClock(pContext,controller);
  pMMCReg->mmc_clkrt = rate;
  XllpMMCStartBusClock(pContext, controller);
  return;
}

/******************************************************************************
  Description:
    The bits of imask which are set  will be unmasked in register MMC_I_MASK. 
    So these types of interrupts will be enabled. Performs a read-modify-write 
    operation that only changes the imask bits in MMC_I_MASK
  Input Parameters:
    pContext
      Pointer to MMC context structure
    imask
      Those interrutps to be enabled
    controller
      Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void
XllpMMCEnableInt(P_XLLP_MMC_CONTEXT_T pContext,
                 XLLP_UINT32_T imask, 
                 XLLP_MMC_CONTROLLER_ID_T controller)
{
  XLLP_UINT32_T   i_reg;
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  i_reg = pMMCReg->mmc_i_mask;
  i_reg &= ~imask;
  pMMCReg->mmc_i_mask = i_reg;
  return;
}

/******************************************************************************
  Description:
    The bits of imask which are set (1) will be masked in register MMC_I_MASK. 
    So these types of interrupts will be disabled. Performs a read-modify-write 
    operation that only changes the imask bits in MMC_I_MASK
  Input Parameters:
    pContext
      Pointer to MMC context structure
    imask
      Those interrutps to be disabled
    controller
      Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCDisableInt(P_XLLP_MMC_CONTEXT_T pContext, 
                  XLLP_UINT32_T imask, 
                  XLLP_MMC_CONTROLLER_ID_T controller)
{
  XLLP_UINT32_T   i_reg;
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  i_reg = pMMCReg->mmc_i_mask;
  i_reg |= imask;
  pMMCReg->mmc_i_mask = i_reg;
  return;
}

/******************************************************************************
  Description:
    Read MMC_I_REG to get status about MMC controller
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
  Output Parameters:
    pIntStatus
      Pointer to a variable which will hold the status read from MMC_I_REG.
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCGetIntStatus(P_XLLP_MMC_CONTEXT_T pContext, 
                    P_XLLP_UINT32_T istat, 
                    XLLP_MMC_CONTROLLER_ID_T controller)
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  *istat = pMMCReg->mmc_i_reg;
  return;
}

/******************************************************************************
  Description:
    Extract the useful response properly by excluding un-needed information
    that is returned in the RES_FIFO
  Input Parameters:
    pContext
      Pointer to MMC context structure
    size
      Response size. Caller must guarantee it is set appropriately. Only CSD 
      and CID response is 120 bits, otherwise 32bits
    controller
      Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCReadResponse(P_XLLP_MMC_CONTEXT_T pContext,
                    P_XLLP_UINT8_T pBuffer, 
                    XLLP_MMC_RESP_SIZE_T size, 
                    XLLP_MMC_CONTROLLER_ID_T controller) 
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
  XLLP_UINT16_T mmc_res = (XLLP_UINT16_T)pMMCReg->mmc_res;
  
  if (XLLP_MMC_RESPONSE_SIZE_32 == size) {
    pBuffer[5] = ((mmc_res >> 8) & 0xff);
    pBuffer[4] = (mmc_res & 0xff);
    
    mmc_res = (XLLP_UINT16_T)pMMCReg->mmc_res;
    pBuffer[3] = ((mmc_res>>8) & 0xff);
    pBuffer[2] = (mmc_res & 0xff);
    
    mmc_res = (XLLP_UINT16_T)pMMCReg->mmc_res;
    pBuffer[1] = ((mmc_res >> 8) & 0xff);
    pBuffer[0] = (mmc_res & 0xff);
  } else if (XLLP_MMC_RESPONSE_SIZE_120 == size) {
    XLLP_INT32_T i=15;
    
    pBuffer[i--] = ((mmc_res >> 8) & 0xff);
    pBuffer[i--] = (mmc_res & 0xff);
    while(i>0) {
      mmc_res = (XLLP_UINT16_T)pMMCReg->mmc_res;
      pBuffer[i--] = ((mmc_res >> 8) & 0xff);
      pBuffer[i--] = (mmc_res & 0xff);
    }
  }
  return;
}

/******************************************************************************
  Description:
    Read MMC_STAT to get current status of the controller
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
  Output Parameters:
    pMMCStat 
      pointer to the variable which will hold the status read from MMC_ STAT
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCReadStatus(P_XLLP_MMC_CONTEXT_T pContext, 
                  P_XLLP_UINT32_T mmcstat, 
                  XLLP_MMC_CONTROLLER_ID_T controller) 
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  *mmcstat = pMMCReg->mmc_stat;
  return;
}

/******************************************************************************
  Description:
    Sets partial full,so that it must be swapped to the other MMC_TXFIFO
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/

void 
XllpMMCSetPartialFull(P_XLLP_MMC_CONTEXT_T pContext, 
                      XLLP_MMC_CONTROLLER_ID_T controller) 
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  pMMCReg->mmc_prtbuf = 0x00000001;
  return;
}
  
/******************************************************************************
  Description:
    Read data from FIFO. Generally, data size used will be 32 bytes. But it 
    does exist to call it with size less than 32, like set password for a 
    lock/unlock operation
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
    size
      The buffer size
  Output Parameters:
    pBuffer
      used to get data from FIFO
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCReadFIFO(P_XLLP_MMC_CONTEXT_T pContext, 
                P_XLLP_UINT8_T pBuffer,
                XLLP_UINT8_T size, 
                XLLP_MMC_CONTROLLER_ID_T controller)
{
  XLLP_UINT8_T i = 0;
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  while (size--) {
    pBuffer[i++] = (XLLP_UINT8_T)pMMCReg->mmc_rxfifo;
  }
  return;
}

/******************************************************************************
  Description:
    Write data to FIFO. Generally, data size used will be 32 bytes. But it 
    does exist to call it with size less than 32, like set password for a 
    lock/unlock operation
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
    pBuffer
      buffer containing data to be written to FIFO
    size
      The buffer size
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCWriteFIFO(P_XLLP_MMC_CONTEXT_T pContext, 
                P_XLLP_UINT8_T pBuffer,
                XLLP_UINT8_T size, 
                XLLP_MMC_CONTROLLER_ID_T controller) 
{
  XLLP_UINT8_T i = 0;
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  while (size--) {
    pMMCReg->mmc_txfifo = pBuffer[i++];
  }
  return;
}

/******************************************************************************
  Description:
    Read the value of MMC_BLKS_REM which contains the number of blocks that 
    were not transferred
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
  Output Parameters:
    pBlkRem
      pointer to the variable which will hold the data read from MMC_BLKS_REM
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCReadBlksRem(P_XLLP_MMC_CONTEXT_T pContext,
                   P_XLLP_UINT32_T count, 
                   XLLP_MMC_CONTROLLER_ID_T controller)
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  *count = (XLLP_UINT16_T)pMMCReg->mmc_blks_rem;
  return; 
}


/******************************************************************************
  Description:
    Set the response timeout value.
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
    timeout  
      the value which will be written into MMC_RESTO
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCSetRespTimeout(P_XLLP_MMC_CONTEXT_T pContext, 
                      XLLP_UINT32_T to, 
                      XLLP_MMC_CONTROLLER_ID_T controller)
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  pMMCReg->mmc_resto = to & 0x7F;
  return;
}

/******************************************************************************
  Description:
    Set the read timeout value
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
    timeout  
      the value which will be written into MMC_RDTO
  Output Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCSetReadTimeout(P_XLLP_MMC_CONTEXT_T pContext, 
                      XLLP_UINT32_T to, 
                      XLLP_MMC_CONTROLLER_ID_T controller) 
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
                          
  pMMCReg->mmc_rdto = to & 0xFFFF;
  return;
}

/******************************************************************************
  Description:
    Set up the registers of the controller to start the transaction to 
    communicate to the card.  The commands are clearly defined in the MMC 
    specification.
  Input Parameters:
    pContext
      Pointer to MMC context structure
    controller
      Controller used in this call
    argument 
      the argument of  the command. MSW is for ARGH and LSW is for ARGL
    flags
      Flags will contain bits indicating command types, response types and 
      transfer types.These bits will be used to set MMC_CMDAT register.
    blockCount 
      data block count which will be transferred, if no data is transferred, 
      it is not used
    blockSize
      data block size which will be transferred, if no data is transferred, 
      it is not used
  utput Parameters:
    None
  Returns:
    None  
*******************************************************************************/
void 
XllpMMCSendCommand(P_XLLP_MMC_CONTEXT_T pContext, 
                  XLLP_UINT32_T cmd, 
                  XLLP_INT32_T argument, 
                  XLLP_UINT32_T flags,
                  XLLP_UINT32_T blockCount, 
                  XLLP_UINT32_T blockSize, 
                  XLLP_MMC_CONTROLLER_ID_T controller)
{
  P_XLLP_MMC_T pMMCReg = (controller==XLLP_MMC_CONTROLLER_1)? \
                          pContext->pMMC1Reg : pContext->pMMC2Reg;
  XLLP_UINT32_T cmd_dat = 0;
  XLLP_UINT32_T transfer_mode;
  
  switch (flags & XLLP_MMC_CMDTYPE_MASK) {
    case XLLP_MMC_NORMAL_COMMAND:
      break;
    case XLLP_MMC_INIT_COMMAND:
      cmd_dat |= XLLP_MMC_CMDAT_INIT;
      break;
    case XLLP_MMC_STOP_COMMAND:
      cmd_dat |= XLLP_MMC_CMDAT_STOP;
      break;
    case XLLP_MMC_SUSPEND_COMMAND:
      cmd_dat |= XLLP_MMC_CMDAT_SDIO_SUSPEND;
      break;
    case XLLP_MMC_RESUME_COMMAND:
      cmd_dat |= XLLP_MMC_CMDAT_SDIO_RESUME;
      break;
  }
  
  // don't support SPI mode currently
  switch (flags & XLLP_MMC_RESPONSE_MASK) {
    case XLLP_MMC_RESPONSE_NONE:
      cmd_dat |= XLLP_MMC_RESPONSE_FMT_NONE;
      break;
    case XLLP_MMC_RESPONSE_R1B:
    case XLLP_MMC_RESPONSE_R5B:
      cmd_dat |= XLLP_MMC_CMDAT_BUSY;
      // Fall through
    case XLLP_MMC_RESPONSE_R1:
    case XLLP_MMC_RESPONSE_R4:
    case XLLP_MMC_RESPONSE_R5:
    case XLLP_MMC_RESPONSE_R6:
      cmd_dat |= XLLP_MMC_RESPONSE_FMT_R1;
      break;
    case XLLP_MMC_RESPONSE_R2:
      cmd_dat |= XLLP_MMC_RESPONSE_FMT_R2;
      break;
    case XLLP_MMC_RESPONSE_R3:
      cmd_dat |= XLLP_MMC_RESPONSE_FMT_R3;
      break;
  }
  
  transfer_mode = flags & XLLP_MMC_TRANSFER_MODE_MASK;
  if (transfer_mode & XLLP_MMC_4BIT_ENABLE)
    cmd_dat |= XLLP_MMC_CMDAT_4DAT;
    
  if (transfer_mode & XLLP_MMC_DMA_ENABLE)
    cmd_dat |= XLLP_MMC_CMDAT_DMA;
    
  if (transfer_mode & XLLP_MMC_DATA_ENABLE) {
    cmd_dat |= XLLP_MMC_CMDAT_DATA;
    cmd_dat &= ~XLLP_MMC_CMDAT_BUSY;
    pMMCReg->mmc_numblk = blockCount;
    pMMCReg->mmc_blklen = blockSize;
  }
  if (transfer_mode & XLLP_MMC_DATA_WRITE)
    cmd_dat |= XLLP_MMC_CMDAT_WRITE;
    
  if (transfer_mode & XLLP_MMC_STREAM_MODE)
    cmd_dat |= XLLP_MMC_CMDAT_STREAM;
    
  if (transfer_mode & XLLP_MMC_SDIO_INT_ENABLE)
    cmd_dat |= XLLP_MMC_CMDAT_SDIO_INT;
    
  pMMCReg->mmc_cmd = cmd;
  pMMCReg->mmc_argh = argument>>16;
  pMMCReg->mmc_argl = argument&0xffff;
  
  // start bus transaction
  pMMCReg->mmc_cmdat = cmd_dat;
}


