/*-------------------------------------------------------------------------
 * Filename:      download.c
 * Version:       $Id: download.c,v 1.1.1.1 2005/03/04 09:36:18 swcai Exp $
 * Copyright:     Copyright (C) 1999, Jan-Derk Bakker
 * Author:        Jan-Derk Bakker <J.D.Bakker@its.tudelft.nl>
 * Description:   download helper stuff
 * Created at:    Mon Aug 23 20:00:00 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sat Mar 25 14:31:16 2000
 *-----------------------------------------------------------------------*/
/*
 * download.c: download helper stuff
 *
 * Copyright (C) 1999 2000 2001
 *   Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl) and
 *   Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: download.c,v 1.1.1.1 2005/03/04 09:36:18 swcai Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/errno.h>
#include <blob/error.h>
#include <blob/md5.h>
#include <blob/md5support.h>
#include <blob/serial.h>
#include <blob/download.h>
#include <blob/command.h>


void PrintSerialSpeed(serial_baud_t speed)
{
	if (speed >= baud_COUNT) printf("(unknown)");
	else printf("%d%i", speed, baud_rates[speed]);
}


int set_download_parameters(char *name, u32 *startAddress, 
				    int *bufLen, int **numRead,
				   u32 **digest)
{
	/* Use of _FLASH_LEN here is WRONG.  This should be
	   the size of the ram buffer */
	if(strncmp(name, "blob", 5) == 0) {
		/* download blob */
		*startAddress = BLOB_RAM_BASE;
		*bufLen = BLOB_FLASH_LEN;
		*numRead = &blob_status.blobSize;
		*digest = blob_status.blob_md5_digest;
		blob_status.blobType = fromDownload;
	} else if(strncmp(name, "param", 6) == 0) {
		/* download param */
		*startAddress = PARAM_RAM_BASE;
		*bufLen = PARAM_FLASH_LEN;
		*numRead = &blob_status.paramSize;
		*digest = blob_status.param_md5_digest;
		blob_status.paramType = fromDownload;
	} else if(strncmp(name, "kernel", 7) == 0) {
		/* download kernel */
		*startAddress = KERNEL_RAM_BASE;
		*bufLen = KERNEL_FLASH_LEN;
		*numRead = &blob_status.kernelSize;
		*digest = blob_status.kernel_md5_digest;
		blob_status.kernelType = fromDownload;
	} else if(strncmp(name, "ramdisk", 8) == 0) {
		/* download ramdisk */
		*startAddress = RAMDISK_RAM_BASE;
		*bufLen = RAMDISK_FLASH_LEN;
		*numRead = &blob_status.ramdiskSize;
		*digest = blob_status.ramdisk_md5_digest;
		blob_status.ramdiskType = fromDownload;
	} else {
		printerror(-EINVAL, name);
		return -EINVAL;
	}

	return 0;
}


int do_reload(char *what)
{
	u32 *dst = 0;
	u32 *src = 0;
	int numWords;

	if(strncmp(what, "blob", 5) == 0) {
		dst = (u32 *)BLOB_RAM_BASE;
		src = (u32 *)BLOB_FLASH_BASE;
		numWords = BLOB_FLASH_LEN / 4;
		blob_status.blobSize = 0;
		blob_status.blobType = fromFlash;
		printf("Loading blob from flash ");
#ifdef PARAM_FLASH_BASE
	} else if(strncmp(what, "param", 6) == 0) {
		dst = (u32 *)PARAM_RAM_BASE;
		src = (u32 *)PARAM_FLASH_BASE;
		numWords = PARAM_FLASH_LEN / 4;
		blob_status.paramSize = 0;
		blob_status.paramType = fromFlash;
		printf("Loading paramater block from flash ");
#endif
	} else if(strncmp(what, "kernel", 7) == 0) {
#if defined(CONFIG_CRAMFS_SUPPORT) || defined(CONFIG_ZIMAGE_SUPPORT) || defined(CONFIG_JFFS2_SUPPORT)
		return load_kernel(&blob_status);
#else
		dst = (u32 *)KERNEL_RAM_BASE;
		src = (u32 *)KERNEL_FLASH_BASE;
		numWords = KERNEL_FLASH_LEN / 4;
		blob_status.kernelSize = 0;
		blob_status.kernelType = fromFlash;
		printf("Loading kernel from flash at 0x%08x to 0x%08x ", KERNEL_FLASH_BASE, KERNEL_RAM_BASE);
#endif
#ifdef RAMDISK_FLASH_BASE
	} else if(strncmp(what, "ramdisk", 8) == 0) {
		dst = (u32 *)RAMDISK_RAM_BASE;
		src = (u32 *)RAMDISK_FLASH_BASE;
		numWords = RAMDISK_FLASH_LEN / 4;
		blob_status.ramdiskSize = 0;
		blob_status.ramdiskType = fromFlash;
		printf("Loading ramdisk from flash at 0x%08x to 0x%08x ", RAMDISK_FLASH_BASE, RAMDISK_RAM_BASE);
#endif
	} else {
		printerror(EINVAL, what);
		return 0;
	}

	MyMemCpy(dst, src, numWords);
	printf(" done\n");

	return 0;
}


void do_md5sum(u32 *addr, size_t len, u32 *digest)
{
#ifdef CONFIG_MD5_SUPPORT
	md5_buffer((const char *)addr, len, digest);
#endif
}


int SetDownloadSpeed(int argc, char *argv[])
{
	if(argc < 2)
		return -ENOPARAMS;

	if(strncmp(argv[1], "1200", 5) == 0) {
		blob_status.downloadSpeed = baud_1200;
	} else if(strncmp(argv[1], "1k2", 4) == 0) {
		blob_status.downloadSpeed = baud_1200;
	} else if(strncmp(argv[1], "9600", 5) == 0) {
		blob_status.downloadSpeed = baud_9600;
	} else if(strncmp(argv[1], "9k6", 4) == 0) {
		blob_status.downloadSpeed = baud_9600;
	} else if(strncmp(argv[1], "19200", 6) == 0) {
		blob_status.downloadSpeed = baud_19200;
	} else if(strncmp(argv[1], "19k2", 5) == 0) {
		blob_status.downloadSpeed = baud_19200;
	} else if(strncmp(argv[1], "38400", 7) == 0) {
		blob_status.downloadSpeed = baud_38400;
	} else if(strncmp(argv[1], "38k4", 5) == 0) {
		blob_status.downloadSpeed = baud_38400;
	} else if(strncmp(argv[1], "57600", 6) == 0) {
		blob_status.downloadSpeed = baud_57600;
	} else if(strncmp(argv[1], "57k6", 5) == 0) {
		blob_status.downloadSpeed = baud_57600;
	} else if(strncmp(argv[1], "115200", 7) == 0) {
		blob_status.downloadSpeed = baud_115200;
	} else if(strncmp(argv[1], "115k2", 6) == 0) {
		blob_status.downloadSpeed = baud_115200;
	} else if(strncmp(argv[1], "230400", 7) == 0) {
		blob_status.downloadSpeed = baud_230400;
	} else if(strncmp(argv[1], "230k4", 6) == 0) {
		blob_status.downloadSpeed = baud_230400;
	} else {
		return -EINVAL;
	}

	printf("Download speed set to ");
	PrintSerialSpeed(blob_status.downloadSpeed);
	printf(" baud\n");

	return 0;
}

static char speedhelp[] = "speed [baudrate]\n"
"Set download speed. Valid baudrates are:\n"
"1200, 9600, 19200, 38400, 57600, 115200, 230400,\n"
" 1k2,  9k6,  19k2,  38k4,  57k6,  115k2,  230k4\n";

__commandlist(SetDownloadSpeed, "speed", speedhelp);

