/*
 *  ether-pxausb.c : "eth-over-usb" driver for BLOB
 *
 *  Copyright (c) 2003, Intel Corporation (yu.tang@intel.com)
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/types.h>
#include <blob/time.h>
#include <blob/serial.h>
#include <blob/util.h>
#include <blob/init.h>
#include <blob/pxa_usb.h>
#include <net/ether.h>
#include <net/net.h>

#include "mv_u2o_ctl.h"

static int usb_rsize=64;
static int usb_wsize=64;

/* cable connection */
extern int usb_connected;
extern int usb_speed;
extern int tftp_t;
extern int arp_t;

/* receive buffer management */
static char rx_buf[ETH_BUF_SIZE];
static int  rx_size = 0;
static int  rx_done = 0 ;

/* transmit buffer management */
static int tx_done = 0 ;

#if 0	//test
__u16 usb_vendor_id = 0x1286;
__u16 usb_product_id = 0x8001;
#else	//test
__u16 usb_vendor_id = 0x8086;
__u16 usb_product_id = 0x07d3;
#endif	//tes

extern void u2o_start(void);

static void usb_eth_tx_callback(int flag, int size)
{
	tx_done = 1;
}

static void usb_eth_rx_callback(int flag, int size)
{
	int left;

	/*printf("%s flag %d size %d left %d rx_done %d rx_size %d", 
		__func__, flag, size, left, rx_done, rx_size); */
	if( flag != 0 ) {
		//printf("%s, flag = %d\n", __FUNCTION__, flag);
		return;
	}

	if (rx_done) { 
		/* printf("overwrite\n"); */
		/* drop */
		rx_done = 0;
		rx_size = 0;
	}

	rx_size += size;

done:
	rx_done = 1;

	/* setup to receive */
	ep2_recv(rx_buf, ETH_BUF_SIZE, usb_eth_rx_callback);

	return;
}

void usb_driver_speed(int speed)
{
	desc_t * pdesc = pxa_usb_get_descriptor_ptr();
	config_desc_t *cfg;
	intf_desc_t *intf;
	ep_desc_t *ep;

	cfg = (config_desc_t*) (pdesc->cdb);
	intf = (config_desc_t *)(cfg + 1);
	ep = (ep_desc_t *) (intf + 1);
	
	if( speed == USB_SPEED_HIGH ){
		ep[0].wMaxPacketSize      = make_word( 512 );
		ep[1].wMaxPacketSize      = make_word( 512 );
	} else {
		ep[0].wMaxPacketSize      = make_word( 64 );
		ep[1].wMaxPacketSize      = make_word( 64 );
	}
}

void usb_driver_reset(void)
{
	desc_t * pdesc = pxa_usb_get_descriptor_ptr();
	config_desc_t *cfg;
	intf_desc_t *intf;
	ep_desc_t *ep;

	/* setup device descriptor */
	pdesc->dev.idVendor	= usb_vendor_id;
	pdesc->dev.idProduct    = usb_product_id;
	pdesc->dev.bNumConfigurations = 1;

	cfg = (config_desc_t*) (pdesc->cdb);

	cfg->bLength             = sizeof( config_desc_t );
	cfg->bDescriptorType     = USB_DESC_CONFIG;
	cfg->wTotalLength        = make_word_c( sizeof(config_desc_t) +
						   sizeof(intf_desc_t) * 1+
						   sizeof(ep_desc_t) * 2);
	cfg->bNumInterfaces      = 1;
	cfg->bConfigurationValue = 1;
	cfg->iConfiguration      = 0;
	cfg->bmAttributes        = USB_CONFIG_BUSPOWERED;
	cfg->MaxPower            = USB_POWER( 500 );

	intf = (intf_desc_t *) ( cfg + 1);
	intf->bLength            = sizeof( intf_desc_t );
	intf->bDescriptorType    = USB_DESC_INTERFACE;
	intf->bInterfaceNumber   = 0; 
	intf->bAlternateSetting  = 0;
	intf->bNumEndpoints      = 2;
	intf->bInterfaceClass    = 0xFF; 
	intf->bInterfaceSubClass = 0;
	intf->bInterfaceProtocol = 0;
	intf->iInterface         = 0;

	ep = (ep_desc_t *) (intf + 1);
	ep[0].bLength             = sizeof( ep_desc_t );
	ep[0].bDescriptorType     = USB_DESC_ENDPOINT;
	ep[0].bEndpointAddress    = USB_EP_ADDRESS( 1, USB_IN );
	ep[0].bmAttributes        = USB_EP_BULK;
	if( usb_speed == USB_SPEED_HIGH )
		ep[0].wMaxPacketSize      = make_word( 512 );
	else
		ep[0].wMaxPacketSize      = make_word( 64 );
	ep[0].bInterval           = 0;

	ep[1].bLength             = sizeof( ep_desc_t );
	ep[1].bDescriptorType     = USB_DESC_ENDPOINT;
	ep[1].bEndpointAddress    = USB_EP_ADDRESS( 1, USB_OUT );
	ep[1].bmAttributes        = USB_EP_BULK;
	if( usb_speed == USB_SPEED_HIGH )
		ep[1].wMaxPacketSize      = make_word( 512 );
	else
		ep[1].wMaxPacketSize      = make_word( 64 );
	ep[1].bInterval           = 0;

	/* reset buffer */
	rx_done = 0;
	rx_size = 0;

}

void ep2_begin(void)
{
	/* setup to receive */
	if(usb_connected) {
		ep2_recv(rx_buf, ETH_BUF_SIZE, usb_eth_rx_callback);
	}
}

static inline int check_usb_connection(void)
{
	unsigned int cnt = 0;

	if (usb_connected) return 0;
	else 
		u2o_start();

	/* waiting util connected */
	printf("***** Plug-in USB cable & config usbdnet now ****** \n");
	
	while (!usb_connected) {
		if ( ICPR3 & 0x1 ) {
			u2o_int_hndlr(0x11, 0);
		}
	}

	while ( cnt < 0x1000) {
		if ( ICPR3 & 0x1 ) {
			u2o_int_hndlr(0x11, 0);
			cnt = 0;
		}
		cnt ++;
	}

	printf("exit check_usb_connection:%d\n", usb_connected);
	return 0;
}

static int usb_eth_init(u32 base)
{
	usb_connected = 0;
	
#if defined(CONFIG_USBDNET_SUPPORT) || defined(CONFIG_USBDNET2_SUPPORT)	//test
	/* set default ip address */
	clientipaddress[0] = 192;
	clientipaddress[1] = 168;
	clientipaddress[2] = 100;
	clientipaddress[3] = 2;

	serveripaddress[0] = 192;
	serveripaddress[1] = 168;
	serveripaddress[2] = 100;
	serveripaddress[3] = 1;

	printf( "\nNet over USB!!!\n" );
#else
	/* set default ip address */
	clientipaddress[0] = 192;
	clientipaddress[1] = 168;
	clientipaddress[2] = 1;
	clientipaddress[3] = 101;

	serveripaddress[0] = 192;
	serveripaddress[1] = 168;
	serveripaddress[2] = 1;
	serveripaddress[3] = 100;
	printf( "\nNet over ether!!!\n" );
#endif	//CONFIG_USBDNET_SUPPORT	//test


	/* initialize PXA USB controller */
	pxa_usb_open("usb-eth");
	pxa_usb_start();

	return 0;
}
static int loop=0;

extern int req_pending;
void dump_buffer(char *buf, unsigned length);

static int usb_eth_rx(u8 *inbuf, int size)
{
	int i=1000;

	if(!usb_connected)check_usb_connection();

	do{
		if ( ICPR3 & 0x1 ) {
			u2o_int_hndlr(0x11, 0);
			i=0;
		}

		DELAY();
		if(rx_done) break;
		i--;
	}while (!rx_done && (i>0));

		
	if (size > rx_size) 
		size = rx_size;

	memcpy(inbuf, rx_buf, size);

	rx_done = 0;
	rx_size = 0;

	return size;
}

static int usb_eth_tx(u8 *outbuf, int size) 
{
	int i,tx_size = size;

	check_usb_connection();

	//if ( 0 == (size % usb_wsize) ) 
	//	size ++;
	tx_done = 0;

	ep1_send(outbuf, size, usb_eth_tx_callback);
	
	return tx_size;
}

static int usb_eth_set_addr( u8 hw_addr[6] )
{
	memcpy(hwaddress, hw_addr, 6);
	return 0;
}

static int usb_eth_get_addr( u8 hw_addr[6] )
{
	memcpy(hw_addr, hwaddress, 6);
	return 0;
}

/* export ethernet driver */
ether_driver_t mvu2o_ether_driver = {
	init:		usb_eth_init,
	rcv:		usb_eth_rx,
	snd:		usb_eth_tx,
	set_addr:	usb_eth_set_addr,
	get_addr:	usb_eth_get_addr,
	base:		0,
	name:		"usb-eth"
};
