/*
 * led_mmap.c: simple memory mapped LED control functions
 * 
 * Copyright (C) 2002, 2003 by SOMA Networks Inc.
 *
 * $Id: led-mmap.c,v 1.1 2003/02/13 01:10:53 erikm Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: led-mmap.c,v 1.1 2003/02/13 01:10:53 erikm Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/led.h>
#include <blob/init.h>

static int led_state;
static int led_locked;

static void mmap_led_on(void)
{
	volatile u16 *ledp;

	if(led_locked)
		return;

	ledp = (volatile u16 *)LED_ADDR;
	*ledp = LED_ON;

	led_state = 1;
}

static void mmap_led_off(void)
{
	volatile u16 *ledp;

	if(led_locked)
		return;

	ledp = (volatile u16 *)LED_ADDR;
	*ledp = LED_OFF;

	led_state = 0;
}

static void mmap_led_toggle(void)
{
	if(led_state)
		mmap_led_off();
	else
		mmap_led_on();

	return;
}

static void mmap_led_lock(void)
{
	led_locked = 1;
}

static void mmap_led_unlock(void)
{
	led_locked = 0;
}

led_driver_t mmap_led_driver = {
	led_on: 	mmap_led_on,
	led_off:	mmap_led_off,
	led_toggle:	mmap_led_toggle,
	led_lock:  	mmap_led_lock,
	led_unlock:	mmap_led_unlock,
};
