/*
 * led-s3c2500.c: simple LED control functions
 *
 * Copyright (C) 1999 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003  Russ Dill (Russ.Dill@asu.edu)
 *
 * $Id: led-s3c2500.c,v 1.1 2003/09/04 18:03:10 russd Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * All functions assume that the LED is properly initialised by the code
 * in ledasm.S.
 *
 */

#ident "$Id: led-s3c2500.c,v 1.1 2003/09/04 18:03:10 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/led.h>
#include <blob/init.h>


static int led_locked;

static void s3c2500_led_on(void)
{
	if(led_locked) return;
	IOPDATA1 &= ~(1L);
}

static void s3c2500_led_off(void)
{
	if(led_locked) return;
	IOPDATA1 |= 1L;
}

static void s3c2500_led_toggle(void)
{
	if (led_locked) return;
	IOPDATA1 ^= 1L;
}

static void s3c2500_led_lock(void)
{
	led_locked = 1;
}

static void s3c2500_led_unlock(void)
{
	led_locked = 0;
}

led_driver_t s3c2500_gpio_led_driver = {
    led_on:     s3c2500_led_on,
    led_off:    s3c2500_led_off,
    led_toggle: s3c2500_led_toggle,
    led_lock:   s3c2500_led_lock,
    led_unlock: s3c2500_led_unlock,
};
