/*
 * reboot-s3c2500.c: Reboot s3c2500
 *
 * Copyright (C) 2001 2003  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003  Russ Dill (Russ.Dill@asu.edu)
 *
 * $Id: reboot-s3c2500.c,v 1.1 2003/09/04 18:07:07 russd Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: reboot-s3c2500.c,v 1.1 2003/09/04 18:07:07 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/reboot.h>


/* Doesn't seem to have a reset, so do it via watchdog */
void s3c2500_reboot(void)
{
	/* Errata for S3C2500X01: WatchDog Timer reset mode operation fail
	 * Symptom
	 * When the S3C2500X uses the PLL output clock as the system
	 * clock, the watchdog timer doesn't operate in the reset mode.
	 * If you set the reset period to "1s", the real time to drive
	 * the chip into reset state can not be anticipated.
	 * Program Guide
	 * It is recommended that you should not use the watchdog timer
	 * in the reset mode.
	 *
	 * If someone understands this korglish, tell me, because I have
	 * no clue what they are talking about, or how to work around it.
	 */
	int i;
	typedef void (*func_t)(void);
	func_t base = 0x0;
	WDT = (1 << 31) | (1 << 30) | 1;
	for(i = 0; i < 10000; i++);
	/* Give up and jump to 0x0000 */
	base();
}


/* export reboot driver */
reboot_driver_t s3c2500_reboot_driver = {
	reboot:	s3c2500_reboot
};
