/*-------------------------------------------------------------------------
 * Filename:      time.c
 * Version:       $Id: time.c,v 1.5 2003/09/04 17:39:01 russd Exp $
 * Copyright:     Copyright (C) 1999, Erik Mouw
 * Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Description:   Some easy timer functions for blob
 * Created at:    Tue Aug 24 21:08:25 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sun Oct  3 21:10:21 1999
 *-----------------------------------------------------------------------*/
/*
 * time.c: Timer functions for blob
 *
 * Copyright (C) 1999 2000 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: time.c,v 1.5 2003/09/04 17:39:01 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/init.h>
#include <blob/led.h>
#include <blob/arch.h>
#include <blob/time.h>
#include <blob/debug.h>

static int numOverflows;
timer_driver_t *timer_driver;

void timer_init(void)
{
	if (!timer_driver) {
		deprintf("No timer driver defined in arch "
			 "specific file\n");
		return;
	}
	timer_driver->init();
	numOverflows = 0;
}

/* returns the time in 1/TICKS_PER_SECOND (tps) seconds */
u32 timer_read(void)
{
	u32 ret = timer_driver->read();

#if 0
	/* turn LED always on after one second so the user knows that
         * the board is on
	 */
	if((ret % timer_driver->tps) < (timer_driver->tps >> 7))
		led_on();
#endif

	return ret;
}

int timer_overflow(void)
{
	return timer_driver->overflow();
}

void timer_overflow_clear(void)
{
	if(timer_overflow())
		numOverflows++;

	timer_driver->overflow_clear();
}

int timer_tps(void)
{
	return timer_driver->tps;
}

void udelay(unsigned int usec)
{
	unsigned int ticks;
	unsigned int start_time;
	unsigned int cur_time;
	unsigned int delta;

	ticks = (usec*325 + 99)/100;
	start_time = OSCR;
	do {
		cur_time = OSCR;
		delta = ((cur_time - start_time) > 0)?
			(cur_time - start_time):(0xffffffff - start_time + cur_time);
	} while(delta < ticks);
}

void msleep(unsigned int msec)
{
	u32 ticks, start, end;
	int will_overflow = 0;
	int has_overflow = 0;
	int reached = 0;
	
	if(msec == 0)
		return;

	ticks = (timer_driver->tps * msec) / 1000;
	
	start = timer_read();

	/* this could overflow, but it nicely wraps around which is
         * exactly what we want
	 */
	end = start + ticks;
	
	/* detect the overflow */
	if(end < start) {
		timer_overflow_clear();
		will_overflow = 1;
	}
	
	do {
		if(will_overflow && !has_overflow) {
			if (timer_overflow())
				has_overflow = 1;

			continue;
		}

		if (timer_read() >= end) reached = 1;
	} while(!reached);
}

