#ifndef _IPPS_WRAP_H_
#define _IPPS_WRAP_H_ 1
#include "sha1.h"

/**wrap struct*/
typedef struct SHA1Context IppsSHA1State;
typedef void IppsSHA256State;

typedef enum
{
  ippStsNoErr,
}IppStatus;

typedef unsigned char   Ipp8u;
typedef unsigned short  Ipp16u;
typedef unsigned int    Ipp32u;

typedef signed char    Ipp8s;
typedef signed short   Ipp16s;
typedef signed int     Ipp32s;

#define __INT64    long long
#define __UINT64   unsigned long long
typedef float    Ipp32f;
typedef __INT64  Ipp64s;
typedef __UINT64 Ipp64u;
typedef double   Ipp64f;

inline unsigned endian2(unsigned x)
{
  unsigned a0, a1, a2, a3;

	a0 = x & 0xff;
	a1 = (x & 0xff00) >> 8;
	a2 = (x & 0xff0000) >> 16;
	a3 = (x & 0xff000000) >> 24;
	return (a0 << 24) + (a1 << 16) + (a2 << 8) + a3;
}

/**warp SHA1 function*/
inline IppStatus ippsSHA1GetSize(int *pSize)
{
  *pSize = sizeof(IppsSHA1State);
  return (IppStatus)0;
}

inline IppStatus ippsSHA1Init(IppsSHA1State *pCtx)
{
  SHA1Reset(pCtx);
  return (IppStatus)0;
}

inline IppStatus ippsSHA1Update(const unsigned char *pMesg, int mesglen, IppsSHA1State *pCtx)
{
  SHA1Input(pCtx, pMesg, mesglen);
  return (IppStatus)0;
}

inline IppStatus ippsSHA1Final(unsigned char *pMD, IppsSHA1State *pCtx)
{
  unsigned *p = (unsigned *)pMD;
  if(!SHA1Result(pCtx))
  {
    return (IppStatus)-1;
  }

  p[0] = endian2(pCtx->Message_Digest[0]);
  p[1] = endian2(pCtx->Message_Digest[1]);
  p[2] = endian2(pCtx->Message_Digest[2]);
  p[3] = endian2(pCtx->Message_Digest[3]);
  p[4] = endian2(pCtx->Message_Digest[4]);
  return (IppStatus)0;
}

inline IppStatus ippsSHA1MessageDigest(const unsigned char *pMesg, int mesglen, unsigned char *pMD)
{
  unsigned *p = (unsigned *)pMD;
  IppsSHA1State ctx;

  SHA1Reset(&ctx);
  SHA1Input(&ctx, pMesg, mesglen);
  if(!SHA1Result(&ctx))
  {
    return (IppStatus)-1;
  }

  p[0] = endian2(ctx.Message_Digest[0]);
  p[1] = endian2(ctx.Message_Digest[1]);
  p[2] = endian2(ctx.Message_Digest[2]);
  p[3] = endian2(ctx.Message_Digest[3]);
  p[4] = endian2(ctx.Message_Digest[4]);
  return (IppStatus)0;
}

/**warp SHA256 function*/
inline IppStatus ippsSHA256GetSize(int *pSize)
{
  return (IppStatus)-1;
}

inline IppStatus ippsSHA256Init(IppsSHA256State *pCtx)
{
  return (IppStatus)-1;
}

inline IppStatus ippsSHA256Update(const unsigned char *pMesg, int mesglen, IppsSHA256State *pCtx)
{
  return (IppStatus)-1;
}

inline IppStatus ippsSHA256Final(unsigned char *pMD, IppsSHA256State *pCtx)
{
  return (IppStatus)-1;
}

inline IppStatus ippsSHA256MessageDigest(const unsigned char *pMesg, int mesglen, unsigned char *pMD)
{
  return (IppStatus)-1;
}
#endif
//EOF
