/******************************************************************************
**  COPYRIGHT  2007 Marvell Inernational Ltd.
**  All Rights Reserved
******************************************************************************/
/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:   Flash.c
**
**  PURPOSE:    Contain template OEM boot code flash operations
**
******************************************************************************/

#include "flash.h"
#include "general.h"
#include "typedef.h"
#include "misc.h"
#include "errors.h"
#include "FM.h"
/****************   Steps to add a new Flash Device **********************
 *
 * 1. add #include 'file.h' below
 * 2. update #define MAX_FLASH_TYPES to the next number
 * 3. add pointer to the initialize function of the new flash device
 *        in the InitRoutines function pointer array at the correct
 *        location - should aligned with Fuse Config number for that flash
 *
 *************************************************************************/

//flash device includes
#include "xip.h"
#include "nand.h"
//#include "sdmmc_api.h"
//#include "onenand_obm_interface.h"
#if MDOC_CODE
#include "msys.h"
#endif
#include "platformconfig.h"
#include "OneNAND_Driver.h"

//increment for each new flash type added


// add a new extry for each new flash device added
// Ensure that slots stay assigned properly, even
// if a falash device type is not present in the
// system
//------------------------------------------------
static InitFlash_F InitRoutines[] =
{
    NULL,                       // Fuse Config = 0

#if MDOC_CODE
    &mDOC_Initialize,           // Fuse Config = 1
#else
   NULL,
#endif

#if ONENAND_CODE
   &InitializeFlexOneNANDDevice,              // Fuse Config = 2
#else
   NULL,
#endif

#if 0
    &InitializeXIPDevice,       // Fuse Config = 3
#else
    NULL,
#endif

#if NAND_CODE
    &InitializeNANDDevice,      // Fuse Config = 4
#else
   NULL,
#endif

#if 0
    &InitializeXIPDevice,       // Fuse Config = 5
#else
    NULL,
#endif

#if NAND_CODE
    &InitializeNANDDevice,      // Fuse Config = 6
#else
   NULL,
#endif

#if 0
    &InitializeXIPDevice,       // Fuse Config = 7
#else
    NULL,
#endif

    NULL       // in place of &InitializeSDMMCDevice
//  &InitializeSDMMCDevice      // Fuse Config = 8
};
#define MAX_FLASH_TYPES (sizeof(InitRoutines)/sizeof(void *)) /* generic pointer size for system */

static FlashProperties_T FlashProp[2];  // 0 - Boot Flash, 1- Caddo Save State Flash

P_FlashProperties_T GetFlashProperties(FlashBootType_T FlashBootType)
{
    if (FlashBootType == BOOT_FLASH)
        return &FlashProp[0];
    else
        return &FlashProp[1];
}

//--------------------------------------------------------------------------------------
// Configure_Flashes()
//
// This routine will configure the flashes according what the fuse settings passed in.
//--------------------------------------------------------------------------------------
extern volatile UINT8_T block0_reset;
UINT_T Configure_Flashes (UINT8_T FlashNum, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT_T i;
    InitFlash_F InitializeFunction;
    P_FlashProperties_T pFlashProp, pFlashPropBoot;

    // Get a pointer to the FlashProperties structure
    pFlashProp = GetFlashProperties(fbt);
    // Need to force grabbing the BOOT_FLASH struct for reasons found in for loop below
    pFlashPropBoot = GetFlashProperties(BOOT_FLASH);

    // Make sure the flash type is a valid value
    if(FlashNum >= MAX_FLASH_TYPES)
        return Retval;

    // Check, Are we initializing the SaveStateFlash and has the BootFlash already been initialized?
    // And are they the save flash device?
    //  If so, no need to init hardware again, just copy FlashProperties
    if ((fbt == SAVE_STATE_FLASH) && (pFlashPropBoot->FlashInitializedState == TRUE))
        if(pFlashPropBoot->FlashDeviceNum == FlashNum)
        {
            for(i = 0; i < (sizeof(FlashProperties_T) / 4); i++)
                ((UINT_T *)pFlashProp)[i] = ((UINT_T *)pFlashPropBoot)[i];
            return NoError;
        }

    //Clear the Flash properties structure
    // (Null out pointers)
    //-------------------------------------
    memset( pFlashProp, 0, sizeof(FlashProperties_T));

    //See if we get a pointer to the appropriate
    //init function for the assigned flash
    //------------------------------------------
    InitializeFunction = InitRoutines[FlashNum];
    if(InitializeFunction == NULL)
        return InvalidBootTypeError;

    Retval = InitializeFunction(FlashNum, fbt);
    if (Retval == NoError)
    {
        pFlashProp->FlashInitializedState = TRUE;
        pFlashProp->FlashDeviceNum == FlashNum;
    
		//If we got here and are configuring SAVE_STATE_FLASH, that means we
		//have two different flashes for BOOT and SAVE_STATE.  This is an issue,
		//since we can only handle 1 flash for BBM.  In this case, turn off BBM
		//for SAVE_STATE.
		if(fbt == SAVE_STATE_FLASH)
    	{
			pFlashProp->UseBBM = 0;
			return Retval;
		}

        //Initialize BBT, regardless of Flash Type
        //Reason: FindBBT will clear out and invalidate BBT structure,
        //  then check the FlashProp struct to see if the UseBBT flag
        //  in the FlashProp structure is set.
        Retval = FindBBT(fbt);

        // since we do not have a BBT we need to create one
        if ( (Retval == BBTReadError) || (block0_reset) )
            Retval = CreateBBT(fbt);
    }

    return Retval;
}

UINT_T Finalize_Flashes(FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
	P_FlashProperties_T pFlashProp = GetFlashProperties(fbt);
    FinalizeFlash_F final = pFlashProp->FinalizeFlash;

	//if the flash is currently no initialized, just return
	if (pFlashProp->FlashInitializedState == FALSE)
		return NoError;

	//Always call to update the BBT
    Retval = UpdateBBT(fbt);

	//Shutdown the flash if the appropriate function is linked
    if(final != NULL)
        Retval = final(fbt);

	//Clear out the Properties structure
	//Clear the Flash properties structure
    // (Null out pointers)
    //-------------------------------------
    memset( pFlashProp, 0, sizeof(FlashProperties_T));

    return Retval;
}


/* ReadFlash
 *  This API function calls the relative Flash device read command to read data.
 *  This function will call the lower level driver to read on a block basis, in
 *  which to use the FlashManagement BBT
 *
 *  Inputs:
 *      flash_addr - absolute address of where to begin read in flash device
 *      buffer - pointer to location of where to load the data to
 *      total_size - amount of data, in bytes, to read
 *      fbt - flash boot type:  BOOT_FLASH or SAVE_STATE_FLASH
 *  Outputs:
 *      None
 *  Retval value:
 *      NoError - on successful completion
 *      <Other> - Flash device specific read error
 */
UINT_T ReadFlash(UINT_T flash_addr, UINT_T buffer, UINT_T total_size, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT_T BlkNum, OffsetMask, read_size, read_addr;
    UINT_T BlkSize;
    ReadFlash_F read;
    P_FlashProperties_T pFlashP = GetFlashProperties(fbt);
//    P_FMProperties_T pFMProps = GetFMProperties();

    read = pFlashP->ReadFromFlash;

    if(read == NULL)
        return FlashFuncNotDefined;

    //Check to see if the selected flash device uses BBM
    if(pFlashP->UseBBM == 1)
    {
        //set up initial values
        BlkSize = pFlashP->BlockSize;
        OffsetMask = (BlkSize - 1);  // Mask to get offset into a block
        //for the first read, only read to the end of the initial block
        read_size = BlkSize - (OffsetMask & flash_addr);

        while (total_size)
        {
            // Find block number
            BlkNum = (flash_addr / BlkSize);
            // Check for relocations
            BlkNum = ScanBBT(BlkNum);
            // Read Address = new block * block address + offset into the block
            read_addr = (BlkNum * BlkSize) + (flash_addr & OffsetMask);
            // Check read amount
            read_size = read_size > total_size ? total_size : read_size;
            // Read in to the buffer
            Retval = read(read_addr, buffer, read_size, fbt);
            if (Retval != NoError)
                break;

            //update counting stats
            // - address pointers
            read_addr += read_size;
            flash_addr += read_size;
            buffer += read_size;
            // - size pointers
            total_size -= read_size;
            read_size = BlkSize;
        }   // End of while
    }
    else  //Flash device NOT using BBM, let driver handle entire read
        Retval = read(flash_addr, buffer, total_size, fbt);
    return Retval;
}

extern UINT_T verify_flash_data(UINT_T flash_saddr, UINT_T flash_eaddr, UINT_T buf, UINT_T bytes, UINT_T spare_bytes, UINT_T do_reloc, FlashBootType_T fbt);
extern volatile UINT8_T is_yaffs;
extern volatile UINT32_T volume_yaffs;
extern volatile UINT8_T block0_reset;
UINT_T WriteFlash(UINT_T flash_addr, UINT_T buffer, UINT_T total_size, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT_T OffsetMask, BlkNum, write_size, write_addr;
    UINT_T BlkSize;
    UINT_T BlkSpareAreaSize;
    P_FlashProperties_T pFlashP = GetFlashProperties(fbt);
    P_FMProperties_T pFMProps = GetFMProperties();
    int bRetry = TRUE;  // onetime retry
    int bytes_left = total_size;
    int spare_bytes;

    WriteFlash_F write = pFlashP->WriteToFlash;
    EraseFlash_F erase = pFlashP->EraseFlash;

    if(write == NULL)
        return FlashFuncNotDefined;

    //Check to see if the selected flash device uses BBM
    if(pFlashP->UseBBM == 1)
    {
        if(erase == NULL)
            return FlashFuncNotDefined;

        BlkSize = pFlashP->BlockSize;

        OffsetMask = (BlkSize - 1);  // Mask to get offset into a block
        //for the first write only, write to the end of the initial block
        write_size = BlkSize - (OffsetMask & flash_addr);
        
        if(is_yaffs)
        {
            bytes_left = (total_size / (2048+64)) << 11;
            if(!block0_reset)
            {
                EraseFlash(flash_addr, volume_yaffs, fbt);
                Delay_ms(50);
            }
            spare_bytes = 64;
        }else spare_bytes = 0;

        while (bytes_left)
        {
            {
                extern void dkb_logo(int color, int mod);
                dkb_logo(0xaaaaaaaa, 1);
            }
            // Find block number
            BlkNum = (flash_addr / BlkSize);
            // Check for relocations
            BlkNum = ScanBBT(BlkNum);
            // Write Address = new block * block address + offset into the block
            write_addr = (BlkNum * BlkSize) + (flash_addr & OffsetMask);

            // Check write amount.  Write size is a block or less
            write_size = write_size > bytes_left ? bytes_left : write_size;

            // calculate SA bytes for the number of pages to write a whole block
            // the spare area bytes will need to be taken from the buffer in addition to
            // all the data bytes for the block
            if ( (pFlashP->bUseSpareArea == 1) && (write_size == BlkSize) )
            {
                BlkSpareAreaSize = (write_size/pFlashP->PageSize)*pFlashP->SASize;
            }
            else
            {
                // no spare area or
                // if write_size is less than a block, then write_size accounts for and buffer
                // contains all the spare area bytes so we don't need to account for any
                BlkSpareAreaSize = 0;
            }
            
            if(is_yaffs)
            {
                BlkSpareAreaSize = 0;
            }

            // write the block to flash
            Retval = write(write_addr, buffer, write_size + BlkSpareAreaSize, fbt);
#ifdef FBW_CODE
            if(Retval != NoError)
            {
                borqs_printf("++++++++++++write error+++++++++++++");
            }
#endif
#if ONENAND_CODE
            if(BlkNum)
            {
                #if ____cache_in_read
                extern void enable_i_d_cache(void);
                extern void disable_i_d_cache(void);
                enable_i_d_cache();
                #endif

                if(verify_flash_data(write_addr, write_addr, buffer, write_size, spare_bytes, 1, fbt) == DORELOC)
                {
                    #if ____cache_in_read
                    disable_i_d_cache();
                    #endif
                    continue;
                }
                #if ____cache_in_read
                disable_i_d_cache();
                #endif
            }
#endif
#if 0
            if ( Retval == -6 ) // DFCDoubleBitError
            {
                if ( bRetry == TRUE )
                {
                    // see if erase makes the block writable, retry write
                    bRetry = FALSE;
                    Retval = (GetFlashProperties(fbt)->EraseFlash)( write_addr, write_size, fbt );//EraseFlash( write_addr, write_size, fbt );
                    if (Retval == NoError)
                    // retry same block write after erase
                        continue;
                    ResetFlash(fbt);
                    Retval = (GetFlashProperties(fbt)->EraseFlash)( write_addr, write_size, fbt );
                    if (Retval == NoError)
                    // retry same block write after erase
                        continue;
                }
                // either we are here because 1st retry failed or
                // attempt to erase failed, so go ahead and relocate block.
                // it's possible the new block will fail on write and it may
                // need to be erased/relocated as well so enable retry again
                bRetry = TRUE;
                RelocateBlock( BlkNum, &pFMProps->BBT, fbt );
                // Write Address = new block * block address + offset into the block
                write_addr = (BlkNum * BlkSize) + (flash_addr & OffsetMask);
                // erase the relocated block so the retry write might work
                Retval = EraseFlash( write_addr, BlkSize, fbt);

                // retry same block write with relocated block
                continue;
            }
            else if (Retval != NoError)
                break;
#endif

            // prepare for next block write
            //update counting stats
            flash_addr += write_size;
            buffer += write_size + BlkSpareAreaSize;
            if(is_yaffs)buffer += 64*64;
            // - size pointers
            bytes_left -= (write_size + BlkSpareAreaSize);

            // if bytes_left goes negative, then write_size or BlkSpareAreaSize calculations are wrong
            if (bytes_left < 0 )
                return -1;

            // reset for next block write
            write_size = BlkSize;
            BlkSpareAreaSize = 0;
            bRetry = TRUE;
        }   // End of while
    }
    else  //Flash device NOT using BBM, let driver handle entire read
    {
        // now calculate SA bytes for the number of pages to write
        if ( pFlashP->bUseSpareArea == 1 )
            BlkSpareAreaSize = (bytes_left/pFlashP->PageSize)*pFlashP->SASize;
        else
            BlkSpareAreaSize = 0;

        Retval = write(flash_addr, buffer, bytes_left + BlkSpareAreaSize, fbt);
    }

    return Retval;

}

extern UINT_T create_bbt(UINT_T start_addr, UINT_T end_addr, FlashBootType_T fbt);
UINT_T EraseFlash(UINT_T flash_addr, UINT_T total_size, FlashBootType_T fbt)
{
    UINT_T Retval = NoError;
    UINT_T erase_addr, total_blks, BlkSize, PageSize;
    P_FlashProperties_T pFlashP = GetFlashProperties(fbt);
    P_FMProperties_T pFMProps = GetFMProperties();
    int bRetry = TRUE;  // onetime retry
    UINT_T tmp_erase_addr;

    EraseFlash_F erase = pFlashP->EraseFlash;

    if(erase == NULL)
        return FlashFuncNotDefined;

    //Check to see if the selected flash device uses BBM
    if(pFlashP->UseBBM == 1)
    {

        //set up initial values
        BlkSize = pFlashP->BlockSize;
        PageSize = pFlashP->PageSize;

        // normalize addr and size to the whole blocks
        erase_addr = (flash_addr / BlkSize) * BlkSize;

        total_blks = ((flash_addr + total_size) / BlkSize) - (erase_addr / BlkSize) ;
        //if ( total_size % BlkSize > 0 )
        //    total_blks += 1;
        if(total_size & (BlkSize-1))
         total_blks += 1;

        //modify BBT State if we are erasing block 0
        // do this here rather than in the following while loop because block 0 can
        // only be the very first block and we don't want to have to do the if statement
        // in every loop iteration
        if( (erase_addr / BlkSize) == 0 )
            SetBBTState(BBT_ERASED, fbt);

        while (total_blks)
        {
            tmp_erase_addr = ScanBBT(erase_addr / BlkSize) * BlkSize;
            Retval = erase( tmp_erase_addr, BlkSize, fbt);
    
            {
                extern void dkb_logo(int color, int mod);
                dkb_logo(0xffff, (flash_addr == 0) ? 2:1);
            }

            if( create_bbt(tmp_erase_addr, tmp_erase_addr, fbt) )
             continue;
#if 0
            if (Retval != NoError)
            {
                if ( bRetry == TRUE )
                {
                    // see if erase makes the block writable, retry write
                    bRetry = FALSE;
                    ResetFlash(fbt);
                }
                else
                {
                    // reset didn't help so go ahead and relocate
                    // enable retry again for next erase
                    bRetry = TRUE;
                    RelocateBlock( (erase_addr / BlkSize), &pFMProps->BBT, fbt );
                }
                continue;
            }
#endif
            //update counting stats
            // - address pointers
            erase_addr += BlkSize;
            // - size pointers
            total_blks--;
        }   // End of while
    }
    else  //Flash device NOT using BBM, let driver handle entire read
        Retval = erase(flash_addr, total_size, fbt);
    return Retval;
}

UINT_T ResetFlash(FlashBootType_T fbt)
{
    ResetFlash_F reset = GetFlashProperties(fbt)->ResetFlash;
    if(reset == NULL)
        return FlashFuncNotDefined;
    else
        return reset(fbt);
}

UINT_T ReadOTP(UINT_T OTPAddress, UINT_T LocalBuffer, UINT_T size, FlashBootType_T fbt)
{
    ReadOTP_F readotp = GetFlashProperties(fbt)->ReadOTP;

    if(readotp == NULL)
        return FlashFuncNotDefined;
    else
        return readotp(OTPAddress, LocalBuffer, size);
}

UINT_T WriteOTP(UINT_T OTPAddress, UINT_T LocalBuffer, UINT_T size, FlashBootType_T fbt)
{
    WriteOTP_F writeotp = GetFlashProperties(fbt)->WriteOTP;

    if(writeotp == NULL)
        return FlashFuncNotDefined;
    else
        return writeotp(OTPAddress, LocalBuffer, size);
}

UINT_T LockOTP(FlashBootType_T fbt)
{
    LockOTP_F lockotp = GetFlashProperties(fbt)->LockOTP;

    if(lockotp == NULL)
        return FlashFuncNotDefined;
    else
        return lockotp();
}
UINT_T GetBlockSize(FlashBootType_T fbt)
{
    return GetFlashProperties(fbt)->BlockSize;
}

UINT_T GetPageSize(FlashBootType_T fbt)
{
    return GetFlashProperties(fbt)->PageSize;
}

UINT_T GetEraseRegionSize(FlashBootType_T fbt)
{
    return GetFlashProperties(fbt)->BlockSize;
}

UINT_T GetUseSpareArea(FlashBootType_T fbt)
{
    return GetFlashProperties(fbt)->bUseSpareArea;
}

UINT_T SetUseSpareArea(UINT_T bUseSpareArea, FlashBootType_T fbt)
{
    P_FlashProperties_T pFlashProps = GetFlashProperties(fbt);
    pFlashProps->bUseSpareArea = bUseSpareArea == 0 ? 0 : 1;

    return GetUseSpareArea(fbt);
}

UINT_T GetSpareAreaSize(FlashBootType_T fbt)
{
    return GetFlashProperties(fbt)->SASize;
}

UINT_T SetSpareAreaSize(UINT_T SASize, FlashBootType_T fbt)
{
    P_FlashProperties_T pFlashProps = GetFlashProperties(fbt);
    pFlashProps->SASize = SASize;

    return GetSpareAreaSize(fbt);
}

UINT_T ConvertToLogicalAddr(UINT_T FlashLocation, FlashBootType_T fbt)
{
    ConvertToLogicalAddr_F logaddr = GetFlashProperties(fbt)->pConvertToLogicalAddr;

    if(logaddr == NULL)
        return FlashLocation;
    else
        return logaddr(FlashLocation, fbt);

}

#if !BOOTROM
UINT_T FlashInitAfterDownload( void *pTimHeader, DataSearchFunc_F FindPackageFunc, DataSearchFunc_F FindFirstPackageFunc,
                               DataSearchFindNext_F FindNextPackageFunc, FlashBootType_T fbt )
{
   UINT_T *pkgIDs;
   FlashInitAfterTimDownload_F FlashInitFunc = GetFlashProperties(fbt)->FlashInitAftDl;

   if( FlashInitFunc == NULL )
     return FlashFuncNotDefined;

   pkgIDs = GetFlashProperties(fbt)->ReservedPkgIDs;
   return FlashInitFunc( pTimHeader, FindPackageFunc, FindFirstPackageFunc, FindNextPackageFunc,  pkgIDs );
}
#endif
