/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Interrupts.h
**
**  PURPOSE: 	BootROM, DKB and OBM interrupt related header file
**
******************************************************************************/

#ifndef __interrupt_h
#define __interrupt_h

#include "platformconfig.h"

// Masks etc
#define UndefinedHandlerException	0x78
#define SwiHandlerException			0x79
#define PrefetchHandlerException	0x7A
#define AbortHandlerException		0x7B

// IM ROM map register
#define SGSR      	(volatile unsigned long *)( 0x4600FE08 ) // BOOT ROM Map register

#define INTC_BASE	(volatile unsigned long *)( 0x40D00000 ) // INTC base address
#define ICIP       	(volatile unsigned long *)( 0x40D00000 ) // Interrupt controller IRQ pending register
#define ICMR       	(volatile unsigned long *)( 0x40D00004 ) // Interrupt controller mask register
#define ICLR       	(volatile unsigned long *)( 0x40D00008 ) // Interrupt controller level register
#define ICFP       	(volatile unsigned long *)( 0x40D0000C ) // Interrupt controller FIQ pending register
#define ICPR       	(volatile unsigned long *)( 0x40D00010 ) // Interrupt controller pending register
#define ICCR       	(volatile unsigned long *)( 0x40D00014 ) // Interrupt controller control register
// Interrupt priority register
#define ICHP       	(volatile unsigned long *)( 0x40D00018 ) // Interrupt controller highest priority register
#define IPR0		(volatile unsigned long *)( 0x40D0001C ) //	Interrupt Priority Registers 0
#define IPR1		(volatile unsigned long *)( 0x40D00020 ) //	Interrupt Priority Registers 1
#define IPR2		(volatile unsigned long *)( 0x40D00024 ) //	Interrupt Priority Registers 2
#define IPR3		(volatile unsigned long *)( 0x40D00028 ) //	Interrupt Priority Registers 3
#define IPR4		(volatile unsigned long *)( 0x40D0002C ) //	Interrupt Priority Registers 4
#define IPR5		(volatile unsigned long *)( 0x40D00030 ) //	Interrupt Priority Registers 5
#define IPR6		(volatile unsigned long *)( 0x40D00034 ) //	Interrupt Priority Registers 6
#define IPR7		(volatile unsigned long *)( 0x40D00038 ) //	Interrupt Priority Registers 7
#define IPR8		(volatile unsigned long *)( 0x40D0003C ) //	Interrupt Priority Registers 8
#define IPR9		(volatile unsigned long *)( 0x40D00040 ) //	Interrupt Priority Registers 9
#define IPR10		(volatile unsigned long *)( 0x40D00044 ) // Interrupt Priority Registers 10
#define IPR11		(volatile unsigned long *)( 0x40D00048 ) //	Interrupt Priority Registers 11
#define IPR12		(volatile unsigned long *)( 0x40D0004C ) //	Interrupt Priority Registers 12
#define IPR13		(volatile unsigned long *)( 0x40D00050 ) //	Interrupt Priority Registers 13
#define IPR14		(volatile unsigned long *)( 0x40D00054 ) //	Interrupt Priority Registers 14
#define IPR15		(volatile unsigned long *)( 0x40D00058 ) //	Interrupt Priority Registers 15
#define IPR16		(volatile unsigned long *)( 0x40D0005C ) //	Interrupt Priority Registers 16
#define IPR17		(volatile unsigned long *)( 0x40D00060 ) // Interrupt Priority Registers 17
#define IPR18		(volatile unsigned long *)( 0x40D00064 ) // Interrupt Priority Registers 18
#define IPR19		(volatile unsigned long *)( 0x40D00068 ) // Interrupt Priority Registers 19
#define IPR20		(volatile unsigned long *)( 0x40D0006C ) //	Interrupt Priority Registers 20
#define IPR21		(volatile unsigned long *)( 0x40D00070 ) //	Interrupt Priority Registers 21
#define IPR22		(volatile unsigned long *)( 0x40D00074 ) //	Interrupt Priority Registers 22
#define IPR23		(volatile unsigned long *)( 0x40D00078 ) // Interrupt Priority Registers 23
#define IPR24		(volatile unsigned long *)( 0x40D0007C ) // Interrupt Priority Registers 24
#define IPR25		(volatile unsigned long *)( 0x40D00080 ) //	Interrupt Priority Registers 25
#define IPR26		(volatile unsigned long *)( 0x40D00084 ) // Interrupt Priority Registers 26
#define IPR27		(volatile unsigned long *)( 0x40D00088 ) // Interrupt Priority Registers 27
#define IPR28		(volatile unsigned long *)( 0x40D0008C ) //	Interrupt Priority Registers 28
#define IPR29		(volatile unsigned long *)( 0x40D00090 ) //	Interrupt Priority Registers 29
#define IPR30		(volatile unsigned long *)( 0x40D00094 ) //	Interrupt Priority Registers 30
#define IPR31		(volatile unsigned long *)( 0x40D00098 ) //	Interrupt Priority Registers 31
//
#define ICIP2      	(volatile unsigned long *)( 0x40D0009C ) // Interrupt controller IRQ pending register
#define ICMR2      	(volatile unsigned long *)( 0x40D000A0 ) // Interrupt controller mask register
#define ICLR2      	(volatile unsigned long *)( 0x40D000A4 ) // Interrupt controller level register
#define ICFP2      	(volatile unsigned long *)( 0x40D000A8 ) // Interrupt controller FIQ pending register
#define ICPR2      	(volatile unsigned long *)( 0x40D000AC ) // Interrupt controller pending register
#define IPR32		(volatile unsigned long *)( 0x40D000B0 ) //	Interrupt Priority Registers 31
#define IPR33		(volatile unsigned long *)( 0x40D000B4 ) //	Interrupt Priority Registers 31

// ICXX3
#define ICIP3      	(volatile unsigned long *)( 0x40D00130 ) // Interrupt controller IRQ pending register
#define ICMR3      	(volatile unsigned long *)( 0x40D00134 ) // Interrupt controller mask register
#define ICLR3      	(volatile unsigned long *)( 0x40D00138 ) // Interrupt controller level register
#define ICFP3      	(volatile unsigned long *)( 0x40D0013C ) // Interrupt controller FIQ pending register
#define ICPR3      	(volatile unsigned long *)( 0x40D00140 ) // Interrupt controller pending register

//Prototypes
unsigned char GetPortInterruptFlag(void);
void ClearPortInterruptFlag(void);
unsigned int EnablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID);
void EnableUdcDmaUsage(void);
void DisableUdcDmaUsage(void);
unsigned int DisablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID);


#endif
