.EXPORT_ALL_VARIABLES:
TOPDIR :=$(shell pwd)/..
INCLUDE_DIRS = -I $(TOPDIR)/DMA \
	-I $(TOPDIR)/Download \
	-I $(TOPDIR)/DSVL \
	-I $(TOPDIR)/Flash \
	-I $(TOPDIR)/Flash/DFC \
	-I $(TOPDIR)/Flash/FlexOneNand \
	-I $(TOPDIR)/Flash/M-SYS \
	-I $(TOPDIR)/Flash/NAND \
	-I $(TOPDIR)/Flash/XIP \
	-I $(TOPDIR)/I2C \
	-I $(TOPDIR)/MainProgram \
	-I $(TOPDIR)/Misc \
	-I $(TOPDIR)/SecureBoot \
	-I $(TOPDIR)/StartUp \
	-I $(TOPDIR)/MainProgram/oms_fota \
	-I $(TOPDIR)/MainProgram/redbend 

ifeq "$(FLASH_TYPE)" "ONENAND"
	ONENAND = 1
	NAND = 0
else
	ifeq "$(FLASH_TYPE)" "NAND8"
		ONENAND = 0
		NAND = 1
	endif
	ifeq "$(FLASH_TYPE)" "NAND16"
		ONENAND = 0
		NAND = 1
	endif

endif


RELEASE = 1
TRUSTED = 0
NOR = 0
MSYS = 0
#USB flags for 1.1, 2.0, and Chip Idea: default disabled
USB1 = 1
USB2 = 0
USBCI = 0


CROSS_COMPILE = arm-linux-
CFLAGS	= -Wno-strict-prototypes -g -Os -fomit-frame-pointer -fpic -fasm -nostdinc -fno-builtin -mcpu=xscale \
	-DDKB=1 -DTRUSTED=$(TRUSTED) -DJTAG_USED=0 -DUSB1=$(USB1) -DUSB2=$(USB2) -DUSBCI=$(USBCI) \
	-D NOR_CODE=$(NOR) -DNAND_CODE=$(NAND) -DONENAND_CODE=$(ONENAND) -DMDOC_CODE=$(MSYS) -DNO_STD_LIB_USE
ifeq "$(BOARDTYPE)" "FBWEVB"
CFLAGS	+= -DFBW_CODE=1 
else
ifeq "$(BOARDTYPE)" "SPARK"
CFLAGS	+= -DFBW_CODE=1 
else
ifeq "$(BOARDTYPE)" "BENZINA"
CFLAGS	+= -DFBW_CODE=1 
else
ifeq "$(BOARDTYPE)" "BLAZE"
CFLAGS	+= -DFBW_CODE=1 
endif
endif
endif
endif	
AFLAGS  = -Wall -mcpu=xscale -mapcs-stack-check
LDFLAGS = -e __main -nostdlib -format elf32-littlearm -T startup.lds

ifeq	"$(TRUSTED)" "1"
	TYPE = T
	LIBS = $(TOPDIR)/DSVL/wtmdsvl.a 
	SECUREOBJS = dkb_security.o
else
	TYPE = NT
	LIBS =
	SECUREOBJS =
endif

ifeq	"$(SDRAM)" "128"
	AFLAGS += --defsym ENABLE_CS1_SDRAM=1
else
	AFLAGS += --defsym SDRAM_SIZE=0
endif

ifeq "$(BOARDTYPE)" "FBWEVB"
	AFLAGS += --defsym PV=1
endif
ifeq "$(BOARDTYPE)" "SPARK"
	AFLAGS += --defsym PV=1
endif

.PHONY: clean
#Currently, we cannot create Symbol files in linux build, so
#default build type is RELEASE
BUILD = Release

USB1OBJS = usbdownload.o	\
	usbdriver.o	\
	xllp_udc.o

USB2OBJS = u2download.o	\
	u2ddriver.o	\
	xllp_u2d.o	\
	xllp_u2d_dma.o

#default PLATFORM is MHLV
ifdef	PLATFORM
	ifeq	"$(PLATFORM)" "MHL"
		USB2 = 0
		USBOBJS = $(USB1OBJS)
		BOARD = ZYLONITE
	else
	ifeq	"$(PLATFORM)" "MHP"
		USB2 = 0
		USBOBJS = $(USB1OBJS)
		BOARD = ZYLONITE
	else
	ifeq	"$(PLATFORM)" "TAVOR"
		USBOBJS = $(USB1OBJS) $(USB2OBJS)
		BOARD = TAVOR_BOERNE
	else
	ifeq	"$(PLATFORM)" "TPV"
		USB2 = 0
		USBOBJS = $(USB1OBJS)
		BOARD = TAVOR_BOERNE
	else
		$(PLATFORM) = MHLV
		USB1 = 0
		USBOBJS = $(USB2OBJS)
		ifeq "$(BOARD_TYPE)" "LITTLETON"
			BOARD = LITTLETON
		else
			BOARD = ZYLONITE
		endif
	endif
	endif
	endif
	endif
else
	PLATFORM = MHLV
	USB1 = 0
	USBOBJS = $(USB2OBJS)
	ifeq "$(BOARD_TYPE)" "LITTLETON"
		BOARD = LITTLETON
	else
		BOARD = ZYLONITE
	endif
endif

## Call perl script to make sure assembly files are in GNU format ##
junk :=$(shell perl makegnuassembly.pl $(PLATFORM))

CFLAGS += -D$(BOARD)=1
INCLUDE_DIRS += -I $(TOPDIR)/$(PLATFORM) -I $(TOPDIR)/$(PLATFORM)/DDR
OUTDIR = $(TOPDIR)/$(PLATFORM)/$(BUILD)

## list of OBJECT files
DOWNLOADOBJS = 	protocolmanager.o \
		$(USBOBJS) \
		uartdownload.o \
		uart.o \
		usb_descriptors.o

DMAOBJS = xllp_dmac.o

FLASHOBJS = flash.o FM.o  

MAINOBJS = dkb.o interrupts.o fota_agent.o oms_trace.o RB_standard.o 

I2COBJS = i2c.o

MISCOBJS = misc.o

PLATFORMOBJS = platformconfig.o platform_setup.o platform_lcd.o platform_pxa_font.o platform_lcd_pxa_printf.o

ifeq "$(BOARDTYPE)" "FBWEVB"
DDROBJS = mini_xlli_lowlev_init_fbw.o ddr_init_fbw.o PPSet.o
else
ifeq "$(BOARDTYPE)" "SPARK"
DDROBJS = mini_xlli_lowlev_init_fbw.o ddr_init_fbw.o PPSet.o
else
ifeq "$(BOARDTYPE)" "BENZINA"
DDROBJS = mini_xlli_lowlev_init_fbw.o ddr_init_fbw.o PPSet.o
else
DDROBJS = mini_xlli_lowlev_init.o ddr_init.o
endif 
endif 
endif 

#Flash driver objects

ifeq	"$(NOR)" "1"
XIPOBJS = xip.o
else
XIPOBJS = 
endif

ifeq	"$(NAND)" "1"
NANDOBJS = nand.o 
DFCOBJS = xllp_dfc.o xllp_dfc_support.o 
else
NANDOBJS = 
DFCOBJS = 
endif

ifeq	"$(ONENAND)" "1"
ONENANDOBJS = FlexOneNAND_Cmd.o	\
		OneNAND_Cmd.o \
		OneNAND_Driver.o
else
ONENANDOBJS =
endif

ifeq	"$(MSYS)" "1"
MDOCOBJS =	boot.o			\
		defs.o			\
		docdrv.o		\
		doch_api.o		\
		doch_ata.o		\
		dochstub.o		\
		flcustom.o		\
		flmalloc.o		\
		flsystem.o		\
		formatdata.o		\
		geometry.o		\
		msys_dkb_interface.o	\
		msys_interface.o	\
		tffs_api.o
else
MDOCOBJS =
endif

OBJS =	dkb_startup.o \
		$(DMAOBJS) \
		$(DOWNLOADOBJS) \
		$(FLASHOBJS) \
		$(I2COBJS) \
		$(MAINOBJS) \
		$(MISCOBJS) \
		$(SECUREOBJS) \
		$(PLATFORMOBJS) \
		$(DDROBJS) \
		$(XIPOBJS) \
		$(DFCOBJS) \
		$(NANDOBJS) \
		$(ONENANDOBJS) \
		$(MDOCOBJS)

AS	=$(CROSS_COMPILE)as $(AFLAGS)
LD	=$(CROSS_COMPILE)ld $(LDFLAGS)
CC	=$(CROSS_COMPILE)gcc $(CFLAGS)
STRIP	=$(CROSS_COMPILE)strip -R .note -R .comment -S
OBJCOPY	=$(CROSS_COMPILE)objcopy -O binary -R .note -R .comment -S
OBJDUMP	=$(CROSS_COMPILE)objdump
MAKE	=make

OBJECTS = $(patsubst %.o, $(OUTDIR)/%.o, $(OBJS))
DKB = $(OUTDIR)/$(PLATFORM)_$(TYPE)DKB

"$(DKB).bin" :	$(OBJS)
		@$(LD) -o "$(DKB)-elf32" $(OBJECTS) $(LIBS) $(TOPDIR)/MainProgram/redbend/libfota.a -Map "$(DKB).map"
		@$(STRIP) "$(DKB)-elf32" -o "$(DKB)-elf32-stripped"
		@$(OBJCOPY) "$(DKB)-elf32-stripped" "$(DKB).bin"
		@echo Build Successful!

dkb_startup.o: $(TOPDIR)/StartUp/dkb_startup.S
	@$(AS) $(INCLUDE_DIRS) "$(TOPDIR)/StartUp/dkb_startup.S" -o "$(OUTDIR)/dkb_startup.o" > dump.txt

$(DDROBJS): %.o : $(TOPDIR)/$(PLATFORM)/DDR/%.S
	@$(AS) $(INCLUDE_DIRS) "$(TOPDIR)/$(PLATFORM)/DDR/$*.S" -o "$(OUTDIR)/$*.o" > dump.txt

$(DOWNLOADOBJS): %.o : $(TOPDIR)/Download/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Download/$*.c" -o "$(OUTDIR)/$*.o"

$(MAINOBJS): %.o : $(TOPDIR)/MainProgram/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/MainProgram/$*.c" -o "$(OUTDIR)/$*.o"

$(FLASHOBJS): %.o : $(TOPDIR)/Flash/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/$*.c" -o "$(OUTDIR)/$*.o"

$(I2COBJS): %.o : $(TOPDIR)/I2C/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/I2C/$*.c" -o "$(OUTDIR)/$*.o"

$(MISCOBJS): %.o : $(TOPDIR)/Misc/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Misc/$*.c" -o "$(OUTDIR)/$*.o"

ifeq "$(TRUSTED)" "1"
$(SECUREOBJS): %.o : $(TOPDIR)/SecureBoot/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/SecureBoot/$*.c" -o "$(OUTDIR)/$*.o"
endif

$(PLATFORMOBJS): %.o : $(TOPDIR)/$(PLATFORM)/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/$(PLATFORM)/$*.c" -o "$(OUTDIR)/$*.o"

$(DMAOBJS): %.o : $(TOPDIR)/DMA/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/DMA/$*.c" -o "$(OUTDIR)/$*.o"

ifeq	"$(NOR)" "1"
$(XIPOBJS): %.o : $(TOPDIR)/Flash/XIP/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/XIP/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq	"$(NAND)" "1"
$(DFCOBJS): %.o : $(TOPDIR)/Flash/DFC/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/DFC/$*.c" -o "$(OUTDIR)/$*.o"

$(NANDOBJS): %.o : $(TOPDIR)/Flash/NAND/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/NAND/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq "$(ONENAND)" "1"
$(ONENANDOBJS): %.o : $(TOPDIR)/Flash/FlexOneNand/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/FlexOneNand/$*.c" -o "$(OUTDIR)/$*.o"
endif

ifeq "$(MSYS)" "1"
$(MDOCOBJS): %.o : $(TOPDIR)/Flash/M-SYS/%.c
	@$(CC) $(INCLUDE_DIRS) -c "$(TOPDIR)/Flash/M-SYS/$*.c" -o "$(OUTDIR)/$*.o"
endif

#make $(SUBDIRS) sequencely : ./dfc ./startup ./main ./flash ./download ./misc
#mobmdirs: $(patsubst %, _dir_%, $(SUBDIRS))
#$(patsubst %, _dir_%, $(SUBDIRS)) :
#	$(MAKE) -C $(patsubst _dir_%, %, $@)
#
clean:
	@echo "FOTA make clean..."
	rm -f $(TOPDIR)/TAVOR/Release/*bin "*map" "*elf32*"
	rm -f $(TOPDIR)/TAVOR/Release/*map
	rm -f $(TOPDIR)/TAVOR/Release/*elf32*
	rm -f $(TOPDIR)/TAVOR/*INC 
	rm -f $(TOPDIR)/Build/*txt
	rm -f $(TOPDIR)/StartUp/*S
	rm -f $(TOPDIR)/StartUp/*INC 
	rm -f $(TOPDIR)/StartUp/*MAC 
	rm -f $(TOPDIR)/TAVOR/DDR/*S
	rm -f $(TOPDIR)/TAVOR/DDR/*INC
#
#$(patsubst %, _dir2_%, $(SUBDIRS)) :
#	$(MAKE) -C $(patsubst _dir2_%, %, $@) clean
#
