/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2005 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	ProtocolManager.h
**
**  PURPOSE: 	This module manages the boot ROM protocol used for communication
**				over the UART or USB ports
**
**
******************************************************************************/
#ifndef _protocol_manager_h
#define _protocol_manager_h

#include "typedef.h"
#include "flash.h"
#include "general.h"

#if TAVOR_BOERNE
#define DOWNLOAD_USED 1
#endif

#if BOOTROM
	#include "test.h"
	#include "security.h"
#endif

#if JTAG_USED
	#include "trustedboot.h"
#endif
/********************* Image Types ***********************************/
#define FIRSTIMAGE				1

/************* Commands definitions **********************************/

//ErrorCodes for Host Application
#define Ack						0x00
#define Nack					0x01
#define SeqError				0x02
#define JtagAccessGranted       0x11
#define JtagAccessDenied        0xFF
#define JtagREIssued			0x05
#define DebugCmdIssued			0x06
#define DebugBootIssued			0x07
#define Untested				0x02

#define GetVersionCmd			0x20
#define StartAddrCmd			0x21
#define DownloadDataCmd			0x22
#define JumpAddrCmd				0x23
#define OEMPublicKeyCmd			0x24
#define OEMSignedKeyCmd			0x25
#define SelectImageCmd			0x26
#define VerifyImageCmd			0x27
#define PasswordCmd				0x28
#define	DataHeaderCmd			0x2a
#define MessageCmd				0x2b
#define ViewOTPCmd				0x2c
#define DebugBootCmd			0x2d
#define	DoneCmd					0x30
#define DisconnectCmd			0x31

#define PROTOCOL_WAITTIME		0x80 // in seconds, max time to wait for command
#define DATA_BLOCK_SIZE			0x000000ff8 //block size + sizeof(ProtocolCmd) must be word alligned
#define LENGTHOFMESSAGES		0x40
#define MAXNUMOFMESSAGES		0x20
#define PREAMBLE_SIZE           0x4

//Download ISR information
typedef struct
{
	volatile UINT8_T PreambleReceived;
	volatile UINT8_T CommandReceived;
	volatile UINT8_T InterruptPort;
	volatile UINT8_T CurrentCommand;
	volatile UINT8_T DownloadsEnabled;
	volatile UINT32_T DataLeft;
	volatile UINT32_T CurrentDownloadAddress;
} ProtocolISR, *pProtocolISR;

typedef struct {
    volatile UINT8_T Command;
    volatile UINT8_T Sequence;
    volatile UINT8_T CID;
    volatile UINT8_T Flags;
    volatile UINT32_T  Length;
    volatile UINT8_T Data[4];
} ProtocolCmd, *pProtocolCmd;

typedef struct {
	UINT8_T Command;
	UINT8_T Sequence;
	UINT8_T CID;
	UINT8_T Status;
	UINT8_T Flags;
	UINT8_T Length;
	UINT8_T Data[12];
} ProtocolRsp, *pProtocolRsp;

typedef struct {
	VUINT_T MessageFlag;
	volatile UINT8_T Messages[MAXNUMOFMESSAGES][LENGTHOFMESSAGES];
	volatile UINT8_T Lengths[MAXNUMOFMESSAGES];
	volatile UINT8_T MessageType[MAXNUMOFMESSAGES];
	UINT8_T CurrentMessageIndex;
	UINT8_T NumMessagesQueued;
} ProtocolMsg, *pProtocolMsg;

typedef struct {
	UINT8_T FFUART_Init;
	UINT8_T ALTUART_Init;
	UINT8_T DIFF_USB_Init;
	UINT8_T SE_USB_Init;
    UINT8_T U2D_USB_Init;
	UINT8_T CI2_USB_Init;
} PortStates, *pPortStates;

#define BTUART_D 		1		// Indicate to the routines which device to use
#define FFUART_D 		0 //2
#define DIFF_USB_D	 	2 //3		//	Unsupported for Bulverde DKB
#define SE_USB_D		3 //4		//	Unsupported for Bulverde DKB
#define ALTUART_D		4
#define U2D_USB_D       5
#define CI2_USB_D       6           // ChipIdea code

//Prototype Definitions
pProtocolISR getProtocolISR(void);
pProtocolCmd getProtocolCmd(void);
pProtocolRsp getProtocolRsp(void);
pProtocolMsg getProtocolMsg(void);

UINT8_T GetPortState(UINT_T);
UINT_T GetCommand(void);
UINT_T GetPreamble(void);
UINT_T SendResponse(UINT8_T *, UINT_T);
UINT_T SendAck(void);
UINT_T SendError(void);
UINT_T InitPort(UINT_T, pFUSE_SET);
void ShutdownPort(UINT_T);
UINT_T HandleDoneCmd(void);
UINT_T HandleDisconnect(void);
UINT_T HandleMessageCmd(void);
UINT_T AddMessage(UINT8_T* message);
UINT_T AddMessageError(UINT8_T);
UINT8_T ProtocolVerifyPreamble( UINT8_T *candidateString );
UINT8_T CheckProtocolTimeOut(UINT_T StartTime, UINT_T *time);
void InitMessageQueue(void);

FUNC_STATUS HandleRequest(UINT_T, UINT_T, P_FlashProperties_T);
#define VERSION_OFFSET 0x24

#if DOWNLOAD_USED
	UINT_T HandleGetVersionCmd(void);
	UINT_T HandleSelectImageCmd(UINT_T imageType);
	UINT_T HandleVerifyImageCmd(void);
	UINT_T HandleDataHeaderCmd(void);
	UINT_T HandleDataCmd(void);
	UINT_T HandleDownloadFlow(UINT_T address, UINT_T image_id);
#endif // #if DOWNLOAD_USED

#if BOOTROM
	UINT_T HandleViewOTPCmd(P_FlashProperties_T pFlashp);
	UINT_T HandleDebugBootCmd(P_FlashProperties_T pFlashp);
#endif

	UINT_T GetTIMValidationStatus(void);
	void SetTIMValidationStatus(UINT_T status);

#if JTAG_USED
	UINT_T HandleJTAGFlow(P_FlashProperties_T pFlashP);
	UINT_T HandlePublicKeyCmd(void);
	UINT_T HandlePublicKeyDataCmd(pJTAGKey, P_FlashProperties_T);
	UINT_T HandlePasswordCmd(UINT8_T *pPassword);
	UINT_T HandleSignedPasswordCmd(void);
	UINT_T HandleSignedPasswordDataCmd(pJTAGKey);
#endif

#endif



