/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2006 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:       U2DDriver.c
**
**  PURPOSE:        This files contains the BootROM specific functions for USB
**                  2.0 Device controller (U2D).
**
**  LAST MODIFIED:
**
******************************************************************************/

/*
*******************************************************************************
*   HEADER FILES
*******************************************************************************
*/

#include "interrupts.h"
#include "misc.h"
#include "xllp_u2d.h"

extern void ClearPortInterruptFlag(void);

extern volatile P_XLLP_U2D_T U2DDeviceController;
extern UINT_T EnablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID);
extern UINT_T DisablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID);

#if 0

//DCB DEBUG FIFO ERR's on EP0
//---------------------------
#define U2D_ISR_HIST_SIZE 64
#define U2D_ISR_HIST_BUF_SIZE (2*U2D_ISR_HIST_SIZE)
static int isrRecordPtr = 0;
static XLLP_UINT32_T isrRecords[U2D_ISR_HIST_BUF_SIZE];

//----------------------------------------------------------------
//  DCB DEBUG
//
//----------------------------------------------------------------
void RecordISRs( XLLP_UINT32_T status, XLLP_UINT32_T dma )
{
  if( isrRecordPtr < (U2D_ISR_HIST_SIZE-1) )
  {
    isrRecords[isrRecordPtr++] = status;
    isrRecords[isrRecordPtr++] = dma;
  }
}
#endif


/******************************************************************************

  Function Name:
    U2DInterruptHandler

  Description:
    This is the UDC's main interrupt handler.

  Global Register Modified:
    None.

  Input Arguments:
    P_XLLP_U2D_T pU2DHandle - a pointer to the UDC's handle

  Output Arguments:
    None

  Return Value:
    None

*******************************************************************************/
void U2DInterruptHandler (void)
{
  P_XLLP_U2D_T pU2DHandle = U2DDeviceController;
  XLLP_UINT32_T statusISR = 0, dmaISR = 0;
  XLLP_UINT32_T  i = 0;
  P_XLLP_U2D_REGISTERS_T pU2DRegs = (P_XLLP_U2D_REGISTERS_T)XLLP_MONAHANS_U2D_REG_BASE;

  // Save status of the U2DISR's
  //-----------------------------
  statusISR = pU2DRegs->U2DISR;
  dmaISR    = pU2DRegs->U2DMAINT;

  // RecordISRs( statusISR, dmaISR );   // DCB DEBUG

  // Clear all U2D USB Event Interrupts and
  // ensure update
  //---------------------------------------
  pU2DRegs->U2DISR = XLLP_U2DISR_EVENTS_MASK;
  i = pU2DRegs->U2DISR;

  // Determine type of U2D interrupt
  // Check if there are any of the event interrupts
  //-----------------------------------------------
  if (statusISR & XLLP_U2DISR_EVENTS_MASK)
  {
      ClearPortInterruptFlag();

      // Is this a Reset interrupt?
      if (statusISR & XLLP_U2DISR_IRRS)
      {
         // Enable the Endpoint 0 interrupts
         //----------------------------------
         XllpU2DEnableInterrupt (pU2DHandle,
                                 U2D_INT_ENDPOINT_0,
                                 U2D_PACKET_COMPL_INT);

         XllpU2DEnableInterrupt (pU2DHandle,
                                 U2D_INT_ENDPOINT_0,
                                 U2D_FIFO_ERROR_INT);

         XllpU2DEnableInterrupt (pU2DHandle,
                                 U2D_INT_ENDPOINT_0,
                                 U2D_SHORT_PACKET_INT);

         // Enable Configuration request interrupts
         //----------------------------------------
         XllpU2DEnableInterrupt (pU2DHandle,
                                 U2D_INT_CONFIG,
                                 0);

         pU2DHandle->enumerateComplete = XLLP_FALSE;

         // Update statistics
         ++(pU2DHandle->interruptStat.ResetIntCount);
      }

      // Is this Suspend interrupt?
      else if (statusISR & XLLP_U2DISR_IRSU)
      {
         // Update statistics
         ++(pU2DHandle->interruptStat.SuspendIntCount);
      }

      // Is this Resume interrupt?
      else if (statusISR & XLLP_U2DISR_IRRU)
      {
         // Do not call this for a resume otherwise
         // U2D does not recover ...
         //-----------------------------------------
         //XllpU2DHWSetup(pU2DHandle);

         // Update statistics
         ++(pU2DHandle->interruptStat.ResumeIntCount);
      }

      // Is this SOF interrupt?
      else if (statusISR & XLLP_U2DISR_IRSOF)
      {
         // Update statistics
         ++(pU2DHandle->interruptStat.SOFIntCount);
      }

      // Is this Configuration Change interrupt?
      else if (statusISR & XLLP_U2DISR_IRCC)
      {
         XllpU2DConfigInterruptHandler(pU2DHandle);
         pU2DRegs->U2DCR |= pU2DRegs->U2DCR;

         pU2DHandle->enumerateComplete = XLLP_TRUE;

         // Update statistics
         ++pU2DHandle->interruptStat.ConfigIntCount;
      }
  }

  // Check if this is Endpoint 0 Packet Complete interrupt
  //-------------------------------------------------------
  if( (statusISR & (XLLP_U2DISR_IEx_PCI_BIT  << (U2D_ENDPOINT_0 * XLLP_U2DISR_CHAN_SHIFT))) ||
      (statusISR & (XLLP_U2DISR_IEx_SPI_BIT  << (U2D_ENDPOINT_0 * XLLP_U2DISR_CHAN_SHIFT)))
    )
  {
     ClearPortInterruptFlag();

     // Call Endpoint 0 interrupt handler
     //----------------------------------
     XllpU2DEp0InterruptHandler(pU2DHandle);

     // Clear the Endpoint 0 Interrupt(s)
     //----------------------------------
     XllpU2DClearInterrupt (pU2DHandle, U2D_INT_ENDPOINT_0, U2D_PACKET_COMPL_INT);
     XllpU2DClearInterrupt (pU2DHandle, U2D_INT_ENDPOINT_0, U2D_SHORT_PACKET_INT);

     // Update statistics
     ++pU2DHandle->interruptStat.EpIntCount[U2D_INT_ENDPOINT_0];
  }

  // Check if this is an Endpoint FIFO error interrupt
  // we check against all the possible Fifo error bits first
  //--------------------------------------------------------
  if( statusISR & 0x00924924 )
  {
    // If it's Endpoint 0, attempt to flush the FIFO
    //----------------------------------------------
    if( statusISR & 0x04 )
    {
      pU2DRegs->U2DCSR_X[U2D_ENDPOINT_0] =  XLLP_U2DCSR0_FTF;
    }

    // If Endpoint A (transmit channel for bootload app)
    // Attempt to flush the FIFO
    //--------------------------
    if( statusISR & 0x020 )
    {
      pU2DRegs->U2DCSR_X[U2D_ENDPOINT_A] =  XLLP_U2DCSRX_FEF;
      i = pU2DRegs->U2DCSR_X[U2D_ENDPOINT_A];  // Ensure in-order operation ...
    }

    // Clear all the pending FIFO error interrupts in a single shot
    //-------------------------------------------------------------
    pU2DRegs->U2DISR &= 0x00924924;
    i = pU2DRegs->U2DISR;            // Ensure in-order operation ...

    for( i=U2D_ENDPOINT_0; i < U2D_ENDPOINT_C; i++ )
    {
      if( statusISR & (XLLP_U2DISR_IEx_FEI_BIT  << (i * XLLP_U2DISR_CHAN_SHIFT)) )
      {

        ClearPortInterruptFlag();

        // Clear the Endpoint  FIFO Interrupt
        //-----------------------------------
        //XllpU2DClearInterrupt (pU2DHandle, i, U2D_FIFO_ERROR_INT);

        // Report the error to the internal debuging list
        // Encode the error and channel together...
        //--------------------------------------------------
        U2DLogError( (U2D_ERR_ENDPOINT_FIFO_ERR << 4) | i );
      }
    }
  }

  // We do this here to catch any U2D DMA errors for the time being
  //--------------------------------------------------------------------
  dmaISR = pU2DRegs->U2DMAINT;

  // U2D has it's own dedicated DMA sub-system
  // to regather DMA descriptors
  //------------------------------------------
#if 0
  if( dmaISR )
    XllpU2DmaInterruptHandler();
#endif

  // Ensure ISR regs synched before
  // leaving interrupt context
  //-------------------------------
  statusISR = pU2DRegs->U2DISR;
  dmaISR    = pU2DRegs->U2DMAINT;
}

//---------------------------------------------------------------
// XllpU2DEnableFirstLevelInterrupt
//
//
//---------------------------------------------------------------
XLLP_STATUS_T XllpU2DEnableFirstLevelInterrupt(void)
{
    // Enable USB IRQ interrupt
    return (EnablePeripheralIRQInterrupt(U2D_CLIENT_INT));
}

//---------------------------------------------------------------
// XllpU2DDisableFirstLevelInterrupt
//
//
//---------------------------------------------------------------
XLLP_STATUS_T XllpU2DDisableFirstLevelInterrupt(void)
{
    // Disable USB IRQ interrupt
    return (DisablePeripheralIRQInterrupt(U2D_CLIENT_INT));
}

//---------------------------------------------------------------
// BootRomU2DHWShutdown
//
//
//---------------------------------------------------------------
XLLP_UINT32_T BootRomU2DHWShutdown (void)
{
    P_XLLP_U2D_T pU2DHandle = U2DDeviceController;
    XLLP_UINT32_T status = FALSE;

    // If we aren't initialized don't start using NULL
    // pointers ...
    //------------------------------------------------
    if( pU2DHandle == NULL )
      return 0;

    status = XllpU2DHWShutdown (pU2DHandle);

    return (0);
}

//---------------------------------------------------------------
// BootRomU2DHWSetup
//
//
//---------------------------------------------------------------
XLLP_UINT32_T BootRomU2DHWSetup(void)
{
    XLLP_UINT32_T Retval     = FALSE;
    P_XLLP_U2D_T  pU2DHandle = U2DDeviceController;

    Retval = XllpU2DHWSetup (pU2DHandle);

    return Retval;
}


//---------------------------------------------------------------
//  BootRomU2DSWInit()
//      This function recieves a handle to the U2D device context
//      and gets initialization of the context structure underway
//---------------------------------------------------------------
void BootRomU2DSWInit (void)
{
    P_XLLP_U2D_T   pU2DHandle = U2DDeviceController;
    XllpU2DSWInit (pU2DHandle, (P_XLLP_U2D_REGISTERS_T)XLLP_MONAHANS_U2D_REG_BASE);
}


