/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:       UsbDriver.c
**
**  PURPOSE:   		This files contains the BootROM specific functions for USB
**					Device controller (UDC).
**
**  LAST MODIFIED: $Modtime: 3/30/06 10:34a $
**
******************************************************************************/

/*
*******************************************************************************
*   HEADER FILES
*******************************************************************************
*/

#include "usbdriver.h"

extern void ClearPortInterruptFlag(void);

extern P_XLLP_UDC_T UsbDeviceController;
extern INT8_T *GlobalDmaDescAlloc(INT_T RequestedSize);
extern UINT_T EnablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID);
extern UINT_T DisablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID);

extern void memset(void *Addr, unsigned char Val, unsigned long Size) ;

#if 0 // ROM space compression
void BootRomUdcSetupInEndpoint (P_XLLP_UDC_T pUdcHandle,
                        	   	XLLP_UDC_EP_T udcEndpointNum,
                           		P_XLLP_UINT32_T pTxBuff,
                           		XLLP_UINT32_T xferLength)
{
    volatile P_XLLP_UDC_REGISTERS_T pRegs = (P_XLLP_UDC_REGISTERS_T)pUdcHandle->pRegsBase;

    // Initilize the UDC's endpoint xfer structure
    pUdcHandle->EpXferTable[udcEndpointNum].pDataEp = pTxBuff;
    pUdcHandle->EpXferTable[udcEndpointNum].xferLength = xferLength;
    pUdcHandle->EpXferTable[udcEndpointNum].xferDataCounter = 0;
    pUdcHandle->EpXferTable[udcEndpointNum].maxPacketSize =
                                ((pRegs->UDCCRZ[udcEndpointNum] & XLLP_UDC_UDCCRZ_MPS_MASK)
                                >> XLLP_UDC_UDCCRZ_MPS_SHIFT);
    pUdcHandle->EpXferTable[udcEndpointNum].TxXferComplete = XLLP_FALSE;
}
#endif

void BootRomUdcSetupOutEndpoint (P_XLLP_UDC_T pUdcHandle,
            	                 XLLP_UDC_EP_T udcEndpointNum,
                		         XLLP_UINT32_T numBytes)
{
    volatile P_XLLP_UDC_REGISTERS_T pRegs = (P_XLLP_UDC_REGISTERS_T)pUdcHandle->pRegsBase;

    // Allocate memory for temp. Rx buffer
    if (pUdcHandle->EpXferTable[udcEndpointNum].pDataEp == NULL)
	//allocates 4096 KBytes for buffer
      pUdcHandle->EpXferTable[udcEndpointNum].pDataEp = (P_XLLP_UINT32_T)GlobalDmaDescAlloc(numBytes);
      //pUdcHandle->EpXferTable[udcEndpointNum].pDataEp = malloc(numBytes);

    // Initilize the UDC's endpoint xfer structure
    pUdcHandle->EpXferTable[udcEndpointNum].xferLength = numBytes;
    pUdcHandle->EpXferTable[udcEndpointNum].xferDataCounter = 0;
    pUdcHandle->EpXferTable[udcEndpointNum].maxPacketSize =
                                ((pRegs->UDCCRZ[udcEndpointNum] & XLLP_UDC_UDCCRZ_MPS_MASK)
                                >> XLLP_UDC_UDCCRZ_MPS_SHIFT);
    pUdcHandle->EpXferTable[udcEndpointNum].RxXferComplete = XLLP_FALSE;
    pUdcHandle->EpXferTable[udcEndpointNum].RxBlockXferComplete = XLLP_FALSE;

    // Clear DMA enable bit in UDCCSR register
    // (writing zero to all other bits would not modify them).
    pRegs->UDCCSR[udcEndpointNum] = 0;

    // Enable endpoint's packet complete interrupt
    XllpUdcEnableInterrupt (pUdcHandle,
                            (XLLP_UDC_INTERRUPTS_T)udcEndpointNum,
                            PACKET_COMPL_INT);

    // Enable endpoint's FIFO error interrupt
    XllpUdcEnableInterrupt (pUdcHandle,
                            (XLLP_UDC_INTERRUPTS_T)udcEndpointNum,
                            FIFO_ERROR_INT);
}

void XllpUdcSetupOutEndpoint (P_XLLP_UDC_T pUdcHandle, XLLP_UDC_EP_T udcEndpointNum)
{
    XLLP_UINT32_T numBytes;

    numBytes = XSUDC_TEMP_OUT_BUFF_SIZE;
    BootRomUdcSetupOutEndpoint (pUdcHandle, udcEndpointNum, numBytes);
}

/******************************************************************************

  Function Name:
    UdcEndpoint0InterruptHandler

  Description:
    This is the UDC's main interrupt handler.

  Global Register Modified:
    None.

  Input Arguments:
    P_XLLP_UDC_T pUdcHandle - a pointer to the UDC's handle

  Output Arguments:
    None

  Return Value:
    None

*******************************************************************************/
void UdcEndpoint0InterruptHandler (void)
{
	P_XLLP_UDC_T pUdcHandle = UsbDeviceController;
    XLLP_UINT32_T statusISR0 = 0;
    XLLP_UINT32_T statusISR1 = 0;
    XLLP_UINT32_T udcEndpointNum = 0, j = 0;
    volatile P_XLLP_UDC_REGISTERS_T pRegs = (P_XLLP_UDC_REGISTERS_T)0x40600000;

    // Save status of the UDCISR0
    statusISR0 = pRegs->UDCISR0;

    // Save status of the UDCISR1
    statusISR1 = pRegs->UDCISR1;

    // Clear all USB Event Interrupts
    pRegs->UDCISR1 |= XLLP_UDC_UDCISR1_EVENTS;

    // Determine type of UDC interrupt
    // Check if there are any of the event interrupts
    if (statusISR1 & XLLP_UDC_UDCISR1_EVENTS)
    {
		ClearPortInterruptFlag();
        // Is this Reset interrupt?
        if (statusISR1 & XLLP_UDC_UDCISR1_IRRS)
        {
            // Enable the Endpoint 0 interrupts
            XllpUdcEnableInterrupt (pUdcHandle,
                                    INT_ENDPOINT_0,
                                    PACKET_COMPL_INT);

            XllpUdcEnableInterrupt (pUdcHandle,
                                    INT_ENDPOINT_0,
                                    FIFO_ERROR_INT);

            // Enable Configuration request interrupts
            XllpUdcEnableInterrupt (pUdcHandle,
                                    INT_CONFIG,
                                    0);
            pUdcHandle->enumerateComplete = XLLP_FALSE;
            // Update statistics
            ++pUdcHandle->interruptStat.ResetIntCount;
        }

        // Is this Suspend interrupt?
        else if (statusISR1 & XLLP_UDC_UDCISR1_IRSU)
        {
            // Update statistics
            ++pUdcHandle->interruptStat.SuspendIntCount;
        }

        // Is this Resume interrupt?
        else if (statusISR1 & XLLP_UDC_UDCISR1_IRRU)
        {
            // Update statistics
            ++pUdcHandle->interruptStat.ResumeIntCount;
        }

        // Is this SOF interrupt?
        else if (statusISR1 & XLLP_UDC_UDCISR1_IRSOF)
        {
            // Update statistics
            ++pUdcHandle->interruptStat.SOFIntCount;
        }

        // Is this Configuration Change interrupt?
        else if (statusISR1 & XLLP_UDC_UDCISR1_IRCC)
        {
            XllpUdcConfigInterruptHandler (pUdcHandle);
            pUdcHandle->enumerateComplete = XLLP_TRUE;

            // Alocate temp. buffers for OUT Endpoints in active configuration
            while (pUdcHandle->listOfActiveEndpoints[j].udcEndpointNum > 0)
            {
                // Check if this is OUT endpoint
                udcEndpointNum = pUdcHandle->listOfActiveEndpoints[j].udcEndpointNum;
                if (!(pRegs->UDCCRZ[udcEndpointNum] & XLLP_UDC_UDCCRZ_ED))
                {
					if (pUdcHandle->EpXferTable[udcEndpointNum].pDataEp == NULL)
						XllpUdcSetupOutEndpoint (pUdcHandle, udcEndpointNum);
                }
                ++j;
            }
            udcEndpointNum = 0;
            // Update statistics
            ++pUdcHandle->interruptStat.ConfigIntCount;
        }
    }

    // Check if this is Endpoint 0 Packet Complete interrupt
    if (statusISR0 & XLLP_UDC_UDCISR0_IR0_0)
    {
        //PostDisplayProgress(ERR_L_XSUDC, ERR_S_XSUDC_MAIN_INT, 1);
		ClearPortInterruptFlag();
        // Call Endpoint 0 interrupt handler
        XllpUdcEp0InterruptHandler (pUdcHandle);
        // Clear the Endpoint 0 Interrupt
        XllpUdcClearInterrupt (pUdcHandle, INT_ENDPOINT_0, PACKET_COMPL_INT);
        // Update statistics
        ++pUdcHandle->interruptStat.EpIntCount[INT_ENDPOINT_0];
    }

    // Check if this is Endpoint 0 FIFO error interrupt
    if (statusISR0 & XLLP_UDC_UDCISR0_IR0_1)
    {
		ClearPortInterruptFlag();
        //PostDisplayProgress(ERR_L_XSUDC, ERR_S_XSUDC_MAIN_INT, 2);
        // Clear the Endpoint 0 Interrupt
        XllpUdcClearInterrupt (pUdcHandle, INT_ENDPOINT_0, FIFO_ERROR_INT);
    }
}


UINT32 BootRomUdcHWShutdown (void)
{
    P_XLLP_UDC_T pUdcHandle = UsbDeviceController;
    UINT32 status = FALSE;
    status = XllpUdcHWShutdown (pUdcHandle);
    // Clear Endpoints' Xfer table
    memset (&pUdcHandle->EpXferTable[0], 0, (sizeof(XLLP_UDC_XFER_T) * XLLP_UDC_MAX_EP_NUM));
    return (0);
}
