/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:   Flash.h
**
**  PURPOSE:    Holds all flash related definitions
**
******************************************************************************/
#ifndef __Flash_H__
#define __Flash_H__

#include "general.h"
#include "typedef.h"

// **************** Flash Boot State Offsets *************
// Default TIM flash locations
#define TIMOffset_CS2                                 0x00000000
#define TIMOffset_NAND                                0x00000000
#define TIMOffset_MDOC                                0x00000002  // Partition 2
#define TIMOffset_ONENAND                             0x00000000
#define TIMOffset_SDMMC                               0x00000001  // Partition 1

// Flash Types
typedef enum
{
    CS0_XIP_FLASH   = 1,
    CS2_XIP_FLASH   = 2,
    NAND_FLASH      = 3,
    ONENAND_FLASH   = 4,
    MSYS_DOC_FLASH  = 5,
    SDMMC_FLASH     = 6
} FlashType_T;

// Flash Structure Types
typedef enum
{
    BOOT_FLASH          = 0,
    SAVE_STATE_FLASH    = 1
} FlashBootType_T;

typedef UINT_T (*InitFlash_F) (pFUSE_SET pFuses, FlashBootType_T FlashBootType);
typedef UINT_T (*FinalizeFlash_F) (FlashBootType_T FlashBootType);
typedef UINT_T (*ReadFlash_F) (UINT_T FlashOffset, UINT_T LocalBuffer, UINT_T Size, FlashBootType_T FlashBootType);
typedef UINT_T (*WriteFlash_F) (UINT_T FlashOffset, UINT_T LocalBuffer, UINT_T Size, FlashBootType_T FlashBootType);
typedef UINT_T (*EraseFlash_F) (UINT_T FlashOffset, UINT_T Size, FlashBootType_T FlashBootType);
typedef UINT_T (*ResetFlash_F) (FlashBootType_T FlashBootType);
typedef UINT_T (*WriteOTP_F) (UINT_T FlashOffset, UINT_T LocalBuffer, UINT_T Size);
typedef UINT_T (*ReadOTP_F) (UINT_T FlashOffset, UINT_T LocalBuffer, UINT_T Size);
typedef UINT_T (*LockOTP_F) (void);
typedef UINT_T (*ConvertToLogicalAddr_F) (UINT_T FlashLocation, FlashBootType_T FlashBootType);

#if !BOOTROM
//-----------------------------------------
// application defined helper func for API.
// in our case FindPackageInReserved()
//-----------------------------------------
typedef UINT_T (*DataSearchFunc_F)(UINT_T *pResult, void *pResvHeader, UINT_T pkgID);
typedef UINT_T (*DataSearchFindNext_F)(UINT_T *pResult);

//------------------------------------------------------------------------------------------------------------
//
// For DKB functionality, the M-sys device needs to be formatted into partitions of various sizes
// which are defined by the customer, and stored in the downloaded TIM by the DKB.
//
// This API func expects to recieve:
//              a ptr to a searchable database, in our case, this is a downloaded TIM module.
//              a pointer to a function which searches the database (FindPackageInReserved()), and
//              the application defined package ID of the applicable reserved area.
//
// If the searh function returns a non-zero value, the API func then casts the return value to pointer to
// a data structure which it understands, and uses the data to format and partition the device.
//------------------------------------------------------------------------------------------------------------
typedef UINT_T (*FlashInitAfterTimDownload_F) ( void *pResvHeader, DataSearchFunc_F, DataSearchFunc_F, DataSearchFindNext_F, UINT_T *myPkgIDs );

#endif



typedef struct
{
    UINT_T BlockSize;
    UINT_T PageSize;
    UINT_T TimLoadAddress;
    UINT_T *ReservedPkgIDs;
    UINT8_T UseBBM;
    UINT8_T FlashInitializedState;
    UINT_T bUseSpareArea;
    UINT_T SASize;
    UINT_T NumBlocks;

    // Function pointers
    FinalizeFlash_F FinalizeFlash;
    ReadFlash_F ReadFromFlash;
    WriteFlash_F WriteToFlash;
    EraseFlash_F EraseFlash;
    ResetFlash_F ResetFlash;
    WriteOTP_F WriteOTP;
    ReadOTP_F ReadOTP;
    LockOTP_F LockOTP;
    ConvertToLogicalAddr_F pConvertToLogicalAddr;
    FlashType_T FlashDeviceNum;
#if !BOOTROM
    FlashInitAfterTimDownload_F FlashInitAftDl;
#endif
}
FlashProperties_T, *P_FlashProperties_T;

// Function Proto's
    UINT_T Configure_Flashes (UINT8_T FlashNum, FlashBootType_T FlashBootType);
    UINT_T Finalize_Flashes (FlashBootType_T FlashBootType);

    UINT_T ReadFlash(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt);
    UINT_T WriteFlash(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt);
    UINT_T EraseFlash(UINT_T FlashOffset, UINT_T size, FlashBootType_T fbt);
    UINT_T ResetFlash(FlashBootType_T fbt);
    UINT_T ReadOTP(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt);
    UINT_T WriteOTP(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt);
    UINT_T LockOTP(FlashBootType_T fbt);

    UINT_T GetBlockSize(FlashBootType_T fbt);
    UINT_T GetPageSize(FlashBootType_T fbt);
    UINT_T GetEraseRegionSize(FlashBootType_T fbt);
    UINT_T GetUseSpareArea(FlashBootType_T fbt);
    UINT_T SetUseSpareArea( UINT_T bUseSpareArea, FlashBootType_T fbt);
    UINT_T GetSpareAreaSize(FlashBootType_T fbt);
    UINT_T ConvertToLogicalAddr(UINT_T FlashLocation, FlashBootType_T FlashBootType);
#if !BOOTROM
   UINT_T FlashInitAfterDownload( void *pTim, DataSearchFunc_F SearchFunc, DataSearchFunc_F FindFirstFunc, DataSearchFindNext_F FindNextFunc, FlashBootType_T fbt  );
#endif
//Helper functions
    P_FlashProperties_T GetFlashProperties(FlashBootType_T FlashBootType);

#endif


