/* @copyright*/


#include "flash.h"
#include "RB_ImageUpdate.h"
#include "RbErrors.h"
#include "RbErrorCodes.h"
#include "fota_const.h"
#include "platform_pxa_font.h"
#include "misc.h"

extern void enable_i_d_cache(void);
extern void disable_i_d_cache(void);

#define NULL 0


//The flash should be readed page by page, every page 2048 byte
//We'll use the memory start at RAM_START_ADDRESS+9*1024*1024
int read_flash_with_align(unsigned long dwStartAddr, unsigned char* pbBuffer,  unsigned long dwSize)
{
   int ret = 0;
   unsigned long i = 0;
   unsigned char* pTempBuf = (unsigned char*)(RAM_START_ADDRESS+9*1024*1024);
   int page_align = 2048;
   int readSize = 0;
   //the flash read address and size should be align to 32.
   unsigned long sizePatch = dwSize - page_align*(dwSize/page_align);
   unsigned long addrPatch = dwStartAddr - page_align*(dwStartAddr/page_align);
   unsigned char* p = NULL;
   
   if(sizePatch == 0 && addrPatch == 0)
   {
       //the start address and readsize is aligned with page
       //direct read.
       ret = ReadFlash(dwStartAddr, pbBuffer, dwSize, BOOT_FLASH);
/* 
 * Tempary remove this code because read 0xC00000 always return 0.
 *
 * if(ret!=0)
       {
           RB_Trace(NULL,"Read flash failed. startAddr=0x%08x, pbBuffer=0x%08x, dwSize=0x%08x\r\n", dwStartAddr, pbBuffer, dwSize);
           return E_RB_READ_ERROR;
       }
*/
     RB_Trace(NULL,"Read flash success. startAddr=0x%08x, pbBuffer=0x%08x, dwSize=0x%08x\r\n", dwStartAddr, pbBuffer, dwSize);
      //dump the first 32 bytes content
/*     for(i=0; i<32; i++)
     {
        ret ++;
        RB_Trace(NULL, "%02x ", pbBuffer[i]);
        if(ret >= 16)
        { 
           RB_Trace(NULL, "\r\n");
           ret = 0;
        }
     }       
  */     
       return S_RB_SUCCESS;
   }
   
   readSize = ((addrPatch +dwSize)/page_align +1) * page_align;
   RB_Trace(NULL, "read_flash_with_align Read Size=%d\r\n", readSize);
   
   ret = ReadFlash(dwStartAddr-addrPatch, pTempBuf, readSize, BOOT_FLASH);
/*   if(ret!=0)
   {
       RB_Trace(NULL, "Read flash failed. startAddr=0x%08x, pbTempBuf=0x%08x, readSize=0x%08x\r\n", dwStartAddr, pTempBuf, readSize);
       return E_RB_READ_ERROR;
   }
  */
  
   RB_Trace(NULL, "Read flash success. startAddr=0x%08x, pbTempBuf=0x%08x, readSize=0x%08x\r\n", dwStartAddr, pTempBuf, readSize);
   
   //copy the needed content from the temp buffer
   for(i=0; i<dwSize; i++)
   {
      pbBuffer[i] = pTempBuf[i+addrPatch];
   }
   
    //dump the first 32 bytes content
/*     for(i=0; i<32; i++)
     {
        ret ++;
        RB_Trace(NULL, "%02x ", pbBuffer[i]);
        if(ret >= 16)
        { 
           RB_Trace(NULL, "\r\n");
           ret = 0;
        }
     }
  */
   
   return S_RB_SUCCESS;
}

/*----------------------------------------------------------------------------------
 RB_GetDelta() - Get the Delta either as a whole or in block pieces 
 params:
     void *pbUserData         -- User data passed to all porting routines 
	 unsigned char *pbBuffer  -- pointer to user buffer 
     unsigned long dwStartAddressOffset  -- offset from start of delta file
     unsigned long dwSize     -- Number of delta-update bytes to be copied into the user buffer. 

 return:
     S_RB_SUCCESS   -- read success
     other          -- failed   
-----------------------------------------------------------------------------------*/
long RB_GetDelta(
	void *pbUserData,       
	unsigned char *pbBuffer, 
    unsigned long dwStartAddressOffset,
    unsigned long dwSize)              
{
     unsigned long ret = 0;
     unsigned long startAddr = DELTA_FILE_START_ADDRESS + dwStartAddressOffset;
     unsigned long endAddr = startAddr + dwSize;
         
     RB_Trace(NULL, "Enter RB_GetDelta. pbBuffer = 0x%08x, dwStartAddressOffSet = 0x%08x, dwSize = %d\r\n", pbBuffer, dwStartAddressOffset, dwSize);
     if(!pbBuffer)
     {   
         RB_Trace(NULL, "Empty pointer encountered in RB_GetDelt.\r\n");
         return E_RB_BAD_PARAMS;
     }
   
     if( endAddr > (DELTA_FILE_START_ADDRESS + DELTA_FILE_MAX_SIZE))
     {
         RB_Trace(NULL, "Expected content end address 0x%08x exceed Delta file max size 0x%08x.\r\n", endAddr,  DELTA_FILE_START_ADDRESS + DELTA_FILE_MAX_SIZE);
         return E_RB_READFILE_SIZE;
     }
     
     RB_Trace(NULL, "Before Read Delta, startAddr=0x%08x pBuff=0x%08x, size=%d\r\n", startAddr, pbBuffer, dwSize);
     ret = read_flash_with_align( startAddr, pbBuffer, dwSize);
     if( S_RB_SUCCESS != ret)	
     {
         RB_Trace(NULL, "Read Delta from flash %0x to %0x failed. readed return %d, expect size %d.\r\n", startAddr, endAddr, ret, dwSize);
         return E_RB_READ_ERROR;
     }
     RB_Trace(NULL, "Leave RB_GetDelta.\r\n");
     return S_RB_SUCCESS;
}

/*------------------------------------------------------------------------------------------------------------------- 
RB_GetBlockSize() - Returns the size of one memory block. 
 -------------------------------------------------------------------------------------------------------------------*/ 
long RB_GetBlockSize(void)/* returns the size of a memory-block */
{
    RB_Trace(NULL, "RB_GetBlockSize = %d\r\n", FLASH_BLOCK_SIZE);
    return FLASH_BLOCK_SIZE;
}

/*------------------------------------------------------------------------------------------------------------------- 
 RB_ReadImage() - Reads data from flash.
 This function should copy data from Flash memory into a RAM buffer. This function is used to read image data.
 params:
    void *pbUserData -- User data passed to all porting routines                      
    unsigned char *pbBuffer -- Pointer to a RAM buffer where the data should be written      
    unsigned long dwStartAddress -- Location in Flash from which the data should be read
    unsigned long dwSize -- Number of bytes to copy, this can exceeds sector size
  return:
     S_RB_SUCCESS  -- OK
     other         -- failed.     
  -------------------------------------------------------------------------------------------------------------------*/  
long RB_ReadImage(
    void *pbUserData,            
    unsigned char *pbBuffer,      
    unsigned long dwStartAddress, 
    unsigned long dwSize)         
{
     unsigned long ret = 0;
    
     RB_Trace(NULL, "Enter RB_ReadImage. pbBuffer = 0x%08x, dwStartAddress = 0x%08x, dwSize = %d\r\n", pbBuffer, dwStartAddress, dwSize);

     //parameter check
     if(!pbBuffer)
     {
         RB_Trace(NULL, "Empty pointer encountered in RB_ReadImage.\r\n");
         return E_RB_BAD_PARAMS;
     }
     if( dwStartAddress > IMAGE_END_ADDRESS ||dwStartAddress < IMAGE_START_ADDRESS)
     {
         RB_Trace(NULL, "Error Block Address %08x in RB_ReadImage.\r\n", dwStartAddress);
         return E_RB_BAD_PARAMS;
     }

     RB_Trace(NULL, "Before Called readFlash.\r\n");
     //Read Flash
     ret = read_flash_with_align( dwStartAddress, pbBuffer, dwSize);
     if( S_RB_SUCCESS != ret)	
     {
         RB_Trace(NULL, "Read flash from %08x failed. Read  return %d expect  size %d.\r\n", dwStartAddress, ret, dwSize);
         return E_RB_READ_ERROR;
     }
      
     RB_Trace(NULL, "Leave RB_ReadImage.\r\n");
    
     return S_RB_SUCCESS;

}

/*------------------------------------------------------------------------------------------------------------------- 
 RB_WriteBlock() - Writes one block to a given location in flash.
 This function should copy one block from RAM to a given location in Flash. The
 function is used by UPI to update the device image. If part of the image on the
 device is encrypted, the device manufacturer needs to take it into account when
 implementing this function.

 params:
    void *pbUserData -- User data passed to all porting routines                      
    unsigned long dwBlockAddress -- Location in Flash where the data must be written
    unsigned char *pbBuffer -- Pointer to user buffer where the data is located and from where it must be copied.    
  return:
     S_RB_SUCCESS  -- OK
     E_RB_WRITE_ERROR -- Failure to write to Flash.
     other   -- failed.     
  -------------------------------------------------------------------------------------------------------------------*/ 
long RB_WriteBlock(
        void *pbUserData, 
        unsigned long dwBlockAddress,
        unsigned char *pbBuffer)     
{
     int ret = 0;

     RB_Trace(NULL, "Enter RB_WriteBlock. dwBlockAddress = 0x%08x  pbBuffer = 0x%08x\r\n", dwBlockAddress, pbBuffer);
     
     //parameter check
     if(!pbBuffer)
     {
         RB_Trace(NULL, "Empty pointer encountered in RB_WriteBlock.\r\n");
         return E_RB_BAD_PARAMS;
     }
     if( dwBlockAddress > IMAGE_END_ADDRESS ||dwBlockAddress < IMAGE_START_ADDRESS)
     {
         RB_Trace(NULL, "Error Block Address %08x in RB_WriteBlock.\r\n", dwBlockAddress);
         return E_RB_BAD_PARAMS;
     }
    
     //Erase flash
     ret = EraseFlash( dwBlockAddress, FLASH_BLOCK_SIZE, BOOT_FLASH);
     if(ret != 0)	
     {
         RB_Trace(NULL, "Erase block %08x failed. returned %d.\r\n", dwBlockAddress, ret);
         return E_RB_ERASE_ERROR;
     }

     //Write Flash
     ret = WriteFlash( dwBlockAddress, pbBuffer,FLASH_BLOCK_SIZE, BOOT_FLASH);
     if(ret != 0)	
     {
         RB_Trace(NULL, "Write flash block %08x failed. returned %d.\r\n", dwBlockAddress, ret);
         return E_RB_WRITE_ERROR;
     }
    
     RB_Trace(NULL, "Leave RB_WriteBlock\r\n");
     return S_RB_SUCCESS;
     
}

/* ------------------------------------------------------------------------------------------------------------------
 RB_ReadBackupBlock() - Copy data from backup block to RAM. 
 This function should copy the content from the address in one of the backup blocks into a RAM location provided by the UPI.
 The RB_ReadBackupBlock function is used by the UPI both for retrieving data during normal update process and when recovering 
 from an interrupted update situation (e.g. when the battery is removed during the update process). In the latter case, 
 RB_ReadBackupBlock is used to retrieve the contents of the backup block and compare them with the image content.
 Params:
    void *pbUserData -- User data passed to all porting routines 
    unsigned char *pbBuffer -- pointer to user buffer in RAM where the data will be copied 
    unsigned long dwBlockAddress -- address of data to read into RAM. Must be inside one of the backup buffer blocks 
    unsigned long dwSize -- Number of bytes to copy into user buffer
 return:
    S_RB_SUCCESS   -- read success
    other          -- failed
 --------------------------------------------------------------------------------------------------------------------*/
long RB_ReadBackupBlock(
        void *pbUserData,             
        unsigned char *pbBuffer,     
        unsigned long dwBlockAddress,
        unsigned long dwSize)
{
     //check the address is in the backup block list.
     int find = 0;
     unsigned long ret = 0;
     int i = 0;
     
     RB_Trace(NULL, "Enter RB_ReadBackupBlock. pbBuffer = 0x%08x, dwBlockAddress = 0x%08x, dwSize = %d\r\n", pbBuffer, dwBlockAddress, dwSize); 
     if(!pbBuffer)
     {
         RB_Trace(NULL, "Empty pointer encountered in RB_ReadBackupBlock.\r\n");
         return E_RB_BAD_PARAMS;
     }

     for (i = 0; i < UPDATE_BACUP_BLOCK_NUM; i++)
     {
         if(dwBlockAddress == UPDATE_BACKUP_START_ADDRESS + i * FLASH_BLOCK_SIZE)
             find = 1;
     }

     if( find != 1 )
     {
         RB_Trace(NULL, "Update Backup address incorrect.\r\n");
         return E_RB_BAD_PARAMS;
     }  
        
     ret = read_flash_with_align( dwBlockAddress, pbBuffer, dwSize);
     if( S_RB_SUCCESS != ret)	
     {
         RB_Trace(NULL, "Read Backup block flash start from %08x failed. read return %d expect size %d.\r\n", dwBlockAddress, ret, dwSize);
         return E_RB_READ_ERROR;
     }
    
     RB_Trace(NULL, "Leave RB_ReadBackupBlock.\r\n");
     return S_RB_SUCCESS;

}

/* ------------------------------------------------------------------------------------------------------------------
 RB_WriteBackupBlock() - Copy data from specified address in RAM to a backup block.
 This function should copy one block from RAM to the specified buffer block address. A full block is always written.
 This function is used by UPI when persistent storage is required during the update process. In essence, this function
 performs two operations: the entire block is erased followed by the write operation.
 Params:
     void *pbUserData -- User data passed to all porting routines 
     unsigned long dwBlockStartAddress -- Location of the buffer block where the data must be written. Will always be 
                                          one of the blocks addresses provided to the RB_ImageUpdate routine.
     unsigned char *pbBuffer -- Pointer to user buffer in RAM where the data is located and from where it must be copied
 Return:
    S_RB_SUCCESS   -- read success
    other          -- failed
 ------------------------------------------------------------------------------------------------------------------*/ 
long RB_WriteBackupBlock(
        void *pbUserData, 
        unsigned long dwBlockStartAddress,
        unsigned char *pbBuffer)
{
    //check the address is in the backup block list.
     int find = 0;
     unsigned long ret = 0;
     int i = 0;
     
     RB_Trace(NULL, "Enter RB_WriteBackupBlock. dwBlockStartAddress = 0x%08x, pbBuffer = 0x%08x\r\n", dwBlockStartAddress, pbBuffer);
     //parameter check
     if(!pbBuffer)
     {
         RB_Trace(NULL, "Empty pointer encountered in RB_WriteBackupBlock.\r\n");
         return E_RB_BAD_PARAMS;
     }

     for (i = 0; i < UPDATE_BACUP_BLOCK_NUM; i++)
     {
         if(dwBlockStartAddress == UPDATE_BACKUP_START_ADDRESS + i * FLASH_BLOCK_SIZE)
             find = 1;
     }

     if( find != 1 )
     {
         RB_Trace(NULL, "Update Backup address incorrect in RB_WriteBackupBlock.\r\n");
         return E_RB_BAD_PARAMS;
     }     

     //Erase block
     ret = EraseFlash( dwBlockStartAddress, FLASH_BLOCK_SIZE, BOOT_FLASH);
     if(ret!= 0)	
     {
         RB_Trace(NULL, "Erase block %08x failed. returned %d.\r\n", dwBlockStartAddress, ret);
         return E_RB_ERASE_ERROR;
     }
    
     //Write Flash
     ret = WriteFlash( dwBlockStartAddress, pbBuffer,FLASH_BLOCK_SIZE, BOOT_FLASH);
     if(ret != 0)	
     {
         RB_Trace(NULL, "Write Backup block %08x failed. returned %d.\r\n", dwBlockStartAddress, ret);
         return E_RB_WRITE_ERROR;
     }
     
     RB_Trace(NULL, "Leave RB_WriteBackupBlock.\r\n");
    
     return S_RB_SUCCESS;

}

/*--------------------------------------------------------------------------------------------------------------------------------
 RB_Progress() - Report the current stage of the update
 This function receives the progress of the image update operation in percents,
 and should use this information to display progress indication to the device user.
 Params:
    void *pbUserData -- User data passed to all porting routines 
    unsigned long uPercent --Progress information in percents
 Returns:
    null
 */
void RB_Progress(
	void *pbUserData,					
	unsigned long uPercent)
{
    LCD_RECT rect;
    char str[] = "0123456789";
    char percent[5];
    int t = uPercent/10;
    rect.x0 = 0;

    if(uPercent > 100)
    {
        RB_Trace(NULL, "Strange error, percent = %u\r\n", uPercent);
        return;
    }

    if(uPercent == 100)
    {
        percent[0] = '1';
        percent[1] = '0';
        percent[2] = '0';
        percent[3] = '%';
        percent[4] = 0;    
    }
 
    else if(uPercent > 9)
    {
        percent[0] = str[uPercent/10];
        percent[1] = str[uPercent -10*(uPercent/10)];
        percent[2] = '%';
        percent[3] = 0;
    }
    
    else
    {
        percent[0] = '0';
        percent[1] = str[uPercent];
        percent[2] = '%';
        percent[3] = 0;
    }
   
    lcd_printf(percent,rect.x0+140,260,4,FONT_COLOR_RED,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);

    RB_Trace(NULL, "Update finished %u percent.\r\n", uPercent);
}			


/*--------------------------------------------------------------------------------------------------------------------------------
 RB_ResetTimerA() - Reset watchdog timer A
 This function is called by the UPI periodically, at least once within each 30-
 second period, and should be used on the device to handle all the watchdogs.
 Params:
       null
 return:
       S_RB_SUCCESS -- ok
       others -- failed
 */
long RB_ResetTimerA(void)
{
   return S_RB_SUCCESS;
}


int updateFotaFlag(char* s, int len)
{
   char *pBuf = (char*)(RAM_START_ADDRESS +9*1024*1024);
   int i=0;
   int ret = 0;
   for(i=0; i<FLASH_PAGE_SIZE; i++) pBuf[i] = 0;


   //Read the flag page, keep the FS flag
   read_flash_with_align(FOTA_FLAG_ADDRESS, pBuf,  32);
   
   for(i=0;i<len; i++)
   pBuf[i] = s[i];
  
   pBuf[32] = 0;  //for print trace

    RB_Trace(NULL, "Erase FOTA flag, save fota result = %s\r\n", pBuf);
    
    //Erase flash
    ret = EraseFlash( FOTA_FLAG_ADDRESS, FLASH_BLOCK_SIZE, BOOT_FLASH);
    if(ret != 0)	
    {
        RB_Trace(NULL, "Erase block %08x failed. returned %d.\r\n", FOTA_FLAG_ADDRESS , ret);
        return E_RB_ERASE_ERROR;
    }

    //Write Flash
    ret = WriteFlash( FOTA_FLAG_ADDRESS, pBuf,FLASH_PAGE_SIZE, BOOT_FLASH);
    if(ret != 0)	
    {
        RB_Trace(NULL, "Write flash block %08x failed. returned %d.\r\n", FOTA_FLAG_ADDRESS , ret);
        return E_RB_WRITE_ERROR;
    }

    return S_RB_SUCCESS;
}



int FOTA_Main(void)
{
     unsigned long operationMode = 0;
     unsigned char* pbyRAM = (unsigned char*)(RAM_START_ADDRESS+10*1024*1024);  //32M memory allocate for Fota UA usage
     LCD_RECT rect;
     unsigned long* pBackupBlock[UPDATE_BACUP_BLOCK_NUM];
     pBackupBlock[0] = UPDATE_BACKUP_START_ADDRESS;
     pBackupBlock[1] = UPDATE_BACKUP_START_ADDRESS + FLASH_BLOCK_SIZE;
     pBackupBlock[2] = UPDATE_BACKUP_START_ADDRESS + 2 * FLASH_BLOCK_SIZE;   
     
    // RB_Trace(NULL, "going to call RB_ReadImage first.\r\n"); 
    // RB_ReadImage(NULL, 0x80100000, 0xc00000, 0x20000);
 
     RB_Trace(NULL, "going to call RB_ImageUpdate with operation mode = %d.\r\n", operationMode);   
    

    enable_i_d_cache();

         
    RB_RETCODE ret = RB_ImageUpdate(
             NULL,					// User data passed to all porting routines 
             0x100000,      		// start address of app to update 
             0xd40000,			    // end address of app to update
             pbyRAM,				// pointer for the ram to use 
             32*1024*1024,  	    // size of the ram 
             NULL,                   //unsigned char *pbyInternalRAM,
             0,                      // unsigned long dwInternalRAMSize
             pBackupBlock,	    // array of backup buffer block (sector) addresses 
             UPDATE_BACUP_BLOCK_NUM,                     //unsigned long dwNumOfBuffers
             0,                     //unsigned long dwOperation,
             0,                     //unsigned long dwUpdateParamsSize,	 must be initialized to sizeof(RB_UpdateParams) 
             NULL);		            // additional update parameters 
            

     /* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      *  It's really important here to reset the FOTA flag, otherwise the phone will 
      *  enter dead loop when power on. 
      * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/         

     
     if(ret == RB_SUCCESS_STATUS)
     {
         //Erase FOTA Flag
         RB_Trace(NULL, "!!!!!!!!!!!!!!!!update success.\r\n");
         updateFotaFlag("SSSS", 4);

         //change display information
         lcd_printf("Update Sucess.",rect.x0+40,310,2,FONT_COLOR_GREEN,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
     }
     else
     {
         updateFotaFlag("FFFF", 4);
         RB_Trace(NULL, "................Update Failed.ret =%x\r\n", ret);

         //change display information
         lcd_printf("Update Failed!",rect.x0+40,310,2,FONT_COLOR_GREEN,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
     }
          
     Delay_ms(4000); //delay 4 seoncds

     disable_i_d_cache();

     return 0;
}















