/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. ?
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	General.h
**
**  PURPOSE: 	Contain boot ROM definitions used accross all components
**
******************************************************************************/
#ifndef __GENERAL_H__
#define __GENERAL_H__

#include <typedef.h>

#ifdef FBW_CODE
#define ____cache_in_read   1
#endif
/****************************************************************************
*   Vital Compiler Switches
****************************************************************************/

// Flash Switches
#define FLASH_ERASE			1	//Allow Erasing of flash
#define COPYIMAGESTOFLASH	1

//SCAN_FOR_BADBLOCKS tells the DKB to erase each block and check to see if erase
// succeeded.  If it does not, it will relocate the block
#define SCAN_FOR_BADBLOCKS	FLASH_ERASE
//Note: This flag is tied to the FLASH_ERASE flag.  If FLASH_ERASE is *NOT* set, then
// this flag is turned off (because there is no way to tests the blocks if the DKB
// is not allowed to erase blocks)


// Protocol Switches
#define DOWNLOAD_USED		1
#define VERBOSE_MODE		0
#define VERBOSE_DEBUG		1	//do NOT set this flag to 1
								//kept only for OBM/DKB comparability

// Security Switches
#define DKBTEST				0

#if TRUSTED					// Set in the make file.
 #define DEVICE_KEY 			0

 #if DEVICE_KEY
		#define PERFORM_OTP_LOCKING	 	1
 #endif
#endif

#define PERFORM_FUSE_PROGRAM 	0

/*********************************************************************************
* Useful constants
**********************************************************************************/
#define BIT0		0x00000001
#define BIT1		0x00000002
#define BIT2		0x00000004
#define BIT3		0x00000008
#define BIT4		0x00000010
#define BIT5		0x00000020
#define BIT6		0x00000040
#define BIT7		0x00000080
#define BIT8		0x00000100
#define BIT9		0x00000200
#define BIT10		0x00000400
#define BIT11		0x00000800
#define BIT12		0x00001000
#define BIT13		0x00002000
#define BIT14		0x00004000
#define BIT15		0x00008000
#define BIT16		0x00010000
#define BIT17		0x00020000
#define BIT18		0x00040000
#define BIT19		0x00080000
#define BIT20		0x00100000
#define BIT21		0x00200000
#define BIT22		0x00400000
#define BIT23		0x00800000
#define BIT24		0x01000000
#define BIT25		0x02000000
#define BIT26		0x04000000
#define BIT27		0x08000000
#define BIT28		0x10000000
#define BIT29		0x20000000
#define BIT30		0x40000000
#define BIT31		0x80000000
#define SET32		0xffffffff
#define CLEAR32		0x00000000

#define KeyLength_16						16
#define KeyLength_32						32
#define KeyLength_64						64
#define WordLengthOf_SHA1		 			5
#define OEMKeyHashDataLengthConst			64 	// Size of OEM Key Hash
#define KeyLengthMask	   					0x000000FF

// For debug purposes only
#define HEX_LED_CONTROL (volatile unsigned long *)( 0x08000040 )
#define HEX_LED_WRITE (volatile unsigned long *)( 0x08000010 )

/********************* DDR Image Verification Area ******************/
#define 	DDR_DKB_IMAGE_DOWNLOAD 				0x80200000

//************** Platform Configuration Fuse Information **************

#define FMRD6	 			6
#define FMRD7	 			7
#define FMRD8 				8
#define VirginEnableMask	0x00000010
#define Boot_DebugE_Mask	0x0000001F

#define Default				0x00000000
#define MSystemDFI			0x4e414e01
#define OneNandDFI			0x4e414e02
#define XIPDDR16EMPI		0x58495003		// x16 Sibley on SMC->EMPI->CS0
#define NAND_1_16_16		0x4e414e04	    // x16 NAND   on DFC->DFI->CS0
#define Sibley_16			0x58495005		// x16 Sibley on SMC->DFI->CS2
#define NAND_1_8_8			0x4e414e06		// x8  NAND   on DFC->DFI->CS0
#define Tyax16DFI			0x58495007		// x16 TYAX   on SMC->DFI->CS2

// **************** Flash Boot State Offsets *************
// Starting Base Addresses
#define CS2_XIP_VIRTUAL_START_ADDRESS  		0x10000000
#define CS0_XIP_VIRTUAL_START_ADDRESS  		0x20000000
#define CS2_XIP_PHYSICAL_START_ADDRESS  	0x10000000
#define CS0_XIP_PHYSICAL_START_ADDRESS  	0x00000000
#define NAND_FLASH_START_ADDRESS			0x00000000

// OBM Download address
#define TIMOffset_CS0						0x0000C000 // offset for TIM on CS0
#define TIMOffset_CS2						0x00000000
#define TIMOffset_NAND						0x00000000
#define CS2Base								CS2_XIP_VIRTUAL_START_ADDRESS
#define CS0Base								CS0_XIP_VIRTUAL_START_ADDRESS
#define ISRAM_IMAGE_LOAD_BASE				0x5C013000
#define ISRAM_PHY_ADDR						0x5c000000
#define DDR_PHY_ADDR						0x80000000
#define OBM_RESTRICTED_DDR_SPACE			0x00040000 // 256 KB at beginning of DDR
#define TIMLOADADDRESS						0x5C008000

// DKB download of Master_Block_Header from FBF file
#define MASTER_BLOCK_HEADER_ADDR            DDR_DKB_IMAGE_DOWNLOAD
#define TEMP_BUFFER_AREA                    0x80220000
#define MBH_IMAGE_DOWNLOAD_AREA             0x80260000

#ifdef FBW_CODE
#define PRETIMLOADADDRESS				   (0x80100000)//0x40000-64*1024)
#else
#define PRETIMLOADADDRESS				   (0x80040000-64*1024)
#endif
/*
 *  Fuse definition structure
 */
struct PLATFORM_FUSES{
	unsigned int DebugEn			:1;		// Bit 0 of FMRD8
	unsigned int PlatformState		:4;		// Bit 1-4 of FMRD8
	unsigned int Caddo_Disable		:1;		// Bit 5 of FMRD8
	unsigned int JTAG_Disable		:1;		// Bit 6 of FMRD8
	unsigned int Patching_Disable	:1;		// Bit 7 of FMRD8
	unsigned int Patching_Revision	:4;		// Bit 8-11	of FMRD8
    unsigned int UARTPort           :1;     // 0 - FFUART, 1 - AltUART
    unsigned int USBPort            :1;     // 0 - DIFFUSB 1 - SEUSB
	unsigned int CaddoDead			:1;		// 1 = Caddo did not come up.
	unsigned int Resume				:1;		// 1 = resume from sleep in process
	unsigned int USBWakeup			:1;		// 1 = USB wakeup from WSOTG
	unsigned int PortEnabled		:1;		// 1 = Ports have been enabled
	unsigned int DDRInitialized		:1; 	// If the BootROM already initializes the DDR
	unsigned int Download_Disable	:1;		// Bit 19 of services FMRD2 - Used to be MH-TV if 1 disable ports
	unsigned int SBE				:1;		// Bit 20 of services FMRD2
	unsigned int SDE				:1;		// Bit 21 of services FMRD2
	unsigned int BootPlatformState	:4;		// There are the bits the BootROM booted from
	unsigned int res				:6;
};

typedef union{
	unsigned int value;
	struct PLATFORM_FUSES bits;
}FUSE_SET, *pFUSE_SET;

typedef enum {
	OEMHash,
	JTAGHash
}OTPHashType;

#endif
