/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Interrupts.c
**
**  PURPOSE: 	Contain Interrupts/Exception handlers for Bulverde B0 Boot ROM
**
******************************************************************************/

#if (BOOTROM || TRUSTED)
 #include "dsvl.h"
#endif
#include "platformconfig.h"
#include "interrupts.h"
#include "protocolmanager.h"
#include "errors.h"
#include "xllp_dmac.h"

static UINT8_T UdcDmaUsageFlag   = FALSE;  // Used to enable/disable DMA usage
static UINT8_T PortInterrupt	 = FALSE;

UINT8_T GetPortInterruptFlag(void){ return PortInterrupt;}


void ClearPortInterruptFlag(void){
	PortInterrupt = FALSE;
	return;
}

UINT_T EnablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID)
{
	VUINT32_T Interrupt = 0;

	if ((InterruptID >= MAX_PLATFORM_INTS) || (InterruptID < 0))
	{
		return (GeneralError);
	}

	if (InterruptID < MAX_ICIP1_INTS)
	{
		Interrupt = *ICMR;  // Read interrupt mask register
		Interrupt |= (1 << InterruptID);
		*ICMR = Interrupt;
	}
	else if (InterruptID < MAX_ICIP2_INTS)
	{
		InterruptID = (InterruptID - MAX_ICIP1_INTS);
		Interrupt = *ICMR2;  // Read interrupt mask register
		Interrupt |= (1 << InterruptID);
		*ICMR2 = Interrupt;
	}
	else
	{
		InterruptID = (InterruptID - MAX_ICIP2_INTS);
		Interrupt = *ICMR3;  // Read interrupt mask register
		Interrupt |= (1 << InterruptID);
		*ICMR3 = Interrupt;
		*ICLR3 = 0;   // Route all Interrupts to CPU IRQ (none to FIQ)
	}

	// Enable ONLY IRQ interrupts
  	*ICLR  = 0;   // Route all Interrupts to CPU IRQ (none to FIQ)
  	*ICLR2 = 0;   // Route all Interrupts to CPU IRQ (none to FIQ)

	return(NoError);
}

void EnableUdcDmaUsage(void)
{
	UdcDmaUsageFlag = TRUE;
}
void DisableUdcDmaUsage(void)
{
	UdcDmaUsageFlag = FALSE;
}

UINT_T DisablePeripheralIRQInterrupt(BULVERDE_INTC_T InterruptID)
{
	VUINT32_T Interrupt = 0;

	if ((InterruptID >= MAX_PLATFORM_INTS) || (InterruptID < 0))
	{
		return (GeneralError);
	}

	if (InterruptID < MAX_ICIP1_INTS)
	{
		Interrupt = *ICMR;  // Read interrupt mask register
		Interrupt &= ~(1 << InterruptID);
		*ICMR = Interrupt;
	}
	else if (InterruptID < MAX_ICIP2_INTS)
	{
		InterruptID = (InterruptID - MAX_ICIP1_INTS);
		Interrupt = *ICMR2;  // Read interrupt mask register
		Interrupt &= ~(1 << InterruptID);
		*ICMR2 = Interrupt;
	}
	else
	{
		InterruptID = (InterruptID - MAX_ICIP2_INTS);
		Interrupt = *ICMR3;  // Read interrupt mask register
		Interrupt &= ~(1 << InterruptID);
		*ICMR3 = Interrupt;
	}
	return(NoError);
}

void UndefinedHandler(void)
{
	// Terminate, simply loop forever for now
	while (1){}
}

void SwiHandler(void)
{
	// Terminate, simply loop forever for now
	while (1){}
}

void PrefetchHandler(void)
{
	// Terminate, simply loop forever for now
	while (1){}
}

void AbortHandler(void)
{
	// Terminate, simply loop forever for now
	while (1){}
}

void IrqHandler(void)
{
	VUINT32_T ICIPRegValue;
	VUINT32_T ICIP2RegValue;
	VUINT32_T ICMRRegValue;
	VUINT32_T ICMR2RegValue;
	VUINT32_T ICHPRegValue;
	UINT_T DINTValue;
	P_XLLP_DMAC_T pDmac_h;

#if USBCI
	VUINT32_T ICIP3RegValue = *ICIP3;
	VUINT32_T ICMR3RegValue = *ICMR3;
#endif

	//INT_T IcipInterrupt  = 0;
	ICHPRegValue = *ICHP;

	// Check to see if we got a valid interrupt

	// Read ICIP (pending interrupts)
	ICIPRegValue  = *ICIP;
	ICIP2RegValue = *ICIP2;
	// Read ICMR (masked interrupts)
	ICMRRegValue  = *ICMR;
	ICMR2RegValue = *ICMR2;


    // Do we have an FFUART interrupt?
	if ((ICIPRegValue & FFUARTInterruptMask) == FFUARTInterruptMask)
	{
		if ((ICMRRegValue & FFUARTInterruptMask) == FFUARTInterruptMask)
		{
			PortInterrupt = TRUE;
			FFUARTInterruptHandler();
		}
	}
	// Verify if we have a USB interrupt pending.  If
	// so call the BootROM USB interrupt handler
	if ((ICIPRegValue & USBInterruptMask) == USBInterruptMask)
	{
		if ((ICMRRegValue & USBInterruptMask) == USBInterruptMask)
		{
			PortInterrupt = TRUE;
			PlatformUsbInterruptHandler();
		}
	}


	// Verify if we have a DMA interrupt pending.  If
	// so call the BootROM DMA interrupt handler
	if ((ICIPRegValue & DMAInterruptMask) == DMAInterruptMask)
	{
		if ((ICMRRegValue & DMAInterruptMask) == DMAInterruptMask)
		{
			// Read in the DMA DINT
			pDmac_h = XLLP_Return_PDmacHandle();
			DINTValue = pDmac_h->DINT;

			// Handle the appropriate DMA channel
 		 	if (DINTValue & (1 << MEM2MEM_CHANNEL))
				#if (!DKB)
					Mem2Mem_ISR();
				#else
					XllpDmacClearDmaInterrupt(MEM2MEM_CHANNEL);
				#endif

			if (DINTValue & (1 << WTM_RX_CHANNEL))
				XllpDmacClearDmaInterrupt(WTM_RX_CHANNEL);

			if (DINTValue & (1 << WTM_TX_CHANNEL))
				XllpDmacClearDmaInterrupt(WTM_TX_CHANNEL);

			if (DINTValue & (1 << WTM_CMD_CHANNEL))
					XllpDmacClearDmaInterrupt(WTM_CMD_CHANNEL);

			if (DINTValue & (1 << USB_TX_CHANNEL))
			{
					if (UdcDmaUsageFlag)
						PlatformUsbDmaHandler();
					else
						XllpDmacClearDmaInterrupt(USB_TX_CHANNEL);
			}
			if (DINTValue & (1 << USB_RX_CHANNEL))
			{
				if (UdcDmaUsageFlag)
					PlatformUsbDmaHandler();
				else
					XllpDmacClearDmaInterrupt(USB_RX_CHANNEL);
			}
		}
	}

	// Verify if we have a U2D interrupt pending.  If
	// so call the U2D interrupt handler
	//--------------------------------------------------------------
	if ((ICIP2RegValue & U2DInterruptMask ) == U2DInterruptMask)
	{
		if ((ICMR2RegValue & U2DInterruptMask) == U2DInterruptMask)
		{
			PortInterrupt = TRUE;
			PlatformU2DInterruptHandler();
		}
	}
#if USBCI
	// Here we need to handle USB CI2 interrupt
	//--------------------------------------------------------------
	if ((ICIP3RegValue & CI2InterruptMask ) == CI2InterruptMask)
	{
		if ((ICMR3RegValue & CI2InterruptMask) == CI2InterruptMask)
		{
			PortInterrupt = TRUE;
         PlatformCI2InterruptHandler();
		}
	}
#endif

#if (TRUSTED || BOOTROM)
	// Do we have a WTM interrupt?
	if ((ICIP2RegValue & WTMINTERRUPTMASK) == WTMINTERRUPTMASK)
		WTMInterruptHandler();
#endif

	return;
}

void FiqHandler(void)
{
   while (1){}
   //return;
}

