/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Security.c
**
**  PURPOSE: 	Contain template OEM code for secure functions using IppCp
**
**	History:	Initial Creation Tolga Aytek, 5/16/05
**
******************************************************************************/

#include "dkb_security.h"
#include "misc.h"

#if NDT
#include "stdlib.h"
#endif

/********************	ValidateImage ******************************************************
*  Validate the Image based on a Hash passed in for the TIM or the hash for individual images
*  can be determined from IMAGE_INFO structure.
*
********************************************************************************************/
UINT_T ValidateImage(UINT_T ImageID, UINT_T* TIM_Hash, pTIM pTIM_h)
{
 UINT_T Retval = NoError;
 UINT_T StartAddr, EndAddr, HashSize, i, IVM_KeyLength;
 UINT_T CalculatedHash[WordLengthOf_SHA1];			// Contains 5 words
 UINT_T* HashInTIM;
 UINT8_T ImageOrder;

 // First Find the Image
 for( i = 0; i < pTIM_h->pConsTIM->NumImages; i++)
 {
	 if(pTIM_h->pImg[i].ImageID == ImageID)
	 {
		ImageOrder = i;
		break;
	 }
 }

 // Get Start Address to begin Hash
 switch (ImageID)
 {
  case TIMIDENTIFIER:
    StartAddr = (UINT_T) pTIM_h->pConsTIM;			  	// Validate in ISRAM
	HashInTIM = TIM_Hash;
	break;
  default:		// For all subsequent images get downloaded to DDR scratch area
  	StartAddr = DDR_DKB_IMAGE_DOWNLOAD;				  	// Validate in DDR
	HashInTIM = pTIM_h->pImg[ImageOrder].Hash;
	break;
 }

 // Get the image size to hash
 HashSize  = pTIM_h->pImg[ImageOrder].ImageSizeToHash;

 // Do the SHA1 Hash
 Retval = WtmSHA1MessageDigest((unsigned char *)StartAddr, HashSize, (unsigned char *)CalculatedHash);
 CheckErrorReturn(Retval);

 // Compare the 2 hashes
 for (i = 0; i < WordLengthOf_SHA1; i++)
 {
	if (HashInTIM[i] != CalculatedHash[i])
		return (InvalidTIMImageError);
 }

 return NoError;
}

/****************** Program Platform Configuration Fuses *************
* This routine is called only if trusted boot is successful so far...
*********************************************************************/
UINT_T ProgramPlatformFuses(UINT_T PlatformSettings)
{
 UINT_T Retval;
 FUSE_SET  Fuses;				// Fuse information passed in from BootROM
 UINT_T CaddoPConfigOutput[3] = {0,0,0};
 unsigned char Msg[9];
 UINT_T MsgInt;
 UINT_T i;

 Fuses.value = 0;
 Fuses.bits.PlatformState = PlatformSettings;
 Fuses.value &= 0x001E;			// Mask out all bits but the flash configuration bits


  // Write it out
 Retval = WtmPlatform_Config (Fuses.value, CaddoPConfigOutput, 0x2A8, 0x2A8, 0x1530);
 AddMessage("Output from Platform Config:\0");
 for (i = 0; i < 3; i++){
		MsgInt = CaddoPConfigOutput[i];
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
 }
 if (Retval != NoError){
		AddMessage((UINT8_T*) ("***  Retval :\0"));
		MsgInt = Retval;
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
  }
// if (Fuses.value != (CaddoPConfigOutput[0] & 0x1E))
//	return (FuseRWError);

 return NoError;
}
/***************** Program WTM State Data *******************************/
UINT_T GenerateWTMStateData(pSaveState pSaveState_h)
{
 UINT_T Retval = NoError;
 UINT_T Subkey_Pass[] = {0x6a09e667, 0xf3bcc908, 0xb2fb1366, 0xea957d43, 0x521f6bb9};
 UINT_T DummyRNGSeed[]    = {0xF07DE2F1, 0x78312A1F, 0xBB7AEE8C, 0x776DB5C1, 0x94D1022B, 0xEED3B6F9};
 UINT16_T i;

 // Clean out Slot 0
 for (i=0; i < sizeof (DynamicState); i++)			// Clean it out
	((UINT8_T *) &pSaveState_h->SlotN)[i] = 0;

 // Clean out Slot 1
 for (i=0; i < sizeof (FixedState); i++)			// Clean it out
	((UINT8_T *) &pSaveState_h->SlotZero)[i] = 0;

 // Insert a magic number
 pSaveState_h->SlotN.MagicNo = MAGICID;

 // Purge the Key Slot
 Retval = WtmPurgeKey (WTM_SLOT_1_ID, 0);
 if (Retval != NoError)
 	return(WTMStateError);

 // Create 5 words of RNG Seed
 for (i=0; i<10; i+=2)		 // Loop 2 bytes at a a time
#if NDT
 	 ((UINT16_T *)(pSaveState_h->SlotN.RNGSeed))[i] = rand();
#else
 	 ((UINT16_T *)(pSaveState_h->SlotN.RNGSeed))[i] = (UINT16_T *)(DummyRNGSeed)[i];
#endif

 // RNG Load Seed
 Retval = WtmRNGLoadSeed (pSaveState_h->SlotN.RNGSeed);
 if (Retval != NoError)
 	return(WTMStateError);

 // Create the Subkey
 Retval = WtmCreateSubkey (WTM_SLOT_1_ID, WTM_KEY_ID, WTM_KEY_RESTRICT_PRIV, Subkey_Pass, (UINT_T *) &pSaveState_h->SlotZero.WTMStateSubKey);
 if (Retval != NoError)
 	return(WTMStateError);

 // Create the Save State
 Retval = WtmSaveState (WTM_SLOT_1_ID, (UINT_T *) &pSaveState_h->SlotN.WTMStateCipher);
 if (Retval != NoError)
 	return(WTMStateError);

 //	Put in Success code
 pSaveState_h->SlotN.WTPSPStatus = WTM_NEW_DEVICE;

 return Retval;
}

UINT_T ValidateTIM(pTIM pTIM_h)
{
	UINT_T Retval = NoError;

	// Digital Signature
	UINT_T *DigitalSignatureEncrypted;
	UINT_T DigitalSignatureDecrypted[KeyLength_64];		// Allow for maximum possible size
	UINT_T ModulusExponentPacked[MAXKEYSIZEWORDS * 2];
	UINT_T FastSignatureVerifyOutput[6], i;

	// Key Lengths
    UINT16_T KeyLength;					// Key Length in words
	FastSignatureVerify FSV_h;

  	#if DKBTEST
     UINT_T DummyRNGSeed[]    = {0xF07DE2F1, 0x78312A1F, 0xBB7AEE8C, 0x776DB5C1, 0x94D1022B, 0xEED3B6F9};
     UINT_T DummyCaddoState[] = {0x3DA4931D, 0x3D7F72C8};
	#endif


  	#if DKBTEST
     WtmReset(0);
     WtmInit (0, DummyCaddoState, DummyRNGSeed, WTM_SLOT_1_ID);
 	#endif

	// Set KeyLength
	KeyLength = ((pTIM_h->pTBTIM_DS->ModulusSize) / 4);  // In number of words
	if (pTIM_h->pTBTIM_DS->DSAlgorithmID == Intel_DS)
	{
		// Decrypt Digital Signature inside TIM
		DigitalSignatureEncrypted = (UINT_T *) pTIM_h->pTBTIM_DS->RSADigS;

		Retval = WtmRSAEncrypt_Int(DigitalSignatureEncrypted,
							  pTIM_h->pTBTIM_DS->RSAModulus,
							  KeyLength,
							  &(pTIM_h->pTBTIM_DS->RSAPublicExponent[0]),
							  1,
							  DigitalSignatureDecrypted);
		CheckErrorReturn(Retval);

		Retval = ValidateImage(TIMIDENTIFIER, DigitalSignatureDecrypted, pTIM_h);
	}
	else   // Validate the TIM only using FastSignaturePI.
	{
	 // Key prep - Fill in the Exponent
	 for (i = 0; i < KeyLength; i++)
		ModulusExponentPacked[i] = pTIM_h->pTBTIM_DS->RSAPublicExponent[i];
	 // Fill in the modules
	 for (i = 0; i < KeyLength; i++)
		ModulusExponentPacked[(KeyLength+i)] = pTIM_h->pTBTIM_DS->RSAModulus[i];

	 FSV_h.Originator = SW_NoVerify_NoBindExport;
	 FSV_h.KeyLength = KeyLength;
	 FSV_h.ByteSwap = No_Swap;
	 FSV_h.PaddingType = PKCS_v15Pad_LittleEndian;
	 FSV_h.ImageAndSHAByteLength[0] = pTIM_h->pImg[0].ImageSizeToHash;  // Only deal with the covered part of the TIM
	 FSV_h.pImageAddress[0] = (UINT_T *)pTIM_h->pConsTIM;
	 FSV_h.pImageSHAAddress[0] = 0;
	 for (i=1; i<7; i++)
	 {
	 	FSV_h.ImageAndSHAByteLength[i] = 0;
		FSV_h.pImageSHAAddress[i] = NULL;
		FSV_h.pImageAddress[i] = NULL;
	 }
	 FSV_h.pDigitalSignature = pTIM_h->pTBTIM_DS->RSADigS;
	 FSV_h.pProvisionedKeyDigest = pTIM_h->pTBTIM_DS->Hash;
	 FSV_h.pRSAPublicKey = ModulusExponentPacked;
	 FSV_h.pOutput = FastSignatureVerifyOutput;
	 Retval = WTMFastSignatureVerify(&FSV_h);
	}
	return Retval;
}

UINT_T FinalizeSecurity (pTIM pTIM_h, UINT_T PlatformSettings, UINT8_T PlatformInitialized)
{
	// Declerations
	UINT_T Retval = NoError;
	SaveState SaveState_h;				// Save State
	
	// Generate WTM State Data
 	AddMessage((UINT8_T*) ("*** Generate WTM State..."));
	Retval = GenerateWTMStateData(&SaveState_h);

	// Device Key
	#if DEVICE_KEY
	if ((Retval == NoError) && (!PlatformInitialized))
	{
 	 AddMessage((UINT8_T*) ("*** Device Key..."));
	 Retval = ProgramOBMImage(pTIM_h);
	}
	#endif

	// Program Platform configuration Fuses
	#if PERFORM_FUSE_PROGRAM
	if (!PlatformInitialized)
	{
 	 AddMessage((UINT8_T*) ("*** Program Configuration Fuses..."));
	 Retval = ProgramPlatformFuses( PlatformSettings );
	}
	#endif

	// Disable WTM
	EnableDisableCaddo(DISABLE_CADDO); 	  // Disable the Caddo Unit

	// Save the error code
 	SaveState_h.SlotN.DKBOBMStatus = Retval;
	if(Retval != NoError)
		AddMessageError((UINT8_T) (0xFF & Retval));

	#if COPYIMAGESTOFLASH
 	Retval = Configure_Flashes(	(UINT8_T) (pTIM_h->pConsTIM->FlashInfo.WTMFlashSign & 0xFF), SAVE_STATE_FLASH);
 	// Write State to Primary WTM block
	if (Retval == NoError)
	 	Retval = WriteWTMStateToFlash ( pTIM_h->pConsTIM->FlashInfo.WTMEntryAddr, &SaveState_h );
	// Write State to Secondary WTM block
	if (Retval == NoError)
	 	Retval = WriteWTMStateToFlash ( pTIM_h->pConsTIM->FlashInfo.WTMEntryAddrBack, &SaveState_h );
	#endif

	return Retval;
}

/* WriteWTMStateToFlash
 *	This function write the fixed state and the initial dynamic Save State to flash
 */
UINT_T WriteWTMStateToFlash ( UINT_T blk_base, pSaveState pSS )
{
	UINT_T Retval = NoError;

	//Write the Fixed state to the base address
	Retval = WriteFlash(blk_base, (UINT_T) &pSS->SlotZero, SLOTZEROSIZE, SAVE_STATE_FLASH);

	//If there are no errors, write the dynamic state to the first available slot
	if (Retval == NoError)
		Retval = WriteFlash(blk_base + SLOTZEROSIZE, (UINT_T) &pSS->SlotN, SLOTNSIZE, SAVE_STATE_FLASH);

	return Retval;
}


// This routine will program TIM verification keys to Caddo via Bind and program OTP registers for XIP.
// JTAG verification keys are also programmed to OTP.
UINT_T ProgramOBMImage(pTIM pTIM_h)
{
	UINT_T   	i, Regval;
	UINT_T		Retval = NoError;
	UINT_T   	ProtectionRegNum;
	UINT_T   	ProgramStatus;
	UINT16_T 	Lock0, Lock1;
	UINT16_T 	ProtectionDataRead[26];
	UINT_T 		WordData;
	USHORT		KeyIndex;
	UINT_T		JtagKeyFound = FALSE;
	unsigned char Msg[9];
 	UINT_T MsgInt;

	// Used for OEM Verfication Key
	static __attribute__((aligned(32))) UINT_T OEMKeyData[BINDKEYSIZE]; 				// Contains key to hash	128 words maximum

	// Hashed KeyData
	UINT_T KeyDataC[WordLengthOf_SHA1 * 2];			// Contains 10 words, 5 for OEM, 5 for JTAG
	UINT_T *OEMKeyDataC, *JTAGKeyDataC;
	// Used for JTAG Verfication Key
	UINT_T 	JTAGKeyData[BINDKEYSIZE];		 		// Used for JTAG

	// Key Lengths
    UINT_T DS_KeyLength;			// Key Length in words
    UINT_T JTAG_KeyLength;		// Key Length in words

	/**************** Calculate Hash for the OEM Verification Keys ********/
	// Extract the KeyLength value from TIM Digital Signature in ISRAM in words
	DS_KeyLength    = ((pTIM_h->pTBTIM_DS->ModulusSize) / 4);

	// Fill in the Exponent
	for (i = 0; i < DS_KeyLength; i++)
		OEMKeyData[i] = pTIM_h->pTBTIM_DS->RSAPublicExponent[i];

	// Fill in the modulus
	for (i = 0; i < DS_KeyLength; i++)
		OEMKeyData[(DS_KeyLength+i)] = pTIM_h->pTBTIM_DS->RSAModulus[i];

	// Call Platform Bind, the output of which will be placed in OTP registers
	OEMKeyDataC = KeyDataC;
	Retval = WtmPlatform_Bind (OEMKeyData, (pTIM_h->pTBTIM_DS->ModulusSize), OEMKeyDataC, 0x2a8, 0x2a8, 0x1530);
	if( Retval == NoError)
	 	 AddMessage((UINT8_T*) ("*** Bind Succesful...\0"));
	else
	{
	 	AddMessage((UINT8_T*) ("*** Bind Failed...\0"));
		AddMessage((UINT8_T*) ("***  Retval :\0"));
		MsgInt = Retval;
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
	}
	AddMessage("Calculated Hash from Bind:\0");
	for (i = 0; i < 5; i++){
		MsgInt = KeyDataC[i];
		ConvertIntToBuf8(Msg, MsgInt, 8,16);
		Msg[8] = NULL;
		AddMessage(Msg);
	}
//disabling OTP writes - mdb
#if 0 
	/**************** Calculate Hash for the JTAG *********************/
	// Find the JTAG Key
	for (KeyIndex = 0; KeyIndex < pTIM_h->pConsTIM->NumKeys; KeyIndex++)
	{
		if (pTIM_h->pKey[KeyIndex].KeyID ==  JTAGIDENTIFIER)
		{
			JtagKeyFound = TRUE;
			break;
		}
  	}
    if (JtagKeyFound == TRUE)
    {
		// Extract the KeyLength value from TIM Digital Signature in ISRAM in words
		JTAG_KeyLength = ((pTIM_h->pKey[KeyIndex].ModulusSize) / 4);

		// Fill in the Exponent
		for (i = 0; i < JTAG_KeyLength; i++)
			JTAGKeyData[i] = pTIM_h->pKey[KeyIndex].RSAPublicExponent[i];

		// Fill in the modules
		for (i = 0; i < JTAG_KeyLength; i++)
			JTAGKeyData[(JTAG_KeyLength+i)] = pTIM_h->pKey[KeyIndex].RSAModulus[i];

		// Use WTM to do the Hash
		JTAGKeyDataC = (UINT_T *) &KeyDataC[WordLengthOf_SHA1];
		Retval = WtmSHA1MessageDigest((unsigned char *)JTAGKeyData, (4*2*JTAG_KeyLength), (unsigned char *)JTAGKeyDataC);
	 }
    /************ Program the OTP's *********************/
#if PERFORM_OTP_LOCKING
 	switch( pFlashP->FlashType )
 	{
     case TYAX_FLASH:
     case SIBLEY_FLASH:
	 {
	  // Check that protection register is locked
	  Retval = ReadProtectionLockRegister(pFlashP, &Lock0, &Lock1);
	  CheckErrorReturn(Retval);

	  // Read protection data
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister0, &ProtectionDataRead[0]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister1, &ProtectionDataRead[4]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister2, &ProtectionDataRead[12]);
	  CheckErrorReturn(Retval);

	  // Program protection data
	  Retval = ProgramProtectionReg(pFlashP,  ProtectionRegister0, (UINT16_T *)&OEMKeyDataC[0], 4);
	  Retval = ProgramProtectionReg(pFlashP,  ProtectionRegister1, (UINT16_T *)&OEMKeyDataC[2], 6);
	  CheckErrorReturn(Retval);
	  if (JtagKeyFound == TRUE)
      {
	    // Now program JTAG key hash into registers 1 and 2
	    WordData = (0x0000FFFF & (JTAGKeyDataC[0]));
	    Retval = ProgramProtectionRegisterWord(pFlashP,  ProtectionRegister1, WordData, 6 );
	    WordData = (0x0000FFFF & ((JTAGKeyDataC[0])>>16));
	    Retval = ProgramProtectionRegisterWord(pFlashP,  ProtectionRegister1, WordData, 7 );
	    Retval = ProgramProtectionReg(pFlashP,  ProtectionRegister2, (UINT16_T *)&JTAGKeyDataC[1], 8);
  	    CheckErrorReturn(Retval);
	  }
 	  #if PERFORM_OTP_LOCKING
		// Lock the protection registers 0 and 1
		Retval = LockProtectionRegister(pFlashP, ProtectionRegister0);
		Retval = LockProtectionRegister(pFlashP, ProtectionRegister1);
		Retval = LockProtectionRegister(pFlashP, ProtectionRegister2);

	   CheckErrorReturn(Retval);
	  #endif // PERFORM_OTP_LOCKING

	  // Check that protection register is locked
	  Retval = ReadProtectionLockRegister(pFlashP, &Lock0, &Lock1);
	  CheckErrorReturn(Retval);

	  // Read back data
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister0, &ProtectionDataRead[0]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister1, &ProtectionDataRead[4]);
	  Retval = ReadProtectionRegisterData(pFlashP, ProtectionRegister2, &ProtectionDataRead[12]);
	  return Retval;
	 }

#if ONENAND_CODE
     case OneNAND:
     {
	  // TBD: Insert WriteOtpArea function for the new One NAND driver when ready.
	  // Write the Platform Keys 160 * 2 bits
/*	  if (JtagKeyFound == TRUE)
        Retval = ONENAND_WriteOtpArea(0, (UINT8_T *) KeyDataC, WordLengthOf_SHA1*4*2);
	  else
  	    Retval = ONENAND_WriteOtpArea(0, (UINT8_T *) KeyDataC, WordLengthOf_SHA1*4);

	  CheckErrorReturn(Retval);
 	  #if PERFORM_OTP_LOCKING
	   Retval = ONENAND_PermanentProtectOtpArea();
 	  #endif
	  return Retval;*/
	  break;
	 }
#endif
#if MDOC_CODE
	 case MSYS_DOC_FLASH:  // Write out to MSYS MDOC
	 {
	  // Write the Platform Keys 160 bits
	  if (JtagKeyFound == TRUE)
	    Retval = mDOC_WriteOTPArea((void *) KeyDataC, WordLengthOf_SHA1*4*2);
	  else 
 	    Retval = mDOC_WriteOTPArea((void *) KeyDataC, WordLengthOf_SHA1*4);
	  return Retval;
	 }
#endif
	 default:
	  break;
	}
#endif
#endif
  return NoError;
}


