;/******************************************************************************
;**	(C) Copyright 2007 Marvell International Ltd.  
;**	All Rights Reserved
;******************************************************************************/

;******************************************************************************
;**
;**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
;**
;**  This software as well as the software described in it is furnished under
;**  license and may only be used or copied in accordance with the terms of the
;**  license. The information in this file is furnished for informational use
;**  only, is subject to change without notice, and should not be construed as
;**  a commitment by Intel Corporation. Intel Corporation assumes no
;**  responsibility or liability for any errors or inaccuracies that may appear
;**  in this document or any software that may be provided in association with
;**  this document.
;**  Except as permitted by such license, no part of this document may be
;**  reproduced, stored in a retrieval system, or transmitted in any form or by
;**  any means without the express written consent of Intel Corporation.
;**
;**  FILENAME:	DKB_StartUp.s
;**
;**  PURPOSE: 	StartUp code for Monahans variant DKB code
;**
;******************************************************************************

   GET dkb_startup._S

   INCLUDE platform_defs.INC
   INCLUDE xlli_platform_defs.INC

   AREA    Init, CODE, READONLY

   ; External Functions
   EXTERN DKBBootMain
   EXTERN UndefinedHandler
   EXTERN SwiHandler
   EXTERN PrefetchHandler
   EXTERN AbortHandler
   EXTERN IrqHandler
   EXTERN FiqHandler


   EXPORT TransferControl
   EXPORT EnableIrqInterrupts
   EXPORT DisableIrqInterrupts
   EXPORT __main
   EXPORT borqs_package_lost

   IMPORT DDRInit


;************************************************************
;* DKB/OBM entry point										*
;************************************************************
__main
    ENTRY
    B ResetHandler
    B UndefinedHandler
    B SwiHandler
    B PrefetchHandler
    B AbortHandler
    NOP    	    			; Reserved Vector
    B SaveProgramState 		; B IRQ Interrupt SaveProgramState
    B FiqHandler ; FIQ interrupts not anabled
	NOP
	DCD	MajorMinor, Date, Processor
	NOP
	; RESET entry point



;************************************************************
;* Initialize INT. Cntl. Mask all Ints, and set them to IRQ
;************************************************************
ResetHandler
	LDR		r0, =InterruptsBase
	MOV		r1, #0
	STR		r1, [r0, #IntsIcmr]
	STR		r1, [r0, #IntsIclr]

;*************************************************************************
; Save off platform settings in r10 to memory until we transfer to C-code
;*************************************************************************
	LDR     r1, = ISRAMDateSpaceBase
	STR		r10, [r1]
	SUB		r10, pc, #84	;;;; !!!!! THIS IS REQUIRED WHEN COPYING TO DDR.  THIS MUST BE UPDATED IF ADDING CODE BEFORE THIS LINE!!!!!!
    STR		r10, [r1, #4]

; turn off the MMU when using the MMU testing code
	ldr     r1, =0x00000078         ; zero for off ([6:3] must be ones)
   	mcr     p15, 0, r1, c1, c0, 0   ; turn off MMU, I/D caches, write buffer
	MOV 	r1, r1        		; CPWAIT
 	SUB 	pc, pc, #4

; Flush TLB's
   	mov      r1, #0xFFFFFFFF
   	mcr      p15, 0, r1, c3, c0, 0   ; set up access to domain 0
   	mcr      p15, 0, r2, c8, c7, 0   ; flush I + D TLBs
   	mcr      p15, 0, r1, c7, c10, 4  ; drain write and fill buffers
   	MOV 	r2, r2        		; CPWAIT
 	SUB 	pc, pc, #4    		; CPWAIT

;************************************************************
;* Assign SP locations for all Modes of operation 			*
;* Reserve space for image that is downloaded by reducing   *
;* the size of internal RAM
;************************************************************
	LDR		r0, =BV_SRAM
	LDR 	r1, =BV_SRAMSIZE

   	MOV    	r2, r0		 ; Get memory for stack pointer
   	ADD    	r2, r2, r1	 ; Go length of that memory
  	SUB    	r2, r2, #4   ; Back off by one word

; Enter IRQ mode and set up the IRQ stack pointer
    MOV     R0, #Mode_IRQ          	; IRQ mode
    MSR     CPSR_c, r0
	; Initialize register r8-r14 in this mode with zeros
    MOV		r1, #0
    MOV		r8, r1
    MOV		r9, r1
    MOV		r10, r1
    MOV		r11, r1
    MOV		r12, r1
    MOV		r13, r1
    MOV		r14, r1
    MOV     SP, r2                 	; Assign stack pointer for IRQ handler

; Enter SVC mode and set up the SVC stack pointer
    MOV     r0, #Mode_SVC          	; SVC mode, No interrupts
    MSR     CPSR_c, r0
	; Initialize register r8-r14 in this mode with zeros
    MOV		r1, #0
    MOV		r8, r1
    MOV		r9, r1
    MOV		r10, r1
    MOV		r11, r1
    MOV		r12, r1
    MOV		r13, r1
    MOV		r14, r1
    SUB     r2, r2, #IrqStackSize
    MOV     SP, r2                 	; Assign stack pointer for SVC handler

ContinueBoot

;***********************************************************************************
;  Enable all coprocessor registers
;***********************************************************************************

   ORR		r0, r0, #0x3f00
   MCR 		p15, 0, r0, c15, c1, 0
   MOV 		r0, r0
   MOV 		r0, r0
   MOV 		r0, r0
   MOV 		r0, r0

;***********************************************************************************
;  Initialize the DDR
;
; Only Initialize DDR if it hasn't already been initialized by the TBR
; if we are running in DDR space, then we need to reload the whole OS
;***********************************************************************************

  mov r1, pc
  ldr r0, =0x80000000
  andals  r0, r0, r1

  bne	__ENABLE_MMU				; Go right to MMU enable and OBMBootMain

  bl DDRInit

;***********************************************************************************
;  Move DKB/OBM to DDR for more space
;
;***********************************************************************************

__CopyDKBOBM
        mov     r5,     #5              	; Will try to copy DKB up to 5 times
		ldr		r2, 	=ISRAMMAXROOM

bc_copy ldr     r1,     = ISRAMDateSpaceBase   		; get the stored address of the beginning of this OBM
        ldr     r0, [r1, #4]
        mov     r4,     r0                  ; Save off a copy in r4
        ldr     r1,     =0x80000000   		; Destination address in DDR

;
;       Copy the DKB code into the destination (load) address (iSRAM)
;
bc_cd   ldr     r3,     [r0],   #4      	; Get data in source address
        str     r3,     [r1],   #4      	; Place data in destination address
        subs    r2,     r2,     #1      	; Decrement doubleword count
        bne     bc_cd                   	; Copy data until it is all moved
;
;       check 48KB of it to verify that the copy is good
;
        ldr     r1,     =0x80000000	    ; Destination address in remapped DDR
        ldr		r2, 	=ISRAMMAXROOM       ; copy 48Kb (12K 32-bit words)
bc_chk  ldr     r0,     [r4],   #4      	; Get source data
        ldr     r3,     [r1],   #4      	; Get destination data
        cmp     r0,     r3              	; Do they match?
        bne     bc_chk_err              	; No - branch to "boot code check error"
        subs    r2,     r2,     #1      	; Yes - more to verify?
        bne     bc_chk                  	; Yes - Check next address
        b       __ENABLE_MMU            	; No - Update PC to enable MMU
;
;       Go here if the copy is no good.
;
bc_chk_err
        subs    r5,     r5,     #1      	; Decrement copy count - try to copy again?
        bne     bc_copy                 	; Yes - start copy from scratch (again!)
bc_loop nop                             	; No - failed too many times - hang it up!
        nop
        nop
        b       bc_loop         			; Loop forever - "hangs" the system.


;***********************************************************************************
;  Enable MMU
;
;***********************************************************************************

__ENABLE_MMU
   INCLUDE mmuenable.INC


;*********************************************************************************
; Initialize memory required by C code
; BAA: problem is in this part of the start.s. It has to do with copying of
; initialized data from ROM to RAM.
;
; There is a solution - mind you not documented currently - and that is to use the
; linker option -INIT DATA which would pack all initialized data and store it in
; an init_data_segment which will be located in the code section
; Furthermore there is a library routine [__inixsc_init] which will unpack the data
; and store it RAM on a given address.
; I.e. the tools can take care of your problem and the usage is illustrated below:
; for this you need an update from the tools drop NDT0.6
;**********************************************************************************

__JUMP_TO_C_CODE					; After the Jump, we will run out of DDR!

 IF :DEF: USE_NDT_INIT
	; we assume stack is set up
	IMPORT __inixsc_init

	MOV		r2, #0
	LDR		r1, =Base0				; pointer to ZERO addresses in .text
	LDR		r0, IniBase 				; address to packed data
	BL		__inixsc_init
 ELSE
    /*
     *  Clear .bss section
     */
	adr r0, LC0
	ldmia r0, {r1, r2}

	mov r0, #0
clear_word
	str r0, [r1], #4
	str r0, [r1], #4
	str r0, [r1], #4
	str r0, [r1], #4
	cmp r1, r2
	blo clear_word
 ENDIF
;************************************************************
;* Branch to OEM Boot Main program							*
;************************************************************
	LDR     r1, =ISRAMDateSpaceBase			; Restore platform configuration passed in by the BootROM
	LDR		r0, [r1]
	B	 	DKBBootMain
LOOPFOREVER  B		LOOPFOREVER				; forever loop

   IF :DEF: USE_NDT_INIT
	IMPORT _sec_init_data_beg_
IniBase DCW _sec_init_data_beg_
Base0 DCW 0,0,0			;;; base to each section  ;;;;0 (not used), dif between linked & load address of code;; ditto data

  ELSE

LC0
	.word __bss_start
	.word __bss_end
  ENDIF
borqs_package_lost
    .word 0
;***********************************************************************************
;  1st step - Move Transfer Control to a Non-MMU translated area of the DDR
;
;  r0 = Boot Image Load Address
;  r1 = Boot Image Size
;  r2 = Boot Image Current Address
;  r3 = Fuse Value
;
;***********************************************************************************
TransferControl
		ldr		r4, =0xFFFFFFF8
		and		r7, r0, r4			; Save the boot image load address in r7 (make sure it is word aligned)
		mov		r8, 	r1			; Save the boot image size in r8.
		and		r9,	r2, r4			; Save the boot image current address in r9 (make it word aligned)
		mov		r10, 	r3			; Save off platform settings to pass to the OBM in r10.

		ldr     r4, 	=__TransferControlCode
        ldr     r1,     =0x5c009000   		; Destination address in DDR
        mov     r2,     #0x100             ; copy 1Kb (32K 32-bit words)
;
;       Copy the DKB code into the destination (load) address (iSRAM)
;
bc_cd1  ldr     r3,     [r4],   #4      	; Get data in source address
        str     r3,     [r1],   #4      	; Place data in destination address
        subs    r2,     r2,     #1      	; Decrement doubleword count
        bne     bc_cd1                   	; Copy data until it is all moved
        ldr     r1,     =0x5c009000   		; Destination address in DDR
		MOV  	pc, 	r1					; Jump to 1:1 Mapped portion of DDR area

;************************************************************
;* 2nd Step - Turn OFF MMU, Flush TLB's
;************************************************************
__TransferControlCode
	; turn off the MMU when using the MMU testing code
	ldr     r1, =0x00000078         ; zero for off ([6:3] must be ones)
   	mcr     p15, 0, r1, c1, c0, 0   ; turn off MMU, I/D caches, write buffer
	CPWAIT   r1
  	CPWAIT   r1

   ; Flush TLB's
   	mov      r1, #0xFFFFFFFF
   	mcr      p15, 0, r1, c3, c0, 0   ; set up access to domain 0
   	mcr      p15, 0, r2, c8, c7, 0   ; flush I + D TLBs
   	mcr      p15, 0, r1, c7, c10, 4  ; drain write and fill buffers
   	CPWAIT   r2
   	CPWAIT   r2

;************************************************************
;* 3nd Step - Copy the boot image to it's load address
;************************************************************
	cmp		r7,	r9				; compare the load address (LA) to current address (CA) of image
	beq		__JumpToBootImage	;  if they are the same, we don't need to move any data
	bls		__CopyFromStart		;  if (LA < CA) CopyFromStart
	add		r1,	r9,	r8			;
	cmp		r7,	r1				;  if (LA > CA + Size) CopyFromStart
	bhi		__CopyFromStart		;
								;  else CopyFromEnd
__CopyFromEnd					;  __CopyFromEnd : copies images starting at end
	sub		r8, r8, #4			; decrement size by 1 word
	ldr     r3, [r9, r8]      	; Get data in current address
	str     r3, [r7, r8]     	; Place data in load address
	cmp		r8, #0
	bgt		__CopyFromEnd		; continue copying until size is less than 0
	b		__JumpToBootImage

__CopyFromStart					;  __CopyFromStart: copies images starting at beginning
	ldr     r3, [r9], #4      	; Get data in current address
	str     r3, [r7], #4      	; Place data in load address
	sub		r8, r8, #4			; decrement size by 1 word
	cmp		r8, #0
	bgt		__CopyFromStart		; continue copying until size is less than 0
	b		__JumpToBootImage

__JumpToBootImage
	MOV  	pc, r0				; Jump next desired code after DKB (ie. OBM, OS Loader)

;************************************************************
; IRQ Vector Handler - Save Program State
;************************************************************

SaveProgramState
	SUB	R14, R14, #4	; Adjust return address before saving it
	STMFD	SP!, {R14}
	MRS	R14, SPSR
	STMFD   SP!, {R0-R12,R14}

	BL 	IrqHandler

	; Modify CPS register
	;MRS	r12, CPSR
	;ORR    	r12,r12,#0x80    ; Disable IRQ interrupts
	;MSR	CPSR_c, r12
    LDR  r0,borqs_package_lost
    cmp  r0,#0x1000
    bge  restor2_noirq

    LDMFD 	SP!, {R0-R12,R14}
	MSR	SPSR_cxsf, R14
	LDMFD 	SP!, {PC}^

restor2_noirq
;    mov  r0,#0
;    str  r0,borqs_package_lost
    ;b  restor2_noirq 
    LDMFD 	SP!, {R0-R12,R14}
    orr r14,r14,#0x80
	MSR	SPSR_cxsf, R14
	LDMFD 	SP!, {PC}^


;************************************************************
; EnableIrqInterrupts
;************************************************************

EnableIrqInterrupts
	STMFD		SP!, {R0-R4, LR}
   	MRS    	r0,CPSR        ; get the processor status
   	BIC    	r0,r0,#0x80    ; Enable IRQ interrupts
   	MSR    	cpsr_cf,r0     ; SVC 32 mode with interrupts disabled
	LDMFD   	SP!,    {R0-R4, PC}

;************************************************************
; DisableIrqInterrupts
;************************************************************

DisableIrqInterrupts
	STMFD		SP!, {R0-R4, LR}
   	MRS    	r0,CPSR        ; get the processor status
   	ORR    	r0,r0,#0x80    ; Disable IRQ interrupts
   	MSR    	cpsr_cf,r0     ; SVC 32 mode with interrupts disabled
	LDMFD   	SP!,    {R0-R4, PC}

    ;//gliethttp_20080318 add icache dcache off  and both on function
    ALIGN 4
    EXPORT flush_i_d_cache
flush_i_d_cache
    stmfd    sp!, {r0,lr}

  	mov	r0, #0x1f00
	orr	r0, r0, #0x00e0
1	mcr	p15, 0, r0, c7, c14, 2		@ clean/invalidate L1 D line
	adds	r0, r0, #0x40000000
	bcc	1b
	subs r0, r0, #0x20 
	bpl	1b
	mcr	p15, 0, ip, c7, c5, 0		@ invalidate L1 I cache and BTB
	mcr	p15, 0, ip, c7, c10, 4		@ data write barrier
	mcr	p15, 0, ip, c7, c5, 4		@ prefetch flush

    ;mcr      p15, 0, r0, c7, c6, 0
    
    ldmfd    sp!, {r0,pc}
    
    ALIGN 4
    EXPORT disable_i_d_cache
disable_i_d_cache
    stmfd    sp!,{r0-r1,lr}
    ;ldmfd    sp!,{r0-r1,pc}

    bl flush_i_d_cache
    
    mov      r1, #0x78
    orr      r1, r1, #0x0001
    mcr      p15, 0, r1, c1, c0, 0

    mrc      p15, 0, r0, c2, c0, 0
    mov      r0, r0
    sub      pc, pc, #4
    
    ldmfd    sp!,{r0-r1,pc}

    ALIGN 4
    EXPORT enable_i_d_cache
enable_i_d_cache
    stmfd    sp!,{r0-r1,lr}
    ;ldmfd    sp!,{r0-r1,pc}

    mov      r1, #0x78
    ;mrc      p15, 0, r1, c2, c0, 0
    orr      r1, r1, #0x0001
    orr      r1, r1, #0x1000         @ Enable Icache
    orr      r1, r1, #0x0004         @ Enable Dcache
    orr      r1, r1, #0x0800         @ Enable BTB
    mcr      p15, 0, r1, c1, c0, 0

    mrc      p15, 0, r0, c2, c0, 0
    mov      r0, r0
    sub      pc, pc, #4
    
    ldmfd    sp!,{r0-r1,pc}

   END



