/**************************************************************************
**
**  (C) Copyright December 2006 Marvell International Ltd.
**
**  All Rights Reserved.
**
**  This software file (the "File") is distributed by Marvell International  
**  Ltd. under the terms of the GNU General Public License Version 2, June  
**  1991 (the "License").  You may use, redistribute and/or modify this File   
**  in accordance with the terms and conditions of the License, a copy of  
**  which is available along with the File in the license.txt file or by   
**  writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
**  Boston, MA 02111-1307 or on the worldwide web at 
**  http://www.gnu.org/licenses/gpl.txt.
**
**  THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
**  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
**  PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
**  details about this warranty disclaimer. 
**
**  Filename: NTBBLIB.cpp
**
**  This file contains miscellaneous functions for the TIM binary image
**  build tool, TBBTIM.
**
******************************************************************************/

#define NTBBLIB
#define WTPDEBUG 1
#define DBGVERIFY 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <limits.h>
#include "Typedef.h"
#include "Tim.h"
#ifdef TRUSTED
#include "ippCP.h"
#endif
#include "TBBLIB.h"

#pragma warning ( disable : 4096 4081 ) // Disable warning messages

bool ImageBuild (FILE *hImageFile,char *szImageFilename,IMAGE_INFO *pImageInfo)
{
	char szImageOutFilename[MAX_PATH];
	FILE *hImageOutFile;
	BYTE Data[8192];
	int iBlockSize = 8192;

	// This routine simply prepends a 32 bit Image ID to a binary image and
	// saves the resultant off as a new file.

	fseek (hImageFile,0L,SEEK_SET); // Set position to SOF

    {//add by b074_gliethttp_20080314 ,fix ../ ./ path bug
        char *TempName;
        char *input_file_name = szImageFilename;
        char *output_file_name = szImageOutFilename;
        char *p;
        int index;

        p = strrchr(input_file_name,'/');
        if(p == NULL)
        {
            p = input_file_name;
        }else p += 1;
        
        index = p - input_file_name;
        TempName = strtok(p,".");

        if(index)
        {char *binname;
            binname = (char*)malloc(strlen(p)+10);
            strncpy(output_file_name,input_file_name,index);
            output_file_name[index] = 0;
            sprintf(binname,"%s_h.bin",TempName);
            strcat(output_file_name,binname);
            free(binname);
        }else
            {
                sprintf(output_file_name,"%s_h.bin",TempName);
            }
        fprintf(stderr,"TempName=%s\n",TempName);
        fprintf(stderr,"output_file_name=%s\n",output_file_name);
    }

	if ((hImageOutFile = fopen (szImageOutFilename,"wb")) == NULL)
	{
		printf ("\nfopen failed to open file: %s.\n",szImageOutFilename);
		rewind (hImageFile);
		return FALSE;
	}

	// Prepend Image ID to output image
	fwrite ((unsigned int *)&pImageInfo->ImageID,sizeof (UINT_T),1,hImageOutFile);

	// Copy input image to output image
	while (iBlockSize == 8192)
	{
		iBlockSize = fread (Data,1,iBlockSize,hImageFile);
		pImageInfo->ImageSize += iBlockSize;
		fwrite (Data,1,iBlockSize,hImageOutFile);
	}

	if (hImageOutFile != NULL) fclose (hImageOutFile);
	rewind (hImageFile);

	return TRUE;
}

bool GenerateImageCrc (FILE *hBinaryImage,int iIndex,UINT32_T *pImageCrc) 
{
	long lFileSize;
	unsigned char *Buffer;

	// A check sum is generated for the entirity of each image, listed 
	// by the TIM, and stored in its appropriate IMAGE_INFO section.

	fseek (hBinaryImage,0L,SEEK_END); // Set position to EOF
	lFileSize = ftell (hBinaryImage); // Get position of file, thus the file size.
	fseek (hBinaryImage,0L,SEEK_SET); // Set position to SOF

	if (NULL == (Buffer = (unsigned char *)malloc (lFileSize)))
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Memory allocation for Buffer failed in GenerateImageCrc()!\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	if (fread (Buffer,1,lFileSize,hBinaryImage) != (unsigned int)lFileSize)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("UntrustedChecksum couldn't read all of the %ld bytes.\n",lFileSize);
		printf ("Sorry couldn't calculate checksum in GenerateImageCrc()!\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (Buffer != NULL) free (Buffer);
		return FALSE;
	}

	*pImageCrc = CheckSum (Buffer,lFileSize);
	printf ("CRC for non trusted Image: %X\n",*pImageCrc);

	if (Buffer != NULL) free (Buffer);
	return TRUE;
}

bool GenerateTimCrc (FILE *hTimFile)
{
	long lFileSize;
	unsigned char *TimBuffer;
	CTIM TimHeader;
	IMAGE_INFO ImageInfo;
	UINT32_T ImageOffset, NextImageOffset;
	UINT32_T ImageCrc;

	// A check sum of the entire TIM is generated by this function. The value
	// is stored in the first image module of the TIM. Refer to header file
	// Tim.h for the layout of the IMAGE_INFO struct. The check sum will reside
	// in the base position of the Hash array within that struct.

	fseek (hTimFile,0L,SEEK_END); // Set position to EOF
	lFileSize = ftell (hTimFile); // Get position of file, thus the file size.
	fseek (hTimFile,0L,SEEK_SET); // Set position to SOF


	if (NULL == (TimBuffer = (unsigned char *)malloc (lFileSize)))
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Memory allocation for TimBuffer failed in GenerateTimCrc()!\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	if (fread (TimBuffer,1,lFileSize,hTimFile) != (unsigned int)lFileSize)
	{

		printf ("////////////////////////////////////////////////////////////\n");
		printf ("GenerateTimCrc couldn't read all of the %ld bytes.\n",lFileSize);
		printf ("Sorry couldn't calculate checksum in GenerateTimCrc()!\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (TimBuffer != NULL) free (TimBuffer);
		return FALSE;
	}

	ImageCrc = CheckSum (TimBuffer,lFileSize);
	printf ("CRC for non trusted TIM: %X\n\n",ImageCrc);
	if (TimBuffer != NULL) free (TimBuffer);

	fseek (hTimFile,0L,SEEK_SET); // Set position to SOF
	fread (&TimHeader,sizeof(CTIM),1,hTimFile);
	ImageOffset = ftell (hTimFile);

	fread (&ImageInfo,sizeof(IMAGE_INFO),1,hTimFile);
	NextImageOffset = ftell (hTimFile);
	ImageInfo.Hash[0] = ImageCrc;
	fseek (hTimFile,ImageOffset,SEEK_SET); // Set position to Image offset
	fwrite (&ImageInfo,sizeof(IMAGE_INFO),1,hTimFile);

	return TRUE;
}

bool ProcessReservedData (FILE *hKeyFile,FILE *hTimFile,int iSize) 
{
	char fLine[100], *pData = NULL;
	DWORD ImageOffset, dwValue;
	int iTotal = 0;
	const DWORD PackageHeaderID[]={0x44447248, 0x42494E44, // "DDRH", "BIND" 
								  0x5465726D, 0x4750494F, // "Term", "GPIO"
								  0x55415254, 0x00555342, // "UART", "USB"
								  0x5265736D, 0x55534200, // "Resm"
								  0x55534201, 0x55534202, 
								  0x55534203, 0x55534204, 
								  0x55534205, 0x55534206,
								  0x55534207, 0x55534208, 
								  0x55534209};
	DWORD packageSize;
	int dataLength, paddingLength;
	bool isPackageSize = false;

	// This function parses out the data from the "Reserved Data" section
	// of the descriptor text file. Each token will be padded out to 32 bits
	// regardless of the field width of the data.

	if (fscanf (hKeyFile,"Reserved Data:\n") != 0)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Declaration statement is missing or corrupted in descriptor file!\n");
		printf ("Reserved Data:\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	while (fgets (fLine,100,hKeyFile) != NULL)
	{
		if ((iTotal * 4) > iSize)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The number of reserved data bytes read is more than\n");
			printf ("the number indicated by 'Size of Reserved in bytes' directive\n");
			printf ("in the descriptor file. SizeOfReserved = %d\n",iSize);
			printf ("////////////////////////////////////////////////////////////\n");
			return FALSE;
		}
		if (strstr (fLine,":") != NULL)
		{
			fseek (hKeyFile,ImageOffset,SEEK_SET); // Set position to Image offset
			break;
		}

		pData = strtok (fLine," \r\n\t");

		if (pData != NULL)
		{
			dwValue = Translate (pData); 
			iTotal++;
			for (int i = 0; i<17;i++)
			{
				// Check if the data read contains a header ID
				if (dwValue == PackageHeaderID[i])
				{
					//next line will contain the package size
					if (fgets (fLine,100,hKeyFile) != NULL)
					{
						pData = strtok (fLine," \n\t");
						if (pData != NULL)
						{
							packageSize = Translate (pData); 
							iTotal++;
							isPackageSize = true;
						}
					}
				}
			}
			if ( (packageSize % 4) == 0 )
			{
				fwrite (&dwValue,sizeof (DWORD),1,hTimFile);
				if (isPackageSize)
				{
					fwrite (&packageSize, sizeof(DWORD),1, hTimFile);
					isPackageSize = false;
				}
			}
			else
			{
				dataLength = strlen (pData)- 2;
				if (dataLength < 8)
					paddingLength = 8 - dataLength;
				if (paddingLength == 1)
					dwValue = dwValue | 0xF;
				else if (paddingLength == 2)
					dwValue = dwValue | 0xFF;
				else if (paddingLength == 3)
					dwValue = dwValue | 0xFFF;
				else if (paddingLength == 4)
					dwValue = dwValue | 0xFFFF;
				else if (paddingLength == 5)
					dwValue = dwValue | 0xFFFFF;
				else if (paddingLength == 6)
					dwValue = dwValue | 0xFFFFFF;
				else if (paddingLength == 7)
					dwValue = dwValue | 0xFFFFFFF;
				fwrite (&dwValue,sizeof (DWORD),1,hTimFile);
				if (isPackageSize)
				{
					fwrite (&packageSize, sizeof(DWORD),1, hTimFile);
					isPackageSize = false;
				}
			}

//			while ((pData = strtok (NULL," \n\t")) != NULL)
//			{
//				dwValue = Translate (pData); iTotal++;
//				fwrite (&dwValue,sizeof (DWORD),1,hTimFile);
//			}
		}
		ImageOffset = ftell (hKeyFile);
	}

	if ((iTotal * 4) != iSize)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("The number of reserved data bytes read, %d, does not equal\n",iTotal * 4);
		printf ("the number indicated by 'Size of Reserved in bytes' directive\n");
		printf ("in the descriptor file. SizeOfReserved = %d\n",iSize);
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	return TRUE;
/*	char fLine[100], *pData = NULL;
	const DWORD PackageHeaderID[]={0x44447248, 0x42494E44, // "DDRH", "BIND" 
								  0x5465726D, 0x4750494F, // "Term", "GPIO"
								  0x55415254, 0x00555342, // "UART", "USB"
								  0x5265736D, 0x55534200, // "Resm"
								  0x55534201, 0x55534202, 
								  0x55534203, 0x55534204, 
								  0x55534205, 0x55534206,
								  0x55534207, 0x55534208, 
								  0x55534209};
	DWORD ImageOffset, dwValue;
	int iTotal = 0;

	// This function parses out the data from the "Reserved Data" section
	// of the descriptor text file. Each token will be padded out to 32 bits
	// regardless of the field width of the data.

	if (fscanf (hKeyFile,"Reserved Data:\n") != 0)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Declaration statement is missing or corrupted in descriptor file!\n");
		printf ("Reserved Data:\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}
	bool HeaderFound = false;
	DWORD PackageSizeInBytes = 0;
	int PackageDataCount = 0;
	int byteSize =0;
	bool paddingRequired = false;
	BYTE defaultPadding = 0xFF;
	while (fgets (fLine,100,hKeyFile) != NULL)
	{
		if ((iTotal * 4) > iSize)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The number of reserved data bytes read is more than\n");
			printf ("the number indicated by 'Size of Reserved in bytes' directive\n");
			printf ("in the descriptor file. SizeOfReserved = %d\n",iSize);
			printf ("////////////////////////////////////////////////////////////\n");
			return FALSE;
		}
		if (strstr (fLine,":") != NULL)
		{
			fseek (hKeyFile,ImageOffset,SEEK_SET); // Set position to Image offset
			break;
		}

		pData = strtok (fLine," \n\t");

		if (pData != NULL)
		{
			dwValue = Translate (pData); 
			iTotal++;
			for (int i = 0; i<17;i++)
			{
				// Check if the data read contains a header ID
				if (dwValue == PackageHeaderID[i])
				{
					HeaderFound = true;
					byteSize+=4;
				}
			}
			if (HeaderFound == true)
				PackageDataCount++;
			// Header ID is followed by package size data
			if ( PackageDataCount == 2 )
			{
				PackageSizeInBytes = dwValue;
				byteSize+=4;
				 
			}
			else if (PackageDataCount > 2)
			{
				if ( (PackageSizeInBytes % 4) != 0 )
				{
					if (dwValue <= 0xFF)
						byteSize +=1;
					else if (dwValue>0xFF && dwValue <= 0xFFFF)
						byteSize +=2;
					else if (dwValue>0xFFFF && dwValue <= 0xFFFFFF)
						byteSize +=3;
					else if (dwValue>0xFFFFFF && dwValue <= 0xFFFFFFFF)
						byteSize +=4;
				}
				else
				{
					byteSize+=4;
				}
			}

			if ((PackageSizeInBytes % 4) ==0 || PackageDataCount ==2 )
			{
				fwrite (&dwValue,sizeof (DWORD),1,hTimFile);
			}
		
			if (byteSize == PackageSizeInBytes && byteSize != 0 )
			{

				if ((PackageSizeInBytes % 4) !=0 )
				{

					if ( ((4-(PackageSizeInBytes % 4))== 1) )
					{
						dwValue = dwValue | 0xFF000000;
					}
					else if ( ((4-(PackageSizeInBytes % 4))== 2) )
					{
						dwValue = dwValue | 0xFFFF0000;
					}
					else if ( ((4-(PackageSizeInBytes % 4))== 3) )
					{
						dwValue = dwValue | 0xFFFFFF00;
					}
					fwrite (&dwValue,sizeof (DWORD),1,hTimFile);
				}
				PackageSizeInBytes = 0;
				PackageDataCount = 0;
				byteSize =0;
				paddingRequired = false;
				HeaderFound = false;
			}
			while ((pData = strtok (NULL," \n\t")) != NULL)
			{
				dwValue = Translate (pData); iTotal++;
				fwrite (&dwValue,sizeof (DWORD),1,hTimFile);
			}
		}
		ImageOffset = ftell (hKeyFile);
	}

	if ((iTotal * 4) != iSize)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("The number of reserved data bytes read, %d, does not equal\n",iTotal * 4);
		printf ("the number indicated by 'Size of Reserved in bytes' directive\n");
		printf ("in the descriptor file. SizeOfReserved = %d\n",iSize);
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	return TRUE;
*/
}

bool VerifyReservedData (FILE *hKeyFile)
{
	char fLine[100],*pData = NULL;
	DWORD ImageOffset, dwValue; 
	int numOfHeaders;
	int iTotal = 0, headerCount = 0;
	bool termIdHeader = FALSE;

	while (fgets (fLine,100,hKeyFile) != NULL)
	{
  
		if (strstr (fLine,"Reserved Data") != NULL)
		{
			while (fgets (fLine,100,hKeyFile) != NULL)
			{

				pData = strtok (fLine," \n\t");

				if (pData != NULL)
				{
					dwValue = Translate (pData); 
					iTotal++;

					// The first field in reserved data has to be WTPRESERVEDAREAID
					if ((iTotal == 1) && (dwValue != WTPRESERVEDAREAID) )
					{
						printf ("////////////////////////////////////////////////////////////\n");
						printf ("Reserved area indicator is missing or is corrupted in descriptor file!\n");
						printf ("////////////////////////////////////////////////////////////\n");
						
						return FALSE;
					}
					else if ( iTotal == 2 )
					{
						numOfHeaders = (int)dwValue;
					}
					else if ((dwValue == DDRID)		|| 
							(dwValue == GPIOID) 	|| 
							(dwValue == AUTOBIND)	||
							(dwValue == UARTID) 	|| 
							(dwValue == USBID) 		|| 
							(dwValue == RESUMEID)	||
							(dwValue == 0x55534200)	||
							(dwValue == 0x55534201)	||
							(dwValue == 0x55534202)	||
							(dwValue == 0x55534203)	||
							(dwValue == 0x55534204)	||
							(dwValue == 0x55534205)	||
							(dwValue == 0x55534206)	||
							(dwValue == 0x55534207)	||
							(dwValue == 0x55534208)	||
							(dwValue == 0x55534209))
					{
						headerCount++;
					}
					// ReservedArea has to contain a terminator ID for the new format
					else if (dwValue == TERMINATORID)
					{
						termIdHeader = TRUE;
					}
				}
			}
		}
	}

	rewind (hKeyFile);

	if ( !termIdHeader)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf("Terminator ID is incorrect or is missing.\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	if ( headerCount != numOfHeaders)
	{
#if 0
		printf ("////////////////////////////////////////////////////////////\n");
		printf("Number of Reserved Area Packages does not equal the number of headers\n");
        printf("headerCount = 0x%x,numOfHeaders = 0x%x",headerCount,numOfHeaders);
		printf ("////////////////////////////////////////////////////////////\n");
#endif 
	}
	return TRUE;
}

bool VerifySizeOfTim (CTIM *pTimHeader) 
{
	int iTotalImageSize;

	iTotalImageSize = sizeof (CTIM);
	iTotalImageSize += sizeof (IMAGE_INFO) * pTimHeader->NumImages;
	iTotalImageSize += sizeof (KEY_MOD) * pTimHeader->NumKeys;
	iTotalImageSize += pTimHeader->SizeOfReserved;
	iTotalImageSize += sizeof (PLAT_DS);

	if (iTotalImageSize > TIMBUFFER)
	{
		printf ("The size of the Tim image is larger than the maximum allowable!\n");
		printf ("The current Tim size allocated is set at %d bytes.\n",TIMBUFFER);
		printf ("The current Tim image size is at %d bytes.\n",iTotalImageSize);
		return FALSE;
	}

	return TRUE;
}

unsigned long int Translate (char *pValue)
{
	int i;
	unsigned int result = 0;
	char *end = NULL;

	// This function converts a string to a number.

	// If pValue contains "0x" then treat it as a hexidecimal value
	// otherwise treat it as a regular integer.

	if (strstr (pValue,"0x") == NULL) return (unsigned long int) strtoul(pValue,&end,10);
	else
	{
		for (i = 2; i < 10 && pValue[i] != '\0'; i++)
		{
			result = (result << 4);

			if (pValue[i] >= 'a' && pValue[i] <= 'f') 
				result += (unsigned int)(pValue[i] - 'a' + 0xA);
			else if (pValue[i] >= 'A' && pValue[i] <= 'F') 
				result += (unsigned int)(pValue[i] - 'A' + 0xA);
			else if (pValue[i] >= '0' && pValue[i] <= '9') 
				result += (unsigned int)(pValue[i] - '0');
			else if (pValue[i] == ' ') 
				return (result >> 4);
			else return 0;
		}
	}

	return result;
}

void Endian_Convert (UINT_T x,UINT_T *y)
{
	UINT_T a0, a1, a2, a3;

	a0 = x & 0xff;
	a1 = (x & 0xff00) >> 8;
	a2 = (x & 0xff0000) >> 16;
	a3 = (x & 0xff000000) >> 24;
	*y = (a0 << 24) + (a1 << 16) + (a2 << 8) + a3;
}

UINT32_T CheckSum (unsigned char *Address, UINT_T length)
{
	UINT32_T Key = 0x04C11DB7;
	UINT32_T Val = 0xFFFFFFFF;
	UINT_T i = 0, t = 0;

	// This routine is used when building Non Trusted images where a
	// check sum is generated from an entire image or buffer of data.

	while (i < length)
	{
		if (((( Val >> 31 ) & 0x1 ) ^ ((Address[i] >> t) & 0x1)) == 1)
			Val = ( Val << 1) ^ Key;
		else
			Val = Val << 1;

		if (t == 7)
		{
			t = 0;
			i++;
		}
		else t++;
	}

	return Val;
}

bool VerifyDescriptorIntegrity (FILE *hKeyFile)
{
	char szDword[11], version[11];
	bool IsOk = TRUE;
	char fLine[100];
	char sizeOfReserved[10];
    char tmp_outfilename[256];
	// This function is the main entry point for verifying integrity of the
	// descriptor text file before any work is done. It is meant to be a quick
	// sanity check.

	printf ("\nVerifying Descriptor Integrity...\n\n");
	if (fscanf (hKeyFile,"Magic Flash FileName: %s\n",tmp_outfilename) != 1)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Magic Flash FileName: declaration statement is missing or corrupted!\n");
		printf ("This statement must be on the first line!\n");
		printf ("Format:Magic Flash FileName: yourfile.mff\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}
	if (fscanf (hKeyFile,"Version: %s\n",version) != 1)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Version: declaration statement is missing or corrupted!\n");
		printf ("This statement must be on the second line!\n");
		printf ("Format: Version: 0x030101\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	if (fscanf (hKeyFile,"Trusted: %s\n",szDword) != 1)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("Trusted: declaration statement is missing or corrupted!\n");
		printf ("Format: Trusted: 0\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	while (fgets (fLine,100,hKeyFile) != NULL)
	{
		if (strstr (fLine,"Size of Reserved in bytes") != NULL)
		{
			if (GetParameter (fLine,sizeOfReserved,(char*)"Size of Reserved in bytes",FALSE) == FALSE) return FALSE;
		}
	}

	rewind (hKeyFile);
	if ((Translate (sizeOfReserved) != 0) && (Translate (version) == (0x030102)))
	{
		if (VerifyReservedData (hKeyFile) == FALSE) IsOk = FALSE;
	}

	if (Translate (szDword) == 0)
	{
		if (VerifyNonTrusted (hKeyFile) == FALSE) IsOk = FALSE;
		if (VerifyNumberOfImages (hKeyFile) == FALSE) IsOk = FALSE;
	}
	else
	{
		if (VerifyNumberOfImages (hKeyFile) == FALSE) IsOk = FALSE;
		#ifdef TRUSTED 
		if (VerifyNumberOfKeys (hKeyFile) == FALSE) IsOk = FALSE;
		#endif
	}

	rewind (hKeyFile);

	return IsOk;
}

bool VerifyNumberOfImages (FILE *hKeyFile)
{
	char fLine[100];
	char szTemp[100];
	int i, NumberFound = 0;

	while (fgets (fLine,100,hKeyFile) != NULL)
	{
		if (strstr (fLine,"Number of Images") != NULL)
		{
			if (GetParameter (fLine,szTemp,(char*)"Number of Images",FALSE) == FALSE) return FALSE;

			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("////////////////////////////////////////////////////////////\n");
					printf ("Number of Images: declaration statement can only be set to numbers!\n");
					printf ("Format: Number of Images: 2\n");
					printf ("////////////////////////////////////////////////////////////\n");
					rewind (hKeyFile);
					return FALSE;
				}

				++i;
			}

			for (i = 0,NumberFound = 0; i < atoi (szTemp); i++)
			{
				while (fgets (fLine,100,hKeyFile) != NULL)
				{
					if (strstr (fLine,"Next Image ID:") != NULL)
					{
						continue;
					}
					if (strstr (fLine,"Image ID:") != NULL)
					{
						NumberFound++;
					}
				}
			}

			if (NumberFound != atoi (szTemp))
			{
				printf ("////////////////////////////////////////////////////////////\n");
				printf ("The number of images found does not match the number of images\n");
				printf ("set in Number of Images: declaration statement!\n\n");
				printf ("Number of Images: %d\n",atoi (szTemp));
				printf ("Number of images found = %d\n",NumberFound);
				printf ("////////////////////////////////////////////////////////////\n");
				rewind (hKeyFile);
				return FALSE;
			}
		}
	}

	rewind (hKeyFile);

	return TRUE;
}

bool VerifyNonTrusted (FILE *hKeyFile)
{
	char fLine[100];

	while (fgets (fLine,100,hKeyFile) != NULL)
	{
		if (strstr (fLine,"WTM Save State") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain any\n");
			printf ("WTM Save State declaration statements!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"Number of Keys") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("Number of Keys: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"Key ID") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("Key ID: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"Modulus Size in bytes") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("Modulus Size in bytes: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"Public Key Size in bytes") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("Public Key Size in bytes: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"RSA Public Key Exponent") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("RSA Public Key Exponent: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"RSA System Modulus") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("RSA System Modulus: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"DSA Algorithm ID") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("DSA Algorithm ID: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"RSA Public Exponent") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("RSA Public Exponent: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
		if (strstr (fLine,"RSA Private Key") != NULL)
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("The non trusted descriptor file should not contain a\n");
			printf ("RSA Private Key: declaration statement!\n\n");
			printf ("////////////////////////////////////////////////////////////\n");
			rewind (hKeyFile);
			return FALSE;
		}
	}

	rewind (hKeyFile);

	return TRUE;
}

bool GetParameter (char *fLine,char *pParameter,char *pDec,bool IsFile)
{
	char *pTemp = NULL;

	// This function is used by the descriptor integrity check routines to parse
	// out values from lines that contain a semicolon. The semicolon is used as
	// part of a directive. Example directive( NumberOfImages: 3 ). In this case
	// the number 3 is extracted signifying the number of images.

	if (strstr (fLine,":") == NULL)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf  ("%s: \nDeclaration statement does not contain a semi-colon sign in it!\n",pDec);
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	strtok (fLine,":");
	pTemp = strtok (NULL,":");

	if (pTemp == NULL)
	{
		printf  ("%s declaration statement is invalid!\n",pDec);
		printf  ("Format: %s: parameter \n",pDec);
		return FALSE;
	}

	if (IsFile)
	{
		while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
		{
			if (isprint (*pTemp) && *pTemp != ' ')
			{
				while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
				{
					if (isprint (*pTemp))
					{
						*pParameter = *pTemp;
						++pParameter;
					}
					++pTemp;
				}
				break;
			}
			++pTemp;
		}
	}
	else
	{
		while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
		{
			if (isprint (*pTemp) && *pTemp != ' ')
			{
				*pParameter = *pTemp;
				++pParameter;
			}
			++pTemp;
		}
	}
	*pParameter = '\0';

	return TRUE;
}
