/**************************************************************************
**
**  (C) Copyright December 2006 Marvell International Ltd.
**
**  All Rights Reserved.
**
**  This software file (the "File") is distributed by Marvell International  
**  Ltd. under the terms of the GNU General Public License Version 2, June  
**  1991 (the "License").  You may use, redistribute and/or modify this File   
**  in accordance with the terms and conditions of the License, a copy of  
**  which is available along with the File in the license.txt file or by   
**  writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
**  Boston, MA 02111-1307 or on the worldwide web at 
**  http://www.gnu.org/licenses/gpl.txt.
**
**  THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
**  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
**  PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
**  details about this warranty disclaimer. 
**
**  Filename: NTBBTIM.cpp
**
**  This program builds the TIM binary image from a TIM descriptor file and all of
**  the binary images that it lists.
**  This is a console application.
**  
******************************************************************************/


#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 
#include <ctype.h> 
#include <sys/types.h>
#include <fcntl.h> 
#include <sys/stat.h> 
#include <limits.h> 
#include "Typedef.h" 
#include "Tim.h"
#ifdef TRUSTED
#include "ippCP.h"
#endif
#include "TBBLIB.h"

/*
 *	This is the switch used for building non-trusted only version of the tbb tool
 *  versus the full version(non-trusted and trusted.) The non-trusted only version
 *  doesn't rely on the ippcp libraries. 
 */

#undef TRUSTED

#define VERBOSE 'v'
#define HELP '?'
#define KEY 'r'
#define TIMIN 'i'
#define TIMOUT 'o'
#define HASH 'h'
#define DS 's'
#define OPTION 'm'
#define TIMVERIFY 'V'
#define RESERVED 'R'
#define CONCATENATE 'C'
#define MIN_ARGS 2

char Version[11] = "3.09";
char Date[11] = "03062008";

bool bIsKeyFile, bIsHashFile, bIsDsFile, bIsTimInFile, bIsTimOutFile, bIsNonTrusted;
bool bIsTimVerify;
char KeyFileName[MAX_PATH], HashFileName[MAX_PATH], DsFileName[MAX_PATH];
char TimInFileName[MAX_PATH], TimOutFileName[MAX_PATH];
char ReservedFileName[MAX_PATH];
int iOption = 1;
bool bVerbose, bConcatenate;
char OutputMFFFileName[256];

bool ParseCommandLine (int argc,char* argv[]);
bool ParseDescriptorFile (char *FileName);
bool ParseNonTrustedDescriptorFile (FILE *hKeyFile,CTIM *pTimHeader);
extern bool ScanImageInfo (FILE *hKeyFile,CTIM *pTimHeader,IMAGE_INFO *pImageInfo,STPreTIMItem* pPreTimInfo);
bool GetDWord (FILE *hKeyFile,char *szString,UINT_T *pValue);
bool CheckYourOptions ();
void PrintUsage ();
bool ParseTrustedDescriptorFile (FILE *hKeyFile,CTIM *pTimHeader);
#ifdef TRUSTED
extern bool BuildDigitalSignature (char *KeyFileName,char *HashFileName);
extern bool AddDigitalSignatureToTim (char *TimFileName,char *TimOutFileName,char *DsFileName);
#endif
int PreTimMerge(char* pDestFileName,STPreTIMItem* pPreTimItem,int itemcount);
#pragma warning ( disable : 4096 4081 ) // Disable warning messages


int main (int argc,char *argv[])
{
	bVerbose = FALSE;
	bIsKeyFile = FALSE;
	bIsHashFile = FALSE;
	bIsDsFile = FALSE;
	bIsTimInFile = FALSE;
	bIsTimOutFile = FALSE;
	bIsTimVerify = FALSE;
	bIsNonTrusted = FALSE;
	bIsReservedDataInFile = FALSE;
	bConcatenate = FALSE;
	memset(OutputMFFFileName,0,256);
	printf ("\n\nTBB Version: %s\n", Version);
	printf ("TBB Date   : %s\n\n", Date);

	if (argc < MIN_ARGS)
	{
		PrintUsage ();
		return FALSE;
	}
	
	if (ParseCommandLine (argc,argv) == FALSE) return FALSE;

	if ((iOption == 1) || (iOption == 2)) 
	{
		ParseDescriptorFile (KeyFileName);
	}
	else if(iOption == 3)
	{ 
		#ifdef TRUSTED
		BuildDigitalSignature (KeyFileName,HashFileName);
		#endif
	}
	else if(iOption == 4)
	{
		#ifdef TRUSTED 
		AddDigitalSignatureToTim (TimInFileName,TimOutFileName,DsFileName);
		#endif
	}
	else
		printf ("  Error: Invalid Option Mode Selected!");

	return TRUE;
}

bool ParseDescriptorFile (char *FileName)
{
	FILE *hKeyFile;
	CTIM TimHeader;
	char szDword[11];
	
//	if(FileName[0] == '/')
//	{
//	   for(int i=0;i<MAX_PATH;i++)
//	   {
//	      FileName[i]=FileName[i+1];
//	   }
//	}

	if ((hKeyFile = fopen (FileName,"r")) == NULL)
	{
		printf ("  Error: Cannot open file name %s !",FileName);
		return FALSE;
	}

	if (VerifyDescriptorIntegrity (hKeyFile) == FALSE)
	{
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}
	memset (&TimHeader,0,sizeof (CTIM));
	TimHeader.VersionBind.Identifier = TIMIDENTIFIER;

	if (fscanf (hKeyFile,"Magic Flash FileName: %s\n",OutputMFFFileName) != 1)
	{
		printf(" Error: Magic Flash FileName parsing error reading\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}
	
	if (fscanf (hKeyFile,"Version: %s\n",szDword) != 1)
	{
		printf ("  Error: Key file parsing error reading, Version:\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}

	TimHeader.VersionBind.Version = Translate (szDword);

	if (fscanf (hKeyFile,"Trusted: %s\n",szDword) != 1)
	{
		printf ("  Error: Key file parsing error reading, Trusted:\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}

	TimHeader.VersionBind.Trusted = Translate (szDword);

	if (TimHeader.VersionBind.Trusted == 0)
	{
		bIsNonTrusted = TRUE;
		if (CheckYourOptions () == FALSE) return FALSE;
		if (iOption == 1) ParseNonTrustedDescriptorFile (hKeyFile,&TimHeader);
		else
		{
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("  Error: You have selected option mode 2, 3 or 4!\n");
			printf ("You can only build a Non Trusted Image in option mode 1!\n");
			printf ("Example: tbb -m 1 -k ntimkey.txt\n\n");
			printf ("Also check the Trusted directive in the TIM descriptor file.\n");
			printf ("Example: Trusted: 0  (for Non Trusted Image)\n");
			printf ("Example: Trusted: 1  (for Trusted Image)\n");
			printf ("////////////////////////////////////////////////////////////\n");
			if (hKeyFile != NULL) fclose (hKeyFile);
			return FALSE;
		}
	}
	else
	{
		#ifdef TRUSTED
		ParseTrustedDescriptorFile (hKeyFile,&TimHeader);
		#else
		printf("  Error: You can only build a Non Trusted Image \n");
		printf("\twith the Non Trusted only version of the TBB Tool! \n\n");
		printf("Processing has stopped!\n");
		return FALSE;
		#endif
	}

	if (hKeyFile != NULL) fclose (hKeyFile);

	return TRUE;
}

bool ParseNonTrustedDescriptorFile (FILE *hKeyFile,CTIM *pTimHeader)
{
	char szTimFilename[MAX_PATH], szImageFilename[MAX_PATH], szFileNamesList[MAX_IMAGE_FILES][MAX_PATH];
	UINT_T IntParam, NextImageID = 0;
	UINT32_T ImageCrc;
	IMAGE_INFO ImageInfo;
	
    char cDownloadToFlash;
	char cYaffsSystem;
	UINT_T YaffsVolume;

	UINT_T HashValue_Tim;

	FILE *hTimFile, *hImage, *hReservedFile, *hBinaryImage;
	char ReservedBuffer[MAX_PATH];
	unsigned int loadAddress[MAX_IMAGE_FILES];
	unsigned int imageSize[MAX_IMAGE_FILES];
	unsigned int flashAddress[MAX_IMAGE_FILES];
	const unsigned int MSysSectorSize = 512;
	unsigned int MSysSectorOffset[MAX_IMAGE_FILES];



	STPreTIMItem* pPreTimItems;


	printf ("\nProcessing non trusted TIM...\n\n");
	
	if (GetDWord  (hKeyFile,(char*)"Issue Date: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->VersionBind.IssueDate = IntParam;
	if (GetDWord (hKeyFile,(char*)"OEM UniqueID: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->VersionBind.OEMUniqueID = IntParam;
	if (GetDWord (hKeyFile,(char*)"Boot Flash Signature: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->FlashInfo.BootFlashSign = IntParam;
	if (GetDWord (hKeyFile,(char*)"Number of Images: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->NumImages = IntParam;
	
	if (bIsReservedDataInFile)
	{
		if ((hReservedFile = fopen (ReservedFileName,"rb")) == NULL)
		{
			printf ("  Error: Cannot open file name %s !",ReservedFileName);
			if (hReservedFile != NULL) fclose (hReservedFile);
			return FALSE;
		}
		fseek (hReservedFile,0L,SEEK_END); // Set position to EOF
		pTimHeader->SizeOfReserved = ftell (hReservedFile); // Get position of file, thus the file size.
		fseek (hReservedFile,0L,SEEK_SET); // Set position to SOF
	}
	else
	{
		if (GetDWord (hKeyFile,(char*)"Size of Reserved in bytes: %s\n",&IntParam) == FALSE) return FALSE;
		pTimHeader->SizeOfReserved = IntParam;
	}

	pTimHeader->FlashInfo.WTMFlashSign = 0xFFFFFFFF;
	pTimHeader->FlashInfo.WTMEntryAddr = 0xFFFFFFFF;
	pTimHeader->FlashInfo.WTMEntryAddrBack = 0xFFFFFFFF;
	pTimHeader->FlashInfo.WTMPatchSign = 0xFFFFFFFF;
	pTimHeader->FlashInfo.WTMPatchAddr = 0xFFFFFFFF;

	//return error if number of images > 10
	if ((int)pTimHeader->NumImages > MAX_IMAGE_FILES )
	{
		printf("  Error: Maximum allowable number of images in the descriptor file is 10!\n");
		return FALSE;
	}

	pPreTimItems = (STPreTIMItem*)malloc( ((pTimHeader->NumImages < 3)? 3:pTimHeader->NumImages) * sizeof(STPreTIMItem));
	memset(pPreTimItems,0,pTimHeader->NumImages * sizeof(STPreTIMItem));

	for (int i = 0; i < (int)pTimHeader->NumImages; i++)
	{
		memset (&ImageInfo,0,sizeof (IMAGE_INFO));
		if (ScanImageInfo (hKeyFile,pTimHeader,&ImageInfo,pPreTimItems+i) == FALSE) return FALSE;
		
		loadAddress[i] = ImageInfo.LoadAddr;
		flashAddress[i] = ImageInfo.FlashEntryAddr;

//		pPreTimItems[i].bDownloadToFlash = ImageInfo.LoadToFlash;
		pPreTimItems[i].iFlashAddress = ImageInfo.FlashEntryAddr;
//		pPreTimItems[i].bYaffsFile = ImageInfo.IsYaffs;
//		pPreTimItems[i].iYaffsSize = ImageInfo.YaffsBlockSize;

		if (i == 0)
		{
			pPreTimItems[i].iFromOffset = strlen("PRET") + 4 + pTimHeader->NumImages * sizeof(STPreTIMItem);
			

			if (ImageInfo.ImageID != TIMIDENTIFIER)
			{
				printf ("  Error: The ImageID value is incorrect for the TIM image!\n");
				return FALSE;
			}

			if (fscanf (hKeyFile,"Image Filename: %s\n",szTimFilename) != 1)
			{
				printf ("  Error: Key file parsing error reading, Image Filename:\n");
				return FALSE;
			}

			printf("the origial filename is %s\n",szTimFilename);
			strcpy(pPreTimItems[i].szFileName,szTimFilename);
			printf("the %d th filename is %s\n",i,pPreTimItems[i].szFileName);


			if ((hTimFile = fopen (szTimFilename,"wb+")) == NULL)
			{
				printf ("  Error: Cannot open file name %s !",szTimFilename);
				return FALSE;
			}

			ImageInfo.ImageSize = sizeof (CTIM) + (sizeof (IMAGE_INFO) * pTimHeader->NumImages)
								+ pTimHeader->SizeOfReserved;

			imageSize[i] = ImageInfo.ImageSize;

			if(ImageInfo.ImageSizeToHash == 0xFF || ImageInfo.ImageSizeToHash == 0xFFFFFFFF) 
				ImageInfo.ImageSizeToHash = ImageInfo.ImageSize;
//			else 
//				ImageInfo.ImageSizeToHash = 0x0;

			HashValue_Tim = ImageInfo.ImageSizeToHash;

			memset (&ImageInfo.Hash,0,8);


			fwrite (pTimHeader,sizeof (CTIM),1,hTimFile);
			pPreTimItems[i].iFileSize = sizeof(CTIM);
		}
		else
		{
			if (ImageInfo.ImageID != NextImageID)
			{
				printf ("  Error: The ImageID doesn't = NextImageID from the previous image!");
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}

			if (fscanf (hKeyFile,"Image Filename: %s\n",szImageFilename) != 1)
			{
				printf ("  Error: Key file parsing error reading, Image Filename:\n");
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}

			for (int m = i-1; m<=i-1;m++)
			{
				for (int n=0; n<MAX_PATH;n++)
				{
				szFileNamesList[m][n] = szImageFilename[n];
				}
			}

			if ((hImage = fopen (szImageFilename,"rb")) == NULL)
			{
				printf ("  Error: Cannot open file name %s !",szImageFilename);
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}






			fseek (hImage,0L,SEEK_END); // Set position to EOF
			imageSize[i] = ftell (hImage); // Get position of file, thus the file size.

            {//add by b074_gliethttp_20080314 ,fix ../ ./ path bug
                char *TempName;
                char *input_file_name = szImageFilename;
                char *output_file_name = pPreTimItems[i].szFileName;
                char *p;
                int index;

                p = strrchr(input_file_name,'/');
                if(p == NULL)
                {
                    p = input_file_name;
                }else p += 1;
                
                index = p - input_file_name;
                TempName = strtok(p,".");

                if(index)
                {char *binname;
                    binname = (char*)malloc(strlen(p)+10);
                    strncpy(output_file_name,input_file_name,index);
                    output_file_name[index] = 0;
                    sprintf(binname,"%s_h.bin",TempName);
                    strcat(output_file_name,binname);
                    free(binname);
                }else
                    {
                        sprintf(output_file_name,"%s_h.bin",TempName);
                    }
                fprintf(stderr,"TempName=%s\n",TempName);
                fprintf(stderr,"pPreTimItems[%d].szFileName=%s\n",i,pPreTimItems[i].szFileName);
            }
			

			pPreTimItems[i].iFromOffset = pPreTimItems[i-1].iFromOffset +  pPreTimItems[i-1].iFileSize;
			pPreTimItems[i].iFileSize = imageSize[i] + 4;

			// Check for overlapping images when the flash isn't MSystems
			if ( ( (loadAddress[i] - loadAddress[i-1]) < imageSize[i-1] ) ||
				(flashAddress[i] - flashAddress[i-1] < imageSize[i-1] ) &&
				(pTimHeader->FlashInfo.BootFlashSign != 0x4e414e01) )
			{
//				printf("  Error: There is overlapping between images.\n(Check flash entry address and load address of the images.)\n\n");
//				printf("Processing has failed!\n\n");
				//return FALSE;
			}
			// Check for overlapping images for MSystems flash
			else if (pTimHeader->FlashInfo.BootFlashSign == 0x4e414e01)
			{
				// The upper 24 bits of flash entry address is the sector offset.
				// (The lower 4 bits are the partition number.)
				MSysSectorOffset[i] = ( ( (ImageInfo.FlashEntryAddr & 0xFFFFFF0 ) >> 4 ) * MSysSectorSize );
				if ( MSysSectorOffset[i] < imageSize[i-1] )
				{
				  //					printf("  Error: There is overlapping between images.\n(Check flash entry address and load address of the images.)\n\n");
				  //printf("Processing has failed!\n\n");
				  //return FALSE;
				}
			}

			// Output new image with ImageID prepended
			if (ImageBuild (hImage,szImageFilename,&ImageInfo) == FALSE)
			{
				if (hTimFile != NULL) fclose (hTimFile);
				if (hImage != NULL) fclose (hImage);
				return FALSE;
			}

			if ((ImageInfo.ImageSizeToHash > ImageInfo.ImageSize) ||
				(ImageInfo.ImageSizeToHash == 0xFFFFFFFF))
			{
				ImageInfo.ImageSizeToHash = ImageInfo.ImageSize;
			}
			imageSize[i] = ImageInfo.ImageSize;
			// Set all hash values to 0 then place ImageCrc in first hash entry
			memset (&ImageInfo.Hash,0,8);
			//#define ARBELIDENTIFIER		0x4152424C		// "ARBL"
			//#define TAVORIDENTIFIER		0x5F424D51		// "_BMQ" //TAVOR CP
//			if(ImageInfo.ImageID == ARBELIDENTIFIER) 
			if(ImageInfo.ImageID == 0x4152424C )
				ImageInfo.ImageSize = 1024*1024*6 + 4;			       
			if(ImageInfo.ImageSizeToHash != 0){
				if (GenerateImageCrc (hImage,ImageInfo.ImageSizeToHash,&ImageCrc) == FALSE)
				{
					if (hTimFile != NULL) fclose (hTimFile);
					if (hImage != NULL) fclose (hImage);
					return FALSE;
				}
				ImageInfo.Hash[0] = ImageCrc;
			}

//			if (hImage != NULL) fclose (hImage);
		}

		fwrite (&ImageInfo,sizeof (IMAGE_INFO),1,hTimFile);

		NextImageID = ImageInfo.NextImageID;
	}
	
	if (bIsReservedDataInFile)
	{
		memset (ReservedBuffer,0,pTimHeader->SizeOfReserved);
		fread (ReservedBuffer,1,pTimHeader->SizeOfReserved,hReservedFile);
		fwrite (ReservedBuffer,sizeof (char),pTimHeader->SizeOfReserved,hTimFile);
		if (hReservedFile != NULL) fclose (hReservedFile);
	}
	else
	{
		// skip the reserved section if size is 0.
		if(pTimHeader->SizeOfReserved != 0){
			if (ProcessReservedData (hKeyFile,hTimFile,pTimHeader->SizeOfReserved) == FALSE)
			{
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}
		}
	}

	if(HashValue_Tim != 0){
		GenerateTimCrc (hTimFile);
	}

	unsigned int sizeOfPadding = 0;
	DWORD padding = 0x11111111;
	BYTE Data[8192];
	int iBlockSize = 8192;

	if (bConcatenate)
	{

		for(int j=0;j<(int)pTimHeader->NumImages - 1;j++)
		{
			sizeOfPadding = (flashAddress[j+1]- flashAddress[j] - imageSize[j])/ 4;
			fseek (hTimFile,0L,SEEK_END); // Set position to EOF
			for (int i = 0; i < sizeOfPadding; i++)
			{
				fwrite(&padding,sizeof (DWORD),1,hTimFile);
			}

			if ((hBinaryImage = fopen (szFileNamesList[j],"rb")) == NULL)
			{
				printf ("  Error: Cannot open file name %s !",szFileNamesList[j]);
				return FALSE;
			}

			while (iBlockSize == 8192)
			{
				iBlockSize = fread (Data,1,iBlockSize,hBinaryImage);
				fwrite (Data,1,iBlockSize,hTimFile);
			}
			iBlockSize = 8192;

		}
		if (hBinaryImage != NULL) fclose (hBinaryImage);

	}

	printf ("  Success: Processing has completed successfully!\n");

	if (hImage != NULL) fclose (hImage);
	if (hTimFile != NULL) fclose (hTimFile);	
    
    {
        char tmp_name[256+10];
        STPreTIMItem *p;
        strcpy(tmp_name,OutputMFFFileName);
        strcat(tmp_name,"__tmp__");
        
        remove(tmp_name);

	    PreTimMerge(tmp_name,pPreTimItems,pTimHeader->NumImages);
        for(int i = 0;i < pTimHeader->NumImages;i++)
         ;//remove(pPreTimItems[i].szFileName);
	memset(pPreTimItems,0,3 * sizeof(STPreTIMItem));

        p = pPreTimItems;
        p->bDownloadToFlash = 0;
        p->bYaffsFile = 0;
        memset(p->szFileName,0,strlen(p->szFileName));
        strcpy(p->szFileName,"NTIM_DKB.bin");

        p++;
        p->bDownloadToFlash = 0;
        p->bYaffsFile = 0;
        memset(p->szFileName,0,strlen(p->szFileName));
        strcpy(p->szFileName,"TAVOR_NTDKB_h.bin");

        p++;
        p->bDownloadToFlash = 0;
        p->bYaffsFile = 0;
        memset(p->szFileName,0,strlen(p->szFileName));
        strcpy(p->szFileName,tmp_name);

        PreTimMerge(OutputMFFFileName,pPreTimItems,3);

        remove(tmp_name);
    }

	free(pPreTimItems);	
	return TRUE;
}

bool ScanImageInfo (FILE *hKeyFile,CTIM *pTimHeader,IMAGE_INFO *pImageInfo,STPreTIMItem* pPreTimInfo)
{

	UINT_T IntParam;

	// This is a helper function to parse image info data from the descriptor
	// text file supplied by the user.

	if (GetDWord (hKeyFile,(char*)"Image ID: %s\n",&IntParam) == FALSE) return FALSE;
	pImageInfo->ImageID = IntParam;
	if (GetDWord (hKeyFile,(char*)"Next Image ID: %s\n",&IntParam) == FALSE) return FALSE;
	pImageInfo->NextImageID = IntParam;
	if (GetDWord (hKeyFile,(char*)"Flash Entry Address: %s\n",&IntParam) == FALSE) return FALSE;
	pImageInfo->FlashEntryAddr = IntParam;

	if (GetDWord (hKeyFile,(char*)"Load Address: %s\n",&IntParam) == FALSE) return FALSE;
		pImageInfo->LoadAddr = IntParam;

	if(pTimHeader->VersionBind.Trusted)
	{
        if (GetDWord (hKeyFile,(char*)"Image Size To Hash in bytes: %s\n",&IntParam) == FALSE) return FALSE;
        pImageInfo->ImageSizeToHash = IntParam;
        if (GetDWord (hKeyFile,(char*)"Hash Algorithm ID: %s\n",&IntParam) == FALSE) return FALSE;
        pImageInfo->HashAlgorithmID = IntParam;
	}
	else
	{
        if (GetDWord (hKeyFile,(char*)"Image Size To CRC in bytes: %s\n",&IntParam) == FALSE) return FALSE;
        pImageInfo->ImageSizeToHash = IntParam;
	}

	printf("Start to analyze my section\r\n");

	if (GetDWord (hKeyFile,(char*)"Download to Flash: %s\n",&IntParam) == FALSE) 
		return FALSE;
	pPreTimInfo->bDownloadToFlash = IntParam;

	printf ("  Download to Flash: %d\n", pPreTimInfo->bDownloadToFlash);

	if (GetDWord (hKeyFile,(char*)"Is Yaffs: %s\n",&IntParam) == FALSE) 
		return FALSE;
	pPreTimInfo->bYaffsFile = IntParam;
	printf (" Is Yaffs: %d\n",pPreTimInfo->bYaffsFile);

	if (GetDWord (hKeyFile,(char*)"Yaffs Volume: %s\n",&IntParam) == FALSE) 
		return FALSE;
	pPreTimInfo->iYaffsSize = IntParam;

	printf ("Yaffs Volume: %d\n",pPreTimInfo->iYaffsSize);


	return TRUE;
}

bool GetDWord (FILE *hKeyFile,char *szString,UINT_T *pValue)
{
	char szDword[11], szDeclaration[50];
	int i = 0;

	// This is a helper function that converts strings to integers.

	if (fscanf (hKeyFile,szString,szDword) != 1)
	{
		strcpy (szDeclaration,szString);

		for (i = 0; i < 50; i++)
		{
			if (szDeclaration[i] == ':') szDeclaration[i] = '\0';
		}

		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: Declaration statement is missing or corrupted in descriptor file!\n");
		printf (" Error: Analyse the %s\r\n",szString);
		printf ("%s\n",szDeclaration);
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}
	*pValue = Translate (szDword);

	return TRUE;
}

bool ParseCommandLine (int argc,char *argv[])
{
	int currentOption = 1;
	BYTE Option;

	while (currentOption < argc)
	{
		if (*argv[currentOption] == '-'/* || 
*argv[currentOption] == '/'*/)
		{
			Option = *++argv[currentOption++];

			switch (Option)
			{
				case OPTION:
					iOption = atoi (argv[currentOption]);
					if (iOption > 4 || iOption < 1)
					{
						iOption = 1;
						printf("\n  Error: -m Parameter Invalid or Missing!\n"); 
						printf("Defaulting to Option Mode 1...\n");
					}
					break;
				case KEY:
					if (currentOption == argc)
					{
						printf ("\n  Error:   Command Line Parsing Error:");
						printf (" Missing Key file name parameter.\n");
						return FALSE;
					}
					strcpy (KeyFileName,argv[currentOption]);
					currentOption++;
					bIsKeyFile = TRUE;
					break;
				case HASH:
					if (currentOption == argc)
					{
						printf ("\n  Error:   Command Line Parsing Error:");
						printf (" Missing hash file name parameter.\n");
						return FALSE;
					}
					strcpy (HashFileName,argv[currentOption]);
					currentOption++;
					bIsHashFile = TRUE;
					break;
				case DS:
					if (currentOption == argc)
					{
						printf ("\n  Error:   Command Line Parsing Error:");
						printf (" Missing hash file name parameter.\n");
						return FALSE;
					}
					strcpy (DsFileName,argv[currentOption]);
					currentOption++;
					bIsDsFile = TRUE;
					break;
				case TIMIN:
					if (currentOption == argc)
					{
						printf ("\n  Error:   Command Line Parsing Error:");
						printf (" Missing hash file name parameter.\n");
						return FALSE;
					}
					strcpy (TimInFileName,argv[currentOption]);
					currentOption++;
					bIsTimInFile = TRUE;
					break;
				case TIMOUT:
					if (currentOption == argc)
					{
						printf ("\n  Error:   Command Line Parsing Error:");
						printf (" Missing hash file name parameter.\n");
						return FALSE;
					}
					strcpy (TimOutFileName,argv[currentOption]);
					currentOption++;
					bIsTimOutFile = TRUE;
					break;
				case VERBOSE:
					bVerbose = TRUE;
					break;
				case CONCATENATE:
					bConcatenate = TRUE;
					break;
				case TIMVERIFY:
					bIsTimVerify = TRUE;
					break;
				case RESERVED:
					if (currentOption == argc)
					{
						printf ("\n  Error:   Command Line Parsing Error:");
						printf (" Missing reserved data file name parameter.\n");
						return FALSE;
					}
					strcpy (ReservedFileName,argv[currentOption]);
					currentOption++;
					bIsReservedDataInFile = TRUE;
					break;
				case HELP:
					PrintUsage();
					break;
				default:
					printf ("  Error: Command line option -%c is invalid!\n",Option);
					return FALSE;
			}
		}
		else
		{
			currentOption++;
		}
	}

	if (CheckYourOptions () == FALSE) return FALSE;

	return TRUE;
}

bool CheckYourOptions ()
{

	// Check for TIM verify functionality for trusted operation.
	if ( ( iOption != 1 ) && ( bIsNonTrusted && bIsTimVerify ) )
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: Invalid Options Error:\n");
		printf ("Missing switch, non-trusted descriptor file or incorrect option mode.\n");
		printf ("Example: tbb -m 1 -r timdescriptorfile.txt -V\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}
	switch (iOption)
	{
		case 1:
			if (!bIsKeyFile)
			{
				printf ("////////////////////////////////////////////////////////////\n");
				printf ("  Error: Invalid Options Error:\n");
				printf ("Missing TIM descriptor file name parameter. (-r filename.txt)\n");
				printf ("Example: tbb -m 1 -r timdescriptorfile.txt\n");
				printf ("////////////////////////////////////////////////////////////\n");
				return FALSE;
			}
			break;
		case 2:
			if (!bIsKeyFile || !bIsHashFile)
			{
				printf ("////////////////////////////////////////////////////////////\n");
				printf ("  Error: Invalid Options Error:\n");
				printf ("Missing TIM descriptor file name parameter. (-r filename.txt) OR\n");
				printf ("Missing output hash key file name parameter. (-h filename.bin)\n");
				printf ("Example: tbb -m 2 -r timdescriptorfile.txt -h timhash.bin\n");
				printf ("////////////////////////////////////////////////////////////\n");
				return FALSE;
			}
			break;
		case 3:
			if (!bIsKeyFile || !bIsHashFile || !bIsDsFile)
			{
				printf ("////////////////////////////////////////////////////////////\n");
				printf ("  Error: Invalid Options Error:\n");
				printf ("Missing private key file name parameter. (-r filename.txt) OR\n");
				printf ("Missing input hash key file name parameter. (-h filename.bin) OR\n");
				printf ("Missing output digital signature file name parameter. (-s filename.bin)\n");
				printf ("////////////////////////////////////////////////////////////\n");
				return FALSE;
			}
			break;
		case 4:
			if (!bIsTimInFile || !bIsTimOutFile || !bIsDsFile)
			{
				printf ("////////////////////////////////////////////////////////////\n");
				printf ("  Error: Invalid Options Error:\n");
				printf ("Missing input TIM file name parameter. (-i filename.bin) OR\n");
				printf ("Missing output TIM file name parameter. (-o filename.bin) OR\n");
				printf ("Missing input digital signature file name parameter. (-s filename.bin)\n");
				printf ("////////////////////////////////////////////////////////////\n");
				return FALSE;
			}
			break;
		default:
			printf ("////////////////////////////////////////////////////////////\n");
			printf ("  Error: Invalid Options Error:\n");
			printf ("You entered an incorrect value for the -m option or\n");
			printf ("Example: tbb -m 1 -r TimDescriptorFile.txt\n");
			printf ("Note: If you leave out the -m option the program will\n");
			printf ("      default this option to 1.\n");
			printf ("////////////////////////////////////////////////////////////\n");
			return FALSE;
	}

	return TRUE;
}

void PrintUsage ()
{
	printf ("\n WTP Trusted Image Module Builder\n\n");
	#ifndef TRUSTED
	printf ("  ***Note: This is the non-trusted version only***\n\n");
	#endif
	printf ("  Usage:\n");
	printf ("  ======\n");
	printf ("  >> TBB.exe <Option>|<Option file> ...\n\n");
	printf ("  Options:\n");
	printf ("  ========\n");
	printf ("  -v\t\tVerbose mode.  Debugging information will be printed.\n\n");
	printf ("  Build a trusted or non trusted image module:\n\tTBB -m 1 -r sample_tim.txt\n\n");
	#ifdef TRUSTED
	printf ("  Build an image module without digital signature and a hash of that image:\n");
	printf ("  \tTBB -m 2 -r sample_tim.txt -h timhash.bin\n\n");
	printf ("  Build a digital signature from image module hash and private key:\n");
	printf ("  \tTBB -m 3 -r privatekey.txt -h timhash.bin -s ds.bin\n\n");
	printf ("  Build a trusted image module from digital signature and image module:\n");
	printf ("  \tTBB -m 4 -s ds.bin -i TIMtestTrustNoDs.bin -o TIMtestTrustFromDs.bin\n\n");
	printf (" TIM Verify:\n");
	printf (" ===========\n");
	printf (" >>TBB.exe -m 1 -r TIM.txt -V\n\n");
	#endif
	printf (" Reading reserved data from external file:\n");
	printf (" =========================================\n");
	printf (" >>TBB.exe -r (N)TIM.txt -R ReservedDataFile.bin\n");
}

