 /******************************************************************************
 *               MODULE IMPLEMENTATION FILE
 *******************************************************************************
 * Title: rtc
 *
 * Filename: diag_stub.c
 *
 * Target, platform: Common Platform, SW platform
 *
 * Authors: Eilam Ben-Dror, Avi Erami
 *
 * Description:
 *
 * Last Updated: 29.07.2001
 *
 * Notes:
 ******************************************************************************/

/*----------- Global include files --------------------------------------------*/

#include "ce_kernel.h"
#include "nkarm.h"
#include <Manitoba.h>
#include "Nucleus.h"
#include <dma.h>
#include <wait.h>
#include <mmap.h>
#include <xsc1.h>
#include <memdefs.h>

void memInvalidate( void *pMem )
{

}

extern PPTE g_pOEMAddressTable;

//Assume input is virtual only if it is in the Shraed Memory.
//Otherwise no translation
DWORD mmuVirtualToPhysicalTranslate(DWORD vfn)
{
	//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
    int i = 0;
    DWORD pa;       // Physical Base Address of section
    DWORD va;       // Virtual Base Address of section
    DWORD vau;      // Virtual Address Upper Bound of section
    DWORD vfnmb;    // PFN rounded down to 1MB
    //
    // The end of the table is marked by an entry with a ZERO size.
    //
    while(g_pOEMAddressTable[i].dwSize) {
        va = g_pOEMAddressTable[i].dwVA & 0x1FF00000;
        pa = g_pOEMAddressTable[i].dwPA & 0xFFF00000;
        vau = va + (g_pOEMAddressTable[i].dwSize << 20) - 1;
        vfnmb = vfn & 0x1ff00000;
        if ((vfnmb >= va) && (vfnmb <= vau))
            return (/*(PVOID)*/ (((vfn - va) & (~0xFFF00000)) + pa /*+ 0x80000000*/));
        i++;
    }
	//We didn't find the address in the table
	//Might be null or physical
    return vfn;
}

//Return Uncached virtual addresses
//For cached virtual address subtract 0x20000000 from the Value
UINT32          mmuPhysicalToVirtualTranslate(UINT32 pfn)
{
    int i = 0;
    DWORD va;       // Virtual Base Address of section
    DWORD pa;       // Physical Base Address of section
    DWORD pau;      // Physical Address Upper Bound of section
    DWORD pfnmb;    // PFN rounded down to 1MB
    //
    // The end of the table is marked by an entry with a ZERO size.
    //
    while(g_pOEMAddressTable[i].dwSize) {
        va = g_pOEMAddressTable[i].dwVA & 0x1FF00000;
        pa = g_pOEMAddressTable[i].dwPA & 0xFFF00000;
        pau = pa + (g_pOEMAddressTable[i].dwSize << 20) - 1;
        pfnmb = pfn & 0xfff00000;
        if ((pfnmb >= pa) && (pfnmb <= pau))
            return  ((pfn - pa) + va + 0xA0000000);
        i++;
    }
    DEBUGMSG(ZONE_PHYSMEM, (TEXT("Phys2Virt() : PFN (0x%08X) not found!\r\n"), pfn));
    //We didn't find the address in the table
	//Might be null or virtual
    return pfn;
}

void *PhysicalToVirtualNonCached(void *inAddress)
{
	return (void *)mmuPhysicalToVirtualTranslate((UINT32)inAddress);
}


#define _UPPER      0x1    /* upper case letter */
#define _LOWER      0x2    /* lower case letter */
#define _DIGIT      0x4    /* digit[0-9] */
#define _SPACE      0x8    /* tab, carriage return, newline, */
#define _PUNCT      0x10    /* punctuation character */
#define _CONTROL    0x20    /* control character */
#define _BLANK      0x40    /* space char */
#define _HEX        0x80    /* hexadecimal digit */
#define _ALPHA      (0x0100|_UPPER|_LOWER)    /* alphabetic character */

int _isctype(int c,int mask)
{
	int type=0;

	if(c>='A' && c<='Z') type|=_UPPER;
	else if(c>='a' && c<='z') type|=_LOWER;
	else if(c>='0' && c<='9') type|=_DIGIT;
	else if(c=='\t' || c=='\r' || c=='\n') type|=_SPACE;
	else if(c==' ') type|=_BLANK;
	else if(c==',' || c=='.' || c==';' || c==':' || c=='\'' || c=='\"')
		type|=_PUNCT;

	if((c>='a' && c<='f') ||
	   (c>='A' && c<='F') ||
	   (type & _DIGIT))
	   type|=_HEX;

	return (mask & type);// Bit masks used by iswXXX macros, CTYPE.H
}

WINBASEAPI
BOOL
WINAPI
IsDBCSLeadByte(
    IN BYTE  TestChar)
{
	return FALSE;
}

LPVOID WINAPI TlsGetValue(DWORD slot)
{
    LPDWORD tlsptr = UTlsPtr ();
    LPVOID lpRet = NULL;
    if (tlsptr && (slot < TLS_MINIMUM_AVAILABLE)) {
        DEBUGCHK ((slot < TLSSLOT_NUMRES) || ((HANDLE) GetCurrentProcessId () == GetOwnerProcess ()));
        // from SDK help:
        // Note  The data stored in a TLS slot can have a value of zero. In this case,
        // the return value is zero and GetLastError returns NO_ERROR.

        // fail TlsGetValue in non-ship build if current process != owner process
#ifndef SHIP_BUILD
        if ((slot >= TLSSLOT_NUMRES) && ((HANDLE) GetCurrentProcessId () != GetOwnerProcess ()))
            SetLastError (ERROR_INVALID_PARAMETER);
        else
#endif
        if (!(lpRet = (LPVOID) tlsptr[slot])) {
            //SetLastError (NO_ERROR);
        }
    } else {
        //SetLastError (ERROR_INVALID_PARAMETER);
    }
    return lpRet;
}

HANDLE xxx_GetOwnerProcess(void)
{
	return SC_GetOwnerProcess();
}

LPVOID
xxx_GetProcAddressW(HANDLE hInst, LPWSTR lpszProc)
{
	return SC_GetProcAddressW(hInst, lpszProc);
}


static LPVOID poolPtr = (LPVOID)INT_POOL_C_VIRTUAL;

typedef struct
{
	int		dataSize;
	void	*pNext;
	void	*pPrev;
	DWORD	data[1];
} PoolRecord;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
HANDLE
CreateCommThread(
    LPTHREAD_START_ROUTINE lpStartAddr,
    LPVOID lpvThreadParm,
    WORD prio,
	DWORD stackSize,
    DWORD dwFlags
    )
{
    CALLBACKINFO cbi;
    HANDLE hTh;
#if 1
	stackSize<<=1;
#endif
	if((DWORD)(&(((BYTE *)poolPtr)[stackSize]))>=INT_POOL_C_VIRTUAL+INT_POOL_SIZE)
		return 0;
    if (pCurProc == &ProcArray[0]) {
        hTh = DoCreateThread(poolPtr, stackSize, (LPVOID)lpStartAddr, lpvThreadParm, dwFlags, 0, TH_KMODE, prio);
    } else {
        cbi.hProc = ProcArray[0].hProc;
        cbi.pfn = (FARPROC)DoCreateThread;
        cbi.pvArg0 = poolPtr;
        hTh = (HANDLE)PerformCallBack(&cbi,stackSize,(LPVOID)lpStartAddr, lpvThreadParm, dwFlags, 0, TH_KMODE, prio);
    }
	if(hTh!=INVALID_HANDLE_VALUE)
		poolPtr=&(((BYTE *)poolPtr)[stackSize]);
    return hTh;
}

static PoolRecord *firstRecord;

void K_initDynamicPool()
{
	PoolRecord *poolRecord = (PoolRecord *)poolPtr;
	poolRecord->pNext = poolRecord->pPrev = NULL;
	poolRecord->dataSize = INT_POOL_C_VIRTUAL+INT_POOL_SIZE - ((DWORD) poolPtr) - sizeof(PoolRecord) + sizeof(DWORD);
	firstRecord = poolRecord;
}

void *K_malloc(int size)
{
	DWORD cpsr;
	PoolRecord *poolRecord;

	size = 4*((size + 3)/4);

	cpsr = disableInterrupts();

	poolRecord= firstRecord;

	while(poolRecord!=NULL)
	{
		if(poolRecord->dataSize>=size)
		{
			//We decided to give poolRecord up to the user.

			//Check if we have enough room for a new free record with at list sizeof(DWORD) free
			if(poolRecord->dataSize >= (size + sizeof(PoolRecord)) )
			{
				//We position a new Free Record after the allocated one.
				PoolRecord *pCurrent = poolRecord;

				//Position the poolRecord pointer that we are going to give up to the user
				poolRecord = (PoolRecord *) ((DWORD)pCurrent->data + pCurrent->dataSize - size - sizeof(PoolRecord) + sizeof(DWORD));

				//We calculate the remaining size for the free Record
				pCurrent->dataSize -= size + sizeof(PoolRecord) - sizeof(DWORD);
				//We give the poolRecord the desired size
				poolRecord->dataSize = size;
			}
			//We don't have enough space for a new Free Record
			else
			{
				if(poolRecord->pPrev!=NULL)
					((PoolRecord *)(poolRecord->pPrev))->pNext = poolRecord->pNext;
				if(poolRecord->pNext!=NULL)
					((PoolRecord *)(poolRecord->pNext))->pPrev = poolRecord->pPrev;

				if(poolRecord==firstRecord)
				{
					if(poolRecord->pNext!=NULL)
						firstRecord = poolRecord->pNext;
					else
						firstRecord = NULL;
				}
			}

			break;
		}
		poolRecord = poolRecord->pNext;
	};

	restoreInterrupts(cpsr);

	if(poolRecord!=NULL)
		return poolRecord->data;
	return NULL;
}

void K_free(void *block)
{
	if(((DWORD)block>=(DWORD)poolPtr) &&
	   ((DWORD)block< INT_POOL_C_VIRTUAL+INT_POOL_SIZE))
	{
		DWORD cpsr;
		PoolRecord *poolRecord,*pPrev,*pNext;
		cpsr = disableInterrupts();

		poolRecord = (PoolRecord *) (((DWORD)block)-sizeof(PoolRecord)+sizeof(DWORD));

		if(firstRecord==NULL)
		{
			firstRecord = poolRecord;
			poolRecord->pNext = poolRecord->pPrev = NULL;
			restoreInterrupts(cpsr);
			return;
		}

		pPrev = firstRecord;

		if((DWORD)pPrev > (DWORD)poolRecord)
		{
			pNext = pPrev;
			pPrev = NULL;
			firstRecord = poolRecord;
		}
		else
		{
			if(pPrev->pNext!=NULL)
			{
				while(pPrev!=NULL)
				{

					if(pPrev->pNext > poolRecord)
					{
						pNext = pPrev->pNext;
						break;
					}

					if(pPrev->pNext!=NULL)
					{
						pPrev = pPrev->pNext;
					}
					else
					{
						pNext = NULL;
						break;
					}
				}
			}
			else
				pNext = NULL;
		}

		poolRecord->pPrev = pPrev;
		poolRecord->pNext = pNext;

		if(pNext!=NULL)
		{
			pNext->pPrev = poolRecord;
		}

		if(pPrev!=NULL)
		{
			pPrev->pNext = poolRecord;
		}


		if(poolRecord->pPrev==NULL)
			poolRecord = poolRecord->pNext;

		if( (poolRecord!=NULL) &&
			(poolRecord->pPrev!=NULL))
		{
			//Check if previous is consecutive
			//If it is, weld them together
			while(
				((DWORD)(((PoolRecord *)(poolRecord->pPrev))->data) +
				((PoolRecord *)(poolRecord->pPrev))->dataSize) ==
				(DWORD)poolRecord)
			{
				if(poolRecord->pNext!=NULL)
					((PoolRecord *)(poolRecord->pNext))->pPrev = poolRecord->pPrev;

				((PoolRecord *)(poolRecord->pPrev))->dataSize+=poolRecord->dataSize + sizeof(PoolRecord) - sizeof(DWORD);

				((PoolRecord *)(poolRecord->pPrev))->pNext = poolRecord->pNext;

				if(poolRecord->pNext!=NULL)
					poolRecord = poolRecord->pNext;
				else
					break;
			}
		}

		restoreInterrupts(cpsr);
	}
	else
		while(1);
}


