/*
 * linux/include/asm-arm/arch-mv88w8xx8/hardware.h
 * For Marvell Single Chip WLAN and VoIP Processor 88W8xx8
 * Copyright 2005 Radlan A Marvell Company, Aharon Gadot <aharong@marvell.com>
 *
 * If you received this File from Marvell, you may opt to use, redistribute
 * and/or modify this File in accordance with the terms and conditions of the
 * General Public License Version 2, June 1991 (the GPL License), a copy of
 * which is available along with the File in the license.txt file or by writing
 * to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 or on the worldwide web at http://www.gnu.org/licenses/gpl.txt.
 *
 * THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY DISCLAIMED.
 * The GPL License provides additional details about this warranty disclaimer.
 *
 */
#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#define CONFIG_ARCH_MV88W8638

#define MV88W8XX8_REGS_BASE1  0x80000000
#define MV88W8XX8_REGS_BASE2  0x90000000
#define MV88W8XX8_SDRAM_REGS  0xB0000000
#define MV88W8XX8_REGS_SIZE   0x10000

#define MV88W8XX8_SRAM_BASE     0xC0000000
#if     defined(CONFIG_ARCH_MV88W8638)
#define MV88W8XX8_SRAM_SIZE     0x20000   /* 128K */
#elif   defined(CONFIG_ARCH_MANHATTAN)
#define MV88W8XX8_SRAM_SIZE     0x20000   /* 128K */
#elif   defined(CONFIG_ARCH_MV88W8618)
#define MV88W8XX8_SRAM_SIZE     0x10000   /* 64k */
#elif   defined(CONFIG_ARCH_MV88W8618_HS31)
#define MV88W8XX8_SRAM_SIZE     0x10000   /* 64k */
#elif   defined(CONFIG_ARCH_MV88W8518)
#define MV88W8XX8_SRAM_SIZE     0x20000   /* 128K */
#else
#error mv88w8618, mv88w8618-HS31, mv88w8638,  mv88w8518 or manhattan must be defined
#endif

#define MV88W8XX8_SRAM_END      (MV88W8XX8_SRAM_BASE + MV88W8XX8_SRAM_SIZE)

/* macro to get at IO space when running virtually */
#define IO_ADDRESS(x)       (x)
#define REAL_ADDRESS(x)     (((x) & 0x00ffffff) + (((x) << 4) & 0xf0000000))

#define MV88W8XX8_REG_WRITE(reg,data)  *(uint_32 *)(IO_ADDRESS(reg)) = data
#define MV88W8XX8_REG_READ(reg)        *(uint_32 *)(IO_ADDRESS(reg))

/*
 *****************************************************************************
 *
 *  Interrupts Registers
 *
 *****************************************************************************
 */
#define MV88W8XX8_INT_BASE    0x90008000

#define MV88W8XX8_INT_ENABLE_CLR  (MV88W8XX8_INT_BASE + 0xC)
#define MV88W8XX8_INT_ENABLE_SET  (MV88W8XX8_INT_BASE + 0x8)
#define MV88W8XX8_INT_STATUS    (MV88W8XX8_INT_BASE + 0x0)

/*
 *****************************************************************************
 *
 *  LBU Bridge Registers
 *
 *****************************************************************************
 */
#define MV88W8XX8_LBUBRIDGE_BASE  0x8000C000
#define MV88W8XX8_LBU_RESET      (MV88W8XX8_LBUBRIDGE_BASE)
#define MV88W8XX8_LBU_CLOCK      (MV88W8XX8_LBUBRIDGE_BASE + 0x04)

/*
 *****************************************************************************
 *
 *  UART Registers
 *
 *****************************************************************************
 */
#define MV88W8XX8_UART1_BASE  0x8000C800
#define MV88W8XX8_UART1_RBR  (MV88W8XX8_UART1_BASE + 0x40)
#define MV88W8XX8_UART1_IER  (MV88W8XX8_UART1_BASE + 0x44)
#define MV88W8XX8_UART1_IIR  (MV88W8XX8_UART1_BASE + 0x48)
#define MV88W8XX8_UART1_LSR  (MV88W8XX8_UART1_BASE + 0x54)
#define MV88W8XX8_UART1_MSR  (MV88W8XX8_UART1_BASE + 0x58)

#define MV88W8XX8_UART2_BASE  0x8000C900
#define MV88W8XX8_UART2_RBR  (MV88W8XX8_UART2_BASE + 0x40)
#define MV88W8XX8_UART2_IER  (MV88W8XX8_UART2_BASE + 0x44)
#define MV88W8XX8_UART2_IIR  (MV88W8XX8_UART2_BASE + 0x48)
#define MV88W8XX8_UART2_LSR  (MV88W8XX8_UART2_BASE + 0x54)
#define MV88W8XX8_UART2_MSR  (MV88W8XX8_UART2_BASE + 0x58)


/*
 *****************************************************************************
 *
 *  Timer Unit Registers
 *
 *****************************************************************************
 */
#define MV88W8XX8_TIMER_BASE    0x90009000

#define MV88W8XX8_TIMER1_LENGTH   (MV88W8XX8_TIMER_BASE + 0x00)
#define MV88W8XX8_TIMER2_LENGTH   (MV88W8XX8_TIMER_BASE + 0x04)
#define MV88W8XX8_TIMER3_LENGTH   (MV88W8XX8_TIMER_BASE + 0x08)
#define MV88W8XX8_TIMER4_LENGTH   (MV88W8XX8_TIMER_BASE + 0x0c)
#define MV88W8XX8_TIMER_CONTROL   (MV88W8XX8_TIMER_BASE + 0x10)
#define MV88W8XX8_TIMER1_VALUE    (MV88W8XX8_TIMER_BASE + 0x14)
#define MV88W8XX8_TIMER2_VALUE    (MV88W8XX8_TIMER_BASE + 0x18)
#define MV88W8XX8_TIMER3_VALUE    (MV88W8XX8_TIMER_BASE + 0x1C)
#define MV88W8XX8_TIMER4_VALUE    (MV88W8XX8_TIMER_BASE + 0x20)
#define MV88W8XX8_TIMER_INT_SOURCE  (MV88W8XX8_TIMER_BASE + 0x24)
#define MV88W8XX8_TIMER_INT_MASK  (MV88W8XX8_TIMER_BASE + 0x28)
#define MV88W8XX8_BOARD_RESET   (MV88W8XX8_TIMER_BASE + 0x34)

#if defined(CONFIG_ARCH_MANHATTAN)
#define HARD_RESET_NOW() \
{ extern void msi_wl_SetGPIOMode(long gpioNum, long Mode);\
  msi_wl_SetGPIOMode(0x8, 0); }
#else
#define HARD_RESET_NOW() \
  MV88W8XX8_REG_WRITE(MV88W8XX8_BOARD_RESET,0x1000);\
  MV88W8XX8_REG_WRITE(MV88W8XX8_BOARD_RESET,0x0000);
#endif

/*******************************************/
/* USB Core Registers                      */
/*******************************************/
//-----------------------------------
// TAVOR PV base address for CI2
//-----------------------------------
#define MV88W8XX8_USB_BASE 0x55502000

#define MV_USB_CORE_ID_REG                  (MV88W8XX8_USB_BASE + 0x00)
#define MV_USB_CORE_GENERAL_REG             (MV88W8XX8_USB_BASE + 0x04)
#define MV_USB_CORE_HOST_REG                (MV88W8XX8_USB_BASE + 0x08)
#define MV_USB_CORE_DEVICE_REG              (MV88W8XX8_USB_BASE + 0x0C)
#define MV_USB_CORE_TX_BUF_REG              (MV88W8XX8_USB_BASE + 0x10)
#define MV_USB_CORE_RX_BUF_REG              (MV88W8XX8_USB_BASE + 0x14)
#define MV_USB_CORE_TTTX_BUF_REG            (MV88W8XX8_USB_BASE + 0x18)
#define MV_USB_CORE_TTRX_BUF_REG            (MV88W8XX8_USB_BASE + 0x1C)
#define MV_USB_CORE_CAP_LENGTH_REG          (MV88W8XX8_USB_BASE + 0x100)
#define MV_USB_CORE_CAP_HCI_VERSION_REG     (MV88W8XX8_USB_BASE + 0x102)
#define MV_USB_CORE_CAP_HCS_PARAMS_REG      (MV88W8XX8_USB_BASE + 0x104)
#define MV_USB_CORE_CAP_HCC_PARAMS_REG      (MV88W8XX8_USB_BASE + 0x108)
#define MV_USB_CORE_CAP_DCI_VERSION_REG     (MV88W8XX8_USB_BASE + 0x120)
#define MV_USB_CORE_CAP_DCC_PARAMS_REG      (MV88W8XX8_USB_BASE + 0x124)
#define MV_USB_CORE_CMD_REG                 (MV88W8XX8_USB_BASE + 0x140)
#define MV_USB_CORE_CMD_RUN_BIT             0
#define MV_USB_CORE_CMD_RUN_MASK            (1 << MV_USB_CORE_CMD_RUN_BIT)
#define MV_USB_CORE_CMD_RESET_BIT           1
#define MV_USB_CORE_CMD_RESET_MASK          (1 << MV_USB_CORE_CMD_RESET_BIT)
#define MV_USB_CORE_STATUS_REG              (MV88W8XX8_USB_BASE + 0x144)
#define MV_USB_CORE_INTR_REG                (MV88W8XX8_USB_BASE + 0x148)
#define MV_USB_CORE_FRAME_INDEX_REG         (MV88W8XX8_USB_BASE + 0x14C)
#define MV_USB_CORE_PERIODIC_LIST_BASE_REG  (MV88W8XX8_USB_BASE + 0x154)
#define MV_USB_CORE_DEV_ADDR_REG            (MV88W8XX8_USB_BASE + 0x154)
#define MV_USB_CORE_ASYNC_LIST_ADDR_REG     (MV88W8XX8_USB_BASE + 0x158)
#define MV_USB_CORE_ENDPOINT_LIST_ADDR_REG  (MV88W8XX8_USB_BASE + 0x158)
#define MV_USB_CORE_BURST_SIZE_REG          (MV88W8XX8_USB_BASE + 0x160)
#define MV_USB_CORE_TX_FILL_TUNING_REG      (MV88W8XX8_USB_BASE + 0x164)
#define MV_USB_CORE_TX_TT_FILL_TUNING_REG   (MV88W8XX8_USB_BASE + 0x168)
#define MV_USB_CORE_ENDPT_NAK_REG           (MV88W8XX8_USB_BASE + 0x178)
#define MV_USB_CORE_ENDPT_NAK_EN_REG        (MV88W8XX8_USB_BASE + 0x17C)
#define MV_USB_CORE_CONFIG_FLAG_REG         (MV88W8XX8_USB_BASE + 0x180)
#define MV_USB_CORE_PORTSC_REG              (MV88W8XX8_USB_BASE + 0x184)
#define MV_USB_CORE_OTGSC_REG               (MV88W8XX8_USB_BASE + 0x1A4)
#define MV_USB_CORE_MODE_REG                (MV88W8XX8_USB_BASE + 0x1A8)
#define MV_USB_CORE_MODE_OFFSET             0
#define MV_USB_CORE_MODE_MASK               (3 << MV_USB_CORE_MODE_OFFSET)
#define MV_USB_CORE_MODE_HOST               (3 << MV_USB_CORE_MODE_OFFSET)
#define MV_USB_CORE_MODE_DEVICE             (2 << MV_USB_CORE_MODE_OFFSET)


#define MV_USB_CORE_ENDPT_SETUP_STAT_REG    (MV88W8XX8_USB_BASE + 0x1AC)
#define MV_USB_CORE_ENDPT_PRIME_REG         (MV88W8XX8_USB_BASE + 0x1B0)
#define MV_USB_CORE_ENDPT_FLUSH_REG         (MV88W8XX8_USB_BASE + 0x1B4)
#define MV_USB_CORE_ENDPT_STATUS_REG        (MV88W8XX8_USB_BASE + 0x1B8)
#define MV_USB_CORE_ENDPT_COMPLETE_REG      (MV88W8XX8_USB_BASE + 0x1BC)
#define MV_USB_CORE_ENDPT_CTRL_REG(ep)      (MV88W8XX8_USB_BASE + 0x1C0 + (ep*4))

/*******************************************/
/* 8618 USB PHY (UTMI) Registers           */
/*******************************************/
#define  CARD_USB_CTL               (MV88W8XX8_USB_BASE+0x304)
#define  CARD_PHY_PLL_CTL           (MV88W8XX8_USB_BASE+0x308)
#define  CARD_PHY_TX_CTL            (MV88W8XX8_USB_BASE+0x30C)
#define  CARD_PHY_RX_CTL            (MV88W8XX8_USB_BASE+0x310)
#define  CARD_PHY_IVREF_CTL         (MV88W8XX8_USB_BASE+0x314)
#define  CARD_PHY_REG_TEST0         (MV88W8XX8_USB_BASE+0x318)
#define  CARD_PHY_REG_TEST1         (MV88W8XX8_USB_BASE+0x31C)
#define  CARD_PHY_REG_TEST2         (MV88W8XX8_USB_BASE+0x320)
#define  CARD_PHY_RESERVE           (MV88W8XX8_USB_BASE+0x324)
#define  CARD_USB_INT               (MV88W8XX8_USB_BASE+0x328)
#define  CARD_DBG_CTL               (MV88W8XX8_USB_BASE+0x32C)

/*   USB PHY Register Default Value */
#define  INIT_USB_CMD               0x00080000
#define  INIT_USB_CTL               0x0
#define  INIT_PHY_PLL_CTL           0x1A3
#define  INIT_PHY_TX_CTL            0x107C3
#define  INIT_PHY_RX_CTL            0x7D5D0185
#define  INIT_PHY_IVREF_CTL         0x635
#define  INIT_PHY_REG_TEST0         0xC09A00
#define  INIT_PHY_REG_TEST1         0x00000000
#define  INIT_PHY_REG_TEST2         0x0
#define  INIT_PHY_RESERVE           0x0
#define  INIT_USB_INT               0x0
#define  INIT_DBG_CTL               0x9


#define REG_UTMI_SELECT           8
#define REG_OPMODE                6  /* 7:6 */
#define REG_TERM_SELECT           5
#define REG_XCVR_SELECT           4
#define UDU2_TEST_MODE            2  /* 3:2 */
#define REG_BIT_PU_PLL            1
#define REG_BIT_PU                0
#define REG_TEST_HS_HDL_SYNC      31
#define REG_TEST_FS_HDL_SYNC      30
#define REG_TEST_DIG_LPBK         20
#define REG_TEST_SKIP             17 /* 19:17 */
#define REG_TEST_FLAG             16
#define REG_TEST_DONE             15
#define REG_TEST_LENGTH           13 /* 14:13 */
#define REG_TEST_TX_PATTERN       5  /* 12:5 */
#define REG_TEST_MODE             3  /* 4:3 */
#define REG_TEST_LPBK_EN          2
#define REG_TEST_BYPASS           1
#define REG_TEST_EN               0
#define REG_LINE_INT              3
#define REG_LINE_INT_EN           1
#define REG_CORE_INT              0

/* BITs in Bridge Interrupt Cause and Mask registers */
#define MV_USB_ADDR_DECODE_ERROR_BIT        0
#define MV_USB_ADDR_DECODE_ERROR_MASK       (1<<MV_USB_ADDR_DECODE_ERROR_BIT)

/* BITs in Windows 0-3 Control and Base Registers */
#define MV_USB_WIN_ENABLE_BIT               0
#define MV_USB_WIN_ENABLE_MASK              (1<<MV_USB_WIN_ENABLE_BIT)

#define MV_USB_WIN_TARGET_OFFSET            4
#define MV_USB_WIN_TARGET_MASK              (0xF<<MV_USB_WIN_TARGET_OFFSET)

#define MV_USB_WIN_ATTR_OFFSET              8
#define MV_USB_WIN_ATTR_MASK                (0xFF<<MV_USB_WIN_ATTR_OFFSET)

#define MV_USB_WIN_SIZE_OFFSET              16
#define MV_USB_WIN_SIZE_MASK                (0xFFFF<<MV_USB_WIN_SIZE_OFFSET)

#define MV_USB_WIN_BASE_OFFSET              16
#define MV_USB_WIN_BASE_MASK                (0xFFFF<<MV_USB_WIN_BASE_OFFSET)

/*******************************************/
/* ARM USB PHY Registers                   */
/*******************************************/
#define MV_USB_PHY_POWER_CTRL_REG           (MV88W8XX8_USB_BASE + 0x400)
#define MV_USB_PHY_PLL_CTRL_REG             (MV88W8XX8_USB_BASE + 0x410)
#define MV_USB_PHY_TX_CTRL_REG              (MV88W8XX8_USB_BASE + 0x420)
#define MV_USB_PHY_RX_CTRL_REG              (MV88W8XX8_USB_BASE + 0x430)
#define MV_USB_PHY_IVREF_CTRL_REG           (MV88W8XX8_USB_BASE + 0x440)
#define MV_USB_PHY_TEST_GROUP_CTRL_REG_0    (MV88W8XX8_USB_BASE + 0x450)
#define MV_USB_PHY_TEST_GROUP_CTRL_REG_1    (MV88W8XX8_USB_BASE + 0x454)
#define MV_USB_PHY_STATUS_REG               (MV88W8XX8_USB_BASE + 0x460)
#define MV_USB_PHY_FIFO_REG                 (MV88W8XX8_USB_BASE + 0x470)


/*******************************************/
/* DMA Registers                           */
/*******************************************/
#define MV88W8XX8_DMA_BASE    0x8000E000

#define MV_DMA_BYTE_COUNT_REG   0x800 /* DMA Byte Count register */
#define MV_DMA_SRC_ADDR_REG   0x810 /* DMA Source Address register */
#define MV_DMA_DEST_ADDR_REG    0x820 /* DMA Destination Address register */
#define MV_DMA_NEXT_DESC_PTR_REG  0x830 /* DMA Next Record Pointer register */
#define MV_DMA_CTRL_REG     0x840 /* DMA control register */
#define MV_DMA_CURRENT_DESC_PTR_REG 0x870 /* DMA  Current Descriptor Pointer register */
#define MV_DMA_INT_MASK_REG   0x880 /* DMA interrupt Mask */
#define MV_DMA_INT_SRC_REG    0x8a0 /* DMA interrupt Source */
#define DMA_ARBITER_CTRL_REG    0x860 /* DMA Arbiter Control register */

/****************************************/
/*  Keypad Controller Registers */
/****************************************/
#define MV88W8XX8_KPC_BASE    (0x8000D400)

#define MV88W8XX8_KPC_CTRL0     (MV88W8XX8_KPC_BASE + 0x00)
#define MV88W8XX8_KPC_CTRL1     (MV88W8XX8_KPC_BASE + 0x04)
#define MV88W8XX8_KPC_DIRECT_CTRL0  (MV88W8XX8_KPC_BASE + 0x08)
#define MV88W8XX8_KPC_DIRECT_CTRL1  (MV88W8XX8_KPC_BASE + 0x0c)
#define MV88W8XX8_KPC_MATRIX_INPUT0   (MV88W8XX8_KPC_BASE + 0x10)
#define MV88W8XX8_KPC_MATRIX_INPUT1 (MV88W8XX8_KPC_BASE + 0x14)
#define MV88W8XX8_KPC_ROW_DATA_0  (MV88W8XX8_KPC_BASE + 0x18)
#define MV88W8XX8_KPC_ROW_DATA_1  (MV88W8XX8_KPC_BASE + 0x1c)
#define MV88W8XX8_KPC_ROW_DATA_2  (MV88W8XX8_KPC_BASE + 0x20)
#define MV88W8XX8_KPC_ROW_DATA_3  (MV88W8XX8_KPC_BASE + 0x24)
#define MV88W8XX8_KPC_ROW_DATA_4  (MV88W8XX8_KPC_BASE + 0x28)
#define MV88W8XX8_KPC_ROW_DATA_5  (MV88W8XX8_KPC_BASE + 0x2c)
#define MV88W8XX8_KPC_ROW_DATA_6  (MV88W8XX8_KPC_BASE + 0x30)
#define MV88W8XX8_KPC_ROW_DATA_7  (MV88W8XX8_KPC_BASE + 0x34)
#define MV88W8XX8_KPC_DEBOUNCE    (MV88W8XX8_KPC_BASE + 0x38)
/* Key Pad CONTROL REG0 MASK */
#define MV88W8XX8_KPC_ROTARY_KEY_0_ENABLE_MASK    (0x00000004)
#define MV88W8XX8_KPC_ROTARY_KEY_1_ENABLE_MASK    (0x00000002)
#define MV88W8XX8_KPC_MATRIX_KEY_PAD_ENABLE_MASK  (0x00000008)
#define MV88W8XX8_KPC_MATRIX_INT_ENABLE_MASK    (0x00000020)
#define MV88W8XX8_KPC_MATRIX_INT_STATUS_MASK    (0x00000080)  /*ROC*/
#define MV88W8XX8_KPC_DIRECT_KEY_PAD_ENABLE_MASK  (0x00000001)
#define MV88W8XX8_KPC_DIRECT_INT_ENABLE_MASK    (0x00000010)
#define MV88W8XX8_KPC_DIRECT_INT_STATUS_MASK    (0x00000040)  /*ROC*/
#define MV88W8XX8_KPC_IGNORE_MULTIPLE_KEY_PRESSES_MASK  (0x00000100)
#define MV88W8XX8_KPC_AUTO_SCAN_ON_DETECTION_MASK (0x00000200)
#define MV88W8XX8_KPC_AUTO_SCAN_MODE_MASK   (0x00000400)

/* Key Pad CONTROL REG1 MASK */
#define MV88W8XX8_KPC_NUMBER_OF_ROWS_MASK   (0x00000007)
#define MV88W8XX8_KPC_NUMBER_OF_COLUMNS_MASK    (0x00000038)
#define MV88W8XX8_KPC_MATRIX_SCAN_OUTPUT_VALUES_MASK  (0x0000ff00)

/* Key pad bits of CTRL1 */
#define MV88W8XX8_KPC_COL_MASK        (0xff << 8)
#define MV88W8XX8_KPC_MS7       (1 << 15)
#define MV88W8XX8_KPC_MS6       (1 << 14)
#define MV88W8XX8_KPC_MS5       (1 << 13)
#define MV88W8XX8_KPC_MS4       (1 << 12)
#define MV88W8XX8_KPC_MS3       (1 << 11)
#define MV88W8XX8_KPC_MS2       (1 << 10)
#define MV88W8XX8_KPC_MS1       (1 << 9)
#define MV88W8XX8_KPC_MS0       (1 << 8)
#define MV88W8XX8_KPC_MKCN_MASK       (7 << 3)
#define MV88W8XX8_KPC_MKCN        (5 << 3)
#define MV88W8XX8_KPC_MKRN_MASK       (7 << 0)
#define MV88W8XX8_KPC_MKRN        (5 << 0)

/* Key pad bits of INPUT0 */
#define MV88W8XX8_KPC_MKP       (1 << 8)
#define MV88W8XX8_KPC_ROW_MASK        (0xff << 0)
#define MV88W8XX8_KPC_MR7       (1 << 7)
#define MV88W8XX8_KPC_MR6       (1 << 6)
#define MV88W8XX8_KPC_MR5       (1 << 5)
#define MV88W8XX8_KPC_MR4       (1 << 4)
#define MV88W8XX8_KPC_MR3       (1 << 3)
#define MV88W8XX8_KPC_MR2       (1 << 2)
#define MV88W8XX8_KPC_MR1       (1 << 1)
#define MV88W8XX8_KPC_MR0       (1 << 0)

/* Matrix Key Pad Input REG1 MASK*/
#define MV88W8XX8_KPC_VALID_DATA_MASK     (0x00000100)  /*ROC*/

/*
---------- FLASH ----------
*/
#define MV88W8XX8_OLD_LINUX_JFFS_PARTITION   0
#define MV88W8XX8_OLD_UBOOT_PARTITION        1

#define MV88W8618_HS31_FLASH_BASE       0xFFC00000
#define MV88W8618_HS31_FLASH_SIZE       0x400000
#define MV88W8618_HS31_FLASH_UBOOT_PLUS_ENV_SIZE   0x40000 /* 256K */

#define MV88W8618_FLASH_BASE            0xFFC00000
#define MV88W8618_FLASH_SIZE            0x400000
#define MV88W8618_FLASH_UBOOT_PLUS_ENV_SIZE   0x40000 /* 256K */
#define MV88W8618_FLASH_RES_SIZE_OLD    0x100000

#define MV88W8638_FLASH_BASE            0xFF800000
#define MV88W8638_FLASH_SIZE            0x800000
#define MV88W8638_FLASH_UBOOT_PLUS_ENV_SIZE   0x40000 /* 256K */
#define MV88W8638_FLASH_LINUX_CFG_SIZE        0x10000 /* 384K */
/* #define MV88W8638_FLASH_LINUX_FS_SIZE              the rest */

#define MANHATTAN_FLASH_BASE            0xFFC00000
#define MANHATTAN_FLASH_SIZE            0x400000
#define MANHATTAN_FLASH_UBOOT_PLUS_ENV_SIZE   0x40000 /* 256K */
#define MANHATTAN_FLASH_LINUX_CFG_SIZE        0x10000 /* 384K */
#define MANHATTAN_FLASH_HIST_SIZE	      0x10000 /*  64K */
/* #define MANHATTAN_FLASH_LINUX_FS_SIZE              the rest */

#if defined(CONFIG_ARCH_MV88W8638) || defined(CONFIG_ARCH_MANHATTAN)
#define BOARD_HAS_LINUX_CONF_PARTITION
#endif

#endif  /* _ASM_ARCH_HARDWARE_H */


