/*******************************************************************************
** File          : $HeadURL$
** Author        : $Author$
** Project       : HSCTRL
** Instances     :
** Creation date :
********************************************************************************
********************************************************************************
** ChipIdea Microelectronica - IPCS
** TECMAIA, Rua Eng. Frederico Ulrich, n 2650
** 4470-920 MOREIRA MAIA
** Portugal
** Tel: +351 229471010
** Fax: +351 229471011
** e_mail: chipidea.com
********************************************************************************
** ISO 9001:2000 - Certified Company
** (C) 2005 Copyright Chipidea(R)
** Chipidea(R) - Microelectronica, S.A. reserves the right to make changes to
** the information contained herein without notice. No liability shall be
** incurred as a result of its use or application.
********************************************************************************
** Modification history:
** $Date$
** $Revision$
*******************************************************************************
*** Description:
***   This file contains the VUSB functions specific to ARM processor.
***
**************************************************************************
**END*********************************************************/
#include "usbPal.h"
//#include "stdlib.h"

/* macros */


typedef struct
{
    int     dataSize;
    void    *pNext;
    void    *pPrev;
    uint_32   data[1];
} PoolRecord;


// PLEASE NOTE: see allocations in _usb_dci_vusb20_init(),
//              _usb_device_init(), _usb_device_register_service()
//              & service registrations in xllpCI2SWInit()
//              to insure proper memory pool sizing
//-------------------------------------------------------------------
#define INT_POOL_SIZE  (1024*4)   /* ... WAS 100k buffer ... now 4k */

uint_32 gUsbMemPool[INT_POOL_SIZE/sizeof(uint_32)];
static uint_32 INT_POOL_C_VIRTUAL = (uint_32)gUsbMemPool;
static PoolRecord *firstRecord;
static void* poolPtr = (void*)gUsbMemPool;



/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _bsp_get_usb_capability_register_base
*  Returned Value : Address of the VUSB1.1 capability register base
*  Comments       :
*        Get the USB capability register base address
*END*-----------------------------------------------------------------*/

pointer _bsp_get_usb_capability_register_base
   (
      /* [IN] the device number */
      uint_8 device_number
   )
{ /* Body */
      return ((pointer)(USB_CORE_CAP_LENGTH_REG));

} /* EndBody */

void K_initDynamicPool()
{
    PoolRecord *poolRecord = (PoolRecord *)poolPtr;
    poolRecord->pNext = poolRecord->pPrev = NULL;
    poolRecord->dataSize = INT_POOL_C_VIRTUAL+INT_POOL_SIZE - ((uint_32) poolPtr) - sizeof(PoolRecord) + sizeof(uint_32);
    firstRecord = poolRecord;
}

void *K_malloc(int size)
{
    uint_32 cpsr;
    PoolRecord *poolRecord;

    // round size up to multiple of 4 bytes
    //-------------------------------------
    //size = 4*((size + 3)/4);
    size = ((size + 3) >> 2) << 2;  //faster

    cpsr = disableInterrupts();

    poolRecord= firstRecord;

    while(poolRecord!=NULL)
    {
        if(poolRecord->dataSize >= size)
        {
            //We decided to give poolRecord up to the user.

            //Check if we have enough room for a new free record with at least sizeof(uint_32) free
            if(poolRecord->dataSize >= (size + sizeof(PoolRecord)) )
            {
                //We position a new Free Record after the allocated one.
                PoolRecord *pCurrent = poolRecord;

                //Position the poolRecord pointer that we are going to give up to the user
                poolRecord = (PoolRecord *) ((uint_32)pCurrent->data + pCurrent->dataSize - size - sizeof(PoolRecord) + sizeof(uint_32));

                //We calculate the remaining size for the free Record
                pCurrent->dataSize -= size + sizeof(PoolRecord) - sizeof(uint_32);
                
                //We give the poolRecord the desired size
                poolRecord->dataSize = size;
            }
            //We don't have enough space for a new Free Record
            else
            {
                if(poolRecord->pPrev!=NULL)
                    ((PoolRecord *)(poolRecord->pPrev))->pNext = poolRecord->pNext;
                if(poolRecord->pNext!=NULL)
                    ((PoolRecord *)(poolRecord->pNext))->pPrev = poolRecord->pPrev;

                if(poolRecord==firstRecord)
                {
                    if(poolRecord->pNext!=NULL)
                        firstRecord = poolRecord->pNext;
                    else
                        firstRecord = NULL;
                }
            }

            break;
        }
        poolRecord = poolRecord->pNext;
    };

    restoreInterrupts(cpsr);

    if(poolRecord!=NULL)
        return poolRecord->data;
    return NULL;
}

void K_free(void *block)
{
    if(((uint_32)block>=(uint_32)poolPtr) &&
       ((uint_32)block< INT_POOL_C_VIRTUAL+INT_POOL_SIZE))
    {
        uint_32 cpsr;
        PoolRecord *poolRecord,*pPrev,*pNext;
        cpsr = disableInterrupts();

        poolRecord = (PoolRecord *) (((uint_32)block)-sizeof(PoolRecord)+sizeof(uint_32));

        if(firstRecord==NULL)
        {
            firstRecord = poolRecord;
            poolRecord->pNext = poolRecord->pPrev = NULL;
            restoreInterrupts(cpsr);
            return;
        }

        pPrev = firstRecord;

        if((uint_32)pPrev > (uint_32)poolRecord)
        {
            pNext = pPrev;
            pPrev = NULL;
            firstRecord = poolRecord;
        }
        else
        {
            if(pPrev->pNext!=NULL)
            {
                while(pPrev!=NULL)
                {

                    if(pPrev->pNext > poolRecord)
                    {
                        pNext = pPrev->pNext;
                        break;
                    }

                    if(pPrev->pNext!=NULL)
                    {
                        pPrev = pPrev->pNext;
                    }
                    else
                    {
                        pNext = NULL;
                        break;
                    }
                }
            }
            else
                pNext = NULL;
        }

        poolRecord->pPrev = pPrev;
        poolRecord->pNext = pNext;

        if(pNext!=NULL)
        {
            pNext->pPrev = poolRecord;
        }

        if(pPrev!=NULL)
        {
            pPrev->pNext = poolRecord;
        }


        if(poolRecord->pPrev==NULL)
            poolRecord = poolRecord->pNext;

        if( (poolRecord!=NULL) &&
            (poolRecord->pPrev!=NULL))
        {
            //Check if previous is consecutive
            //If it is, weld them together
            while(
                ((uint_32)(((PoolRecord *)(poolRecord->pPrev))->data) +
                ((PoolRecord *)(poolRecord->pPrev))->dataSize) ==
                (uint_32)poolRecord)
            {
                if(poolRecord->pNext!=NULL)
                    ((PoolRecord *)(poolRecord->pNext))->pPrev = poolRecord->pPrev;

                ((PoolRecord *)(poolRecord->pPrev))->dataSize+=poolRecord->dataSize + sizeof(PoolRecord) - sizeof(uint_32);

                ((PoolRecord *)(poolRecord->pPrev))->pNext = poolRecord->pNext;

                if(poolRecord->pNext!=NULL)
                    poolRecord = poolRecord->pNext;
                else
                    break;
            }
        }

        restoreInterrupts(cpsr);
    }
    else
    {
       // set a debugging error bit
    }
}


//--------------------------------------------------------
// USB_MEM_ALIGN
//     Take a memory pointer and round it up to the next
//     alignment.
//
//     Make sure that the initial allocation is one
//     alignment block larger than you require so that
//     we dont have buffer clashes, memory aborts, etc.
//
//     Do not call this directly to avoid mis-specified
//     alignSize variable.  Instead use macros defined in
//     usbpal.h
//--------------------------------------------------------
uint_32 USB_MEM_ALIGN(  uint_32 memLoc, uint_32 alignSize)
{
   uint_32 mask;

   mask = alignSize - 1;

   if( memLoc & mask )
   {
       // alignment not kosher
       // realign - wastes minimal memory
       //--------------------------------
       memLoc += (alignSize-(memLoc & mask));
   }

   return memLoc;
}


/* EOF */

