#include "mvUsbDevApi.h"
#include "mvUsbDevPrv.h"
#include "mvUsbDefs.h"
#include "mvUsbCh9.h"               // for REQ_TYPE_X
#include "xllp_ciu2d.h"
#include "mvUsbDebug.h"
#if defined MV_USB_TRACE_PRINT
    #include "uart.h"
#endif



USB_DEV_STATE_STRUCT_PTR   gCI2Handle;             // should point to USB_DEV_STATE_STRUCT
static P_XLLP_U2D_EP_CONFIG_TABLE_T gEPCfgTbl;     // table with all required endpoints and their attributes.


#define MAX_RECV_FIFO_LEN      (4*1024)
uint_8 ci2recvBufPtr[MAX_RECV_FIFO_LEN];



#ifndef CI2_GETMSG_INSTALLED_AS_SERVICE
  static uint_8 RECV_FLAG = 0;
  static uint_32 RECV_COUNT = 0;

#endif

void xllpCI2ControlProcessIdle(void* handle, uint_8  type, boolean  setup, uint_8 direction, uint_8_ptr buffer, uint_32 length, uint_8  error);
void xllpCI2ControlReset(void* handle, uint_8  type, boolean  setup, uint_8 direction, uint_8_ptr buffer, uint_32 length, uint_8  error);
void xllpCI2TransactionComplete(void* handle, uint_8  type, boolean  setup, uint_8 direction, uint_8_ptr buffer, uint_32 length, uint_8  error);






extern USB_DEV_STATE_STRUCT ci2Device;



//------------------------------------

// CI2_ResetBufPosition()

// call this whenever transfers are

// cancelled and before priming a new

// transfer

//------------------------------------

static  UINT32 ci2_double_buf_pos = 0;

void CI2_ResetBufPosition( XLLP_U2D_EP_T  usbEndPtId )

{

  _usb_device_handle  lHandle = gCI2Handle ;

  uint_8 error;



  ci2_double_buf_pos = 0;



#ifndef CI2_GETMSG_INSTALLED_AS_SERVICE

  RECV_FLAG = 0;

#endif



  error = _usb_device_recv_data ( lHandle, usbEndPtId,  ci2recvBufPtr, MAX_RECV_FIFO_LEN);

  if (error != USB_OK)
  {

    MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in _usb_device_recv_data, error=%d\n", __FUNCTION__, (int)error);

  }



}



//------------------------------------

//

//

//------------------------------------
void CI2InterruptHandler()
{
  _usb_dci_vusb20_isr(gCI2Handle);
}


void CI2Enable ( void )
{
  _usb_dci_vusb20_enable(gCI2Handle);
}

//--------------------------------------------------------------------
// xllpCI2HWShutdown()

//

//   Shut down the CI2 peripheral operation

//

//--------------------------------------------------------------------

XLLP_STATUS_T xllpCI2HWShutdown()
{
    XLLP_STATUS_T retError = XLLP_TRUE;


    // This unregisters services
    // cancels any transfers in progress
    // and shuts down the CI2 device

    //-----------------------------------

    _usb_device_shutdown(gCI2Handle);

    return retError;
}


//--------------------------------------------------------------------------------------------------

//  xllpCI2SWInit()

//

//

//--------------------------------------------------------------------------------------------------
XLLP_UINT32_T xllpCI2SWInit (P_XLLP_U2D_EP_CONFIG_TABLE_T pEPCfgTbl, void (*pCPUCI2IntEnable)(void), void (*pPlaformReady)(void), UINT32 optionFlags)
{
    UINT_T error, i, epNum, epDir, epType;
    int   lockKey;

    gEPCfgTbl = pEPCfgTbl;

    gCI2Handle = &ci2Device;

#if defined MV_USB_TRACE_PRINT
   setup_uart( 1, BAUDRATE_115200 );
#endif



    // Init the driver context struct NOW

    //------------------------------------------------------

    memset( &ci2Device, 0x00, sizeof(USB_DEV_STATE_STRUCT) );



    // Init the driver memory pool sub-system
    // service structs and other alloc's

    // come from pool

    //--------------------------------------

    K_initDynamicPool();



    //------------------------------------------------

    // services should be registered before peripheral

    // is "connected" to USB transport bus we put the

    // EP2 transaction complete service at the head of

    // the service list to reduce search time for

    // this particular service

    //------------------------------------------------



#ifdef CI2_GETMSG_INSTALLED_AS_SERVICE

    // resgister the EP2 rx function with the driver

    //------------------------------------------------

    error = _usb_device_register_service(gCI2Handle, MV_USB_SERVICE_EP2, CI2GetMsg);

    if (error != USB_OK)

    {

        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in registration of EP2 transaction handler, error=%d\n", __FUNCTION__, (int)error);

        error = GeneralError;

        goto CI2INITFail;

    }

#else
    error = _usb_device_register_service(gCI2Handle, MV_USB_SERVICE_EP2, xllpCI2TransactionComplete);
    if (error != USB_OK)
    {
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in registration for transaction complete, error=%d\n", __FUNCTION__, (int)error);
    }
#endif


    // register the Ep0 control endpoint function

    //-------------------------------------------

    error = _usb_device_register_service(gCI2Handle, MV_USB_SERVICE_EP0, xllpCI2ControlProcessIdle);

    if (error != USB_OK)

    {

        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: registration of EP0 transaction handler failed, error=%d\n", __FUNCTION__, (int)error);

        error = GeneralError;

        goto CI2INITFail;

    }



    // resgister the "reset" function with the driver

    //------------------------------------------------

    error = _usb_device_register_service(gCI2Handle, MV_USB_SERVICE_BUS_RESET, xllpCI2ControlReset);

    if (error != USB_OK)

    {

        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: registration of USB Bus Reset handler failed, error=%d\n", __FUNCTION__, (int)error);

        error = GeneralError;

        goto CI2INITFail;

    }



    lockKey = USB_lock();


    // This call brings up the entire CI2 peripheral
    // including memory allocation, device reset,

    // and determining device base address and offsets

    //-------------------------------------------------

    error = _usb_device_init(0, (_usb_device_handle *)(&gCI2Handle), pCPUCI2IntEnable, pPlaformReady, optionFlags);
    if (error != USB_OK)
    {
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: device init failed, error=%d\n", __FUNCTION__, (int)error);
        error = GeneralError;

        goto CI2INITFail;

    }




     /* This is a Self-Powered Device so set device status to show that. */

    _usb_device_set_status(gCI2Handle, MV_USB_STATUS_DEVICE, MV_USB_SELF_POWERED);

    USB_unlock(lockKey);
CI2INITFail:
   return error;

}


//-------------------------------------------------------------------------------

// xllpCI2Xfer

//

//

//-------------------------------------------------------------------------------

XLLP_UINT8_T xllpCI2Xfer ( XLLP_U2D_EP_T    usbEndPtId, // USB Endpoint

                           XLLP_USB_EP_DIR_T    epXferDir,

                           P_XLLP_UINT8_T       bufPtr,     // Memory buffer
                           XLLP_UINT32_T        bufSize,    // Buffer Size
                           XLLP_INT32_T         *actualCount )       // actual # bytes received
{
    uint_8 error = NoError;
   _usb_device_handle  lHandle = gCI2Handle ;



    *actualCount = 0;



#ifdef CI2_GETMSG_INSTALLED_AS_SERVICE

    switch( xllpCI2GetEndPointTransferDirection ( usbEndPtId ) )

    {

      case USB_IN:

           error = _usb_device_send_data (lHandle, usbEndPtId, bufPtr, bufSize);

           if (error != USB_OK)

           {

               MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in _usb_device_send_data, error=%d\n", __FUNCTION__, (int)error);

           }



           break;



  #ifndef CI2_GETMSG_INSTALLED_AS_SERVICE

      case USB_OUT:

           MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING, "xllpCI2Xfer, RECV_FLAG=%d, RECV_COUNT=%d, bufSize=%d\n", RECV_FLAG, RECV_COUNT, bufSize);

           if( RECV_FLAG && (bufSize > 0) && RECV_COUNT )

           {

               bufSize = (bufSize <= RECV_COUNT) ? bufSize : RECV_COUNT;

               memcpy (bufPtr, (ci2recvBufPtr+ci2_double_buf_pos), bufSize);

               *actualCount = bufSize;

               ci2_double_buf_pos += bufSize;

               if ( bufSize < RECV_COUNT)
                  RECV_COUNT -= bufSize;
               else
                 RECV_FLAG = 0;

           }

           else

           {

               error = UsbRxError;
           }



           break;

  #else

      case USB_OUT:

           MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING, "xllpCI2Xfer, RECV_FLAG=%d, RECV_COUNT=%d, bufSize=%d\n", RECV_FLAG, RECV_COUNT, bufSize);

           {

               memcpy (bufPtr, (ci2recvBufPtr+ci2_double_buf_pos), bufSize);

               *actualCount = bufSize;

               ci2_double_buf_pos += bufSize;

           }



           break;

  #endif

    }

#else



#if 1   // DCB RETURN! old reference code for entire func, remove after testing

    if ( xllpCI2GetEndPointTransferDirection ( usbEndPtId ) ==  USB_IN )

    {

        error = _usb_device_send_data (lHandle, usbEndPtId, bufPtr, bufSize);

        if (error != USB_OK)

        {

            MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in _usb_device_send_data, error=%d\n", __FUNCTION__, (int)error);

        }

    }

    else if ( (xllpCI2GetEndPointTransferDirection ( usbEndPtId ) ==  USB_OUT) )

    {

        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING, "xllpCI2Xfer, RECV_FLAG=%d, RECV_COUNT=%d, bufSize=%d\n", RECV_FLAG, RECV_COUNT, bufSize);

        if ( RECV_FLAG && (bufSize > 0) && (bufSize <= RECV_COUNT) )

        {

            memcpy (bufPtr, (ci2recvBufPtr+ci2_double_buf_pos), bufSize);

            *actualCount = bufSize;

            ci2_double_buf_pos += bufSize;

            if ( bufSize < RECV_COUNT)
                RECV_COUNT -= bufSize;
            else
                RECV_FLAG = 0;

        }

        else

        {

           error = UsbRxError;
        }

    }

#endif  // #if 1

#endif // CI2 not installed as service
    return error;
}


#ifndef CI2_GETMSG_INSTALLED_AS_SERVICE

XLLP_UINT8_T xllpCI2EPInterruptReceived ( XLLP_U2D_EP_T   usbEndPtId )
{

    return RECV_FLAG ;

}



//----------------------------------------------------------------------------------------------

XLLP_INT32_T    xllpCI2GetTransferLength (XLLP_U2D_EP_T epNum, XLLP_USB_EP_DIR_T direction)
{
    return RECV_COUNT;
}
#endif



//------------------------------------------
//

//

//

//------------------------------------------

XLLP_BOOL_T xllpIsEnumCompleted  ()
{
    USB_DEV_STATE_STRUCT_PTR lHandle = gCI2Handle;

    return ((lHandle->USB_STATE == MV_USB_STATE_CONFIG) ? XLLP_TRUE :  XLLP_FALSE);
}


//------------------------------------------

//

//

//

//------------------------------------------
XLLP_UINT32_T  xllpCI2GetEndPointTransferDirection (XLLP_U2D_EP_T endPtId)
{
    uint_8 i = 1;
    while( (gEPCfgTbl[i].usbEndpointNum) != 0)
    {
        if (gEPCfgTbl[i].udcEndpointNum == endPtId)
            return gEPCfgTbl[i].usbEndpointDirection ;

        i++;
    }


    return 0xFFFFFFFF;    /* TO DO CHANGE IT TO INVALID VALUE ENUM*/          /* USB_IN */
}




//------------------------------------------

//

//

//

//------------------------------------------

void xllpCI2InterruptEnable()

{

    USB_DEV_STATE_STRUCT_PTR lHandle = gCI2Handle;



    (lHandle->DEV_PTR)->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR &= EHCI_INTR_INT_EN;

}



//------------------------------------------

//

//

//

//------------------------------------------
void xllpCI2InterruptDisable()

{

    USB_DEV_STATE_STRUCT_PTR lHandle = gCI2Handle;



    (lHandle->DEV_PTR)->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR &= ~EHCI_INTR_INT_EN;

}



//------------------------------------------

//

//

//

//------------------------------------------


void xllpCI2ControlProcessIdle(void* handle, uint_8  type, boolean  setup, uint_8 direction, uint_8_ptr buffer, uint_32 length, uint_8  error)
{

   static SETUP_STRUCT ci_otg_setupPacket;


   if ( setup )
   {
        _usb_device_read_setup_data(handle, MV_USB_SERVICE_EP0, (uint_8*)&ci_otg_setupPacket);
   }
   switch (ci_otg_setupPacket.REQUESTTYPE & 0x60)
   {
         case REQ_TYPE_STANDARD:
             switch (ci_otg_setupPacket.REQUEST)
             {
                case REQ_GET_STATUS:
                   mvUsbCh9GetStatus(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_CLEAR_FEATURE:
                   mvUsbCh9ClearFeature(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_SET_FEATURE:
                   mvUsbCh9SetFeature(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_SET_ADDRESS:
                   mvUsbCh9SetAddress(handle, setup, &ci_otg_setupPacket);

                   break;

                case REQ_GET_DESCRIPTOR:
                   mvUsbCh9GetDescriptior(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_SET_DESCRIPTOR:
                   mvUsbCh9SetDescriptior(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_GET_CONFIGURATION:
                   mvUsbCh9GetConfig(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_SET_CONFIGURATION:
                   mvUsbCh9SetConfig(handle, setup, &ci_otg_setupPacket);

                   break;

                case REQ_GET_INTERFACE:
                   mvUsbCh9GetInterface(handle, setup, &ci_otg_setupPacket);
                   break;

                case REQ_SET_INTERFACE:
                   mvUsbCh9SetInterface(handle, setup, &ci_otg_setupPacket);
                   break;

 /*            // No Synch frame handler

                case REQ_SYNCH_FRAME:
                   ch9SynchFrame(handle, setup, &ci_otg_setupPacket);
                   break;    */


 /* // disable stall!
                default:
                   _usb_device_stall_endpoint(handle, 0, 0);
                   break;
 */
             } /* Endswitch */

         break;

      case REQ_TYPE_CLASS:
         /* class specific request */
         ch9Class(handle, setup, &ci_otg_setupPacket);
         return;

      case REQ_TYPE_VENDOR:
         /* vendor specific request can be handled here*/

         mvUsbCh9ProcessVendorRequest( handle, setup, &ci_otg_setupPacket);
         break;


      //default:  //Disable stall
      //   _usb_device_stall_endpoint(handle, 0, 0);
      //   break;

   } /* Endswitch */

   return;
}


#ifndef CI2_GETMSG_INSTALLED_AS_SERVICE

  void xllpCI2TransactionComplete(void* handle, uint_8  type, boolean  setup, uint_8 direction, uint_8_ptr buffer, uint_32 length, uint_8  error)
  {
    uint_8 errCode = 0;
    if ( direction == MV_USB_RECV && (error == USB_OK)) // OUT
    {
        RECV_FLAG = 1;
        RECV_COUNT = length;
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_WARNING, "OUT size=%ld\n", length);
    }
  }

#endif








//----------------------------------------------------------------

// This routine is installed as a service to the Chip Idea LLD

// and called by the default reset handler.

//

// cancel transfers on all endpoints

//----------------------------------------------------------------
void xllpCI2ControlReset(void* handle, uint_8  type, boolean  setup, uint_8 direction, uint_8_ptr buffer, uint_32 length, uint_8  error)
{

    uint_8 i;

    USB_DEV_STATE_STRUCT_PTR dev_ptr;



    dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;



    // It's a reset signal which is from low-level USB signal not EP0...

    // cancel transfers on all endpoints

    //-----------------------------------------

    for( i=0; i < dev_ptr->MAX_ENDPOINTS; i++ )

    {

      _usb_device_cancel_transfer(handle, i, MV_USB_SEND);
      _usb_device_cancel_transfer(handle, i, MV_USB_RECV);

    }


    /* Restart Control Endpoint #0 */
    mv_usb_start_ep0(handle);
}


//------------------------------------

//

//

//------------------------------------
uint_8 mv_usb_start_ep0(void* handle)
{
    uint_8 error = USB_OK;
    error = _usb_device_init_endpoint(handle, 0, gEPCfgTbl[0].maxPacketSize, MV_USB_SEND,  MV_USB_CONTROL_ENDPOINT, 0);
    if (error != USB_OK)
    {
        MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in _usb_device_init_endpoint SEND, error=%d\n", __FUNCTION__, (int)error);
    }
    else
    {
        error = _usb_device_init_endpoint(handle, 0, gEPCfgTbl[0].maxPacketSize, MV_USB_RECV, MV_USB_CONTROL_ENDPOINT, 0);
        if (error != USB_OK)
        {
            MV_USB_DEBUG_TRACE(MV_USB_DEBUG_FLAG_ERROR, "%s: failed in _usb_device_init_endpoint RECV, error=%d\n", __FUNCTION__, (int)error);
        }
    }
    return error;
}

