/******************************************************************************
**	(C)Copyright Marvell. All Rights Reserved. 2007 
**
**  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MARVELL.
**	The copyright notice above does not evidence any actual or intended  
**  publication of such source code.	
**
**  This Module contains Proprietary Information of Marvell and should be treated 
**  as Confidential.
**	  
**  The information in this file is provided for the exclusive use of the licensees 
**  of Marvell.	
**	Such users have the right to use, modify, and incorporate this code into products 
**  for purposes authorized by the license agreement provided they include this notice 
**  and the associated copyright notice with any such product. 
**  The information in this file is provided "AS IS" without warranty.
**
**  FILENAME: FlexOneNAND_Cmd.c
**
**  PURPOSE:  The driver for Flex One NAND device.
**
******************************************************************************/
#include "FlexOneNAND_Cmd.h"
#include "OneNAND_Driver.h"
#include "errors.h"

UINT_T FlexOneNAND_HotReset();
extern void memset(void *Addr, unsigned char Val, unsigned long Size) ;
extern P_FlexOneNAND_Properties_T GetFlexOneNANDProperties(void);

extern P_FLEX_REGISTERS_T pFlexRegs;
/*
 * LoadFlexPage:  
 *	This function uses the Load command to retrive
 *	up to one full page of data into the Flex OneNAND part
 *
 * Inputs:
 *	FlashOffset - location in flash to burn data
 *	buffer - pointer to location of data to be written
 *	size - number of bytes to be written from buffer
 *			
 *		Note: address + size CANNOT span multiple pages
 *			This MUST be handled in upper level driver
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Timeout Error
 *			Read Error
 */
UINT_T LoadFlexPage(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, PageNum, SectorCount, PageMask, ByteMask, SectorMask, pre_bytes, post_bytes,BufferRAMSectorAddr,SectorAddr;
	UINT_T ER[8];
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
	VUINT16_T *pDataRAM, *pBuff;
	int i;
	UINT_T EccFlag = ECC_ON_FLAG; //Turn ECC flag OFF for now. 
	UINT16_T device_select = 0;

	//Setup Pointers	
	pDataRAM = (volatile unsigned short *) FLEX_DATARAM_MAIN_BASE;
	pBuff = (volatile unsigned short *) buffer;

	//Initial Calculations:
	PageMask = (pFlashProp->BlockSize - pFlexProp->PageSize);
	SectorMask = pFlexProp->PageSize - (pFlexProp->PageSize >> 2);
	ByteMask = (pFlexProp->PageSize >> 2)- 1;
	
	//Determine the block we are on
	BlockNum = FlashOffset >> pFlexProp->BlockSizeDivShift;
	//Determine the page we are on
	PageNum	= (FlashOffset & PageMask) >> pFlexProp->PageSizeDivShift;
	//Determine pre-bytes for this sector
	pre_bytes = FlashOffset & ByteMask;
	//Determine left over bytes (fill out to end of last sector)
	post_bytes = (pFlexProp->PageSize - size - pre_bytes) & ByteMask;
	//Determine how many sectors we are reading (total size / sector size)
	//SectorCount = (pre_bytes + size + post_bytes) >> (pFlexProp->PageSizeDivShift - 4);
	// 8 sectors/page
	SectorCount = 8;
	
	if (FlashOffset >> 10)
		device_select = FLEX_SA2R_DBS_FLAG;

    pFlexRegs->SA1 = (device_select) | (BlockNum & FLEX_SA1R_FBA_MASK);

	pFlexRegs->SA2 = device_select;

	if (PageNum >> 6)
		pFlexRegs->SA8 = (PageNum << 2) | 0x0;
	else
		pFlexRegs->SA8 = ((PageNum & FLEX_SA8R_FPA_MASK) << 2) | 0x0;

	pFlexRegs->SBR = (UINT16_T) (SBR_WRITE_DEFAULT) | (SectorCount & FLEX_SBR_BSC_MASK);	

			
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
	pFlexRegs->COMMAND =FLEX_LOAD_PAGE_INTO_BUFFER;

    if( WaitForCommandCompletion(PEND_READ, (UINT32)ONENAND_TIMEOUT_MS*3 )!= NoError)
    {
       return ReadError;
    }

	// Read ECC Status Registers
	ER[0] = pFlexRegs->ECCSR1 & FLEX_ECCSR_ER_MASK;
	ER[1] = ( pFlexRegs->ECCSR1 >> 8 ) & FLEX_ECCSR_ER_MASK;

	ER[2] = pFlexRegs->ECCSR2 & FLEX_ECCSR_ER_MASK;
	ER[3] = ( pFlexRegs->ECCSR2 >> 8 ) & FLEX_ECCSR_ER_MASK;

	ER[4] = pFlexRegs->ECCSR3 & FLEX_ECCSR_ER_MASK;
	ER[5] = ( pFlexRegs->ECCSR3 >> 8 ) & FLEX_ECCSR_ER_MASK;

	ER[6] = pFlexRegs->ECCSR4 & FLEX_ECCSR_ER_MASK;
	ER[7] = ( pFlexRegs->ECCSR4 >> 8 ) & FLEX_ECCSR_ER_MASK;

	// Host reads data from DataRAM
	for (i=0;i<size;i+=2)
	{
		*pBuff++ = *pDataRAM++; 
	}

	return Retval;
}

UINT_T SuperLoadFlexPage()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LSBPageRecoveryRead()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T PIUpdate()
{
	UINT_T Retval = NoError;
	return Retval;
}

/*
 * ProgramFlexPage:  
 *	This function uses the Program command to program
 *	one full page of data into the OneNAND part
 *
 * Inputs:
 *	address - location in flash to burn data
 *	buffer - pointer to location of data to be written
 *	size - number of bytes to be written from buffer
 *			
 *		Note: address + size CANNOT span multiple pages
 *			This MUST be handled in upper level driver
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Flash is Locked
 *			Timeout Error
 *			Program Error
 */
UINT_T ProgramFlexPage(UINT_T address, UINT_T buffer, UINT_T size, FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, PageNum, PageMask, ByteMask, pre_bytes, post_bytes;
	VUINT16_T *pDataRAM, *pBuff;
	UINT16_T device_select = 0;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	//Initial Calculations:
	//Determine the block we are on
	BlockNum = address >> pFlexProp->BlockSizeDivShift;
	//Determine the page we are on
	PageMask = (pFlashProp->BlockSize - pFlexProp->PageSize);
	PageNum	= (address & PageMask) >> pFlexProp->PageSizeDivShift;
	//Determine pre-bytes for this page
	ByteMask = (pFlexProp->PageSize - 1);
	pre_bytes = (address & ByteMask);
	//Determine left over bytes for this page
	post_bytes = pFlexProp->PageSize - size - pre_bytes;

	//Setup Pointers	
	pDataRAM = (volatile unsigned short *) FLEX_DATARAM_MAIN_BASE;
	pBuff = (volatile unsigned short *) buffer;

	//Step 1:
	//Set DBS bit in Start Address 2 register
	if (address >> 10)
		device_select = FLEX_SA2R_DBS_FLAG;

	pFlexRegs->SA2 = device_select;
			   
	//Step 2: Load data into BufferRAM
	//Fill any initial values with 1's
	for( ; pre_bytes > 0 ; pre_bytes -= 2 )
		*pDataRAM++ = 0xFFFF;
	
	//Write the data 16 bits at a time
	for( ; size > 0 ; size -= 2 )
		*pDataRAM++ = *pBuff++;

	//If there is any leftover space in the page, fill it with 1's
	for( ; post_bytes > 0 ; post_bytes -= 2 )
		*pDataRAM++ = 0xFFFF;

	//Step 3:
	//Fill out Start Address 1 Register 
	pFlexRegs->SA1 = (device_select) | (BlockNum & FLEX_SA1R_FBA_MASK);

	//Step 4:
	//Read Write Protection Status
	if (pFlexRegs->WPS & FLEX_WPSR_UNLOCKED != FLEX_WPSR_UNLOCKED)
		return FlashLockError;

	//Step 5:
	//Write Page and Sector Addres (must be 0) to Start Address 8
	pFlexRegs->SA8 = (PageNum << 2)| 0x0;

	//Step 6:
	//Fill out Start Buffer Register info (must be 0x0800 for Program)
	pFlexRegs->SBR = (UINT16_T) SBR_WRITE_DEFAULT;

	//Step 7:
	//Set System Config Register - leave at default 
	//uncomment next line to turn of ECC		
	//pFlexRegs->SYSCONF1 = CONF1_WRITE_DEFAULT | CONF1_ECC_OFF; //Turn ECC OFF for now

	//Step 8:
	//Clear Interrupt Register
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
	
	//Step 9:
	//Write PROGRAM Cmd to command register
	pFlexRegs->COMMAND = FLEX_PROGRAM_PAGE_FROM_BUFFER;

	//Step 10:
	//Wait for WRITE interrupt
	Retval = WaitForCommandCompletion(PEND_WRITE, (UINT32)ONENAND_TIMEOUT_MS );
    if (Retval != NoError)
		return Retval;

	//Step 11:
	//Read Controller status register
	if ((pFlexRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = ProgramError;

	return Retval;
}


UINT_T FinishCacheProgram()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T CacheProgram()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T UnlockFlexArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LockFlexArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T LockTightFlexArray()
{
	UINT_T Retval = NoError;
	return Retval;
}

/*
 * UnlockAllFlexBlocks:  
 *	This function uses the Unlock command prior to
 *  writing data into the Flex OneNAND part.
 *
 * Inputs:
 *	None.
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			General Error
 *			Timeout Error
 */
UINT_T UnlockAllFlexBlocks()
{
	UINT_T Retval = NoError;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	//Step 1: Write DFS to Start Address 1 Register
	pFlexRegs->SA1 = pFlexProp->MuxType << 15;

	//Step 2: Set DBS for Start Address 2 Register
	pFlexRegs->SA2 = pFlexProp->MuxType << 15;

	//Step 3: Write SBA to SBA Register
	pFlexRegs->SBA = 0;

	//Step 4: Clear Interrupt Register
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
	
	//Step 5: Write all block unlock command
	pFlexRegs->COMMAND = FLEX_ALL_BLOCK_UNLOCK;

	//Step 6: Wait for Interrupt
	Retval = WaitForCommandCompletion(INT_STAT_REG, (UINT32)ONENAND_TIMEOUT_MS*3 );
    if (Retval != NoError)
		return Retval;

	//Step 7: Read Controller Status Register
	if ((pFlexRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = GeneralError;
		
	return Retval;
}

/*
 * FlexBlockErase:  
 *	This function uses the Block Erase command to
 *  erase a Flex OneNAND part prior to writing data.
 *
 * Inputs:
 *	None.
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Timeout Error
 *			Erase Error
 */
UINT_T FlexBlockErase( UINT32 blockNumber )
{
	UINT_T Retval;
    FLEX_SA1_REG sa1reg;
    
    //First unlock Blocks
    //-------------------
    UnlockAllFlexBlocks();
    
    // setup the FBA in SA1
    // bits0-9 = blockNumber
    // bit 15 = blocks #'s above 1023
    //-------------------------------
    if( blockNumber > 1023 )
    {
      sa1reg.bits.FLEX_DFS = 1;
      sa1reg.bits.FLEX_RESERVED = 0;
      sa1reg.bits.FLEX_FBA = blockNumber-1024;
    }
    else
    {
      sa1reg.bits.FLEX_DFS = 0;
      sa1reg.bits.FLEX_RESERVED = 0;
      sa1reg.bits.FLEX_FBA = blockNumber;   
    }
      
    //Write FBA & DFS
    //----------------------------
    pFlexRegs->SA1 = sa1reg.value;

	//Step 4:
	//Read Write Protection Status
	if (pFlexRegs->WPS & FLEX_WPSR_UNLOCKED != FLEX_WPSR_UNLOCKED)
		return FlashLockError;
    
    //set DBS in SA2
    //----------------
    if(blockNumber > 1023)
      pFlexRegs->SA2 = ( 1 << 15);
    else
      pFlexRegs->SA2 = 0;
    
	//Clear Interrupt Register
    //ensure write complete
    //------------------------
	pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
    sa1reg.value = pFlexRegs->INTRPT;
    
    //Write the command
    pFlexRegs->COMMAND = FLEX_BLOCK_ERASE;
    
    //Wait for operation complete
    //---------------------------
	Retval = WaitForCommandCompletion(PEND_ERASE, (UINT32)ONENAND_TIMEOUT_MS*3 );
    if (Retval != NoError)
		return EraseError;

	//Read Controller Status Register
	if ((pFlexRegs->CNTSTAT & FLEX_CSR_ERROR) == FLEX_CSR_ERROR)
		Retval = EraseError;
        
	return Retval;
}

UINT_T FlexEraseSuspend()
{
	UINT_T Retval = NoError;
	return Retval;
}

UINT_T FlexEraseResume()
{
	UINT_T Retval = NoError;
	return Retval;
}

/*
 * ResetFlexNANDCore:  
 *	This function uses the Reset command to
 *  reset the Flex One NAND Core.
 *
 * Inputs:
 *	None.
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Timeout Error
 *			General Error
 */
UINT_T ResetFlexNANDCore()
{
	UINT_T Retval = NoError;
	UINT16 IntStatReg = 0;
	int count = 0;

	do{
		 // This will return if there's no FlexOneNAND device connected.
		 if(count > INT_CLEAR_RETRY_CNT)
		 {
		   return GeneralError;
		 }
		// clear interrupt register
		pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
		IntStatReg = pFlexRegs->INTRPT;
		count++;
	// This checks for interrupt status register bit 15(INT)
	}while (IntStatReg & INT_STAT_REG);	
	 
	pFlexRegs->COMMAND = FLEX_NAND_CORE_RESET;

	if( WaitForCommandCompletion(INT_STAT_REG_RSTI_PEND, (UINT32)RESET_TIMEOUT_MS )!= NoError)
	{
	   return GeneralError;
	}

	return Retval;
}

/*
 * ResetFlexOneNAND:  
 *	This function uses the Reset command to
 *  reset the Flex One NAND device.
 *
 * Inputs:
 *	None.
 *
 * Outputs:
 *	Return value - returns NoError on success
 *		Failures can be:
 *			Timeout Error
 *			General Error
 */
UINT_T ResetFlexOneNAND()
{
	UINT_T Retval = NoError;
	UINT16 IntStatReg = 0;
	int count = 0;

	do{
		 // This will return if there's no FlexOneNAND device connected.
		 if(count > INT_CLEAR_RETRY_CNT)
		 {
		   return GeneralError;
		 }
		// clear interrupt register
		pFlexRegs->INTRPT = INT_STAT_REG_CLEAR;
		IntStatReg = pFlexRegs->INTRPT;
		count++;
	// This checks for interrupt status register bit 15(INT)
	}while (IntStatReg & INT_STAT_REG);	
	 
	pFlexRegs->COMMAND = FLEX_MUX_ONENAND_RESET;

	if( WaitForCommandCompletion(INT_STAT_REG_RSTI_PEND, (UINT32)RESET_TIMEOUT_MS )!= NoError)
	{
	   return GeneralError;
	}

	return Retval;

}