/******************************************************************************
**	(C)Copyright Marvell. All Rights Reserved. 2007 
**
**  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MARVELL.
**	The copyright notice above does not evidence any actual or intended  
**  publication of such source code.	
**
**  This Module contains Proprietary Information of Marvell and should be treated 
**  as Confidential.
**	  
**  The information in this file is provided for the exclusive use of the licensees 
**  of Marvell.	
**	Such users have the right to use, modify, and incorporate this code into products 
**  for purposes authorized by the license agreement provided they include this notice 
**  and the associated copyright notice with any such product. 
**  The information in this file is provided "AS IS" without warranty.
**
**  FILENAME: OneNAND_Driver.c
**
**  PURPOSE:  The driver interface for One NAND and Flex devices.
**
******************************************************************************/
#include "OneNAND_Driver.h"
#include "FlexOneNAND_Cmd.h"
#include "OneNAND_Cmd.h"

P_FLEX_REGISTERS_T pFlexRegs = (P_FLEX_REGISTERS_T)FLEX_ONENAND_REG_BASE;
P_ONE_NAND_REGISTERS_T pOneNANDRegs = (P_ONE_NAND_REGISTERS_T)FLEX_ONENAND_REG_BASE;

extern UINT_T ResetOneNAND();
extern UINT_T ResetFlexOneNAND();
extern UINT_T LoadSector(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, P_FlashProperties_T pFlashProp);
extern UINT_T LoadFlexPage(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, P_FlashProperties_T pFlashProp);
extern UINT32 ComputeDivisorShiftFromPowerOf2( UINT32 power );

FlexOneNAND_Properties_T FlexOneNAND_Prop;	// Only need one

P_FlexOneNAND_Properties_T GetFlexOneNANDProperties(void)
{
	return &FlexOneNAND_Prop;
}

//-----------------------------------------------------------------------
// InitializeFlexOneNANDDevice()
//
//     Determines the type of flash device. Resets if device is unresponsive.
//
//     RETURNS 0 if all went well
//-----------------------------------------------------------------------
UINT_T InitializeFlexOneNANDDevice( pFUSE_SET pFuses, FlashBootType_T FlashBootType, FlashProperties_T *pFlashProp )
{

  UINT16 FlexManID=0, OneNANDManID=0, DevID = 0, DevType, DevDensity;
  XLLP_BOOL_T deviceFound = XLLP_FALSE;
  UINT_T Retval = NoError;
  int i;

    //Initialize Flash properties
  P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
  
  //Setup the OneNAND chip select
  ChipSelectOneNAND();

  // Read manufacturer ID for Flex and OneNAND devices.
  FlexManID = pFlexRegs->MANFID;
  OneNANDManID = pOneNANDRegs->MANFID;

  if (FlexManID == 0x00EC || OneNANDManID == 0x00EC)  
  {
	deviceFound = XLLP_TRUE;
	DevID = pFlexRegs->DEVID;
	pFlashProp->FlashType = OneNAND;
	DevType = DevID >> 8;
	if ( DevType == 2){
		pFlexProp->DeviceType = FLEX_ONE_NAND;
		Retval = ResetFlexOneNAND();
		}	
	else if ( DevType == 0 ){
		pFlexProp->DeviceType = ONE_NAND;
		Retval = ResetOneNAND();
		}
  }

	if (deviceFound == XLLP_FALSE)
	{

		Retval = ResetFlexOneNAND();
		if ( Retval == NoError)
		{
			deviceFound = XLLP_TRUE;
			DevID = pFlexRegs->DEVID;
			pFlexProp->DeviceType = FLEX_ONE_NAND;
		}
		else
		{
		Retval= ResetOneNAND();
			if ( Retval == NoError)
			{
				deviceFound = XLLP_TRUE;
				DevID = pOneNANDRegs->DEVID;
				pFlexProp->DeviceType = ONE_NAND;
			}
		}	
	}
	if (deviceFound == XLLP_TRUE)
	{	
			DevID = pFlexRegs->DEVID;
			DevType = DevID >> 8;
			DevDensity = (DevID >> 4) & 0xF;
			if (DevID & 0x8)
				pFlexProp->MuxType = DUAL_CHIP;
			else
				pFlexProp->MuxType = SINGLE_CHIP; 
			switch(DevType){
			case 0:
				pFlashProp->BlockSize = 128*KBYTES; 
				pFlashProp->NAND_Prop.PageSize = 2*KBYTES;
				pFlexProp->PageSize = 2*KBYTES;
			break;
			case 2:
				pFlashProp->BlockSize = 256*KBYTES;
				pFlashProp->NAND_Prop.PageSize = 4*KBYTES;
				pFlexProp->PageSize = 4*KBYTES; 

			break;
			}
			pFlexProp->BlockSizeDivShift = ComputeDivisorShiftFromPowerOf2( pFlashProp->BlockSize );
			pFlexProp->PageSizeDivShift = ComputeDivisorShiftFromPowerOf2( pFlexProp->PageSize );
			pFlexProp->NumOfBlocks = ((16 * MEGABYTES) << DevDensity) >> pFlexProp->BlockSizeDivShift;

	}

	if (!deviceFound)
	{
		return FlashDriverInitError;
	}

	Retval = GetBBT( BOOT_FLASH, pFlashProp);

  return NoError;
}

//-----------------------------------------------------------------------
// WaitForCommandCompletion()
//
//     Waits for an interrupt from the device after a command has been issued.
//
//     RETURNS 0 for no error.
//-----------------------------------------------------------------------
UINT_T WaitForCommandCompletion( UINT16 eventGroup, UINT32 TimeOutMillisec )
{
  UINT32 startTime;
  UINT_T Retval = NoError;
  UINT16 IntStatReg;

  startTime = GetOSCR0();

  do
  {
	if (pFlexRegs->INTRPT == eventGroup ) 
     return Retval;

  }while( OSCR0IntervalInMilli(startTime, GetOSCR0()) < TimeOutMillisec );

  return TimeOutError;
}

//-----------------------------------------------------------------------
// Read_Flash()
//
//     Reads a page at a time from the appropriate device based on device type.
//
//     RETURNS 0 for no error.
//-----------------------------------------------------------------------
UINT_T Read_Flash(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	UINT_T pageAligned;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
	UINT_T start_addr = FlashOffset;
	UINT_T BlockNum, PageMask, ByteMask, address, read_size;

	PageMask = pFlashProp->BlockSize - pFlexProp->PageSize;
	ByteMask = pFlexProp->PageSize - 1;
	do{
		//Determine Block - Check Relo Table
		BlockNum = FlashOffset >> pFlexProp->BlockSizeDivShift;
		BlockNum = IsBlockRelocated(BlockNum, &pFlexProp->ReloTable);
		
		//adjust the offset to account for relocated block
		address = (BlockNum * pFlashProp->BlockSize) + (FlashOffset & (PageMask | ByteMask));

		//calculate how much to write from buffer
		//  note: this normally should be a page size,
		//		Exceptions:
		//			- Start writing in the middle of a page
		//			- End writing in the middle of a page
		read_size = pFlexProp->PageSize - (address & ByteMask);
		read_size = read_size > size ? size : read_size;

		//Write the page
		
		if(pFlexProp->DeviceType == ONE_NAND)
			Retval = LoadSector(address, buffer, read_size, fbt, pFlashProp);
		else
			Retval = LoadFlexPage(address, buffer, read_size, fbt, pFlashProp);
		
		//Check and handle errors
		switch(Retval){
		case NoError:
			FlashOffset += read_size;
			size -= read_size;
			buffer+= read_size;
			break;
		case ReadDisturbError:
			//TBD
			break;
		case TimeOutError:
		default:
			//Reset device
			if(pFlexProp->DeviceType == FLEX_ONE_NAND)
				Retval = ResetFlexOneNAND();
			else
				Retval = ResetOneNAND();
			// No need to loop around again. Return after the reset.
			return Retval; 
			break;
		}
	}while ((size > 0) && (Retval == NoError));

	return Retval;
}

//-----------------------------------------------------------------------
// Write_Flash()
//
//     Writes a page at a time to the appropriate device based on device type.
//
//     RETURNS 0 for no error.
//-----------------------------------------------------------------------
UINT_T Write_Flash(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	UINT_T start_addr = FlashOffset;
	UINT_T BlockNum, PageMask, ByteMask, address, write_size;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
	UINT_T *pBuffer = (UINT_T *) buffer;
	int i, j=0;
 
	PageMask = pFlashProp->BlockSize - pFlexProp->PageSize;
	ByteMask = pFlexProp->PageSize - 1;
	do{
		//Determine Block - Check Relo Table
		BlockNum = FlashOffset >> pFlexProp->BlockSizeDivShift;
		BlockNum = IsBlockRelocated(BlockNum, &pFlexProp->ReloTable);
		
		//adjust the offset to account for relocated block
		address = (BlockNum * pFlashProp->BlockSize) + (FlashOffset & (PageMask | ByteMask));

		//calculate how much to write from buffer
		//  note: this normally should be a page size,
		//		Exceptions:
		//			- Start writing in the middle of a page
		//			- End writing in the middle of a page
		write_size = pFlexProp->PageSize - (address & ByteMask);
		write_size = write_size > size ? size : write_size;

		//Write the page
		
		if(pFlexProp->DeviceType == ONE_NAND)
			Retval = ProgramSector(address, buffer, write_size, fbt, pFlashProp);
		else
			Retval = ProgramFlexPage(address, buffer, write_size, fbt, pFlashProp);
		
		//Check and handle errors
		switch(Retval){
		case NoError:
			FlashOffset += write_size;
			size -= write_size;
			buffer += write_size;
			break;
		case ProgramError:
			//Relocate Block?
			break;
		case FlashLockError:
			//Unlock Device
			if(pFlexProp->DeviceType == ONE_NAND)
				Retval = UnlockNANDArray();
			else
				Retval = UnlockAllFlexBlocks();
			break;
		case TimeOutError:
		default:
			//Reset device
			if(pFlexProp->DeviceType == FLEX_ONE_NAND)
				Retval = ResetFlexOneNAND();
			else
				Retval = ResetOneNAND();
			// No need to loop around again. Return after the reset.
			return Retval; 
			break;

		}
	}while ((size > 0) && (Retval == NoError));

	return Retval;
}

//-----------------------------------------------------------------------
// Erase_Flash()
//
//     Erases the appropriate device based on device type.
//
//     RETURNS 0 for no error.
//-----------------------------------------------------------------------
UINT_T Erase_Flash(UINT_T FlashOffset, UINT_T size, FlashBootType_T fbt)
{
  UINT_T retVal=NoError, blockNum, relocBlk, blockCount;
  P_FlexOneNAND_Properties_T pDeviceProp;
  
  //Get Device properties pointer
  //-----------------------------
  pDeviceProp = GetFlexOneNANDProperties();
  
  //Calc starting block
  //Calc Number of blocks
  //Create remainder mask from BlockSize
  //------------------------------------
  blockNum = FlashOffset >> (pDeviceProp->BlockSizeDivShift);
  blockCount =  size >> (pDeviceProp->BlockSizeDivShift);
  blockCount += (size & ((1 << pDeviceProp->BlockSizeDivShift)-1)) ? 1 : 0;
  
  //Erase blocks and relocate if erase fails;
  //until possibly, relocation fails
  //-----------------------------------------
  while( blockCount )
  {    
    //First, see if it's been relocated
    //---------------------------------
    relocBlk = IsBlockRelocated( blockNum, &pDeviceProp->ReloTable );
    
    switch( pDeviceProp->DeviceType )
    {    
      case ONE_NAND:
           retVal = BlockErase( relocBlk );
      break;
      
      case FLEX_ONE_NAND: 
           retVal = FlexBlockErase( relocBlk );
      break;
    }
    
    if(retVal != NoError )
    {
      retVal = FlexRelocateBlock( &relocBlk, &pDeviceProp->ReloTable);
      
      // Check to see if we ran
      // out of relocation blocks
      //-------------------------
      if( retVal != NoError )
        return EraseError;
        
      //loop back to while()
      // to ensure relocated block erases
      //---------------------------------
    }
    else
    {
      //Next block
      //----------
      blockNum++;
      blockCount--;
    }
  
  }
  
  return retVal;
}

UINT_T ReadOTP(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
  UINT_T Retval = NoError;

  return Retval;
}

UINT_T Write_OTP(UINT_T FlashOffset, UINT_T buffer, UINT_T size, FlashBootType_T fbt)
{
  UINT_T Retval = NoError;

  return Retval;
}

UINT_T LockOTP(FlashBootType_T fbt)
{
  UINT_T Retval = NoError;

  return Retval;
}

UINT_T GetBlockSize(FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	UINT16 DevID = 0, DevType;

	DevID = pFlexRegs->DEVID;
	DevType = DevID >> 8;

	switch(DevType){
	case 0:
		pFlashProp->BlockSize = 128*KBYTES; 
	break;
	case 2:
		pFlashProp->BlockSize = 256*KBYTES; 
	break;
	}

	return Retval;
}

UINT_T GetPageSize(FlashBootType_T fbt)
{
	UINT_T Retval = NoError;
	UINT16 DevID = 0, DevType;

	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	DevID = pFlexRegs->DEVID;
	DevType = DevID >> 8;

	switch(DevType){
	case 0: 
		pFlexProp->PageSize = 2*KBYTES;

	break;
	case 2: 
		pFlexProp->PageSize = 4*KBYTES;

	break;
	}

	return Retval;
}

UINT_T GetEraseRegionSize(FlashBootType_T fbt)
{
  UINT_T Retval = NoError;

  return Retval;
}

UINT_T ConvertToLogicalAddr(UINT_T FlashLocation, FlashBootType_T fbt)
{
  UINT_T Retval = NoError;

  return Retval;
}

UINT_T GetBBT(FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval;
	UINT_T BlkSz, PgSz, PagesPerBlk, bbt_sz;
	UINT_T CurrentBlockOffset = 0;
	UINT_T MaxSlot, MinSlot, CurrentSlot, PreviousSlot, NumberSlots;
	Reloc_T Temp_BBT;
	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();

	pFlexProp->BBT_State = Retval = BBT_NOTFOUND;
	BlkSz = pFlashProp->BlockSize;
	PgSz = pFlexProp->PageSize ;
	bbt_sz = sizeof(Reloc_T);
	pFlexProp->NumBBTSlots = (BlkSz - TIMBUFFER) / PgSz;

	//First check to see if a BBT was loaded for MFG blocks
	CurrentBlockOffset = (BlkSz - PgSz);
	Read_Flash(CurrentBlockOffset, (UINT_T)&pFlexProp->ReloTable, bbt_sz, fbt, pFlashProp);

	//if we don't find a BBT, just return
	if(pFlexProp->ReloTable.Header != VALID_HEADER)
		return Retval;

	//else BBT for MFG blocks was found, next check if we ever had any runtime entries
	pFlexProp->BBT_State = BBT_UNCHANGED;
	PagesPerBlk = (BlkSz / PgSz);
	NumberSlots = pFlexProp->NumBBTSlots;
	MinSlot = PagesPerBlk - pFlexProp->NumBBTSlots;
	MaxSlot = PagesPerBlk - 2;//second to last page (MFG BBT in last page)
	// We do a binary search starting at the second to last page
	CurrentSlot = MaxSlot;
	PreviousSlot = MaxSlot + 1;//keep track of last valid slot

	do{
		CurrentBlockOffset = (CurrentSlot * PgSz);
		Retval = Read_Flash(CurrentBlockOffset, (UINT_T)&Temp_BBT, bbt_sz, fbt, pFlashProp);
		if(Retval != NoError)
			break;

		// if we find a good table, overwrite previous table
		if(Temp_BBT.Header == VALID_HEADER)
		{
			memcpy(&pFlexProp->ReloTable, &Temp_BBT, bbt_sz);
			PreviousSlot = CurrentSlot;
			MaxSlot = CurrentSlot - 1;
			NumberSlots = MaxSlot - MinSlot;
			CurrentSlot = (NumberSlots >> 1) + MinSlot;
		}
		else
		{
			//otherwise, we went too far... back up
			MinSlot = CurrentSlot + 1;
			NumberSlots = MaxSlot - MinSlot;
			CurrentSlot = MinSlot + (NumberSlots >> 1);
		}

		//once our slot boundaries intersect, we know we are done
	}while(MaxSlot > MinSlot);

	//update slot info
	pFlexProp->BBT_Slot = PreviousSlot;

	return Retval;
}

UINT_T CalculateBlockNumber(FlashBootType_T fbt, P_FlashProperties_T pFlashProp)
{
	UINT_T Retval = NoError;
	UINT16 DevID = 0, DevDensity;

	P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
	DevID = pFlexRegs->DEVID;

	DevDensity = (DevID >> 4) & 0xF;
	
	pFlexProp->NumOfBlocks = ((128 * MEGABYTES) / pFlashProp->BlockSize) << DevDensity; 


	return Retval;
}
UINT_T IsBlockRelocated(UINT_T BlockNum, Reloc_T *pBBT)
{
  UINT_T Retval = NoError;

	UINT_T i;

	//block 0 is guaranteed by manufacture
	if(BlockNum == 0)
		return 0;

	// Do we have a valid Bad Block Table?
	if (pBBT->Header != VALID_HEADER)
		return BlockNum;

	// Has this block been relocated?	If so we'll need to use the relocated block
	for (i = 0; i < pBBT->NumReloc; i++)
	{
		if (pBBT->Relo[i].From == BlockNum)
		{
			BlockNum = pBBT->Relo[i].To;
			i = 0; 	// Start over in case remapped block is now bad
		}
	}
	return BlockNum;
}
UINT32 FlexRelocateBlock( UINT32 *badBlock, Reloc_T *pBBT )
{
	UINT32 Retval = NoError;

	return Retval;
}

UINT_T CheckReadDisturb(UINT8 *pMBuf, UINT_T noSectors, UINT16* ECCResultsMain)
{
  UINT_T Retval = NoError;
  P_FlexOneNAND_Properties_T pFlexProp = GetFlexOneNANDProperties();
  int i;
  UINT16 ECCStatus, ECCMainMask, ECCResult, ECCResultByte;
  UINT8 ECCResultWord, ECCResultBit;
  UINT8 ChkMask; 

	if ( pFlexProp->DeviceType == ONE_NAND)
	{
		ECCStatus = pOneNANDRegs->ECCSR;
		if ( ECCStatus == 0 )
  			return Retval;

		for (i = 0;i < noSectors;i++)
		{
			ECCMainMask = (0x04 << (i * 4));

			if (pMBuf != NULL)
			{
				if (ECCStatus & ECCMainMask)
				{
					/* 1-Bit error */
					ECCResult = ECCResultsMain[i];

					ECCResultWord = (UINT8)((ECCResult & 0x0ff0)>>4);
					ECCResultBit = (UINT8)(ECCResult & 0xf);

					ECCResultByte = i * 512 + ECCResultWord * 2;

					if (ECCResultBit >= 8)
					{
						ECCResultByte++;
						ECCResultBit -= 8;
					}

					ChkMask = 0x01 << ECCResultBit;

					if ((pMBuf[ECCResultByte] & ChkMask) != 0)
					{
						return ReadDisturbError;
					}
				}
			}
		}
	}
	else
	{
		if ( pFlexRegs->ECCSR1 == 0 ||
			 pFlexRegs->ECCSR2 == 0 ||
			 pFlexRegs->ECCSR3 == 0 ||
			 pFlexRegs->ECCSR4 == 0 )
			return Retval;

		else
		{
			for (i = 0;i < noSectors;i++)
			{
				ECCMainMask = (0x1 << (i * 8));

				if (pMBuf != NULL)
				{
					if ( (pFlexRegs->ECCSR1 & ECCMainMask) ||
						(pFlexRegs->ECCSR2 & ECCMainMask) ||
						(pFlexRegs->ECCSR3 & ECCMainMask) ||
						(pFlexRegs->ECCSR4 & ECCMainMask) )
					{
						/* 1-Bit error */
						ECCResult = ECCResultsMain[i];

						ECCResultWord = (UINT8)((ECCResult & 0x1f00)>>8);
						ECCResultBit = (UINT8)(ECCResult & 0xf);

						ECCResultByte = i * 512 + ECCResultWord * 2;

						if (ECCResultBit >= 8)
						{
							ECCResultByte++;
							ECCResultBit -= 8;
						}

						ChkMask = 0x01 << ECCResultBit;

						if ((pMBuf[ECCResultByte] & ChkMask) != 0)
						{
							return ReadDisturbError;
						}
					}
				}
			}
		}
	}
  return Retval;
}

UINT_T GetPartitionTable()
{
  UINT_T Retval = NoError;

  return Retval;
}









  

