/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Flash.c
**
**  PURPOSE: 	Contain template OEM boot code flash operations
**
******************************************************************************/

#include "flash.h"

#if MDOC_CODE
#include "msys.h"
#endif

#if ONENAND_CODE
#include "FlexOBMInterface.h"
#endif


#if NAND_CODE
/*
* Relocate block and reposition the entry pointers
* Block Number specified by Block will be relocated, BBEntry and pReloEntry will be updated.
*/

UINT_T RelocateBlock(P_FlashProperties_T pFlashP, UINT_T Block, UINT_T *BBEntry, USHORT *pReloEntry)
{
 UINT_T t, BlockRelocatedTo;
 pFlashP->NAND_Prop.BBT_State = BBT_CHANGED;

 // Must avoid situation where BBEntry been re-located to Block... This can happen if both in reserve area
 for (t = 0; t < *pReloEntry; t++)
 	if ((pFlashP->NAND_Prop.pLocalRelotable->Relo[t].To == Block) && (pFlashP->NAND_Prop.pLocalRelotable->Relo[t].From == *BBEntry))
 			(*BBEntry)--;

 // Relocate it
 BlockRelocatedTo = *BBEntry;
 pFlashP->NAND_Prop.pLocalRelotable->Relo[*pReloEntry].To = *BBEntry;
 pFlashP->NAND_Prop.pLocalRelotable->Relo[*pReloEntry].From = Block;
 (*BBEntry)--;
 (*pReloEntry)++;

 return BlockRelocatedTo;
}

/* WriteNANDWithBBM
*  This routine will write data consulting the BBM for each block before writing to it.
*  If an error during write, it will relocate the block and write to another block.
*  NOTE: In case of relocation, existing contents currently not copied!
*
*  - added (mdbrown) - For DKB, we erase a block before writing to it
*		need to make sure we save the BBT
*/

UINT_T WriteNANDWithBBM (UINT_T Address, UINT_T WriteAmount, P_DFC_BUFFER Buffer, P_FlashProperties_T pFlashP)
{
	UINT_T Retval = NoError;
	UINT_T BlockNum, SBlockNum, i;
	UINT_T SA, ThisWriteAmount, Offset;
	UINT_T BBEntry;
	UINT_T ValidBBT = TRUE;
	USHORT *pReloEntry = &(pFlashP->NAND_Prop.pLocalRelotable->NumReloc);
	UINT_T LocalAddr = (UINT_T) Buffer;

	// Is the BBT still valid?  This should never happen?
	if (pFlashP->NAND_Prop.pLocalRelotable->Header != VALID_HEADER)
		ValidBBT = FALSE;
	else
	{
		if (*pReloEntry == 0)
			BBEntry = pFlashP->NumOfBlocks - 1;
		else
			BBEntry = pFlashP->NAND_Prop.pLocalRelotable->Relo[*pReloEntry - 1].To - 1;	// Last Entry left off
	}
	// Figure out the starting Block number
	SBlockNum = (Address / pFlashP->BlockSize);
	Offset = Address - (SBlockNum * pFlashP->BlockSize);
	ThisWriteAmount = ((SBlockNum + 1) * pFlashP->BlockSize) - Address;
	if (ThisWriteAmount > WriteAmount)
		ThisWriteAmount = WriteAmount;

	while (WriteAmount)
	{
		BlockNum = SBlockNum;
		if(ValidBBT){
			// Has this block been relocated?	If so we'll need to use the relocated block
			for (i = 0; i < *pReloEntry; i++)
			{
				if (pFlashP->NAND_Prop.pLocalRelotable->Relo[i].From == BlockNum)
				{
					BlockNum = pFlashP->NAND_Prop.pLocalRelotable->Relo[i].To;
					i = 0; 					// Start over
				}
			}
		}
		// Write out
		while (1)
		{
			SA = (BlockNum * pFlashP->BlockSize);
#if (DKB & FLASH_ERASE)	// For DKB, we must erase the block before we write to it
			Retval = XllpDfcErase(SA, pFlashP);
#endif
			// If there is an error, skip the writing and go right the relocation
			if(Retval == NoError)
			{
				SA += Offset;
				Retval = XllpDfcWrite((P_DFC_BUFFER) LocalAddr, SA, ThisWriteAmount, pFlashP);
			}
#if (DKB)	// OBM and BootROM currently do not relocate blocks, leave it to WTPSP

			if (Retval == NoError)	 // Write was successful
				break;
			if(ValidBBT == FALSE)
				return Retval;

			BlockNum = RelocateBlock(pFlashP, BlockNum, &BBEntry, pReloEntry);
#else
			break;
#endif
		}

		// Update for next Block
		LocalAddr += ThisWriteAmount;
		WriteAmount -= ThisWriteAmount;
		ThisWriteAmount = (pFlashP->BlockSize > WriteAmount) ? WriteAmount : pFlashP->BlockSize;
		Offset = 0;							// Starting at the begining of the next block
		SBlockNum++;
	}
	return Retval;	// Return value
}

/*	This routine will read in data from the DFC and will consult the BBM.
 If ReadAmount spills over to the next consecutive block, then BBM will be used there as well.
*/
UINT_T ReadNANDWithBBM (UINT_T Address, UINT_T ReadAmount, P_DFC_BUFFER Buffer, P_FlashProperties_T pFlashP)
{
  UINT_T Retval = NoError;
  UINT_T BlockNum, SBlockNum, i;
  UINT_T SA, ThisReadAmount;
  INT_T	WriteRet = GeneralError;
  UINT_T BlockCount = 0;
  UINT_T SOffset;
  UINT_T ValidBBT = TRUE;
  USHORT *pReloEntry = &(pFlashP->NAND_Prop.pLocalRelotable->NumReloc);

  // Is the BBT still valid?
  if (pFlashP->NAND_Prop.pLocalRelotable->Header != VALID_HEADER)
  	ValidBBT = FALSE;

  // Find starting block number	and offset
  SBlockNum = (Address / pFlashP->BlockSize);
  SOffset =  Address - (SBlockNum * pFlashP->BlockSize);

  while (ReadAmount)
  {
	BlockNum = SBlockNum + BlockCount;
   	BlockCount++;

	if(ValidBBT){
		// Has this block been relocated?	If so we'll need to use the relocated block
		for (i = 0; i < *pReloEntry; i++)
		{
			if (pFlashP->NAND_Prop.pLocalRelotable->Relo[i].From == BlockNum)
			{
				BlockNum = pFlashP->NAND_Prop.pLocalRelotable->Relo[i].To;
				i = 0; 					// Start over
			}
		}
	}
	SA = (BlockNum * pFlashP->BlockSize) + SOffset;
    SOffset = 0;	   // Next block we start at the begining

    ThisReadAmount =  ((BlockNum + 1) * pFlashP->BlockSize) - SA;
   	if (ThisReadAmount > ReadAmount)
		ThisReadAmount = ReadAmount;

    // Read in to the buffer
    Retval = XllpDfcRead(Buffer, SA, ThisReadAmount, NULL, pFlashP);
    if (Retval != NoError)
		return Retval;

	Buffer = (UINT_T *) (((UINT_T) Buffer) + ThisReadAmount);
    ReadAmount -= ThisReadAmount;
  }	// End of while

  return Retval;
}

#endif //NAND_CODE

#if NOR_CODE

static const unsigned int PRLockRegisterAddress_32[]=
{
    PRLockReg0Addr_32,
    PRLockReg1Addr_32,
    PRLockReg2Addr_32,
    PRLockReg3Addr_32,
    PRLockReg4Addr_32,
    PRLockReg5Addr_32,
    PRLockReg6Addr_32,
    PRLockReg7Addr_32,
    PRLockReg8Addr_32,
    PRLockReg9Addr_32,
    PRLockReg10Addr_32,
    PRLockReg11Addr_32,
    PRLockReg12Addr_32,
    PRLockReg13Addr_32,
    PRLockReg14Addr_32,
    PRLockReg15Addr_32,
	PRLockReg16Addr_32
};

static const unsigned int PRLockRegisterAddress_16[]=
{
    PRLockReg0Addr_16,
    PRLockReg1Addr_16,
    PRLockReg2Addr_16,
    PRLockReg3Addr_16,
    PRLockReg4Addr_16,
    PRLockReg5Addr_16,
    PRLockReg6Addr_16,
    PRLockReg7Addr_16,
    PRLockReg8Addr_16,
    PRLockReg9Addr_16,
    PRLockReg10Addr_16,
    PRLockReg11Addr_16,
    PRLockReg12Addr_16,
    PRLockReg13Addr_16,
    PRLockReg14Addr_16,
    PRLockReg15Addr_16,
	PRLockReg16Addr_16
};

UINT_T ReadProtectionRegisterData(P_FlashProperties_T pFlashP,
    							  ProtectionRegister_T ProtectionRegToRead,
								  UINT16_T *ProtectionRegData)
{
	VUINT_T ProtectionRegAddr;
	UINT_T  ProtectionRegCount;
	UINT_T  EndCount = 0;
	UINT_T	BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	BaseAddr += (BaseAddr == 0) ? pFlashP->BlockSize : 0;

	if (ProtectionRegToRead > ProtectionRegister16)
	{
		return (ReadError);
	}

	if (ProtectionRegToRead == ProtectionRegister0)
	{
		EndCount = 4;
	}
	else
	{
		EndCount = 8;
	}

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_32[ProtectionRegToRead] + BaseAddr;
		*(VUINT_T *)BaseAddr = ReadIdentifierCmd; 	  // Issue read ID command
		for (ProtectionRegCount = 0; ProtectionRegCount < EndCount; ProtectionRegCount++)
		{
			ProtectionRegData[ProtectionRegCount] = *(UINT16_T *)ProtectionRegAddr;
			ProtectionRegAddr   = (ProtectionRegAddr + 4);
		}
    	*(VUINT_T *)BaseAddr = ReadArrayCmd;  	// Restore read array mode
	}
	else
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_16[ProtectionRegToRead] + BaseAddr;
		*(VUINT16_T *)BaseAddr  = ReadIdentifierCmd_16; 	  // Issue read ID command
		for (ProtectionRegCount = 0; ProtectionRegCount < EndCount; ProtectionRegCount++)
		{
			ProtectionRegData[ProtectionRegCount] = *(UINT16_T *)ProtectionRegAddr;
			ProtectionRegAddr   = (ProtectionRegAddr + 2);
		}
    	*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;  	// Restore read array mode
	}
	return (NoError);
}

UINT_T ReadProtectionLockRegister(P_FlashProperties_T pFlashP, UINT16_T *LockRegister0, UINT16_T *LockRegister1)
{
	UINT_T BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	BaseAddr += (BaseAddr == 0) ? pFlashP->BlockSize : 0;

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
	  	*(VUINT_T *)BaseAddr = ReadIdentifierCmd;	  // Issue read identifier command
		*LockRegister0       =  *(VUINT16_T *)(OTPLockRegister0_32 + BaseAddr);

   		*(VUINT_T *)BaseAddr = ReadIdentifierCmd;	  // Issue read identifier command
		*LockRegister1       =  *(VUINT16_T *)(OTPLockRegister1_32 + BaseAddr);

   		*(VUINT_T *)BaseAddr = ReadArrayCmd; 		  // Restore read array mode command
	}
	else
	{
	  	*(VUINT16_T *)BaseAddr = ReadIdentifierCmd_16; // Issue read identifier command
		*LockRegister0       =  *(VUINT16_T *)(OTPLockRegister0_16 + BaseAddr);

   		*(VUINT16_T *)BaseAddr = ReadIdentifierCmd_16; // Issue read identifier command
		*LockRegister1       =  *(VUINT16_T *)(OTPLockRegister1_16 + BaseAddr);

   		*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;	   // Restore read array mode command
	}
	return (NoError);
}


void RestoreFlashDefaultMode(P_FlashProperties_T pFlashP)
{
 	UINT_T BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	BaseAddr += (BaseAddr == 0) ? pFlashP->BlockSize : 0;

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
    	*(VUINT_T *)BaseAddr = ClearStatusRegisterCmd;
    	*(VUINT_T *)BaseAddr = ReadArrayCmd;
    }
    else
	{
    	*(VUINT16_T *)BaseAddr = ClearStatusRegisterCmd_16;
    	*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;
    }
}

/**
 * Verify single word program by reading back programmed word
 **/
UINT_T VerifySingleWordProgram(P_FlashProperties_T pFlashP, VUINT_T WordVerifyAddr, UINT_T WordToVerify)
{
	UINT16_T  Word;
	RestoreFlashDefaultMode(pFlashP);
	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		if (*(VUINT_T *)WordVerifyAddr != WordToVerify)
		{
			return (ProgramError);
		}
	}
	else
	{
		Word = (UINT16_T)(0x0000FFFF & WordToVerify);
		if (*(VUINT16_T *)WordVerifyAddr != Word)
		{
			return (ProgramError);
		}
		Word = (UINT16_T)(WordToVerify >> 16);
		WordVerifyAddr +=2;
		if (*(VUINT16_T *)WordVerifyAddr != Word)
		{
			return (ProgramError);
		}

	}
	return (NoError);
}

/**
 * Program a word into flash with Sibley Support
 **/
UINT_T ProgramSingleWord(P_FlashProperties_T pFlashP, UINT_T WordProgramAddr, UINT_T WordToProgram)
{
    VUINT_T ProgramStatus;
    UINT16_T ProgramData, ProgramData1;

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		// Issue clear status command to clear status register
    	*(VUINT_T *)WordProgramAddr = ClearStatusRegisterCmd;

    	// Issue block unlock setup command.  Done to unlock block, just in case....
    	*(VUINT_T *)WordProgramAddr = UnlockBlockCmd;
    	*(VUINT_T *)WordProgramAddr = UnlockBlockConfirmCmd; // Issue unlock block confirm
     		  							     	  // Ignore optional unlock block status check
    	do
    	{
       		//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
       	 	ProgramStatus   = *(VUINT_T *)WordProgramAddr;
    	} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);

		// Issue clear status command to clear status register
    	*(VUINT_T *)WordProgramAddr = ClearStatusRegisterCmd;

		if (pFlashP->FlashType == TYAX_FLASH) // if Tyax
		{
			// Issue program word command
   			*(VUINT_T *)WordProgramAddr = WordProgramCmd;
		}
		else
		{
			// Issue Sibley program word command
   			*(VUINT_T *)WordProgramAddr = WordProgramCmd_Sibley;
		}

		// Write data to target address
   		*(VUINT_T *)WordProgramAddr = WordToProgram;
    	do
    	{
       		//*WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
       	 	ProgramStatus   = *(VUINT_T *)WordProgramAddr;
    	} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);

    	//*pBlockBaseAddr = ClearStatusRegisterCmd;  // Issue clear status register command
    	*(VUINT_T *)WordProgramAddr = ReadArrayCmd;          	 // Restore read array mode

    	// check for program error
    	if ((ProgramStatus & ProgramErrorStatus) == ProgramErrorStatus)
    	{
        	return (ProgramError);
    	}
	}
	else
	{
		ProgramData  = (UINT16_T)(0x0000FFFF & WordToProgram);
		ProgramData1 = (UINT16_T)(WordToProgram >> 16);

		// Issue clear status command to clear status register
    	*(VUINT16_T *)WordProgramAddr = ClearStatusRegisterCmd_16;

    	// Issue block unlock setup command.  Done to unlock block, just in case....
    	*(VUINT16_T *)WordProgramAddr = UnlockBlockCmd_16;
    	*(VUINT16_T *)WordProgramAddr = UnlockBlockConfirmCmd_16; // Issue unlock block confirm
     		  							     	  // Ignore optional unlock block status check
		// Read status Register
    	do
    	{
       	 	ProgramStatus   = *(VUINT16_T *)WordProgramAddr;
    	} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

		// Issue clear status command to clear status register
    	*(VUINT16_T *)WordProgramAddr = ClearStatusRegisterCmd_16;

		// Issue program word command
		if (pFlashP->FlashType == TYAX_FLASH) // if Tyax
		{
   			*(VUINT16_T *)WordProgramAddr = WordProgramCmd_16;
		}
		else // Sibley
		{
   			*(VUINT16_T *)WordProgramAddr = WordProgramCmd_16_Sibley;
		}

		// Write data to target address
   		*(VUINT16_T *)WordProgramAddr = ProgramData;
    	do
    	{
       	 	ProgramStatus   = *(VUINT16_T *)WordProgramAddr;
    	} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

		// Increment address to program
		WordProgramAddr += 2;

		// Issue clear status command to clear status register
    	*(VUINT16_T *)WordProgramAddr = ClearStatusRegisterCmd_16;

		if (pFlashP->FlashType == TYAX_FLASH) // if Tyax
		{
	   		*(VUINT16_T *)WordProgramAddr = WordProgramCmd_16;
		}
		else // Sibley
		{
	   		*(VUINT16_T *)WordProgramAddr = WordProgramCmd_16_Sibley;
		}

   		*(VUINT16_T *)WordProgramAddr = ProgramData1;
    	do
    	{
       		//*WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
       	 	ProgramStatus   = *(VUINT16_T *)WordProgramAddr;
    	} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

    	*(VUINT16_T *)WordProgramAddr = ReadArrayCmd_16; // Restore read array mode

    	// check for program error
    	if ((ProgramStatus & ProgramErrorStatus_16) == ProgramErrorStatus_16)
    	{
        	return (ProgramError);
    	}
	}
	// Done programming, return success
    return (NoError);
}


/**
 * Determine product identification
 **/
UINT_T DetermineProductID(P_FlashProperties_T pFlashP, UINT_T *ManufacturerCode, UINT_T *DeviceCode)
{
    VUINT_T JunkFlashAddr;
    VUINT_T *FlashAddr;
    UINT_T  TmpDeviceCode;
	VUINT_T BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	BaseAddr += (BaseAddr == 0) ? pFlashP->BlockSize : 0;
	FlashAddr = (VUINT_T *)BaseAddr;
	JunkFlashAddr = BaseAddr;
	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
    	*FlashAddr = ReadIdentifierCmd;		  	  // Issue read identifier command
    	*ManufacturerCode = (VUINT_T)*FlashAddr;  // Manufacturer code
    	*DeviceCode = (VUINT_T)*(FlashAddr + 1);  // Device ID code
    	*FlashAddr = ReadArrayCmd; // Restore read array mode command
    }
    else
    {
    	*(VUINT16_T *)FlashAddr    = ReadIdentifierCmd_16;  // Issue read identifier command
    	*ManufacturerCode = *(VUINT16_T *)FlashAddr;  	  // Manufacturer code
		JunkFlashAddr += 2;
    	TmpDeviceCode = *(VUINT16_T *)JunkFlashAddr;    // Device ID code
    	*(VUINT16_T *)FlashAddr    = ReadArrayCmd_16; // Restore read array mode command
    	TmpDeviceCode = ((TmpDeviceCode << 16) | (0x0000FFFF & TmpDeviceCode));
    	*DeviceCode   = TmpDeviceCode;
    }

	*DeviceCode = (*DeviceCode >> 16);
    return (NoError);
}


UINT_T  FlashMainProgram(P_FlashProperties_T pFlashP,
					  	 FlashCommand_T ReqdAction,
					  	 UINT_T   *InOut1,
					  	 UINT_T   *InOut2)
{
	UINT_T  Retval     = NoError;
	UINT16_T LockReg0, LockReg1;
	UINT_T   ManufacturerCode, DeviceCode;
	ProtectionRegister_T ProtectionReg;

	switch(ReqdAction)
	{
		case ReadFlashIDCmd:
		{
			Retval = DetermineProductID(pFlashP, &ManufacturerCode, &DeviceCode);
	    	if (Retval == NoError)
	    	{
	    		*InOut1 = DeviceCode;
				*InOut2 = ManufacturerCode;
	    	}
    		break;
		}
		case ReadProtectionLockRegCmd:
		{
			Retval  = ReadProtectionLockRegister(pFlashP, &LockReg0, &LockReg1);
			*InOut1 = (UINT_T)LockReg0;
			*InOut2 = (UINT_T)LockReg1;
			break;
		}
		case ReadProtectionRegDataCmd:
		{
			ProtectionReg = (ProtectionRegister_T)*InOut2;
			Retval  = ReadProtectionRegisterData(pFlashP, ProtectionReg, (UINT16_T *)InOut1);
			break;
		}
		default:
		{
			Retval = NotSupportedError;
			break;
		}
	}
	// Restore flash read array mode before exiting function.
    RestoreFlashDefaultMode(pFlashP);

	return (Retval);
}

#endif //NOR_CODE

#if (TRUSTED || BOOTROM)

/******************** WriteWTMStateToFlash *************************************************
* This routine will write the WTM state to areas in flash including back up state
* The routine will write at StartSlotNumber and decrement until it gets to EndSlotNumber.
* If any of the slot Numbers are Slot 0, it will write a static slot appropriately.
*******************************************************************************************/
UINT_T WriteWTMStateToFlash (pFLASH_I pFLASH_I_h, P_FlashProperties_T pFlashP, pSaveState pSaveState_h, UINT_T StartSlotNumber, UINT_T EndSlotNumber)
{
 UINT_T Retval = NoError;
 UINT_T i;
 UINT_T Address = 0;
 UINT_T AddressB = 0;
 UINT16_T count;
 UINT_T	StateSize;
 UINT_T* pWTM_State;
 UINT_T SlotIncrement = SLOTNSIZE;
 UINT_T sectorOffset,partition;

 if (pFlashP->FlashType == NAND_FLASH)
	SlotIncrement = pFlashP->NAND_Prop.PageSize; 		// Override the Slot Increment for NAND Flash
 else
	Address = AddressB = pFlashP->XIP_Prop.BaseAddr;	// Override the Base Address for XIP

 switch (StartSlotNumber)
 {
  case SLOT0:
	Address += pFLASH_I_h->WTMEntryAddr;
 	AddressB += pFLASH_I_h->WTMEntryAddrBack;
	StateSize = sizeof(FixedState); 	// First Cover Fixed State size
	pWTM_State = (UINT_T *) &(pSaveState_h->SlotZero);
	break;
  default:
	Address += (pFLASH_I_h->WTMEntryAddr + SLOTZEROSIZE) + (SlotIncrement * (StartSlotNumber - 1));
 	AddressB += (pFLASH_I_h->WTMEntryAddrBack + SLOTZEROSIZE) + (SlotIncrement * (StartSlotNumber - 1));
	StateSize = sizeof(DynamicState); 	// Cover Dynamic State size
	pWTM_State = (UINT_T *) &(pSaveState_h->SlotN);
	break;
 }

 // For XIP we loop and write out the whole WTM State that's been maintained so far
 switch( pFlashP->FlashType )
 {
#if NOR_CODE
   case TYAX_FLASH:
   case SIBLEY_FLASH:
        for (i=0, count=0; i < StateSize; i+=4)       // Loop 1 word at a a time
        {
	      if (pFlashP->FlashType == TYAX_FLASH)
	      {
		    Retval = ProgramSingleWord(pFlashP, Address + i, pWTM_State[i/4] );
		    CheckErrorReturn(Retval);
		    Retval = ProgramSingleWord(pFlashP, AddressB + i, pWTM_State[i/4] );
		    CheckErrorReturn(Retval);
	      }
	      else if ((pFlashP->FlashType == SIBLEY_FLASH) && (pFlashP->FlashBusWidth == FlashBusWidth16))	// Program 16 bytes for x16 Sibley	(4 Words)
	      {
		    if (count == 4)			// If we programmed 4 words
		    {
			  Address += 16;		// Skip 16 bytes
			  AddressB+= 16;
			  count = 0;
		    }
		    Retval = ProgramSingleWord(pFlashP, Address + i,  pWTM_State[i/4] );
		    CheckErrorReturn(Retval);
		    Retval = ProgramSingleWord(pFlashP, AddressB + i, pWTM_State[i/4] );
		    CheckErrorReturn(Retval);
		    count++;
          }
	      else if ((pFlashP->FlashType == SIBLEY_FLASH) && (pFlashP->FlashBusWidth == FlashBusWidth32))	// Program 32 bytes for x32 Sibley	(8 Words)
	      {
		    if (count == 8)			// If we programmed 8 words
		    {
			  Address += 32;		// Skip 32 bytes
			  AddressB += 32;
			  count = 0;
		    }
		      Retval = ProgramSingleWord(pFlashP, Address + i, pWTM_State[i/4] );
		      CheckErrorReturn(Retval);
		      Retval = ProgramSingleWord(pFlashP, AddressB + i, pWTM_State[i/4] );
		      CheckErrorReturn(Retval);
		      count++;
          }
        }        // End of for loop
   break;
#endif //NOR_CODE

#if NAND_CODE
   case NAND_FLASH:   // NAND
       {
         // Write out to the NAND
         Retval = WriteNANDWithBBM (Address, StateSize, (P_DFC_BUFFER) pWTM_State, pFlashP);
         CheckErrorReturn(Retval);
         Retval = WriteNANDWithBBM (AddressB, StateSize, (P_DFC_BUFFER) pWTM_State, pFlashP);
         CheckErrorReturn(Retval);
       }
    break;
#endif //NAND_CODE

#if ONENAND_CODE
   case OneNAND:
       {
         // Write out to OneNAND
		 Retval = Flex_WriteFlash(Address, StateSize, pWTM_State, pFlashP);
         CheckErrorReturn(Retval);
         Retval = Flex_WriteFlash(AddressB, StateSize, pWTM_State, pFlashP);
         CheckErrorReturn(Retval);
       }
    break;
#endif

#if MDOC_CODE
    case MSYS_DOC_FLASH:  // Write out to MSYS MDOC
       {
         // NOTE: There will be problems if the Addresses do not align to 512 bytes
         //       for m-systems
         partition    = (Address & MDOC_PARTITION_MASK) >> MDOC_PARTITION_SHIFT;
         sectorOffset = (Address & MDOC_SECTOR_MASK) >> MDOC_SECTOR_SHIFT;
         Retval = mDOC_WriteImage((UINT8 *) pWTM_State, sectorOffset, StateSize, partition);
         CheckErrorReturn(Retval);

         partition    = (AddressB & MDOC_PARTITION_MASK) >> MDOC_PARTITION_SHIFT;
         sectorOffset = (AddressB & MDOC_SECTOR_MASK) >> MDOC_SECTOR_SHIFT;
         Retval = mDOC_WriteImage((UINT8 *) pWTM_State, sectorOffset, StateSize, partition);
         CheckErrorReturn(Retval);
       }
    break;
#endif

  default:
    return WriteError;
}
 // Recursive call till we get to EndSlotNumber
 if (StartSlotNumber != EndSlotNumber)
 	Retval = WriteWTMStateToFlash (pFLASH_I_h, pFlashP, pSaveState_h, StartSlotNumber-1, EndSlotNumber);

 return Retval;
}

#endif
