/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd.
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	Flash.c
**
**  PURPOSE: 	Contain template OEM boot code flash operations
**
******************************************************************************/

#include "flash.h"
#include "general.h"
#include "platformconfig.h"
#if MDOC_CODE
#include "msys.h"
#endif

#if ONENAND_CODE
#include "FlexOBMInterface.h"
#endif

	// Used to move data from XIP to ISRAM if needed
static volatile UINT8_T Mem2Mem_Complete = FALSE;
static volatile UINT_T Mem2Mem_Error = NoError;


// ISR for a MEM2MEM descriptor based DMA move
void Mem2Mem_ISR()
{
	UINT_T DCSRValue;
	P_XLLP_DMAC_T pDmac_h = XLLP_Return_PDmacHandle();

	// Get copy of the DCSR
	DCSRValue = pDmac_h->DCSR[MEM2MEM_CHANNEL];

	// Wait for the channel to stop
	while ( (pDmac_h->DCSR[MEM2MEM_CHANNEL] & (1 << XLLP_DMAC_STOP_INT)) == 0)
	{
	}

	// Clear the interrupts source
	XllpDmacClearDmaInterrupt(MEM2MEM_CHANNEL);

	// This set of descriptors completed.
	Mem2Mem_Complete = TRUE;

	// Return if last set failed
	if (Mem2Mem_Error == ReadError)
		return;

	if ((DCSRValue & (1 << XLLP_DMAC_END_INT)) == (1 << XLLP_DMAC_END_INT))
		Mem2Mem_Error = NoError;
	if ((DCSRValue & (1 << XLLP_DMAC_BUS_ERR_INT)) == (1 << XLLP_DMAC_BUS_ERR_INT))
		Mem2Mem_Error = ReadError;

	return;
}

UINT_T ExtractKeyHashFromOTP(UINT16_T *OTPKeyHashData, P_FlashProperties_T pFlashP, OTPHashType HashType)
{
	UINT_T i, Retval;
	UINT_T Protection1RegToRead, Protection2RegToRead;
	UINT16_T ProtectionReg1Data[8], ProtectionReg2Data[8];

	/************ Readinng the OTP's *********************/
	switch( pFlashP->FlashType )
	{
#if NOR_CODE
	case TYAX_FLASH:
	case SIBLEY_FLASH:
		{
			if( HashType == OEMHash)
			{
				Protection1RegToRead = 0;
				Protection2RegToRead = 1;
			}
			else
			{
				Protection1RegToRead = 1;
				Protection2RegToRead = 2;
			}
			Retval = FlashMainProgram(pFlashP,
				ReadProtectionRegDataCmd,
				(UINT_T *)&ProtectionReg1Data[0],
				&Protection1RegToRead);

			Retval = FlashMainProgram(pFlashP,
				ReadProtectionRegDataCmd,
				(UINT_T *)&ProtectionReg2Data[0],
				&Protection2RegToRead);
			if( HashType == OEMHash)
			{
				for (i=0; i<4; i++)
				{
					OTPKeyHashData[i] = ProtectionReg1Data[i];
				}
				for (i=0; i<6; i++)
				{
					OTPKeyHashData[(i+4)] = ProtectionReg2Data[i];
				}
			}
			else
			{
				OTPKeyHashData[0] = ProtectionReg1Data[6];
				OTPKeyHashData[1] = ProtectionReg1Data[7];
				for (i=0; i<8; i++)
				{
					OTPKeyHashData[(i+2)] = ProtectionReg2Data[i];
				}
			}
			break;
		}
#endif //NOR_CODE

#if ONENAND_CODE
	case OneNAND:
		{
			// TBD: Read Otp Area based on hash type here once the new driver is ready.
			/*        if( HashType == OEMHash)
			Retval = ONENAND_ReadOtpArea(0, (UINT8 *)OTPKeyHashData, WordLengthOf_SHA1*4 );
			else
			Retval = ONENAND_ReadOtpArea(WordLengthOf_SHA1*4, (UINT8 *)OTPKeyHashData, WordLengthOf_SHA1*4 );*/
			break;
		}
#endif
#if MDOC_CODE
	case MSYS_DOC_FLASH:  // Write out to MSYS MDOC
		{
			if( HashType == OEMHash)
				Retval = mDOC_ReadOtp((UINT8_T *)OTPKeyHashData, 0, WordLengthOf_SHA1*4 );
			else
				Retval = mDOC_ReadOtp((UINT8_T *)OTPKeyHashData, WordLengthOf_SHA1*4, WordLengthOf_SHA1*4 );
			break;
		}
#endif
	default:
		Retval = NoOTPFound;
	}
	return (Retval);
}

/* ****************** CopyImageFlashToLocal ***************************************************
* This routine will copy an image as selected by ImageType
* The destination and source addresses can be found using the TIM
* The flash information will come from pFlashP, which should be pre-initialized.
*/
UINT_T CopyImageFlashToLocal(UINT_T ImageID, pTIM pTIM_h, P_FlashProperties_T pFlashP, UINT_T *PhysicalJumpAddr )
{
	// Declerations
	UINT_T   	Retval = NoError;
	UINT_T 	Flash_SA;			// Starting Address for flash
	UINT_T 	DDR_A;				// Address for ISRAM or DDR
	UINT_T		ImageSize; 			// Size of image to copy
	UINT_T 	TransferSize = 0;	// Transfer Size of the specific descriptor (XIP only)
	UINT_T		i;
	UINT_T     sectorOffset = 0, partition = 0;
	pIMAGE_INFO imageInfo;

	// Mem2Mem DMA if needed
				 XLLP_DMAC_COMMAND_T LocalCopyDmaCommand;
#if NOR_CODE
	static  __attribute__((aligned(16))) XLLP_DMAC_DESCRIPTOR_T LocalCopyDesc[NumMem2MemDesc];
#endif
	XLLP_DMAC_DESC_ENABLE_T ChanControl;

	// Set up the channel
	XLLP_DMAC_CHANNEL_T LocalMemCopyChannel   = MEM2MEM_CHANNEL;

	// First find the image in the TIM
	imageInfo = FindImageInTIM(pTIM_h, ImageID);
	if(imageInfo == NULL) return ImageNotFound;

	DDR_A = imageInfo->LoadAddr;
	if (PhysicalJumpAddr)
		*PhysicalJumpAddr = imageInfo->LoadAddr;
	ImageSize = imageInfo->ImageSize;
	Flash_SA = imageInfo->FlashEntryAddr;

	//if the load address is in ISRAM, make sure the image will fit in memory
	if((DDR_A >= ISRAM_PHY_ADDR) && (DDR_A < (ISRAM_PHY_ADDR + PlatformISRAMLimit)))
		if((DDR_A + ImageSize) > (ISRAM_PHY_ADDR + PlatformISRAMLimit))
			return WriteError;

 #if (OBM)
	// Prevent User to load an image over where OBM is executing.
	if((DDR_A >= DDR_PHY_ADDR) && (DDR_A < (DDR_PHY_ADDR + OBM_RESTRICTED_DDR_SPACE)))
		return WriteError;
 #endif

	// For XIP flash, Flash address must be combined with BaseAddress
	// Add any new XIP devices supported
	//---------------------------------------------------------------
#if NOR_CODE
	switch( pFlashP->FlashType )
	{
	case TYAX_FLASH:
	case SIBLEY_FLASH:
		Flash_SA += pFlashP->XIP_Prop.BaseAddr;
		break;

	default:
		break;
		// Do nothing for non-XIP
	}
#endif

	switch (pFlashP->FlashType)
	{
#if NOR_CODE
	case TYAX_FLASH:
	case SIBLEY_FLASH:
		{
			//if the load address equals the flash start address no need to copy run from XIP
			if(DDR_A == Flash_SA)
				break;

#ifndef RUN_FROM_SRAM
			// Enable DMA interrupts
			EnablePeripheralIRQInterrupt(DMA_CNTL_INT);
#endif

			// Set up the Command - clear the stack based struct
			//--------------------------------------------------
			memset( &LocalCopyDmaCommand, 0, sizeof(XLLP_DMAC_COMMAND_T));
			LocalCopyDmaCommand.aWidth      = XLLP_DMAC_WIDTH_16;
			LocalCopyDmaCommand.aSize       = XLLP_DMAC_BURSTSIZE_16;
			LocalCopyDmaCommand.aIncTrgAddr = 1;
			LocalCopyDmaCommand.aIncSrcAddr = 1;

			while (ImageSize > 0)
			{
				Mem2Mem_Complete = FALSE;					   // This set of descriptors have not completed
				LocalCopyDmaCommand.aEndIrqEn   = 0;		   // Don't produce an interrupt until the last descriptor
				ChanControl = XLLP_DMAC_DESC_RUN_CHANNEL;	   // Run until last descriptor

				// Configure descriptors for 4K transfers
				for (i=0; (ImageSize > 0) && (i < NumMem2MemDesc - 1); i++)
				{
					Flash_SA += TransferSize;
					DDR_A += TransferSize;
					TransferSize = (ImageSize > 0x1F00) ? 0x1F00 : ImageSize;	 // Transfer about 8KB at a time, descriptor limit 8K-1
					ImageSize -= TransferSize;
					LocalCopyDmaCommand.aLen = TransferSize;

					if ((ImageSize == 0) || (i == NumMem2MemDesc-2))
					{
						LocalCopyDmaCommand.aEndIrqEn   = 1;
						ChanControl = XLLP_DMAC_DESC_STOP_CHANNEL;
					}

					// Fill chained descriptors
					XllpDmacFillLinkedDesc(
						&LocalCopyDesc[i],
						&LocalCopyDesc[i+1],
						ChanControl,
						XLLP_DMAC_DISABLE_DESC_BRANCH,
						Flash_SA,
						DDR_A,
						&LocalCopyDmaCommand
						);
				}

				// Configure for descriptor fetch
				XllpDmacCfgChannelDescTransfer(
					&LocalCopyDesc[0],
					LocalMemCopyChannel,
					XLLP_DMAC_MEM2MEM_MOVE,
					XLLP_DMAC_ALIGNMENT_ON
					);

				XllpDmacStartTransfer(LocalMemCopyChannel);

				// Wait for the ISR...
#ifndef RUN_FROM_SRAM
				while (!Mem2Mem_Complete) {
				}
#else
				Mem2Mem_ISR();
#endif

				if (Mem2Mem_Error != NoError)
				{
					DisablePeripheralIRQInterrupt(DMA_CNTL_INT);
					return Mem2Mem_Error;
				}
			}

			DisablePeripheralIRQInterrupt(DMA_CNTL_INT);
			Retval = Mem2Mem_Error;
			break;
		}
#endif

#if NAND_CODE
	case NAND_FLASH:
		{
			Retval = ReadNANDWithBBM (Flash_SA, ImageSize, (P_DFC_BUFFER) DDR_A, pFlashP);
			break;
		}
#endif

#if MDOC_CODE
	case MSYS_DOC_FLASH:
		{
			partition    = (Flash_SA & MDOC_PARTITION_MASK) >> MDOC_PARTITION_SHIFT;
			sectorOffset = (Flash_SA & MDOC_SECTOR_MASK) >> MDOC_SECTOR_SHIFT;
			Retval = mDOC_GetImage( (unsigned char *)DDR_A, sectorOffset, ImageSize, partition );
			break;
		}
#endif

#if ONENAND_CODE
	case OneNAND:
		{
			Retval = Flex_ReadFlash(Flash_SA, ImageSize, DDR_A, pFlashP);
			break;
		}
#endif

	default:
#if (!NOFLASH_CODE)
		return NotFoundError;
#endif
		break;
	}

	return Retval;
}

//------------------------------------------------------------------------------
// CopyFlashParams
//        Grunt routine saves ROM space
//------------------------------------------------------------------------------
static void CopyFlashParams( P_FlashProperties_T pFlashProp, FlashType_T flashType,
	unsigned int busWidth, unsigned int baseAddr, unsigned int blkSize )
{
	pFlashProp->FlashType         = flashType;
	pFlashProp->FlashBusWidth     = busWidth;
	pFlashProp->XIP_Prop.BaseAddr = baseAddr;
	pFlashProp->BlockSize         = blkSize ;
}


// This routine will configure the flashes accorinding what the fuse settings passed in.
UINT_T Configure_Flashes (pFUSE_SET pFuses, P_FlashProperties_T pFlashProp)
{
	UINT_T   Retval = NoError;

	// Download TIM based on fuse settings
    switch(pFuses->bits.PlatformState){
	case 0x0:
		Retval = InvalidPlatformConfigError;
		break;
#if MDOC_CODE
	case 0x1:                   // M-system DOC device on CS2
		Retval = (mDOC_Initialize() != M_SYS_TRUE ) ? MDOCInitFailed : NoError;
		CopyFlashParams( pFlashProp, MSYS_DOC_FLASH, FlashBusWidth16, CS2Base, DOCH_SECTOR_SIZE );
		break;
#endif
#if ONENAND_CODE
	case 0x2:                  // Samsung OneNAND
		Retval = Flex_Initialize( pFlashProp );
		if  (Retval != NoError)
			return OneNandInitFailed;
		//         CopyFlashParams( pFlashProp, FLEXONENAND, FlashBusWidth16, CS2Base, ONENAND_BLK_SZ );
		pFlashProp->FlashType = OneNAND;
		pFlashProp->FlashBusWidth = FlashBusWidth16;
		pFlashProp->XIP_Prop.BaseAddr = CS2Base;
		break;
#endif
#if NOR_CODE
	case 0x3:                  // x16 Sibley on SMC->EMPI->CS0
		ChipSelect0();
		CopyFlashParams( pFlashProp, SIBLEY_FLASH, FlashBusWidth16, CS0Base, SIBLEY_BLK_SZ);
		break;
#endif
#if NAND_CODE
	case 0x4:                 // x16 NAND   on DFC->DFI->CS0
		Retval = XllpDfcInit( 16, pFlashProp);
		if  (Retval != NoError)
			return DFCInitFailed;
		pFlashProp->FlashType = NAND_FLASH;
		break;
#endif
#if NOR_CODE
	case 0x5:                // x16 Sibley on SMC->DFI->CS2
		ChipSelect2();
		CopyFlashParams( pFlashProp, SIBLEY_FLASH, FlashBusWidth16, CS2Base, SIBLEY_BLK_SZ);
		break;
#endif
#if NAND_CODE
	case 0x6:               // x8  NAND   on DFC->DFI->CS0
		Retval = XllpDfcInit( 8, pFlashProp);
		if  (Retval != NoError)
			return DFCInitFailed;
		pFlashProp->FlashType = NAND_FLASH;
		break;
#endif
#if NOR_CODE
	case 0x7:               // x16 TYAX   on SMC->DFI->CS2
		ChipSelect2();
		CopyFlashParams( pFlashProp, TYAX_FLASH, FlashBusWidth16, CS2Base, TYAX_BLK_SZ );
		break;
#endif

	case 0x8:
	case 0x9:
	case 0xa:
	case 0xb:
	case 0xc:
	case 0xd:
	case 0xe:
	case 0xf:
	default:
#if (!NOFLASH_CODE)
		Retval = InvalidPlatformConfigError;
		AddMessage((UINT8_T*) ("*** Error: Flash driver/flash number mismatch ***\0"));
#endif
		break;

	}

	//CommonChipSelect();

	switch(pFlashProp->FlashType)
	{
#if NAND_CODE || ONENAND_CODE
	case NAND_FLASH:                     //set-up the BBT for NAND & OneNAND
	case OneNAND:
		switch(pFlashProp->BlockSize)    // 1 Relo table should be shared by both Boot flash and WTM flash.
			{
		case 16*1024 : pFlashProp->NAND_Prop.pRelotable =  (Reloc_T *) RELOC_16KB;
			break;
		case 32*1024 : pFlashProp->NAND_Prop.pRelotable =  (Reloc_T *) RELOC_32KB;
			break;
		case 64*1024 : pFlashProp->NAND_Prop.pRelotable =  (Reloc_T *) RELOC_64KB;
			break;
		case 128*1024: pFlashProp->NAND_Prop.pRelotable =  (Reloc_T *) RELOC_128KB;
			break;
		default: break;
		}

#if NAND_CODE
		// Don't attempt to retrieve the BBT from the OneNAND device if the
		// init() failed
		//-----------------------------------------------------------------
		if( Retval == NoError )
		{
			// Copy the relocation table into ISRAM.
			// Note we do not check errors here.  If we did not find a valid table let
			// the ReadWithBBT function handle it.
			//------------------------------------------------------------------------

			if(pFlashProp->FlashType == NAND_FLASH)
				FindNANDBBT (pFlashProp);
			break;
		}
#endif

		pFlashProp->NAND_Prop.BBT_State = BBT_UNCHANGED;
		break;
#endif

#if MDOC_CODE
	case MSYS_DOC_FLASH:
		break;
#endif
	default:   // else we're XIP case
#if NOR_CODE
		{
			// Only get NOR flash information for trusted boot
			if(pFuses->bits.SBE == 1){
				FlashMainProgram(pFlashProp,
					ReadFlashIDCmd,
					&(pFlashProp->FlashID),
					&(pFlashProp->ManufacturerCode));
			}
		}
#endif
	break;
	}
	return Retval;
}

	// This routine will configure the WTM flash accorinding what the TIM settings passed in.
UINT_T Configure_SaveStateFlash(pFLASH_I pFLASH_I_h, P_FlashProperties_T pFlashProp)
{
    pFUSE_SET pFuses;
	UINT_T Retval = NoError;

	UINT_T *temp = (UINT_T *)&(pFlashProp[0]);
	UINT_T *temp1 = (UINT_T *)&(pFlashProp[1]);
	UINT_T i;

	//check to see if the boot FLASH is the same as the WTM flash
	if((pFLASH_I_h->WTMFlashSign & FLASHSIGMASK) == (pFLASH_I_h->BootFlashSign & FLASHSIGMASK))
	{
		//flash already configured when we loaded TIM no need to do this again
		// copy information from boot FLASH flash properties struct to wtm flash properties struct
		for(i = 0; i < (sizeof(FlashProperties_T) / 4); i++)
			temp1[i] = temp[i];
	}
	else
	{
		//configure the WTM flash
        pFuses->bits.PlatformState = pFLASH_I_h->WTMFlashSign & FLASHSIGMASK;
        Configure_Flashes ( pFuses, &pFlashProp[1]);
	}

	return Retval;
}

UINT_T LoadTim( P_FlashProperties_T pFlashProp, UINT8_T *TIMArea, TIM *pTIM_h)
{

	UINT_T Retval = NoError;
	UINT_T Offset = 0x0;
	UINT_T TimSize = 0;
	pIMAGE_INFO pImageInfo;
	UINT_T i = 0;

	if(pFlashProp == NULL)
	{
		// if no Flash properties were passed in then this was from a download and the TIM is in ISRAM
		// just set the pointers correctly
		SetTIMPointers(TIMArea, pTIM_h);
		if( pTIM_h->pConsTIM->VersionBind.Identifier != TIMIDENTIFIER )
			Retval = TIMNotFound;

		return Retval;
	}


	switch( pFlashProp->FlashType )
	{
#if NOR_CODE
	case TYAX_FLASH:
	case SIBLEY_FLASH:
		if(pFlashProp->XIP_Prop.BaseAddr == CS0Base)
			Offset = TIMOffset_CS0;

		SetTIMPointers((UINT8_T *)(pFlashProp->XIP_Prop.BaseAddr + Offset), pTIM_h);

		if( pTIM_h->pConsTIM->VersionBind.Identifier != TIMIDENTIFIER )
		{
			Retval = TIMNotFound;
		}
		else
		{
			pImageInfo = FindImageInTIM(pTIM_h, TIMIDENTIFIER);
			if(pImageInfo == NULL)
				return TIMNotFound;

			TimSize = pImageInfo->ImageSize;

			for(i = 0; i < TimSize; i+=4)
				*(UINT32_T*)(TIMArea + i)= *(UINT32_T *)(pFlashProp->XIP_Prop.BaseAddr + Offset + i);

			SetTIMPointers(TIMArea, pTIM_h);
		}
		break;
#endif //NOR_CODE
	default:
		SetTIMPointers(TIMArea, pTIM_h);
		if( pTIM_h->pConsTIM->VersionBind.Identifier != TIMIDENTIFIER )
			Retval = TIMNotFound;
	}

	return Retval;
}

UINT_T SetTIMPointers( UINT8_T * StartAddr, TIM *pTIM_h)
{
	pTIM_h->pConsTIM = (pCTIM) StartAddr;	// Overlap Contant Part of TIM with actual TIM...
	pTIM_h->pImg = (pIMAGE_INFO) (StartAddr + sizeof (CTIM));
	pTIM_h->pKey = (pKEY_MOD) ((UINT_T)pTIM_h->pImg + ((pTIM_h->pConsTIM->NumImages) * sizeof (IMAGE_INFO)));
	pTIM_h->pReserved = (PUINT) ((UINT_T)pTIM_h->pKey + ((pTIM_h->pConsTIM->NumKeys) * sizeof (KEY_MOD)));
	pTIM_h->pTBTIM_DS = (pPLAT_DS) ((UINT_T)pTIM_h->pReserved + pTIM_h->pConsTIM->SizeOfReserved);
	return (NoError);
}


#if (BOOTROM || TRUSTED)
	/*
	*	LoadSaveState function
	*		The flash has already been configured via the Configure_SaveStateFlash function
	*		Now we must load the save state both the static area and the dynamic area.
	*		The dynamic area must be searched for to find the last slot that was used by the
	*		upper layers of software.
	*/
UINT_T LoadSaveState(P_FlashProperties_T pFlashProp, pSaveState	State, TIM *pTIM_h){

	UINT_T Retval = NoError;
	UINT_T SlotNotFound = TRUE;
	UINT_T StartPointOffset = 0;
	UINT_T CurrentPageOffset = 0;
	UINT_T NumberSlots = 0;
	UINT_T CurrentSlot = 0;
	UINT_T MaxSlot, MinSlot;
	UINT_T LastMagicSlot = 0;
	DynamicState	CurrentState;
	UINT_T *temp, *temp1;
	UINT_T i;
	UINT_T SlotDifference;
	UINT_T EntryOffset;

	EntryOffset = pTIM_h->pConsTIM->FlashInfo.WTMEntryAddr;
	temp = (UINT_T *)&State->SlotN;
#if ((ONENAND_CODE) || (NAND_CODE))
	if (pFlashProp->FlashType == NAND_FLASH ||
		pFlashProp->FlashType == OneNAND )
	{
		//First read the static part of the save state which is 2KB in size
#if NAND_CODE
		if( pFlashProp->FlashType == NAND_FLASH )
			Retval = ReadNANDWithBBM (EntryOffset, sizeof(FixedState), (P_DFC_BUFFER)&State->SlotZero, pFlashProp);
#endif
#if ONENAND_CODE
		if(	pFlashProp->FlashType == OneNAND )
			Retval = Flex_ReadFlash(EntryOffset, sizeof(FixedState), (UINT_T )&State->SlotZero, pFlashProp);
#endif

		if (Retval != NoError)
			return SaveStateNotFound;
		//Now we must locate the last used slot of dynamic state.  Do this by searching for the magic number.
		//The last slot programmed will have the magic number in the first word of the slot, the next slot will be all FF's
		// Start in the middle of the block and do a binary search to locate the last used slot.
		// We must keep track of previous blocks visited.
		NumberSlots = (pFlashProp->BlockSize - SLOTZEROSIZE) / pFlashProp->NAND_Prop.PageSize;
		CurrentSlot = NumberSlots / 2;
		StartPointOffset = (CurrentSlot * pFlashProp->NAND_Prop.PageSize) + SLOTZEROSIZE + EntryOffset; //CurrentSlot * page size + static
		CurrentPageOffset = StartPointOffset;
		MinSlot = SLOTZEROSIZE / pFlashProp->NAND_Prop.PageSize;
		MaxSlot = pFlashProp->BlockSize / pFlashProp->NAND_Prop.PageSize;
		temp1 = (UINT_T *)&CurrentState;
		while(TRUE)
		{
#if NAND_CODE
			if( pFlashProp->FlashType == NAND_FLASH )
				Retval = ReadNANDWithBBM (CurrentPageOffset, sizeof(CurrentState), (P_DFC_BUFFER)&CurrentState, pFlashProp);
#endif
#if ONENAND_CODE
			if(	pFlashProp->FlashType == OneNAND )
				Retval = Flex_ReadFlash(CurrentPageOffset, sizeof(CurrentState), (UINT_T )&CurrentState, pFlashProp);
#endif
			//if (Retval != NoError){
			//	Retval = DownloadError;
			//}
			if(CurrentState.MagicNo == MAGICID)
			{// if we search towards the top
				SlotNotFound = FALSE;
				//keep track of this slot as the last slot with the magic ID
				LastMagicSlot = CurrentSlot;
				//Set new minimum slot to current page
				MinSlot = CurrentSlot;
				//adjust current slot to midpoint of max and min to get next offset
				NumberSlots = MaxSlot - MinSlot;
				CurrentSlot += (NumberSlots / 2)? NumberSlots / 2 : 1;
				CurrentPageOffset = (CurrentSlot * pFlashProp->NAND_Prop.PageSize) + EntryOffset;
				//Save this slot
				for(i = 0; i < (sizeof(CurrentState)/4); i++){
					temp[i] = temp1[i];
				}
			}else{ // we search towards the bottom
				SlotDifference = CurrentSlot - MinSlot;
				//check to see if we hit the crossover point.  If so then break out of loop assuming
				// at least one slot has been found. Last slot saved above is the current slot in use.
				if((SlotDifference == 1) && (SlotNotFound == FALSE))
					break;
				//adjust the maximum slot to the current position
				MaxSlot = CurrentSlot;
				//adjust current slot to midpoint of max and min to get next offset
				NumberSlots = MaxSlot - MinSlot;
				CurrentSlot -= (NumberSlots / 2)? NumberSlots / 2 : 1;
				CurrentPageOffset = (CurrentSlot * pFlashProp->NAND_Prop.PageSize) + EntryOffset;
			}
			if(NumberSlots == 0)//The last slot or first slot has been used	Max will eventually  equal Min
				break;
		}
	}
	else
#endif
#if NOR_CODE
	{
		//Now we must locate the last used slot of dynamic state.  Do this by searching for the magic number.
		//The last slot programmed will have the magic number in the first word of the slot, the next slot will be all FF's
		// Start in the middle of the block and do a binary search to locate the last used slot.
		// We must keep track of previous blocks visited.
		NumberSlots = (pFlashProp->BlockSize - SLOTZEROSIZE) / SLOTNSIZE;
		CurrentSlot = NumberSlots / 2;
		StartPointOffset = (CurrentSlot * SLOTNSIZE) + SLOTZEROSIZE + EntryOffset + pFlashProp->XIP_Prop.BaseAddr; //CurrentSlot * slot size + static
		CurrentPageOffset = StartPointOffset;
		MinSlot = SLOTZEROSIZE / SLOTNSIZE;
		MaxSlot = pFlashProp->BlockSize / SLOTNSIZE;
		while(TRUE)
		{
			temp1 = (UINT_T *)CurrentPageOffset;
			if(*temp1 == MAGICID)
			{ 	// if we search towards the top
				SlotNotFound = FALSE;
				//keep track of this slot as the last slot with the magic ID
				LastMagicSlot = CurrentSlot;
				//Set new minimum slot to current page
				MinSlot = CurrentSlot;
				//adjust current slot to midpoint of max and min to get next offset
				NumberSlots = MaxSlot - MinSlot;
				CurrentSlot += (NumberSlots / 2)? NumberSlots / 2 : 1;
				CurrentPageOffset = (CurrentSlot * SLOTNSIZE) + EntryOffset + pFlashProp->XIP_Prop.BaseAddr;
				//Save this slot
				for(i = 0; i < (sizeof(CurrentState)/4); i++){
					temp[i] = temp1[i];
				}
			}
			else
			{ // we search towards the bottom
				SlotDifference = CurrentSlot - MinSlot;
				//check to see if we hit the crossover point.  If so then break out of loop
				// Last slot saved above is the current slot in use.
				if((SlotDifference == 1) && (SlotNotFound == FALSE))
					break;
				//adjust the maximum slot to the current position
				MaxSlot = CurrentSlot;
				//adjust current slot to midpoint of max and min to get next offset
				NumberSlots = MaxSlot - MinSlot;
				CurrentSlot -= (NumberSlots / 2)? NumberSlots / 2 : 1;
				CurrentPageOffset = (CurrentSlot * SLOTNSIZE) + EntryOffset + pFlashProp->XIP_Prop.BaseAddr;
			}
			if(NumberSlots == 0)//The last slot or first slot has been used	Max will eventually  equal Min
				break;

		}
	}
#endif

	if(SlotNotFound)
		return SaveStateNotFound;

	pFlashProp->NextSlotAvailable = CurrentSlot + 1;
	return Retval;
}
#endif

#if NAND_CODE
	/*
	This routine finds the slot that contains the last Bad Block Table.
	The routine uses a binary search algorithm to find the last slot used.
	The first the BBT will be placed will be the last slot of the block and will
	grow down from there.

	The BBT will be located in the first good block of the flash.

	*** TBD *** - Make EntryOffset an input containing address to the first good block of the NAND part
	*/
UINT_T FindNANDBBT (P_FlashProperties_T pFlashProp)
{
	UINT_T SlotFound = FALSE;
	UINT_T CurrentBlockOffset = 0;
	UINT_T NumberSlots = 0;
	UINT_T MaxSlot, MinSlot, Change;
	UINT_T EntryOffset = 0;		// Start with block 0.
	UINT_T* CurrentSlot;
	Reloc_T TempBBT;

	// Set the current slot.
	CurrentSlot = &pFlashProp->NAND_Prop.BBT_Slot;

	// We do a binary search starting in the middle.
	NumberSlots = pFlashProp->NAND_Prop.NumBBTSlots;
	MinSlot = (pFlashProp->BlockSize / pFlashProp->NAND_Prop.PageSize) - NumberSlots;
	MaxSlot = (pFlashProp->BlockSize / pFlashProp->NAND_Prop.PageSize) - 1;

	//First check to see if a BBT was loaded for MFG blocks
	*CurrentSlot = MaxSlot;
	CurrentBlockOffset = (*CurrentSlot * pFlashProp->NAND_Prop.PageSize) + EntryOffset; //CurrentSlot * page size + TIM
	XllpDfcRead((P_DFC_BUFFER) pFlashProp->NAND_Prop.pLocalRelotable, CurrentBlockOffset, sizeof(Reloc_T), NULL, pFlashProp);
	//BBT for MFG blocks was found, next check if we ever had any runtime entries
	if(pFlashProp->NAND_Prop.pLocalRelotable->Header == VALID_HEADER)
	{
		*CurrentSlot = MaxSlot - 1;
		CurrentBlockOffset = (*CurrentSlot * pFlashProp->NAND_Prop.PageSize) + EntryOffset; //CurrentSlot * page size + TIM
		XllpDfcRead((P_DFC_BUFFER) &TempBBT, CurrentBlockOffset, sizeof(Reloc_T), NULL, pFlashProp);
	}
	else
		return BBTReadError;

	// if there has been at least one runtime entry search else return with the MFG bbt table.
	if(TempBBT.Header == VALID_HEADER)
	{
		*CurrentSlot = (NumberSlots / 2) + MinSlot;
		CurrentBlockOffset = (*CurrentSlot * pFlashProp->NAND_Prop.PageSize) + EntryOffset; //CurrentSlot * page size + TIM
		XllpDfcRead((P_DFC_BUFFER) pFlashProp->NAND_Prop.pLocalRelotable, CurrentBlockOffset, sizeof(Reloc_T), NULL, pFlashProp);

		while(TRUE)
		{
			if(pFlashProp->NAND_Prop.pLocalRelotable->Header == VALID_HEADER)		// go down
				{
				SlotFound = TRUE;

				// This is the highest slot
				MaxSlot = *CurrentSlot;
				// Are we done?
				if (MinSlot == *CurrentSlot)
					return NoError;

				NumberSlots = MaxSlot - MinSlot;
				// Calculate next slot
				Change = (NumberSlots / 2) ? (NumberSlots / 2) : 1;
				*CurrentSlot = MaxSlot - Change;

			}
			else   // go up
				{
				MinSlot = *CurrentSlot + 1;
				// Are we done?
				if (MaxSlot == *CurrentSlot)
				{
					if (SlotFound != TRUE)
						return BBTReadError;
					else
						return NoError;
				}
				NumberSlots = MaxSlot - MinSlot;
				// This is the lowest slot
				Change = (NumberSlots / 2);
				*CurrentSlot = MinSlot + Change;
			}
			CurrentBlockOffset = (*CurrentSlot * pFlashProp->NAND_Prop.PageSize) + EntryOffset; //CurrentSlot * page size + TIM
			XllpDfcRead ((P_DFC_BUFFER) pFlashProp->NAND_Prop.pLocalRelotable, CurrentBlockOffset, sizeof(Reloc_T), NULL, pFlashProp);
		}
	}
	return NoError;
}

#endif

#if (!TRUSTED)
	/* ProbeFlash routine in the OBM
	*
	*	This would be done for Non-Trusted only and when being used in conjunction with older variants of the TBR where probing
	*	took place but the information was not passed to the OBM.
	*/

UINT_T ProbeFlash(pFUSE_SET pFuses, pTIM pTIM_h, P_FlashProperties_T pFlashProperties, Reloc_T *pLocalRelo, UINT8_T *TIM_Area)
{
	UINT_T Retval = NoError;
	UINT_T count = 0;
	FUSE_SET  TempFuses;

	SaveDefaultConfig();
	do
	{
		switch (count)
		{
		// cycle through x8 NAND, CS2, x16 NAND, OneNand, M-Systems, CS0.
		case 0:TempFuses.bits.PlatformState = 0x4;  // x16 NAND
			break;
		case 1:TempFuses.bits.PlatformState = 0x5;  // x16 XIP on CS2
			break;
		case 2:TempFuses.bits.PlatformState = 0x6;  // x8 NAND
			break;
		case 3:TempFuses.bits.PlatformState = 0x2;  // OneNAND
			break;
		case 4:TempFuses.bits.PlatformState = 0x1;  // MSystems
			break;
		case 5:TempFuses.bits.PlatformState = 0x7;  // x16 Tyax on CS2
			break;
		case 6:TempFuses.bits.PlatformState = 0x3;  // x16 Sibley on CS0
			break;
		}

		// configure the boot flash
        Retval = Configure_Flashes( &TempFuses, pFlashProperties);
		if(Retval == NoError)
		{
			//first determine if we have a TIM or Image module present
			Retval = LoadTim(pFlashProperties, TIM_Area, pTIM_h);
		}
		if(Retval != NoError)
		{
			count++;
			RestoreDefaultConfig();
		}
		else
		{
			if(((pTIM_h->pConsTIM->FlashInfo.BootFlashSign & 0xFF) == 0x7) && (TempFuses.bits.PlatformState == 0x5))
				TempFuses.bits.PlatformState = 0x7;
			if((pTIM_h->pConsTIM->FlashInfo.BootFlashSign & 0xFF) != TempFuses.bits.PlatformState)
			{
				count++;
				continue;
			}

			//TIM has been found so set the LOAD Mechanism flag for later use
			Retval = ValidateCheckSum(TIMIDENTIFIER, pTIM_h, pFlashProperties);
			if(Retval != NoError)
			{
				AddMessage("NTIM CRC check failed\0");
				count++;
				continue;
			}
			break;
		}
	}
	while(count < 7);

	if (count < 7)
		return NoError;
	else
		return ReadError;
}
#endif

#if NOR_CODE
static const unsigned int PRLockRegisterAddress_32[]=
{
    PRLockReg0Addr_32,
    PRLockReg1Addr_32,
    PRLockReg2Addr_32,
    PRLockReg3Addr_32,
    PRLockReg4Addr_32,
    PRLockReg5Addr_32,
    PRLockReg6Addr_32,
    PRLockReg7Addr_32,
    PRLockReg8Addr_32,
    PRLockReg9Addr_32,
    PRLockReg10Addr_32,
    PRLockReg11Addr_32,
    PRLockReg12Addr_32,
    PRLockReg13Addr_32,
    PRLockReg14Addr_32,
    PRLockReg15Addr_32,
	PRLockReg16Addr_32
};

static const unsigned int PRLockRegisterAddress_16[]=
{
    PRLockReg0Addr_16,
    PRLockReg1Addr_16,
    PRLockReg2Addr_16,
    PRLockReg3Addr_16,
    PRLockReg4Addr_16,
    PRLockReg5Addr_16,
    PRLockReg6Addr_16,
    PRLockReg7Addr_16,
    PRLockReg8Addr_16,
    PRLockReg9Addr_16,
    PRLockReg10Addr_16,
    PRLockReg11Addr_16,
    PRLockReg12Addr_16,
    PRLockReg13Addr_16,
    PRLockReg14Addr_16,
    PRLockReg15Addr_16,
	PRLockReg16Addr_16
};

UINT_T LockProtectionRegister(P_FlashProperties_T pFlashP, UINT_T ProtectionRegLockNum)
{
	UINT_T BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	UINT_T   ProgramStatus;
	UINT_T   ProtectionLockAddr;
	UINT_T  ProtectionData = 0xFFFF;
	UINT16_T ProtectionData_16 = 0xFFFF;

	if (ProtectionRegLockNum > ProtectionRegister16)
	{
		return (ReadError);
	}

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		if (ProtectionRegLockNum == ProtectionRegister0)
		{
			ProtectionLockAddr = (OTPLockRegister0_32 + BaseAddr);
			ProtectionData &= ~(1 << (ProtectionRegLockNum + 1));
		}
		else
		{
			ProtectionLockAddr = (OTPLockRegister1_32 + BaseAddr);
			ProtectionData &= ~(1 << (ProtectionRegLockNum - 1));
		}
		ProtectionData  = ((ProtectionData << 16) | (ProtectionData & 0x0000FFFF));

		// Clear the protection register bit to be locked
		// Issue clear status command to clear status register
		*(VUINT_T *)BaseAddr = ClearStatusRegisterCmd;

		*(VUINT_T *)ProtectionLockAddr = ProgramProtectionRegCmd;  // Issue protection program command
		*(VUINT_T *)ProtectionLockAddr = ProtectionData;
		do
		{
			//*WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT_T *)BaseAddr;
		} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);
		*(VUINT_T *)BaseAddr = ReadArrayCmd;  	// Restore read array mode
	}
	else
	{
		if (ProtectionRegLockNum == ProtectionRegister0)
		{
			ProtectionLockAddr = (OTPLockRegister0_16 + BaseAddr);
			ProtectionData_16 &= ~(1 << (ProtectionRegLockNum + 1));
		}
		else
		{
			ProtectionLockAddr = (OTPLockRegister1_16 + BaseAddr);
			ProtectionData_16 &= ~(1 << (ProtectionRegLockNum - 1));
		}

		// Clear the protection register bit to be locked
		// Issue clear status command to clear status register
		*(VUINT16_T *)BaseAddr = ClearStatusRegisterCmd_16;
		*(VUINT16_T *)ProtectionLockAddr = ProgramProtectionRegCmd_16;  // Issue protection program command
		*(VUINT16_T *)ProtectionLockAddr = ProtectionData_16;
		do
		{
			//*WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT16_T *)BaseAddr;
		} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);
		*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;  	// Restore read array mode
	}
	return (NoError);
}

UINT_T ProgramProtectionReg(P_FlashProperties_T pFlashP,
	UINT_T   ProtectionRegNum,
	UINT16_T *ProtectionRegData,
	UINT8_T  NumWordsToProgram)
{
	UINT_T  ProgramStatus;
	UINT_T  ProtectionData;
	VUINT_T ProtectionRegAddr;
	UINT_T  ProtectionRegCount;
	UINT_T  EndCount = 0;
	UINT_T	BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	if (ProtectionRegNum > ProtectionRegister16)
	{
		return (ReadError);
	}

	if (ProtectionRegNum == ProtectionRegister0)
	{
		if (NumWordsToProgram > 4)
		{
			return (ProgramError);
		}
		else
		{
			EndCount = NumWordsToProgram;
		}
	}
	else
	{
		if (NumWordsToProgram > 8)
		{
			return (ProgramError);
		}
		else
		{
			EndCount = NumWordsToProgram;
		}
	}

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_32[ProtectionRegNum] + BaseAddr;
		// Issue block unlock setup command.  Done to unlock block, just in case....
		*(VUINT_T *)ProtectionRegAddr = UnlockBlockCmd;
		*(VUINT_T *)ProtectionRegAddr = UnlockBlockConfirmCmd; // Issue unlock block confirm
		do
		{
			//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);
		for (ProtectionRegCount = 0; ProtectionRegCount < EndCount; ProtectionRegCount++)
		{
			ProtectionData = ProtectionRegData[ProtectionRegCount];
			ProtectionData = ((ProtectionData << 16) | (0x0000FFFF & ProtectionData));

			// Issue clear status command to clear status register
			*(VUINT_T *)ProtectionRegAddr = ClearStatusRegisterCmd;
			*(VUINT_T *)ProtectionRegAddr = ProgramProtectionRegCmd;  // Issue protection program command
			*(VUINT_T *)ProtectionRegAddr = ProtectionData;
			do
			{
				ProgramStatus   = *(VUINT_T *)ProtectionRegAddr;
			} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);

			// PERFORM FULL STATUS CHECK HERE IF DESIRED
			ProtectionRegAddr   = (ProtectionRegAddr + 4);
		}
		*(VUINT_T *)BaseAddr = ReadArrayCmd;  	// Restore read array mode
	}
	else
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_16[ProtectionRegNum] + BaseAddr;
		// Issue block unlock setup command.  Done to unlock block, just in case....
		*(VUINT16_T *)ProtectionRegAddr = UnlockBlockCmd_16;
		*(VUINT16_T *)ProtectionRegAddr = UnlockBlockConfirmCmd_16; // Issue unlock block confirm
		do
		{
			//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT16_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);
		for (ProtectionRegCount = 0; ProtectionRegCount < EndCount; ProtectionRegCount++)
		{
			ProtectionData = ProtectionRegData[ProtectionRegCount];
			//ProtectionData = ((ProtectionData << 16) | (0x0000FFFF & ProtectionData));

			// Issue clear status command to clear status register
			*(VUINT16_T *)ProtectionRegAddr = ClearStatusRegisterCmd_16;
			*(VUINT16_T *)ProtectionRegAddr = ProgramProtectionRegCmd_16;  // Issue protection program command
			*(VUINT16_T *)ProtectionRegAddr = ProtectionData;
			do
			{
				ProgramStatus   = *(VUINT16_T *)ProtectionRegAddr;
			} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

			// PERFORM FULL STATUS CHECK HERE IF DESIRED
			ProtectionRegAddr   = (ProtectionRegAddr + 2);
		}
		*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;  	// Restore read array mode
	}
	return (NoError);
}

UINT_T ProgramProtectionRegister(P_FlashProperties_T pFlashP, UINT_T ProtectionRegNum, UINT16_T *ProtectionRegData)
{
	UINT_T  ProgramStatus;
	UINT_T  ProtectionData;
	VUINT_T ProtectionRegAddr;
	UINT_T  ProtectionRegCount;
	UINT_T  EndCount = 0;
	UINT_T	BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	if (ProtectionRegNum > ProtectionRegister16)
	{
		return (ReadError);
	}

	if (ProtectionRegNum == ProtectionRegister0)
	{
		EndCount = 4;
	}
	else
	{
		EndCount = 8;
	}

	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_32[ProtectionRegNum] + BaseAddr;
		// Issue block unlock setup command.  Done to unlock block, just in case....
		*(VUINT_T *)ProtectionRegAddr = UnlockBlockCmd;
		*(VUINT_T *)ProtectionRegAddr = UnlockBlockConfirmCmd; // Issue unlock block confirm
		do
		{
			//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);
		for (ProtectionRegCount = 0; ProtectionRegCount < EndCount; ProtectionRegCount++)
		{
			ProtectionData = ProtectionRegData[ProtectionRegCount];
			ProtectionData = ((ProtectionData << 16) | (0x0000FFFF & ProtectionData));

			// Issue clear status command to clear status register
			*(VUINT_T *)ProtectionRegAddr = ClearStatusRegisterCmd;
			*(VUINT_T *)ProtectionRegAddr = ProgramProtectionRegCmd;  // Issue protection program command
			*(VUINT_T *)ProtectionRegAddr = ProtectionData;
			do
			{
				ProgramStatus   = *(VUINT_T *)ProtectionRegAddr;
			} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);

			// PERFORM FULL STATUS CHECK HERE IF DESIRED
			ProtectionRegAddr   = (ProtectionRegAddr + 4);
		}
		*(VUINT_T *)BaseAddr = ReadArrayCmd;  	// Restore read array mode
	}
	else
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_16[ProtectionRegNum] + BaseAddr;
		// Issue block unlock setup command.  Done to unlock block, just in case....
		*(VUINT16_T *)ProtectionRegAddr = UnlockBlockCmd_16;
		*(VUINT16_T *)ProtectionRegAddr = UnlockBlockConfirmCmd_16; // Issue unlock block confirm
		do
		{
			//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT16_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);
		for (ProtectionRegCount = 0; ProtectionRegCount < EndCount; ProtectionRegCount++)
		{
			ProtectionData = ProtectionRegData[ProtectionRegCount];
			//ProtectionData = ((ProtectionData << 16) | (0x0000FFFF & ProtectionData));

			// Issue clear status command to clear status register
			*(VUINT16_T *)ProtectionRegAddr = ClearStatusRegisterCmd_16;
			*(VUINT16_T *)ProtectionRegAddr = ProgramProtectionRegCmd_16;  // Issue protection program command
			*(VUINT16_T *)ProtectionRegAddr = ProtectionData;
			do
			{
				ProgramStatus   = *(VUINT16_T *)ProtectionRegAddr;
			} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

			// PERFORM FULL STATUS CHECK HERE IF DESIRED
			ProtectionRegAddr   = (ProtectionRegAddr + 2);
		}
		*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;  	// Restore read array mode
	}
	return (NoError);
}

UINT_T ProgramProtectionRegisterWord(P_FlashProperties_T pFlashP,
	UINT_T ProtectionRegNum,
	UINT16_T  Data,
	UINT8_T RegisterWord)
{
	UINT_T  ProgramStatus;
	UINT_T  ProtectionData;
	VUINT_T ProtectionRegAddr;
	UINT_T	BaseAddr = pFlashP->XIP_Prop.BaseAddr;

	if (ProtectionRegNum > ProtectionRegister16)
	{
		return (ProtectionRegProgramError);
	}

	if (RegisterWord > 7)
	{
		return (ProtectionRegProgramError);
	}
	if (pFlashP->FlashBusWidth == FlashBusWidth32)
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_32[ProtectionRegNum] + BaseAddr;
		// Issue block unlock setup command.  Done to unlock block, just in case....
		*(VUINT_T *)ProtectionRegAddr = UnlockBlockCmd;
		*(VUINT_T *)ProtectionRegAddr = UnlockBlockConfirmCmd; // Issue unlock block confirm
		do
		{
			//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);

		ProtectionData = ((Data << 16) | (0x0000FFFF & Data));
		ProtectionRegAddr = (ProtectionRegAddr + (RegisterWord*4));
		// Issue clear status command to clear status register
		*(VUINT_T *)ProtectionRegAddr = ClearStatusRegisterCmd;
		*(VUINT_T *)ProtectionRegAddr = ProgramProtectionRegCmd;  // Issue protection program command
		*(VUINT_T *)ProtectionRegAddr = ProtectionData;
		do
		{
			ProgramStatus   = *(VUINT_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus) != DeviceReadyStatus);

		// PERFORM FULL STATUS CHECK HERE IF DESIRED
		*(VUINT_T *)BaseAddr = ReadArrayCmd;  	// Restore read array mode
	}
	else
	{
		// Retrieve the base address for the protection register
		// to be read
		ProtectionRegAddr    = PRLockRegisterAddress_16[ProtectionRegNum] + BaseAddr;
		// Issue block unlock setup command.  Done to unlock block, just in case....
		*(VUINT16_T *)ProtectionRegAddr = UnlockBlockCmd_16;
		*(VUINT16_T *)ProtectionRegAddr = UnlockBlockConfirmCmd_16; // Issue unlock block confirm
		do
		{
			//*(volatile UINT_T *)WordProgramAddr =  ReadStatusRegisterCmd; // Issue read status command
			ProgramStatus   = *(VUINT16_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

		ProtectionData = Data;
		ProtectionRegAddr = (ProtectionRegAddr + (RegisterWord*2));
		// Issue clear status command to clear status register
		*(VUINT16_T *)ProtectionRegAddr = ClearStatusRegisterCmd_16;
		*(VUINT16_T *)ProtectionRegAddr = ProgramProtectionRegCmd_16;  // Issue protection program command
		*(VUINT16_T *)ProtectionRegAddr = ProtectionData;
		do
		{
			ProgramStatus   = *(VUINT16_T *)ProtectionRegAddr;
		} while ((ProgramStatus & DeviceReadyStatus_16) != DeviceReadyStatus_16);

		// PERFORM FULL STATUS CHECK HERE IF DESIRED
		*(VUINT16_T *)BaseAddr = ReadArrayCmd_16;  	// Restore read array mode
	}
	return (NoError);
}

#endif //NOR_CODE

