/******************************************************************************
**
**  COPYRIGHT (C) 2005 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	data_to_os.h
**
**  PURPOSE: 	This file provides basic type definitions for A0 chip model
**
******************************************************************************/
#ifndef _DATA_TO_OS_H_
#define _DATA_TO_OS_H_



/***************************************************************************************************/
/***************************************************************************************************/

#define OBM_OS_DATA_VERSION					( 0x00000001 )


#if (OBM)
#define OBM_OS_BUFFRE_BASE_ADDRESS 			( 0x81002000 )  // The addres in the DDR which the TLV will be written.
#endif

typedef struct
{
	unsigned long		signature;
	unsigned long		version;
	unsigned long		reserved1;
	unsigned long		reserved2;
} OBM_OS_DATA_HEADER;

#define OBM_OS_BUFFRE_HEADER_SIZE				( sizeof(OBM_OS_DATA_HEADER) )

#define OBM_OS_DATA_FIRST_ENTRY_ADDRESS 		( OBM_OS_BUFFRE_BASE_ADDRESS + OBM_OS_BUFFRE_HEADER_SIZE )

/***************************************************************************************************/
/***************************************************************************************************/

typedef enum
{
	BOARD_ENTRY				= 0x424F4152,      // BOAR
	CHIP_ENTRY				= 0x43484950,	   // CHIP
	DDR_BANK_ENTRY			= 0x44445242,      // DDRB
	DDR_SIZE_ENTRY			= 0x44445253,      // DDRS
	FLASH_ENTRY				= 0x464C4153,      // FLAS
	GPS_AVAILABLE_ENTRY		= 0x47505341,	   // GPSA
	MICCO_ENTRY				= 0x4D494343,      // MICC
	MMC_TYPE_ENTRY			= 0x4D4D4354,	   // MMCT
	MVED_EXIST_ENTRY    	= 0X4D564544,	   // MVED
	OBM_VERSION_ENTRY		= 0x4F424D56,      // OBMV
	PIN_MUX_MODE_ENTRY		= 0x50494E4D,      // PINM
	PRODUCT_MODE_TYPE_ENTRY = 0x50524F44,      // PROD
	CHIP_STEP_ENTRY			= 0x53544550,      // STEP
	DATA_TERMINATOR_ID		= 0x5445524D,      // TERM
	DATA_SIGNATURE_ID 		= 0x544C5653,      // TLVS
	USB_TYPE_ENTRY			= 0x55534254,	   // USBT
	WAKEUP_REASON_ENTRY		= 0x57414B45       // WAKE
} OBM_TO_OS_ID;



typedef enum
{
	DATA_AMOUNT_0,
	DATA_AMOUNT_1,
	DATA_AMOUNT_2,
	DATA_AMOUNT_3,
	DATA_AMOUNT_4,
	DATA_AMOUNT_5,
	DATA_AMOUNT_6,
	DATA_AMOUNT_7,
	DATA_AMOUNT_8,
	DATA_AMOUNT_9
} DATA_AMOUNT;



typedef enum
{
	OBM_EVB_P_BOARD,
	OBM_EVB_PV_BOARD,
	OBM_SAAR_2_5_BOARD,
	OBM_SAAR_2_6_BOARD,
	OBM_SAAR_PV_3_1_BOARD,
	OBM_SAAR_PV_3_2_BOARD,
	OBM_YARDEN_BOARD,
	OBM_P65_BOARD,
	MAX_BOARD_TYPE
} OBM_BOARD_TYPE;




typedef enum
{
	CPU_TAVOR_A0,
	CPU_TAVOR_B0,
	CPU_TAVOR_B1,
	CPU_TAVOR_B2,
	CPU_TAVOR_PV_DIS_Z0,
	CPU_TAVOR_PV_DIS_A0,
	CPU_TAVOR_PV_DIS_B0,
	CPU_TAVOR_PV_DIS_B1,
	CPU_TAVOR_PV_DIS_C0,
	CPU_TAVOR_PV2_Z0,
	CPU_TAVOR_PV2_A0,
	CPU_TAVOR_PV2_B0,
	CPU_TAVOR_PV2_C0,
	CPU_TAVOR_PV_POP_Z0,
	CPU_TAVOR_PV_POP_A0,
	CPU_TAVOR_PV_POP_B0,
	CPU_TAVOR_PV_POP_C0,
	CPU_TAVOR_P65_Z0,
	CPU_TAVOR_P65_A0,
	CPU_TAVOR_P65_B0,
	CPU_TAVOR_P65_C0,
	CPU_TAVOR_P_PLUS_Z0,
	CPU_TAVOR_P_PLUS_A0,
	CPU_TAVOR_P_PLUS_B0,
	CPU_TAVOR_P_PLUS_C0,
    CPU_OTHER
}CPU_VERSION;


typedef enum
{
   OBM_MICCO_A0 = 0x00,
   OBM_MICCO_B0 = 0x10,
   OBM_MICCO_C0 = 0x20,
   OBM_MICCO_EA = 0x30,
   OBM_MICCO_EB = 0x31
} MICCO_TYPE;



typedef enum
{
	OBM_TYAX_FLASH   	= 1,
	OBM_SIBLEY_FLASH	= 2,
	OBM_NAND_FLASH  	= 3,
	OBM_ONENAND_FLASH 	= 4,
    OBM_MSYS_DOC_FLASH  = 5,
	OBM_UNKNOWN_FLASH
} OBM_FlashType;



typedef enum
{
	OBM_64M_BYTES_SIZE   	= 1,
	OBM_128M_BYTES_SIZE   	= 2
} OBM_DdrSise;


typedef enum
{
	OBM_DDR_BANK_1   	= 1,
	OBM_DDR_BANK_2   	= 2
} OBM_DdrBank;


typedef enum
{
	OBM_VER_HIGH_VAL   	= 0,
	OBM_VER_LOW_VAL   	= 1
} OBM_Version;



typedef enum
{
 	ARSR_HARDWARE_RESET			= 1,
 	ARSR_WATCHDOG_RESET    		= 2,
 	ARSR_LOW_PWR_MODE_RESET		= 4,
 	ARSR_GPIO_RESET 	   		= 8
} OBM_WakeupReason;



typedef enum
{
	OBM_GPS_NOT_AVAILABLE  	= 0,
	OBM_GPS_AVAILABLE   	= 1
} OBM_GpsAvailable;



typedef enum
{
	OBM_USB1  	= 0,			// USB1
	OBM_USB2  	= 1,            // USB2 - ULPI
	OBM_USBCI  	= 2,            // USB CI
	OBM_USB1_2 	= 3             // USB1 and ULPI --- Only in Tavor P boards.
} OBM_UsbType;



typedef enum
{
	OLD_MMC  	= 0,
	NEW_MMC  	= 1
} OBM_MmcType;



typedef enum
{
	PRODUCTION_MODE_DISABLE  	= 0,
	PRODUCTION_MODE_ENABLE  	= 1,
	PRODUCTION_MODE_EXTERNAL	= 2
} OBM_ProductionMode;



typedef enum
{
	MVED_NOT_EXIST  	= 0,
	MVED_EXIST  		= 1
} OBM_MvedExist;

/***************************************************************************************************/
/***************************************************************************************************/


#if (OBM)

typedef enum
{
	NON_PRODUCTION_MODE_STATE  	= 0,
	PRODUCTION_MODE_STATE  		= 1
} OBM_ProModeState;

void OBM_updtaeOsBuffre ( OBM_TO_OS_ID   data_entry , DATA_AMOUNT   data_amount , unsigned long   *data_value );
#endif
/***************************************************************************************************/
/***************************************************************************************************/






#endif   // _DATA_TO_OS_H_
