/******************************************************************************
**	(C) Copyright 2007 Marvell International Ltd. 
**	All Rights Reserved
******************************************************************************/

/******************************************************************************
**
**  COPYRIGHT (C) 2002, 2003 Intel Corporation.
**
**  This software as well as the software described in it is furnished under
**  license and may only be used or copied in accordance with the terms of the
**  license. The information in this file is furnished for informational use
**  only, is subject to change without notice, and should not be construed as
**  a commitment by Intel Corporation. Intel Corporation assumes no
**  responsibility or liability for any errors or inaccuracies that may appear
**  in this document or any software that may be provided in association with
**  this document.
**  Except as permitted by such license, no part of this document may be
**  reproduced, stored in a retrieval system, or transmitted in any form or by
**  any means without the express written consent of Intel Corporation.
**
**  FILENAME:	PlatformConfig.h
**
**  PURPOSE: 	Platform specific header to set parameters
**
**
******************************************************************************/

#ifndef __platform_config_h
#define __platform_config_h

#include "typedef.h"

//
// Clock in nanoseconds. Not likely to be changing all that often...
//
#if (PV_CHIP)
#define NAND_CONTROLLER_CLOCK  156  // MHz
#else
#define NAND_CONTROLLER_CLOCK  104  // MHz
#endif

// Bit 19 of control register must be written as 0
#define DFC_INITIAL_CONTROL 0xCC021FFF
#define	DFC_INITIAL_TIMING0	0x003F3F3F
//#define DFC_INITIAL_TIMING1 0x1FF0C0FF //tR set to 52.4 usec, enable wait_mode, prescale
#define DFC_INITIAL_TIMING1 0x154900FF //tR set to 39.23usec


#define PlatformISRAMLimit	0xD000 //128KB - 76KB (0x20000 - 0x13000)

//     GPIO
#define GPLRx (volatile unsigned long *)( 0x40E00000 )  // GPIO pin-level register GPIO<31:0>
#define GPLRy (volatile unsigned long *)( 0x40E00004 )  // GPIO pin-level register GPIO<63:32>
#define GPLRz (volatile unsigned long *)( 0x40E00008 )  // GPIO pin-level register GPIO<80:64>
#define GPLRa (volatile unsigned long *)( 0x40E00100 )  // GPIO pin-level register GPIO<127:96>
#define GPDRx (volatile unsigned long *)( 0x40E0000C )  // GPIO pin direction register GPIO<31:0>
#define GPDRy (volatile unsigned long *)( 0x40E00010 )  // GPIO pin direction register GPIO<63:32>
#define GPDRz (volatile unsigned long *)( 0x40E00014 )  // GPIO pin direction register GPIO<80:64>
#define GPDRa (volatile unsigned long *)( 0x40E0010C )  // GPIO pin direction register GPIO<120:96>
#define GPSRx (volatile unsigned long *)( 0x40E00018 )  // GPIO pin output set register GPIO<31:0>
#define GPSRy (volatile unsigned long *)( 0x40E0001C )  // GPIO pin output set register GPIO<63:32>
#define GPSRz (volatile unsigned long *)( 0x40E00020 )  // GPIO pin output set register GPIO<80:64>
#define GPSRa (volatile unsigned long *)( 0x40E00118 )  // GPIO pin output set register GPIO<127:96>
#define GPCRx (volatile unsigned long *)( 0x40E00024 )  // GPIO pin output clear register GPIO<31:0>
#define GPCRy (volatile unsigned long *)( 0x40E00028 )  // GPIO pin output clear register GPIO <63:32>
#define GPCRz (volatile unsigned long *)( 0x40E0002C )  // GPIO pin output clear register GPIO <80:64>
#define GPCRa (volatile unsigned long *)( 0x40E00124 )  // GPIO pin output clear register GPIO <127:96>
#define GRERx (volatile unsigned long *)( 0x40E00030 )  // GPIO rising-edge detect register GPIO<31:0>
#define GRERy (volatile unsigned long *)( 0x40E00034 )  // GPIO rising-edge detect register GPIO<63:32>
#define GRERz (volatile unsigned long *)( 0x40E00038 )  // GPIO rising-edge detect register GPIO<80:64>
#define GRERa (volatile unsigned long *)( 0x40E00130 )  // GPIO rising-edge detect register GPIO<127:96>
#define GFERx (volatile unsigned long *)( 0x40E0003C )  // GPIO falling-edge detect register GPIO<31:0>
#define GFERy (volatile unsigned long *)( 0x40E00040 )  // GPIO falling-edge detect register GPIO<63:32>
#define GFERz (volatile unsigned long *)( 0x40E00044 )  // GPIO falling-edge detect register GPIO<80:64>
#define GFERa (volatile unsigned long *)( 0x40E0013C )  // GPIO falling-edge detect register GPIO<127:96>
#define GEDRx (volatile unsigned long *)( 0x40E00048 )  // GPIO edge detect status register GPIO<31:0>
#define GEDRy (volatile unsigned long *)( 0x40E0004C )  // GPIO edge detect status register GPIO<63:32>
#define GEDRz (volatile unsigned long *)( 0x40E00050 )  // GPIO edge detect status register GPIO<80:64>
#define GEDRa (volatile unsigned long *)( 0x40E00148 )  // GPIO edge detect status register GPIO<127:96>

#define ASCR	(volatile unsigned long *)(0x40F40000 ) // Application Subsystem Power Status/Configuration Register
#define ARSR	(volatile unsigned long *)(0x40F40004 ) // Application Subsystem Reset Status Register
#define AD3SR	(volatile unsigned long *)(0x40F4000C ) // Application Subsystem Wake-Up from D3 Status Register
#define AD3R	(volatile unsigned long *)(0x40F40030 ) // Application Subsystem D3 Configuration Register
#define UP2OCR  (volatile unsigned long *)(0x40600020)

#define PMCR    (volatile unsigned long *)(0x40F50000)
#define PSR     (volatile unsigned long *)(0x40F50004)

#if (PV_CHIP)
#define CHIPBOOTTYPE (volatile unsigned int *)(0x40F50200)
#else
#define CHIPBOOTTYPE (volatile unsigned long *)(0x40F50208)
#endif
#define UDCCR_REG (volatile unsigned long *)(0x40600000) // USB1.1 client control register
#define OSCC	(volatile unsigned long *)(0x41350000 ) // Oscillator Configuration Register
#if (PV_CHIP)
#define ACCR     (volatile unsigned long *)(0x41340000 )        // Application SubSystem Clock Config register
#define ACSR     (volatile unsigned long *)(0x41340004 )        // Application SubSystem Clock Status register
#define D0CKEN_A (volatile unsigned long *)(0x4134000C )        // D0 mode Clock Enable Register A
#define D0CKEN_B (volatile unsigned long *)(0x41340010 )        // D0 mode Clock Enable Register B
#define ACCR1    (volatile unsigned long *)(0x41340020 )        // Application Subsystem Clock Configuration Register 1
#define D0CKEN_C (volatile unsigned long *)(0x41340024 )        // D0 mode Clock Enable Register C
#define ACCR1_OTG_PHY_ENABLE_MASK ((1<<12) | (1<<11) | (1<<10)) // mask to OR on bits in ACCR1 for OTG PHY enable
#define D0CKEN_C_OTG_CLK_ENABLE_MASK ((1<<10) | (1<<6) )        // mask to enable OTG AXI bus & Peripheral clocks
#define ACSR_USB_PLL_LOCK_MASK (1<<27)                          // mask to check status of USB_PLL LOCK state
#define USB2_PHY_T0 (volatile unsigned long *)(0x5550A010)     // phy control register
#define USB2_PHY_T0_OTG_CONTROL_BIT (1 << 28)                   // set phy control to slave from OTG
#define USB2_U2PRX (volatile unsigned long *)(0x5550A008)		// USB PHY RX control
#define USB2_U2PTX (volatile unsigned long *)(0x5550A004)		// USB PHY tX control
#define USB2_U2PPLL (volatile unsigned long *)(0x5550A000)		// USB PHY PLL control
#define	USB2_PLL_READY					BIT23			/* PLL_READY */
#define	USB2_PLL_VCOCAL_START			BIT21			/* VCOCAL_START */
#define USB2_REG_RCAL_START				BIT13			// REG_RCAL_START
#define SQ_THRESH_MASK 0xFFFFFF3F 								//SQ Thresh mask fo U2PRX register to set SQ_THRESH to 3

#endif


// Pad Power Supply and Control Register Addresses.
// Using the PAD Name as the address name to be accessed.
// Defining only relevant ones to the boot ROM
#define MD0 		(volatile unsigned long *)( 0x40E10000 )  // MD0 Pad
#define MD1 		(volatile unsigned long *)( 0x40E10004 )  // MD1 Pad
#define nCS0 		(volatile unsigned long *)( 0x40E100C0 )  //
#define nCS1 		(volatile unsigned long *)( 0x40E100C4 )  //
#define EMPI_SCLK 	(volatile unsigned long *)( 0x40E100CC )  //

#define U2DOTGCR_	(volatile unsigned long *)( 0x54100020 )  //U2D OTG Control Register
#define U2DOTGICR_	(volatile unsigned long *)( 0x54100024 )  //U2D OTG Interrupt Control Register
#define U2DOTGUCR_	(volatile unsigned long *)( 0x54100030 )  //U2D OTG ULPI Control Register

// Tavor UART GPIO definition
#define GPIO47_TAVOR (volatile unsigned long *)( 0x40E102BC )  // FFUART_DSR
#define GPIO48_TAVOR (volatile unsigned long *)( 0x40E102B4 )  // FFUART_DTR
#define GPIO49_TAVOR (volatile unsigned long *)( 0x40E102B8 )  // FFUART_RI
#define GPIO50_TAVOR (volatile unsigned long *)( 0x40E102C8 )  // FFUART_DCD
#define GPIO51_TAVOR (volatile unsigned long *)( 0x40E102C0 )  // FFUART_CTS
#define GPIO52_TAVOR (volatile unsigned long *)( 0x40E102C4 )  // FFUART_RTS
#define GPIO53_TAVOR (volatile unsigned long *)( 0x40E102D0 )  // FFUART_RXD
#define GPIO54_TAVOR (volatile unsigned long *)( 0x40E102CC )  // FFUART_TXD
#define GPIO63_TAVOR (volatile unsigned long *)( 0x40E10640 )  // ALT UART TXD
#define GPIO64_TAVOR (volatile unsigned long *)( 0x40E1065C )  // ALT UART RXD
// Tavor USB PORT_2 GPIO definition
#define GPIO70_TAVOR (volatile unsigned long *)( 0x40E102A0 )  // USB_P2_4
#define GPIO71_TAVOR (volatile unsigned long *)( 0x40E10294 )  // USB_P2_8
#define GPIO72_TAVOR (volatile unsigned long *)( 0x40E10298 )  // USB_P2_5
#define GPIO73_TAVOR (volatile unsigned long *)( 0x40E1029C )  // USB_P2_2
#define GPIO74_TAVOR (volatile unsigned long *)( 0x40E102A8 )  // USB_P2_7
#define GPIO75_TAVOR (volatile unsigned long *)( 0x40E102A4 )  // USB_P2_1
#define GPIO76_TAVOR (volatile unsigned long *)( 0x40E102B0 )  // USB_P2_3
#define GPIO77_TAVOR (volatile unsigned long *)( 0x40E102AC )  // USB_P2_6

// Tavor ULPI GPIO's
#define GPIO30 		(volatile unsigned long *)( 0x40E10618 )  // ULPI Data 0
#define GPIO31 		(volatile unsigned long *)( 0x40E10610 )  // .
#define GPIO33 		(volatile unsigned long *)( 0x40E1061C )  // .
#define GPIO34 		(volatile unsigned long *)( 0x40E10620 )  // .
#define GPIO35 		(volatile unsigned long *)( 0x40E10628 )  // .
#define GPIO36 		(volatile unsigned long *)( 0x40E1062C )  // .
#define GPIO37 		(volatile unsigned long *)( 0x40E10630 )  // ULPI Data 6
#define GPIO38		(volatile unsigned long *)( 0x40E10634 )  // ULPI_CLK
#define GPIO39 		(volatile unsigned long *)( 0x40E10638 )  // ULPI STP
#define GPIO40 		(volatile unsigned long *)( 0x40E1063C )  // ULPI Data 7
#define GPIO41 		(volatile unsigned long *)( 0x40E10614 )  // ULPI DIR
#define GPIO42 		(volatile unsigned long *)( 0x40E10624 )  // ULPI NXT

// INTERRUPT INFO
//
#define DMAInterruptMask			0x2000000
#define WTMINTERRUPTMASK			0x1
#define USBInterruptMask			0x800
#define FFUARTInterruptMask			0x400000
#define U2DInterruptMask            0x000004000
#define MMC1_InterruptMask			0x00800000
#define MMC2_InterruptMask			0x20
#define GPIO_XInterruptMask         0x200

#if (PV_CHIP)
#define CI2InterruptMask			0x1
#define MMCX_MM4InterruptMask		0x700
#endif

#define MAX_ICIP1_INTS    		32
#define MAX_ICIP2_INTS          64
#if (PV_CHIP)
#define MAX_PLATFORM_INTS       75
#else
#define MAX_PLATFORM_INTS       56
#endif

#if (PV_CHIP)
// Tavor PV/P65 interrupts
typedef enum
{
    SSP_3_INT               = 0,
    MSL_INT					= 1,
    USB_HOST_2_INT          = 2,
    USB_HOST_1_INT          = 3,
    KEYPAD_CNTL_INT         = 4,
    AC_IPC0_INT		        = 5,
    POWER_I2C_INT           = 6,
    OS_TIMER_INT4_11        = 7,
    GPIO_0_INT              = 8,
    GPIO_1_INT              = 9,
    GPIO_2_120_INT          = 10,
    USB_CLIENT_INT          = 11,
    PML_INT		            = 12,
    SSP_4_INT               = 13,
    AC97_CNTL_INT           = 14,
    USIM_INTERFACE_INT      = 15,
    SSP_2_INT               = 16,
    LCD_CNTL_INT            = 17,
    I2C_INT                 = 18,
    AC_ICP2_INT             = 19,
    STUART_INT              = 20,
    BTUART_INT              = 21,
    FFUART_INT              = 22,
    MMC1_CNTL_INT 			= 23,
    SSP_1_INT               = 24,
    DMA_CNTL_INT            = 25,
    OS_TIMER_0_INT          = 26,
    OS_TIMER_1_INT          = 27,
    OS_TIMER_2_INT          = 28,
    OS_TIMER_3_INT          = 29,
    RTC_ONE_HZ_CLK_INT      = 30,
    RTC_EQUAL_ALARM_REG_INT = 31,
    TPM_UNIT_INT            = 32,
    CAMERA_INTERFACE_INT   	= 33,
    CONSUMER_INFRARED_INT   = 34,
    COMM_WDT_INT            = 35,
    RESERVED1               = 36,
    ENHANCED_ROTARY_INT     = 37,
    RESERVED2               = 38,
    GRAPHICS_CNTL_INT       = 39,
	AC_IPC1_INT             = 40,
   	MMC2_CNTL_INT 			= 41,
    RESERVED3               = 42,
    TRACKBALL_INT           = 43,
    ONE_WIRE_INT            = 44,
    NAND_INT              	= 45,
    U2D_CLIENT_INT          = 46,
	SGP						= 47,
	MVED_DMA				= 48,
    EXT_WAKEUP0_INT         = 49,
    EXT_WAKEUP1_INT         = 50,
	DMEM_INT				= 51,
	SGP_CPU_CCU_INT			= 52,
	BPB2IMG_INT				= 53,
	MVED_INT				= 54,
    RESERVED4 				= 55,
    RESERVED5 				= 56,
    RESERVED6 				= 57,
    RESERVED7 				= 58,
    RESERVED8 				= 59,
    RESERVED9 				= 60,
    RESERVED10 				= 61,
    RESERVED11 				= 62,
    RESERVED12 				= 63,
	USB0_OTG_INT			= 64,
	USB0_HOST1_INT			= 65,
    RESERVED13 				= 66,
    RESERVED14 				= 67,
    RESERVED15 				= 68,
    RESERVED16 				= 69,
    RESERVED17 				= 70,
    RESERVED18 				= 71,
	MMC0_MM4_INT			= 72,
	MMC1_MM4_INT			= 73,
	MMC2_MM4_INT			= 74,
   USB0_OTG_SE1         = 93
}BULVERDE_INTC_T;


#else


// Boerne interrupts
typedef enum
{
    SSP_3_INT               = 0,
    BASEBAND_INTERFACE_INT  = 1,
    USB_HOST_2_INT          = 2,
    USB_HOST_1_INT          = 3,
    KEYPAD_CNTL_INT         = 4,
    MEMORY_STICK_INT        = 5,
    POWER_I2C_INT           = 6,
    OS_TIMER_INT4_11        = 7,
    GPIO_0_INT              = 8,
    GPIO_1_INT              = 9,
    GPIO_2_120_INT          = 10,
    USB_CLIENT_INT          = 11,
    MEGACELL_INT            = 12,
    I2S_INT                 = 13,
    AC97_CNTL_INT           = 14,
    USIM_INTERFACE_INT      = 15,
    SSP_2_INT               = 16,
    LCD_CNTL_INT            = 17,
    I2C_INT                 = 18,
    ICP_INT                 = 19,
    STUART_INT              = 20,
    BTUART_INT              = 21,
    FFUART_INT              = 22,
    MMC1_CNTL_INT 			= 23,
    SSP_1_INT               = 24,
    DMA_CNTL_INT            = 25,
    OS_TIMER_0_INT          = 26,
    OS_TIMER_1_INT          = 27,
    OS_TIMER_2_INT          = 28,
    OS_TIMER_3_INT          = 29,
    RTC_ONE_HZ_CLK_INT      = 30,
    RTC_EQUAL_ALARM_REG_INT = 31,
    TPM_UNIT_INT            = 32,
    CAPTURE_INTERFACE_INT   = 33,
    RESERVED0               = 34,
    RESERVED1               = 35,
    RESERVED2               = 36,
    RESERVED3               = 37,
    RESERVED4               = 38,
    RESERVED5               = 39,
    RESERVED6               = 40,
    MMC2_CNTL_INT 			= 41,
    RESERVED8               = 42,
    RESERVED9               = 43,
    RESERVED10              = 44,
    RESERVED11              = 45,
    U2D_CLIENT_INT          = 46,
	SGP						= 47,
	MVED_DMA				= 48,
    RESERVED12              = 49,
    RESERVED13              = 50,
	DMEM_INT				= 51,
	RESERVED14 				= 52,
	BPB2IMG					= 53,
	MVED					= 54,
    FLASH_CARD_INTERFACE_MMC3_INT = 55,
}BULVERDE_INTC_T;

#endif

//
// Relevant and common GPIO memory mapped register needed for configuring flash
// signal list.
//
#define APPS_PAD_BASE  0x40E10000
#define SMEMC_BASE	   0x4A000000

// OneNand
#define FLEX_BOOTRAM_MAIN_BASE	0x10000000

//
// Multifunction Padring Structures
//
typedef struct
{
  volatile int *    registerAddr;
  unsigned int      regValue;
  unsigned int      defaultValue;
}CS_REGISTER_PAIR_S, *P_CS_REGISTER_PAIR_S;


//Prototypes
void ChipSelect2(void);
UINT_T ChipSelect0(void);
void ChipSelectDFC(void);
void RestoreDefaultConfig(void);
void SaveDefaultConfig(void);
void ChipSelectOneNAND( void );
UINT_T *PlatformGetOneNANDBaseAddr( void );
UINT_T getPlatformPortSelection(unsigned int *);
unsigned int GetDefaultUsbPort( void );
UINT_T PlatformUARTConfig(void);
UINT_T PlatformAltUARTConfig(void);
//UDC 1.1 Functions
UINT_T SetupSingleEndedPort2(void);
UINT_T PlatformUsbTransmit(UINT_T, UINT8_T*);
void PlatformUsbInit(UINT_T);
void PlatformUsbShutdown(void);
void PlatformUsbDmaHandler(void);
void PlatformUsbInterruptHandler(void);
//U2D Functions
void PlatformU2DInit(void);
UINT_T PlatformU2DTransmit(UINT_T, UINT8_T*, UINT_T);
void PlatformU2DShutdown(void);
void PlatformU2DInterruptHandler(void);

#if (USBCI)
//CI2 USB Functions
void PlatformCI2Init(void);
UINT_T PlatformCI2Transmit(UINT_T, UINT8_T*, UINT_T);
void PlatformCI2Shutdown(void);
void PlatformCI2InterruptHandler(void);
#endif

//external prototypes

extern UINT_T GetOSCR0(void);
extern UINT_T OSCR0IntervalInSec(UINT_T Before, UINT_T After);
extern UINT_T OSCR0IntervalInMilli(UINT_T Before, UINT_T After);
extern UINT_T OSCR0IntervalInMicro(UINT_T Before, UINT_T After);

#endif

