#ifndef PRDCFG_ELEMENT_LOOKUP_TBL_H
#define PRDCFG_ELEMENT_LOOKUP_TBL_H
/*******************************************************************************
Copyright (C), 200{x}, Borqs Tech. Co., Ltd.
File Name: 
    PRDCFG_Element_lookup_tbl.h
Description: 
 
History:  
    Date          Author                         Comments
    -----------    ---------------------------    --------------------
    2008-08-13      Bai yu                                          Created
*******************************************************************************/

/*==================================================================================================
                                           INCLUDE FILES
==================================================================================================*/
#include "PRDCFG_General.h"


/*==================================================================================================
                                           STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/
typedef struct
{
    PRDCFG_ELEMENT_OFFSET offset;
    int length;
    PRDCFG_ELEMENT_ATTRIBUTES attribute;

}PRDCFG_CFG_ELEMENT_TABLE;

typedef struct
{
    PRDCFG_ELEMENT_OFFSET offset;
    PRDCFG_ELEMENT_ATTRIBUTES attribute;
}PRDCFG_BIT_CFG_ELEMENT_TABLE;

typedef struct
{
    PRDCFG_ELEMENT_OFFSET offset;
    int length;
    PRDCFG_ELEMENT_ATTRIBUTES attribute;
}PRDCFG_PHASING_ELEMENT_TABLE;

typedef struct
{
    PRDCFG_ELEMENT_OFFSET offset;
    int length;
    PRDCFG_ELEMENT_ATTRIBUTES attribute;
}PRDCFG_OTP_ELEMENT_TABLE;
/*==================================================================================================
                                           GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/
extern const PRDCFG_CFG_ELEMENT_TABLE  prdcfg_cfg_element_table[];
extern const PRDCFG_BIT_CFG_ELEMENT_TABLE prdcfg_bit_cfg_element_table[];
extern const PRDCFG_PHASING_ELEMENT_TABLE prdcfg_phasing_element_table[];
extern const PRDCFG_OTP_ELEMENT_TABLE prdcfg_otp_element_table[];

/*==================================================================================================
                                           ENUMS
==================================================================================================*/

enum
{
    PRDCFG_EEH_LIVETIME_REPORT_EN_OFFSET = 0x00 ,
    PRDCFG_EFEM_FEATURE_EN_OFFSET,
    PRDCFG_EFEM_FEATURE_EN_DEFAULT_OFFSET,
    PRDCFG_DIAG_LAUNCH_EN_OFFSET,
    PRDCFG_DIAG_LAUNCH_EN_DEGAULT_OFFSET,
};

/************************    Access permission bit mask definition *******************************************/
#define PRDCFG_ELEMENT_ACCESS_FORBID    0x00
#define PRDCFG_ELEMENT_READ_ALLOW               0x01
#define PRDCFG_ELEMENT_WRITE_ALLOW             0x02
#define PRDCFG_ELEMENT_READ_WRITE_ALLOW  (PRDCFG_ELEMENT_READ_ALLOW | PRDCFG_ELEMENT_WRITE_ALLOW)
#define PRDCFG_ELEMENT_MC_DEFAULT_VALUE   0x04     //default value, should be copied back when MC
#define PRDCFG_ELEMENT_MR_DEFAULT_VALUE   0x08     //default value, should be copied back when MR
#define  PRDCFG_ELEMENT_SECURITY_BIT_MASK  0x10

/****************************************   Element frame structure  ******************************************
                  ______________________________________________________________________
                 |       |                                                   |       |                                                    |
                 | 5A  | Element data (Element length -4)  | A5  |    Element checksum ( 2bytes )       |
                 |____|_____________________________|____|______________________________|

 ********************************** ********************************** ***********************************/
#define PRDCFG_ELEMENT_DATA_PREAMBLE 0x5A
#define PRDCFG_ELEMENT_DATA_EPILOGUE 0xA5
#define PRDCFG_ELEMENT_DATA_PREAMBLE_EPILOGUE_LENGTH 1
#define PRDCFG_ELEMENT_DATA_CHECKSUM_LENGTH 2
#define PRDCFG_ELEMENT_DATA_APPEND_LENGTH  (PRDCFG_ELEMENT_DATA_PREAMBLE_EPILOGUE_LENGTH*2 + \
                            PRDCFG_ELEMENT_DATA_CHECKSUM_LENGTH) 
#define PRDCFG_PHASING_ELEMENT_LENGTH_FIELD_SIZE 4
#define PRDCFG_RF_NVM_FILE_NAME_LENGTH 64
#define PRDCFG_RF_NVM_FILE_SIZE_FIELD_LENGTH 64
#define PRDCFG_RF_NVM_FILE_HEADER 0x1E
#define PRDCFG_RF_STUFF_HEADER 0x2E


/******************************      PRDCFG   memory  map      ******************************
                                          ____     ______________________
                                                     |       Product config         |
                                           256     |      control element        |
                                          ____    |____________________ |
                                                     |                                    |
                                                     |     Bit configuration        |
                                           4 K     |                                    |
                                                     |          element               |
                                          ____    |____________________ |
                                                     |                                    |
                                                     |                                    |
                                                     |   Normal configuration   |
                                          500K    |                                    |
                                                     |          elements             |
                                                     |                                    |
                                          ____    |____________________ |
                                                     |                                    |
                                                     |                                    |
                                                     |                                    |
                                          256K    |        Phasing files          |
                                                     |                                    |
                                                     |                                    |
                                          ____    |____________________ |


/**********************************    Element length                 *******************************************/
#if 0  // only for x86 test
#define PRDCFG_WHOLE_IMG_SIZE     0xA524    //  544 bytes
#define PRDCFG_VERSION_INFO_GROUP_SIZE  0x10  //16bytes
#define PRDCFG_BIT_CFG_GROUP_SIZE 0x10   // 16 bytes
#define PRDCFG_CFG_GROUP_SIZE  0x100  // 256 bytes
#define PRDCFG_PHASING_FILE_SIZE  0xA404 // 41K bytes

// element size
#define PRDCFG_VERSION_INFO_SIZE    0x10   //16 bytes

#define PRDCFG_TCMD_ELEMENT_ID_SIZE     0x08   //8 types
#define PRDCFG_CFG_TEST_1_SIZE     0x06   //6 types
#define PRDCFG_CFG_TEST_2_SIZE     0x0A   //10 types
#define PRDCFG_CFG_TEST_3_SIZE     0x0E   //14 types
#define PRDCFG_CFG_TEST_4_SIZE     0x14   //16 types




#define PRDCFG_PHASING_RF_PARA_SIZE  0xA004   // 40K+4 bytes
#define PRDCFG_PHASING_VOLTAGE_SIZE  0x22  // 34 bytes
#define PRDCFG_PHASING_TEMPERATURE_SIZE  0x22  // 34 bytes
#define PRDCFG_PHASING_BT_ADDR_SIZE  0x0A  // 10 bytes
#define PRDCFG_PHASING_IMEI_SIZE  0x10// 16 types

#define PRDCFG_OTP_IMEI_SIZE   0x40  
#else
// group size
#define PRDCFG_WHOLE_IMG_SIZE    0xC0000    //  768K bytes
#define PRDCFG_FACTORY_LOG_SIZE  0x0400  // 1K bytes
#define PRDCFG_VERSION_INFO_GROUP_SIZE  0x100   //256 bytes
#define PRDCFG_BIT_CFG_GROUP_SIZE 0x1000   // 4K bytes
#define PRDCFG_CFG_GROUP_SIZE  0x7EB00  // about 500K bytes
#define PRDCFG_PHASING_FILE_SIZE  0x20000 // 128K bytes

// element size
#define PRDCFG_VERSION_INFO_SIZE    0x100   //256 bytes

// Normal config element size definition
#define PRDCFG_EEH_LIVETIME_PERIOD_SIZE     (0x01+PRDCFG_ELEMENT_DATA_APPEND_LENGTH)  // 1 bytes + 4bytes 
#define PRDCFG_DIAG_EXT_INTERFACE_SETTING_SIZE (0x01+PRDCFG_ELEMENT_DATA_APPEND_LENGTH)  // 1 bytes + 4bytes 

//NVM element size definition
#define PRDCFG_PHASING_HEADER_SIZE  (0x80)  // 128 bytes
#define PRDCFG_PHASING_SETTING_INFO_SIZE (0x4000)    //16K bytes, no header and checksum
#define PRDCFG_PHASING_RF_PARA_SIZE  (0xA000 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH + PRDCFG_PHASING_ELEMENT_LENGTH_FIELD_SIZE)  // 40k bytes
#define PRDCFG_PHASING_VOLTAGE_SIZE  (0x10 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)  // 16 bytes
#define PRDCFG_PHASING_TEMPERATURE_SIZE  (0x10 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)  // 16 bytes
#define PRDCFG_PHASING_BT_ADDR_SIZE  (0x0C + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)  // 12 bytes
#define PRDCFG_PHASING_FACTORY_ID_SIZE  (0x14 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)// 20 types
#define PRDCFG_PHASING_TEST_INFO_SIZE (0x14 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)// 20 types
#define PRDCFG_PHASING_MAT_FLAG_SIZE (0x06 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)// 6byte
#define PRDCFG_PHASING_ENG_MODE_FLAG_SIZE (0x02 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH) //2byte

#define PRDCFG_OTP_IMEI_SIZE   (0x40 + PRDCFG_ELEMENT_DATA_APPEND_LENGTH)  
#endif


/**********************************    Element offset                 *******************************************/
//  Product config  control element
#define PRDCFG_FACTORY_LOG_OFFSET    0     //  from start addr
#define PRDCFG_VERSION_INFO_GROUP_OFFSET (PRDCFG_FACTORY_LOG_OFFSET +  PRDCFG_FACTORY_LOG_SIZE)   //  from start addr

// Bit configuration element
#define PRDCFG_BIT_CFG_GROUP_OFFSET (PRDCFG_VERSION_INFO_GROUP_SIZE + PRDCFG_VERSION_INFO_GROUP_OFFSET)

// Normal configuration elements
#define PRDCFG_CFG_GROUP_OFFSET  (PRDCFG_BIT_CFG_GROUP_OFFSET+PRDCFG_BIT_CFG_GROUP_SIZE)
#define PRDCFG_EEH_LIVETIME_PERIOD_OFFSET (PRDCFG_CFG_GROUP_OFFSET+0)
#define PRDCFG_DIAG_EXT_INTERFACE_SETTING_OFFSET (PRDCFG_EEH_LIVETIME_PERIOD_OFFSET + PRDCFG_EEH_LIVETIME_PERIOD_SIZE)


// Phasing files
//#define PRDCFG_PHASING_FILE_GROUP_OFFSET  (PRDCFG_WHOLE_IMG_SIZE - PRDCFG_PHASING_FILE_SIZE )   //  phasing files block always locate at end of whole PRDCG block
#define PRDCFG_PHASING_FILE_GROUP_OFFSET  0  //  phasing files block always locate at end of whole PRDCG block
#define PRDCFG_PHASING_HEADER_OFFSET  (PRDCFG_PHASING_FILE_GROUP_OFFSET+0)
#define PRDCFG_PHASING_SETTING_INFO_OFFSET (PRDCFG_PHASING_HEADER_OFFSET+PRDCFG_PHASING_HEADER_SIZE)
#define PRDCFG_PHASING_RF_PARA_OFFSET   (PRDCFG_PHASING_SETTING_INFO_OFFSET+PRDCFG_PHASING_SETTING_INFO_SIZE)
#define PRDCFG_PHASING_VOLTAGE_OFFSET  (PRDCFG_PHASING_RF_PARA_OFFSET + PRDCFG_PHASING_RF_PARA_SIZE)
#define PRDCFG_PHASING_TEMPERATURE_OFFSET (PRDCFG_PHASING_VOLTAGE_OFFSET + PRDCFG_PHASING_VOLTAGE_SIZE) 
#define PRDCFG_PHASING_BT_ADDR_OFFSET (PRDCFG_PHASING_TEMPERATURE_OFFSET + PRDCFG_PHASING_TEMPERATURE_SIZE)
#define PRDCFG_PHASING_FACTORY_ID_OFFSET (PRDCFG_PHASING_BT_ADDR_OFFSET + PRDCFG_PHASING_BT_ADDR_SIZE)
#define PRDCFG_PHASING_TEST_INFO_OFFSET (PRDCFG_PHASING_FACTORY_ID_OFFSET + PRDCFG_PHASING_FACTORY_ID_SIZE)
#define PRDCFG_PHASING_MAT_FLAG_OFFSET (PRDCFG_PHASING_TEST_INFO_OFFSET + PRDCFG_PHASING_TEST_INFO_SIZE)
#define PRDCFG_PHASING_ENG_MODE_FLAG_OFFSET (PRDCFG_PHASING_MAT_FLAG_OFFSET+PRDCFG_PHASING_MAT_FLAG_SIZE)

// OTP data
#define PRDCFG_OTP_IMEI_OFFSET 0   // form start of OTP region   

#endif

