#include "obm_pxa_ssp.h"
#include "misc.h"

#define TIMEOUT 100000



static struct resource pxa27x_resource_ssp1[] = {
	[0] = {
		.start	= 0x41000000,
		.end	= 0x4100003f,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= IRQ_SSP,
		.end	= IRQ_SSP,
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		/* DRCMR for RX */
		.start	= 13,
		.end	= 13,
		.flags	= IORESOURCE_DMA,
	},
	[3] = {
		/* DRCMR for TX */
		.start	= 14,
		.end	= 14,
		.flags	= IORESOURCE_DMA,
	},
};


int ssp_config(struct ssp_dev *dev, u32 mode, u32 flags, u32 psp_flags, u32 speed)
{
	struct ssp_device *ssp = dev->ssp;

	dev->mode = mode;
	dev->flags = flags;
	dev->psp_flags = psp_flags;
	dev->speed = speed;

	/* set up port type, speed, port settings */
	__raw_writel((dev->speed | dev->mode), ssp->mmio_base + SSCR0);
	__raw_writel(dev->flags, ssp->mmio_base + SSCR1);
	__raw_writel(dev->psp_flags, ssp->mmio_base + SSPSP);

	return 0;
}

void ssp_enable(struct ssp_dev *dev)
{
	struct ssp_device *ssp = dev->ssp;
	uint32_t sscr0;

	sscr0 = __raw_readl(ssp->mmio_base + SSCR0);
	sscr0 |= SSCR0_SSE;
	__raw_writel(sscr0, ssp->mmio_base + SSCR0);
}

void ssp_disable(struct ssp_dev *dev)
{
	struct ssp_device *ssp = dev->ssp;
	uint32_t sscr0;

	sscr0 = __raw_readl(ssp->mmio_base + SSCR0);
	sscr0 &= ~SSCR0_SSE;
	__raw_writel(sscr0, ssp->mmio_base + SSCR0);
}

static struct resource pxa3xx_resource_ssp4[] = {
	[0] = {
		.start	= 0x41a00000,
		.end	= 0x41a0003f,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= IRQ_SSP4,
		.end	= IRQ_SSP4,
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		/* DRCMR for RX */
		.start	= 2,
		.end	= 2,
		.flags	= IORESOURCE_DMA,
	},
	[3] = {
		/* DRCMR for TX */
		.start	= 3,
		.end	= 3,
		.flags	= IORESOURCE_DMA,
	},
};

struct platform_device pxa3xx_device_ssp4 = {
	/* PXA3xx SSP is basically equivalent to PXA27x */
	.name		= "pxa27x-ssp",
	.id		= 3,
//	.dev		= {
//		.dma_mask = &pxa3xx_ssp4_dma_mask,
//		.coherent_dma_mask = DMA_BIT_MASK(32),
//	},
	.resource	= pxa3xx_resource_ssp4,
	.num_resources	= ARRAY_SIZE(pxa3xx_resource_ssp4),
};


struct platform_device pxa27x_device_ssp1 = {
	.name		= "pxa27x-ssp",
	.id		= 0,
#if 0
	.dev		= {
		.dma_mask = &pxa27x_ssp1_dma_mask,
		.coherent_dma_mask = DMA_BIT_MASK(32),
	},
#endif
	.resource	= pxa27x_resource_ssp1,
	.num_resources	= ARRAY_SIZE(pxa27x_resource_ssp1),
};





static void clk_pxa3xx_cken_enable(struct clk *clk)
{
	unsigned long mask = 1ul << (clk->cken & 0x1f);

	local_irq_disable();

	if (clk->cken < 32)
		CKENA |= mask;
#if SAAR_PV
    else if (clk->cken < 64)
		CKENB |= mask;
	else
		CKENC |= mask;
#else
	else
		CKENB |= mask;
#endif

	local_irq_enable();
}

static void clk_pxa3xx_cken_disable(struct clk *clk)
{
	unsigned long mask = 1ul << (clk->cken & 0x1f);

	local_irq_disable();

	if (clk->cken < 32)
		CKENA &= ~mask;
#if SAAR_PV
    else if (clk->cken < 64)
		CKENB &= ~mask;
	else
		CKENC &= ~mask;
#else
	else
		CKENB &= ~mask;
#endif

	local_irq_enable();
}

static const struct clkops clk_pxa3xx_cken_ops = {
	.enable		= clk_pxa3xx_cken_enable,
	.disable	= clk_pxa3xx_cken_disable,
};

static struct clk pxa3xx_clks[] = {
#if SAAR_PV
    PXA3xx_CKEN("SSPCLK", SSP1, 13000000, 0, 0),//&pxa3xx_device_ssp1.dev),
#else
    PXA3xx_CKEN("SSPCLK", SSP4, 13000000, 0, 0),//&pxa3xx_device_ssp4.dev),
#endif
};
static struct ssp_device pxa_ssp;

int ssp_init(struct ssp_dev *dev, u32 port, u32 init_flags)
{
#if SAAR_PV

	struct ssp_device *ssp;
	int ret;

	//ssp = ssp_request(port, "SSP");
    ssp = &pxa_ssp;
    //gliethttp_20080508 mainly from ssp_probe functions
    {
        struct resource *res;
        struct platform_device *pdev;
        
        pdev = &pxa27x_device_ssp1;

        ssp->clk = &pxa3xx_clks[0];

        res = &pxa27x_resource_ssp1[0];
        ssp->phys_base = res->start;
        ssp->mmio_base = (void __iomem *)res->start;
        
        res = &pxa27x_resource_ssp1[2];
        ssp->drcmr_rx = res->start;
        
        res = &pxa27x_resource_ssp1[3];
        ssp->drcmr_tx = res->start;
        
        ssp->port_id = pdev->id + 1;
	    ssp->use_count = 0;
	    ssp->type = PXA27x_SSP;
    }
    
    if(ssp->use_count == 0)
    {
        ssp->port_id = port;
        ssp->use_count++;
        ssp->label = "SSP";
    }else ssp = NULL;
    
	if (ssp == NULL)
		return -10;//ENODEV;

	dev->ssp = ssp;
	dev->port = port;

	/* do we need to get irq */
//	if (!(init_flags & SSP_NO_IRQ)) {
//		ret = request_irq(ssp->irq, ssp_interrupt,
//				0, "SSP", dev);
//	    	if (ret)
//			goto out_region;
//		dev->irq = ssp->irq;
//	} else
		dev->irq = 0;

	/* turn on SSP port clock */
	clk_enable(ssp->clk);
	return 0;

out_region:
//	ssp_free(ssp);
	return -1;//ret;
#else
	struct ssp_device *ssp;
	int ret;

	//ssp = ssp_request(port, "SSP");
    ssp = &pxa_ssp;
    //gliethttp_20080508 mainly from ssp_probe functions
    {
        struct resource *res;
        struct platform_device *pdev;
        
        pdev = &pxa3xx_device_ssp4;

        ssp->clk = &pxa3xx_clks[0];

        res = &pxa3xx_resource_ssp4[0];
        ssp->phys_base = res->start;
        ssp->mmio_base = (void __iomem *)res->start;
        
        res = &pxa3xx_resource_ssp4[2];
        ssp->drcmr_rx = res->start;
        
        res = &pxa3xx_resource_ssp4[3];
        ssp->drcmr_tx = res->start;
        
        ssp->port_id = pdev->id + 1;
	    ssp->use_count = 0;
	    ssp->type = PXA27x_SSP;
    }
    
    if(ssp->use_count == 0)
    {
        ssp->port_id = port;
        ssp->use_count++;
        ssp->label = "SSP";
    }else ssp = NULL;
    
	if (ssp == NULL)
		return -10;//ENODEV;

	dev->ssp = ssp;
	dev->port = port;

	/* do we need to get irq */
//	if (!(init_flags & SSP_NO_IRQ)) {
//		ret = request_irq(ssp->irq, ssp_interrupt,
//				0, "SSP", dev);
//	    	if (ret)
//			goto out_region;
//		dev->irq = ssp->irq;
//	} else
		dev->irq = 0;

	/* turn on SSP port clock */
	clk_enable(ssp->clk);
	return 0;

out_region:
//	ssp_free(ssp);
	return -1;//ret;
#endif
}

static void ssp_reconfig(struct ssp_dev *dev, int nparam)
{
        static int last_nparam = -1;
    	int sscr0;

        /* check if it is necessary to re-config SSP */
        if (nparam == last_nparam)
            return;

        ssp_disable(dev);
#if SAAR_PV
        if( nparam >16 )
            sscr0 = 0x00107f80 | ((nparam-1) & 0xf);
        else
            sscr0 = 0x00007f80 | ((nparam-1) & 0xf);

        ssp_config(dev, sscr0, 0x18, 0, 0);
#else
        ssp_config(dev, (nparam == 2) ? 0x00107f87 : 0x00007f8f, 0x18, 0, 0);

#endif
        last_nparam = nparam;
}

int ssp_write_word(struct ssp_dev *dev, u32 data)
{
	struct ssp_device *ssp = dev->ssp;
	int timeout = TIMEOUT;

	while (!(__raw_readl(ssp->mmio_base + SSSR) & SSSR_TNF)) {
	        if (!--timeout)
	        	return -ETIMEDOUT;
		cpu_relax();
	}

	__raw_writel(data, ssp->mmio_base + SSDR);

	return 0;
}

int ssp_flush(struct ssp_dev *dev)
{
	struct ssp_device *ssp = dev->ssp;
	int timeout = TIMEOUT * 2;

	/* ensure TX FIFO is empty instead of not full */
	if (cpu_is_pxa3xx())
		while (__raw_readl(ssp->mmio_base + SSSR) & 0xf00) ;

	do {
		while (__raw_readl(ssp->mmio_base + SSSR) & SSSR_RNE) {
		        if (!--timeout)
		        	return -ETIMEDOUT;
			(void)__raw_readl(ssp->mmio_base + SSDR);
		}
	        if (!--timeout)
	        	return -ETIMEDOUT;
	} while (__raw_readl(ssp->mmio_base + SSSR) & SSSR_BSY);

	return 0;
}

#define SPARK_LCD_DSS 	9 	//Data size select for LCD spi

void ssp_send_cmd(uint32_t *cmd, int num)
{
#if SAAR_PV
    static int ssp_initialized = 0;
	static struct ssp_dev ssp1;

	int i;

	if (!ssp_initialized) {
		ssp_init(&ssp1, 1, SSP_NO_IRQ);
		ssp_initialized = 1;
	}

	clk_enable(ssp1.ssp->clk);
	for (i = 0; i < num; i++, cmd++) {
		ssp_reconfig(&ssp1, SPARK_LCD_DSS);
		ssp_write_word(&ssp1, *cmd & 0x3fffffff);

		/* FIXME: ssp_flush() is mandatory here to work */
		ssp_flush(&ssp1);
	}
	clk_disable(ssp1.ssp->clk);

#else
        static int ssp_initialized = 0;
        static struct ssp_dev ssp4;

        int i;

        if (!ssp_initialized) {
                ssp_init(&ssp4, 4, SSP_NO_IRQ);
                ssp_initialized = 1;
        }

        clk_enable(ssp4.ssp->clk);
//	printk("%s: send %d commands\n", __FUNCTION__, num);
        for (i = 0; i < num; i++, cmd++) {
                ssp_reconfig(&ssp4, (*cmd >> 30) & 0x3);
                ssp_write_word(&ssp4, *cmd & 0x3fffffff);

                /* FIXME: ssp_flush() is mandatory here to work */
                ssp_flush(&ssp4);
        Delay_ms(10);
//		mdelay(10);
        }
        clk_disable(ssp4.ssp->clk);
#endif
}

