#include "obm_sd.h"

static struct resource pxamci_resources[] = {
	[0] = {
		.start	= 0x41100000,
		.end	= 0x41100fff,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= IRQ_MMC,
		.end	= IRQ_MMC,
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		/* DRCMR for RX */
		.start	= 21,
		.end	= 21,
		.flags	= IORESOURCE_DMA,
	},
	[3] = {
		/* DRCMR for TX */
		.start	= 22,
		.end	= 22,
		.flags	= IORESOURCE_DMA,
	},
};

extern void clk_pxa3xx_cken_enable(struct clk *clk);
extern void clk_pxa3xx_cken_disable(struct clk *clk);
extern const struct clkops clk_pxa3xx_cken_ops;
static struct clk pxa3xx_clks[] =
{
    PXA3xx_CKEN("MMCCLK", MMC1, 19500000, 0, 0),//&pxa_device_mci.dev),
};

static void pxamci_start_clock(struct pxamci_host *host)
{
	writel(host->clkrt, host->base + MMC_CLKRT);
	writel(START_CLOCK, host->base + MMC_STRPCL);
}
static void pxamci_stop_clock(struct pxamci_host *host)
{
	if (readl(host->base + MMC_STAT) & STAT_CLK_EN) {
		unsigned long timeout = 100000;
		unsigned int v;

		writel(STOP_CLOCK, host->base + MMC_STRPCL);

		do {
			v = readl(host->base + MMC_STAT);
			if (!(v & STAT_CLK_EN))
				break;
			udelay(1);
		} while (timeout--);

		//if (v & STAT_CLK_EN)
		//	dev_err(mmc_dev(host->mmc), "unable to stop clock\n");
	}
}

static inline void mmc_set_ios(struct mmc_host *host)
{
	struct mmc_ios *ios = &host->ios;

	pr_debug("%s: clock %uHz busmode %u powermode %u cs %u Vdd %u "
		"width %u timing %u\n",
		 mmc_hostname(host), ios->clock, ios->bus_mode,
		 ios->power_mode, ios->chip_select, ios->vdd,
		 ios->bus_width, ios->timing);
	host->ops->set_ios(host, ios);
}
static void mmc_power_off(struct mmc_host *host)
{
	host->ios.clock = 0;
	host->ios.vdd = 0;
	if (!mmc_host_is_spi(host)) {
		host->ios.bus_mode = MMC_BUSMODE_OPENDRAIN;
		host->ios.chip_select = MMC_CS_DONTCARE;
	}
	host->ios.power_mode = MMC_POWER_OFF;
	host->ios.bus_width = MMC_BUS_WIDTH_1;
	host->ios.timing = MMC_TIMING_LEGACY;
	mmc_set_ios(host);
}

static inline void *sg_virt(struct scatterlist *sg)
{
	//return page_address(sg_page(sg)) + sg->offset;
    return (sg->dma_address);
}

int
dma_map_sg(/*struct device *dev, */struct scatterlist *sg, int nents,
		enum dma_data_direction dir)
{
	int i;
    #if 0
	dev_dbg(dev, "%s(sg=%p,nents=%d,dir=%x)\n",
		__func__, sg, nents, dir);

	BUG_ON(dir == DMA_NONE);
    #endif
	for (i = 0; i < nents; i++, sg++) {
		//struct page *page = sg_page(sg);
		unsigned int offset = sg->offset;
		unsigned int length = sg->length;
		//void *ptr = page_address(page) + offset;

		sg->dma_address += offset;//dma_address is the real address for transfering 
			//map_single(dev, ptr, length, dir);
	}

	return nents;
}

static void pxamci_setup_data(struct pxamci_host *host, struct mmc_data *data)
{
	unsigned int nob = data->blocks;
	unsigned long long clks;
	unsigned int timeout;
	u32 dcmd;
	int i;

	host->data = data;

	if (data->flags & MMC_DATA_STREAM)
		nob = 0xffff;

	writel(nob, host->base + MMC_NOB);
	writel(data->blksz, host->base + MMC_BLKLEN);

	clks = (unsigned long long)data->timeout_ns * host->clkrate;
	do_div(clks, 1000000000UL);
	timeout = (unsigned int)clks + (data->timeout_clks << host->clkrt);
	writel((timeout + 255) / 256, host->base + MMC_RDTO);

	if (data->flags & MMC_DATA_READ) {
		host->dma_dir = DMA_FROM_DEVICE;
		dcmd = DCMD_INCTRGADDR | DCMD_FLOWTRG;
		DRCMR(host->dma_drcmrtx) = 0;
		DRCMR(host->dma_drcmrrx) = host->dma | DRCMR_MAPVLD;
	} else {
		host->dma_dir = DMA_TO_DEVICE;
		dcmd = DCMD_INCSRCADDR | DCMD_FLOWSRC;
		DRCMR(host->dma_drcmrrx) = 0;
		DRCMR(host->dma_drcmrtx) = host->dma | DRCMR_MAPVLD;
	}

	dcmd |= DCMD_BURST32 | DCMD_WIDTH1;

	host->dma_len = dma_map_sg(/*mmc_dev(host->mmc), */data->sg, data->sg_len,
				   host->dma_dir);

	for (i = 0; i < host->dma_len; i++) {
		unsigned int length = sg_dma_len(&data->sg[i]);
		host->sg_cpu[i].dcmd = dcmd | length;
		if (length & 31 && !(data->flags & MMC_DATA_READ))
			host->sg_cpu[i].dcmd |= DCMD_ENDIRQEN;
		if (data->flags & MMC_DATA_READ) {
			host->sg_cpu[i].dsadr = host->res->start + MMC_RXFIFO;
			host->sg_cpu[i].dtadr = sg_dma_address(&data->sg[i]);
		} else {
			host->sg_cpu[i].dsadr = sg_dma_address(&data->sg[i]);
			host->sg_cpu[i].dtadr = host->res->start + MMC_TXFIFO;
		}
		host->sg_cpu[i].ddadr = host->sg_dma + (i + 1) *
					sizeof(struct pxa_dma_desc);
	}
	host->sg_cpu[host->dma_len - 1].ddadr = DDADR_STOP;

	__asm__ __volatile__ ("" : : : "memory");//wmb();

	DDADR(host->dma) = host->sg_dma;
	DCSR(host->dma) = DCSR_RUN;
}

static void pxamci_enable_irq(struct pxamci_host *host, unsigned int mask)
{
	unsigned long flags;

	spin_lock_irqsave(&host->lock, flags);
	host->imask &= ~mask;
	writel(host->imask, host->base + MMC_I_MASK);
	spin_unlock_irqrestore(&host->lock, flags);
}

static void pxamci_disable_irq(struct pxamci_host *host, unsigned int mask)
{
	unsigned long flags;

	spin_lock_irqsave(&host->lock, flags);
	host->imask |= mask;
	writel(host->imask, host->base + MMC_I_MASK);
	spin_unlock_irqrestore(&host->lock, flags);
}

static void pxamci_finish_request(struct pxamci_host *host, struct mmc_request *mrq)
{
	host->mrq = NULL;
	host->cmd = NULL;
	host->data = NULL;
	//mmc_request_done(host->mmc, mrq);
	//unset_dvfm_constraint();
#ifdef CONFIG_ANDROID_POWER
	//android_unlock_suspend(&pxamci_suspend_lock);
#endif
}

static int pxamci_cmd_done(struct pxamci_host *host, unsigned int stat)
{
	struct mmc_command *cmd = host->cmd;
	int i;
	u32 v;

	if (!cmd)
		return 0;

	host->cmd = NULL;

	/*
	 * Did I mention this is Sick.  We always need to
	 * discard the upper 8 bits of the first 16-bit word.
	 */
	v = readl(host->base + MMC_RES) & 0xffff;
	for (i = 0; i < 4; i++) {
		u32 w1 = readl(host->base + MMC_RES) & 0xffff;
		u32 w2 = readl(host->base + MMC_RES) & 0xffff;
		cmd->resp[i] = v << 24 | w1 << 8 | w2 >> 8;
		v = w2;
	}

	if (stat & STAT_TIME_OUT_RESPONSE) {
		cmd->error = -ETIMEDOUT;
	} else if (stat & STAT_RES_CRC_ERR && cmd->flags & MMC_RSP_CRC) {
#ifdef CONFIG_PXA27x
		/*
		 * workaround for erratum #42:
		 * Intel PXA27x Family Processor Specification Update Rev 001
		 * A bogus CRC error can appear if the msb of a 136 bit
		 * response is a one.
		 */
		if (cmd->flags & MMC_RSP_136 && cmd->resp[0] & 0x80000000) {
			pr_debug("ignoring CRC from command %d - *risky*\n", cmd->opcode);
		} else
#endif
		cmd->error = -EILSEQ;
	}

	pxamci_disable_irq(host, END_CMD_RES);
	if (host->data && !cmd->error) {
		pxamci_enable_irq(host, DATA_TRAN_DONE);
	} else {
		pxamci_finish_request(host, host->mrq);
	}

	return 1;
}

static void pxamci_start_cmd(struct pxamci_host *host, struct mmc_command *cmd, unsigned int cmdat);
static int pxamci_data_done(struct pxamci_host *host, unsigned int stat)
{
	struct mmc_data *data = host->data;
	int odd, i;
	u8 *buf, tmp;

	if (!data)
		return 0;

	DCSR(host->dma) = 0;
	//dma_unmap_sg(mmc_dev(host->mmc), data->sg, host->dma_len,
	//	     host->dma_dir);

	if (stat & STAT_READ_TIME_OUT)
		data->error = -ETIMEDOUT;
	else if (stat & (STAT_CRC_READ_ERROR|STAT_CRC_WRITE_ERROR))
		data->error = -EILSEQ;

	/*
	 * There appears to be a hardware design bug here.  There seems to
	 * be no way to find out how much data was transferred to the card.
	 * This means that if there was an error on any block, we mark all
	 * data blocks as being in error.
	 */
	if (!data->error)
		data->bytes_xfered = data->blocks * data->blksz;
	else
		data->bytes_xfered = 0;

	pxamci_disable_irq(host, DATA_TRAN_DONE);

	host->data = NULL;
	if (host->mrq->stop) {
		pxamci_stop_clock(host);
		pxamci_start_cmd(host, host->mrq->stop, host->cmdat);
	} else {
		odd = data->blksz%32;
		if (odd) {
			buf = (u8 *)sg_virt(data->sg);
			if (data->flags & MMC_DATA_READ) {
				for (i = 0; i < 32; i++) {
					tmp = readb(host->base + MMC_RXFIFO);
					if (i >= 32 - odd) {
						buf[i+odd-32] = tmp;
					}
				}
			}
		}
		pxamci_finish_request(host, host->mrq);
	}

	return 1;
}

static int wait_for_sd_complete(struct pxamci_host *host, unsigned int mask)
{
    //Leith
    //pick up from pxamci_irq() =>pxamci_cmd_done
    //                          =>pxamci_data_done
    //                          =>mmc_signal_sdio_irq
	unsigned int ireg;
    int handled;
__go_on_receive:
    
    handled = 0;
    
	do
    {
        Delay_us(500);
        ireg = readl(host->base + MMC_I_REG) & ~readl(host->base + MMC_I_MASK);
    }while(ireg == 0);
    
	if (ireg) {
		unsigned stat = readl(host->base + MMC_STAT);

		pr_debug("PXAMCI: irq %08x stat %08x\n", ireg, stat);

		if (ireg & END_CMD_RES)
			handled |= pxamci_cmd_done(host, stat);
		if (ireg & DATA_TRAN_DONE)
			handled |= pxamci_data_done(host, stat);
#if 0
		if (ireg & SDIO_INT) {
			mmc_signal_sdio_irq(host->mmc);
			handled = 1;
		}
#endif
        if(host->data)
        {
            //wait for dma complete
            //pxamci_finish_request
            goto __go_on_receive;//also has data stage
        }
	}
}

static void pxamci_start_cmd(struct pxamci_host *host, struct mmc_command *cmd, unsigned int cmdat)
{
	WARN_ON(host->cmd != NULL);
	host->cmd = cmd;

	if (cmd->flags & MMC_RSP_BUSY)
		cmdat |= CMDAT_BUSY;

#define RSP_TYPE(x)	((x) & ~(MMC_RSP_BUSY|MMC_RSP_OPCODE))
	switch (RSP_TYPE(mmc_resp_type(cmd))) {
	case RSP_TYPE(MMC_RSP_R1): /* r1, r1b, r6, r7 */
		cmdat |= CMDAT_RESP_SHORT;
		break;
	case RSP_TYPE(MMC_RSP_R3):
		cmdat |= CMDAT_RESP_R3;
		break;
	case RSP_TYPE(MMC_RSP_R2):
		cmdat |= CMDAT_RESP_R2;
		break;
	default:
		break;
	}

	writel(cmd->opcode, host->base + MMC_CMD);
	writel(cmd->arg >> 16, host->base + MMC_ARGH);
	writel(cmd->arg & 0xffff, host->base + MMC_ARGL);
	writel(cmdat, host->base + MMC_CMDAT);

	pxamci_start_clock(host);

	pxamci_enable_irq(host, END_CMD_RES);

    if(wait_for_sd_complete(host, END_CMD_RES) < 0)
    {
        //
    }
}

static void pxamci_request(struct mmc_host *mmc, struct mmc_request *mrq)
{
	struct pxamci_host *host = mmc_priv(mmc);
	unsigned int cmdat;

	WARN_ON(host->mrq != NULL);

	/* release this constraint when request is finished */
	//set_dvfm_constraint();
#ifdef CONFIG_ANDROID_POWER
	//android_lock_suspend(&pxamci_suspend_lock);
#endif

	host->mrq = mrq;

	pxamci_stop_clock(host);

	cmdat = host->cmdat;
	host->cmdat &= ~CMDAT_INIT;

	if (mrq->data) {
		pxamci_setup_data(host, mrq->data);

		cmdat &= ~CMDAT_BUSY;
		cmdat |= CMDAT_DATAEN | CMDAT_DMAEN;
		if (mrq->data->flags & MMC_DATA_WRITE)
			cmdat |= CMDAT_WRITE;

		if (mrq->data->flags & MMC_DATA_STREAM)
			cmdat |= CMDAT_STREAM;
	}

	pxamci_start_cmd(host, mrq->cmd, cmdat);
}

/* micco_power_module[] should be consistent with enum
 * in include/asm-arm/arch-pxa/pxa3xx_pmic.h */
static struct power_supply_module miccoB0_power_modules[] = {
	/* {command,		power_module}, */
	{VCC_CORE,		BUCK1},
	{VCC_SRAM,		LDO2},
	{VCC_MVT,		LDO1},
	{VCC_3V_APPS,		LDO3},
	{VCC_SDIO,		LDO13},
	{VCC_CAMERA_ANA,	LDO9},
	{VCC_USB,		LDO5},
	{VCC_LCD,		LDO3},
	{VCC_TSI,		0},
	{VCC_CAMERA_IO,		0},
	{VCC_1P8V,		LDO4},
	{VCC_MEM,		BUCK3},
	{HDMI_TX,		0},
	{TECH_3V,		0},
	{TECH_1P8V,		0},
	{VCC_BT,		LDO10},
};

/* micco_power_module[] should be consistent with enum
 *  * in include/asm-arm/arch-pxa/pxa3xx_pmic.h */
static struct power_supply_module miccoEA_power_modules[] = {
	/* {command,		power_module}, */
	{VCC_CORE,		BUCK1},
	{VCC_SRAM,		BUCK2},
	{VCC_MVT,		LDO1},
	{VCC_3V_APPS,		LDO3},
	{VCC_SDIO,		LDO13},
	{VCC_CAMERA_ANA,	LDO9},
	{VCC_USB,		LDO5},
	{VCC_LCD,		LDO3},
	{VCC_TSI,		0},
	{VCC_CAMERA_IO,		0},
	{VCC_1P8V,		LDO4},
	{VCC_MEM,		LDO2},
	{HDMI_TX,		0},
	{TECH_3V,		0},
	{TECH_1P8V,		0},
	{VCC_BT,		LDO10},
};

static struct power_chip micco_chips[] = {
	{MICCO_B0_ID,	"miccoB0",	miccoB0_power_modules},
	{MICCO_EA_ID,	"miccoEA",	miccoEA_power_modules},
	{MICCO_EB_ID,	"miccoEB",	miccoEA_power_modules},
	{0,	NULL,		NULL},
};
static struct power_supply_module *micco_power_module;

static int get_power_supply_module(int cmd)
{
	int command, power_module;

	if (cmd < VCC_CORE || cmd >= MAX_CMD) {
		printk(KERN_WARNING "input wrong command: %d\n", cmd);
		return -EINVAL;
	}
	command = micco_power_module[cmd - VCC_CORE].command;
	if (command != cmd) {
		printk(KERN_WARNING "micco_power_module[] is error: %d\n", cmd);
		return -EINVAL;
	}
	power_module = micco_power_module[cmd - VCC_CORE].power_module;
	if (power_module == 0) {
		printk(KERN_WARNING "the command not supported: %d\n", cmd);
		return -EINVAL;
	}

	return power_module;
}
static int set_micco_voltage(int cmd, int mv)
{
	int power_module, status = 0;
	u8 val;

	power_module = get_power_supply_module(cmd);

	//start_calc_time();
	switch (power_module) {
	case BUCK1:
		if ((mv < MICCO_VBUCK1_BASE) || (mv > MICCO_VBUCK1_MAX))
			return -EINVAL;
		status = micco_read(MICCO_BUCK1_DVC2, &val);
		if (status)
			return status;

		val &= 0xe0;
		val = val | ((mv - MICCO_VBUCK1_BASE) / MICCO_VBUCK1_STEP);
		status = micco_write(MICCO_BUCK1_DVC2, val);
		/* Update the voltage output. Otherwise, voltage won't change */
		status = micco_write(MICCO_VCC1, 0x33);
		break;
	case BUCK2: /* TODO */
		return 0;
	case BUCK3: /* TODO */
		return 0;
	case LDO1:
		if ((mv < MICCO_VLDO1_BASE) || (mv > MICCO_VLDO1_MAX))
			return -EINVAL;
		status = micco_read(MICCO_LDO1_MDTV1, &val);
		if (status)
			return status;

		val &= 0xf0;
		val = val | ((mv - MICCO_VLDO1_BASE) / MICCO_VLDO1_STEP);
		status = micco_write(MICCO_LDO1_MDTV1, val);
		break;
	case LDO2:
		if ((mv < MICCO_VSRAM_BASE) || mv > (MICCO_VSRAM_MAX))
			return -EINVAL;
		status = micco_read(MICCO_SRAM_DVC2, &val);
		if (status)
			return status;

		val &= 0xe0;
		val = val | ((mv - MICCO_VSRAM_BASE) / MICCO_VSRAM_STEP);
		status = micco_write(MICCO_SRAM_DVC2, val);
		/* Update the voltage output. Otherwise, voltage won't change */
		status = micco_write(MICCO_VCC1, 0x33);
		break;
	case LDO3:
		if ((mv < MICCO_VLDO3_BASE) || (mv > MICCO_VLDO3_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO643, &val);
		if (status)
			return status;

		val &= 0xf0;
		val = val | ((mv - MICCO_VLDO3_BASE) / MICCO_VLDO3_STEP);
		status = micco_write(MICCO_LDO643, val);
		break;
	case LDO4:
		if ((mv != 2900) && (mv != 1800))
			return -EINVAL;
		status = micco_read(MICCO_LDO643, &val);
		if (2900 == mv)
			val |= 0x10;
		else
			val &= ~0x10;
		status = micco_write(MICCO_LDO643, val);
		break;
	case LDO5: /* TODO */
		return 0;
	case LDO6:
		if ((mv < MICCO_VLDO6_BASE) || (mv > MICCO_VLDO6_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO643, &val);
		if (status)
			return status;

		val &= 0x1f;
		val |= ((mv - MICCO_VLDO6_BASE) / MICCO_VLDO6_STEP) << 5;
		status = micco_write(MICCO_LDO643, val);
		break;
	case LDO7:
		printk(KERN_WARNING "power supply module TODO: %d\n", cmd);
		return -EINVAL;
	case LDO8:
		printk(KERN_WARNING "power supply module TODO: %d\n", cmd);
		return -EINVAL;
	case LDO9:
		if ( mv == 0 )
		{
			/* disable LDO9 */
			status = micco_read(MICCO_APP_OVER2, &val);
			if (status)
				return status;
			val &= ~MICCO_LDO9_ENABLE;
			micco_write(MICCO_APP_OVER2, val);
			break;
		}

		if ((mv < MICCO_VLDO9_BASE) || (mv > MICCO_VLDO9_MAX))
			return -EINVAL;
		status = micco_read(MICCO_LDO987, &val);
		if (status)
			return status;

		val &= 0x1f;
		val |= ((mv - MICCO_VLDO9_BASE) / MICCO_VLDO9_STEP) << 5;
		status = micco_write(MICCO_LDO987, val);

		/* enable LDO9 */
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		val |= MICCO_LDO9_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO10:
		if ((mv < MICCO_VLDO10_BASE) || (mv > MICCO_VLDO10_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO1110, &val);
		if (status)
			return status;

		val &= 0xf8;
		val = val | ((mv - MICCO_VLDO10_BASE) / MICCO_VLDO10_STEP);
		status = micco_write(MICCO_LDO1110, val);
		break;
	case LDO11:
		if ((mv < MICCO_VLDO11_BASE) || (mv > MICCO_VLDO11_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO1110, &val);
		if (status)
			return status;

		val &= 0x0f;
		val |= ((mv - MICCO_VLDO11_BASE) / MICCO_VLDO11_STEP) << 4;
		status = micco_write(MICCO_LDO1110, val);
		break;
	case LDO12:
		if ((mv < MICCO_VLDO12_BASE) || (mv > MICCO_VLDO12_MAX))
			return -EINVAL;


		status = micco_read(MICCO_LDO1312, &val);
		if (status)
			return status;

		val &= 0xf8;
		if (mv > MICCO_VLDO12_BASE1) {
			val = val | ((mv - MICCO_VLDO12_BASE1) /	\
				MICCO_VLDO12_STEP);
			val |= 0x08;
		} else {
			val = val | ((mv - MICCO_VLDO12_BASE) /		\
				MICCO_VLDO12_STEP);
			val &= ~0x08;
		}
		status = micco_write(MICCO_LDO1312, val);
		break;
	case LDO13:
		if ((mv < MICCO_VLDO13_BASE) || (mv > MICCO_VLDO13_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO1312, &val);
		if (status)
			return status;

		val &= 0x0f;
		val |= ((mv - MICCO_VLDO13_BASE) / MICCO_VLDO13_STEP) << 4;
		status = micco_write(MICCO_LDO1312, val);
		break;
	case LDO14:
		if ((mv < MICCO_VLDO14_BASE) || (mv > MICCO_VLDO14_MAX))
			return -EINVAL;
		status = micco_read(MICCO_LDO1514, &val);
		if (status)
			return status;

		val &= 0xf0;
		val = val | ((mv - MICCO_VLDO14_BASE) / MICCO_VLDO14_STEP);
		status = micco_write(MICCO_LDO1514, val);
		break;
	case LDO15:
		if ((mv < MICCO_VLDO15_BASE) || (mv > MICCO_VLDO15_MAX))
			return -EINVAL;

		status = micco_read(MICCO_LDO1514, &val);
		if (status)
			return status;

		val &= 0x0f;
		val |= ((mv - MICCO_VLDO15_BASE) / MICCO_VLDO15_STEP) << 4;
		status = micco_write(MICCO_LDO1514, val);
		break;
	case USB_OTG: /* TODO */
	case LDO_GPADC: /* TODO */
	case LDO_AUDIO: /* TODO */
	case LDO_PMCORE: /* TODO */
	case LDO_BBAT: /* TODO */
		printk(KERN_WARNING "power supply module TODO: %d\n", cmd);
		return -EINVAL;
	default:
		printk(KERN_INFO "error command\n");
		return -EINVAL;
	}

	if (status != 0)
		return status;

	//end_calc_time();
	return status;
}

static int enable_micco_voltage(int cmd, int enable)
{
	int power_module, status = 0;
	u8 val;
	power_module = get_power_supply_module(cmd);
	switch (power_module) {
	case BUCK1:
		status = micco_read(MICCO_OVER1, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_BUCK1_ENABLE;
		else
			val &= ~MICCO_BUCK1_ENABLE;
		micco_write(MICCO_OVER1, val);
		break;
	case BUCK2:
		status = micco_read(MICCO_OVER1, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_BUCK2_ENABLE;
		else
			val &= ~MICCO_BUCK2_ENABLE;
		micco_write(MICCO_OVER1, val);
		break;
	case BUCK3:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_BUCK3_ENABLE;
		else
			val &= ~MICCO_BUCK3_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO1:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO1_ENABLE;
		else
			val &= ~MICCO_LDO1_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO2:
		status = micco_read(MICCO_OVER1, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO2_ENABLE;
		else
			val &= ~MICCO_LDO2_ENABLE;
		micco_write(MICCO_OVER1, val);
		break;
	case LDO3:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO3_ENABLE;
		else
			val &= ~MICCO_LDO3_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO4:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO4_ENABLE;
		else
			val &= ~MICCO_LDO4_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO5:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO5_ENABLE;
		else
			val &= ~MICCO_LDO5_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO6:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO6_ENABLE;
		else
			val &= ~MICCO_LDO6_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO7:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO7_ENABLE;
		else
			val &= ~MICCO_LDO7_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO8:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO8_ENABLE;
		else
			val &= ~MICCO_LDO8_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO9:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO9_ENABLE;
		else
			val &= ~MICCO_LDO9_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO10:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO10_ENABLE;
		else
			val &= ~MICCO_LDO10_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO11:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO11_ENABLE;
		else
			val &= ~MICCO_LDO11_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO12:
		status = micco_read(MICCO_APP_OVER2, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO12_ENABLE;
		else
			val &= ~MICCO_LDO12_ENABLE;
		micco_write(MICCO_APP_OVER2, val);
		break;
	case LDO13:
		status = micco_read(MICCO_APP_OVER3, &val);
		if (status)
			return status;
		if (enable)
			val |= MICCO_LDO13_ENABLE;
		else
			val &= ~MICCO_LDO13_ENABLE;
		micco_write(MICCO_APP_OVER3, val);
		break;
	case LDO14:
	default:
		printk(KERN_INFO "error command\n");
		return -EINVAL;
	}

	return status;
}

int pxa3xx_pmic_enable_voltage(int cmd, int enable)
{
    //Leith
    //pick up from drivers/i2c/chips/micco.c->micco_pmic_ops
    return enable_micco_voltage(cmd, enable);
}

static void jades_mci_setpower(unsigned int vdd)
{
    unsigned int ocr_mask = MMC_VDD_32_33|MMC_VDD_33_34;
    if (( 1 << vdd) & ocr_mask) {
		pxa3xx_pmic_enable_voltage(VCC_SDIO, 1);
	} else {
		pxa3xx_pmic_enable_voltage(VCC_SDIO, 0);
	}
}

int micco_probe(void)
{
    int ret;
	int chip_id;
	int i;

	micco_read(MICCO_CHIP_ID, &chip_id);
    for (i = 0; micco_chips[i].chip_id != 0; i++) {
		if (micco_chips[i].chip_id == chip_id) {
			micco_power_module = \
				micco_chips[i].power_supply_modules;
			break;
		}
	}
	if (micco_chips[i].chip_id == 0)
	{/* if not found a match version, then use default */
		micco_power_module = micco_chips[0].power_supply_modules;
		//panic("This Micco chip is not supported");
	}
}



static void mmc_power_up(struct mmc_host *host)
{
	int bit = fls(host->ocr_avail) - 1;

	host->ios.vdd = bit;
	if (mmc_host_is_spi(host)) {
		host->ios.chip_select = MMC_CS_HIGH;
		host->ios.bus_mode = MMC_BUSMODE_PUSHPULL;
	} else {
		host->ios.chip_select = MMC_CS_DONTCARE;
		host->ios.bus_mode = MMC_BUSMODE_OPENDRAIN;
	}
	host->ios.power_mode = MMC_POWER_UP;
	host->ios.bus_width = MMC_BUS_WIDTH_1;
	host->ios.timing = MMC_TIMING_LEGACY;
	mmc_set_ios(host);

	/*
	 * This delay should be sufficient to allow the power supply
	 * to reach the minimum voltage.
	 */
	mmc_delay(2);

	host->ios.clock = host->f_min;
	host->ios.power_mode = MMC_POWER_ON;
	mmc_set_ios(host);

	/*
	 * This delay must be at least 74 clock sizes, or 1 ms, or the
	 * time required to reach a stable voltage.
	 */
	mmc_delay(2);
}

static void pxamci_set_ios(struct mmc_host *mmc, struct mmc_ios *ios)
{
	struct pxamci_host *host = mmc_priv(mmc);
    
	//set_dvfm_constraint();
#ifdef CONFIG_ANDROID_POWER
	//android_lock_suspend(&pxamci_suspend_lock);
#endif
	if (ios->clock) {
		unsigned long rate = host->clkrate;
		unsigned int clk = rate / ios->clock;

		if (host->clkrt == CLKRT_OFF)
			clk_enable(host->clk);

		if (ios->clock == 26000000) {
			/* to support 26MHz on pxa300/pxa310 */
			host->clkrt = 7;
		} else {
			/* to handle (19.5MHz, 26MHz) */
			if (!clk)
				clk = 1;

			/*
			 * clk might result in a lower divisor than we
			 * desire.  check for that condition and adjust
			 * as appropriate.
			 */
			if (rate / clk > ios->clock)
				clk <<= 1;
			host->clkrt = fls(clk) - 1;
		}

		/*
		 * we write clkrt on the next command
		 */
	} else {
		pxamci_stop_clock(host);
		if (host->clkrt != CLKRT_OFF) {
			host->clkrt = CLKRT_OFF;
			clk_disable(host->clk);
		}
	}

	if (host->power_mode != ios->power_mode) {
		host->power_mode = ios->power_mode;

		//if (host->pdata && host->pdata->setpower)
			//host->pdata->setpower(mmc_dev(mmc), ios->vdd);
        //Leith 
        //power up
        jades_mci_setpower(ios->vdd);
		if (ios->power_mode == MMC_POWER_ON)
			host->cmdat |= CMDAT_INIT;
	}

	if (ios->bus_width == MMC_BUS_WIDTH_4)
		host->cmdat |= CMDAT_SD_4DAT;
	else
		host->cmdat &= ~CMDAT_SD_4DAT;

	//unset_dvfm_constraint();
#ifdef CONFIG_ANDROID_POWER
	//android_unlock_suspend(&pxamci_suspend_lock);
#endif
	pr_debug("PXAMCI: clkrt = %x cmdat = %x\n",
		 host->clkrt, host->cmdat);
}

static void pxamci_enable_sdio_irq(struct mmc_host *host, int enable)
{
	struct pxamci_host *pxa_host = mmc_priv(host);

	if (enable)
		pxamci_enable_irq(pxa_host, SDIO_INT);
	else
		pxamci_disable_irq(pxa_host, SDIO_INT);
}

static const struct mmc_host_ops pxamci_ops = {
	.request		= pxamci_request,
	//.get_ro			= pxamci_get_ro,
	.set_ios		= pxamci_set_ios,
	.enable_sdio_irq	= pxamci_enable_sdio_irq,
};

unsigned long clk_get_rate(struct clk *clk);

static struct mmc_host borqs_mmc_host;
int sd_init(void)
{
    struct mmc_host *mmc;
    struct pxamci_host *host = NULL;
	struct resource *r, *dmarx, *dmatx;
	int ret, irq;
    
    r = &pxamci_resources[0];
    
    mmc = &borqs_mmc_host;
    mmc->max_hw_segs = 1;
	mmc->max_phys_segs = 1;
    mmc->max_seg_size = PAGE_CACHE_SIZE;
    
    mmc->max_req_size = PAGE_CACHE_SIZE;
    mmc->max_blk_size = 512;
	mmc->max_blk_count = PAGE_CACHE_SIZE / 512;
    
    mmc->ops = &pxamci_ops;
	/*
	 * We can do SG-DMA, but we don't because we never know how much
	 * data we successfully wrote to the card.
	 */
	mmc->max_phys_segs = NR_SG;
    
	/*
	 * Our hardware DMA can handle a maximum of one page per SG entry.
	 */
	mmc->max_seg_size = PAGE_SIZE;
    
	/*
	 * Block length register is only 10 bits before PXA27x.
	 */
	mmc->max_blk_size = 2048;//(cpu_is_pxa21x() || cpu_is_pxa25x()) ? 1023 : 2048;
    
	/*
	 * Block count register is 16 bits.
	 */
	mmc->max_blk_count = 65535;
    
    host = mmc_priv(mmc);
	host->mmc = mmc;
	host->dma = -1;
	//host->pdata = pdev->dev.platform_data;
	host->clkrt = CLKRT_OFF;
    
    host->clk = &pxa3xx_clks[0];
    host->clkrate = clk_get_rate(host->clk);//host->clk
    
    /*
	 * Calculate minimum clock rate, rounding up.
	 */
	mmc->f_min = (host->clkrate + 63) / 64;
	//if (cpu_is_pxa300() || cpu_is_pxa310())
		mmc->f_max = 26000000;
	//else
	//	mmc->f_max = host->clkrate;
    
	mmc->ocr_avail = //host->pdata ?
			 MMC_VDD_32_33|MMC_VDD_33_34;//host->pdata->ocr_mask :
			 //MMC_VDD_32_33|MMC_VDD_33_34;
	mmc->caps = 0;
	host->cmdat = 0;
	//if (!cpu_is_pxa21x() && !cpu_is_pxa25x()) 
    {
		mmc->caps |= MMC_CAP_4_BIT_DATA | MMC_CAP_SDIO_IRQ;
		host->cmdat |= CMDAT_SDIO_INT_EN;
		//if (cpu_is_pxa300() || cpu_is_pxa310())
			mmc->caps |= MMC_CAP_MMC_HIGHSPEED |
				     MMC_CAP_SD_HIGHSPEED;
	}
    
    //host->sg_cpu = dma_alloc_coherent(&pdev->dev, PAGE_SIZE, &host->sg_dma, GFP_KERNEL);
    host->sg_dma = (dma_addr_t)host->sg_cpu;

    host->res = r;
	//host->irq = irq;
	host->imask = MMC_I_MASK_ALL;
    host->base = (void __iomem*)r->start;//ioremap(r->start, SZ_4K);

    pxamci_stop_clock(host);
	writel(0, host->base + MMC_SPI);
	writel(64, host->base + MMC_RESTO);
	writel(host->imask, host->base + MMC_I_MASK);
    
    //usb 1.1 usd static XLLP_DMAC_CHANNEL_T UdcDmaRxChannel   = USB_RX_CHANNEL;
    //4,5 dma chanel
    //pxa_init_dma(32)) pxa310 total 32 dma chanel

    //dma_irq_handler
    //channel->irq_handler(i, channel->data);
    //channel->irq_handler = pxamci_dma_irq;
    host->dma = 22;//XLLP_DMAC_CHANNEL_22;
                //pxa_request_dma(DRIVER_NAME, DMA_PRIO_LOW,
				//    pxamci_dma_irq, host);

    dmarx = &pxamci_resources[2];
    host->dma_drcmrrx = dmarx->start;
    dmatx = &pxamci_resources[3];
    host->dma_drcmrtx = dmatx->start;

    //Leith
    //pick from mmc_rescan to start mmc controller
#ifdef DDR_WORKAROUND
    micco_probe();
#endif
    mmc_power_up(mmc);
    return 0;
}

