/******************************************************************************

**	(C) Copyright 2007 Marvell International Ltd.  

**	All Rights Reserved
******************************************************************************/

#include "protocolmanager.h"
#include "flash.h"
#include "obm_pxa_lcd.h"
#include "tim.h"
#include "misc.h"

#if TRUSTED
#include "obm_security.h"
#endif

#if MDOC_CODE
#include "msys.h"
#endif

#if ONENAND_CODE
#include "FlexOBMInterface.h"
#endif

#if TAVOR_BOERNE
#include "PinMux_defs.h"
#include "i2c.h"
#include "product_id.h"

#if (!YARDEN)
#define RUN_SW_UPGRADE			1
#endif
#define RELEASE_COMM_RESET		1


#endif

#include "powerreason.h"
#include "obm_micco.h"
#include "obm_pxa930.h"
#include "obm_pxa930_gpio.h"
extern void fota_handle(P_FlashProperties_T pFlashProp);

#if SAAR_PV
void set_power(void)
{
    gpio_direction_output(MFP_PIN_GPIO64, 0); //set gpio64(charge enable) to enable charge.
    gpio_direction_output(MFP_PIN_GPIO14, 1); //change electric current to 500mA via set gpio14&15 10
    gpio_direction_output(MFP_PIN_GPIO15, 0); 
}
#endif


void power2saving(void)
{
 /*
     micco_write(MICCO_OVER1, 0x07);
     micco_write(MICCO_APP_OVER2, 0x66);
     micco_write(MICCO_APP_OVER3, 0xfd);
*/
    gpio_direction_output(MFP_PIN_GPIO84, 0); //set gpio 84 as output and low
    gpio_direction_output(MFP_PIN_GPIO97, 0); //set gpio 97 as output and low
    gpio_direction_output(MFP_PIN_GPIO98, 0); //set gpio 98 as output and low
}

void power2saving_micco(void)
{
     micco_write(MICCO_OVER1, 0x07);
     micco_write(MICCO_APP_OVER2, 0x66);
     micco_write(MICCO_APP_OVER3, 0xfd);
}

UINT8_T check_usb_id_pin(void);

void SavePowerUpReason( powerup_reason_t reason )
{
    PSPR = reason;
}

void bytes2hex(int value, char *out_buf, char bytes)
{
	char *p;

	inline char bcd2hex(char BCD)
	{
		return (BCD > 9) ? (BCD-10+'A'):(BCD+'0');
	}
	inline void char2hex(char value,char **out)
	{
	*(*out)++ = bcd2hex(value >> 4);
	*(*out)++ = bcd2hex(value & 0x0f);
	}

	p = out_buf;
	*p++ = '0';
	*p++ = 'x';
	if(bytes > 3)char2hex(value >> 24,&p);
	if(bytes > 2)char2hex(value >> 16,&p);
	if(bytes > 1)char2hex(value >>  8,&p);
	char2hex(value &0xff,&p);
	*p = 0;
}

extern INT_T strlen (const char * str);
void poweron_reason_print( char* str, int data, int bytes)
{
	char buf[256];

	memcpy( buf, str, strlen(str) );
	bytes2hex( data, buf+strlen(str), bytes );
	AddMessage(buf);
}

unsigned int reboot_reason = REBOOT_MAX;

powerup_reason_t get_reboot_reason( void )
{
	powerup_reason_t powerup_reason = PU_REASON_MAX;
//	UINT_T *reboot_setting_addr;

//	reboot_setting_addr = (UINT_T *)(REBOOT_SETTING_ADDR);
//	reboot_reason = *reboot_setting_addr;

	reboot_reason = PSPR;
	poweron_reason_print( "reboot_reason = ", reboot_reason, 4 );

	switch( reboot_reason )
	{
        case REBOOT_DKB:
		case REBOOT_NORMAL:
			powerup_reason = PU_REASON_REBOOT_NORMAL;
			break;
		case REBOOT_FACTORY:
			powerup_reason = PU_REASON_REBOOT_FACTORY;
			break;
		case REBOOT_RECOVERY:
			powerup_reason = PU_REASON_REBOOT_RECOVERY;
			break;
		case REBOOT_ASSERT:
			powerup_reason = PU_REASON_REBOOT_ASSERT;
			break;
        case REBOOT_FOTA:
			powerup_reason = PU_REASON_REBOOT_FOTA;
		case REBOOT_FLASH:
			break;

		default:
			break;
	}

	return powerup_reason;
}

powerup_reason_t parse_poweron_reason( void )
{       
	unsigned char micco_status_a, micco_status_b, micco_fault_log;
	unsigned int AD3SR_val, PWSR_val, ARSR_val;

	micco_read( 0x05, &micco_status_a );
	micco_read( 0x06, &micco_status_b );
	micco_read( 0x0d, &micco_fault_log );
	AD3SR_val = *AD3SR_R;
	PWSR_val = *PWSR_R;
	ARSR_val = *ARSR_R;

	poweron_reason_print( "micco_status_a = ", micco_status_a, sizeof(micco_status_a) );
	poweron_reason_print( "micco_status_b = ", micco_status_b, sizeof(micco_status_b) );
	poweron_reason_print( "micco_fault_log = ", micco_fault_log, sizeof(micco_fault_log) );
	poweron_reason_print( "PWSR_val = ", PWSR_val, sizeof(PWSR_val) );
	poweron_reason_print( "ARSR_val = ", ARSR_val, sizeof(ARSR_val) );
	poweron_reason_print( "AD3SR_val = ", AD3SR_val, sizeof(AD3SR_val) );

#if 1
	if( check_usb_id_pin() )	//Factory Cable
		return PU_REASON_USB_FACTORY;
#endif

	if( !( micco_status_a & 0x1 ) )	//nONKEY
		return PU_REASON_PWR_KEY_PRESS;
	//if( !( micco_status_a & 0x8 ) )	//, CHDET
	//	return PU_REASON_USB_CHARGER;

	if( ( PWSR_val & 0x80000000 ) ) //RTC
		return PU_REASON_RTC_ALARM;

	if( ( ARSR_val & 0x00000002 ) )//Watchdog
		return PU_REASON_WATCHDOG;

	if( ( ARSR_val & 0x00000008 ) )//GPIO Reset
		return get_reboot_reason();

	if( micco_status_b & 0x1 )      //USB_DEV
		return PU_REASON_USB_CABLE;

	if( AD3SR_val & 0x80000000 )	//RTC
		return PU_REASON_RTC_ALARM;

	if( AD3SR_val & 0x40000000 )	//OST
		return PU_REASON_RTC_ALARM;

	return PU_REASON_MAX;
}

void pm_select_wakeup_src( void )
{
	*PWSR_R = 0xFFFFFFFF;
	*PWER_R = 0x80000005;
}

void mhn_pm_set_cken( void )
{
	*CKENB_R |= 0x00000200; //CKENB_HSIO2 /* This bit must be enabled before entering low power mode. */
}

void mhn_clear_pm_status( void )
{
	unsigned int tmp;

	/* clear power manager status */
	tmp = *PSR_R;
	tmp &= 0x07;
	*PSR_R = tmp;

	/* clear application system status */
	tmp = *ASCR_R;
	tmp &= 0x07;
	*ASCR_R = tmp;
	/* clear all application subsystem reset status */
	tmp = *ARSR_R;
	*ARSR_R = tmp;
}

void Micco_reset_workaround( void )
{
	*PMCR_R |= 0x01;
}

void do_powerdown( void )
{
	AddMessage((UINT8_T*)("Deepsleeping ...\n"));

	Micco_reset_workaround();
	pm_select_wakeup_src();
	mhn_pm_set_cken();
	mhn_clear_pm_status();

	asm(" \
	mov     r1, #0x07 \n\
	mov     r11, r1 @save the sleep type \n\
	mcr     p14, 0, r11, c7, c0, 0 \n\
20: \n\
	nop \n\
	b       20b \n\
	"
	);
}
//#define ___output_micco_register_log
#undef ___output_micco_register_log
#ifdef ___output_micco_register_log
void print2uart(UINT8_T* message)
{
    UINT_T Retval = 0;
	UINT_T size;
	UINT_T i;
    if(GetPortState(FFUART_D))
	{
		size = strlen((char *)message);
		for(i = 0; i < size ; i++) {
			Retval = WriteCharToFFUart((char)message[i]);
			if (Retval != NoError) break;
		}
	}
}
void printf_micco_registers(UINT8_T from, UINT8_T to)
{
    int i;
    char l_data;
    char buffer[20];
    print2uart("===============================\r\n");
    for(i = from;i <= to;i++)
    {
        micco_read(i,&l_data);
        bytes2hex(i,buffer,1);
        print2uart(buffer);
        print2uart(" = ");
        bytes2hex(l_data,buffer,1);
        print2uart(buffer);
        print2uart("h\r\n");
        //sprintf(buffer,"0x%2x=%2x\n",i,dataPtr[1]);
    }
}
#endif

UINT8_T check_usb_id_pin(void)
#if 0
{
  	//volatile unsigned int *GPLR1 = (volatile unsigned int *)0x40E00004;
	//volatile unsigned int *GPDR1 = (volatile unsigned int *)0x40E00010;
	//volatile unsigned int *GSDR1 = (volatile unsigned int *)0x40E00404;
	//volatile unsigned int *GCDR1 = (volatile unsigned int *)0x40E00424;
	//volatile unsigned int *GPCR1 = (volatile unsigned int *)0x40E00028;

       volatile unsigned int *MFPR_gpio52 = (volatile unsigned int *) (0x40E102C4) ;
  	unsigned int 		   mfpr_gpio52_val;
	unsigned int 		   GPDR1_val;
	UINT8_T 		 	   Factory_mod = FALSE ;
    #define __GLIETHTTP_PIN_52  ( 0x01 << (52-32) )

    	// Save the value of GPDR0 and MFPR-GPIO0
	GPDR1_val = *GPDR1;
	mfpr_gpio52_val = *MFPR_gpio52;

	AddMessage((UINT8_T*) ("\0"));
	AddMessage((UINT8_T*) ("*** Factory Checking .... ***\0"));
	

	// Set GPIO 52 direction to be -- input
	*GCDR1 = __GLIETHTTP_PIN_52;

	// GPIO 52 - set to disable pull
	*MFPR_gpio52 &= ~(0xe000);
	// GPIO52 - enable pull & set pull down
	*MFPR_gpio52  |= (0xa000);

       Delay_ms(2) ; // delay 2ms wait for pull down stable .

	// check if value of GPIO0 is HIGH , if HIGH - "*" key is pressed... go to update SW mode
	if ( (*GPLR1 & __GLIETHTTP_PIN_52) == __GLIETHTTP_PIN_52 )
	{
		Factory_mod = TRUE;
		AddMessage((UINT8_T*) ("*** Factory ...\0"));
	}
	else
	{  
	       AddMessage((UINT8_T*) ("*** No Factory ...\0"));
	}
	AddMessage((UINT8_T*) ("\0"));	
	 //restore reset values of registers
	*MFPR_gpio52 = mfpr_gpio52_val;
	*GPDR1 = GPDR1_val;

	return Factory_mod;
}
#else
{
//    unsigned int 		   mfpr_val;
//    unsigned int 		   pre_GPDR;
//    unsigned int 		   tmp;
    UINT8_T 		 	   Factory_mod = FALSE ;
#define GPIO52_FACTORY_IN  MFP_CFG_X(GPIO52, AF0, DS02X, PULL_HIGH)
    static mfp_cfg_t jades_mfp_cfg[] = {
            GPIO52_FACTORY_IN,
    };
    pxa3xx_mfp_config(ARRAY_AND_SIZE(jades_mfp_cfg));
    /*
    pre_GPDR = gpio_direction_reg_get(MFP_PIN_GPIO52);
    mfpr_val = pxa3xx_mfp_read(MFP_PIN_GPIO52);   

    AddMessage((UINT8_T*) ("\0"));
	AddMessage((UINT8_T*) ("*** Factory Checking .... ***\0"));

    gpio_direction_input(MFP_PIN_GPIO52); 
    //pxa3xx_mfp_config( (MFP_PIN_GPIO52 << 16) | | MFP_AF0 , 1);
    tmp  = mfpr_val & ~0xe000;
    tmp |= 0xa000;
    pxa3xx_mfp_write(MFP_PIN_GPIO52,tmp);
    Delay_ms(2) ; // delay 2ms wait for pull down stable .
    */
    if(!gpio_get(MFP_PIN_GPIO52))
    {
        Factory_mod = TRUE;
		AddMessage((UINT8_T*) ("*** Factory ...\0"));
    }else
        {
            AddMessage((UINT8_T*) ("*** No Factory ...\0"));
        }
    //restore reset values of registers
//    pxa3xx_mfp_write(MFP_PIN_GPIO52, mfpr_val);
//    gpio_direction_reg_set(MFP_PIN_GPIO52, pre_GPDR);
    return Factory_mod;
}
#endif

powerup_reason_t powerup_reason = PU_REASON_MAX;

int check_flashmode_status( void )
{
	if( REBOOT_FLASH == reboot_reason )
		return TRUE;

	return FALSE;
}

void check_redkey_status( int delay )
{
	if( PU_REASON_PWR_KEY_PRESS == powerup_reason )
	{
		unsigned char micco_status_a;

		AddMessage((UINT8_T*)("Check redkey status ...\0"));

		if( delay > 0 )
			Delay_seconds( delay );

		micco_read( 0x05, &micco_status_a );
		if( micco_status_a & 0x1 )	//nONKEY
			do_powerdown();
	}
}

void determine_power_supply(P_FlashProperties_T pFlashProp)
{
#if 1
	if( FlashKeyChk() )
	{
		reboot_reason = REBOOT_FLASH;
		return;
	}
#endif

	powerup_reason = parse_poweron_reason();

	poweron_reason_print( "powerup_reason = ", powerup_reason, 4 );

	switch( powerup_reason )
	{
		case PU_REASON_HARDWARE_RESET:
		case PU_REASON_WATCHDOG:
		case PU_REASON_RTC_ALARM:
		case PU_REASON_PWR_KEY_PRESS:
		case PU_REASON_USB_CABLE:
		case PU_REASON_USB_FACTORY:
		case PU_REASON_USB_CHARGER:
		case PU_REASON_REBOOT_NORMAL:
		case PU_REASON_REBOOT_FACTORY:
		case PU_REASON_REBOOT_RECOVERY:
		case PU_REASON_REBOOT_ASSERT:
		case PU_REASON_LOW_PWR_RESET:
			SavePowerUpReason( powerup_reason );
			break;
		case PU_REASON_REBOOT_FOTA:
            fota_handle(pFlashProp);
			break;
		case PU_REASON_MAX:
			if( check_flashmode_status() )
				break;
		default:
			do_powerdown();
			break;
	}
}

/* if poweron reason is assert reboot, we display a message on lcd.*/
void check_assert_poweron_reason(void)
{
    if (powerup_reason == PU_REASON_REBOOT_ASSERT)
    {
    #if SAAR_PV
        lcd_printf("security reboot",185,20,2,FONT_COLOR_RED,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
    #elif SAAR
        lcd_printf("security reboot",90,20,1,FONT_COLOR_RED,FONT_COLOR_BLACK,LCD_DRAWMODE_NORMAL);
    #endif
    }
}


/* read the micco VBAT channel(0), the value from here is the voltage of battery */
int micco_adc_read(UINT8_T mux)
{
	UINT8_T i, val, adc1, adc2;
	unsigned short data;

	micco_write(MICCO_ADC_MAN_CONTROL, 
		MICCO_ADC_MAN_CONT_CONV | MICCO_ADC_MAN_LDO_ADC_ENABLE | mux);

	for (i = 0; i < 10; i++) {
		Delay(80);
		micco_read(MICCO_ADC_MAN_CONTROL, &val);
		if (val & MICCO_ADC_MAN_CONT_CONV) {
	        AddMessage((UINT8_T *)"read adc timeout\n");
			continue;
		}
		micco_read(MICCO_MAN_RES_LSB, &adc1);
		micco_read(MICCO_MAN_RES_MSB, &adc2);
		data = (adc2 << 2) | (adc1 & 0x03);
		return data;
	}
	return 0;
}

fota_indentifier check_fota_mode(P_FlashProperties_T pFlashProp)
{
    char buffer[8];
    char normal_fota_identifier[8] = "5A5A";
    char recovery_fota_identifier[8] = "5B5B";
    int i;
    AddMessage("check fota indentifier");

#if SAAR_PV
    Flex_ReadFlash(0x0F400000, 8, (P_DFC_BUFFER)buffer, pFlashProp);
#elif SAAR
    ReadNANDWithBBM(0x0F400000, 8, (P_DFC_BUFFER)buffer, pFlashProp);
#endif
    
    AddMessage(buffer);

    for(i = 0; i < 4; i++)
    {
        if (buffer[i] != normal_fota_identifier[i])    
            break;
    }
    if (i == 4)
        return NORMAL_FOTA_MODE;

    for(i = 4; i < 8; i++)
    {
        if (buffer[i] != recovery_fota_identifier[i - 4])    
            break;
    }
    if (i == 8)
        return RECOVERY_FOTA_MODE;

    return NON_FOTA;
}

