#!/bin/bash



ParamErrorPrint()
{
echo "                                                                                             "
echo "                                                                                             "
echo "                                                                                             "
echo "  USAGE:: lin_make_release.bat  TrustedMode  ALL/BoardType  FlashType  ObmMode  Special      "
echo "                                                                                             "
echo "         TrustedMode == NT or T                                                              "
echo "         ALL         == ALL or next parameter                                                "
echo "         BoardType   == EVB_PV or P65 or YARDEN or SAAR or SAAR_PV or EVB                    "
echo "         FlashType   == NAND8 or ONENAND or MDOC or NOR or NOFLASH or NAND16                 "
echo "         ObmMode     == MODE0 or MODE1 or MODE2 or MODE3 or MODE4                            "
echo "         Special     == DDR or BLOCK1 or SRAM or space                                       "
echo "                                                                                             "
echo "                   DDR    = means the OBM will be copy directly to DDR instead of SRAM.      "
echo "                   Block0 = means the OBM will reside in flash in block 0. The default is 1. "
echo "                   SRAM   = means the OBM will run from SRAM and will not be copy to the DDR."
echo "                                                                                             "
echo "                                                                                             "
echo "         PAY ATTENTION -- Befor running the compilation command, make sure to                "
echo "                          update the pin-mux table by the Excel file, than press             "
echo "                          the build button to create the relevant H file.                    "
echo "                          In addition, make sure all the path in the NTIM/TIM                "
echo "                          text file are correct.                                             "
echo "                                                                                             "
echo "                                                                                             "
echo "         Some commands for example:                                                          "
echo "                                                                                             "
echo "                   win_make_release.bat  NT  ALL                                             "
echo "                   win_make_release.bat  NT  ALL  EVB2                                       "
echo "                   win_make_release.bat   T  EVB  NAND8  MODE1                               "
echo "                   win_make_release.bat  NT  EVB  NAND8  MODE1                               "
echo "                   win_make_release.bat  NT  SAAR  ONENAND  MODE0                            "
echo "                   win_make_release.bat  NT  EVB  NAND8  MODE0  DDR                          "
echo "                   win_make_release.bat  NT  EVB  NAND8  MODE0  DDR  BLOCK0                  "
echo "                   win_make_release.bat  NT  YARDEN  NAND8  MODE1                            "
echo "                   win_make_release.bat  NT  EVB_PV  NOFLASH  MODE1                          "
echo "                   win_make_release.bat  NT  EVB_PV  NOR  MODE1                              "
echo "                   win_make_release.bat  NT  ALL BLOCK0                                      "
echo "                                                                                             "
echo "                                                                                             "
exit 1
}



CheckObmEnvironment()
{

if ! [ -e ../Binaries/ntbb_linux.exe -o -e ../Binaries/tbb_linux.exe ]
		then
    echo "   "
    echo "   *********************************************************************************"
		echo "    !!! ERROR - the file tbb/ntbb_linux.exe or ../Binaries directory does not exist"
    echo "   *********************************************************************************"
    echo "   "
	  exit 1
fi

if ! [ -e ../app_bin/Arbel.bin -o -e ../app_bin/boerne.bin -o -e ../app_bin/ReliableData.bin -o -e ../app_bin/Tavor_M05_AI_B0_Flash.bin -o -e ../app_bin/TAVOR_RBOOT.bin ]
		then
    echo "   "
    echo "   **********************************************************************************"
    echo "    !!! ERROR - the APP_BIN directory or one of the application files does not exist"
    echo "   **********************************************************************************"
    echo "   "
	  exit 1
fi

}
########################################################################################################################

if [ $1 == clean ]
then
    make -C Build -f obm_linux.mak clean   
    exit
fi

if [ xx$1 == xx -o xx$2 == xx ] ; then
	ParamErrorPrint
fi

if [ $1 != T -a $1 != NT ]
then
	ParamErrorPrint
fi

if [ $1 == "T" ]
then
	Tmode="tobm"
else
	Tmode="ntobm"
fi

pTmode=$1



if [ $2 == "ALL" ]
then
	Btype="_all"
	if [ xx$3 != xx ]
	then
			if [ $3 == "EVB_PV" -o $3 == "P65" -o $3 == "YARDEN" -o $3 == "SAAR" -o $3 == "SAAR_PV" -o $3 == "EVB" ]
			then
				pBtype=$3
			else
				ParamErrorPrint
			fi
	fi
else
	Btype=""
	if [ $2 == "EVB_PV" -o $2 == "P65" -o $2 == "YARDEN" -o $2 == "SAAR" -o $2 == "SAAR_PV" -o $2 == "EVB" ]
	then
			pBtype=$2
	else
			ParamErrorPrint
	fi
fi



if [ $2 != "ALL" ]
then
		if [ $3 == NAND8 -o $3 == ONENAND -o $3 == MDOC -o $3 == NAND16 -o $3 == NOFLASH -o $3 == NOR ]
		then
			pFlash=$3
		else
			ParamErrorPrint
		fi


		if [ $4 == MODE0 -o $4 == MODE1 -o $4 == MODE2 -o $4 == MODE3 -o $4 == MODE4 -o $4 == MODE6 ]
		then
			pMode=$4
		else
			ParamErrorPrint
		fi


		if [ xx$5 != xx ]
		then
			if [ $5 == DDR -o $5 == SRAM ]
			then
				pOption=$5
            elif [ $5 == SPARK -o $5 == BLAZE -o $5 == BENZINA ]
            then
                pProductType=$5 
            else
				ParamErrorPrint
			fi
		fi
fi


CheckObmEnvironment


USER=`whoami`
if [ "$USER" == "root" ]
then
   SudoCmd=""
else
   SudoCmd="sudo"
fi


$SudoCmd ./BAT/lin_make$Btype\_$Tmode\_release.sh $pTmode $pBtype $pFlash $pMode $pOption $pProductType

echo "   "
echo "   "

