/**************************************************************************
**
**  (C) Copyright December 2006 Marvell International Ltd.
**
**  All Rights Reserved.
**
**  This software file (the "File") is distributed by Marvell International  
**  Ltd. under the terms of the GNU General Public License Version 2, June  
**  1991 (the "License").  You may use, redistribute and/or modify this File   
**  in accordance with the terms and conditions of the License, a copy of  
**  which is available along with the File in the license.txt file or by   
**  writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
**  Boston, MA 02111-1307 or on the worldwide web at 
**  http://www.gnu.org/licenses/gpl.txt.
**
**  THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
**  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
**  PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
**  details about this warranty disclaimer. 
**
**  Filename: TBBTIM.cpp
**
**  This program builds the TIM binary image from a TIM descriptor file and all of
**  the binary images that it lists.
**  This is a console application.
**  
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#if LINUX
  #include <sys/types.h>
  #include <fcntl.h>
  #include <sys/stat.h>
  #include <limits.h>
  #include "typedef.h"
#endif
 
#if WINDOWS
  
#endif

#if TRUSTED
  #include "ippcp.h"
  #include "TBBLIB.h"
#endif

#include "NTBBLIB.h"
#include "tim.h"

Ipp32u DigitalSignature[MAXKEYSIZEWORDS];
UINT_T RsaPrivateKey[MAXKEYSIZEWORDS];
UINT_T RsaPublicKey[MAXKEYSIZEWORDS];

extern bool bIsTimVerify, bConcatenate;
extern char KeyFileName[MAX_PATH], HashFileName[MAX_PATH], DsFileName[MAX_PATH];
extern char TimInFileName[MAX_PATH], TimOutFileName[MAX_PATH];
extern char ReservedFileName[MAX_PATH];
extern int iOption;

extern bool CheckImageOverlap(pIMAGE_INFO pImageInfo, unsigned int num_images, unsigned int BootFlashSign);
extern bool ParseTrustedDescriptorFile (FILE *hKeyFile,CTIM *pTimHeader);
bool ScanKeyInfo (FILE *hKeyFile,CTIM *pTimHeader,KEY_MOD *pKeyInfo);
bool GetDWord (FILE *hKeyFile,char *szString,UINT_T *pValue);
bool ProcessKeyInfo (FILE *hKeyFile,CTIM *pTimHeader,KEY_MOD *pKeyInfo,FILE *hTimFile);
bool ProcessDsaInfo (FILE *hKeyFile,CTIM *pTimHeader,FILE *hTimFile,PLAT_DS *pDs);
bool BuildDigitalSignature (char *KeyFileName,char *HashFileName);
bool AddDigitalSignatureToTim (char *TimFileName,char *TimOutFileName,char *DsFileName);

bool ScanImageInfo (FILE *hKeyFile,CTIM *pTimHeader,IMAGE_INFO *pImageInfo);

#pragma warning ( disable : 4096 4081 ) // Disable warning messages

bool ParseTrustedDescriptorFile (FILE *hKeyFile,CTIM *pTimHeader)
{
	char szTimFilename[MAX_PATH], szImageFilename[MAX_PATH], szFileNamesList[MAX_IMAGE_FILES][MAX_PATH];
	UINT_T IntParam, NextImageID = 0;
	pIMAGE_INFO pImageInfo;
	KEY_MOD KeyInfo;
	PLAT_DS Ds;
	FILE *hTimFile, *hImage, *hReservedFile, *hTimHashFile, *hBinaryImage;
	Ipp8u DsHash[256];
	unsigned int HashSize;
	unsigned int TimSizeWithNoDS;
	char ReservedBuffer[MAX_PATH];
	
	if (iOption == 1)
		printf ("\nProcessing trusted TIM...\n\n");
	else
	{
		printf ("\nProcessing trusted TIM without digital signature and\n");
		printf ("building TIM hash binary file...\n\n");
	}

	if (GetDWord  (hKeyFile,"Issue Date: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->VersionBind.IssueDate = IntParam;
	if (GetDWord (hKeyFile,"OEM UniqueID: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->VersionBind.OEMUniqueID = IntParam;
	if (GetDWord (hKeyFile,"WTM Save State Flash Signature: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->FlashInfo.WTMFlashSign = IntParam;
	if (GetDWord (hKeyFile,"WTM Save State Entry: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->FlashInfo.WTMEntryAddr = IntParam;
	if (GetDWord (hKeyFile,"WTM Save State Backup Entry: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->FlashInfo.WTMEntryAddrBack = IntParam;

	pTimHeader->FlashInfo.WTMPatchSign = 0xFFFFFFFF;
	pTimHeader->FlashInfo.WTMPatchAddr = 0xFFFFFFFF;

	if (GetDWord (hKeyFile,"Boot Flash Signature: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->FlashInfo.BootFlashSign = IntParam;
	if (GetDWord (hKeyFile,"Number of Images: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->NumImages = IntParam;
	if (GetDWord (hKeyFile,"Number of Keys: %s\n",&IntParam) == FALSE) return FALSE;
	pTimHeader->NumKeys = IntParam;
	if (bIsReservedDataInFile)
	{
		if ((hReservedFile = fopen (ReservedFileName,"rb")) == NULL)
		{
			printf ("  Error: Cannot open file name %s !",ReservedFileName);
			if (hReservedFile != NULL) fclose (hReservedFile);
			return FALSE;
		}
		fseek (hReservedFile,0L,SEEK_END); // Set position to EOF
		pTimHeader->SizeOfReserved = ftell (hReservedFile); // Get position of file, thus the file size.
		fseek (hReservedFile,0L,SEEK_SET); // Set position to SOF
	}
	else
	{
		if (GetDWord (hKeyFile,"Size of Reserved in bytes: %s\n",&IntParam) == FALSE) return FALSE;
		pTimHeader->SizeOfReserved = IntParam;
	}

	//return error if number of images > 10
	if ((int)pTimHeader->NumImages > MAX_IMAGE_FILES )
	{
		printf("  Error: Maximum allowable number of images in the descriptor file is 10!\n");
		return FALSE;
	}
	
	// allocate space for all of the Image ID blocks
	pImageInfo = (pIMAGE_INFO) calloc((int)pTimHeader->NumImages, sizeof(IMAGE_INFO));

	for (int i = 0; i < (int)pTimHeader->NumImages; i++)
	{
		if (ScanImageInfo (hKeyFile,pTimHeader,&pImageInfo[i]) == FALSE) return FALSE;
		
		if (i == 0)
		{
			if (pImageInfo[i].ImageID != TIMIDENTIFIER)
			{
				printf ("  Error: The ImageID value is incorrect for the TIM image!\n");
				return FALSE;
			}

			if (fscanf (hKeyFile,"Image Filename: %s\n",szTimFilename) != 1)
			{
				printf ("  Error: Key file parsing error reading, Image Filename:\n");
				return FALSE;
			}

			if ((hTimFile = fopen (szTimFilename,"wb+")) == NULL)
			{
				printf ("  Error: Cannot open file name %s !",szTimFilename);
				return FALSE;
			}

			pImageInfo[i].ImageSize = sizeof (CTIM) + (sizeof (IMAGE_INFO) * pTimHeader->NumImages)
								+ (pTimHeader->SizeOfReserved)
								+ (sizeof (KEY_MOD) * pTimHeader->NumKeys) + sizeof (PLAT_DS);
			
			TimSizeWithNoDS = pImageInfo[i].ImageSize - (MAXKEYSIZEWORDS * 4);

			if ((pImageInfo[i].ImageSizeToHash > TimSizeWithNoDS) ||
				(pImageInfo[i].ImageSizeToHash == 0xFFFFFFFF) ||
				(pImageInfo[i].ImageSizeToHash == 0xFF) )
			{

				pImageInfo[i].ImageSizeToHash = TimSizeWithNoDS;
				// Full (N)TIM hash
				HashSize = pImageInfo[i].ImageSizeToHash;
			}
			else
			{
				// Partial hash
				HashSize = pImageInfo[i].ImageSizeToHash;
			}
			
			fwrite(pTimHeader, sizeof (CTIM),1,hTimFile);
		}
		else
		{
			if (pImageInfo[i].ImageID != NextImageID)
			{
				printf ("  Error: The ImageID doesn't = NextImageID from the previous image!");
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}

			if (fscanf (hKeyFile,"Image Filename: %s\n",szImageFilename) != 1)
			{
				printf ("  Error: Key file parsing error reading, Image Filename:\n");
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}

			for (int m = i-1; m<=i-1;m++)
			{
				for (int n=0; n<MAX_PATH;n++)
				{
				szFileNamesList[m][n] = szImageFilename[n];
				}
			}

			if ((hImage = fopen (szImageFilename,"rb")) == NULL)
			{
				printf ("  Error: Cannot open file name %s !",szImageFilename);
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}
			fseek (hImage,0L,SEEK_END); // Set position to EOF
			pImageInfo[i].ImageSize = ftell (hImage); // Get position of file, thus the file size.

			// Output new image with ImageID prepended
			if (PrependID (hImage,szImageFilename,&pImageInfo[i]) == FALSE)
			{
				if (hTimFile != NULL) fclose (hTimFile);
				if (hImage != NULL) fclose (hImage);
				return FALSE;
			}

			if ((pImageInfo[i].ImageSizeToHash > pImageInfo[i].ImageSize) ||
				(pImageInfo[i].ImageSizeToHash == 0xFFFFFFFF))
			{
				pImageInfo[i].ImageSizeToHash = pImageInfo[i].ImageSize;
			}

			if (GenerateHash (hImage,szImageFilename,&pImageInfo[i]) == FALSE)
			{
				if (hTimFile != NULL) fclose (hTimFile);
				if (hImage != NULL) fclose (hImage);
				return FALSE;
			}

			if (hImage != NULL) fclose (hImage);
		}

		if (pImageInfo[i].HashAlgorithmID == 2 || pImageInfo[i].HashAlgorithmID == 256)
			pImageInfo[i].HashAlgorithmID = 32; // SHA256
		else
			pImageInfo[i].HashAlgorithmID = 20; // SHA1

		fwrite (&pImageInfo[i],sizeof (IMAGE_INFO),1,hTimFile);

		NextImageID = pImageInfo[i].NextImageID;
	}

	//check to make sure the images do not overlap each other
	if(CheckImageOverlap(pImageInfo, pTimHeader->NumImages, pTimHeader->FlashInfo.BootFlashSign) == FALSE)
	{
		return FALSE;
	}

	if (ProcessKeyInfo (hKeyFile,pTimHeader,&KeyInfo,hTimFile) == FALSE)
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	if (bIsReservedDataInFile)
	{
		memset (ReservedBuffer,0,pTimHeader->SizeOfReserved);
		fread (ReservedBuffer,1,pTimHeader->SizeOfReserved,hReservedFile);
		fwrite (ReservedBuffer,sizeof (char),pTimHeader->SizeOfReserved,hTimFile);
		if (hReservedFile != NULL) fclose (hReservedFile);
	}
	else
	{
		// skip the reserved section if size is 0.
		if(pTimHeader->SizeOfReserved != 0){
			if (ProcessReservedData (hKeyFile,hTimFile,pTimHeader->SizeOfReserved) == FALSE)
			{
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}
		}
	}

	if (ProcessDsaInfo (hKeyFile,pTimHeader,hTimFile,&Ds) == FALSE)
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	if (VerifySizeOfTim (pTimHeader) == FALSE)
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	if (GenerateTimHash (hTimFile,pTimHeader,&Ds,(Ipp8u *)&DsHash, HashSize) == FALSE)
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	// If option 2 is selected the digital signature will be installed later

	if (iOption == 2)
	{
		DumpTimHashToBinFile (&Ds,(Ipp8u *)&DsHash, HashFileName);
		if (hTimFile != NULL) fclose (hTimFile);
		return TRUE;
	}

	if (EncryptSignature (true,&Ds,(Ipp8u *)&DsHash) == FALSE)
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}
	ConvertMSWToLSWAndFlip(&Ds);
	if (InstallDigitalSignature (szTimFilename,true) == FALSE)
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	if ( bIsTimVerify == true )
	{
		memset (DsHash,0,256);

		fseek (hTimFile,TimSizeWithNoDS,SEEK_SET); // Set position to DS

		fread (DsHash,sizeof(Ipp32u),MAXKEYSIZEWORDS,hTimFile);

		if (EncryptSignature (false,&Ds,(Ipp8u *)&DsHash) == FALSE)
		{
			if (hTimFile != NULL) fclose (hTimFile);
			return FALSE;
		}
//		ConvertMSWToLSWAndFlip(&Ds);
		if ( HashSize == TimSizeWithNoDS )
		{
				// Compare full Tim(minus DS) against the decrypted DS.
				if (VerifyTimHash (szTimFilename,&Ds,(Ipp8u *)&DsHash, TimSizeWithNoDS) == FALSE)
				{
					if (hTimFile != NULL) fclose (hTimFile);
					return FALSE;
				}
		}
		else
		{
			// Compare partial Tim against the decrypted DS.
			if (VerifyTimHash (szTimFilename,&Ds,(Ipp8u *)&DsHash, HashSize) == FALSE)
			{
				if (hTimFile != NULL) fclose (hTimFile);
				return FALSE;
			}
		}
	}

	unsigned int location = 0;
	BYTE padding = 0x0;
	BYTE Data[8192];
	int iBlockSize = 8192;

	if (bConcatenate)
	{
		if(pTimHeader->FlashInfo.BootFlashSign == 0x4e414e01)
		{
			printf("Cannot concatenate images for an MDOC device!\n");
			return FALSE;
		}

	 	
		for(int j=1;j<(int)pTimHeader->NumImages - 1;j++)
		{
			fseek (hTimFile,0L,SEEK_END); // Set position to EOF
			location = ftell(hTimFile);
			
			//pad out the current binary until we get to the location of where the next image should start
			if(pImageInfo[j].FlashEntryAddr > location)
				fwrite(&padding,1,(pImageInfo[j].FlashEntryAddr - location),hTimFile);
			
			//open the next image file
			if ((hBinaryImage = fopen (szFileNamesList[j],"rb")) == NULL)
			{
				printf ("  Error: Cannot open file name %s !",szFileNamesList[j]);
				return FALSE;
			}

			//now write out image in 8K chunks
			while (iBlockSize == 8192)
			{
				iBlockSize = fread (Data,1,iBlockSize,hBinaryImage);
				fwrite (Data,1,iBlockSize,hTimFile);
			}
			iBlockSize = 8192;
			if (hBinaryImage != NULL)
				fclose (hBinaryImage);
		}
	}

	printf ("  Success: Processing has completed successfully!\n");

	if (hTimFile != NULL) fclose (hTimFile);
	free(pImageInfo);
	return TRUE;
}

bool ProcessKeyInfo (FILE *hKeyFile,CTIM *pTimHeader,KEY_MOD *pKeyInfo,FILE *hTimFile)
{
	bool IsOk;

	for (int i = 0; i < (int)pTimHeader->NumKeys; i++)
	{
		memset (pKeyInfo,0,sizeof (KEY_MOD));
		if (ScanKeyInfo (hKeyFile,pTimHeader,pKeyInfo) == FALSE) return FALSE;

		if (pKeyInfo->HashAlgorithmID == 2 || pKeyInfo->HashAlgorithmID == 256)
			pKeyInfo->HashAlgorithmID = 32; // SHA256
		else
			pKeyInfo->HashAlgorithmID = 20; // SHA1

		if (pKeyInfo->ModulusSize != 128 && pKeyInfo->ModulusSize != 256)
		{
			printf ("  Error: The Modulus Size is incorrect for Key ID %X\n",pKeyInfo->KeyID);
			return FALSE;
		}

		if (fscanf (hKeyFile,"RSA Public Key Exponent:\n") != 0)
		{
			printf ("  Error: Key file parsing error reading, RSA Public Key Exponent:\n");
			return FALSE;
		}
		IsOk = FillDataArrays ( hKeyFile,(unsigned long *)&pKeyInfo->RSAPublicExponent[0],
								pKeyInfo->PublicKeySize );
		if (IsOk == FALSE)
		{
			printf ("Public Key Size in bytes = %d\n\n",pKeyInfo->PublicKeySize);
			printf ("  Error: The amount of data read from key file doesn't match what is\n");
			printf ("indicated by 'Public Key Size in bytes' directive!\n");
			printf ("Error: RSA Public Key Exponent for Key ID: %X\n",pKeyInfo->KeyID);
			printf ("////////////////////////////////////////////////////////////\n");
			return FALSE;
		}

		if (fscanf (hKeyFile,"RSA System Modulus:\n") != 0)
		{
			printf ("Key file parsing error reading, RSA System Modulus:\n");
			return FALSE;
		}
		IsOk = FillDataArrays ( hKeyFile,(unsigned long *)&pKeyInfo->RSAModulus[0],
								pKeyInfo->ModulusSize );
		if (IsOk == FALSE)
		{
			printf ("Modulus Size in bytes = %d\n\n",pKeyInfo->ModulusSize);
			printf ("  Error: The amount of data read from key file doesn't match what is\n");
			printf ("indicated by 'Modulus Size in bytes' directive!\n");
			printf ("Error: RSA System Modulus for Key ID: %X\n",pKeyInfo->KeyID);
			printf ("////////////////////////////////////////////////////////////\n");
			return FALSE;
		}

		if (GenerateKeyHash (pKeyInfo) == FALSE)
		{
			printf ("  Error: Generating the Key Hash has failed!\n");
			return FALSE;
		}

		fwrite (pKeyInfo,sizeof (KEY_MOD),1,hTimFile);
	}

	return TRUE;
}

bool ProcessDsaInfo (FILE *hKeyFile,CTIM *pTimHeader,FILE *hTimFile,PLAT_DS *pDs)
{
	bool IsOk;
	UINT_T IntParam;
	unsigned int HashBuffer[256];
	unsigned int KeyBuffer[256];
	int i, j;
	unsigned int KeyLength;
	PLAT_DS pTemp_Ds;

	memset (pDs,0,sizeof (PLAT_DS));

	if (GetDWord (hKeyFile,"DSA Algorithm ID: %s\n",&IntParam) == FALSE) return FALSE;
	pDs->DSAlgorithmID = IntParam;
	if (GetDWord (hKeyFile,"Hash Algorithm ID: %s\n",&IntParam) == FALSE) return FALSE;
	pDs->HashAlgorithmID = IntParam;
	if (GetDWord (hKeyFile,"Modulus Size in bytes: %s\n",&IntParam) == FALSE) return FALSE;
	pDs->ModulusSize = IntParam;

	if (pDs->DSAlgorithmID != 1 && pDs->DSAlgorithmID != 2) pDs->DSAlgorithmID = 0;

	if (pDs->HashAlgorithmID == 2 || pDs->HashAlgorithmID == 256)
		pDs->HashAlgorithmID = 32; // SHA256
	else
		pDs->HashAlgorithmID = 20; // SHA1

	if (fscanf (hKeyFile,"RSA Public Exponent:\n") != 0)
	{
		printf ("  Error: Key file parsing error reading, RSA Public Exponent:\n");
		return FALSE;
	}
	IsOk = FillDataArrays ( hKeyFile,(unsigned long *)pDs->RSAPublicExponent,
							pDs->ModulusSize );
	if (IsOk == FALSE)
	{
		printf ("Modulus Size in bytes = %d\n\n",pDs->ModulusSize);
		printf ("  Error: The amount of data read from key file doesn't match what is\n");
		printf ("indicated by 'Modulus Size in bytes' directive!\n");
		printf ("Error: RSA Public Exponent for DSA Algorithm\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	if (fscanf (hKeyFile,"RSA System Modulus:\n") != 0)
	{
		printf ("  Error: Key file parsing error reading, RSA System Modulus:\n");
		return FALSE;
	}
	IsOk = FillDataArrays ( hKeyFile,(unsigned long *)pDs->RSAModulus,
							pDs->ModulusSize );
	if (IsOk == FALSE)
	{
		printf ("Modulus Size in bytes = %d\n\n",pDs->ModulusSize);
		printf ("  Error: The amount of data read from key file doesn't match what is\n");
		printf ("indicated by 'Modulus Size in bytes' directive!\n");
		printf ("Error: RSA System Modulus for DSA Algorithm\n");
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	if (iOption == 1)
	{
		if (fscanf (hKeyFile,"RSA Private Key:\n") != 0)
		{
			printf ("Key file parsing error reading, RSA Private Key:\n");
			return FALSE;
		}
		IsOk = FillDataArrays ( hKeyFile,(unsigned long *)RsaPrivateKey,
							pDs->ModulusSize );
		if (IsOk == FALSE)
		{
			printf ("Modulus Size in bytes = %d\n\n",pDs->ModulusSize);
			printf ("  Error: The amount of data read from key file doesn't match what is\n");
			printf ("indicated by 'Modulus Size in bytes' directive!\n");
			printf ("Error: RSA Private Key for DSA Algorithm\n");
			printf ("////////////////////////////////////////////////////////////\n");
			return FALSE;
		}
	}
	// PKCS_1.5: The data read from the descriptor file is in LSW, Little Endian format. 
	// Endian conversions and word swaps are required for PKCS 1.5 in order to make the
	// data compatible with the Fast signature PI.
	if (pDs->DSAlgorithmID == PKCS1_v1_5)
	{

		// PKCS_1.5: Store data in a temporary structure just for PKCS. This structure will  
		// be written into TIM binary.
		memset (&pTemp_Ds,0,sizeof (PLAT_DS));
		pTemp_Ds.DSAlgorithmID = pDs->DSAlgorithmID;
		pTemp_Ds.HashAlgorithmID = pDs->HashAlgorithmID;
		pTemp_Ds.ModulusSize = pDs->ModulusSize;
		KeyLength = pDs->ModulusSize / 4;
		// PKCS_1.5:Convert Public Exponent, the modulus from LSW to MSW and to Big Endian.
		// BE format is required for all ippcp functions.
		for (i = 0, j = (int)KeyLength - 1; i <(int)KeyLength; i++, j--)
		{
			KeyBuffer[j] = pDs->RSAPublicExponent[i];
			Endian_Convert(*(unsigned int *)&KeyBuffer[j], (unsigned int *)&KeyBuffer[j]);
		}
		for (i = 0, j=(int)(KeyLength * 2)-1; i <(int)KeyLength; i++, j--)
		{
			KeyBuffer[j] = pDs->RSAModulus[i];
			Endian_Convert(*(unsigned int *)&KeyBuffer[j], (unsigned int *)&KeyBuffer[j]);
		}
		// PKCS_1.5:Compute SHA hash.
		if (pDs->HashAlgorithmID == 32){
			ippsSHA256MessageDigest ((Ipp8u *)KeyBuffer,KeyLength*4*2,(Ipp8u *)HashBuffer);
		}
		else
		{
			ippsSHA1MessageDigest ((Ipp8u *)KeyBuffer,KeyLength*4*2,(Ipp8u *)HashBuffer);
		}
		//PKCS_1.5:Endian convert the hash and store it in the temp struct.
		for (i = 0; i < 5; i++)
		{
			Endian_Convert (HashBuffer[i], &HashBuffer[i]);
			pTemp_Ds.Hash[i] = (UINT_T)HashBuffer[i];
		}
		
		for (i = 0; i <(int)(KeyLength * 2); i++)
		{
			Endian_Convert(*(unsigned int *)&KeyBuffer[i], (unsigned int *)&KeyBuffer[i]);
		}

		// PKCS_1.5:Store Public exponent in temp struct
		for (i = 0; i <(int)KeyLength; i++)
		{
			pTemp_Ds.RSAPublicExponent[i] = (UINT_T)KeyBuffer[i];
		}
		// PKCS_1.5:Store Modulus in temp struct
		for (i = 0, j=KeyLength; i <KeyLength; i++, j++)
		{
			pTemp_Ds.RSAModulus[i] = (UINT_T)KeyBuffer[j];
		}

	
	
	}		 

	GenerateOtpHash (pTimHeader,pDs);

	if (pDs->DSAlgorithmID == PKCS1_v1_5)
	{
		fwrite (&pTemp_Ds,sizeof (PLAT_DS) - (sizeof (UINT_T) * MAXKEYSIZEWORDS),1,hTimFile);
	}
	else
	{
		fwrite (pDs,sizeof (PLAT_DS) - (sizeof (UINT_T) * MAXKEYSIZEWORDS),1,hTimFile);
	}

	return TRUE;
}

bool BuildDigitalSignature (char *KeyFileName,char *HashFileName)
{
	FILE *hTimHash, *hKeyFile;
	char szDword[11];
	Ipp8u DsHash[256];
	long lFileSize;
	PLAT_DS Ds;
	bool IsOk;

	// This function is used in option mode 3 where a digital signature
	// is produced from a binary hash of the TIM and a private key file.

	printf ("\nBuilding Digital Signature...\n\n");

	// Reading TIM hash from a file

	if ((hTimHash = fopen (HashFileName,"rb")) == NULL)
	{
		printf ("\n  Error: Cannot open file name %s !\n",HashFileName);
		return FALSE;
	}

	fseek (hTimHash,0L,SEEK_END); // Set position to EOF
	lFileSize = ftell (hTimHash); // Get position of file, thus the file size.
	fseek (hTimHash,0L,SEEK_SET); // Set position to SOF

	if (lFileSize > 256)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: BuildDigitalSignature Error:\n");
		printf ("The size of the input hash file is greater than 256 bytes!\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hTimHash != NULL) fclose (hTimHash);
		return FALSE;
	}

	memset (DsHash,0,256);
	fread (DsHash,1,lFileSize,hTimHash);

	if (hTimHash != NULL) fclose (hTimHash);

	// Reading private key from a file

	if ((hKeyFile = fopen (KeyFileName,"r")) == NULL)
	{
		printf ("\n  Error: Cannot open file name %s !\n",KeyFileName);
		return FALSE;
	}

	if (fscanf (hKeyFile,"DSA Algorithm ID: %s\n",szDword) != 1)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: Private Key File Parsing Error:\n");
		printf ("Key file parsing error reading, DSA Algorithm ID:\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}

	Ds.DSAlgorithmID = Translate (szDword);

	if (fscanf (hKeyFile,"Modulus Size in bytes: %s\n",szDword) != 1)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: Private Key File Parsing Error:\n");
		printf ("Key file parsing error reading, Modulus Size in bytes:\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}

	Ds.ModulusSize = Translate (szDword);

	if (fscanf (hKeyFile,"RSA System Modulus:\n") != 0)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: Private Key File Parsing Error:\n");
		printf ("Key file parsing error reading, RSA System Modulus:\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}
	IsOk = FillDataArrays ( hKeyFile,(unsigned long *)Ds.RSAModulus,
							Ds.ModulusSize );
	if (IsOk == FALSE)
	{
		printf ("  Error: Private Key File Parsing Error:\n");
		printf ("Modulus Size in bytes = %d\n\n",Ds.ModulusSize);
		printf ("The amount of data read from key file doesn't match what is\n");
		printf ("indicated by 'Modulus Size in bytes' directive!\n");
		printf ("Error: RSA System Modulus for DSA Algorithm\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}

	if (fscanf (hKeyFile,"RSA Private Key:\n") != 0)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: Private Key File Parsing Error:\n");
		printf ("Key file parsing error reading, RSA Private Key:\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}
	IsOk = FillDataArrays ( hKeyFile,(unsigned long *)RsaPrivateKey,
							Ds.ModulusSize );
	if (IsOk == FALSE)
	{
		printf ("  Error: Private Key File Parsing Error:\n");
		printf ("Modulus Size in bytes = %d\n\n",Ds.ModulusSize);
		printf ("The amount of data read from key file doesn't match what is\n");
		printf ("indicated by 'Modulus Size in bytes' directive!\n");
		printf ("Error: RSA Private Key for DSA Algorithm\n");
		printf ("////////////////////////////////////////////////////////////\n");
		if (hKeyFile != NULL) fclose (hKeyFile);
		return FALSE;
	}

	if (hKeyFile != NULL) fclose (hKeyFile);

//	if (EncryptSignature (NULL,&Ds,(Ipp8u *)&DsHash) == FALSE)
	if (EncryptSignature (true,&Ds,(Ipp8u *)&DsHash) == FALSE)
	{
		return FALSE;
	}
	ConvertMSWToLSWAndFlip(&Ds);
	// Writing digital signature to a file

	InstallDigitalSignature (DsFileName,false);

	return TRUE;
}

bool AddDigitalSignatureToTim (char *TimInFileName,char *TimOutFileName,char *DsFileName)
{
	FILE *hDsFile;
	long lFileSize, lSizeOfDs = sizeof (UINT_T) * MAXKEYSIZEWORDS;

	// This function is used in option mode 4 where a digital signature is read
	// from a binary file and appended at the end of a TIM binary file.

	printf ("\nInstalling Digital Signature...\n\n");

	// Reading digital signature from a file

	if ((hDsFile = fopen (DsFileName,"rb")) == NULL)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: AddDigitalSignatureToTim Error:\n");
		printf ("Cannot open file name %s !\n",DsFileName);
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}

	fseek (hDsFile,0L,SEEK_END); // Set position to EOF
	lFileSize = ftell (hDsFile); // Get position of file, thus the file size.
	fseek (hDsFile,0L,SEEK_SET); // Set position to SOF

	if (lFileSize != lSizeOfDs)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: AddDigitalSignatureToTim Error:\n");
		printf ("The size of the digital signature file != %d!\n",lSizeOfDs);
		printf ("////////////////////////////////////////////////////////////\n");
		if (hDsFile != NULL) fclose (hDsFile);
		return FALSE;
	}

	memset (DigitalSignature,0,MAXKEYSIZEWORDS);
	fread (DigitalSignature,1,lFileSize,hDsFile);

	if (hDsFile != NULL) fclose (hDsFile);

/*	if (CopyFile (TimInFileName,TimOutFileName,FALSE) == 0)
	{
		printf ("////////////////////////////////////////////////////////////\n");
		printf ("  Error: AddDigitalSignatureToTim Error:\n");
		printf ("Failed to open TIM input image file: %s!\n",TimInFileName);
		printf ("////////////////////////////////////////////////////////////\n");
		return FALSE;
	}*/

	InstallDigitalSignature (TimOutFileName,true);

	return TRUE;
}

bool ScanKeyInfo (FILE *hKeyFile,CTIM *pTimHeader,KEY_MOD *pKeyInfo)
{
	UINT_T IntParam;

	// This is a helper function to parse key info data from the descriptor
	// text file supplied by the user.

	if (GetDWord (hKeyFile,"Key ID: %s\n",&IntParam) == FALSE) return FALSE;
	pKeyInfo->KeyID = IntParam;
	if (GetDWord (hKeyFile,"Hash Algorithm ID: %s\n",&IntParam) == FALSE) return FALSE;
	pKeyInfo->HashAlgorithmID = IntParam;
	if (GetDWord (hKeyFile,"Modulus Size in bytes: %s\n",&IntParam) == FALSE) return FALSE;
	pKeyInfo->ModulusSize = IntParam;
	if (GetDWord (hKeyFile,"Public Key Size in bytes: %s\n",&IntParam) == FALSE) return FALSE;
	pKeyInfo->PublicKeySize = IntParam;

	return TRUE;
}


