/***************************************************************************
**
**  (C) Copyright December 2006 Marvell International Ltd.
**
**  All Rights Reserved.
**
**  This software file (the "File") is distributed by Marvell International  
**  Ltd. under the terms of the GNU General Public License Version 2, June  
**  1991 (the "License").  You may use, redistribute and/or modify this File   
**  in accordance with the terms and conditions of the License, a copy of  
**  which is available along with the File in the license.txt file or by   
**  writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
**  Boston, MA 02111-1307 or on the worldwide web at 
**  http://www.gnu.org/licenses/gpl.txt.
**
**  THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
**  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
**  PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
**  details about this warranty disclaimer. 
**
**  FILENAME:	TIM.h
**
**  PURPOSE: 	Defines the Trusted/Non-Trusted Image Module
**  
****************************************************************************/

#ifndef __TIM_H__
#define __TIM_H__

#define DEBUG 1
/******************* TIM Layout ************************************/
/* Below Presents the Maxiumum Size TIM layout in memory

VERSION_I      	VersionBind;         			// 0                                             
FLASH_I        	FlashInfo;           			// 0x10                                                 
UINT_T         	NumImages;           			// 0x20                
UINT_T         	NumKeys;						// 0x24
UINT_T         	SizeOfReserved;					// 0x28
IMAGE_INFO 		IMG[MAX_IMAGES];                // 0x2C                    
KEY_MOD        	Key[MAX_KEYS];                  // 0x284
UINT_T         	Reserved[0x4E8];       			// 0x908  (0x4E8 bytes)                                     
PLAT_DS        	TBTIM_DS;                       // 0xDF0                                  
												// 0x1000 = 4 KB
Below Presents the Maxiumum Size NTIM layout in memory

VERSION_I   	VersionBind;         			// 0                                             
FLASH_I     	FlashInfo;           			// 0x10                                                 
UINT_T      	NumImages;           			// 0x20                
UINT_T         	NumKeys;						// 0x24
UINT_T      	SizeOfReserved;					// 0x28
IMAGE_INFO 		IMG[MAX_IMAGES];                // 0x2C                      
UINT_T      	Reserved[0xD80];       			// 0x284                                       
NTIM, *pNTIM;									// 0x1000 = 4KB
/******************* TIM Layout ************************************/

// B1 TIM
//#include "typedef.h"
#define MAX_IMAGES			10
#define MAX_KEYS	 		3
#define MAX_TIM_SIZE 		0x1000
#define TIMBUFFER 			4096    		//4k for a Tim structure size
#define MAXKEYSIZEWORDS		64				// 2048 bits

// Predefined Image Identifiers
#define TIMIDENTIFIER		0x54494D48		// "TIMH"
#define OBMIDENTIFIER		0x4F424D49		// "OBMI"
#define DKBIDENTIFIER		0x444B4249		// "DKBI"
#define JTAGIDENTIFIER		0x4A544147		// "JTAG"
#define PATCHIDENTIFIER		0x50415443		// "PATC"
#define TCAIDENTIFIER		0x5443414B		// "TCAK"
#define OSLOADERID			0x4F534C4F		// "OSLO"

// WTP Format Recognized Reserved Area Indicator
#define WTPRESERVEDAREAID	0x4F505448      // "OPTH"

// Reserved Area Package Headers
#define DDRID				0x44447248		// "DDRH"
#define AUTOBIND			0x42494E44		// "BIND"
#define TERMINATORID		0x5465726D		// "Term"
#define GPIOID				0x4750494F		// "GPIO"
#define UARTID				0x55415254		// "UART"
#define USBID				0x00555342		// "USB"
#define RESUMEID			0x5265736D		// "Resm"
#define USBVENDORREQ		0x56524551		// "VREQ"

#define 	DESCRIPTOR_RES_ID			0x55534200
#define 	NUM_USB_DESCRIPTORS			10
#define 	MAX_USB_STRINGS 			7

typedef enum
{
 USB_DEVICE_DESCRIPTOR   				= (DESCRIPTOR_RES_ID | 0),
 USB_CONFIG_DESCRIPTOR   				= (DESCRIPTOR_RES_ID | 1),
 USB_INTERFACE_DESCRIPTOR 				= (DESCRIPTOR_RES_ID | 2),
 USB_LANGUAGE_STRING_DESCRIPTOR   		= (DESCRIPTOR_RES_ID | 3),
 USB_MANUFACTURER_STRING_DESCRIPTOR   	= (DESCRIPTOR_RES_ID | 4),
 USB_PRODUCT_STRING_DESCRIPTOR   	  	= (DESCRIPTOR_RES_ID | 5),
 USB_SERIAL_STRING_DESCRIPTOR   	  	= (DESCRIPTOR_RES_ID | 6),
 USB_INTERFACE_STRING_DESCRIPTOR   		= (DESCRIPTOR_RES_ID | 7),
 USB_DEFAULT_STRING_DESCRIPTOR			= (DESCRIPTOR_RES_ID | 8),
 USB_ENDPOINT_DESCRIPTOR 				= (DESCRIPTOR_RES_ID | 9)
} USB_DESCRIPTORS;


// Global Identifiers
#define FFIDENTIFIER		0x00004646		// "FF"
#define ALTIDENTIFIER		0x00414C54		// "ALT"
#define DIFFIDENTIFIER		0x44696666		// "Diff"
#define SEIDENTIFIER		0x00005345		// "SE"
#define U2DIDENTIFIER		0x55534232		// "USB2"
#define PINSIDENTIFIER		0x50696E73		// "Pins"

// Miscellanous
#define BINDKEYSIZE			128				// (MAXKEYSIZEWORDS * 2) 
#define FLASHSIGMASK		0x0000000F		// mask off the 4 bits of platform state

typedef unsigned short UINT16_T;
typedef unsigned int UINT_T;
typedef unsigned long UINT32_T;

/********** WTP Recognized Reserved Area Layout ********************************
*
*	WTPTP_Defined_Reserved_Format_ID    \	  This clues BR, OBM and DKB that the reserved area is in a known format
*	Number of Reserved Area Packages    /	  For each package there is a header, payload size and payload	
*
*	Header	 	 		\	  Indicates what type of a Reserved Area Package	
*	Size		  		 \	  Size Comprises a single Reserved Area Package
*	Payload		  		 /	  There may be any number of Packages so long as TIM/NTIM < 4KB	
*			 			/	
*
*	Header		 		\	  The Last one should be a Package with a Terminator Header
*	Size		 		/	  The size should be 2 words (8 bytes) and there should be no Payload
*
**********************************************************************************/
typedef struct
{
 UINT_T WTPTP_Reserved_Area_ID;	  	// This clues BR, OBM and DKB that the reserved area is in a known format
 UINT_T	NumReservedHeaders;	  		// For each package there is a header, payload size and payload	
}WTP_RESERVED_AREA, *pWTP_RESERVED_AREA;

typedef struct
{
 UINT_T Identifier;					// Identification of this reserved area entry
 UINT_T Size;						// Size  = Payload Size + 2 words (8 bytes).
}WTP_RESERVED_AREA_HEADER, *pWTP_RESERVED_AREA_HEADER;

typedef struct
{
 WTP_RESERVED_AREA_HEADER WRAH;
 UINT_T 	bmRequestType;
 UINT_T 	bRequest;
 UINT_T		wValue;
 UINT_T		wIndex;
 UINT_T		wLength;
 UINT_T		wData; 							// First word of the proceeding Data. Note, there may be more traialing data
} USB_VENDOR_REQ, *pUSB_VENDOR_REQ;			// There is no restriction that data need be 32 bit aligned.

typedef struct
{
 volatile int *Addr;
 UINT_T Value;
}GPIO_DEF, *pGPIO_DEF;

typedef struct
{
 WTP_RESERVED_AREA_HEADER WRAH;
 UINT_T PortType;
 UINT_T Port;
}OPT_PROTOCOL_SET, *pOPT_PROTCOL_SET;

typedef struct
{
 WTP_RESERVED_AREA_HEADER WRAH;
 UINT_T NumGpios;
 pGPIO_DEF	GPIO;
}OPT_GPIO_SET, *pOPT_GPIO_SET;

typedef struct
{
 WTP_RESERVED_AREA_HEADER WRAH;
 UINT_T ACCR_VALUE;
 UINT_T MDCNFG_VALUE;
 UINT_T DDR_HCAL_VALUE;
 UINT_T MDREFR_VALUE;
}OPT_DDR_SET, *pOPT_DDR_SET;

typedef struct
{
 WTP_RESERVED_AREA_HEADER WRAH;
 UINT_T	ImageAddr;
 UINT_T	ImageSize;
 UINT_T	ImageCRC; 
}OPT_RESM_LOC, *pOPT_RESM_LOC;

typedef struct
{
 WTP_RESERVED_AREA_HEADER WRAH;
 UINT_T AutoBind;
}OPT_AUTOBIND, *pOPT_AUTOBIND;

/****  Used only for backwards compatability!!! ***/
typedef struct
{
 UINT_T Identifier;
 UINT_T PortType;
 UINT_T Port;
 UINT_T GPIOPresent;
 UINT_T NumGpios;
 pGPIO_DEF	GPIO;
}OPT_SET, *pOPT_SET;

//mDOC general formatting parameters
// look in flstruct.h and flcommon.h
// for parameter definitions
// not spelled out here
//----------------------------------
typedef struct
{								/* byte totals */
 WTP_RESERVED_AREA_HEADER WRAH;	/*0x08 bytes */
 UINT_T percentUse;				/*0x0C */        /* NAND flash inherently contains some bad blocks. TrueFFS handles
                                                  * those blocks by managing a pool of spare blocks also called transfer
                                                  * units. This format parameter specifies the percentage of the media
                                                  * to use.  Starting from TrueFFS 6.3.0 alpha 4 this field is no longer a
                                                  * configurable formatting argument. You must set this field to
                                                  * TL_DEFAULT_PERCENTAGE = 0, letting TrueFFS to choose this value for you. */
 UINT_T noOfBDTLPartitions;	    /*0x10 */  /* Indicates the number of Disk partition (1 to 4). */
 UINT_T noOfBinaryPartitions;   /*0x14 */  /* Indicates the number of binary partitions (up to 3). 0 will
                                            * cause formatting with no binary partition. This value is ignored
                                            * if TL_LEAVE_BINARY_AREA flag is set in the irFlags field the ioreq */
 UINT_T cascadedDeviceNo;       /*0x18*/  /* Not used */
 UINT_T noOfCascadedDevices;    /*0x1C*/  /* Not used */
 UINT_T progressCallback;  	    /*0x20*/  /* Progress callback routine, will be called if not NULL.
                                             The callback routine is called after erasing each unit,
                                             and its parameters are the total number of erase units
                                             to format and the number erased so far.
                                             The callback routine returns a Status value. A value of
                                             OK (0) allows formatting to continue. Any other value
                                             will abort the formatting with the returned status code. */
                                          /* In case the FL_REPORT_MOUNT_PROGRESS compilation flag is set the
                                             routine will also report the mount progress of the mount routine
                                             for each of the Disk partitions. A 0,0 return values will report
                                             the end of a format status and of a mount operation */
 UINT_T FP_0_RFU_0;             /*0x24*/  /* Reserved 0 */
 UINT_T FP_1_RFU_0;             /*0x28*/  /* Reserved 0 */

  /* Note the following section is not used by for DiskOnChips */
  /*************************************************************/
 UINT_T   bootImageLen;         /*0x2C*/  /* Reserved 0     */
 UINT_T   vmAddressingLimit;    /*0x30*/  /* Reserved 0     */
 UINT_T   embeddedCISlength;    /*0x34*/  /* Reserved 0     */
 UINT_T   *embeddedCIS;         /*0x38*/  /* Reserved NULL  */
}MDOC_FORMAT_INFO, *pMDOCFORMAT_INFO;

//mDOC BDTL partition type
//------------------------
typedef	struct
{
  WTP_RESERVED_AREA_HEADER WRAH;	/*0x08 bytes */
  UINT_T  length;                   /*0x0c */      /* The size in bytes of the usable storage space. The size
                                                    rounded upwards to a multiplication of a block size.
                                                    The size of the last partition will calculated automatically,
                                                    but if the requested size is greater then the remaining space
                                                    an error code will be returned,. Requesting zero size for any
                                                    partition but the last will generate an flBadParameters status. */

  UINT_T  fastAreaLength;			/*0x10*/
  UINT_T  lengthType;				/*0x14*/ 
  UINT_T  fastAreaLengthType;		/*0x18*/
  UINT_T  fastAreaVirtualFactor;    /*0x1C*/ /* Should be set to FL_NORMAL_SPACING = 1 */
  UINT_T  noOfSpareUnits;           /*0x20*/ /* BDTL needs at least one spare erase unit in order to function
                                               as a read/write media. It is possible to specify more than one
                                               spare unit, which takes more media space. The advantage of
                                               specifying more than one spare unit is that if one of the flash
                                               erase units becomes bad and un-erasable in the future, then one
                                               of the spare units can replace it. In that case, a second spare
                                               unit enables TrueFFS to continue its read/write functionality,
                                               whereas if no second spare unit is available the media goes into
                                               read-only mode. The standard value used is 3 */

  UINT_T  BDTLFP_0_RFU_0;           /*0x24*/  /* Reserved 0  */
  UINT_T  BDTLFP_1_RFU_0;           /*0x28*/  /* Reserved 0  */
  UINT_T  BDTLFP_2_RFU_0;           /*0x2C*/  /* Reserved 0  */
  UINT_T  BDTLFP_3_RFU_0;           /*0x30*/  /* Reserved 0  */
  UINT_T  protectionKey[8];         /*0x50*/  /* The key for the protection*/
  UINT_T  protectionType;           /*0x54*/  /* PROTECTABLE          - Can receive protection           */
                                              /* READ_PROTECTED       - Protect against read operations  */
                                              /* WRITE_PROTECTED      - Protect against write operations */
                                              /* LOCK_ENABLED         - Enables the hardware lock signal */
                                              /* PROTECTABLE          - This partition can be protected  */
                                              /* CHANGEABLE_PROTECTION - protection type can be changed  */
                                              /* The ratio between the real media size and the virtual size
                                                reported to the file system when compression is active. */

}MDOC_BDTL_PARTITION, *pMDOC_BDTL_PARTITION;

//mDOC binary partition type
//------------------------
typedef	struct
{
  WTP_RESERVED_AREA_HEADER WRAH;  /*0x08 bytes */
  UINT_T  length;                 /*0x0c*/            /* Required number of usable bytes (rounded to units) */
                                                      /* in the  partition.                                 */

  UINT_T   lengthType;            /*0x10*/       /* Must be FL_LENGTH_IN_BYTES = 0                     */
  UINT_T   sign[4];               /*0x20*/       /* signature of the binary  partition to format. Typically "BIPO"
                                                    The signature 0xFFFF FFFF is not a valid signature  */

  UINT_T  flags;                  /*0x24*/       /* TL_NORMAL_FORMAT=0  / TL_SPL_FORMAT=0xF0            */
  UINT_T  spareBlocks;            /*0x28*/   /* Spare blocks for DBB support, should be at least 1              */
  UINT_T  BDKFP_0_RFU_0;          /*0x2C*/           /* Reserved 0  */
  UINT_T  BDKFP_1_RFU_0;          /*0x30*/           /* Reserved 0  */
  UINT_T  BDKFP_2_RFU_0;          /*0x34*/           /* Reserved 0  */
  UINT_T  BDKFP_3_RFU_0;          /*0x38*/           /* Reserved 0  */
  UINT_T  protectionKey[8];       /*0x58*/     /* The key for the protection*/
  UINT_T  protectionType;         /*0x5C*/             /* PROTECTABLE          - Can receive protection           */
                                               /* READ_PROTECTED       - Protect against read operations  */
                                               /* WRITE_PROTECTED      - Protect against write operations */
                                               /* LOCK_ENABLED         - Enables the hardware lock signal */
                                               /* PROTECTABLE          - This partition can be protected  */
                                               /* CHANGEABLE_PROTECTION - protection type can be changed  */
}MDOC_BINARY_PARTITION, *pMDOC_BINARY_PARTITION;

/*********************************************************************************/

typedef enum
{
	NetworkOperatorCAKey, 
	SoftwareUpdateCAKey, 
	DRMDeviceCAKey, 
	OEMReservedKey1, 
	OEMReservedKey2, 
	NUMKEYMODULES
} 
KEYMODULES_T;

typedef enum
{
	Intel_DS, 
	PKCS1_v1_5, 
	PKCS1_v2_1
} 
DSALGORITHMID_T;

typedef struct 
{
 UINT_T Version;
 UINT_T	Identifier;					// "TIMH" 
 UINT_T Trusted;					// 1- Trusted, 0 Non                          
 UINT_T IssueDate;                       
 UINT_T OEMUniqueID;                     
} VERSION_I, *pVERSION_I;			// 0x10 bytes		

typedef struct 
{
 UINT_T WTMFlashSign;                    
 UINT_T WTMEntryAddr;                    
 UINT_T WTMEntryAddrBack;
 UINT_T WTMPatchSign;
 UINT_T WTMPatchAddr;
 UINT_T BootFlashSign;                   
} FLASH_I, *pFLASH_I;				// 0x10 bytes

typedef struct 
{
 UINT_T	KeyID;						// Associate an ID with this key
 UINT_T HashAlgorithmID;            // 160 - SHA1, 256 - SHA2
 UINT_T ModulusSize;				// 1024 bit or 2048 bit keys
 UINT_T PublicKeySize;		
 UINT_T RSAPublicExponent[MAXKEYSIZEWORDS];		// Contents depend on PublicKeySize	
 UINT_T RSAModulus[MAXKEYSIZEWORDS];// Up to 2K bits	
 UINT_T KeyHash[8]; 				// Reserve 256 bits for the hash
} KEY_MOD, *pKEY_MOD;				// 0x22C bytes	

typedef struct
{
 UINT_T ImageID;					// Indicate which Image
 UINT_T NextImageID;				// Indicate next image in the chain
 UINT_T FlashEntryAddr;			 	// Block numbers for NAND
 UINT_T LoadAddr;
 UINT_T ImageSize;
 UINT_T ImageSizeToHash;
 UINT_T HashAlgorithmID;            // 160 - SHA1, 256 - SHA2
 UINT_T Hash[8];					// Reserve 256 bits for the hash
} IMAGE_INFO, *pIMAGE_INFO;			// 0x3C bytes

typedef struct
{
 UINT_T DSAlgorithmID; 				// See DSALGORITHMID_T
 UINT_T HashAlgorithmID;            // 160 - SHA1, 256 - SHA2
 UINT_T ModulusSize;               	// 1024 bit or 2048 bit keys
 UINT_T Hash[8];					// Reserve 256 bits for the hash
 UINT_T RSAPublicExponent[MAXKEYSIZEWORDS];        
 UINT_T RSAModulus[MAXKEYSIZEWORDS];           	// Up to 2K bits	
 UINT_T RSADigS[MAXKEYSIZEWORDS];				// Contains TIM Hash   
} PLAT_DS, *pPLAT_DS;				// 0x780 bytes

// Constant part of the TIMs
typedef struct
{
 VERSION_I      VersionBind;         			// 0                                             
 FLASH_I        FlashInfo;           			// 0x10                                                 
 UINT_T         NumImages;           			// 0x20                
 UINT_T         NumKeys;						// 0x24
 UINT_T         SizeOfReserved;					// 0x28
} CTIM, *pCTIM;									// 0x2C

// TIM structure for use by DKB/OBM/BootROM
typedef struct
{
 pCTIM 			pConsTIM;			// Constant part
 pIMAGE_INFO	pImg;				// Pointer to Images
 pKEY_MOD		pKey;				// Pointer to Keys
 UINT_T*		pReserved;			// Pointer to Reserved Area
 pPLAT_DS       pTBTIM_DS;			// Pointer to Digital Signature
} TIM, *pTIM;

// NTIM structure for use by DKB/OBM/BootROM
typedef struct
{
 pCTIM 			pConsTIM;			// Constant part
 pIMAGE_INFO	pImg;				// Pointer to Images
 UINT_T*		pReserved;			// Pointer to Reserved Area
} NTIM, *pNTIM;
                      
#endif

