/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED. 
**
**  FILENAME:	WTPDEF.h
**
**  PURPOSE: 	Definition file for the WTPTP software tools..
**  
******************************************************************************/

#ifndef WTPDEF // Only make declarations once.
#define WTPDEF

//#define USE_LOG_FILE 0

//#if USE_LOG_FILE
//#define WTP_PRINT(log, string, ...) fprintf(log, string, __VA_ARGS__)
//#else
//#define WTP_PRINT(log, string, ...) printf(string, __VA_ARGS__)
//#endif


#define DATALENGTH 4090
#define RETURNDATALENGTH 4089
#define MESSAGELENGTH 256
#define WTPSTATUSLENGTH 3996

#ifndef _WINDEF_
typedef unsigned char BYTE;
#endif

#ifndef __cplusplus
	BOOL ISMSGPORT = FALSE; // Use this to enable port handshaking errors for debugging.
#endif

#define USB 0
#define UART 1
#define UNKNOWNPORT -1

#define ACK 0
#define NACK 1
#define MESSAGEPENDING 1
#define MESSAGESTRING 0
#define MESSAGEERRORCODE 1

// Image ID definitions
#define TIMH 0x54494D48
#define OBMI 0x4F424D49
// identifier for common tools Marvell Format image file
#define FBFI       0x46424649      // "FBFI"

// Command line parameter definitions
#define TIM 't'
#define OBM 'f'
#define KEY 'k'
#define FBF 'F'
#define COMPORT 'C'
#define BAUDRATE 'B'
#define UARTDELAY 'S'
#define OTP 'O'
#define DBGBOOT 'D'
#define VERBOSE 'V'
#define PORTTYPE 'P'
#define INTERFACENAME 'i'
#define HELP 'h'
#define MSGMODE 'M'
#define LOGFILE 'L'

typedef enum {
   DISABLED = 1,		// Standard WTPTP download
   MSG_ONLY = 2,		//Turn on message mode(listen for msgs forever)
   MSG_AND_DOWNLOAD = 3 //Msg mode and WTPTP download
   } MESSAGE_MODES;
typedef enum {
   IMAGE_DOWNLOAD_ERROR = 1,		// Download of binary failed
   } WTPTP_ERRORS;

// Protocol command definitions
#define PREAMBLE 0x00
#define PUBLICKEY 0x24
#define PASSWORD 0x28
#define SIGNEDPASSWORD 0x25
#define GETVERSION 0x20
#define SELECTIMAGE 0x26
#define VERIFYIMAGE 0x27
#define DATAHEADER 0x2A
#define DATA 0x22
#define MESSAGE 0x2B
#define OTPVIEW 0x2C
#define DEBUGBOOT 0x2D
#define DONE 0x30
#define DISCONNECT 0x31
#define UNKNOWNCOMMAND 0xFF

#pragma pack(push,1)
typedef struct
{
	BYTE CMD;
	BYTE SEQ;
	BYTE CID;
	BYTE Flags;
	long LEN;
	BYTE Data[DATALENGTH];
} WTPCOMMAND;
#pragma pack(pop)

#pragma pack(push,1)
typedef struct
{
	BYTE CMD;
	BYTE SEQ;
	BYTE CID;
	BYTE Status;
	BYTE Flags;
	BYTE DLEN;
	BYTE Data[RETURNDATALENGTH];
} WTPSTATUS;
#pragma pack(pop)

#endif
