/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.   
**
**  FILENAME:	WTPCOMMCLASS.cpp
**
**  PURPOSE: 	Provides download capabilities either thru the USB port or the 
**              UART port. This class insulates the top level API from what type 
**              of port is being utilized for the download proceedure. In the 
**              future other transfer technologies could be easily added without 
**              effecting the top level API.
**                  
******************************************************************************/

#include "stdafx.h"
#include "WTPCOMMCLASS.h"
#include "..\WTPUARTLIB\WTPUARTLIB.h"
#include "..\WTPUSBLIB\WTPUSBLIB.h"
#include "..\Include\ErrorCodes.h"
FILE *hLog=NULL;
CWtpComm::CWtpComm ()
{
	PortType = UNKNOWNPORT;
	iPortNumber = 1;
	dwBaudRate = 115200;
	dwImageSectionDelay = 200;
	Timeout = 0;
	FlashType = 0x00;
	bVerbose = false;
	bMsg = false;
	MessageMode = DISABLED;
	bBaudRateSet = false;
	bInterfaceSpecified = false;
}

CWtpComm::~CWtpComm ()
{
}

void CWtpComm::SetPortType (int iSelect)
{
	switch (iSelect)
	{
		case USB:
			PortType = USB;
			break;
		case UART:
			PortType = UART;
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}
}

int CWtpComm::GetPortType ()
{
	return PortType;
}

void CWtpComm::SetUartPort (int iPort)
{
	iPortNumber = iPort;
}
void CWtpComm::SetInterfaceName (char* pInterfaceName)
{
	strcpy( InterfaceName, pInterfaceName );
	bInterfaceSpecified = true;
}

int CWtpComm::GetUartPort ()
{
	return iPortNumber;
}

void CWtpComm::SetBaudRate (DWORD dwRate)
{
	dwBaudRate = dwRate;
//	bBaudRateSet = true;
}

void CWtpComm::SetUsbPortTimeout (long int lTimeout)
{
	Timeout = lTimeout;
}

void CWtpComm::SetImageSectionDelay (DWORD dwSectionDelay)
{
	dwImageSectionDelay = dwSectionDelay;
}

void CWtpComm::SetFlashType (BYTE FlashCode)
{
	FlashType = FlashCode;
}

BYTE CWtpComm::GetFlashType ()
{
	return FlashType;
}

void CWtpComm::SetVerboseMode (bool bOnOff)
{
	bVerbose = bOnOff;
	EnableUartVerboseMode (false); // Set to true for UART debug info
}

void CWtpComm::SetLogFileMode (bool bLogOnOff)
{
	bLogFile = bLogOnOff;
	//EnableUartVerboseMode (false); // Set to true for UART debug info
}


void CWtpComm::SetMessageMode (MESSAGE_MODES MessageModeOnOff)
{
	MessageMode = MessageModeOnOff;
}

bool CWtpComm::OpenPort ()
{
	bool IsOk = true;
	UartSettings Uart;

	switch (PortType)
	{
		case USB:
			if (OpenUsbPort (Timeout, bInterfaceSpecified? InterfaceName : NULL) == false)
			{
				IsOk = false;
			}
			break;
		case UART:
		    // if a baud rate wasn't specified by user, use autobaud.
//		    if ( bBaudRateSet == false )
//			{
//			   dwBaudRate = 0; // set to zero to indicate autobaud.
//			}
			Uart.iPortNumber = iPortNumber;
			Uart.EventChar = '\001';
			Uart.dwBaudRate = dwBaudRate;
			if (OpenUartPort (Uart) == false)
			{
				IsOk = false;
			}
			break;
		default:
			MessageBox (NULL,"  Error: OpenPort failed! Must specify the port type.",
						"WTPCOMMCLASS",MB_ICONSTOP|MB_OK);
			IsOk = false;
			break;
	}
	return IsOk;
}

void CWtpComm::ClosePort ()
{
	switch (PortType)
	{
		case USB:
			CloseUsbPort ();
			break;
		case UART:
			CloseUartPort ();
			break;
		default:
			break;
	}
}

bool CWtpComm::SendPreamble ()
{
	bool IsOk = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {PREAMBLE,0xD3,0x02,0x2B,0x00D3022B};
	WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	DWORD BytesRead;

	switch (PortType)
	{
		case USB: // Only 4 byte preamble for USB
			if (WriteUsb (WtpCmd,4,&WtpStatus,4) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (&WtpStatus,4,&BytesRead) == false)
			{
				if (bLogFile)
				{
					fprintf (hLog,"Sent preamble. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent preamble. Host waiting for target to reply!\n");
				}				
				//WTP_PRINT (hLog,"Sent preamble. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (&WtpStatus,BytesRead,4);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get preamble response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get preamble response from the target!\n");
					}
					//WTP_PRINT (hLog,"  Error: Did not get preamble response from the target!\n");
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			IsOk = false;
			break;
	}

	if (WtpStatus.CMD != 0x00 && WtpStatus.SEQ != 0xD3 && WtpStatus.CID != 0x02)
	{
		IsOk = false;
		sprintf (Message,"  Error: SendPreamble returned a NACK!");
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		//WTP_PRINT (hLog,"\n%s\n",Message);
	}

	return IsOk;
}

bool CWtpComm::GetVersion (WTPSTATUS *pWtpStatus)
{
	bool IsOk = true, EnableMessages = true, bFoundChar = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {GETVERSION,iSequence,0x00,0x00,0x00000000};
	DWORD BytesRead, StatusLength;
	unsigned long int Seconds, DelayTime = 60;
	int i;
	unsigned int tmp = 0;

	//set to defaults
	pWtpStatus->Flags = 0;
	pWtpStatus->Status = 0;
	
	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,30) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				EnableMessages = false;
				if (bLogFile)
				{
					fprintf (hLog,"Sent GetVersion. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent GetVersion. Host waiting for target to reply!\n");
				}
				//WTP_PRINT (hLog,"Sent GetVersion. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					return IsOk;
				}
			}
			StatusLength = (DWORD)pWtpStatus->DLEN;

			// Grab the remaining bytes determined by StatusLength
			if (StatusLength > 0)
			{
				for (i = 0; i < (int)StatusLength; i++)
				{
					ReadUartCharBuffer ((char*)&pWtpStatus->Data[i],1,&BytesRead);
					if (BytesRead != 1)
					{
						bFoundChar = false;
						Seconds = GetTickCount () / 1000L;
						while ((GetTickCount () / 1000L) < (Seconds + DelayTime))
						{
							ReadUartCharBuffer ((char *)&pWtpStatus->Data[i],1,&BytesRead);
							if (BytesRead == 1) { bFoundChar = true; break; }
						}
					}
					if (bFoundChar == false)
					{
						
						if (bLogFile)
						{
							fprintf (hLog,"  Error: Lost UART syncronization during GetVersion Operation!\n");
						}
						else
						{
							printf ("  Error: Lost UART syncronization during GetVersion Operation!\n");
						}						
						//WTP_PRINT (hLog,"  Error: Lost UART syncronization during GetVersion Operation!\n");
						return false;
					}
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: GetVersion returned a NACK!");
		#ifdef _CONSOLE
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		//WTP_PRINT (hLog,"\n%s\n",Message);
		#else
		gThMsg (Message);
		#endif
	}
	else if (pWtpStatus->Status == ACK)
	{
		if (bLogFile)
		{
			fprintf(hLog,"\nVersion: %c%c%c%c",pWtpStatus->Data[3],pWtpStatus->Data[2],pWtpStatus->Data[1],pWtpStatus->Data[0]);
			fprintf(hLog,"\nDate: %8X",*(unsigned long *)&pWtpStatus->Data[4]);
			fprintf(hLog,"\nProcessor: %c%c%c%2X",pWtpStatus->Data[11],pWtpStatus->Data[10],pWtpStatus->Data[9],pWtpStatus->Data[8]);
			fprintf(hLog,"\n");
		}
		else
		{
			printf("\nVersion: %c%c%c%c",pWtpStatus->Data[3],pWtpStatus->Data[2],pWtpStatus->Data[1],pWtpStatus->Data[0]);
			printf("\nDate: %8X",*(unsigned long *)&pWtpStatus->Data[4]);
			printf("\nProcessor: %c%c%c%2X",pWtpStatus->Data[11],pWtpStatus->Data[10],pWtpStatus->Data[9],pWtpStatus->Data[8]);
			printf("\n");
		}
/*
		WTP_PRINT(hLog,"\nVersion: %c%c%c%c",pWtpStatus->Data[3],pWtpStatus->Data[2],pWtpStatus->Data[1],pWtpStatus->Data[0]);
		WTP_PRINT(hLog,"\nDate: %8X",*(unsigned long *)&pWtpStatus->Data[4]);
		WTP_PRINT(hLog,"\nProcessor: %c%c%c%2X",pWtpStatus->Data[11],pWtpStatus->Data[10],pWtpStatus->Data[9],pWtpStatus->Data[8]);
		WTP_PRINT(hLog,"\n");
*/
	}
	else
	{
		//Do nothing!
	}

	if ( (MessageMode != DISABLED ) && ((pWtpStatus->Flags & 0x1) == MESSAGEPENDING) )
	{
		GetTargetMessage (pWtpStatus);
		if ( pWtpStatus->Data[0] == PlatformBusy )
		{
			do{
					GetTargetMessage(pWtpStatus);
				}while(pWtpStatus->Data[0] != PlatformReady);		
		}
	}

	return IsOk;
}

bool CWtpComm::SelectImage (PDWORD pImageType)
{
	bool IsOk = true, EnableMessages = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {SELECTIMAGE,iSequence,0x00,0x00,0x00000000};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead;

	//set to defaults
	pWtpStatus->Flags = 0;
	pWtpStatus->Status = 0;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,10) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,10,&BytesRead) == false)
			{
				EnableMessages = false;
				if (bLogFile)
				{
					fprintf (hLog,"Sent SelectImage. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent SelectImage. Host waiting for target to reply!\n");
				}
				//WTP_PRINT (hLog,"Sent SelectImage. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,10);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}					
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (pWtpStatus != NULL) delete (pWtpStatus);
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	*pImageType = pWtpStatus->Data[0];
	*pImageType |= (pWtpStatus->Data[1] << 8);
	*pImageType |= (pWtpStatus->Data[2] << 16);
	*pImageType |= (pWtpStatus->Data[3] << 24);

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: SelectImage returned a NACK!");
		#ifdef _CONSOLE
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		//WTP_PRINT (hLog,"\n%s\n",Message);
		#else
		gThMsg (Message);
		#endif
	}

//	if ((pWtpStatus->Flags & 0x1) == MESSAGEPENDING) GetTargetMessage (pWtpStatus);

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::VerifyImage (BYTE AckOrNack)
{
	bool IsOk = true, EnableMessages = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {VERIFYIMAGE,iSequence,0x00,0x00,0x00000001,AckOrNack};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead;

	//set to defaults
	pWtpStatus->Flags = 0;
	pWtpStatus->Status = 0;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,9,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,9) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				EnableMessages = false;
				if (bLogFile)
				{
					fprintf (hLog,"Sent VerifyImage. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent VerifyImage. Host waiting for target to reply!\n");
				}
				//WTP_PRINT (hLog,"Sent VerifyImage. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (pWtpStatus != NULL) delete (pWtpStatus);
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (AckOrNack == NACK)
	{
		IsOk = false;
		//sprintf (Message,"  Error: Image asked for by target does not exist!");
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Image asked for by target does not exist!");
		}
		else
		{
			printf ("\n  Error: Image asked for by target does not exist!");
		}
		//WTP_PRINT (hLog,"\n  Error: Image asked for by target does not exist!");
	}
	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: VerifyImage returned a NACK!");
		#ifdef _CONSOLE
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		//WTP_PRINT (hLog,"\n%s\n",Message);
		#else
		gThMsg (Message);
		#endif
	}

//	if ((pWtpStatus->Flags & 0x1) == MESSAGEPENDING) GetTargetMessage (pWtpStatus);

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::DataHeader (unsigned int uiRemainingData,WTPSTATUS *pWtpStatus)
{
	bool IsOk = true;
	WTPCOMMAND WtpCmd = {DATAHEADER,++iSequence,0x00,0x00,0x00000004};
	DWORD BytesRead;

	WtpCmd.Data[0] = uiRemainingData & 0x000000FF;
	WtpCmd.Data[1] = (uiRemainingData & 0x0000FF00) >> 8;
	WtpCmd.Data[2] = (uiRemainingData & 0x00FF0000) >> 16;
	WtpCmd.Data[3] = (uiRemainingData & 0xFF000000) >> 24;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,12,pWtpStatus,10) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,12) == false) IsOk = false;
			if (ReadUart (pWtpStatus,10,&BytesRead) == false)
			{
				if (bLogFile)
				{
					fprintf (hLog,"Sent DataHeader. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent DataHeader. Host waiting for target to reply!\n");
				}				
				//WTP_PRINT (hLog,"Sent DataHeader. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,10);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}						
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: DataHeader returned a NACK!");
		#ifdef _CONSOLE
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}	
		//WTP_PRINT (hLog,"\n%s\n",Message);
		#else
		gThMsg (Message);
		#endif
	}

	return IsOk;
}

bool CWtpComm::Data (PBYTE pData,long Length)
{
	bool IsOk = true;
	WTPCOMMAND WtpCmd = {DATA,++iSequence,0x00,0x00,Length};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead, StatusLength;
	unsigned long int Seconds, DelayTime = 60;
	int i;
	unsigned int tmp = 0;
	bool bFoundChar = true;

	for (int i = 0; i < (int)Length; i++)
	{
		WtpCmd.Data[i] = pData[i];
	}

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8 + (DWORD)Length,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8 + (DWORD)Length) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				if (bLogFile)
				{
					fprintf (hLog,"Sent Data. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Data. Host waiting for target to reply!\n");
				}				
				//WTP_PRINT (hLog,"Sent Data. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (pWtpStatus != NULL) delete (pWtpStatus);
					return IsOk;
				}
			}
			StatusLength = (DWORD)pWtpStatus->DLEN;

			// Grab the remaining bytes determined by StatusLength
			if (StatusLength > 0)
			{
				for (i = 0; i < (int)StatusLength; i++)
				{
					ReadUartCharBuffer ((char*)&pWtpStatus->Data[i],1,&BytesRead);
					if (BytesRead != 1)
					{
						bFoundChar = false;
						Seconds = GetTickCount () / 1000L;
						while ((GetTickCount () / 1000L) < (Seconds + DelayTime))
						{
							ReadUartCharBuffer ((char *)&pWtpStatus->Data[i],1,&BytesRead);
							if (BytesRead == 1) { bFoundChar = true; break; }
						}
					}
					if (bFoundChar == false)
					{
						if (bLogFile)
						{
							fprintf (hLog,"  Error: Lost UART syncronization during Data Operation!\n");
						}
						else
						{
							printf ("  Error: Lost UART syncronization during Data Operation!\n");
						}						
						//WTP_PRINT (hLog,"  Error: Lost UART syncronization during Data Operation!\n");
						return false;
					}
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: Data returned a NACK!");
		#ifdef _CONSOLE
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		//WTP_PRINT (hLog,"\n%s\n",Message);
		#else
		gThMsg (Message);
		#endif
	}

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::Done ()
{
	bool IsOk = true, EnableMessages = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {DONE,iSequence,0x00,0x00,0x00000000};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				EnableMessages = false;
				if (bLogFile)
				{
					fprintf (hLog,"Sent Done. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Done. Host waiting for target to reply!\n");
				}
				//WTP_PRINT (hLog,"Sent Done. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}					
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (pWtpStatus != NULL) delete (pWtpStatus);
					return IsOk;
				}
			}
//			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: Done returned a NACK!");
		#ifdef _CONSOLE
		//WTP_PRINT (hLog,"\n%s\n",Message);
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		#else
		gThMsg (Message);
		#endif
	}

	if ( (MessageMode != DISABLED ) && ((pWtpStatus->Flags & 0x1) == MESSAGEPENDING) )
	{
		GetTargetMessage (pWtpStatus);
	}

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::Disconnect (WTPSTATUS* pWtpStatus)
{
	bool IsOk = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {DISCONNECT,iSequence,0x00,0x00,0x00000000};
	DWORD BytesRead;

	//attempt to disconnect: first try
	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				if (bLogFile)
				{
					fprintf (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Disconnect. Host waiting for target to reply!\n");
				}				
				//WTP_PRINT (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}					
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	//if there is an NACK, target most likely has messages to send
	//attempt to read messages
	if (pWtpStatus->Status == NACK)
	{
		if(MessageMode != DISABLED){
			pWtpStatus->Status = ACK;
			GetTargetMessage (pWtpStatus);
		}
		//now try disconnect once more 
		switch (PortType)
		{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				//WTP_PRINT (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Disconnect. Host waiting for target to reply!\n");
				}
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					//WTP_PRINT (hLog,"Did not get a response from the target!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Did not get a response from the target!\n");
				}
				else
				{
					printf ("Did not get a response from the target!\n");
				}
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
		}
	}
	if(IsOk == false)
		pWtpStatus->Status = NACK;
	return IsOk;
}

bool CWtpComm::PublicKey ()
{
	bool IsOk = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {PUBLICKEY,iSequence,0x00,0x00,0x00000000};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead, StatusLength;
	unsigned long int Seconds, DelayTime = 60;
	int i;
	unsigned int tmp = 0;
	bool bFoundChar = true;

//			Sleep (16000);
	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,10) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd, 8) == false) IsOk = false;
			if (ReadUart (pWtpStatus, 6, &BytesRead) == false)
			{
				//WTP_PRINT (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Disconnect. Host waiting for target to reply!\n");
				}
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					return IsOk;
				}
			}
			StatusLength = (DWORD)pWtpStatus->DLEN;
			// Grab the remaining bytes determined by StatusLength
			if (StatusLength > 0)
			{
				for (i = 0; i < (int)StatusLength; i++)
				{
					ReadUartCharBuffer ((char*)&pWtpStatus->Data[i],1,&BytesRead);
					if (BytesRead != 1)
					{
						bFoundChar = false;
						Seconds = GetTickCount () / 1000L;
						while ((GetTickCount () / 1000L) < (Seconds + DelayTime))
						{
							ReadUartCharBuffer ((char *)&pWtpStatus->Data[i],1,&BytesRead);
							if (BytesRead == 1) { bFoundChar = true; break; }
						}
					}
					if (bFoundChar == false)
					{
						//WTP_PRINT (hLog,"  Error: Lost UART syncronization during PublicKey Operation!\n");
						if (bLogFile)
						{
							fprintf (hLog,"  Error: Lost UART syncronization during PublicKey Operation!\n");
						}
						else
						{
							printf ("  Error: Lost UART syncronization during PublicKey Operation!\n");
						}
						return false;
					}
				}
			}

			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: PublicKey returned a NACK!");
		#ifdef _CONSOLE
		//WTP_PRINT (hLog,"\n%s\n",Message);
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		#else
		gThMsg (Message);
		#endif
	}

//	GetTargetMessage (pWtpStatus);

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::Password (WTPSTATUS *pWtpStatus)
{
	bool IsOk = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {PASSWORD,iSequence,0x00,0x00,0x00000000};
	DWORD BytesRead;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,14) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,14,&BytesRead) == false)
			{
				//WTP_PRINT (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Disconnect. Host waiting for target to reply!\n");
				}
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,13);
				if (IsOk == false)
				{
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: Password returned a NACK!");
		#ifdef _CONSOLE
		//WTP_PRINT (hLog,"\n%s\n",Message);
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		#else
		gThMsg (Message);
		#endif
	}

//	GetTargetMessage (pWtpStatus);

	return IsOk;
}

bool CWtpComm::SignedPassword ()
{
	bool IsOk = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {SIGNEDPASSWORD,iSequence,0x00,0x00,0x00000000};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				//WTP_PRINT (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Sent Disconnect. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent Disconnect. Host waiting for target to reply!\n");
				}
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: SignedPassword returned a NACK!");
		#ifdef _CONSOLE
		//WTP_PRINT (hLog,"\n%s\n",Message);
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		#else
		gThMsg (Message);
		#endif
	}

//	GetTargetMessage (pWtpStatus);

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::OtpView ()
{
	bool IsOk = true, EnableMessages = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {OTPVIEW,iSequence,0x00,0x00,0x00000001,FlashType};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead;
	int i;

	// Use this method to view OTP register values. It will be necessary to
	// specify the flash type when using this feature.

	//WTP_PRINT (hLog,"Flash Type Selected: %d\n",FlashType);
	if (bLogFile)
	{
		fprintf (hLog,"Flash Type Selected: %d\n",FlashType);
	}
	else
	{
		printf ("Flash Type Selected: %d\n",FlashType);
	}

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,9,pWtpStatus,46) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,9) == false) IsOk = false;
			if (ReadUart (pWtpStatus,46,&BytesRead) == false) IsOk = false;
			{
				EnableMessages = false;
				//WTP_PRINT (hLog,"Sent OtpView. Host waiting for target to reply!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Sent OtpView. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent OtpView. Host waiting for target to reply!\n");
				}
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,46);
				if (IsOk == false)
				{
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					if (pWtpStatus != NULL) delete (pWtpStatus);
					return IsOk;
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = true;//don't return an error
		sprintf (Message,"  Error: OtpView returned a NACK!");
		#ifdef _CONSOLE
		//WTP_PRINT (hLog,"\n%s\n",Message);
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		#else
		gThMsg (Message);
		#endif
	}
	else
	{
		if ((pWtpStatus->Flags & 0x40) == 0x40)
		{
			if (bLogFile)
			{
				fprintf (hLog,"OTP Register 0 is Locked\n");
			}
			else
			{
				printf ("OTP Register 0 is Locked\n");
			} 
		//WTP_PRINT (hLog,"OTP Register 0 is Locked\n");
		}
		else 
		{
			if (bLogFile)
			{
				fprintf (hLog,"\nOTP Register 0 is Unlocked\n");
			}
			else
			{
				printf ("\nOTP Register 0 is Unlocked\n");
			}
		//WTP_PRINT (hLog,"\nOTP Register 0 is Unlocked\n");
		}

		if ((pWtpStatus->Flags & 0x80) == 0x80) 
		{
			if (bLogFile)
			{
				fprintf (hLog,"OTP Register 1 is Locked\n");
			}
			else
			{
				printf ("OTP Register 1 is Locked\n");
			}

		//WTP_PRINT (hLog,"OTP Register 1 is Locked\n");
		}
		else
		{
			if (bLogFile)
			{
				fprintf (hLog,"OTP Register 1 is Unlocked\n");
			}
			else
			{
				printf ("OTP Register 1 is Unlocked\n");
			}
		 //WTP_PRINT (hLog,"OTP Register 1 is Unlocked\n");
		}

		//WTP_PRINT (hLog,"OTP Register 0:");
		if (bLogFile)
		{
			fprintf (hLog,"OTP Register 0:");
		}
		else
		{
			printf ("OTP Register 0:");
		}

		for (i = 0; i < 8; i++) 
		{
			if (bLogFile)
			{
				fprintf (hLog,"%X",pWtpStatus->Data[i]); 
				fprintf (hLog,"\n");
			}
			else
			{
				printf ("%X",pWtpStatus->Data[i]); 
				printf ("\n");
			}
		//WTP_PRINT (hLog,"%X",pWtpStatus->Data[i]); 
		//WTP_PRINT (hLog,"\n");
		}

		printf ("OTP Register 1:");
		for (i = 8; i < 24; i++) //WTP_PRINT (hLog,"%X",pWtpStatus->Data[i]); WTP_PRINT (hLog,"\n");
		{
			if (bLogFile)
			{
				fprintf (hLog,"%X",pWtpStatus->Data[i]); 
				fprintf (hLog,"\n");
			}
			else
			{
				printf ("%X",pWtpStatus->Data[i]); 
				printf ("\n");
			}
		//WTP_PRINT (hLog,"%X",pWtpStatus->Data[i]); 
		//WTP_PRINT (hLog,"\n");
		}

		printf ("OTP Register 2:");
		for (i = 24; i < 40; i++) //WTP_PRINT (hLog,"%X",pWtpStatus->Data[i]); WTP_PRINT (hLog,"\n");
		{
			if (bLogFile)
			{
				fprintf (hLog,"%X",pWtpStatus->Data[i]); 
				fprintf (hLog,"\n");
			}
			else
			{
				printf ("%X",pWtpStatus->Data[i]); 
				printf ("\n");
			}
		//WTP_PRINT (hLog,"%X",pWtpStatus->Data[i]); 
		//WTP_PRINT (hLog,"\n");
		}
	}

	if ((pWtpStatus->Flags & 0x1) == MESSAGEPENDING) GetTargetMessage (pWtpStatus);

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::GetMessage (WTPSTATUS *pWtpStatus)
{
	bool IsOk = true, bFoundChar = true;
	iSequence = 0x00;
	unsigned long int Seconds, DelayTime = 60;
	WTPCOMMAND WtpCmd = {MESSAGE,iSequence,0x00,0x00,0x00000000};
	DWORD StatusLength, BytesRead;
	char WtpMessage[MESSAGELENGTH];
	int i;

	memset (WtpMessage,'\0',256);

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,8,pWtpStatus,6 + MESSAGELENGTH) == false) IsOk = false;
			if (!(pWtpStatus->DLEN > 0)) {pWtpStatus->Status = NACK; break;}
			if (pWtpStatus->Status != ACK) {pWtpStatus->Status = NACK; break;}

			if (((pWtpStatus->Flags & 0x02) >> 1) == MESSAGESTRING)
			{
				StatusLength = (DWORD)pWtpStatus->DLEN;
				//WTP_PRINT(hLog,"---\n");
				if (bLogFile)
				{
					fprintf (hLog,"---\n");
				}
				else
				{
					printf ("---\n");
				}

				#ifdef _CONSOLE
				for (i = 0; i < (int)StatusLength; i++)
				{
					//WTP_PRINT (hLog,"%c",pWtpStatus->Data[i]);
					if (bLogFile)
					{
						fprintf (hLog,"%c",pWtpStatus->Data[i]);
					}
					else
					{
						printf ("%c",pWtpStatus->Data[i]);
					}

				}
				//WTP_PRINT (hLog,"\n");
				if (bLogFile)
				{
					fprintf (hLog,"---\n");
				}
				else
				{
					printf ("---\n");
				}

				#else
				memcpy (WtpMessage,pWtpStatus->Data,StatusLength);
				gThMsg (WtpMessage);
				#endif
			}
			else
			{
				for (i = 0; i < MAXERRORCODES; i++)
				{
					if (ErrorCodes[i].ErrorCode == pWtpStatus->Data[0])
					{
						#ifdef _CONSOLE
						//WTP_PRINT (hLog,"\nCode: %s %X\n",ErrorCodes[i].Description,pWtpStatus->Data[0]);
						if (bLogFile)
						{
							fprintf (hLog,"\nCode: %s %X\n",ErrorCodes[i].Description,pWtpStatus->Data[0]);
						}
						else
						{
							printf ("\nCode: %s %X\n",ErrorCodes[i].Description,pWtpStatus->Data[0]);
						}

						#else
						sprintf (WtpMessage,"Error Code: %s",ErrorCodes[i].Description);
						gThMsg (WtpMessage);
						#endif
					}
				}
			}
			break;
		case UART:
			if (MessageMode != DISABLED)
			{
//				printf("waiting for messages...\n");
				Sleep(100);
			}
			if (WriteUart (WtpCmd,8) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false)
			{
				//WTP_PRINT (hLog,"Sent GetMessage. Host waiting for target to reply!\n");
				if (bLogFile)
				{
					fprintf (hLog,"Sent GetMessage. Host waiting for target to reply!\n");
				}
				else
				{
					printf ("Sent GetMessage. Host waiting for target to reply!\n");
				}
				IsOk = WaitForHostReply (pWtpStatus,BytesRead,6);
				if (IsOk == false)
				{
					//WTP_PRINT (hLog,"  Error: Did not get a response from the target!\n");
					if (bLogFile)
					{
						fprintf (hLog,"  Error: Did not get a response from the target!\n");
					}
					else
					{
						printf ("  Error: Did not get a response from the target!\n");
					}
					return IsOk;
				}
			}
			StatusLength = (DWORD)pWtpStatus->DLEN;

			// Host is too fast for UART, therefore read 1 character at a time.
			if (StatusLength > 0)
			{
				for (i = 0; i < (int)StatusLength; i++)
				{
					ReadUartCharBuffer (&WtpMessage[i],1,&BytesRead);
					if (BytesRead != 1)
					{
						bFoundChar = false;
						Seconds = GetTickCount () / 1000L;
						while ((GetTickCount () / 1000L) < (Seconds + DelayTime))
						{
							ReadUartCharBuffer (&WtpMessage[i],1,&BytesRead);
							if (BytesRead == 1) { bFoundChar = true; break; }
						}
					}
					if (bFoundChar == false)
					{
						//WTP_PRINT (hLog,"  Error: Lost UART syncronization during GetMessage Operation!\n");
						if (bLogFile)
						{
							fprintf (hLog,"  Error: Lost UART syncronization during GetMessage Operation!\n");
						}
						else
						{
							printf ("  Error: Lost UART syncronization during GetMessage Operation!\n");
						}
						return false;
					}
				}
				memcpy (pWtpStatus->Data,WtpMessage, StatusLength);
				if (((pWtpStatus->Flags & 0x02) >> 1) == MESSAGESTRING)
				{

					#ifdef _CONSOLE
					for (i = 0; i < (int)StatusLength; i++)
					{
						//WTP_PRINT (hLog,"%c",WtpMessage[i]);
						if (bLogFile)
						{
							fprintf (hLog,"%c",WtpMessage[i]);
						}
						else
						{
							printf ("%c",WtpMessage[i]);
						}
					}
					//WTP_PRINT (hLog,"\n");
					if (bLogFile)
					{
						fprintf (hLog,"\n");
					}
					else
					{
						printf ("\n");
					}
					#else
					gThMsg (WtpMessage);
					#endif
				}
				else
				{
					for (i = 0; i < MAXERRORCODES; i++)
					{
						if (ErrorCodes[i].ErrorCode == WtpMessage[0])
						{
							#ifdef _CONSOLE
							//WTP_PRINT (hLog,"\nCode: %s %X\n",ErrorCodes[i].Description,WtpMessage[0]);
							if (bLogFile)
							{
								fprintf (hLog,"\nCode: %s %X\n",ErrorCodes[i].Description,WtpMessage[0]);
							}
							else
							{
								printf ("\nCode: %s %X\n",ErrorCodes[i].Description,WtpMessage[0]);
							}
							#else
							sprintf (WtpMessage,"Code: %s",ErrorCodes[i].Description);
							gThMsg (WtpMessage);
							#endif
						}
					}
				}
			}
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
			break;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		//sprintf (Message,"GetMessage returned a NACK!");
		#ifndef _CONSOLE
		gThMsg (Message);
		#endif
	}

	return IsOk;
}

bool CWtpComm::GetTargetMessage (WTPSTATUS *pWtpStatus)
{
	bool IsOk = true;
	unsigned long int Seconds, DelayTime = 120;

	Seconds = GetTickCount () / 1000L;

	while ( (pWtpStatus->Status == ACK) && 
			((GetTickCount () / 1000L) < (Seconds + DelayTime)) )
	{
		if (bMsg) 
		{
		//WTP_PRINT (hLog,"\nMessage Pending Flag status: %X\n",pWtpStatus->Flags);
			if (bLogFile)
			{
				fprintf (hLog,"\nMessage Pending Flag status: %X\n",pWtpStatus->Flags);
			}
			else
			{
				printf ("\nMessage Pending Flag status: %X\n",pWtpStatus->Flags);
			}
		}
		if (GetMessage (pWtpStatus) == false) {IsOk = false; break;}
		// This is to add a delay for WTPTP while the flash is being erased by DKB.
		if (pWtpStatus->Data[0] == PlatformBusy )
		{	
			do{
				GetMessage(pWtpStatus);
			}while(pWtpStatus->Data[0] != PlatformReady);
		}
		if (((pWtpStatus->Flags & 0x1) != MESSAGEPENDING)){	break; }
		Sleep (dwImageSectionDelay);
		if (bMsg)
		{ 
		//WTP_PRINT (hLog,"\nNext Message Pending Flag status: %X\n",pWtpStatus->Flags);
		if (bLogFile)
		{
			fprintf (hLog,"\nNext Message Pending Flag status: %X\n",pWtpStatus->Flags);
		}
		else
		{
			printf ("\nNext Message Pending Flag status: %X\n",pWtpStatus->Flags);
		}

		}
	}

	return IsOk;
}

bool CWtpComm::DebugBoot (PBYTE pFlashType)
{
	bool IsOk = true;
	iSequence = 0x00;
	WTPCOMMAND WtpCmd = {DEBUGBOOT,iSequence,0x00,0x00,0x00000001,FlashType};
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	DWORD BytesRead;

	WtpCmd.Data[0] = *pFlashType;

	switch (PortType)
	{
		case USB:
			if (WriteUsb (WtpCmd,9,pWtpStatus,6) == false) IsOk = false;
			break;
		case UART:
			if (WriteUart (WtpCmd,9) == false) IsOk = false;
			if (ReadUart (pWtpStatus,6,&BytesRead) == false) IsOk = false;
			Sleep (dwImageSectionDelay);
			break;
		default:
			PortType = UNKNOWNPORT;
	}

	if (pWtpStatus->Status == NACK)
	{
		IsOk = false;
		sprintf (Message,"  Error: BootState returned a NACK!");
		#ifdef _CONSOLE
		//WTP_PRINT (hLog,"\n%s\n",Message);
		if (bLogFile)
		{
			fprintf (hLog,"\n%s\n",Message);
		}
		else
		{
			printf ("\n%s\n",Message);
		}
		#else
		gThMsg (Message);
		#endif
	}

	delete (pWtpStatus);
	return IsOk;
}

bool CWtpComm::WaitForHostReply (WTPSTATUS *pWtpStatus,int nCharsRead,int nTotalChars)
{
	bool IsOk = true;
	int nCharacters = nTotalChars - nCharsRead;
	unsigned long int Seconds, DelayTime = 360, ReadSeconds, ReadInterval = 10;
	unsigned long int MessageInterval = 150, MsgSeconds;
//	unsigned long int Seconds, DelayTime = 180, ReadSeconds, ReadInterval = 10;
//	unsigned long int MessageInterval = 10, MsgSeconds;
	DWORD BytesRead = 0;
	char Data[RETURNDATALENGTH + 6];

	// This routine is only called if the UART did not read all of the expected
	// characters back from the target device. This will simply give the UART
	// a chance to receive the remaining characters. The DelayTime variable can
	// be changed to set the maximum time allowable for receiving what is left.

	memcpy (Data,pWtpStatus,nCharsRead);

	Seconds = GetTickCount () / 1000L;
	MsgSeconds = GetTickCount () / 1000L;
	ReadSeconds = GetTickCount () / 1000L;

	EnableWaitCommEvent (FALSE);
//	EnableUartVerboseMode (true); // Set to true for UART debug info
	EnableUartVerboseMode (false); // Set to true for UART debug info

	while ((GetTickCount () / 1000L) < (Seconds + DelayTime))
	{
		Sleep (1000);
		if ((GetTickCount () / 1000L) > (MsgSeconds + MessageInterval))
		{
			//WTP_PRINT (hLog,"Host waiting for target to reply!\n");
			if (bLogFile)
			{
				fprintf (hLog,"Host waiting for target to reply!\n");
			}
			else
			{
				printf ("Host waiting for target to reply!\n");
			}
			if (bVerbose) 
			{
				//WTP_PRINT (hLog,"Number of chars read so far: %d\n",nCharsRead);
				if (bLogFile)
				{
					fprintf (hLog,"Number of chars read so far: %d\n",nCharsRead);
				}
				else
				{
					printf ("Number of chars read so far: %d\n",nCharsRead);
				}
			}
			MsgSeconds = GetTickCount () / 1000L;
		}
		if ((GetTickCount () / 1000L) > (ReadSeconds + ReadInterval))
		{
			ReadUartCharBuffer (&Data[nCharsRead],nCharacters,&BytesRead);
			ReadSeconds = GetTickCount () / 1000L;
		}
		nCharacters -= BytesRead; nCharsRead += BytesRead;
		if (nCharacters == 0) { IsOk = true; break; } // We got all the characters.
		if (nCharacters < 0) { IsOk = false; break; } // Shouldn't get here.
	}

	memcpy (pWtpStatus,Data,nTotalChars);

	EnableUartVerboseMode (false);
	EnableWaitCommEvent (TRUE);
	return IsOk;
}


