/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.  
**
**  FILENAME:	TimParser.c
**
**  PURPOSE: 	Windows Dll routines 
**                  
******************************************************************************/

#pragma warning ( disable : 4244 4715 4047 4024) // Disable warning messages

#include <time.h>
#include <stdio.h>
#include <windows.h>
#include <math.h>
#include "WTPSECURITYLIB.h"
#if TRUSTED
	#include "ippCP.h"
#endif
#include "tim.h"
#include "../Include/WTPDEF.h"
#include "../../WtptpConsole/WtpDownload.h"


BOOL SetImageId (IMAGEFILES *pImageFiles,int nFiles,UINT_T *pImageId,int nImages);

BOOL ParseTim (IMAGEFILES *pImageFiles,int nFiles,int *pNumberOfImages)
{
	int i, j;
	UINT_T nImages, *ImageId;
	BOOL IsTim = FALSE;
	DWORD FileType, TimId;
	FILE *hTimFile;
	CTIM TimHeader;
	IMAGE_INFO ImageInfo;

	for (i = 0; i < nFiles; i++)
	{
		FileType = pImageFiles->FileType;
		if (bVerbose) printf ("\nParseTim opened TIM file: %s, %X\n",pImageFiles->FileName,FileType);

		if (pImageFiles->FileType == TIMH)
		{
			IsTim = TRUE; break;
		}
		pImageFiles++;
	}
	pImageFiles -= i;

	if (!IsTim)
	{
		printf ("\nA Tim image file was not supplied! Exiting program.\n");
		return FALSE;
	}

	if ((hTimFile = fopen (pImageFiles->FileName,"rb")) == NULL)
	{
		printf ("\nfopen failed to open Tim file: %s.\n",pImageFiles->FileName);
		return FALSE;
	}

	fseek (hTimFile,sizeof(DWORD),SEEK_SET); // Set position to Version.Identifier
	fread (&TimId,sizeof(DWORD),1,hTimFile);

	if (TimId != TIMH)
	{
		printf ("\nAn invalid Tim file was supplied\n");
		printf ("Correct ID: %X, ID of supplied image: %X\n",FileType,TimId);
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	fseek (hTimFile,0L,SEEK_SET); // Set position to SOF
	fread (&TimHeader,sizeof(CTIM),1,hTimFile);
	nImages = TimHeader.NumImages;

	printf ("Number of Images listed in TIM: %d\n",nImages);
	*pNumberOfImages = nImages;

	if (NULL == (ImageId = (UINT_T)malloc (sizeof (UINT_T) * nImages)))
	{
		if (hTimFile != NULL) fclose (hTimFile);
		return FALSE;
	}

	for (i = 0; i < (int)nImages; i++)
	{
		fread (&ImageInfo,sizeof(IMAGE_INFO),1,hTimFile);
		*(ImageId + i) = ImageInfo.ImageID;
	}

	printf ("\nTIM image ID list:\n");
	for (i = 0; i < (int)nImages; i++)
	{
		printf ("Image ID: ");
		for (j = 3; j >= 0; j--)
		{
			printf ("%c",(ImageId[i] >> (j * 8)));
		}
		printf ("\n");
		//printf ("Image ID: %X\n",ImageId[i]);
	}
	if (hTimFile != NULL) fclose (hTimFile); // Close TIM

	if (SetImageId (pImageFiles,nFiles,ImageId,(int)nImages) == FALSE) return FALSE;

	if (ImageId != NULL) free (ImageId);

	return TRUE;
}

BOOL SetImageId (IMAGEFILES *pImageFiles,int nFiles,UINT_T *pImageId,int nImages)
{
	int i, iImageIndex;
	FILE *hFile;
	DWORD FileId;
	BOOL IsImage = FALSE;
	char Message[MAX_PATH];
	BOOL SECURITYDEBUG = FALSE;

	for (iImageIndex = 0; iImageIndex < nImages; iImageIndex++)
	{
		if (pImageId[iImageIndex] == TIMH) continue;
		if (SECURITYDEBUG) printf ("\nID listed in TIM %X\n",pImageId[iImageIndex]);

		for (i = 0; i < nFiles; i++,pImageFiles++)
		{
			if (pImageFiles->FileType == TIMH) continue;
			if (SECURITYDEBUG) printf ("Load Image file %s\n",pImageFiles->FileName);

			if ((hFile = fopen (pImageFiles->FileName,"rb")) == NULL)
			{
				sprintf (Message,"\nfopen failed to open %s while obtaining ID.\n",pImageFiles->FileName);
				#ifdef _CONSOLE
					printf ("%s",Message);
				#else
					MessageBox (NULL,Message,"TIM Parser",MB_ICONSTOP|MB_OK);
				#endif
				
				continue;
			}

			fseek (hFile,0L,SEEK_SET);
			fread (&FileId,sizeof(DWORD),1,hFile);

			if (SECURITYDEBUG) printf ("ID read from file %X\n",FileId);
			if (hFile != NULL) fclose (hFile);

			if (FileId == pImageId[iImageIndex])
			{
				pImageFiles->FileType = pImageId[iImageIndex];
				IsImage = TRUE; break;
			}
		}
		pImageFiles -= i;
		if (!IsImage)
		{
			sprintf (Message,"An image file with ID# %X was not supplied by user!",pImageId[iImageIndex]);
			#ifdef _CONSOLE
				printf ("%s\n",Message);
			#else
				MessageBox (NULL,Message,"TIM Parser",MB_ICONSTOP|MB_OK);
			#endif
			return FALSE;
		}
		IsImage = FALSE;
	}

	return TRUE;
}