/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.   
**
**  FILENAME:	WTPUSBLIB.c
**
**  PURPOSE: 	Windows Dll routines 
**                  
******************************************************************************/

#pragma warning ( disable : 4244 4715 4047 4024) // Disable warning messages

#include "STDAFX.H"
#include "WTPUSBLIB.h"
#include "..\Include\WTPIOCTL.h"


HANDLE hUsb = INVALID_HANDLE_VALUE;
char DeviceName[_MAX_PATH];
BOOL USBDEBUG = FALSE;
BOOL POPUP_WINDOW = FALSE;

WTPUSBLIB_API BOOL OpenUsbPort (long int DelayTime, PBYTE pInterfaceName)
{
	char Message[100];
	unsigned long int Seconds;
	BOOL DeviceFound = FALSE,MessageEnable = TRUE;
	DWORD dwAccess = GENERIC_READ | GENERIC_WRITE;
	DWORD dwShareMode = FILE_SHARE_READ | FILE_SHARE_WRITE;
	DWORD dwFlags = FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED;

	if (DelayTime > 0) MessageEnable = FALSE;
	Seconds = GetTickCount () / 1000L;

	if( pInterfaceName != NULL )
	{
		DeviceFound = TRUE;
		strcpy( DeviceName, pInterfaceName );
	}
	else
	{
		do
		{
			if ((DeviceFound = GetUsbDeviceName (DeviceName,MessageEnable))) break;
			Sleep (5);
		} while ((GetTickCount () / 1000L) < (Seconds + DelayTime));
	}
	

	if (DeviceFound == FALSE) return FALSE;
	if(POPUP_WINDOW)
	{
		MessageBox( NULL, DeviceName, "Selected Interface:", MB_OK);
	}
	hUsb = CreateFile (DeviceName,dwAccess,dwShareMode,NULL,OPEN_EXISTING,dwFlags,NULL);

	if (hUsb == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: CreateFile failed with error %d",GetLastError());
		MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	return TRUE;
}

WTPUSBLIB_API BOOL GetUsbDeviceName (char *DeviceName,BOOL IsPrint)
{
	char Message[100];
	HDEVINFO hDevInfo;
	SP_DEVICE_INTERFACE_DATA SpDeviceInterfaceData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA pSpDeviceInterfaceDetailData = NULL;
	DWORD dwSize = 0, dwFlags = DIGCF_PRESENT | DIGCF_INTERFACEDEVICE;
	BOOL IsOk;

	hDevInfo = SetupDiGetClassDevs ((LPGUID)&GUID_BOOT,NULL,NULL,dwFlags);

	if (hDevInfo == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: SetupDiGetClassDevs failed with error %d.",GetLastError());
		if (IsPrint) MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	SpDeviceInterfaceData.cbSize = sizeof (SpDeviceInterfaceData);

	if (SetupDiEnumDeviceInterfaces (hDevInfo,NULL,(LPGUID)&GUID_BOOT,0,&SpDeviceInterfaceData) == NULL)
	{
		if (hDevInfo) SetupDiDestroyDeviceInfoList (hDevInfo);
		sprintf (Message,"  Error: SetupDiEnumDeviceInterfaces failed with error %d.",GetLastError());
		if (IsPrint) MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	// Get PSP_DEVICE_INTERFACE_DETAIL_DATA struct size to allocate memory for it.
	SetupDiGetDeviceInterfaceDetail (hDevInfo,&SpDeviceInterfaceData,NULL,0,&dwSize,NULL);
	pSpDeviceInterfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc (dwSize);

	sprintf (Message,"%d",dwSize);
	OutputDebugString (Message);

	if (pSpDeviceInterfaceDetailData == NULL)
	{
		if (hDevInfo) SetupDiDestroyDeviceInfoList (hDevInfo);
		sprintf (Message,"  Error: Failed to allocate memory for PSP_DEVICE_INTERFACE_DETAIL_DATA.");
		if (IsPrint) MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	pSpDeviceInterfaceDetailData->cbSize = sizeof (SP_DEVICE_INTERFACE_DETAIL_DATA);

	IsOk = SetupDiGetDeviceInterfaceDetail (hDevInfo,&SpDeviceInterfaceData,
											pSpDeviceInterfaceDetailData,
											dwSize,NULL,NULL);

	if (!IsOk)
	{
		if (pSpDeviceInterfaceDetailData) free (pSpDeviceInterfaceDetailData);
		if (hDevInfo) SetupDiDestroyDeviceInfoList (hDevInfo);
		sprintf (Message,"  Error: SetupDiGetDeviceInterfaceDetail failed with error %d.",GetLastError());
		if (IsPrint) MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	strcpy (DeviceName,pSpDeviceInterfaceDetailData->DevicePath);

	if (USBDEBUG)
	{
		sprintf (Message,"%s",pSpDeviceInterfaceDetailData->DevicePath);
		MessageBox (NULL,Message,"USB COMMUNICATIONS DEBUG",MB_ICONSTOP|MB_OK);
	}

	if (pSpDeviceInterfaceDetailData) free (pSpDeviceInterfaceDetailData);
	if (hDevInfo) SetupDiDestroyDeviceInfoList (hDevInfo);

	return TRUE;
}

WTPUSBLIB_API BOOL WriteUsb (WTPCOMMAND WtpCmd,DWORD dwCmdSize,WTPSTATUS *WtpStatus,DWORD dwStatusSize)
{
	char Message[100];
	DWORD IOCTL_CMD, dwActualBytes = 0;
	BOOL IsOk;

	if (hUsb == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: OpenUsbPort must be called prior to calling WriteUsb!");
		MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	if ((IOCTL_CMD = GetIoControlCode (WtpCmd.CMD)) == UNKNOWNCOMMAND)
	{
		sprintf (Message,"  Error: An UNKNOWN command type was received! CMD = 0x%X",WtpCmd.CMD);
		MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	IsOk = DeviceIoControl (hUsb,IOCTL_CMD,&WtpCmd,dwCmdSize,WtpStatus,
							dwStatusSize,&dwActualBytes,NULL);
	if (!IsOk)
	{
		sprintf (Message,"  Error: WriteUsb failed with error %d",GetLastError());
		if (ISMSGPORT) MessageBox (NULL,Message,"USB COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
	}

	if (USBDEBUG)
	{
		sprintf (Message,"IOCTL code: %X",IOCTL_CMD);
		MessageBox (NULL,Message,"USB COMMUNICATIONS DEBUG",MB_ICONSTOP|MB_OK);
	}

	return TRUE;
}

DWORD GetIoControlCode (BYTE CMD)
{
	DWORD Code;

	switch (CMD)
	{
		case PREAMBLE:
			Code = IOCTL_BOOT_PREAMBLE;
			break;
		case PUBLICKEY:
			Code = IOCTL_BOOT_PUBLICKEYXMIT;
			break;
		case PASSWORD:
			Code = IOCTL_BOOT_SEEDPASSWORDRECEIVE;
			break;
		case SIGNEDPASSWORD:
			Code = IOCTL_BOOT_SIGNEDPASSWORDXMIT;
			break;
		case GETVERSION:
			Code = IOCTL_BOOT_GETVERSION;
			break;
		case SELECTIMAGE:
			Code = IOCTL_BOOT_SELECTIMAGE;
			break;
		case VERIFYIMAGE:
			Code = IOCTL_BOOT_VERIFYIMAGE;
			break;
		case DATAHEADER:
			Code = IOCTL_BOOT_DATAHEADER;
			break;
		case DATA:
			Code = IOCTL_BOOT_IMAGEDATAXMIT;
			break;
		case OTPVIEW:
			Code = IOCTL_BOOT_OTPVIEW;
			break;
		case DEBUGBOOT:
			Code = IOCTL_BOOT_DEBUGBOOT;
			break;
		case DONE:
			Code = IOCTL_BOOT_DONE;
			break;
		case DISCONNECT:
			Code = IOCTL_BOOT_DISCONNECT;
			break;
		case MESSAGE:
			Code = IOCTL_BOOT_MESSAGE;
			break;
		default:
			Code = UNKNOWNCOMMAND;
			break;
	}

	return Code;
}

WTPUSBLIB_API void CloseUsbPort ()
{
	if (hUsb != INVALID_HANDLE_VALUE) CloseHandle (hUsb);
}
