/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
*
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.  
**
**  FILENAME:	WtpDownload.cpp
**
**  PURPOSE: 	Provides download capabilities either thru the USB port or  
** 				the UART port. This is the command line version.
**                  
******************************************************************************/

#include "stdafx.h"
#include "..\Common\WTPCOMMCLASS\WTPCOMMCLASS.h"
#include "..\Common\WTPSECURITYLIB\WTPSECURITYLIB.h"
#include "..\Common\Include\FlashCodes.h"
#include "WtpDownload.h"
#include "..\Common\Include\downloader.h"
#pragma warning ( disable : 4800 ) // Disable warning messages

#define SLE_TESTING 0
#define MIN_ARGS 2
char Version[11] = "3.11";
char Date[11] = "04182008";

CWtpComm WtpComm;
int FileIndex = 0;
static MasterBlockHeader MBH;
static TCHAR szFBFFileName[MAX_PATH];
bool IsScriptFile (int argc,char* argv[],char *ScriptFileName,bool *IsScript);
bool GetImageFile (DWORD ImageType,char *TypeName,BYTE Option,int Argc,char *ImageFile);
bool ParseCommandLine (int argc,char* argv[]);
bool ParseScriptFile (char *ScriptFileName);
bool GetParameter (char *fLine,char *pParameter,const char *pDec,bool IsFile);
bool CheckYourOptions ();
BYTE AckWithImageName (DWORD ImageType,TCHAR *szFileName);
bool DownloadImage (int iFile);
bool DownloadMasterBlockHeader();
bool DownloadFBFImage();
#if TRUSTED
bool JtagReEnable ();
#endif
bool OtpView ();
bool DebugBoot ();
void PrintUsage ();
extern FILE *hLog;
static int fileCount=1;

int main (int argc,char *argv[])
{
    int currentOption = 1, i=0, nImages=0;
	bool bScript = FALSE;
	char ScriptFileName[MAX_PATH];
	bVerbose = FALSE;
	bDebug = FALSE;
	bLogFile = FALSE;
	bInterfaceSpecified = FALSE;
	IsTim = FALSE;
	IsOtpView = FALSE;
	IsDebugBoot = FALSE;
	IsJtag = FALSE;
    IsFbf = FALSE;
    IsImageFile = FALSE;
	WTPSTATUS *pWtpStatus = new WTPSTATUS;
	bool IsOk = true;
	currentMessageMode = DISABLED;
	WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00};
	bool Connected = FALSE;
	bool RetVal = TRUE;
	char LogFilename[250];
	sprintf(LogFilename, "%s%d.txt","WtptpLog", fileCount);
	fileCount++;
	if ((hLog = fopen (LogFilename,"a+")) == NULL)
	{
		printf ("\n  Error: fopen failed to open log file: WtptpLog.txt.\n");
		return FALSE;
	}
	
	//WTP_PRINT (hLog,"\n\nWTPTP Tool Version: %s\n", Version);
	//WTP_PRINT (hLog,"WTPTP Tool Date   : %s\n\n", Date);

	if (argc < MIN_ARGS)
	{
		PrintUsage ();
		return FALSE;
	}
/*
	if(exitProcess == 1)
	{
		ExitProcess(15);
		return FALSE;
	}
*/
	if (IsScriptFile (argc,argv,ScriptFileName,&bScript) == FALSE) return FALSE;

	if (bScript)
	{
		if (ParseScriptFile (ScriptFileName) == FALSE) return FALSE;
	}
	else
	{
		if (ParseCommandLine (argc,argv) == FALSE) return FALSE;
	}

	if (bLogFile)
	{
		fprintf (hLog,"\n\nWTPTP Tool Version: %s\n", Version);
		fprintf (hLog,"WTPTP Tool Date   : %s\n\n", Date);
	}
	else
	{
		printf ("\n\nWTPTP Tool Version: %s\n", Version);
		printf ("WTPTP Tool Date   : %s\n\n", Date);
	}

	if (bVerbose == false )
	{
		if (bLogFile)
		{
			fprintf (hLog,"Verbose mode disabled...\n");
		}
		else
		{
			printf ("Verbose mode disabled...\n");
		}
		//WTP_PRINT(hLog,"Verbose mode disabled...\n");
	}

	if (IsTim)
	{
		if (ParseTim (&ImageFiles[0],FileIndex,&nImages) == FALSE) return FALSE;
		FileIndex = nImages; // Just in case user supplies more files than required
	}

	if (!IsFbf && !IsOtpView && !IsJtag && (currentMessageMode == MSG_AND_DOWNLOAD))
		if (FileIndex == 0) 
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: Did not supply any files for download!\n");
			}
			else
			{
				printf ("  Error: Did not supply any files for download!\n");
			} 
			//WTP_PRINT (hLog,"  Error: Did not supply any files for download!\n");
			if (hLog != NULL ) fclose (hLog); 
			return FALSE; 
		}

	if (WtpComm.GetPortType() == UART)
	{ 
		if (bLogFile)
		{
			fprintf (hLog,"\nPort Selected: UART\n");
		}
		else
		{
			printf ("\nPort Selected: UART\n");
		}		
		//WTP_PRINT (hLog,"\nPort Selected: UART\n");
	}
	else if (WtpComm.GetPortType() == USB)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nPort Selected: USB\n");
		}
		else
		{
			printf ("\nPort Selected: USB\n");
		}
		//WTP_PRINT (hLog,"\nPort Selected: USB\n");
	}
	else 
	{
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Port Selected: UNDEFINED\n");
		}
		else
		{
			printf ("\n  Error: Port Selected: UNDEFINED\n");
		} 
		//WTP_PRINT (hLog,"\n  Error: Port Selected: UNDEFINED\n");
		if (hLog != NULL ) fclose (hLog); 
		return FALSE; 
	}

	if (WtpComm.GetPortType() == USB)
	{
		WtpComm.SetUsbPortTimeout (30); // Default = 0
	}
	
	if (WtpComm.OpenPort() == FALSE) return FALSE;
	
	if (IsOtpView) 
	{ 
		if (bLogFile)
		{
			fprintf (hLog,"\nOTP Viewer Mode\n");
		}
		else
		{
			printf ("\nOTP Viewer Mode\n");
		}		
		//WTP_PRINT (hLog,"\nOTP Viewer Mode\n"); 
		OtpView (); 
	}
#if TRUSTED
	else if(IsTim && IsJtag && IsImageFile)
	{		
		if (bLogFile)
		{
			fprintf (hLog,"\nNumber of files to download: %d\n",FileIndex);
		}
		else
		{
			printf ("\nNumber of files to download: %d\n",FileIndex);
		}		
		//WTP_PRINT (hLog,"\nNumber of files to download: %d\n",FileIndex);

		for (i = 0; i < FileIndex; i++)
		{
			if (DownloadImage (i) == FALSE)
			{
				if (bLogFile)
				{
					fprintf (hLog,"\n  Error: Download failed for file number: %d\n",i + 1);
				}
				else
				{
					printf ("\n  Error: Download failed for file number: %d\n",i + 1);
				}
				//WTP_PRINT (hLog,"\n  Error: Download failed for file number: %d\n",i + 1);
				IsOk = false;
				//SetLastError (IMAGE_DOWNLOAD_ERROR);
				//ExitProcess(5);
				break;
			}
			else
			{
				if (bLogFile)
				{
					fprintf (hLog,"  Success: Download file complete!\n",i + 1);
				}
				else
				{
					printf ("  Success: Download file complete!\n",i + 1);
				}
				//WTP_PRINT (hLog,"  Success: Download file complete!\n",i + 1);
			}
			// After downloading TIM, allow for a JTAG reenablement.
			if ( i == 0 )
			{
				if (bLogFile)
				{
					fprintf (hLog,"\nJTAG Reenablement Mode\n");
				}
				else
				{
					printf ("\nJTAG Reenablement Mode\n");
				}
				//WTP_PRINT (hLog,"\nJTAG Reenablement Mode\n"); 
				if(JtagReEnable () == FALSE)
				{
					if (bVerbose)
					{ 
						if (bLogFile)
						{
							fprintf (hLog,"\nSending Done...\n");
						}
						else
						{
							printf ("\nSending Done...\n");
						}						
						//WTP_PRINT (hLog,"\nSending Done...\n");
					}
					WtpComm.Done();
				}
			}
		}
	}
	else if (IsJtag)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nJTAG Reenablement Mode\n");
		}
		else
		{
			printf ("\nJTAG Reenablement Mode\n");
		} 
		//WTP_PRINT (hLog,"\nJTAG Reenablement Mode\n"); 
		if(JtagReEnable () == FALSE)
		{	
			if (bVerbose)
			{
				if (bLogFile)
				{
					fprintf (hLog,"\nSending Done...\n");
				}
				else
				{
					printf ("\nSending Done...\n");
				} 
				//WTP_PRINT (hLog,"\nSending Done...\n");
			}
			WtpComm.Done();
		}
	}
#endif
	else if (IsDebugBoot) 
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nDebug Boot Mode\n");
		}
		else
		{
			printf ("\nDebug Boot Mode\n");
		} 
		//WTP_PRINT (hLog,"\nDebug Boot Mode\n"); 
		DebugBoot (); 
	}
            else if (currentMessageMode == MSG_ONLY )

            {
				if (bLogFile)
				{
					fprintf (hLog,"******************************\n");
					fprintf (hLog,"*****Message mode enabled*****\n");
					fprintf (hLog,"******************************\n");
				}
				else
				{
					printf ("******************************\n");
					printf ("*****Message mode enabled*****\n");
					printf ("******************************\n");
				}
                //WTP_PRINT(hLog,"******************************\n");
                //WTP_PRINT(hLog,"*****Message mode enabled*****\n");
                //WTP_PRINT(hLog,"******************************\n");
                if (bVerbose)
				{
					if (bLogFile)
					{
						fprintf (hLog,"\nSending preamble...\n");
					}
					else
					{
						printf ("\nSending preamble...\n");
					}
                	//WTP_PRINT (hLog,"\nSending preamble...\n");
				}
                while (TRUE){
                    if (Connected == FALSE){
                        if (WtpComm.SendPreamble() == false)
                        {
                                    Sleep(1);
                                    continue;
                        }
                        else
                        {
                                    Connected = TRUE;
                        }
                    }
                    WtpStatus.Status = ACK;
                    for(;;)
                    {
                        if ( (WtpStatus.Flags & 0x1) == MESSAGEPENDING ) 
                        {                                                            
                            WtpComm.GetTargetMessage (&WtpStatus); 
                        }
                        else
                        {
                            WtpComm.Disconnect(pWtpStatus);
                            if (pWtpStatus->Status == ACK)
                            {
                                WtpComm.ClosePort();
                                return TRUE;
                            }
                        }
                    }
                }
            }

    else if (IsFbf)
    {
        if (DownloadMasterBlockHeader() == FALSE)
        {
    		if (bLogFile)
			{
				fprintf (hLog,"\n  Image type requested by target is not FBF. Retry!");
			}
			else
			{
				printf ("\n  Image type requested by target is not FBF. Retry!");
			}
            //WTP_PRINT (hLog,"\n  Image type requested by target is not FBF. Retry!");
            if (DownloadMasterBlockHeader() == FALSE)
            {
	    		if (bLogFile)
				{
					fprintf (hLog,"\n  Error: Retry FBF download failed.!\n");
				}
				else
				{
					printf ("\n  Error: Retry FBF download failed.!\n");
				}
                //WTP_PRINT (hLog,"\n  Error: Retry FBF download failed.!\n");
                IsOk = false;
            }
            else
            {
                // if DKB NACKS first attempt but ACKs 2nd attempt, handle images here
	    		if (bLogFile)
				{
					fprintf (hLog,"  Success: MasterBlockHeader Download complete!\n");
					fprintf (hLog,"  Download all images in FBF file!\n");
				}
				else
				{
					printf ("  Success: MasterBlockHeader Download complete!\n");
					printf ("  Download all images in FBF file!\n");
				}
                //WTP_PRINT (hLog,"  Success: MasterBlockHeader Download complete!\n");
                //WTP_PRINT (hLog,"  Download all images in FBF file!\n");
                while ( DownloadFBFImage() == TRUE )
				{
		    		if (bLogFile)
					{
						fprintf (hLog,"  Success: Image Download complete!\n");
					}
					else
					{
						printf ("  Success: Image Download complete!\n");
					}
                    //WTP_PRINT (hLog,"  Success: Image Download complete!\n");
				}
                
				if (bLogFile)
				{
					fprintf (hLog,"  Success: All Images Downloaded!\n");
				}
				else
				{
					printf ("  Success: All Images Downloaded!\n");
				}                
                //WTP_PRINT (hLog,"  Success: All Images Downloaded!\n");
          }
        }
        else
        {
            // if DKB ACKs the first try for FBF, then handle images here
			if (bLogFile)
			{
				fprintf (hLog,"  Success: MasterBlockHeader Download complete!\n");
				fprintf (hLog,"  Download all images in FBF file!\n");
			}
			else
			{
				printf ("  Success: MasterBlockHeader Download complete!\n");
				printf ("  Download all images in FBF file!\n");
			}
            //WTP_PRINT (hLog,"  Success: MasterBlockHeader Download complete!\n");
            //WTP_PRINT (hLog,"  Download all images in FBF file!\n");
            while ( DownloadFBFImage() == TRUE )
			{
				if (bLogFile)
				{
					fprintf (hLog,"  Success: Image Download complete!\n");
				}
				else
				{
					printf ("  Success: Image Download complete!\n");
				}
                //WTP_PRINT (hLog,"  Success: Image Download complete!\n");
			}
            
			if (bLogFile)
			{
				fprintf (hLog,"  Success: All Images Downloaded!\n");
			}
			else
			{
				printf ("  Success: All Images Downloaded!\n");
			}            
            //WTP_PRINT (hLog,"  Success: All Images Downloaded!\n");
        }
    }
	else
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nNumber of files to download: %d\n",FileIndex);
		}
		else
		{
			printf ("\nNumber of files to download: %d\n",FileIndex);
		}
		//WTP_PRINT (hLog,"\nNumber of files to download: %d\n",FileIndex);

		for (i = 0; i < FileIndex; i++)
		{
			if (DownloadImage (i) == FALSE)
			{
				if (bLogFile)
				{
					fprintf (hLog,"\n  Error: Download failed for file number: %d\n",i + 1);
				}
				else
				{
					printf ("\n  Error: Download failed for file number: %d\n",i + 1);
				}
				//WTP_PRINT (hLog,"\n  Error: Download failed for file number: %d\n",i + 1);
				IsOk = false;
				if (hLog != NULL ) fclose (hLog);
				ExitProcess(25);
				return FALSE;
				break;
			}
			else
			{
				if (bLogFile)
				{
					fprintf (hLog,"  Success: Download file complete!\n",i + 1);
				}
				else
				{
					printf ("  Success: Download file complete!\n",i + 1);
				}				
				//WTP_PRINT (hLog,"  Success: Download file complete!\n",i + 1);
			}
		#if SLE_TESTING
		if(i==0)
		Sleep(120000);
		#endif
		}
	}
	if ( currentMessageMode == DISABLED || currentMessageMode == MSG_AND_DOWNLOAD )
	{
		if(IsOtpView || IsDebugBoot)
			;
		else
		do{
			WtpComm.Disconnect(pWtpStatus);
		}while(pWtpStatus->Status == NACK);
		WtpComm.ClosePort();
	}
	if (hLog != NULL ) fclose (hLog);
	return RetVal;
}

bool DownloadImage (int iFile)
{
	WTPCOMMAND WtpCmd;
	WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x01, 0x00};
	DWORD ImageType;
	BYTE IsImage = NACK;
	unsigned int uiRemainingBytes = 0xAA55FF00;
	TCHAR szFileName[MAX_PATH];
	long lFileSize = 0,BytesRead;
	FILE* hFile;
	int i = 0;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending preamble...\n");
		}
		else
		{
			printf ("\nSending preamble...\n");
		} 
		//WTP_PRINT (hLog,"\nSending preamble...\n");
	}

	if (WtpComm.SendPreamble() == false) return FALSE;

	if ( currentMessageMode == MSG_AND_DOWNLOAD )
	{
		WtpStatus.Status = ACK;
		WtpComm.GetTargetMessage(&WtpStatus);
		WtpStatus.Flags = 0x0;
	}
	
	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending GetVersion command...\n");
		}
		else
		{
			printf ("\nSending GetVersion command...\n");
		}		 
		//WTP_PRINT (hLog,"\nSending GetVersion command...\n");
	}
	if (WtpComm.GetVersion(&WtpStatus) == false) return FALSE;
	
	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending SelectImage command...\n");
		}
		else
		{
			printf ("\nSending SelectImage command...\n");
		} 
	//WTP_PRINT (hLog,"\nSending SelectImage command...\n");
	}
	if (WtpComm.SelectImage(&ImageType) == false) return FALSE;
	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nImage Type: %X",ImageType);
		}
		else
		{
			printf ("\nImage Type: %X",ImageType);
		}
	// WTP_PRINT (hLog,"\nImage Type: %X",ImageType);
	}

	if (!IsTim) ImageType = OBMI + iFile;
	IsImage = AckWithImageName(ImageType,szFileName);

	if (IsImage == ACK) 
	{ 
		if (bVerbose) 
		{
			if (bLogFile)
			{
				fprintf (hLog,"\nDownloading file: %s\n",szFileName);
			}
			else
			{
				printf ("\nDownloading file: %s\n",szFileName);
			}
			//WTP_PRINT (hLog,"\nDownloading file: %s\n",szFileName); 
		}
	}
	else
	{
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Don't have image type asked for by target: %X\n",ImageType);
		}
		else
		{
			printf ("\n  Error: Don't have image type asked for by target: %X\n",ImageType);
		}
		//WTP_PRINT (hLog,"\n  Error: Don't have image type asked for by target: %X\n",ImageType);
	}

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending VerifyImage command...\n");
		}
		else
		{
			printf ("\nSending VerifyImage command...\n");
		}
	 //WTP_PRINT (hLog,"\nSending VerifyImage command...\n");
	}
	if (WtpComm.VerifyImage(IsImage) == false)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Download file number %d has aborted!\n",iFile + 1);
		}
		else
		{
			printf ("\n  Error: Download file number %d has aborted!\n",iFile + 1);
		}
		//WTP_PRINT (hLog,"\n  Error: Download file number %d has aborted!\n",iFile + 1);
		WtpComm.Done();
		return FALSE;
	}

	if ((hFile = fopen (szFileName,"rb")) == NULL)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: fopen failed to open %s.",szFileName);
		}
		else
		{
			printf ("  Error: fopen failed to open %s.",szFileName);
		}
		//WTP_PRINT (hLog,"  Error: fopen failed to open %s.",szFileName);
		return FALSE;
	}

	fseek (hFile,0L,SEEK_END); // Set position to EOF
	lFileSize = ftell (hFile); // Get position of file, thus the file size.

	if (ImageType == TIMH)
	{
		fseek (hFile,0L,SEEK_SET); // Set position to SOF
		uiRemainingBytes = (unsigned int)lFileSize;
	}
	else
	{
		fseek (hFile,4L,SEEK_SET); // Set position to SOF
		uiRemainingBytes = (unsigned int)lFileSize - 4;
	}

	while (uiRemainingBytes > 0)
	{
		if (bVerbose)
		{
		 //WTP_PRINT (hLog,"\nSending DataHeader command...\n");
		}
		if (WtpComm.DataHeader(uiRemainingBytes,&WtpStatus) == false)
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: WtpComm.DataHeader failed to receive an ACK!");
			}
			else
			{
				printf ("  Error: WtpComm.DataHeader failed to receive an ACK!");
			}
			//WTP_PRINT (hLog,"  Error: WtpComm.DataHeader failed to receive an ACK!");
			if (hFile != NULL) fclose (hFile);
			return FALSE;
		}

		unsigned int uiBufferSize = 0;
		for (int i = 0; i < (int)WtpStatus.DLEN; i++)
		{
			uiBufferSize |= WtpStatus.Data[i] << (8 * i);
		}

		if(uiBufferSize > uiRemainingBytes) uiBufferSize = uiRemainingBytes;
		uiRemainingBytes -= uiBufferSize; // Calculate remaining bytes

		if (bVerbose)
		{
			if (bLogFile)
			{
				fprintf (hLog,"Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
			}
			else
			{
				printf ("Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
			} 
			//WTP_PRINT (hLog,"Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
		}

		if ((BytesRead = fread (&WtpCmd.Data,1,uiBufferSize,hFile)) != (long)uiBufferSize)
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			}
			else
			{
				printf ("  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			}
			//WTP_PRINT (hLog,"  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			if (hFile != NULL) fclose (hFile);
			return FALSE;
		}
		else
		{
			#if SLE_TESTING
            short totalSize=0;
		    unsigned int mark = 0;
            const short PACKET_SIZE = 512;
			unsigned int packetSize = PACKET_SIZE;
            totalSize = uiBufferSize;
            if (totalSize > PACKET_SIZE )
            {
                do
                {
                    if (bVerbose)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
						}
						else
						{
							printf ("\nSending data block with buffer size of %d...\n",packetSize);
						}                     
                    //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
					}
                    if (WtpComm.Data(&(WtpCmd.Data[mark]),packetSize) == false)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
						}
						else
						{
							printf ("  Error: WtpComm.Data failed to receive an ACK!");
						}                        
                        //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                        if (hFile != NULL) fclose (hFile);
                        return FALSE;
                    }
                    totalSize-=packetSize;
					mark += packetSize;

					if (totalSize <= packetSize)				
						packetSize = totalSize;					

				// Sleep for 1 minute
                Sleep (90000);
                }while (totalSize > 0);
            }
			else
			{
				if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
				{
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					}
					else
					{
						printf ("  Error: WtpComm.Data failed to receive an ACK!");
					}
					//WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					if (hFile != NULL) fclose (hFile);
					return FALSE;
				}
			}
			#else
			if (WtpComm.GetPortType() == USB)
			{
			    unsigned int totalSize = 0;
			    unsigned int mark = 0;
	            const short PACKET_SIZE = 512;
				unsigned int packetSize = PACKET_SIZE;
		        totalSize = uiBufferSize;
			    if (totalSize > PACKET_SIZE )
				{
					do
					{	    				
					    if (bVerbose)
					    {
							if (bLogFile)
							{
								fprintf (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
							}
							else
							{
								printf ("\nSending data block with buffer size of %d...\n",packetSize);
							}
					     //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",packetSize); 
						}
					    if (WtpComm.Data(&(WtpCmd.Data[mark]),packetSize) == false)				
					    {
							if (bLogFile)
							{
								fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
							}
							else
							{
								printf ("  Error: WtpComm.Data failed to receive an ACK!");
							}				
							
							//WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");					
							if (hFile != NULL) fclose (hFile);					
							return FALSE;					
					    }				
					    mark += packetSize;				
					    totalSize-=packetSize;				

						if (totalSize <= packetSize)				
							packetSize = totalSize;					
											
					}while (totalSize > 0);
			
			    }		
			    else
			    {
					if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
					{
						if (bLogFile)
						{
							fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
						}
						else
						{
							printf ("  Error: WtpComm.Data failed to receive an ACK!");
						}					
					    //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");			    
					    if (hFile != NULL) fclose (hFile);					
					    return FALSE;
					}			
			    }
			}
			else if (WtpComm.GetPortType() == UART)
			{
			    if (bVerbose)
			    {
					if (bLogFile)
					{
						fprintf (hLog,"\nSending data block with buffer size of %d...\n",uiBufferSize);
					}
					else
					{
						printf ("\nSending data block with buffer size of %d...\n",uiBufferSize);
					} 
			    //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",uiBufferSize);
				}
			    if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
			    {
					
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					}
					else
					{
						printf ("  Error: WtpComm.Data failed to receive an ACK!");
					}					
					//WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					if (hFile != NULL) fclose (hFile);
					return FALSE;
			    }
			}
			#endif
		}
	}

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Done command...\n");
		}
		else
		{
			printf ("\nSending Done command...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Done command...\n");
	}
	WtpComm.Done();

	if (hFile != NULL) fclose (hFile);

	return TRUE;
}
bool DownloadMasterBlockHeader()
{
    WTPCOMMAND WtpCmd;
    WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x01, 0x00};
    DWORD ImageType;
    BYTE IsImage = NACK;
    unsigned int uiRemainingBytes = 0xAA55FF00;
    TCHAR szFileName[MAX_PATH];
    long lFileSize = 0,BytesRead;
    FILE* hFile;
    int i = 0;
    MasterBlockHeader* pMBH = &MBH;
    if ( bVerbose == FALSE )
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nForce bVerbose to TRUE for debug in DownloadMasterBlockHeader. RAR \n");
		}
		else
		{
			printf ("\nForce bVerbose to TRUE for debug in DownloadMasterBlockHeader. RAR \n");
		}        
        //WTP_PRINT( hLog,"\nForce bVerbose to TRUE for debug in DownloadMasterBlockHeader. RAR \n");
        bVerbose = TRUE;
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending preamble...\n");
		}
		else
		{
			printf ("\nSending preamble...\n");
		} 
    //WTP_PRINT (hLog,"\nSending preamble...\n");
	}
    if (WtpComm.SendPreamble() == false) return FALSE;
    if ( currentMessageMode == MSG_AND_DOWNLOAD )
    {
        WtpStatus.Status = ACK;
        WtpComm.GetTargetMessage(&WtpStatus);
        WtpStatus.Flags = 0x0;
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending GetVersion command...\n");
		}
		else
		{
			printf ("\nSending GetVersion command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending GetVersion command...\n");
	}
    if (WtpComm.GetVersion(&WtpStatus) == false) return FALSE;
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending SelectImage command...\n");
		}
		else
		{
			printf ("\nSending SelectImage command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending SelectImage command...\n");
	}
    if (WtpComm.SelectImage(&ImageType) == false) return FALSE;
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nImage Type: %X",ImageType);
		}
		else
		{
			printf ("\nImage Type: %X",ImageType);
		} 
    //WTP_PRINT (hLog,"\nImage Type: %X",ImageType);
	}
    IsImage = AckWithImageName(ImageType,szFileName);
    strcpy( szFBFFileName, szFileName );
    if (IsImage == ACK) 
    { 
    	if (bVerbose)
    	{
			if (bLogFile)
			{
				fprintf (hLog,"\nDownloading file: %s\n",szFileName);
			}
			else
			{ 
	    		printf ("\nDownloading file: %s\n",szFileName);
	    	} 
		}
    }
    else
    {
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Don't have image type asked for by target: %X\n",ImageType);
		}
		else
		{
			printf ("\n  Error: Don't have image type asked for by target: %X\n",ImageType);
		}
        //WTP_PRINT (hLog,"\n  Error: Don't have image type asked for by target: %X\n",ImageType);
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending VerifyImage command...\n");
		}
		else
		{
			printf ("\nSending VerifyImage command...\n");
		} 
    	//WTP_PRINT (hLog,"\nSending VerifyImage command...\n");
	}
    if (WtpComm.VerifyImage(IsImage) == false)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Download FBF MasterBlockHeader has aborted!\n");
		}
		else
		{
			printf ("\n  Error: Download FBF MasterBlockHeader has aborted!\n");
		}        
        //WTP_PRINT (hLog,"\n  Error: Download FBF MasterBlockHeader has aborted!\n");
        WtpComm.Done();
        return FALSE;
    }
    if ((hFile = fopen (szFileName,"rb")) == NULL)
    {
		if (bLogFile)
		{
			fprintf (hLog,"  Error: fopen failed to open %s.",szFileName);
		}
		else
		{
			printf ("  Error: fopen failed to open %s.",szFileName);
		}        
        //WTP_PRINT (hLog,"  Error: fopen failed to open %s.",szFileName);
        return FALSE;
    }
    // need to download sector 0
    lFileSize = BLOCK_DEVICE_SECTOR_SIZE;
	if (bLogFile)
	{
		fprintf (hLog,"  MasterBlockHeader size %d loading...",lFileSize);
	}
	else
	{
		printf ("  MasterBlockHeader size %d loading...",lFileSize);
	}
    //WTP_PRINT (hLog,"  MasterBlockHeader size %d loading...",lFileSize);
    if (ImageType == FBFI)
    {
        fseek (hFile,0L,SEEK_SET); // Set position to SOF
        uiRemainingBytes = (unsigned int)lFileSize;
    }
    while (uiRemainingBytes > 0)
    {
        if (bVerbose)
        {
			if (bLogFile)
			{
				fprintf (hLog,"\nSending DataHeader command...\n");
			}
			else
			{
				printf ("\nSending DataHeader command...\n");
			}
         //WTP_PRINT (hLog,"\nSending DataHeader command...\n");

		}
        if (WtpComm.DataHeader(uiRemainingBytes,&WtpStatus) == false)
        {
			if (bLogFile)
			{
				fprintf (hLog,"  Error: WtpComm.DataHeader failed to receive an ACK!");
			}
			else
			{
				printf ("  Error: WtpComm.DataHeader failed to receive an ACK!");
			}            
            //WTP_PRINT (hLog,"  Error: WtpComm.DataHeader failed to receive an ACK!");
            if (hFile != NULL) fclose (hFile);
            return FALSE;
        }
        unsigned int uiBufferSize = 0;
        for (int i = 0; i < (int)WtpStatus.DLEN; i++)
        {
            uiBufferSize |= WtpStatus.Data[i] << (8 * i);
        }
        if(uiBufferSize > uiRemainingBytes) uiBufferSize = uiRemainingBytes;
        uiRemainingBytes -= uiBufferSize; // Calculate remaining bytes
        if (bVerbose)
		{
			if (bLogFile)
			{
				fprintf (hLog,"Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
			}
			else
			{
				printf ("Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
			}
            //WTP_PRINT (hLog,"Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
		}
        if ((BytesRead = fread (&WtpCmd.Data,1,uiBufferSize,hFile)) != (long)uiBufferSize)
        {
			if (bLogFile)
			{
				fprintf (hLog,"  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			}
			else
			{
				printf ("  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			}
            //WTP_PRINT (hLog,"  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
            if (hFile != NULL) fclose (hFile);
            return FALSE;
        }
        else
        {
            #if SLE_TESTING
            short totalSize=0;
		    unsigned int mark = 0;
            const short PACKET_SIZE = 512;
			unsigned int packetSize = PACKET_SIZE;
            totalSize = uiBufferSize;
            if (totalSize > PACKET_SIZE )
            {
                do
                {
                    if (bVerbose)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
						}
						else
						{
							printf ("\nSending data block with buffer size of %d...\n",packetSize);
						}                     
                     //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
					}
                    if (WtpComm.Data(&(WtpCmd.Data[mark]),packetSize) == false)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
						}
						else
						{
							printf ("  Error: WtpComm.Data failed to receive an ACK!");
						}                        
                        //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                        if (hFile != NULL) fclose (hFile);
                        return FALSE;
                    }
                    totalSize-=packetSize;
					mark += packetSize;

					if (totalSize <= packetSize)				
						packetSize = totalSize;					

				// Sleep for 1 minute
                Sleep (90000);
                }while (totalSize > 0);
            }
            else
            {
                if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
                {
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					}
					else
					{
						printf ("  Error: WtpComm.Data failed to receive an ACK!");
					}
                    //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                    if (hFile != NULL) fclose (hFile);
                    return FALSE;
                }
            }
            #else
            if (bVerbose)
            {
				if (bLogFile)
				{
					fprintf (hLog,"\nSending data block with buffer size of %d...\n",uiBufferSize);
				}
				else
				{
					printf ("\nSending data block with buffer size of %d...\n",uiBufferSize);
				} 
            //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",uiBufferSize);
			}
            short totalSize=0;
		    unsigned int mark = 0;
            const short PACKET_SIZE = 512;
			unsigned int packetSize = PACKET_SIZE;
	        totalSize = uiBufferSize;
		    if (totalSize > PACKET_SIZE )
            {
                do
                {
                    if (bVerbose)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
						}
						else
						{
							printf ("\nSending data block with buffer size of %d...\n",packetSize);
						} 
                    //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
					}
                    if (WtpComm.Data(&(WtpCmd.Data[mark]),packetSize) == false)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
						}
						else
						{
							printf ("  Error: WtpComm.Data failed to receive an ACK!");
						}                        
                        //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                        if (hFile != NULL) fclose (hFile);
                        return FALSE;
                    }
                    totalSize -= packetSize;
					mark += packetSize;

					if (totalSize <= packetSize)				
						packetSize = totalSize;					

				}while (totalSize > 0);
            }
            else
            {
	            if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
	            {
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					}
					else
					{
						printf ("  Error: WtpComm.Data failed to receive an ACK!");
					}	                
	                //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
	                if (hFile != NULL) fclose (hFile);
	                return FALSE;
	            }
			}
            #endif
            // Save data in MBH
            memcpy( pMBH, &WtpCmd.Data, uiBufferSize );
            pMBH = (MasterBlockHeader*)((int)pMBH + uiBufferSize);
        }
        if (bVerbose)
        {
			if (bLogFile)
			{
				fprintf (hLog,"\nMBH Format Version: %d\n", MBH.Format_Version);
				fprintf (hLog,"\nNumber of Devices: %d\n", MBH.nOfDevices);
			}
			else
			{
				printf ("\nMBH Format Version: %d\n", MBH.Format_Version);
				printf ("\nNumber of Devices: %d\n", MBH.nOfDevices);
			} 
        //WTP_PRINT (hLog,"\nMBH Format Version: %d\n", MBH.Format_Version);
       	//WTP_PRINT (hLog,"\nNumber of Devices: %d\n", MBH.nOfDevices);
	   }
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Done command...\n");
		}
		else
		{
			printf ("\nSending Done command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending Done command...\n");
	}
    WtpComm.Done();
    if (hFile != NULL) fclose (hFile);
    return TRUE;
}
bool DownloadFBFImage()
{
    WTPCOMMAND WtpCmd;
    WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x01, 0x00};
    DWORD DeviceImageNums;
    UINT DeviceNum;
    UINT ImageNum;
    BYTE IsImage = NACK;
    unsigned int uiRemainingBytes = 0xAA55FF00;
    TCHAR szFileName[MAX_PATH];
    long lImageSize = 0,BytesRead;
    FILE* hFile;
    int i = 0;
    PDeviceHeader pDeviceHeader = 0;
    PImageStruct pImageStruct = 0;
    long ImageStartByte = 0L;
#define DEV_IMAGE_NUMS_EXTRACT( devimgnums ) DeviceNum = (devimgnums & 0xFFFF0000)>>16; ImageNum = devimgnums & 0xFFFF;
    if ( bVerbose == FALSE )
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nForce bVerbose to TRUE for debug in Download FBFImage. RAR \n");
		}
		else
		{
			printf ("\nForce bVerbose to TRUE for debug in Download FBFImage. RAR \n");
		}        
        //WTP_PRINT( hLog,"\nForce bVerbose to TRUE for debug in Download FBFImage. RAR \n");
        bVerbose = TRUE;
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending preamble...\n");
		}
		else
		{
			printf ("\nSending preamble...\n");
		} 
    //WTP_PRINT (hLog,"\nSending preamble...\n");
	}
    if (WtpComm.SendPreamble() == false) return FALSE;
    if ( currentMessageMode == MSG_AND_DOWNLOAD )
    {
        WtpStatus.Status = ACK;
        WtpComm.GetTargetMessage(&WtpStatus);
        WtpStatus.Flags = 0x0;
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending GetVersion command...\n");
		}
		else
		{
			printf ("\nSending GetVersion command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending GetVersion command...\n");
	}
    if (WtpComm.GetVersion(&WtpStatus) == false) return FALSE;
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending SelectImage command...\n");
		}
		else
		{
			printf ("\nSending SelectImage command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending SelectImage command...\n");
	}
    if (WtpComm.SelectImage(&DeviceImageNums) == false)
    {
        if (bVerbose)
        {
			if (bLogFile)
			{
				fprintf (hLog,"\nSelectImage failed in WTPTP!");
			}
			else
			{
				printf ("\nSelectImage failed in WTPTP!");
			} 
        //WTP_PRINT (hLog,"\nSelectImage failed in WTPTP!");
		}
        return FALSE;
    }
    DEV_IMAGE_NUMS_EXTRACT( DeviceImageNums );
    if ( DeviceImageNums == 0xFFFFFFFF )
    {
        // this may not be the right thing to do but if I don't DKB hangs on the SelectImage command
        // on the other hand, if I do this, DKB hangs on a timeout on HandleDisconnect after WTPTP
        // has already shutdown.....What to do?????
        if (bVerbose)
        {
			if (bLogFile)
			{
				fprintf (hLog,"\nSending Done command...\n");
			}
			else
			{
				printf ("\nSending Done command...\n");
			} 
        //WTP_PRINT (hLog,"\nSending Done command...\n");
		}
        WtpComm.Done();
        // breakout of download image loop
        return FALSE;
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nDeviceNum: %d   ImageNum: %d\n",DeviceNum, ImageNum);
		}
		else
		{
			printf ("\nDeviceNum: %d   ImageNum: %d\n",DeviceNum, ImageNum);
		} 
    //WTP_PRINT (hLog,"\nDeviceNum: %d   ImageNum: %d\n",DeviceNum, ImageNum);
	}
    if ( DeviceNum >= 0 && DeviceNum < MBH.nOfDevices )
        pDeviceHeader = (PDeviceHeader)(((int)&MBH) + MBH.deviceHeaderOffset[ DeviceNum ]);
    else
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nError - Unknown Device Number - DeviceNum: %d\n",DeviceNum);
		}
		else
		{
			printf ("\nError - Unknown Device Number - DeviceNum: %d\n",DeviceNum);
		}
        //WTP_PRINT (hLog,"\nError - Unknown Device Number - DeviceNum: %d\n",DeviceNum);
        return FALSE;
    }
    if ( ImageNum >= 0 && ImageNum < pDeviceHeader->nOfImages )
         pImageStruct = &pDeviceHeader->imageStruct[ ImageNum ];
    else
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nError - Unknown Image Number - ImageNum: %d\n",ImageNum);
		}
		else
		{
			printf ("\nError - Unknown Image Number - ImageNum: %d\n",ImageNum);
		}        
        //WTP_PRINT (hLog,"\nError - Unknown Image Number - ImageNum: %d\n",ImageNum);
        return FALSE;
    }
//    IsImage = AckWithImageName(FBFI,szFileName);
    IsImage = ACK;
    strcpy( szFileName, szFBFFileName );
#if 0
    if (IsImage == ACK)
    {
        if (bVerbose)
        {
			if (bLogFile)
			{
				fprintf (hLog,"\nDownloading image from file: %s\n",szFileName);
			}
			else
			{
				printf ("\nDownloading image from file: %s\n",szFileName);
			} 
        //WTP_PRINT (hLog,"\nDownloading image from file: %s\n",szFileName);
		}
    }
    else
    {
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Download failed in WTPTP!\n");
		}
		else
		{
			printf ("\n  Error: Download failed in WTPTP!\n");
		}
        //WTP_PRINT (hLog,"\n  Error: Download failed in WTPTP!\n");
        return FALSE;
    }
#endif
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending VerifyImage command...\n");
		}
		else
		{
			printf ("\nSending VerifyImage command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending VerifyImage command...\n");
	}
    if (WtpComm.VerifyImage(IsImage) == false)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Download FBF image has aborted!\n");
		}
		else
		{
			printf ("\n  Error: Download FBF image has aborted!\n");
		}         
        //WTP_PRINT (hLog,"\n  Error: Download FBF image has aborted!\n");
        WtpComm.Done();
        return FALSE;
    }
    if ((hFile = fopen (szFileName,"rb")) == NULL)
    {
		if (bLogFile)
		{
			fprintf (hLog,"  Error: fopen failed to open %s.",szFileName);
		}
		else
		{
			printf ("  Error: fopen failed to open %s.",szFileName);
		}        
        //WTP_PRINT (hLog,"  Error: fopen failed to open %s.",szFileName);
        return FALSE;
    }
    fseek (hFile,0L,SEEK_SET); // Set position to SOF
//    lFileSize = ftell (hFile); // Get position of file, thus the file size.
    // need to download sector 0
    lImageSize = pImageStruct->length;
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\n  Image size %d loading...\n", lImageSize);
		}
		else
		{
			printf ("\n  Image size %d loading...\n", lImageSize);
		} 
    //WTP_PRINT (hLog,"\n  Image size %d loading...\n", lImageSize);
	}
    ImageStartByte = pImageStruct->First_Sector*BLOCK_DEVICE_SECTOR_SIZE;
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\n  File start position... %d\n", ImageStartByte);
		}
		else
		{
			printf ("\n  File start position... %d\n", ImageStartByte);
		} 
    //WTP_PRINT (hLog,"\n  File start position... %d\n", ImageStartByte);
	}
    fseek (hFile,ImageStartByte,SEEK_SET); // Set position to start of image
    uiRemainingBytes = (unsigned int)lImageSize;
    while (uiRemainingBytes > 0)
    {
        if (bVerbose)
        {
			if (bLogFile)
			{
				fprintf (hLog,"\nSending DataHeader command...\n");
			}
			else
			{
				printf ("\nSending DataHeader command...\n");
			} 
        //WTP_PRINT (hLog,"\nSending DataHeader command...\n");
		}
        if (WtpComm.DataHeader(uiRemainingBytes,&WtpStatus) == false)
        {
			if (bLogFile)
			{
				fprintf (hLog,"  Error: WtpComm.DataHeader failed to receive an ACK!");
			}
			else
			{
				printf ("  Error: WtpComm.DataHeader failed to receive an ACK!");
			}            
            //WTP_PRINT (hLog,"  Error: WtpComm.DataHeader failed to receive an ACK!");
            if (hFile != NULL) fclose (hFile);
            return FALSE;
        }
        unsigned int uiBufferSize = 0;
        for (int i = 0; i < (int)WtpStatus.DLEN; i++)
        {
            uiBufferSize |= WtpStatus.Data[i] << (8 * i);
        }
        if(uiBufferSize > uiRemainingBytes) uiBufferSize = uiRemainingBytes;
        uiRemainingBytes -= uiBufferSize; // Calculate remaining bytes
        if (bVerbose)
		{
			if (bLogFile)
			{
				fprintf (hLog,"Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
			}
			else
			{
				printf ("Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
			}
            //WTP_PRINT (hLog,"Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);
		}
        if ((BytesRead = fread (&WtpCmd.Data,1,uiBufferSize,hFile)) != (long)uiBufferSize)
        {
			if (bLogFile)
			{
				fprintf (hLog,"  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			}
			else
			{
				printf ("  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			}            
            //WTP_PRINT (hLog,"  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
            if (hFile != NULL) fclose (hFile);
            return FALSE;
        }
        else
        {
            #if SLE_TESTING
            short totalSize=0;
		    unsigned int mark = 0;
            const short PACKET_SIZE = 512;
			unsigned int packetSize = PACKET_SIZE;
            totalSize = uiBufferSize;
            if (totalSize > PACKET_SIZE )
            {
                do
                {
                    if (bVerbose)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
						}
						else
						{
							printf ("\nSending data block with buffer size of %d...\n",packetSize);
						}  
                    //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
					}
                    if (WtpComm.Data(&(WtpCmd.Data[mark]),packetSize) == false)
                    {
						if (bLogFile)
						{
							fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
						}
						else
						{
							printf ("  Error: WtpComm.Data failed to receive an ACK!");
						}                        
                        //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                        if (hFile != NULL) fclose (hFile);
                        return FALSE;
                    }
                    totalSize-=packetSize;
					mark += packetSize;

					if (totalSize <= packetSize)				
						packetSize = totalSize;					

				// Sleep for 1 minute
                Sleep (90000);
                }while (totalSize > 0);
            }
            else
            {
                if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
                {
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					}
					else
					{
						printf ("  Error: WtpComm.Data failed to receive an ACK!");
					} 
                    //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                    if (hFile != NULL) fclose (hFile);
                    return FALSE;
                }
            }
            #else
            short totalSize=0;
		    unsigned int mark = 0;
            const short PACKET_SIZE = 512;
			unsigned int packetSize = PACKET_SIZE;
	        totalSize = uiBufferSize;
		    if (totalSize > PACKET_SIZE )
            {
                    do
                    {
                        if (bVerbose)
                        {
							if (bLogFile)
							{
								fprintf (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
							}
							else
							{
								printf ("\nSending data block with buffer size of %d...\n",packetSize);
							} 
                        //WTP_PRINT (hLog,"\nSending data block with buffer size of %d...\n",packetSize);
						}
                        if (WtpComm.Data(&(WtpCmd.Data[mark]),packetSize) == false)
                        {
							if (bLogFile)
							{
								fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
							}
							else
							{
								printf ("  Error: WtpComm.Data failed to receive an ACK!");
							}
                            //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
                            if (hFile != NULL) fclose (hFile);
                            return FALSE;
                        }
                        totalSize-=packetSize;
						mark += packetSize;

						if (totalSize <= packetSize)				
							packetSize = totalSize;					

					}while (totalSize > 0);
            }
            else
            {
                if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
	            {
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
					}
					else
					{
						printf ("  Error: WtpComm.Data failed to receive an ACK!");
					}
	                //WTP_PRINT (hLog,"  Error: WtpComm.Data failed to receive an ACK!");
	                if (hFile != NULL) fclose (hFile);
	                return FALSE;
	            }
			}
            #endif
        }
    }
    if (bVerbose)
    {
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Done command...\n");
		}
		else
		{
			printf ("\nSending Done command...\n");
		} 
    //WTP_PRINT (hLog,"\nSending Done command...\n");
	}
        WtpComm.Done();
    if (hFile != NULL) fclose (hFile);
    return TRUE;
}
bool OtpView ()
{
	BYTE FT = WtpComm.GetFlashType ();

	if (FT >= MAXFLASHTYPES) return FALSE;
	if (bLogFile)
	{
		fprintf (hLog,"Flash type selected: %s\n",FlashTypeCodes[FT].Description);
	}
	else
	{
		printf ("Flash type selected: %s\n",FlashTypeCodes[FT].Description);
	}
	//WTP_PRINT (hLog,"Flash type selected: %s\n",FlashTypeCodes[FT].Description);

	if (FT == 0 || FT == 1 || FT == 2 || FT == 4 || FT == 6)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: OTP registers are not available for NAND flash types!\n");
		}
		else
		{
			printf ("  Error: OTP registers are not available for NAND flash types!\n");
		}		
		//WTP_PRINT (hLog,"  Error: OTP registers are not available for NAND flash types!\n");
		return TRUE;
	}

	if (bLogFile)
	{
		fprintf (hLog,"\nSending preamble...\n");
	}
	else
	{
		printf ("\nSending preamble...\n");
	}
	//WTP_PRINT (hLog,"\nSending preamble...\n");
	if (WtpComm.SendPreamble() == false) return FALSE;

	if (bLogFile)
	{
		fprintf (hLog,"\nSending OtpView command...\n");
	}
	else
	{
		printf ("\nSending OtpView command...\n");
	}
	//WTP_PRINT (hLog,"\nSending OtpView command...\n");
	if (WtpComm.OtpView() == false) return FALSE;

	return TRUE;
}

bool DebugBoot ()
{
	BYTE FT = WtpComm.GetFlashType ();

	if (FT >= MAXFLASHTYPES) return FALSE;
	if (bLogFile)
	{
		fprintf (hLog,"Flash type selected: %s\n",FlashTypeCodes[FT].Description);
	}
	else
	{
		printf ("Flash type selected: %s\n",FlashTypeCodes[FT].Description);
	}
	//WTP_PRINT (hLog,"Flash type selected: %s\n",FlashTypeCodes[FT].Description);

	if (bLogFile)
	{
		fprintf (hLog,"\nSending preamble...\n");
	}
	else
	{
		printf ("\nSending preamble...\n");
	}
	//WTP_PRINT (hLog,"\nSending preamble...\n");
	if (WtpComm.SendPreamble() == false) return FALSE;

	if (bLogFile)
	{
		fprintf (hLog,"\nSending DebugBOOT command...\n");
	}
	else
	{
		printf ("\nSending DebugBOOT command...\n");
	}
	//WTP_PRINT (hLog,"\nSending DebugBOOT command...\n");
	if (WtpComm.DebugBoot(&FT) == false) return FALSE;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Done command...\n");
		}
		else
		{
			printf ("\nSending Done command...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Done command...\n");
	}
	WtpComm.Done();

	return TRUE;
}
#if TRUSTED
bool JtagReEnable ()
{
	WTPCOMMAND WtpCmd;
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	TCHAR szFileName[MAX_PATH];
	pKeyInfo pKey = new KeyInfo;
	FILE *KeyFile, *file;
	int ErrorStatus;
	unsigned int *t1, t2;
	BYTE Password[256];
	BYTE DigitalSignature[256];

	if (AckWithImageName(KEY,szFileName) == NACK)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: Did not supply KEY file for JTAG re-enablement!");
		}
		else
		{
			printf ("  Error: Did not supply KEY file for JTAG re-enablement!");
		}		
		//WTP_PRINT (hLog,"  Error: Did not supply KEY file for JTAG re-enablement!");
		return FALSE;
	}

	if ((KeyFile = fopen (szFileName,"r")) == NULL)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: fopen failed to open key file %s.",szFileName);
		}
		else
		{
			printf ("  Error: fopen failed to open key file %s.",szFileName);
		}
		//WTP_PRINT (hLog,"  Error: fopen failed to open key file %s.",szFileName);
		exit(0);
		return FALSE;
	}

	ErrorStatus = ReadKeyFile ( KeyFile,pKey->PublicKey,pKey->PrivateKey,
								pKey->Modulus,&pKey->KeyLength );
	if (ErrorStatus != NoError)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: ReadKeyFile failed while processing file %s.!",szFileName);
		}
		else
		{
			printf ("  Error: ReadKeyFile failed while processing file %s.!",szFileName);
		}
		//WTP_PRINT (hLog,"  Error: ReadKeyFile failed while processing file %s.!",szFileName);
		fclose (KeyFile);
		return FALSE;
	}

	memcpy (&WtpCmd.Data[0],pKey->PublicKey,4 * pKey->KeyLength);
	memcpy (&WtpCmd.Data[4 * pKey->KeyLength],pKey->Modulus,4 * pKey->KeyLength);
	fclose (KeyFile);

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending preamble...\n");
		}
		else
		{
			printf ("\nSending preamble...\n");
		} 
	//WTP_PRINT (hLog,"\nSending preamble...\n");
	}
	if (WtpComm.SendPreamble() == false) return FALSE;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Public Key...\n");
		}
		else
		{
			printf ("\nSending Public Key...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Public Key...\n");
	}
	if (WtpComm.PublicKey() == false) return FALSE;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Data...\n");
		}
		else
		{
			printf ("\nSending Data...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Data...\n");
	}
	if (WtpComm.Data(WtpCmd.Data,8 * pKey->KeyLength) == false) return FALSE;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Password...\n");
		}
		else
		{
			printf ("\nSending Password...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Password...\n");
	}
	if (WtpComm.Password(pWtpStatus) == false) return FALSE;
	memcpy (&Password,pWtpStatus->Data,8);
	if (bLogFile)
	{
		fprintf (hLog,"Password: %08x %08x\n", (unsigned int)*((unsigned int*)&Password[0]), 
		(unsigned int)*((unsigned int*)&Password[4]));
	}
	else
	{
		printf ("Password: %08x %08x\n", (unsigned int)*((unsigned int*)&Password[0]), 
		(unsigned int)*((unsigned int*)&Password[4]));
	}
	//WTP_PRINT(hLog,"Password: %08x %08x\n", (unsigned int)*((unsigned int*)&Password[0]), 
//		(unsigned int)*((unsigned int*)&Password[4]));
//	PadPasswordToSign ((unsigned int *)Password,pKey->KeyLength);

	RSA (   (unsigned int *)pKey->PrivateKey,
			(unsigned int *)pKey->Modulus,
			pKey->KeyLength,
			(unsigned int *)Password,
			(unsigned int *)DigitalSignature   );

	memcpy (&WtpCmd.Data[0],DigitalSignature,4 * pKey->KeyLength);

	if (bLogFile)
	{
		fprintf (hLog,"Signed Password:\n");
	}
	else
	{
		printf ("Signed Password:\n");
	}
	//WTP_PRINT(hLog,"Signed Password:\n");	
	for(int c = 0; c < pKey->KeyLength * 4; c+=8)
	{
		if (bLogFile)
		{
			fprintf (hLog,"%08x %08x \n", (unsigned int)*((unsigned int *)&DigitalSignature[c]),
	   			(unsigned int)*((unsigned int *)&DigitalSignature[c+4]));
		}
		else
		{
			printf ("%08x %08x \n", (unsigned int)*((unsigned int *)&DigitalSignature[c]),
	   			(unsigned int)*((unsigned int *)&DigitalSignature[c+4]));
		}
	   //	WTP_PRINT(hLog,"%08x %08x \n", (unsigned int)*((unsigned int *)&DigitalSignature[c]),
	   //	(unsigned int)*((unsigned int *)&DigitalSignature[c+4]));
	}

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Signed Password...\n");
		}
		else
		{
			printf ("\nSending Signed Password...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Signed Password...\n");
	}
	if (WtpComm.SignedPassword() == false) return FALSE;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Data...\n");
		}
		else
		{
			printf ("\nSending Data...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Data...\n");
	}
	if (WtpComm.Data(WtpCmd.Data,4 * pKey->KeyLength) == false) return FALSE;

	if (bVerbose)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\nSending Done...\n");
		}
		else
		{
			printf ("\nSending Done...\n");
		} 
	//WTP_PRINT (hLog,"\nSending Done...\n");
	}
	WtpComm.Done();

	return TRUE;
}
#endif
BYTE AckWithImageName (DWORD ImageType,TCHAR *szFileName)
{
    BYTE IsImage = NACK;
    if ( ImageType == FBFI )
    {
        strcpy (szFileName,ImageFiles[0].FileName);
        IsImage = ACK;
    }
    else
    {
        for (int i = 0; i < MAXIMAGEFILES; i++)
        {
            if (ImageFiles[i].FileType == 0) continue;
            if (ImageFiles[i].FileType == ImageType)
            {
                IsImage = ACK;
                strcpy (szFileName,ImageFiles[i].FileName);
                break;
            }
        }
    }
    return IsImage;
}

bool IsScriptFile (int argc,char *argv[],char *ScriptFileName,bool *IsScript)
{
	int currentOption = 1;
	*IsScript = false;
	bool optionIsCommandLine = false;

	// This function will determine whether or not a script file is being used
	// to provision the program by looking for a (-x script.ini) parameter in
	// the command line.

	while (currentOption < argc)
	{
		if (*argv[currentOption] == '-' || *argv[currentOption] == '/')
		{
			if (*++argv[currentOption] == 'x')
			{
				if (++currentOption == argc)
				{
					if (bLogFile)
					{
						fprintf (hLog,"\n  Error: Missing input script file.\n");
					}
					else
					{
						printf ("\n  Error: Missing input script file.\n");
					}
					//WTP_PRINT (hLog,"\n  Error: Missing input script file.\n");
					return FALSE;
				}
				while (*argv[currentOption] != NULL)
				{
					*ScriptFileName++ = *argv[currentOption]++;
				}
				*ScriptFileName = '\0';
				*IsScript = true;
				//break;
			}
			else
			{
				optionIsCommandLine = true;
			}
			--argv[currentOption++];
		}
		else
		{
			currentOption++;
		}
	}

	if (optionIsCommandLine == true && *IsScript == true )
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: Cannot select script file with other command line options\n");
		}
		else
		{
			printf ("\n  Error: Missing input script file.\n");
		}		
		//WTP_PRINT(hLog,"  Error: Cannot select script file with other command line options\n");
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

bool GetImageFile (DWORD ImageType,char *TypeName,BYTE Option,int Argc,char *ImageFile)
{
	// This function is used by ParseCommandLine (...) and ParseScriptFile (...) to
	// obtain image file names as supplied by the user. It will allow only one TIM
	// file to be loaded. It will accept multiple OBM files. If the TIM option is 
	// selected the ParseTim (...) function will populate the ImageFiles[i].FileType
	// for each image from the 32 bit ImageID header of that image. If the TIM option
	// is not selected the FileType defaults, as populated by this function, will be
	// used. Having said that this utility has only been documented to be used in 
	// conjunction with the TIM and it's associated images.

	if (Option == Argc)
	{
		if (bLogFile)
		{
			fprintf (hLog,"\n  Error: Missing %s file name parameter!\n",TypeName);
		}
		else
		{
			printf ("\n  Error: Missing %s file name parameter!\n",TypeName);
		}		
		//WTP_PRINT (hLog,"\n  Error: Missing %s file name parameter!\n",TypeName);
		return FALSE;
	}

	if (ImageType == TIMH) // Can only load 1 TIM file
	{
		for (int i = 0; i < MAXIMAGEFILES; i++)
		{
			if (ImageFiles[i].FileType == ImageType)
			{
				if (bLogFile)
				{
					fprintf (hLog,"  Error: Can't open more than 1 %s file!",TypeName);
				}
				else
				{
					printf ("  Error: Can't open more than 1 %s file!",TypeName);
				}				
				//WTP_PRINT (hLog,"  Error: Can't open more than 1 %s file!",TypeName);
				return FALSE;
			}
		}
	}
    if (ImageType == FBFI)
    {
        strcpy (ImageFiles[FileIndex].FileName, ImageFile);
		if (bLogFile)
		{
			fprintf (hLog,"FBF file name is - %s", ImageFile);
		}
		else
		{
			printf ("FBF file name is - %s", ImageFile);
		}        
        //WTP_PRINT (hLog,"FBF file name is - %s", ImageFile);
        return TRUE;
    }
	strcpy (ImageFiles[FileIndex].FileName,ImageFile);

	if (ImageType == OBMI) // This is for the -f option where you might wan't
	{                      // to load multiple files and TIM is not used.
		ImageFiles[FileIndex++].FileType = ImageType + FileIndex;
	}
	else
	{
		ImageFiles[FileIndex++].FileType = ImageType;
	}

	return TRUE;
}

bool ParseCommandLine (int argc,char *argv[])
{
	unsigned int DownloadBaudRate = 115200;
	int currentOption = 1;
	BYTE Option;
	bool IsPortTypeSet = false;
	bool IsComPortSet = false;
	bool IsBaudRateSet = false;
	bool IsUartDelaySet = false;
	bool IsMsgModeSet = false;
	bool IsInterfaceNameSet = false;

	while (currentOption < argc)
	{
		if (*argv[currentOption] == '-' || *argv[currentOption] == '/')
		{
			Option = *++argv[currentOption++];

			switch (Option)
			{
				case TIM:
					if (IsTim == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: TIM file parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: TIM file parameter must be specified only once.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: TIM file parameter must be specified only once.\n");
						return FALSE;
					}
					if (GetImageFile (TIMH,"TIM",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
					IsTim = TRUE;
					break;
				case OBM:
					if (GetImageFile (OBMI,"IMAGE",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
					IsImageFile = TRUE;
					break;
				case KEY:
#if TRUSTED
					if (IsJtag == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Key file parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: Key file parameter must be specified only once.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Key file parameter must be specified only once.\n");
						return FALSE;
					}
					if (GetImageFile ((DWORD)KEY,"JTAGKEY",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
					IsJtag = TRUE;
#else
					if (bLogFile)
					{
						fprintf (hLog,"  Error: WTPTP Non-trusted only version!!!\n");
					}
					else
					{
						printf ("  Error: WTPTP Non-trusted only version!!!\n");
					}					
					//WTP_PRINT(hLog,"  Error: WTPTP Non-trusted only version!!!\n");
#endif
					break;
                case FBF:
                    if (IsFbf == true )
                    {
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: FBF file parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: FBF file parameter must be specified only once.\n");
						}
                        //WTP_PRINT (hLog,"\n  Error: FBF file parameter must be specified only once.\n");
                        return FALSE;
                    }
                    if (GetImageFile ((DWORD)FBFI,"FBF",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
                    IsFbf = TRUE;
                    break;
				case OTP:
					if (IsOtpView == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: OTP parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: OTP parameter must be specified only once.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: OTP parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing OTP flash type parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing OTP flash type parameter.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Missing OTP flash type parameter.\n");
						return FALSE;
					}
					IsOtpView = TRUE;
					if (atoi (argv[currentOption]) >= MAXFLASHTYPES)
					{
						if (bLogFile)
						{
							fprintf (hLog,"Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						}
						else
						{
							printf ("Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						}						
						//WTP_PRINT (hLog,"Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						return FALSE;
					}
					WtpComm.SetFlashType (atoi (argv[currentOption]));
					break;
				case DBGBOOT:
					if (IsDebugBoot == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: DebugBoot parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: DebugBoot parameter must be specified only once.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: DebugBoot parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing DBG Boot flash type parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing DBG Boot flash type parameter.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Missing DBG Boot flash type parameter.\n");
						return FALSE;
					}
					IsDebugBoot = TRUE;
					if (atoi (argv[currentOption]) >= MAXFLASHTYPES)
					{
						if (bLogFile)
						{
							fprintf (hLog,"Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						}
						else
						{
							printf ("Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						}
						//WTP_PRINT (hLog,"Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						return FALSE;
					}
					WtpComm.SetFlashType (atoi (argv[currentOption]));
					break;
				case PORTTYPE:
					if (IsPortTypeSet == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Port Type parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: Port Type parameter must be specified only once.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Port Type parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing Port Type parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing Port Type parameter.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Missing Port Type parameter.\n");
						return FALSE;
					}
					if (strstr (argv[currentOption],"USB") != NULL)
					{
						WtpComm.SetPortType (USB);
						IsPortTypeSet = true;
						if ((argc % 2) == 0)
						{
							if ( (strstr (argv[currentOption + 1],"UART") != NULL) ||
							 (strstr (argv[currentOption + 1],"USB") != NULL) )
							{
								if (bLogFile)
								{
									fprintf (hLog,"\n  Error: Port Type parameter must be specified only once.\n");
								}
								else
								{
									printf ("\n  Error: Port Type parameter must be specified only once.\n");
								}								
								//WTP_PRINT (hLog,"\n  Error: Port Type parameter must be specified only once.\n");
								return FALSE;
							}
						}
					}
					else if (strstr (argv[currentOption],"UART") != NULL)
					{
						WtpComm.SetPortType (UART);
						IsPortTypeSet = true;
						if ((argc % 2) == 0)
						{

							if ( (strstr (argv[currentOption + 1],"UART") != NULL) ||
							 (strstr (argv[currentOption + 1],"USB") != NULL) )
							{
								if (bLogFile)
								{
									fprintf (hLog,"\n  Error: Port Type parameter must be specified only once.\n");
								}
								else
								{
									printf ("\n  Error: Port Type parameter must be specified only once.\n");
								}
								//WTP_PRINT (hLog,"\n  Error: Port Type parameter must be specified only once.\n");
								return FALSE;
							}
						}
					}
					else
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Port Type parameter must be USB or UART.\n");
						}
						else
						{
							printf ("\n  Error: Port Type parameter must be USB or UART.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Port Type parameter must be USB or UART.\n");
						return FALSE;
					}
					break;
				case COMPORT:
					if (IsComPortSet == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: COM Port parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: COM Port parameter must be specified only once.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: COM Port parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing Com port parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing Com port parameter.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Missing Com port parameter.\n");
						return FALSE;
					}
					WtpComm.SetUartPort (atoi (argv[currentOption]));
					if (atoi (argv[currentOption + 1]) >= 1 )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: COM Port parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: COM Port parameter must be specified only once.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: COM Port parameter must be specified only once.\n");
						return FALSE;
					}
					IsComPortSet = true;
					break;
				case BAUDRATE:
					if (IsBaudRateSet == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Baud Rate parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: Baud Rate parameter must be specified only once.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Baud Rate parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing Com port baudrate parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing Com port baudrate parameter.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Missing Com port baudrate parameter.\n");
						return FALSE;
					}
					DownloadBaudRate = atoi (argv[currentOption]);
					if ( (atoi (argv[currentOption + 1]) == 2400 ) ||
						 (atoi (argv[currentOption + 1]) == 4800 ) ||
						 (atoi (argv[currentOption + 1]) == 9600 ) ||
						 (atoi (argv[currentOption + 1]) == 19200 ) ||
						 (atoi (argv[currentOption + 1]) == 38400 ) ||
						 (atoi (argv[currentOption + 1]) == 57600 ) ||
						 (atoi (argv[currentOption + 1]) == 115200 ))
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Baud Rate parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: Baud Rate parameter must be specified only once.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Baud Rate parameter must be specified only once.\n");
						return FALSE;
					}					 
					if (DownloadBaudRate > 115200) DownloadBaudRate = 115200;
					WtpComm.SetBaudRate (DownloadBaudRate);
					IsBaudRateSet = true;
					break;
				case UARTDELAY:
					if (IsUartDelaySet == true )
					{
						//WTP_PRINT (hLog,"\n  Error: UART delay parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing Com port parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing Com port parameter.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Missing Com port parameter.\n");
						return FALSE;
					}
					WtpComm.SetImageSectionDelay (atoi (argv[currentOption]));
					IsUartDelaySet = true;
					break;
				case INTERFACENAME:
					if (IsInterfaceNameSet == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Interface name parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: Interface name parameter must be specified only once.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Interface name parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing interface name parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing interface name parameter.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Missing interface name parameter.\n");
						return FALSE;
					}
					WtpComm.SetInterfaceName (argv[currentOption]);
					IsInterfaceNameSet = true;
					break;
				case VERBOSE:
					bVerbose = TRUE;
					WtpComm.SetVerboseMode (bVerbose);
					break;
				case LOGFILE:
					bLogFile = TRUE;
					WtpComm.SetLogFileMode (bLogFile);
					break;
				case HELP:
					ShellExecute(NULL,"open","WTPTOOLHELP.chm",NULL,NULL,SW_SHOWNORMAL);
					break;
				case MSGMODE:
					if (IsMsgModeSet == true )
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Message Mode parameter must be specified only once.\n");
						}
						else
						{
							printf ("\n  Error: Message Mode parameter must be specified only once.\n");
						}						
						//WTP_PRINT (hLog,"\n  Error: Message Mode parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						if (bLogFile)
						{
							fprintf (hLog,"\n  Error: Missing message mode parameter.\n");
						}
						else
						{
							printf ("\n  Error: Missing message mode parameter.\n");
						}
						//WTP_PRINT (hLog,"\n  Error: Missing message mode parameter.\n");
						return FALSE;
					}
					if ( atoi (argv[currentOption]) == 1 )
					{
						currentMessageMode = DISABLED;
					}
					else if ( atoi (argv[currentOption]) == 2 )
					{
						currentMessageMode = MSG_ONLY;
					}
					else if  ( atoi (argv[currentOption]) == 3 )
					{
						currentMessageMode = MSG_AND_DOWNLOAD; 
					} 					
					WtpComm.SetMessageMode (currentMessageMode);
					IsMsgModeSet = true;
					break;
				default:
						if (bLogFile)
						{
							fprintf (hLog,"  Error: Command line option -%c is invalid!\n",Option);
						}
						else
						{
							printf ("  Error: Command line option -%c is invalid!\n",Option);
						}					
					//WTP_PRINT (hLog,"  Error: Command line option -%c is invalid!\n",Option);
					return FALSE;
			}
		}
		else
		{
			currentOption++;
		}
	}

	if (CheckYourOptions () == FALSE) return FALSE;

	return TRUE;
}

bool ParseScriptFile (char *ScriptFileName)
{
	unsigned int DownloadBaudRate = 115200;
	char DownLoadFile[MAX_PATH];
	char szMessage[100];
	char fLine[100];
	char szTemp[100];
	FILE* ScriptFile;
	int i = 0;

	ScriptFile = fopen (ScriptFileName,"r");
	if (ScriptFile == NULL)
	{
		sprintf (szMessage,"  Error: Can not open %s !",ScriptFileName);
		#ifdef _CONSOLE
			printf ("%s\n",szMessage);
		#else
			MessageBox (NULL,szMessage,"FILE OPEN ERROR",MB_ICONSTOP|MB_OK);
		#endif
		return FALSE;
	}

	if (fgets (fLine,100,ScriptFile) != NULL)
	{
		if (strstr (fLine,"[WTPDOWNLOAD]") == NULL)
		{
			printf ("////////////////////////////////////////////////\n");
			printf ("  Error: Script file, %s, is invalid!\n",ScriptFileName);
			printf ("[WTPDOWNLOAD] must be included on the first line\n");
			printf ("to insure correct file type.\n");
			printf ("////////////////////////////////////////////////\n");
			fclose (ScriptFile);
			return FALSE;
		}
	}
	else
	{
		printf ("////////////////////////////////////////////////\n");
		printf ("  Error: Script file, %s, is completely empty!\n",ScriptFileName);
		printf ("////////////////////////////////////////////////\n");
		fclose (ScriptFile);
		return FALSE;
	}

	while (fgets (fLine,100,ScriptFile) != NULL)
	{
		if (strstr (fLine,"!") != NULL)
		{
			// This must be the first check in the code. If this is
			// found anywhere in the line of the script the line is
			// considered to be commented out.
			continue;
		}

		if (strstr (fLine,"VERBOSE") != NULL)
		{
			if (strstr (fLine,"=") == NULL)
			{
				printf  ("  Error: VERBOSE declaration statement does not contain an = sign in it!\n");
				printf  ("Format: VERBOSE = TRUE \n");
				fclose (ScriptFile);
				return FALSE;
			}

			strtok (fLine,"=");
			char *IsTrue = strtok (NULL,"=");

			if (strstr (IsTrue,"TRUE") != NULL)
			{
				printf ("\nVerbose mode is enabled.\n");
				bVerbose = TRUE;
			}
			WtpComm.SetVerboseMode (bVerbose);

			continue;
		}

		if (strstr (fLine,"UARTDELAY") != NULL)
		{
			if (GetParameter (fLine,szTemp,"IMAGESECTIONDELAY",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("  Error:  IMAGESECTIONDELAY can only be set to numbers!\n");
					printf ("Format: IMAGESECTIONDELAY = 300\n");
					fclose (ScriptFile);
					return FALSE;
				}

				++i;
			}
			WtpComm.SetImageSectionDelay (atoi (szTemp));
			continue;
		}

		if (strstr (fLine,"TIM") != NULL)
		{
			if (GetParameter (fLine,DownLoadFile,"TIM",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (GetImageFile (TIMH,"TIM",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
			IsTim = TRUE;
			continue;
		}

		if (strstr (fLine,"OBM") != NULL)
		{
			if (GetParameter (fLine,DownLoadFile,"OBM",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (GetImageFile (OBMI,"OBM",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
			IsImageFile = TRUE;
			continue;
		}

		if (strstr (fLine,"KEY") != NULL)
		{
			if (GetParameter (fLine,DownLoadFile,"JTAGKEY",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (GetImageFile ((DWORD)KEY,"KEY",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
			IsJtag = TRUE;
			continue;
		}
        if (strstr (fLine,"FBF") != NULL)
        {
            if (GetParameter (fLine,DownLoadFile,"FBF",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
            if (GetImageFile (FBFI,"FBF",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
            IsFbf = TRUE;
            continue;
        }
		if (strstr (fLine,"OTPVIEW") != NULL)
		{
			if (GetParameter (fLine,szTemp,"OTPVIEW",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (strstr (szTemp,"TRUE") != NULL)
			{
				IsOtpView = TRUE;
			}
			continue;
		}

		if (strstr (fLine,"FLASHTYPE") != NULL)
		{
			if (GetParameter (fLine,szTemp,"FLASHTYPE",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("  Error:  FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
					printf ("Format: FLASHTYPE = 7\n");
					fclose (ScriptFile);
					return FALSE;
				}

				++i;
			}
			if (atoi (szTemp) >= MAXFLASHTYPES)
			{
				printf ("  Error:  FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
				fclose (ScriptFile);
				return FALSE;
			}
			WtpComm.SetFlashType (atoi (szTemp));
			continue;
		}

		if (strstr (fLine,"DEBUGBOOT") != NULL)
		{
			if (GetParameter (fLine,szTemp,"DEBUGBOOT",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (strstr (szTemp,"TRUE") != NULL)
			{
				IsDebugBoot = TRUE;
			}
			continue;
		}

		if (strstr (fLine,"PORTTYPE") != NULL)
		{
			if (GetParameter (fLine,szTemp,"PORTTYPE",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (strstr (szTemp,"USB") != NULL)
			{
				WtpComm.SetPortType (USB);
			}
			else if (strstr (szTemp,"UART") != NULL)
			{
				WtpComm.SetPortType (UART);
			}
			else
			{
				printf ("\n  Error: Port Type parameter must be USB or UART.\n");
				fclose (ScriptFile);
				return FALSE;
			}
			continue;
		}

		if (strstr (fLine,"COMPORT") != NULL)
		{
			if (GetParameter (fLine,szTemp,"COMPORT",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			WtpComm.SetUartPort (atoi (szTemp));
			continue;
		}

		if (strstr (fLine,"BAUDRATE") != NULL)
		{
			if (GetParameter (fLine,szTemp,"BAUDRATE",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("  Error:  BAUDRATE can only be set to numbers!\n");
					printf ("Format: BAUDRATE = 115200\n");
					fclose (ScriptFile);
					return FALSE;
				}

				++i;
			}
			DownloadBaudRate = atoi (szTemp);
			if (DownloadBaudRate > 115200) DownloadBaudRate = 115200;
			WtpComm.SetBaudRate (DownloadBaudRate);
			continue;
		}
	}

	fclose (ScriptFile);

	if (CheckYourOptions () == FALSE) return FALSE;

	return TRUE;
}

bool CheckYourOptions ()
{
	int nOptionsEnabled = 0;

	if (IsTim)
	{
		if (IsOtpView || IsDebugBoot  || (currentMessageMode == MSG_ONLY))
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: Cannot select OTP view, Debug Boot or Message Only\n");
				fprintf (hLog,"options with TIM file downloads!\n");

			}
			else
			{
				printf ("  Error: Cannot select OTP view, Debug Boot or Message Only\n");
				printf ("options with TIM file downloads!\n");
			}			
			//WTP_PRINT (hLog,"  Error: Cannot select OTP view, Debug Boot or Message Only\n");
			//WTP_PRINT (hLog,"options with TIM file downloads!\n");
			return FALSE;
		}
	}
	else if( currentMessageMode == MSG_ONLY )
	{
		if (IsJtag || IsOtpView || IsDebugBoot || IsTim || IsImageFile )
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: Can only select port type \n");
				fprintf (hLog,"option with Message Mode!\n");

			}
			else
			{
				printf ("  Error: Can only select port type \n");
				printf ("option with Message Mode!\n");
			}			
			//WTP_PRINT (hLog,"  Error: Can only select port type \n");
			//WTP_PRINT (hLog,"option with Message Mode!\n");
			return FALSE;
		}
	}
	else if( currentMessageMode == MSG_AND_DOWNLOAD)
	{
		if (IsJtag || IsOtpView || IsDebugBoot )
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: Cannot select JTAG reenablement, OTP view or Debug Boot \n");
				fprintf (hLog,"options with Message and Download Mode!\n");

			}
			else
			{
				printf ("  Error: Cannot select JTAG reenablement, OTP view or Debug Boot \n");
				printf ("options with Message and Download Mode!\n");
			}	
			//WTP_PRINT (hLog,"  Error: Cannot select JTAG reenablement, OTP view or Debug Boot \n");
			//WTP_PRINT (hLog,"options with Message and Download Mode!\n");
			return FALSE;
		}
	}
	else
	{
		if (IsJtag) nOptionsEnabled++;
		if (IsDebugBoot) nOptionsEnabled++;
		if (IsOtpView) nOptionsEnabled++;
		if (IsImageFile) nOptionsEnabled++;

		if (nOptionsEnabled > 1)
		{
			if (bLogFile)
			{
				fprintf (hLog,"  Error: Cannot select image file downloads, OTP view, Debug Boot\n");
				fprintf (hLog,"or JTAG reenablement options simultaneously!\n");

			}
			else
			{
				printf ("  Error: Cannot select image file downloads, OTP view, Debug Boot\n");
				printf ("or JTAG reenablement options simultaneously!\n");
			}
			//WTP_PRINT (hLog,"  Error: Cannot select image file downloads, OTP view, Debug Boot\n");
			//WTP_PRINT (hLog,"or JTAG reenablement options simultaneously!\n");
			return FALSE;
		}
	}

	return TRUE;
}

bool GetParameter (char *fLine,char *pParameter,const char *pDec,bool IsFile)
{
	char *pTemp = NULL;

	// This function is used by the ParseScriptFile function to parse out
	// values from lines that contain an equal sign. An equal sign is used as
	// part of a directive. Example directive( PORTTYPE = UART ). In this case
	// the token UART is extracted from the string to signify that the UART
	// will be used to download images.

	if (strstr (fLine,"=") == NULL)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: %s declaration statement does not contain an = sign in it!\n",pDec);
			fprintf (hLog,"Format: %s = filename \n",pDec);

		}
		else
		{
			printf ("  Error: %s declaration statement does not contain an = sign in it!\n",pDec);
			printf ("Format: %s = filename \n",pDec);
		}		
		//WTP_PRINT  (hLog,"  Error: %s declaration statement does not contain an = sign in it!\n",pDec);
		//WTP_PRINT  (hLog,"Format: %s = filename \n",pDec);
		return FALSE;
	}

	strtok (fLine,"=");
	pTemp = strtok (NULL,"=");

	if (pTemp == NULL)
	{
		if (bLogFile)
		{
			fprintf (hLog,"  Error: %s declaration statement is invalid!\n",pDec);
			fprintf (hLog,"Format: %s = filename \n",pDec);

		}
		else
		{
			printf ("  Error: %s declaration statement is invalid!\n",pDec);
			printf ("Format: %s = filename \n",pDec);
		}
		//WTP_PRINT  (hLog,"  Error: %s declaration statement is invalid!\n",pDec);
		//WTP_PRINT  (hLog,"Format: %s = filename \n",pDec);
		return FALSE;
	}

	if (IsFile)
	{
		while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
		{
			if (isprint (*pTemp) && *pTemp != ' ')
			{
				while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
				{
					if (isprint (*pTemp))
					{
						*pParameter = *pTemp;
						++pParameter;
					}
					++pTemp;
				}
				break;
			}
			++pTemp;
		}
	}
	else
	{
		while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
		{
			if (isprint (*pTemp) && *pTemp != ' ')
			{
				*pParameter = *pTemp;
				++pParameter;
			}
			++pTemp;
		}
	}
	*pParameter = '\0';

	return TRUE;
}

void PrintUsage ()
{
	if (bLogFile)
	{
		fprintf (hLog,"\n WTPTP Host Application for Downloading ");

	}
	else
	{
		printf ("\n WTPTP Host Application for Downloading ");
	}
	//WTP_PRINT (hLog,"\n WTPTP Host Application for Downloading ");
#if !(TRUSTED)
	if (bLogFile)
	{
		fprintf (hLog,"\n\n Non-trusted only version\n\n");

	}
	else
	{
		printf ("\n\n Non-trusted only version\n\n");
	}
	//WTP_PRINT(hLog,"\n\n Non-trusted only version\n\n");
#else
	if (bLogFile)
	{
		fprintf (hLog,"and Jtag Re-Enablement\n\n");

	}
	else
	{
		printf ("and Jtag Re-Enablement\n\n");
	}
	//WTP_PRINT(hLog,"and Jtag Re-Enablement\n\n");
#endif
	if (bLogFile)
	{
		fprintf (hLog,"and Jtag Re-Enablement\n\n");
		fprintf (hLog,"  Usage:\n");
		fprintf (hLog,"  ======\n");
		fprintf (hLog,"  >> WTPTP.exe <Option>|<Option file> ...\n\n");
		fprintf (hLog,"  Options:\n");
		fprintf (hLog,"  ========\n");
		fprintf (hLog,"  -V\t\tVerbose mode.  Debugging information will be printed.\n");
		fprintf (hLog,"  -f <file>\tinput any image file (binary)\n");

	}
	else
	{
		printf ("and Jtag Re-Enablement\n\n");
		printf ("  Usage:\n");
		printf ("  ======\n");
		printf ("  >> WTPTP.exe <Option>|<Option file> ...\n\n");
		printf ("  Options:\n");
		printf ("  ========\n");
		printf ("  -V\t\tVerbose mode.  Debugging information will be printed.\n");
		printf ("  -f <file>\tinput any image file (binary)\n");
	}
/*
	WTP_PRINT (hLog,"  Usage:\n");
	WTP_PRINT (hLog,"  ======\n");
	WTP_PRINT (hLog,"  >> WTPTP.exe <Option>|<Option file> ...\n\n");
	WTP_PRINT (hLog,"  Options:\n");
	WTP_PRINT (hLog,"  ========\n");
	WTP_PRINT (hLog,"  -V\t\tVerbose mode.  Debugging information will be printed.\n");
	WTP_PRINT (hLog,"  -f <file>\tinput any image file (binary)\n");
*/
#if TRUSTED
	if (bLogFile)
	{
		fprintf (hLog,"  -k <file>\tinput Jtag Key File (ASCII)\n");

	}
	else
	{
		printf ("  -k <file>\tinput Jtag Key File (ASCII)\n");
	}
	//WTP_PRINT (hLog,"  -k <file>\tinput Jtag Key File (ASCII)\n");
#endif
	if (bLogFile)
	{

		fprintf (hLog,"  -t <file>\tinput TIM file (binary)\n");
	    fprintf (hLog,"  -F <file>\tinput an FBF formatted binary file with multiple images\n");
		fprintf (hLog,"  -O flash type\tSet flash type between 0 thru 7. Runs the OTP viewer.\n");
		fprintf (hLog,"  -D Runs the Debug Boot command.\n");
		fprintf (hLog,"  -S delay time\tThe delay time value is in milliseconds. Used with UART.\n\n");
		fprintf (hLog,"  -P <port type>\t Port types: UART or USB\n");
		fprintf (hLog,"  -i Interface name. Optional. Valid only when Port type is USB.\n");
		fprintf (hLog,"  -C COM Port number. Used with UART.\n");
		fprintf (hLog,"  -B Baud rate. Used with UART.\n");
		fprintf (hLog,"  -x Script file.\n");
		fprintf (hLog,"  -h Help\n");
		fprintf (hLog,"  -M <MessageMode>  2 is MSG_ONLY, 3 is MessageMode and Download\n");
		fprintf (hLog,"  NOTE: The -f option is not compatible with the -k and -O options!\n");
	}
	else
	{
		printf ("  -t <file>\tinput TIM file (binary)\n");
	    printf ("  -F <file>\tinput an FBF formatted binary file with multiple images\n");
		printf ("  -O flash type\tSet flash type between 0 thru 7. Runs the OTP viewer.\n");
		printf ("  -D Runs the Debug Boot command.\n");
		printf ("  -S delay time\tThe delay time value is in milliseconds. Used with UART.\n\n");
		printf ("  -P <port type>\t Port types: UART or USB\n");
		printf ("  -i Interface name. Optional. Valid only when Port type is USB.\n");
		printf ("  -C COM Port number. Used with UART.\n");
		printf ("  -B Baud rate. Used with UART.\n");
		printf ("  -x Script file.\n");
		printf ("  -h Help\n");
		printf ("  -M <MessageMode>  2 is MSG_ONLY, 3 is MessageMode and Download\n");
		printf ("  NOTE: The -f option is not compatible with the -k and -O options!\n");
	}

/*
	WTP_PRINT (hLog,"  -t <file>\tinput TIM file (binary)\n");
    WTP_PRINT (hLog,"  -F <file>\tinput an FBF formatted binary file with multiple images\n");
	WTP_PRINT (hLog,"  -O flash type\tSet flash type between 0 thru 7. Runs the OTP viewer.\n");
	WTP_PRINT (hLog,"  -D Runs the Debug Boot command.\n");
	WTP_PRINT (hLog,"  -S delay time\tThe delay time value is in milliseconds. Used with UART.\n\n");
	WTP_PRINT (hLog,"  -P <port type>\t Port types: UART or USB\n");
	WTP_PRINT (hLog,"  -i Interface name. Optional. Valid only when Port type is USB.\n");
	WTP_PRINT (hLog,"  -C COM Port number. Used with UART.\n");
	WTP_PRINT (hLog,"  -B Baud rate. Used with UART.\n");
	WTP_PRINT (hLog,"  -x Script file.\n");
	WTP_PRINT (hLog,"  -h Help\n");
	WTP_PRINT (hLog,"  -M <MessageMode>  2 is MSG_ONLY, 3 is MessageMode and Download\n");
	WTP_PRINT (hLog,"  NOTE: The -f option is not compatible with the -k and -O options!\n");
*/
#if TRUSTED
	if (bLogFile)
	{
		fprintf (hLog,"        -k is for Jtag Re-Enabling. -f is for Downloading.\n");

	}
	else
	{
		printf ("        -k is for Jtag Re-Enabling. -f is for Downloading.\n");
	}
//	WTP_PRINT (hLog,"        -k is for Jtag Re-Enabling. -f is for Downloading.\n");
#endif
	if (bLogFile)
	{
		fprintf (hLog,"        -O is for the OTP viewer.\n\n");
		fprintf (hLog,"        The -t option is not compatible with the -k and -O options.\n\n");
		fprintf (hLog,"  Examples:\n");
		fprintf (hLog,"  =========\n");

	}
	else
	{
		printf ("        -O is for the OTP viewer.\n\n");
		printf ("        The -t option is not compatible with the -k and -O options.\n\n");
		printf ("  Examples:\n");
		printf ("  =========\n");
	}
/*
	WTP_PRINT (hLog,"        -O is for the OTP viewer.\n\n");
	WTP_PRINT (hLog,"        The -t option is not compatible with the -k and -O options.\n\n");
	WTP_PRINT (hLog,"  Examples:\n");
	WTP_PRINT (hLog,"  =========\n");
*/
#if TRUSTED
	if (bLogFile)
	{
		fprintf (hLog,"  To use WTPTP for Jtag Re-Enabling:\n");
		fprintf (hLog,"\t>>WTPTP.exe -k JtagOEMkeyfile.txt\n\n");

	}
	else
	{
		printf ("  To use WTPTP for Jtag Re-Enabling:\n");
		printf ("\t>>WTPTP.exe -k JtagOEMkeyfile.txt\n\n");
	}
/*
	WTP_PRINT (hLog,"  To use WTPTP for Jtag Re-Enabling:\n");
	WTP_PRINT (hLog,"\t>>WTPTP.exe -k JtagOEMkeyfile.txt\n\n");
*/
#endif
	if (bLogFile)
	{
		fprintf (hLog,"  To use WTPTP for Downloading:\n");
		fprintf (hLog,"\t>>WTPTP.exe -P USB -t TIM.bin -f OBM_h.bin\n\n");
		fprintf (hLog,"  Message-only Mode(Mode 2) Usage:\n");
		fprintf (hLog,"  ========================\n");
		fprintf (hLog,"\t>>wtptp -P USB -M 2\n\n");
		fprintf (hLog,"  Message Mode and Download(Mode 3) Usage:\n");
		fprintf (hLog,"  ========================\n");
		fprintf (hLog,"\t>>wtptp -P USB -t <NTIM/TIM.bin> -f <Image_h.bin> -M 3\n");
	}
	else
	{
		printf ("  To use WTPTP for Downloading:\n");
		printf ("\t>>WTPTP.exe -P USB -t TIM.bin -f OBM_h.bin\n\n");
		printf ("  Message-only Mode(Mode 2) Usage:\n");
		printf ("  ========================\n");
		printf ("\t>>wtptp -P USB -M 2\n\n");
		printf ("  Message Mode and Download(Mode 3) Usage:\n");
		printf ("  ========================\n");
		printf ("\t>>wtptp -P USB -t <NTIM/TIM.bin> -f <Image_h.bin> -M 3\n");
	}
/*
	WTP_PRINT (hLog,"  To use WTPTP for Downloading:\n");
	WTP_PRINT (hLog,"\t>>WTPTP.exe -P USB -t TIM.bin -f OBM_h.bin\n\n");
	WTP_PRINT (hLog,"  Message-only Mode(Mode 2) Usage:\n");
	WTP_PRINT (hLog,"  ========================\n");
	WTP_PRINT (hLog,"\t>>wtptp -P USB -M 2\n\n");
	WTP_PRINT (hLog,"  Message Mode and Download(Mode 3) Usage:\n");
	WTP_PRINT (hLog,"  ========================\n");
	WTP_PRINT (hLog,"\t>>wtptp -P USB -t <NTIM/TIM.bin> -f <Image_h.bin> -M 3\n");
*/
#if TRUSTED
	if (bLogFile)
	{
		fprintf (hLog,"  Download of TIM-JTAG Reenable-Image Usage:\n");
		fprintf (hLog,"  ========================\n");
		fprintf (hLog,"\t>>wtptp -P UART -t <NTIM/TIM.bin> -f <Image_h.bin> -k JtagOEMKeyfile.txt\n");
	}
	else
	{
		printf ("  Download of TIM-JTAG Reenable-Image Usage:\n");
		printf ("  ========================\n");
		printf ("\t>>wtptp -P UART -t <NTIM/TIM.bin> -f <Image_h.bin> -k JtagOEMKeyfile.txt\n");
	}
/*
	WTP_PRINT (hLog,"  Download of TIM-JTAG Reenable-Image Usage:\n");
	WTP_PRINT (hLog,"  ========================\n");
	WTP_PRINT (hLog,"\t>>wtptp -P UART -t <NTIM/TIM.bin> -f <Image_h.bin> -k JtagOEMKeyfile.txt\n");
*/
#endif
}

