/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
** This software file (the "File") is distributed by Marvell International  
** Ltd. under the terms of the GNU General Public License Version 2, June  
** 1991 (the "License").  You may use, redistribute and/or modify this File   
** in accordance with the terms and conditions of the License, a copy of  
** which is available along with the File in the license.txt file or by   
** writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
** Boston, MA 02111-1307 or on the worldwide web at 
** http://www.gnu.org/licenses/gpl.txt.
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
** details about this warranty disclaimer. 
**
**  FILENAME:	WTPUARTLIB.c
**
**  PURPOSE: 	Windows Dll routines 
**                  
******************************************************************************/

#pragma warning ( disable : 4244 4715 4047 4024) // Disable warning messages

#include <time.h>
#include <stdio.h>
#include <windows.h>
#include <math.h>
#include "WTPUARTLIB.h"

HANDLE hUart = INVALID_HANDLE_VALUE;
DCB Dcb;
COMMTIMEOUTS CommTimeOuts = { MAXDWORD,1000,0,1000,0 };
DWORD BytesWritten, BytesRead;
DWORD CommEventMask;
OVERLAPPED Ovl;
COMSTAT ComStat;
BOOL UARTDEBUG = FALSE;
BOOL EnableWaitEvent = TRUE;
COMMCONFIG CommConfig;

WTPUARTLIB_API BOOL OpenUartPort (UartSettings ComPortConfig)
{
	char pCom[6];
	char Message[100];
	DWORD CommConfigSize;

	sprintf (pCom,"COM%d",ComPortConfig.iPortNumber);

	hUart = CreateFile (pCom,GENERIC_READ | GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);

	if (hUart == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: OpenUartPort failed with error %d for %s.\n",GetLastError(),pCom);
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	if (GetCommState (hUart,&Dcb) == NULL)
	{
		sprintf (Message,"  Error: GetCommState failed with error %d for %s.\n",GetLastError(),pCom);
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	Dcb.EvtChar          = '\001'; // Probably won't be used with new protocal.
	Dcb.fNull            = FALSE;
	Dcb.fInX             = FALSE;
	Dcb.fOutX            = FALSE;
	Dcb.fDsrSensitivity  = FALSE;
	Dcb.fOutxDsrFlow     = FALSE;
	Dcb.fOutxCtsFlow     = FALSE;
	Dcb.fDtrControl      = DTR_CONTROL_DISABLE;
	Dcb.fRtsControl      = RTS_CONTROL_DISABLE;
	Dcb.StopBits         = ONESTOPBIT;
	Dcb.Parity           = NOPARITY;
	Dcb.ByteSize         = 8;
	// baud rate wasn't specified so auto-detect the baud rate.
//	if ( ComPortConfig.dwBaudRate == 0 )
//	{
//		CommConfigSize = sizeof(COMMCONFIG);
		
//		if (GetCommConfig (hUart,&CommConfig,&CommConfigSize ) == NULL)
//		{
//			sprintf (Message,"GetCommConfig failed with error %d for %s.\n",GetLastError(),pCom);
//			MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
//			return FALSE;
//		}
//		Dcb.BaudRate		 = CommConfig.dcb.BaudRate;
//	}
//	else
		Dcb.BaudRate         = ComPortConfig.dwBaudRate;

	Dcb.fAbortOnError    = TRUE;   
	Dcb.fParity          = FALSE;
	Dcb.fBinary          = TRUE;
	Dcb.fNull            = FALSE;

	if (SetCommState (hUart,&Dcb) == NULL)
	{
		sprintf (Message,"  Error: SetCommState failed with error %d for %s.\n",GetLastError(),pCom);
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	if (SetCommMask (hUart,EV_RXCHAR) == NULL)
	{
		sprintf (Message,"  Error: SetCommMask failed with error %d for %s.\n",GetLastError(),pCom);
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	if (SetupComm (hUart,4352,4352) == NULL)//4096
	{
		sprintf (Message,"  Error: SetupComm failed with error %d for %s.\n",GetLastError(),pCom);
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	if (SetCommTimeouts (hUart,&CommTimeOuts) == NULL)
	{
		sprintf (Message,"  Error: SetCommTimeouts failed with error %d for %s.\n",GetLastError(),pCom);
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}
	
	return TRUE;
}

WTPUARTLIB_API BOOL WriteUart (WTPCOMMAND WtpCmd,DWORD dwSize)
{
	char Message[100];

	if (hUart == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: OpenUartPort must be called prior to calling WriteUart!");
		#ifdef _CONSOLE
			printf ("%s\n",Message);
		#else
			MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		#endif
		return FALSE;
	}

	if (WriteFile (hUart,&WtpCmd,dwSize,&BytesWritten,NULL) == NULL)
	{
		sprintf (Message,"  Error: WriteFile failed with error %d.\n",GetLastError());
		if (UARTDEBUG) printf ("%s\n",Message);
		return FALSE;
	}

	return TRUE;
}

WTPUARTLIB_API BOOL ReadUart (WTPSTATUS *WtpStatus,DWORD dwSize,PDWORD pBytesRead)
{
	char Message[100];
	DWORD ErrorFlags;
	DWORD Mask;

	if (hUart == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: OpenUartPort must be called prior to calling ReadUart!");
		#ifdef _CONSOLE
			printf ("%s\n",Message);
		#else
			MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		#endif
		return FALSE;
	}

	if (EnableWaitEvent) WaitCommEvent (hUart,&Mask,NULL);
	if (ReadFile (hUart,WtpStatus,dwSize,&BytesRead,NULL) == NULL)
	{
		sprintf (Message,"  Error: ReadFile failed with error %d.\n",GetLastError());
		if (UARTDEBUG) printf ("%s\n",Message);
		if (ClearCommError (hUart,&ErrorFlags,&ComStat) == NULL)
		{
			sprintf (Message,"  Error: ClearCommError failed with error %d.\n",GetLastError());
			if (ISMSGPORT) 
				MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		}
		return FALSE;
	}
	if (UARTDEBUG) printf ("\nBytes read: %X Bytes expected: %X\n",BytesRead,dwSize);

	*pBytesRead = BytesRead;
	if (BytesRead != dwSize) return FALSE;
	
	return TRUE;
}

WTPUARTLIB_API BOOL ReadUartCharBuffer (char *WtpMessage,DWORD dwSize,PDWORD pBytesRead)
{
	char Message[100];
	DWORD ErrorFlags;

	if (hUart == INVALID_HANDLE_VALUE) 
	{
		sprintf (Message,"  Error: OpenUartPort must be called prior to calling ReadUart!");
		#ifdef _CONSOLE
			printf ("%s\n",Message);
		#else
			MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		#endif
		return FALSE;
	}

	if (ReadFile (hUart,WtpMessage,dwSize,&BytesRead,NULL) == NULL)
	{
		sprintf (Message,"  Error: ReadFile failed with error %d.\n",GetLastError());
		if (UARTDEBUG) printf ("%s\n",Message);
		if (ClearCommError (hUart,&ErrorFlags,&ComStat) == NULL)
		{
			sprintf (Message,"  Error: ClearCommError failed with error %d.\n",GetLastError());
			if (ISMSGPORT) 
				MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		}
		return FALSE;
	}
	if (ISMSGPORT) printf ("\nBytes read: %X Bytes expected: %X\n",BytesRead,dwSize);

	*pBytesRead = BytesRead;
	if (BytesRead != dwSize) return FALSE;
	
	return TRUE;
}

WTPUARTLIB_API BOOL UartPortPriority (int iPriority,DWORD dwPriorityClass)
{
	char Message[100];

	if (SetThreadPriority (GetCurrentThread (),iPriority) == NULL)
	{
		sprintf (Message,"  Error: SetThreadPriority failed!\n");
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	if (SetPriorityClass (GetCurrentProcess (),dwPriorityClass) == NULL)
	{
		sprintf (Message,"  Error: SetPriorityClass failed!\n");
		MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	return TRUE;
}

WTPUARTLIB_API void EnableUartVerboseMode (BOOL OnOff)
{
	UARTDEBUG = OnOff;
}

WTPUARTLIB_API void EnableWaitCommEvent (BOOL OnOff)
{
	EnableWaitEvent = OnOff;
}

WTPUARTLIB_API void CloseUartPort ()
{
	if (hUart != INVALID_HANDLE_VALUE) CloseHandle (hUart);
}
