/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
** This software file (the "File") is distributed by Marvell International  
** Ltd. under the terms of the GNU General Public License Version 2, June  
** 1991 (the "License").  You may use, redistribute and/or modify this File   
** in accordance with the terms and conditions of the License, a copy of  
** which is available along with the File in the license.txt file or by   
** writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
** Boston, MA 02111-1307 or on the worldwide web at 
** http://www.gnu.org/licenses/gpl.txt.
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
** details about this warranty disclaimer. 
**
**  FILENAME:	WTPUARTLIB.c
**
**  PURPOSE: 	Windows Dll routines 
**                  
******************************************************************************/

#pragma warning ( disable : 4244 4715 4047 4024) // Disable warning messages

#include <time.h>
#include <stdio.h>
#include <math.h>
#include "../Include/Typedef.h"
#include "WTPUARTLIB.h"
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/signal.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/errno.h>

#define MODEMDEVICE "/dev/ttyS0"

int hUart;
DWORD BytesWritten, BytesRead;
DWORD CommEventMask;
BOOL UARTDEBUG = TRUE;
BOOL EnableWaitEvent = TRUE;
struct stat stt;
int portuid, portgid, real_uid, real_gid;

BOOL OpenUartPort (UartSettings ComPortConfig)
{
	struct termios Dcb;
	char Message[100];

	real_uid = getuid();
	real_gid = getgid();

	if (real_uid !=0)
		chown("/dev/ttyS0",(uid_t)real_uid,(gid_t)real_gid);

	hUart = open("/dev/ttyS0", O_RDWR | O_NOCTTY);

	if ( hUart < 0 )
	{
		printf("  Error: OpenUARTPort failed\n");
		return FALSE;
	}
	
	tcgetattr(hUart, &Dcb);

	// Set baud rate to 115200
	cfsetispeed(&Dcb, B115200);
	cfsetospeed(&Dcb, B115200);
	
    // 8 data bits
	Dcb.c_cflag = (Dcb.c_cflag & ~CSIZE) | CS8;

	// Raw no echo mode
    Dcb.c_iflag = IGNBRK;
	Dcb.c_lflag = 0;
	Dcb.c_oflag = 0;
	Dcb.c_cflag |= CLOCAL | CREAD;

	// No HW flow control
	Dcb.c_cflag &= ~CRTSCTS;
	
	Dcb.c_cc[VMIN] = 1; //Read at least 1 byte
	Dcb.c_cc[VTIME]=10; //Timeout 1 sec
	
	Dcb.c_iflag &= ~(IXON|IXOFF|IXANY);

	// No parity
	Dcb.c_cflag &= ~(PARENB | PARODD);

	// Stop bits
	Dcb.c_cflag &= ~CSTOPB;
	
	tcsetattr(hUart,TCSANOW, &Dcb);

	return TRUE;
}

BOOL WriteUart(WTPCOMMAND WtpCmd, DWORD dwSize)
{
	char Message[100];
	BYTE Cmd[4];

	if (hUart < 0)
	{
		printf("  Error: OpenUartPort must be called prior to caling WriteUart!\n");
		return FALSE;
	}

	if (write(hUart, &WtpCmd, dwSize)== 0 )
	{
		sprintf (Message, "  Error: WriteFile failed\n");
		if (UARTDEBUG) printf("%s\n",Message);
		return FALSE;
	}
	return TRUE;
}
BOOL ReadUart(WTPSTATUS *WtpStatus, DWORD dwSize, PDWORD pBytesRead)
{
	char Message[100];
	DWORD ErrorFlags;
	DWORD Mask;

	if (hUart < 0)
	{
		printf ("  Error: OpenUartPort must be called prior to calling ReadUart!");
		return FALSE;
	}

	BytesRead = read(hUart, WtpStatus, dwSize);
	if ( BytesRead == -1 )
	{
		printf("  Error: ClearComError failed\n");
		if (UARTDEBUG) printf("%s\n", Message);
		return FALSE;
	}
	if (UARTDEBUG) printf ("\nBytes read: %X Bytes expected: %X\n", BytesRead, dwSize);

	*pBytesRead = BytesRead;
	if (BytesRead != dwSize) return FALSE;

	return TRUE;
}

BOOL ReadUartCharBuffer (char *WtpMessage,DWORD dwSize,PDWORD pBytesRead)
{
	char Message[100];
	DWORD ErrorFlags;

	if (hUart < 0) 
	{
		printf ("  Error: OpenUartPort must be called prior to calling ReadUart!");
		return FALSE;
	}
	BytesRead = read(hUart, WtpMessage, dwSize);
	if (BytesRead == -1 )
	{
		printf ("  Error: ReadFile failed with error\n");
		if (UARTDEBUG) printf ("%s\n",Message);
//		if (ClearCommError (hUart,&ErrorFlags,&ComStat) == NULL)
//		{
//			printf ("  Error: ClearCommError failed with error %d.\n",GetLastError());
//			if (ISMSGPORT) 
//				MessageBox (NULL,Message,"SERIAL COMMUNICATIONS ERROR",MB_ICONSTOP|MB_OK);
//		}
		return FALSE;
	}
//	if (ISMSGPORT) printf ("\nBytes read: %X Bytes expected: %X\n",BytesRead,dwSize);

	*pBytesRead = BytesRead;
	if (BytesRead != dwSize) return FALSE;
	
	return TRUE;
}

void EnableUartVerboseMode (BOOL OnOff)
{
	UARTDEBUG = OnOff;
}


void EnableWaitCommEvent (BOOL OnOff)
{
	EnableWaitEvent = OnOff;
}

void CloseUartPort()
{
	if (hUart != 0) close(hUart);
}
