/**************************************************************************
**
** (C) Copyright December 2006 Marvell International Ltd.
**
** All Rights Reserved.
**
** This software file (the "File") is distributed by Marvell International  
** Ltd. under the terms of the GNU General Public License Version 2, June  
** 1991 (the "License").  You may use, redistribute and/or modify this File   
** in accordance with the terms and conditions of the License, a copy of  
** which is available along with the File in the license.txt file or by   
** writing to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   
** Boston, MA 02111-1307 or on the worldwide web at 
** http://www.gnu.org/licenses/gpl.txt.
**
** THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE   
** IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR  
** PURPOSE ARE EXPRESSLY DISCLAIMED.  The License provides additional
** details about this warranty disclaimer. 
**
**  FILENAME:	WtpDownload.cpp
**
**  PURPOSE: 	Provides download capabilities either thru the USB port or  
** 				the UART port. This is the command line version.
**                  
******************************************************************************/

#if WINDOWS
#include "..\Common\WTPCOMMCLASS\WTPCOMMCLASS.h"
#include "..\Common\WTPSECURITYLIB\WTPSECURITYLIB.h"
#include "..\Common\Include\FlashCodes.h"
#endif

#if LINUX
	#include "../Common/Include/Typedef.h"
	#include <unistd.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <ctype.h>
	#include "../Common/WTPCOMMCLASS/WTPCOMMCLASS.H"
	#include "../Common/WTPSECURITYLIB/WTPSECURITYLIB.h"
	#include "../Common/Include/FlashCodes.h"
#endif

//#if WINDOWS
//#endif

#include "WtpDownload.h"

#pragma warning ( disable : 4800 ) // Disable warning messages

#define SLE_TESTING 0

#define MIN_ARGS 2

char Version[11] = "3.09";
char Date[11] = "10192007";

CWtpComm WtpComm;
int FileIndex = 0;

#if LINUX
IMAGEFILES ImageFiles[MAXIMAGEFILES];
bool bVerbose;
bool bDebug;
bool IsTim;
bool IsOtpView;
bool IsDebugBoot;
bool IsJtag;
bool IsImageFile;
MESSAGE_MODES currentMessageMode;
#endif

bool IsScriptFile (int argc,char* argv[],char *ScriptFileName,bool *IsScript);
bool GetImageFile (DWORD ImageType,char *TypeName,BYTE Option,int Argc,char *ImageFile);
bool ParseCommandLine (int argc,char* argv[]);
bool ParseScriptFile (char *ScriptFileName);
bool GetParameter (char *fLine,char *pParameter,const char *pDec,bool IsFile);
bool CheckYourOptions ();
BYTE AckWithImageName (DWORD ImageType,TCHAR *szFileName);
bool DownloadImage (int iFile);

#if LINUX
extern bool ParseTim(IMAGEFILES *pImageFiles, int nFiles, int *pNumberOfImages);
#endif

#if TRUSTED
bool JtagReEnable ();
#endif
bool OtpView ();
bool DebugBoot ();
void PrintUsage ();

int main (int argc,char *argv[])
{
	int currentOption = 1, i, nImages;
	bool bScript = FALSE;
	char ScriptFileName[MAX_PATH];
	bVerbose = FALSE;
	bDebug = FALSE;
	IsTim = FALSE;
	IsOtpView = FALSE;
	IsDebugBoot = FALSE;
	IsJtag = FALSE;
	IsImageFile = FALSE;
	WTPSTATUS *pWtpStatus = new WTPSTATUS;
	bool IsOk = true;
	currentMessageMode = DISABLED;
	WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00};
	bool Connected = FALSE;

	printf ("\n\nWTPTP Tool Version: %s\n", Version);
	printf ("WTPTP Tool Date   : %s\n\n", Date);

	if (argc < MIN_ARGS)
	{
		PrintUsage ();
		return FALSE;
	}

	if (IsScriptFile (argc,argv,ScriptFileName,&bScript) == FALSE) return FALSE;

	if (bScript)
	{
		if (ParseScriptFile (ScriptFileName) == FALSE) return FALSE;
	}
	else
	{
		if (ParseCommandLine (argc,argv) == FALSE) return FALSE;
	}

	if (bVerbose == false )
	{
		printf("Verbose mode disabled...\n");
	}

	if (IsTim)
	{
		if (ParseTim (&ImageFiles[0],FileIndex,&nImages) == FALSE) return FALSE;
		FileIndex = nImages; // Just in case user supplies more files than required
	}

	if (!IsOtpView && !IsJtag && (currentMessageMode == MSG_AND_DOWNLOAD))
		if (FileIndex == 0) { printf ("  Error: Did not supply any files for download!\n"); return FALSE; }

	if (WtpComm.GetPortType() == UART) printf ("\nPort Selected: UART\n");
	else if (WtpComm.GetPortType() == USB) printf ("\nPort Selected: USB\n");
	else { printf ("\n  Error: Port Selected: UNDEFINED\n"); return FALSE; }

	if (WtpComm.GetPortType() == USB)
	{
		WtpComm.SetUsbPortTimeout (30); // Default = 0
	}
	
	if (WtpComm.OpenPort() == FALSE) return FALSE;
	
	if (IsOtpView) { printf ("\nOTP Viewer Mode\n"); OtpView (); }
#if TRUSTED
	else if(IsTim && IsJtag && IsImageFile)
	{
		printf ("\nNumber of files to download: %d\n",FileIndex);

		for (i = 0; i < FileIndex; i++)
		{
			if (DownloadImage (i) == FALSE)
			{
				printf ("\n  Error: Download failed for file number: %d\n",i + 1);
				IsOk = false;
				break;
			}
			else
				printf ("  Success: Download file complete!\n",i + 1);
			// After downloading TIM, allow for a JTAG reenablement.
			if ( i == 0 )
			{
				printf ("\nJTAG Reenablement Mode\n"); 
				if(JtagReEnable () == FALSE)
				{
					if (bVerbose) printf ("\nSending Done...\n");
					WtpComm.Done();
				}
			}
		}
	}
	else if (IsJtag)
	{ 
		printf ("\nJTAG Reenablement Mode\n"); 
		if(JtagReEnable () == FALSE)
		{	
			if (bVerbose) printf ("\nSending Done...\n");
			WtpComm.Done();
		}
	}
#endif
	else if (IsDebugBoot) { printf ("\nDebug Boot Mode\n"); DebugBoot (); }
            else if (currentMessageMode == MSG_ONLY )

            {
                printf("******************************\n");
                printf("*****Message mode enabled*****\n");
                printf("******************************\n");
                if (bVerbose) printf ("\nSending preamble...\n");
                while (TRUE){
                    if (Connected == FALSE){
                        if (WtpComm.SendPreamble() == false)
                        {
							#if LINUX
				    		sleep (1);
							#endif
							#if WINDOWS
				    		Sleep (1);
							#endif

                            continue;
                        }
                        else
                        {
							Connected = TRUE;
                        }
                    }
                    WtpStatus.Status = ACK;
                    for(;;)
                    {
                        if ( (WtpStatus.Flags & 0x1) == MESSAGEPENDING ) 
                        {                                                            
                            WtpComm.GetTargetMessage (&WtpStatus); 
                        }
                        else
                        {
                            WtpComm.Disconnect(pWtpStatus);
                            if (pWtpStatus->Status == ACK)
                            {
                                WtpComm.ClosePort();
                                return TRUE;
                            }
                        }
                    }
                }
            }

	else
	{
		printf ("\nNumber of files to download: %d\n",FileIndex);

		for (i = 0; i < FileIndex; i++)
		{

			if (DownloadImage (i) == FALSE)
			{
				printf ("\n  Error: Download failed for file number: %d\n",i + 1);
				IsOk = false;
				break;
			}
			else
			{
				printf ("  Success: Download file complete!\n",i + 1);
			}
//		#if SLE_TESTING

//		if(i==0)
//		#if LINUX
//		sleep(12000);
//		#endif
//		#if WINDOWS
//		Sleep(12000);
//		#endif

//		#endif
		}
	}
	if ( currentMessageMode == DISABLED || currentMessageMode == MSG_AND_DOWNLOAD )
	{
		if(IsOtpView || IsDebugBoot)
			;
		else
		do{
			WtpComm.Disconnect(pWtpStatus);
		}while(pWtpStatus->Status == NACK);
		WtpComm.ClosePort();
	}

	return TRUE;
}

bool DownloadImage (int iFile)
{
	WTPCOMMAND WtpCmd;
	WTPSTATUS WtpStatus = {0x00, 0x00, 0x00, 0x00, 0x01, 0x00};
	DWORD ImageType;
	BYTE IsImage = NACK;
	unsigned int uiRemainingBytes = 0xAA55FF00;
	TCHAR szFileName[MAX_PATH];
	long lFileSize = 0,BytesRead;
	FILE* hFile;
	int i = 0;

	if (bVerbose) printf ("\nSending preamble...\n");

	#if LINUX
	sleep(3);	  //needed for DKB to initialize flash - mdb
	#endif
	if (WtpComm.SendPreamble() == false) return FALSE;

	if ( currentMessageMode == MSG_AND_DOWNLOAD )
	{
		WtpStatus.Status = ACK;
		WtpComm.GetTargetMessage(&WtpStatus);
		WtpStatus.Flags = 0x0;
	}
	if (bVerbose) printf ("\nSending GetVersion command...\n");
	if (WtpComm.GetVersion(&WtpStatus) == false) return FALSE;
	
	if (bVerbose) printf ("\nSending SelectImage command...\n");
	if (WtpComm.SelectImage(&ImageType) == false) return FALSE;
	if (bVerbose) printf ("\nImage Type: %X",ImageType);

	if (!IsTim) ImageType = OBMI + iFile;
	IsImage = AckWithImageName(ImageType,szFileName);

	if (IsImage == ACK) { if (bVerbose) printf ("\nDownloading file: %s\n",szFileName); }
	else
	{
		printf ("\n  Error: Don't have image type asked for by target: %X\n",ImageType);
	}

	if (bVerbose) printf ("\nSending VerifyImage command...\n");
	if (WtpComm.VerifyImage(IsImage) == false)
	{
		printf ("\n  Error: Download file number %d has aborted!\n",iFile + 1);
		WtpComm.Done();
		return FALSE;
	}

	if ((hFile = fopen (szFileName,"rb")) == NULL)
	{
		printf ("  Error: fopen failed to open %s.",szFileName);
		return FALSE;
	}

	fseek (hFile,0L,SEEK_END); // Set position to EOF
	lFileSize = ftell (hFile); // Get position of file, thus the file size.

	if (ImageType == TIMH)
	{
		fseek (hFile,0L,SEEK_SET); // Set position to SOF
		uiRemainingBytes = (unsigned int)lFileSize;
	}
	else
	{
		fseek (hFile,4L,SEEK_SET); // Set position to SOF
		uiRemainingBytes = (unsigned int)lFileSize - 4;
	}

	while (uiRemainingBytes > 0)
	{
		if (bVerbose) printf ("\nSending DataHeader command...\n");
		if (WtpComm.DataHeader(uiRemainingBytes,&WtpStatus) == false)
		{
			printf ("  Error: WtpComm.DataHeader failed to receive an ACK!");
			if (hFile != NULL) fclose (hFile);
			return FALSE;
		}

		unsigned int uiBufferSize = 0;
		for (int i = 0; i < (int)WtpStatus.DLEN; i++)
		{
			uiBufferSize |= WtpStatus.Data[i] << (8 * i);
		}
	
		if(uiBufferSize > uiRemainingBytes) uiBufferSize = uiRemainingBytes;
		uiRemainingBytes -= uiBufferSize; // Calculate remaining bytes
		if (bVerbose) 
			printf ("Buffer size = %u, Bytes remaining = %u\n",uiBufferSize,uiRemainingBytes);

		if ((BytesRead = fread (&WtpCmd.Data,1,uiBufferSize,hFile)) != (long)uiBufferSize)
		{
			printf ("  Error: fread only read %ld, should've read %ld",BytesRead,uiBufferSize);
			if (hFile != NULL) fclose (hFile);
			return FALSE;
		}
		else
		{
			#if SLE_TESTING
			short totalSize=0;
			const short PACKET_SIZE = 512;
	        	if (uiBufferSize > PACKET_SIZE )
			{
				totalSize = uiBufferSize;
				uiBufferSize = PACKET_SIZE;
					do
					{				    
						if (bVerbose) printf ("\nSending data block with buffer size of %d...\n",uiBufferSize);	
						if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
						{
							printf ("  Error: WtpComm.Data failed to receive an ACK!");
							if (hFile != NULL) fclose (hFile);
							return FALSE;
						}
						totalSize-=uiBufferSize;
//					#if WINDOWS
					// Sleep for 1 minute
//					Sleep (90000);
//					#endif
//					#if LINUX
					// Sleep for 1 minute
//					sleep (90000);
//					#endif
					}while (totalSize > 0);
			}
			else
			{
				if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
				{
					printf ("  Error: WtpComm.Data failed to receive an ACK!");
					if (hFile != NULL) fclose (hFile);
					return FALSE;
				}
			}
			
			#else
			if (WtpComm.GetPortType() == USB)
			{
			    unsigned int totalSize = 0;
			    unsigned int mark = 0;
			    const unsigned int PACK_SIZE = 504;
			    if (uiBufferSize > PACK_SIZE)
			    {
				totalSize = uiBufferSize;
				uiBufferSize = PACK_SIZE;	
				do
				{	    				
				    if (bVerbose) printf ("\nSending data block with buffer size of %d...\n",uiBufferSize); 
				    if (WtpComm.Data(&(WtpCmd.Data[mark]),uiBufferSize) == false)				
				    {				
					printf ("  Error: WtpComm.Data failed to receive an ACK!");					
					if (hFile != NULL) fclose (hFile);					
					return FALSE;					
				    }				
				    mark += uiBufferSize;				
				    totalSize-=uiBufferSize;				
				    if (totalSize <= uiBufferSize)				
				    {			    
					uiBufferSize = totalSize;					
				    }
										
				}while (totalSize > 0);
			
			    }		
			    else
			    {
				if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
				{					
				    printf ("  Error: WtpComm.Data failed to receive an ACK!");			    
				    if (hFile != NULL) fclose (hFile);					
				    return FALSE;
				}			
			    }
			}
			else if (WtpComm.GetPortType() == UART)
			{
			    if (bVerbose) printf ("\nSending data block with buffer size of %d...\n",uiBufferSize);
			    if (WtpComm.Data(WtpCmd.Data,uiBufferSize) == false)
			    {
				printf ("  Error: WtpComm.Data failed to receive an ACK!");
				if (hFile != NULL) fclose (hFile);
				return FALSE;
			    }
			}
			#endif
		}
	}
	if (bVerbose) printf ("\nSending Done command...\n");
	WtpComm.Done();

	if (hFile != NULL) fclose (hFile);

	return TRUE;
}

bool OtpView ()
{
	BYTE FT = WtpComm.GetFlashType ();

	if (FT >= MAXFLASHTYPES) return FALSE;
	printf ("Flash type selected: %s\n",FlashTypeCodes[FT].Description);

	if (FT == 0 || FT == 1 || FT == 2 || FT == 4 || FT == 6)
	{
		printf ("  Error: OTP registers are not available for NAND flash types!\n");
		return TRUE;
	}

	printf ("\nSending preamble...\n");
	if (WtpComm.SendPreamble() == false) return FALSE;

	printf ("\nSending OtpView command...\n");
	if (WtpComm.OtpView() == false) return FALSE;

	return TRUE;
}

bool DebugBoot ()
{
	BYTE FT = WtpComm.GetFlashType ();

	if (FT >= MAXFLASHTYPES) return FALSE;
	printf ("Flash type selected: %s\n",FlashTypeCodes[FT].Description);

	printf ("\nSending preamble...\n");
	if (WtpComm.SendPreamble() == false) return FALSE;

	printf ("\nSending DebugBOOT command...\n");
	if (WtpComm.DebugBoot(&FT) == false) return FALSE;

	if (bVerbose) printf ("\nSending Done command...\n");
	WtpComm.Done();

	return TRUE;
}
#if TRUSTED
bool JtagReEnable ()
{
	WTPCOMMAND WtpCmd;
	WTPSTATUS *pWtpStatus = NULL;
	pWtpStatus = new WTPSTATUS;
	TCHAR szFileName[MAX_PATH];
	pKeyInfo pKey = new KeyInfo;
	FILE *KeyFile, *file;
	int ErrorStatus;
	unsigned int *t1, t2;
	BYTE Password[256];
	BYTE DigitalSignature[256];

	if (AckWithImageName(KEY,szFileName) == NACK)
	{
		printf ("  Error: Did not supply KEY file for JTAG re-enablement!");
		return FALSE;
	}

	if ((KeyFile = fopen (szFileName,"r")) == NULL)
	{
		printf ("  Error: fopen failed to open key file %s.",szFileName);
		exit(0);
		return FALSE;
	}

	ErrorStatus = ReadKeyFile ( KeyFile,pKey->PublicKey,pKey->PrivateKey,
								pKey->Modulus,&pKey->KeyLength );
	if (ErrorStatus != NoError)
	{
		printf ("  Error: ReadKeyFile failed while processing file %s.!",szFileName);
		fclose (KeyFile);
		return FALSE;
	}

	memcpy (&WtpCmd.Data[0],pKey->PublicKey,4 * pKey->KeyLength);
	memcpy (&WtpCmd.Data[4 * pKey->KeyLength],pKey->Modulus,4 * pKey->KeyLength);
	fclose (KeyFile);

	if (bVerbose) printf ("\nSending preamble...\n");
	if (WtpComm.SendPreamble() == false) return FALSE;

	if (bVerbose) printf ("\nSending Public Key...\n");
	if (WtpComm.PublicKey() == false) return FALSE;

	if (bVerbose) printf ("\nSending Data...\n");
	if (WtpComm.Data(WtpCmd.Data,8 * pKey->KeyLength) == false) return FALSE;

	if (bVerbose) printf ("\nSending Password...\n");
	if (WtpComm.Password(pWtpStatus) == false) return FALSE;
	memcpy (&Password,pWtpStatus->Data,8);

	printf("Password: %08x %08x\n", (unsigned int)*((unsigned int*)&Password[0]), 
		(unsigned int)*((unsigned int*)&Password[4]));
//	PadPasswordToSign ((unsigned int *)Password,pKey->KeyLength);

	RSA (   (unsigned int *)pKey->PrivateKey,
			(unsigned int *)pKey->Modulus,
			pKey->KeyLength,
			(unsigned int *)Password,
			(unsigned int *)DigitalSignature   );

	memcpy (&WtpCmd.Data[0],DigitalSignature,4 * pKey->KeyLength);

	printf("Signed Password:\n");	
	for(int c = 0; c < pKey->KeyLength * 4; c+=8)
		printf("%08x %08x \n", (unsigned int)*((unsigned int *)&DigitalSignature[c]),
		(unsigned int)*((unsigned int *)&DigitalSignature[c+4]));

	if (bVerbose) printf ("\nSending Signed Password...\n");
	if (WtpComm.SignedPassword() == false) return FALSE;

	if (bVerbose) printf ("\nSending Data...\n");
	if (WtpComm.Data(WtpCmd.Data,4 * pKey->KeyLength) == false) return FALSE;

	if (bVerbose) printf ("\nSending Done...\n");
	WtpComm.Done();

	return TRUE;
}
#endif
BYTE AckWithImageName (DWORD ImageType,TCHAR *szFileName)
{
	BYTE IsImage = NACK;

	for (int i = 0; i < MAXIMAGEFILES; i++)
	{
		if (ImageFiles[i].FileType == 0) continue;
		if (ImageFiles[i].FileType == ImageType)
		{
			IsImage = ACK;
			strcpy (szFileName,ImageFiles[i].FileName);
			break;
		}
	}

	return IsImage;
}

bool IsScriptFile (int argc,char *argv[],char *ScriptFileName,bool *IsScript)
{
	int currentOption = 1;
	*IsScript = false;
	bool optionIsCommandLine = false;

	// This function will determine whether or not a script file is being used
	// to provision the program by looking for a (-x script.ini) parameter in
	// the command line.

	while (currentOption < argc)
	{
		if (*argv[currentOption] == '-' || *argv[currentOption] == '/')
		{
			if (*++argv[currentOption] == 'x')
			{
				if (++currentOption == argc)
				{
					printf ("\n  Error: Missing input script file.\n");
					return FALSE;
				}
				while (*argv[currentOption] != NULL)
				{
					*ScriptFileName++ = *argv[currentOption]++;
				}
				*ScriptFileName = '\0';
				*IsScript = true;
				//break;
			}
			else
			{
				optionIsCommandLine = true;
			}
			--argv[currentOption++];
		}
		else
		{
			currentOption++;
		}
	}

	if (optionIsCommandLine == true && *IsScript == true )
	{
		printf("  Error: Cannot select script file with other command line options\n");
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

bool GetImageFile (DWORD ImageType,char *TypeName,BYTE Option,int Argc,char *ImageFile)
{
	// This function is used by ParseCommandLine (...) and ParseScriptFile (...) to
	// obtain image file names as supplied by the user. It will allow only one TIM
	// file to be loaded. It will accept multiple OBM files. If the TIM option is 
	// selected the ParseTim (...) function will populate the ImageFiles[i].FileType
	// for each image from the 32 bit ImageID header of that image. If the TIM option
	// is not selected the FileType defaults, as populated by this function, will be
	// used. Having said that this utility has only been documented to be used in 
	// conjunction with the TIM and it's associated images.

	if (Option == Argc)
	{
		printf ("\n  Error: Missing %s file name parameter!\n",TypeName);
		return FALSE;
	}

	if (ImageType == TIMH) // Can only load 1 TIM file
	{
		for (int i = 0; i < MAXIMAGEFILES; i++)
		{
			if (ImageFiles[i].FileType == ImageType)
			{
				printf ("  Error: Can't open more than 1 %s file!",TypeName);
				return FALSE;
			}
		}
	}

	strcpy (ImageFiles[FileIndex].FileName,ImageFile);

	if (ImageType == OBMI) // This is for the -f option where you might wan't
	{                      // to load multiple files and TIM is not used.
		ImageFiles[FileIndex++].FileType = ImageType + FileIndex;
	}
	else
	{
		ImageFiles[FileIndex++].FileType = ImageType;
	}

	return TRUE;
}

bool ParseCommandLine (int argc,char *argv[])
{
	unsigned int DownloadBaudRate = 115200;
	int currentOption = 1;
	BYTE Option;
	bool IsPortTypeSet = false;
	bool IsComPortSet = false;
	bool IsBaudRateSet = false;
	bool IsUartDelaySet = false;
	bool IsMsgModeSet = false;

	while (currentOption < argc)
	{
		if (*argv[currentOption] == '-' || *argv[currentOption] == '/')
		{
			Option = *++argv[currentOption++];

			switch (Option)
			{
				case TIM:
					if (IsTim == true )
					{
						printf ("\n  Error: TIM file parameter must be specified only once.\n");
						return FALSE;
					}
					if (GetImageFile (TIMH,"TIM",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
					IsTim = TRUE;
					break;
				case OBM:
					if (GetImageFile (OBMI,"IMAGE",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
					IsImageFile = TRUE;
					break;
				case KEY:
#if TRUSTED
					if (IsJtag == true )
					{
						printf ("\n  Error: Key file parameter must be specified only once.\n");
						return FALSE;
					}
					if (GetImageFile ((DWORD)KEY,"JTAGKEY",currentOption,argc,argv[currentOption]) == FALSE) return FALSE;
					IsJtag = TRUE;
#else
					printf("  Error: WTPTP Non-trusted only version!!!\n");
#endif
					break;
				case OTP:
					if (IsOtpView == true )
					{
						printf ("\n  Error: OTP parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing OTP flash type parameter.\n");
						return FALSE;
					}
					IsOtpView = TRUE;
					if (atoi (argv[currentOption]) >= MAXFLASHTYPES)
					{
						printf ("Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						return FALSE;
					}
					WtpComm.SetFlashType (atoi (argv[currentOption]));
					break;
				case DBGBOOT:
					if (IsDebugBoot == true )
					{
						printf ("\n  Error: DebugBoot parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing DBG Boot flash type parameter.\n");
						return FALSE;
					}
					IsDebugBoot = TRUE;
					if (atoi (argv[currentOption]) >= MAXFLASHTYPES)
					{
						printf ("Error: FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
						return FALSE;
					}
					WtpComm.SetFlashType (atoi (argv[currentOption]));
					break;
				case PORTTYPE:
					if (IsPortTypeSet == true )
					{
						printf ("\n  Error: Port Type parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing Port Type parameter.\n");
						return FALSE;
					}
					if (strstr (argv[currentOption],"USB") != NULL)
					{
						WtpComm.SetPortType (USB);
						IsPortTypeSet = true;
						if ((argc % 2) == 0)
						{
							if ( (strstr (argv[currentOption + 1],"UART") != NULL) ||
							 (strstr (argv[currentOption + 1],"USB") != NULL) )
							{
								printf ("\n  Error: Port Type parameter must be specified only once.\n");
								return FALSE;
							}
						}
					}
					else if (strstr (argv[currentOption],"UART") != NULL)
					{
						WtpComm.SetPortType (UART);
						IsPortTypeSet = true;
						if ((argc % 2) == 0)
						{

							if ( (strstr (argv[currentOption + 1],"UART") != NULL) ||
							 (strstr (argv[currentOption + 1],"USB") != NULL) )
							{
								printf ("\n  Error: Port Type parameter must be specified only once.\n");
								return FALSE;
							}
						}
					}
					else
					{
						printf ("\n  Error: Port Type parameter must be USB or UART.\n");
						return FALSE;
					}
					break;
				case COMPORT:
					if (IsComPortSet == true )
					{
						printf ("\n  Error: COM Port parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing Com port parameter.\n");
						return FALSE;
					}
					WtpComm.SetUartPort (atoi (argv[currentOption]));
					if (atoi (argv[currentOption + 1]) >= 1 )
					{
						printf ("\n  Error: COM Port parameter must be specified only once.\n");
						return FALSE;
					}
					IsComPortSet = true;
					break;
				case BAUDRATE:
					if (IsBaudRateSet == true )
					{
						printf ("\n  Error: Baud Rate parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing Com port baudrate parameter.\n");
						return FALSE;
					}
					DownloadBaudRate = atoi (argv[currentOption]);
					if ( (atoi (argv[currentOption + 1]) == 2400 ) ||
						 (atoi (argv[currentOption + 1]) == 4800 ) ||
						 (atoi (argv[currentOption + 1]) == 9600 ) ||
						 (atoi (argv[currentOption + 1]) == 19200 ) ||
						 (atoi (argv[currentOption + 1]) == 38400 ) ||
						 (atoi (argv[currentOption + 1]) == 57600 ) ||
						 (atoi (argv[currentOption + 1]) == 115200 ))
					{
						printf ("\n  Error: Baud Rate parameter must be specified only once.\n");
						return FALSE;
					}					 
					if (DownloadBaudRate > 115200) DownloadBaudRate = 115200;
					WtpComm.SetBaudRate (DownloadBaudRate);
					IsBaudRateSet = true;
					break;
				case UARTDELAY:
					if (IsUartDelaySet == true )
					{
						printf ("\n  Error: UART delay parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing Com port parameter.\n");
						return FALSE;
					}
					WtpComm.SetImageSectionDelay (atoi (argv[currentOption]));
					IsUartDelaySet = true;
					break;
				case VERBOSE:
					bVerbose = TRUE;
					WtpComm.SetVerboseMode (bVerbose);
					break;
				case HELP:
					#if WINDOWS
					ShellExecute(NULL,"open","WTPTOOLHELP.chm",NULL,NULL,SW_SHOWNORMAL);
					#endif
					#if LINUX
					system("WTPTOOLHELP.chm");
					#endif
					break;
				case MSGMODE:
					if (IsMsgModeSet == true )
					{
						printf ("\n  Error: Message Mode parameter must be specified only once.\n");
						return FALSE;
					}
					if (currentOption == argc)
					{
						printf ("\n  Error: Missing message mode parameter.\n");
						return FALSE;
					}
					if ( atoi (argv[currentOption]) == 1 )
					{
						currentMessageMode = DISABLED;
					}
					else if ( atoi (argv[currentOption]) == 2 )
					{
						currentMessageMode = MSG_ONLY;
					}
					else if  ( atoi (argv[currentOption]) == 3 )
					{
						currentMessageMode = MSG_AND_DOWNLOAD; 
					} 					
					WtpComm.SetMessageMode (currentMessageMode);
					IsMsgModeSet = true;
					break;
				default:
					printf ("  Error: Command line option -%c is invalid!\n",Option);
					return FALSE;
			}
		}
		else
		{
			currentOption++;
		}
	}

	if (CheckYourOptions () == FALSE) return FALSE;

	return TRUE;
}

bool ParseScriptFile (char *ScriptFileName)
{
	unsigned int DownloadBaudRate = 115200;
	char DownLoadFile[MAX_PATH];
	char szMessage[100];
	char fLine[100];
	char szTemp[100];
	FILE* ScriptFile;
	int i = 0;

	ScriptFile = fopen (ScriptFileName,"r");
	if (ScriptFile == NULL)
	{
		sprintf (szMessage,"  Error: Can not open %s !",ScriptFileName);
/*		#ifdef _CONSOLE
			printf ("%s\n",szMessage);
		#else
			#if WINDOWS
			MessageBox (NULL,szMessage,"FILE OPEN ERROR",MB_ICONSTOP|MB_OK);
			#endif
		#endif*/
		return FALSE;
	}

	if (fgets (fLine,100,ScriptFile) != NULL)
	{
		if (strstr (fLine,"[WTPDOWNLOAD]") == NULL)
		{
			printf ("////////////////////////////////////////////////\n");
			printf ("  Error: Script file, %s, is invalid!\n",ScriptFileName);
			printf ("[WTPDOWNLOAD] must be included on the first line\n");
			printf ("to insure correct file type.\n");
			printf ("////////////////////////////////////////////////\n");
			fclose (ScriptFile);
			return FALSE;
		}
	}
	else
	{
		printf ("////////////////////////////////////////////////\n");
		printf ("  Error: Script file, %s, is completely empty!\n",ScriptFileName);
		printf ("////////////////////////////////////////////////\n");
		fclose (ScriptFile);
		return FALSE;
	}

	while (fgets (fLine,100,ScriptFile) != NULL)
	{
		if (strstr (fLine,"!") != NULL)
		{
			// This must be the first check in the code. If this is
			// found anywhere in the line of the script the line is
			// considered to be commented out.
			continue;
		}

		if (strstr (fLine,"VERBOSE") != NULL)
		{
			if (strstr (fLine,"=") == NULL)
			{
				printf  ("  Error: VERBOSE declaration statement does not contain an = sign in it!\n");
				printf  ("Format: VERBOSE = TRUE \n");
				fclose (ScriptFile);
				return FALSE;
			}

			strtok (fLine,"=");
			char *IsTrue = strtok (NULL,"=");

			if (strstr (IsTrue,"TRUE") != NULL)
			{
				printf ("\nVerbose mode is enabled.\n");
				bVerbose = TRUE;
			}
			WtpComm.SetVerboseMode (bVerbose);

			continue;
		}

		if (strstr (fLine,"UARTDELAY") != NULL)
		{
			if (GetParameter (fLine,szTemp,"IMAGESECTIONDELAY",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("  Error:  IMAGESECTIONDELAY can only be set to numbers!\n");
					printf ("Format: IMAGESECTIONDELAY = 300\n");
					fclose (ScriptFile);
					return FALSE;
				}

				++i;
			}
			WtpComm.SetImageSectionDelay (atoi (szTemp));
			continue;
		}

		if (strstr (fLine,"TIM") != NULL)
		{
			if (GetParameter (fLine,DownLoadFile,"TIM",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (GetImageFile (TIMH,"TIM",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
			IsTim = TRUE;
			continue;
		}

		if (strstr (fLine,"OBM") != NULL)
		{
			if (GetParameter (fLine,DownLoadFile,"OBM",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (GetImageFile (OBMI,"OBM",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
			IsImageFile = TRUE;
			continue;
		}

		if (strstr (fLine,"KEY") != NULL)
		{
			if (GetParameter (fLine,DownLoadFile,"JTAGKEY",TRUE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (GetImageFile ((DWORD)KEY,"KEY",0,1,DownLoadFile) == FALSE) {fclose (ScriptFile); return FALSE;}
			IsJtag = TRUE;
			continue;
		}

		if (strstr (fLine,"OTPVIEW") != NULL)
		{
			if (GetParameter (fLine,szTemp,"OTPVIEW",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (strstr (szTemp,"TRUE") != NULL)
			{
				IsOtpView = TRUE;
			}
			continue;
		}

		if (strstr (fLine,"FLASHTYPE") != NULL)
		{
			if (GetParameter (fLine,szTemp,"FLASHTYPE",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("  Error:  FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
					printf ("Format: FLASHTYPE = 7\n");
					fclose (ScriptFile);
					return FALSE;
				}

				++i;
			}
			if (atoi (szTemp) >= MAXFLASHTYPES)
			{
				printf ("  Error:  FLASHTYPE can only be set to numbers 0 thru %d!\n",MAXFLASHTYPES - 1);
				fclose (ScriptFile);
				return FALSE;
			}
			WtpComm.SetFlashType (atoi (szTemp));
			continue;
		}

		if (strstr (fLine,"DEBUGBOOT") != NULL)
		{
			if (GetParameter (fLine,szTemp,"DEBUGBOOT",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (strstr (szTemp,"TRUE") != NULL)
			{
				IsDebugBoot = TRUE;
			}
			continue;
		}

		if (strstr (fLine,"PORTTYPE") != NULL)
		{
			if (GetParameter (fLine,szTemp,"PORTTYPE",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			if (strstr (szTemp,"USB") != NULL)
			{
				WtpComm.SetPortType (USB);
			}
			else if (strstr (szTemp,"UART") != NULL)
			{
				WtpComm.SetPortType (UART);
			}
			else
			{
				printf ("\n  Error: Port Type parameter must be USB or UART.\n");
				fclose (ScriptFile);
				return FALSE;
			}
			continue;
		}

		if (strstr (fLine,"COMPORT") != NULL)
		{
			if (GetParameter (fLine,szTemp,"COMPORT",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			WtpComm.SetUartPort (atoi (szTemp));
			continue;
		}

		if (strstr (fLine,"BAUDRATE") != NULL)
		{
			if (GetParameter (fLine,szTemp,"BAUDRATE",FALSE) == FALSE) {fclose (ScriptFile); return FALSE;}
			i = 0;
			while (szTemp[i] != '\0')
			{
				if ( !(isdigit (szTemp[i])) )
				{
					printf ("  Error:  BAUDRATE can only be set to numbers!\n");
					printf ("Format: BAUDRATE = 115200\n");
					fclose (ScriptFile);
					return FALSE;
				}

				++i;
			}
			DownloadBaudRate = atoi (szTemp);
			if (DownloadBaudRate > 115200) DownloadBaudRate = 115200;
			WtpComm.SetBaudRate (DownloadBaudRate);
			continue;
		}
	}

	fclose (ScriptFile);

	if (CheckYourOptions () == FALSE) return FALSE;

	return TRUE;
}

bool CheckYourOptions ()
{
	int nOptionsEnabled = 0;

	if (IsTim)
	{
		if (IsOtpView || IsDebugBoot  || (currentMessageMode == MSG_ONLY))
		{
			printf ("  Error: Cannot select OTP view, Debug Boot or Message Only\n");
			printf ("options with TIM file downloads!\n");
			return FALSE;
		}
	}
	else if( currentMessageMode == MSG_ONLY )
	{
		if (IsJtag || IsOtpView || IsDebugBoot || IsTim || IsImageFile )
		{
			printf ("  Error: Can only select port type \n");
			printf ("option with Message Mode!\n");
			return FALSE;
		}
	}
	else if( currentMessageMode == MSG_AND_DOWNLOAD)
	{
		if (IsJtag || IsOtpView || IsDebugBoot )
		{
			printf ("  Error: Cannot select JTAG reenablement, OTP view or Debug Boot \n");
			printf ("options with Message and Download Mode!\n");
			return FALSE;
		}
	}
	else
	{
		if (IsJtag) nOptionsEnabled++;
		if (IsDebugBoot) nOptionsEnabled++;
		if (IsOtpView) nOptionsEnabled++;
		if (IsImageFile) nOptionsEnabled++;

		if (nOptionsEnabled > 1)
		{
			printf ("  Error: Cannot select image file downloads, OTP view, Debug Boot\n");
			printf ("or JTAG reenablement options simultaneously!\n");
			return FALSE;
		}
	}

	return TRUE;
}

bool GetParameter (char *fLine,char *pParameter,const char *pDec,bool IsFile)
{
	char *pTemp = NULL;

	// This function is used by the ParseScriptFile function to parse out
	// values from lines that contain an equal sign. An equal sign is used as
	// part of a directive. Example directive( PORTTYPE = UART ). In this case
	// the token UART is extracted from the string to signify that the UART
	// will be used to download images.

	if (strstr (fLine,"=") == NULL)
	{
		printf  ("  Error: %s declaration statement does not contain an = sign in it!\n",pDec);
		printf  ("Format: %s = filename \n",pDec);
		return FALSE;
	}

	strtok (fLine,"=");
	pTemp = strtok (NULL,"=");

	if (pTemp == NULL)
	{
		printf  ("  Error: %s declaration statement is invalid!\n",pDec);
		printf  ("Format: %s = filename \n",pDec);
		return FALSE;
	}

	if (IsFile)
	{
		while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
		{
			if (isprint (*pTemp) && *pTemp != ' ')
			{
				while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
				{
					if (isprint (*pTemp))
					{
						*pParameter = *pTemp;
						++pParameter;
					}
					++pTemp;
				}
				break;
			}
			++pTemp;
		}
	}
	else
	{
		while (*pTemp != '\0' && *pTemp != '\n' && *pTemp != '\r')
		{
			if (isprint (*pTemp) && *pTemp != ' ')
			{
				*pParameter = *pTemp;
				++pParameter;
			}
			++pTemp;
		}
	}
	*pParameter = '\0';

	return TRUE;
}

void PrintUsage ()
{
	printf ("\n WTPTP Host Application for Downloading ");
#if !(TRUSTED)
	printf("\n\n Non-trusted only version\n\n");
#else
	printf("and Jtag Re-Enablement\n\n");
#endif
	printf ("  Usage:\n");
	printf ("  ======\n");
	printf ("  >> WTPTP.exe <Option>|<Option file> ...\n\n");
	printf ("  Options:\n");
	printf ("  ========\n");
	printf ("  -V\t\tVerbose mode.  Debugging information will be printed.\n");
	printf ("  -f <file>\tinput any image file (binary)\n");
#if TRUSTED
	printf ("  -k <file>\tinput Jtag Key File (ASCII)\n");
#endif
	printf ("  -t <file>\tinput TIM file (binary)\n");
	printf ("  -O flash type\tSet flash type between 0 thru 7. Runs the OTP viewer.\n");
	printf ("  -D Runs the Debug Boot command.\n");
	printf ("  -S delay time\tThe delay time value is in milliseconds. Used with UART.\n\n");
	printf ("  -P <port type>\t Port types: UART or USB\n");
	printf ("  -C COM Port number. Used with UART.\n");
	printf ("  -B Baud rate. Used with UART.\n");
	printf ("  -x Script file.\n");
	printf ("  -h Help\n");
	printf ("  -M <MessageMode>  2 is MSG_ONLY, 3 is MessageMode and Download\n");
	printf ("  NOTE: The -f option is not compatible with the -k and -O options!\n");
#if TRUSTED
	printf ("        -k is for Jtag Re-Enabling. -f is for Downloading.\n");
#endif
	printf ("        -O is for the OTP viewer.\n\n");
	printf ("        The -t option is not compatible with the -k and -O options.\n\n");
	printf ("  Examples:\n");
	printf ("  =========\n");
#if TRUSTED
	printf ("  To use WTPTP for Jtag Re-Enabling:\n");
	printf ("\t>>WTPTP.exe -k JtagOEMkeyfile.txt\n\n");
#endif
	printf ("  To use WTPTP for Downloading:\n");
	printf ("\t>>WTPTP.exe -P USB -t TIM.bin -f OBM_h.bin\n\n");
	printf ("  Message-only Mode(Mode 2) Usage:\n");
	printf ("  ========================\n");
	printf ("\t>>wtptp -P USB -M 2\n\n");
	printf ("  Message Mode and Download(Mode 3) Usage:\n");
	printf ("  ========================\n");
	printf ("\t>>wtptp -P USB -t <NTIM/TIM.bin> -f <Image_h.bin> -M 3\n");
#if TRUSTED
	printf ("  Download of TIM-JTAG Reenable-Image Usage:\n");
	printf ("  ========================\n");
	printf ("\t>>wtptp -P UART -t <NTIM/TIM.bin> -f <Image_h.bin> -k JtagOEMKeyfile.txt\n");
#endif
}

